package it.asterisk.manager;

import it.asterisk.manager.response.ManagerError;
import it.asterisk.manager.response.ManagerResponse;

/**
 * Callback interface to send
 * {@link it.asterisk.manager.action.EventGeneratingAction} asynchronously.
 *
 * @see it.asterisk.manager.ManagerConnection#sendEventGeneratingAction(it.asterisk.manager.action.EventGeneratingAction, SendEventGeneratingActionCallback)
 *
 * Initial response is passed to one of {@link #onResponse(ManagerResponse)} or
 * {@link #onErrorResponse(ManagerResponse)}. but not both.
 */
public interface SendEventGeneratingActionCallback
{
    /**
     * Called to notify that
     * {@link ManagerConnection#sendEventGeneratingAction(it.asterisk.manager.action.EventGeneratingAction, SendEventGeneratingActionCallback)}
     * is done.
     * <p>
     * If connection is lost before action is completed then
     * {@link ResponseEvents#getResponse()} can be null or list of events might
     * be lacking some events including end event. The
     * {@link ResponseEvents#getResponse()} can also be {@link ManagerError}
     *
     * @param responseEvents
     *            the response at whatever state it was when action ended.
     */
    public void onResponse(ResponseEvents responseEvents);
}