package it.asterisk.pbx.agi;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import it.asterisk.fastagi.AgiChannel;
import it.asterisk.fastagi.AgiException;
import it.asterisk.fastagi.AgiHangupException;
import it.asterisk.pbx.AgiChannelActivityAction;
import it.asterisk.pbx.Channel;
import it.callite.log.AsteriskLog;

public class AgiChannelActivityHold implements AgiChannelActivityAction
{

    CountDownLatch latch = new CountDownLatch(1);
    volatile boolean callReachedAgi = false;
    long timer = System.currentTimeMillis();

    @Override
    public void execute(AgiChannel channel, Channel ichannel) throws AgiException, InterruptedException
    {
        try
        {
            callReachedAgi = true;
            channel.answer();
            channel.playMusicOnHold();
            long secondsOnHold = Math.abs(System.currentTimeMillis() - timer) / 1000;
            if (secondsOnHold > 600)
            {
                AsteriskLog.info(ichannel + " is still on hold after " + secondsOnHold + " seconds");
            }
            if (latch.await(10, TimeUnit.SECONDS))
            {
                try
                {
                    channel.stopMusicOnHold();
                }
                catch (AgiHangupException e)
                {
                    AsteriskLog.info("Channel hungup " + channel.getName());
                }
                catch (Exception e)
                {
                    AsteriskLog.warn(e);
                }
            }
            else
            {
                if (channel.getName().startsWith("Local") && secondsOnHold > 3600)
                {
                    // cleanup Local channels older than 1 hour
                    AsteriskLog.error("Hanging up local channel that has been on hold for 1 hour " + channel.getName());
                    channel.hangup();
                }
            }
        }
        catch (AgiHangupException e)
        {
            AsteriskLog.warn(e);
        }

    }

    @Override
    public boolean isDisconnect(ActivityAgi activityAgi)
    {
        return false;
    }

    @Override
    public void cancel()
    {
        latch.countDown();

    }

    public boolean hasCallReachedAgi()
    {
        return callReachedAgi;
    }
}
