package it.asterisk.pbx.agi;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import it.asterisk.fastagi.AgiChannel;
import it.asterisk.fastagi.AgiException;
import it.asterisk.fastagi.AgiHangupException;
import it.asterisk.pbx.AgiChannelActivityAction;
import it.asterisk.pbx.Channel;
import it.callite.log.AsteriskLog;

/**
 * Waits in silence for 10 seconds, then hangs up. This is useful where channels
 * are waiting for some external action to redirect/bridge them.
 * 
 * @author rsutton
 */
public class AgiChannelActivityTransientHoldSilence implements AgiChannelActivityAction
{

    CountDownLatch latch = new CountDownLatch(1);
    volatile boolean callReachedAgi = false;
    boolean firstPass = true;

    @Override
    public void execute(AgiChannel channel, Channel ichannel) throws AgiException, InterruptedException
    {
        try
        {
            callReachedAgi = true;
            channel.answer();
            if (!firstPass)
            {
                AsteriskLog.error(ichannel + " is still on hold after first pass, Hanging up");
                channel.hangup();
            }
            else
            {
                if (!latch.await(10, TimeUnit.SECONDS))
                {
                    AsteriskLog.warn("Exiting with timeout");
                }
            }
        }
        catch (AgiHangupException e)
        {
            AsteriskLog.warn(e);
        }
        finally
        {
            firstPass = false;
        }

    }

    @Override
    public boolean isDisconnect(ActivityAgi activityAgi)
    {
        return false;
    }

    @Override
    public void cancel()
    {
        latch.countDown();

    }

    public boolean hasCallReachedAgi()
    {
        return callReachedAgi;
    }
}
