package it.asterisk.pbx.internal.activity;

import java.util.HashSet;

import it.asterisk.pbx.ActivityCallback;
import it.asterisk.pbx.Channel;
import it.asterisk.pbx.ListenerPriority;
import it.asterisk.pbx.PBXException;
import it.asterisk.pbx.PBXFactory;
import it.asterisk.pbx.activities.HoldActivity;
import it.asterisk.pbx.agi.AgiChannelActivityHold;
import it.asterisk.pbx.asterisk.wrap.events.ManagerEvent;
import it.asterisk.pbx.internal.core.AsteriskPBX;
import it.callite.log.AsteriskLog;

public class HoldActivityImpl extends ActivityHelper<HoldActivity> implements HoldActivity
{
    private final Channel _channel;

    public HoldActivityImpl(final Channel channel, final ActivityCallback<HoldActivity> callback)
    {
        super("HoldCall", callback);

        this._channel = channel;
        this.startActivity(false);
    }

    @Override
    public boolean doActivity() throws PBXException
    {
        boolean success = false;

        AsteriskLog.debug("*******************************************************************************");
        AsteriskLog.info("***********                    begin Hold Audio               ****************");
        AsteriskLog.info("***********                  " + this._channel + " ****************");
        AsteriskLog.debug("*******************************************************************************");
        try
        {
            AsteriskPBX pbx = (AsteriskPBX) PBXFactory.getActivePBX();

            if (!pbx.moveChannelToAgi(_channel))
            {
                throw new PBXException("Channel: " + _channel + " couldn't be moved to agi");
            }

            _channel.setCurrentActivityAction(new AgiChannelActivityHold());
            success = true;

        }
        catch (IllegalArgumentException | IllegalStateException e)
        {
            AsteriskLog.error(e, e);
            AsteriskLog.error(e, e);
            throw new PBXException(e);
        }
        return success;
    }

    public Channel getChannel()
    {
        return this._channel;
    }

    @Override
    public HashSet<Class< ? extends ManagerEvent>> requiredEvents()
    {
        HashSet<Class< ? extends ManagerEvent>> required = new HashSet<>();
        // no events requried.

        return required;
    }

    @Override
    public ListenerPriority getPriority()
    {
        return ListenerPriority.NORMAL;
    }

    @Override
    public void onManagerEvent(ManagerEvent event)
    {
        // NOOP

    }

}
