package it.asterisk.pbx.internal.core;

import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeSet;

import it.asterisk.pbx.asterisk.wrap.events.ManagerEvent;
import it.callite.log.AsteriskLog;

public class ListenerManager
{
    private final TreeSet<FilteredManagerListenerWrapper> listeners = new TreeSet<>(new ListenerPriorityComparator());


    final class ListenerPriorityComparator implements Comparator<FilteredManagerListenerWrapper>
    {
        @Override
        public int compare(FilteredManagerListenerWrapper lhs, FilteredManagerListenerWrapper rhs)
        {

            int result = lhs._listener.getPriority().compare(rhs._listener.getPriority());

            if (result == 0)
                result = lhs.equalityBuster.compareTo(rhs.equalityBuster);

            return result;
        }
    }

    public void clear()
    {
        listeners.clear();
    }

    public int size()
    {
        return listeners.size();
    }

    public Iterator<FilteredManagerListenerWrapper> iterator()
    {
        return listeners.iterator();
    }

    public void addListener(FilteredManagerListener<ManagerEvent> listener)
    {
        listeners.add(new FilteredManagerListenerWrapper(listener));
    }

    List<FilteredManagerListenerWrapper> getCopyAsList()
    {
        List<FilteredManagerListenerWrapper> list = new LinkedList<>();
        for (FilteredManagerListenerWrapper listener : listeners)
        {
            list.add(listener);
        }

        return list;
    }

    boolean removeListener(FilteredManagerListener<ManagerEvent> toRemove)
    {
        boolean removed = false;
        Iterator<FilteredManagerListenerWrapper> itr = listeners.iterator();
        while (itr.hasNext())
        {
            FilteredManagerListenerWrapper container = itr.next();
            // AsteriskLog.error("Checking " + container._listener + " " + toRemove);
            if (container._listener == toRemove)
            {
                AsteriskLog.debug("Removing listener " + toRemove, null);
                itr.remove();
                removed = true;
                break;
            }
        }
        return removed;
    }

}
