package it.callite.action;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;

import org.springframework.beans.factory.annotation.Autowired;

import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.UserDao;
import it.callite.message.Message;
import it.callite.model.Utente;
import it.callite.model.report.MonitorOperatore;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.RuoloMap;
@SuppressWarnings("serial")
public class UtenteBean implements Serializable {

	private RuoloMap ruoli;
	private Utente utente;
	@Autowired
	private UserDao userDao;
	@Autowired
	private ColonnaBean colonnaBean;

	public UtenteBean() {
		ruoli = new RuoloMap();
	}
	public void chiama() {
		Utente utente = getUtente();
		if(utente!=null) {
			Map<String, String> map = ApplicationUtil.getRequestParameterMap();
			if(map != null) {
				String telefono  = ApplicationUtil.trimToEmpty((String) map.get("telefono"));
				String richiama  = ApplicationUtil.trimToEmpty((String) map.get("richiama"));
				String contatto  = ApplicationUtil.trimToEmpty((String) map.get("contatto"));
				String context   = ApplicationUtil.trimToEmpty((String) map.get("context"));
				String ingresso  = ApplicationUtil.trimToEmpty((String) map.get("ingresso"));
				userDao.chiama(utente, telefono, richiama, contatto, ingresso, context);
			}
		}
	}
	public void forceEndorse(ActionEvent event) {
		Utente utente = getUtente();
		if (utente != null) {
			Chiamate.getCodaOperatori().endorse(utente);
		}
	}
	public void forceReload() {
		userDao.bloccaReload(utente);
		ricaricaUtente();
		ApplicationUtil.goToUrl(ConstantsGlobal.path_web_home);
	}
	public Boolean getAdmin() {
		Utente utente = getUtente();
		if (utente != null) {
			return  utente.isAdmin() && ApplicationUtil.isTrue(utente.getBarra());
		} 
		return false;
	}

	public String getTelefono() {
		Utente utente = getUtente();
		if (utente != null) {
			return utente.getInternoString();
		} 
		return "";
	}
	
	public Boolean getDeprecated() {
		String browser = getBrowserType(); 
		return !browser.contains("Trident") && browser.contains("MSIE");
	}
	
	public Boolean getTridentA() {
		String browser = getBrowserType(); 
		return browser.contains("Trident") && !browser.contains("MSIE") ;
	}
	
	public Boolean getTrident() {
		String browser = getBrowserType(); 
		return browser.contains("Trident");
	}

	public Boolean getSafari() {
		String browser = getBrowserType(); 
		return (browser.contains("iPhone") || browser.contains("iPad")) && browser.contains("Safari");
	}
	
	public String getBrowserType() {
		Utente utente = getUtente();
		if (utente != null) {
			return utente.getBrowser();
		} 
		return ApplicationUtil.browser();
	}


	public Boolean getCambioProgrammatoPassword() {
		Utente utente = getUtente();
		if (utente != null) {
			return utente.getCambioProgrammatoPassword();
		}
		return false;
	}
	
	public Integer getDisattivaMessaggistica() {
		Utente utente = getUtente();
		if (utente != null) {
			return utente.getDisattivaMessaggistica();
		}
		return 0;
	}
	public String getEditor() {
		if(getTridentA()) {
			return ConstantsMap.source;
		} else {
			return userDao.getEditor();
		}
	}
	public Boolean getFlash() {
		return userDao.getFlash();
	}
	public Long getId() {
		Utente utente = getUtente();
		if (utente != null) {
			return  utente.getId();
		} 
		return null;
	}
	public Boolean getInterno() {
		Utente utente = getUtente();
		if (utente != null) {
			return ApplicationUtil.isTrue(utente.getBarra());
		} 
		return false;
	}

	public Boolean getSegreteria() {
		Utente utente = getUtente();
		if (utente != null) {
			return utente.getSegreteria() && ApplicationUtil.isTrue(utente.getBarra());
		} 
		return false;
	}

	public Boolean getSegreteriaOK() {
		Utente utente = getUtente();
		if (utente != null) {
			return utente.getSegreteria() && userDao.getSegreteriaOK(utente.getInternoString());
		} 
		return false;
	}

	public String getMessaggio() {
		Utente utente = getUtente();
		if (utente != null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(utente);
			if(mop != null) {
				List<String> messaggi = mop.getMessaggi();
				if(ApplicationUtil.notEmpty(messaggi)) {
					return messaggi.remove(0);
				}
			}
		}
		return "";
	}
	public Boolean getMonitor() {
		Utente utente = getUtente();
		if (utente != null) {
			return Chiamate.getCodaOperatori().contains(utente);
		}
		return false;
	}
	public Boolean getOperativo() {
		Utente utente = getUtente();
		if (utente != null) {
			return  utente.getRegistrato();
		} 
		return false;
	}

	public String getPhone() {
		Utente utente = getUtente();
		if (utente != null) {
			return utente.isLocale() ? utente.getInterno().getPhoneS() : "";
		}
		return "";
	}

	public Boolean getPhoneOK() {
		return !getPhone().equals(ConstantsMap.phone1);
	}

	public String getPhoneS() {
		Utente utente = getUtente();
		if (utente != null) {
			return utente.getPhoneS();
		}
		return "";
	}
	public Boolean getReload() {
		Utente utente = getUtente();
		if (utente != null) {
			MonitorOperatore mop = Chiamate.getCodaOperatori().get(utente);
			if(mop != null) {
				return mop.getReload();
			}
		}
		return false;
	}
	public RuoloMap getRuoli() {
		return ruoli;
	}

	public String getTemplateLeft() {
		return "auto";
	}

	public String getTemplateTop() {
		if(ApplicationUtil.isTrue(getSafari())) {
			return "100";
		} else { 
			return "auto";
		}
	}
	
	public String getTemplateSize() {
		boolean interno = ApplicationUtil.isTrue(getInterno());
		if(!interno) {
			return "0";
		} else {
			return "100";
		}
	}
	
	public Utente getUtente() {
		if(utente == null) {
			reloadUserRole();
		}
		return utente;
	}

	public boolean isAgente() {
		Utente utente = getUtente();
		if(utente!=null) {
			return utente.isAgente();
		}
		return false;
	}

	public boolean isOperatore() {
		Utente utente = getUtente();
		if(utente!=null) {
			return utente.isOperatore();
		}
		return false;
	}

	public boolean isSupervisore() {
		Utente utente = getUtente();
		if(utente!=null) {
			return utente.isSupervisore();
		}
		return false;
	}
	
	public boolean isChat() {
		Utente utente = getUtente();
		if(utente!=null) {
			return utente.isChat();
		}
		return false;
	}

	public List<SelectItem> getRuoliSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		Utente utente = getUtente();
		if(utente!=null) {
			if(utente.isOperatore()) {
				sis.add(new SelectItem(Boolean.FALSE, Message.addCatalogText("operatore")));
			}
			if(utente.isSupervisore()) {
				sis.add(new SelectItem(Boolean.TRUE, Message.addCatalogText("supervisore")));
			}
		}
		return sis;
	}

	public boolean isShell() {
		Utente utente = getUtente();
		if (utente != null) {
			return  utente.isShell();
		} 
		return false;
	}

	public void reloadUserRole() {
		utente = ApplicationUtil.getUsernameLoggato();
		if(utente != null) {
			ruoli.setRuoli(utente.getRuoliAll());
			colonnaBean.setUtente(utente);
			colonnaBean.loadColonne();
		}
	}

	public void ricaricaUtente() {
		utente = userDao.reloadUser(utente);
		reloadUserRole();
		userDao.ricaricaCode(utente);
	}
	public Integer getLoginMultiplo() {
		Utente utente = getUtente();
		if(utente!=null) {
			return utente.getLoginMultiplo();
		}
		return 0;
	}

	public Boolean getForceReload() {
		Utente utente = getUtente();
		if(utente!=null) {
			return utente.getForceReload();
		}
		return false;
	}

	public Integer getAttivoControl() {
		Utente utente = getUtente();
		if(utente!=null) {
			return utente.getAttivoControl();
		}
		return 0;
	}

}