package it.callite.chiamate;




import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Restrictions;

import it.callite.model.AutoContatto;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.report.PannelloModel;
import it.callite.utility.ApplicationUtil;


public class CodaAutodialer {

	private Map<Long, Contatto> data;


	public CodaAutodialer() {
		data = new HashMap<Long, Contatto>();
	}

	public synchronized Contatto get(Long id){
		return data.get(id);
	}

	public synchronized void clear() {
		data.clear();
	}

	public synchronized void remove(Long id) {
		data.remove(id);
	}

	public synchronized void criteria(Boolean tutti, Campagna campagna, DetachedCriteria criteria){
		Set<Long> ln = ApplicationUtil.isTrue(tutti) ? data.keySet() :  getKey(campagna);
		if (ApplicationUtil.notEmpty(ln)) {
			criteria.add(Restrictions.not(Restrictions.in("id", ln)));
		}
	}
	
	public synchronized void put(Contatto contatto) {
		if(contatto != null) {
			data.put(contatto.getId(), contatto);
		}
	}

	public synchronized List<PannelloModel> list(int campagna){
		List<PannelloModel> lsch = new ArrayList<PannelloModel>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Contatto ch = data.get(lnt.next());
				if (ch != null) {
					PannelloModel pm = new PannelloModel(ch, campagna);
					lsch.add(pm);
				}
			}
		}
		return lsch;
	}

	public synchronized int getSize() {
		return data.size();
	}

	public synchronized Set<Long>  keySet() {
		return data.keySet();
	}

	public synchronized void removeContatti(List<Contatto> contatti) {
		if(ApplicationUtil.notEmpty(data) && ApplicationUtil.notEmpty(contatti)) {
			Set<Long> lngList = data.keySet();
			Set<Long> removeList = new HashSet<Long>();
			Iterator<Contatto> lnt = contatti.iterator();
			while (lnt.hasNext()) {
				Contatto tmp = lnt.next();
				if(lngList.contains(tmp.getId())) {
					removeList.add(tmp.getId());
				}
			}
			if(ApplicationUtil.notEmpty(removeList)) {
				data.keySet().removeAll(removeList);
			}
		}
	}

	public synchronized void remove(List<AutoContatto> contatti) {
		if(ApplicationUtil.notEmpty(data) && ApplicationUtil.notEmpty(contatti)) {
			Set<Long> lngList = data.keySet();
			Set<Long> removeList = new HashSet<Long>();
			Iterator<AutoContatto> lnt = contatti.iterator();
			while (lnt.hasNext()) {
				AutoContatto tmp = lnt.next();
				if(lngList.contains(tmp.getContatto().getId())) {
					removeList.add(tmp.getContatto().getId());
				}
			}
			if(ApplicationUtil.notEmpty(removeList)) {
				data.keySet().removeAll(removeList);
			}
		}
	}

	public synchronized boolean contains(Long contatto){
		if (ApplicationUtil.notEmpty(data) && ApplicationUtil.gtZero(contatto)) {
			return data.containsKey(contatto);
		}
		return false;
	}

	public synchronized List<Contatto> get(Campagna campagna) {	
		List<Contatto> lsch = new ArrayList<Contatto>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Contatto tmp = data.get(lnt.next());
				if(tmp!=null && tmp.getCampagna().equals(campagna)) {
					lsch.add(tmp);
				}
			}
		}
		return lsch;
	}

	public synchronized void remove(Campagna campagna) {
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Set<Long> removeList = new HashSet<Long>();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Contatto tmp = data.get(lnt.next());
				if(tmp!=null && tmp.getCampagna().equals(campagna)) {
					removeList.add(tmp.getId());
				}
			}
			if(ApplicationUtil.notEmpty(removeList)) {
				data.keySet().removeAll(removeList);
			}
		}
	}

	public synchronized List<Contatto> get(List<Long> cmp) {	
		List<Contatto> lsch = new ArrayList<Contatto>();
		if(ApplicationUtil.notEmpty(data) && ApplicationUtil.notEmpty(cmp)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Contatto tmp = data.get(lnt.next());
				if(tmp!=null && cmp.contains(tmp.getCampagna().getId())) {
					lsch.add(tmp);
				}
			}
		}
		return lsch;
	}


	public synchronized Set<Long> getKey(Campagna campagna) {	
		Set<Long> lsch = new HashSet<Long>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Contatto tmp = data.get(lnt.next());
				if(tmp!=null && tmp.getCampagna().equals(campagna)) {
					lsch.add(tmp.getId());
				}
			}
		}
		return lsch;
	}
	
	public synchronized List<Contatto> list(){
		return new ArrayList<Contatto>(data.values());
	}
	
	
	public synchronized List<Contatto> monitor(Campagna campagna, List<Long> campagne) {	
		List<Contatto> lsch = new ArrayList<Contatto>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				Contatto tmp = data.get(lnt.next());
				if(tmp!=null && campagne.contains(tmp.getCampagna().getId())) {
					if(campagna == null || (campagna != null && tmp.getCampagna().equals(campagna))) {
						lsch.add(tmp);
					}
				}
			}
		}
		return lsch;
	}
	

}