package it.callite.chiamate;



import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Restrictions;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.report.ChiamateAttive;
import it.callite.model.report.PannelloModel;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.sincronizzazione.InfoContatto;


public class CodaContatti {

	private Map<Long, InfoContatto> data;

	public CodaContatti() {
		data = new HashMap<Long, InfoContatto>();
	}

	public synchronized void clear() {
		data.clear();
	}

	public synchronized void remove(Long id) {
		data.remove(id);
	}

	public synchronized InfoContatto get(Long id) {
		return	data.get(id);
	}

	public synchronized void put(Long id, InfoContatto ic) {
		data.put(id, ic);
	}

	public synchronized List<PannelloModel> get(int campagna, List<Long> utentiId) {
		List<PannelloModel> lsch = new ArrayList<PannelloModel>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				InfoContatto ch = data.get(lnt.next());
				boolean b = (utentiId == null) || (utentiId != null && ch.getUtente() != null && utentiId.contains(ch.getUtente().getId()));
				if(ch != null && b) {
					PannelloModel pm = new PannelloModel(ch.getItem(), campagna);
					lsch.add(pm);
				}
			}
		}
		return lsch;
	}

	public synchronized List<ChiamateAttive> get(Integer privata) {	
		List<ChiamateAttive> lsch = new ArrayList<ChiamateAttive>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				InfoContatto ch = data.get(lnt.next());
				if(ch != null) {
					ChiamateAttive chAtt = new ChiamateAttive(ch.getUtente(), ch.getItem(), ch.isPrivata());
					if(ApplicationUtil.eq(privata, 1) && ch.isPrivata()) {
						lsch.add(chAtt);
					} else if(ApplicationUtil.eq(privata, 2) && !ch.isPrivata()) {
						lsch.add(chAtt);
					} else {
						lsch.add(chAtt);
					}
				}
			}
		}
		return lsch;
	}


	public synchronized List<InfoContatto> get(Long campagnaId, Integer privata) {	
		List<InfoContatto> lsch = new ArrayList<InfoContatto>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				InfoContatto tmp = data.get(lnt.next());
				if(tmp!=null && campagnaId != null && campagnaId.equals(tmp.getItem().getCampagna().getId())) {
					if(ApplicationUtil.eq(privata, 1) && tmp.isPrivata()) {
						lsch.add(tmp);
					} else if(ApplicationUtil.eq(privata, 2) && !tmp.isPrivata()) {
						lsch.add(tmp);
					} else {
						lsch.add(tmp);
					}
				}
			}
		}
		return lsch;
	}

	public synchronized List<ChiamateAttive> get(List<Long> contatti, Integer privata, List<Long> utentiId) {	
		List<ChiamateAttive> lsch = new ArrayList<ChiamateAttive>();
		if(ApplicationUtil.notEmpty(data) && ApplicationUtil.notEmpty(contatti) && ApplicationUtil.notEmpty(utentiId)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				InfoContatto ch = data.get(lnt.next());
				if(ch != null  && ch.getUtente()!= null && contatti.contains(ch.getItem().getId()) && utentiId.contains(ch.getUtente().getId())) {
					ChiamateAttive chAtt = new ChiamateAttive(ch.getUtente(), ch.getItem(), ch.isPrivata());
					if(ApplicationUtil.eq(privata, 1) && ch.isPrivata()) {
						lsch.add(chAtt);
					} else if(ApplicationUtil.eq(privata, 2) && !ch.isPrivata()) {
						lsch.add(chAtt);
					} else {
						lsch.add(chAtt);
					}
				}
			}
		}
		return lsch;
	}

	public synchronized int getSize() {
		return data.size();
	}

	public synchronized Set<Long>  keySet() {
		return data.keySet();
	}

	public synchronized List<InfoContatto> timeout(Date date) {
		List<InfoContatto> lsch = new ArrayList<InfoContatto>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				InfoContatto info = data.get(lnt.next());
				if(info!= null && info.getTempoApertura() != null && ((date.getTime() - info.getTempoApertura().longValue()) > ConstantsGlobal.timeoutContatto)) {
					lsch.add(info);
				}
			}
		}
		return lsch;
	}

	public synchronized boolean contains(Long contatto){
		if(ApplicationUtil.notEmpty(data) && ApplicationUtil.gtZero(contatto)) {
			return data.containsKey(contatto);
		}
		return false;
	}

	public synchronized void removeContatti(List<Contatto> contatti) {
		if(ApplicationUtil.notEmpty(data) && ApplicationUtil.notEmpty(contatti)) {
			Set<Long> lngList = data.keySet();
			Set<Long> removeList = new HashSet<Long>();
			Iterator<Contatto> lnt = contatti.iterator();
			while (lnt.hasNext()) {
				Contatto tmp = lnt.next();
				if(lngList.contains(tmp.getId())) {
					removeList.add(tmp.getId());
				}
			}
			if(ApplicationUtil.notEmpty(removeList)) {
				data.keySet().removeAll(removeList);
			}
		}
	}

	public synchronized void criteria(Boolean tutti, Campagna campagna, DetachedCriteria criteria){
		Set<Long> ln = ApplicationUtil.isTrue(tutti) ? data.keySet() :  getKey(campagna);
		if(ApplicationUtil.notEmpty(ln)) {
			criteria.add(Restrictions.not(Restrictions.in("id", ln)));
		}
	}

	public synchronized List<InfoContatto> list(){
		return new ArrayList<InfoContatto>(data.values());
	}

	public synchronized void update(Long id) {
		InfoContatto ic = data.get(id);
		if(ic != null) {
			long time = System.currentTimeMillis();
			ic.setTempoApertura(time);
			//ic.setOperatore(user);
			//put(id, ic);
		} 
	}

	public synchronized Set<Long> getKey(Campagna campagna) {	
		Set<Long> lsch = new HashSet<Long>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				InfoContatto tmp = data.get(lnt.next());
				if(tmp!=null && campagna != null && campagna.equals(tmp.getItem().getCampagna()) && !tmp.isPrivata()) {
					lsch.add(tmp.getItem().getId());
				}
			}
		}
		return lsch;
	}


	public synchronized List<InfoContatto> monitor(Campagna campagna, List<Long> campagne, Integer privata) {	
		List<InfoContatto> lsch = new ArrayList<InfoContatto>();
		if(ApplicationUtil.notEmpty(data)) {
			Set<Long> lngList = data.keySet();
			Iterator<Long> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				InfoContatto tmp = data.get(lnt.next());
				if(tmp!=null && campagne.contains(tmp.getItem().getCampagna().getId())) {
					if(campagna == null || (campagna != null && tmp.getItem().getCampagna().equals(campagna))) {
						if(ApplicationUtil.eq(privata, 1) && tmp.isPrivata()) {
							lsch.add(tmp);
						} else if(ApplicationUtil.eq(privata, 2) && !tmp.isPrivata()) {
							lsch.add(tmp);
						} else {
							lsch.add(tmp);
						}
					}
				}
			}
		}
		return lsch;
	}


}