package it.callite.criteria.generic;

import java.io.Serializable;
import java.util.Date;
import java.util.Map;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.ImpostazioneDao;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.IntervalloData;

public abstract class BaseCriteria extends UnitedCriteria implements Serializable {

	private static final long serialVersionUID = 1L;

	private IntervalloData intervallo = new IntervalloData();
	private Boolean tasto = false;
	private Boolean visibileId = false;
	private String ids;

	public boolean isRicercabile() {
		return false;
	}

	public String getIds() {
		return ApplicationUtil.trimToNull(ids);
	}

	public void setIds(String ids) {
		this.ids = ApplicationUtil.trimToNull(ids);
	}



	public Boolean getVisibileId() {
		return visibileId;
	}

	public void setVisibileId(Boolean visibileId) {
		this.visibileId = visibileId;
	}

	public Boolean getTasto() {
		return tasto;
	}


	public void doImposta(){
		if(ApplicationUtil.isNotBlank(intervallo.getPeriodo())) {
			if(intervallo.getPeriodo().equals("reset")) {
				intervallo.reset();
			} else {
				String [] p = intervallo.getPeriodo().split(ConstantsGlobal.colonSeparator);
				String periodo = p[0];
				Integer ii = Integer.parseInt(p[1]);
				if(periodo.equals(ConstantsMap.p_g)){
					intervallo.setInizio(ApplicationUtil.inizioGiornata(ii));
					intervallo.setFine(ApplicationUtil.fineGiornata(ii));
				}
				if(periodo.equals(ConstantsMap.p_s)){
					intervallo.setInizio(ApplicationUtil.inizioSettimana(ii));
					intervallo.setFine(ApplicationUtil.fineSettimana(ii));
				}
				if(periodo.equals(ConstantsMap.p_m)){
					intervallo.setInizio(ApplicationUtil.inizioMese(ii));
					intervallo.setFine(ApplicationUtil.fineMese(ii));
				}
				if(periodo.equals(ConstantsMap.p_a)){
					intervallo.setInizio(ApplicationUtil.inizioAnno(ii));
					intervallo.setFine(ApplicationUtil.fineAnno(ii));
				}
			}
		} 
	}

	public Date getFine() {
		return intervallo.getFine();
	}

	public Date getInizio() {
		return intervallo.getInizio();
	}

	public IntervalloData getIntervallo() {
		return intervallo;
	}


	public BaseCriteria loadCriteria() {
		BaseCriteria c = (BaseCriteria) ApplicationUtil.deserialize(getSerialize());
		if(c != null) {
			c.doImposta();
			return c;
		}
		return this;
	}

	public void replace(String periodo) {
		if(periodo.equals(ConstantsMap.p_g)){
			intervallo.setInizio(ApplicationUtil.inizioGiornata(0));
			intervallo.setFine(ApplicationUtil.fineGiornata(0));
		}
		if(periodo.equals(ConstantsMap.p_s)){
			intervallo.setInizio(ApplicationUtil.inizioSettimana(0));
			intervallo.setFine(ApplicationUtil.fineSettimana(0));
		}
		if(periodo.equals(ConstantsMap.p_m)){
			intervallo.setInizio(ApplicationUtil.inizioMese(0));
			intervallo.setFine(ApplicationUtil.fineMese(0));
		}
		if(periodo.equals(ConstantsMap.p_a)){
			intervallo.setInizio(ApplicationUtil.inizioAnno(0));
			intervallo.setFine(ApplicationUtil.fineAnno(0));
		}
	}


	public void salvaCriteria(ImpostazioneDao impostazioneDao, Integer b)  {
		if(ApplicationUtil.isTrue(impostazioneDao.getAbilitaRicercaManuale())) {
			setCerca(true);
			tasto = false;
		} else {
			setCerca(ApplicationUtil.ne(b, 1));
			tasto = ApplicationUtil.eq(b, 1);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getAbilitaRicercaTasto())) {
			if(ApplicationUtil.gtZero(b)) {
				this.setSerialize(null);
				setSerialize(ApplicationUtil.serialize(this));
			}
		} else {
			setSerialize(null);
		}
	}

	public void setFine(Date dataFine) {
		intervallo.setFine(dataFine);
	}

	public void setInizio(Date dataInizio) {
		intervallo.setInizio(dataInizio);
	}

	public void setIntervallo(Date dataInizio, Date dataFine) {
		intervallo.setInizio(dataInizio);
		intervallo.setFine(dataFine);
	}

	public void setIntervallo(IntervalloData intervallo) {
		this.intervallo = intervallo;
	}

	private String serialize;

	public String getSerialize() {
		return serialize;
	}

	public void setSerialize(String serialize) {
		this.serialize = serialize;
	}

	public boolean isRicarica() {
		if(serialize != null) {
			try {
				String [] escluse = {"serialize"};
				Map<String, Object> a = ApplicationUtil.proprieta(this, escluse);
				BaseCriteria c = (BaseCriteria) ApplicationUtil.deserialize(serialize);
				if(c != null) {
					c.doImposta();
				}
				Map<String, Object> b = ApplicationUtil.proprieta(c, escluse);
				return !ApplicationUtil.equals(a, b);
			} catch (Exception e) {
			}
		}
		return false;
	}


}
