package it.callite.criteria.generic;

import java.io.Serializable;
import java.util.Map;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.ImpostazioneDao;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.intervallo.Intervallo1;
import it.callite.utility.intervallo.Intervallo2;

public abstract class GenericFileCriteria extends UnitedCriteria implements Serializable {
	
	private static final long serialVersionUID = 1L;

	private Intervallo1 file1 = new Intervallo1();
	private Intervallo2 file2 = new Intervallo2();
	private Boolean tasto = false;
	private Boolean visibileId = false;
	private String ids;
	
	public boolean isRicercabile() {
		return false;
	}

	public String getIds() {
		return ApplicationUtil.trimToNull(ids);
	}

	public void setIds(String ids) {
		this.ids = ApplicationUtil.trimToNull(ids);
	}



	public Boolean getVisibileId() {
		return visibileId;
	}

	public void setVisibileId(Boolean visibileId) {
		this.visibileId = visibileId;
	}

	public Boolean getTasto() {
		return tasto;
	}


	public void doImposta(){
		if(ApplicationUtil.isNotBlank(file1.getPeriodo())) {
			if(file1.getPeriodo().equals("reset")) {
				file1.reset();
			} else {
				String [] p = file1.getPeriodo().split(ConstantsGlobal.colonSeparator);
				String periodo = p[0];
				Integer ii = Integer.parseInt(p[1]);
				if(periodo.equals(ConstantsMap.p_g)){
					file1.setInizio(ApplicationUtil.inizioGiornata(ii));
					file1.setFine(ApplicationUtil.fineGiornata(ii));
				}
				if(periodo.equals(ConstantsMap.p_s)){
					file1.setInizio(ApplicationUtil.inizioSettimana(ii));
					file1.setFine(ApplicationUtil.fineSettimana(ii));
				}
				if(periodo.equals(ConstantsMap.p_m)){
					file1.setInizio(ApplicationUtil.inizioMese(ii));
					file1.setFine(ApplicationUtil.fineMese(ii));
				}
				if(periodo.equals(ConstantsMap.p_a)){
					file1.setInizio(ApplicationUtil.inizioAnno(ii));
					file1.setFine(ApplicationUtil.fineAnno(ii));
				}
			}
		} 
	}

	public Intervallo1 getFile1() {
		return file1;
	}

	public Intervallo2 getFile2() {
		return file2;
	}


	public boolean impostaRicercaDate(long lunghezza, long modifica) {
		boolean a = ApplicationUtil.dataIntervalD(file1.getInizio(), file1.getFine(), ApplicationUtil.date(modifica));
		boolean b = ApplicationUtil.dataIntervalL(file2.getInizio(), file2.getFine(), lunghezza);
		return a && b;
	}

	public GenericFileCriteria loadCriteria() {
		GenericFileCriteria c = (GenericFileCriteria) ApplicationUtil.deserialize(getSerialize());
		if(c != null) {
			c.doImposta();
			return c;
		}
		return this;
	}

	public void replace(String periodo) {
		if(periodo.equals(ConstantsMap.p_g)){
			file1.setInizio(ApplicationUtil.inizioGiornata(0));
			file1.setFine(ApplicationUtil.fineGiornata(0));
		}
		if(periodo.equals(ConstantsMap.p_s)){
			file1.setInizio(ApplicationUtil.inizioSettimana(0));
			file1.setFine(ApplicationUtil.fineSettimana(0));
		}
		if(periodo.equals(ConstantsMap.p_m)){
			file1.setInizio(ApplicationUtil.inizioMese(0));
			file1.setFine(ApplicationUtil.fineMese(0));
		}
		if(periodo.equals(ConstantsMap.p_a)){
			file1.setInizio(ApplicationUtil.inizioAnno(0));
			file1.setFine(ApplicationUtil.fineAnno(0));
		}
	}
	private String serialize;

	public String getSerialize() {
		return serialize;
	}

	public void setSerialize(String serialize) {
		this.serialize = serialize;
	}
	public boolean isRicarica() {
		if(serialize != null) {
			try {
				String [] escluse = {"serialize"};
				Map<String, Object> a = ApplicationUtil.proprieta(this, escluse);
				GenericFileCriteria c = (GenericFileCriteria) ApplicationUtil.deserialize(serialize);
				if(c != null) {
					c.doImposta();
				}
				Map<String, Object> b = ApplicationUtil.proprieta(c, escluse);
				return !ApplicationUtil.equals(a, b);
			} catch (Exception e) {
			}
		}
		return false;
	}


	public void salvaCriteria(ImpostazioneDao impostazioneDao, Integer b)  {
		if(ApplicationUtil.isTrue(impostazioneDao.getAbilitaRicercaManuale())) {
			setCerca(true);
			tasto = false;
		} else {
			setCerca(ApplicationUtil.ne(b, 1));
			tasto = ApplicationUtil.eq(b, 1);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getAbilitaRicercaTasto())) {
			if(ApplicationUtil.gtZero(b)) {
				this.setSerialize(null);
				setSerialize(ApplicationUtil.serialize(this));
			}
		} else {
			setSerialize(null);
		}
	}


	public void setFile1(Intervallo1 file1) {
		this.file1 = file1;
	}

	public void setFile2(Intervallo2 file2) {
		this.file2 = file2;
	}


}
