package it.callite.criteria.generic;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import it.callite.dao.DataDao;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.DatoAggiuntivoAnagraficaRicerca;
import it.callite.utility.DatoAggiuntivoContrattoRicerca;
import it.callite.utility.Label;

public abstract class RicercaAnagraficaCriteria extends GenericAllCriteria implements Serializable {
	
	private static final long serialVersionUID = 1L;

	private Long campoAnagraficaId;
	private Long campoContrattoId;

	private String valoriDatoAnagrafica;
	private String valoriDatoContratto;


	private List<DatoAggiuntivoAnagraficaRicerca> datiAggiuntivoAnagrafica = new ArrayList<DatoAggiuntivoAnagraficaRicerca>();
	private List<DatoAggiuntivoContrattoRicerca> datiAggiuntivoContratto = new ArrayList<DatoAggiuntivoContrattoRicerca>();
	
	private Integer indirizzo;
	private String provincia;
	private String regione;
	private String cap;
	private String strada;
	private String comune;
	private List<Label> comuni = new ArrayList<Label>();
	private List<Label> province = new ArrayList<Label>();
	private List<Label> regioni = new ArrayList<Label>();
	
	private Integer consenso_tfisso = 0;
	private Integer consenso_tmobile = 0;
	private Integer consenso_tfax = 0;
	private Integer consenso_taltro = 0;
	private Integer consenso_email = 0;
	private Integer consenso_rmail = 0;


	public void addDatiAggiuntivoAnagrafica(DatoAggiuntivoAnagraficaRicerca dato) {
		if(dato != null && !datiAggiuntivoAnagrafica.contains(dato)) {
			datiAggiuntivoAnagrafica.add(dato);
		}
	}

	public void addDatiAggiuntivoContratto(DatoAggiuntivoContrattoRicerca dato) {
		if(dato != null && !datiAggiuntivoContratto.contains(dato)) {
			datiAggiuntivoContratto.add(dato);
		}
	}
	public void cambiaProvincia(DataDao dataDao) {
		comune = null;
		if(ApplicationUtil.isNotBlank(provincia)) {
			regione = dataDao.getRegione(provincia);
		} 
		comuni = dataDao.getComuni(provincia);
	}

	public void  cambiaRegione(DataDao dataDao) {
		comune = null;
		provincia = null;
		comuni = dataDao.getComuni(null);
		province = dataDao.getProvince(regione);
	}

	public Long getCampoAnagraficaId() {
		return campoAnagraficaId;
	}

	public Long getCampoContrattoId() {
		return campoContrattoId;
	}

	public String getCap() {
		return ApplicationUtil.trimToNull(cap);
	}

	public String getComune() {
		return ApplicationUtil.trimToNull(comune);
	}

	public List<Label> getComuni() {
		return comuni;
	}

	public List<DatoAggiuntivoAnagraficaRicerca> getDatiAggiuntivoAnagrafica() {
		return datiAggiuntivoAnagrafica;
	}

	public List<DatoAggiuntivoContrattoRicerca> getDatiAggiuntivoContratto() {
		return datiAggiuntivoContratto;
	}

	public Integer getIndirizzo() {
		return indirizzo;
	}

	public List<Label> getProvince() {
		return province;
	}

	public String getProvincia() {
		return ApplicationUtil.trimToNull(provincia);
	}

	public String getRegione() {
		return ApplicationUtil.trimToNull(regione);
	}

	public List<Label> getRegioni() {
		return regioni;
	}
	public Boolean getValidoAnagrafica() {
		if(campoAnagraficaId != null) {
			return true;
		} else { 
			return false;
		}
	}
	public Boolean getValidoContratto() {
		if(campoContrattoId != null) {
			return true;
		} else { 
			return false;
		}
	}



	public String getValoriDatoAnagrafica() {
		return ApplicationUtil.trimToNull(valoriDatoAnagrafica);
	}

	public String getValoriDatoContratto() {
		return ApplicationUtil.trimToNull(valoriDatoContratto);
	}

	public void removeDatiAggiuntivoAnagrafica() {
		datiAggiuntivoAnagrafica.clear();
	}


	public void removeDatiAggiuntivoAnagrafica(DatoAggiuntivoAnagraficaRicerca dato) {
		datiAggiuntivoAnagrafica.remove(dato);
	}


	public void removeDatiAggiuntivoContratto() {
		datiAggiuntivoContratto.clear();
	}
	
	public void removeDatiAggiuntivoContratto(DatoAggiuntivoContrattoRicerca dato) {
		datiAggiuntivoContratto.remove(dato);
	}
	public void reset(Integer ok, DataDao dataDao) {
		if(ApplicationUtil.eq(ok, 1)) {
			comune = ApplicationUtil.trimToNull(getComune());
			provincia = ApplicationUtil.trimToNull(getProvincia());
			if(ApplicationUtil.isNotBlank(getRegione())) {
				regione = ApplicationUtil.trimToNull(getRegione());
			} else {
				regione = ApplicationUtil.trimToNull(dataDao.getRegione(provincia));
			}
			regioni = dataDao.getRegioni();
			province = dataDao.getProvince(regione);
			comuni = dataDao.getComuni(provincia);
		}
	}
	public void setCampoAnagraficaId(Long campoAnagraficaId) {
		this.campoAnagraficaId = campoAnagraficaId;
	}
	
	public void setCampoContrattoId(Long campoContrattoId) {
		this.campoContrattoId = campoContrattoId;
	}

	public void setCap(String cap) {
		this.cap = ApplicationUtil.trimToNull(cap);
	}

	public void setComune(String comune) {
		this.comune = ApplicationUtil.trimToNull(comune);
	}

	public void setDatiAggiuntivoAnagrafica(List<DatoAggiuntivoAnagraficaRicerca> datiAggiuntivoAnagrafica) {
		this.datiAggiuntivoAnagrafica = datiAggiuntivoAnagrafica;
	}
	
	public void setDatiAggiuntivoContratto(List<DatoAggiuntivoContrattoRicerca> datiAggiuntivoContratto) {
		this.datiAggiuntivoContratto = datiAggiuntivoContratto;
	}
	
	
	public void setIndirizzo(Integer indirizzo) {
		this.indirizzo = indirizzo;
	}
	public void setProvincia(String provincia) {
		this.provincia = ApplicationUtil.trimToNull(provincia);
	}

	public void setRegione(String regione) {
		this.regione = ApplicationUtil.trimToNull(regione);
	}
	

	public void setValoriDatoAnagrafica(String valoriDatoAnagrafica) {
		this.valoriDatoAnagrafica = ApplicationUtil.trimToNull(valoriDatoAnagrafica);
	}
	
	public void setValoriDatoContratto(String valoriDatoContratto) {
		this.valoriDatoContratto = ApplicationUtil.trimToNull(valoriDatoContratto);
	}

	public String getStrada() {
		return ApplicationUtil.trimToNull(strada);
	}

	public void setStrada(String strada) {
		this.strada = ApplicationUtil.trimToNull(strada);
	}

	public Integer getConsenso_tfisso() {
		return consenso_tfisso;
	}

	public void setConsenso_tfisso(Integer consenso_tfisso) {
		this.consenso_tfisso = consenso_tfisso;
	}

	public Integer getConsenso_tmobile() {
		return consenso_tmobile;
	}

	public void setConsenso_tmobile(Integer consenso_tmobile) {
		this.consenso_tmobile = consenso_tmobile;
	}

	public Integer getConsenso_tfax() {
		return consenso_tfax;
	}

	public void setConsenso_tfax(Integer consenso_tfax) {
		this.consenso_tfax = consenso_tfax;
	}

	public Integer getConsenso_taltro() {
		return consenso_taltro;
	}

	public void setConsenso_taltro(Integer consenso_taltro) {
		this.consenso_taltro = consenso_taltro;
	}

	public Integer getConsenso_email() {
		return consenso_email;
	}

	public void setConsenso_email(Integer consenso_email) {
		this.consenso_email = consenso_email;
	}

	public Integer getConsenso_rmail() {
		return consenso_rmail;
	}

	public void setConsenso_rmail(Integer consenso_rmail) {
		this.consenso_rmail = consenso_rmail;
	}

	

	
}
