package it.callite.dao;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.column.generic.GenericColumn;
import it.callite.constants.ConstantsColumn;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsPannello;
import it.callite.criteria.AgenteCriteria;
import it.callite.criteria.AppuntamentoCriteria;
import it.callite.criteria.NotaCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Agente;
import it.callite.model.Appuntamento;
import it.callite.model.Contatto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.EsitoAppuntamento;
import it.callite.model.Indirizzo;
import it.callite.model.Nota;
import it.callite.model.StoriaAppuntamento;
import it.callite.model.TemplateImportAppuntamento;
import it.callite.model.Utente;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.service.config.ConfigurazionePannelloService;
import it.callite.utility.Allegato;
import it.callite.utility.Annotazione;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.AppuntamentoDistanza;
import it.callite.utility.AppuntamentoVO;
import it.callite.utility.CalendarioAgenteVO;
import it.callite.utility.GiornoCalendarioAgenteVO;
import it.callite.utility.LoggerColonna;
import it.callite.utility.Orario;
import it.callite.utility.importa.ImportModelAppuntamento;
import it.callite.utility.item.EsitoAppuntamentoSI;
@SuppressWarnings("serial")
@Repository("appuntamentoDao")
public class AppuntamentoDao extends GenericDao {
	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private ConfigurazionePannelloService configurazionePannelloService;
	@Autowired
	private UserDao userDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private PannelloDao pannelloDao;
	@Autowired
	private DataDao dataDao;
	
	
	public List<Appuntamento> impostaConteggioAppuntamento(Utente utente, GenericColumn c,List<Appuntamento> appuntamenti) {
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			if(impostazioneDao.getTabellaDatiAggiuntivi() && c.getAggiuntiviA())  {
				Iterator<Appuntamento> objIter = appuntamenti.iterator();
				Session session = openSession();
				while(objIter.hasNext()) {
					Appuntamento appuntamento = objIter.next();
					Appuntamento a = (Appuntamento) session.get(Appuntamento.class, appuntamento.getId());
					Hibernate.initialize(a.getContatto().getAnagrafica().getDettagli());
					Iterator<DatoAggiuntivoAnagrafica> objd = a.getContatto().getAnagrafica().getDettagli().iterator();
					while(objd.hasNext()) {
						DatoAggiuntivoAnagrafica da = objd.next();
						appuntamento.getContatto().getAnagrafica().addDatoM(da.getCampo_aggiuntivo().getId(), da);
					}
				}
				closeSession(session);
			}
			List<Long> ll = ApplicationUtil.prendiIdAppuntamento(appuntamenti);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Appuntamento.class, new ColonnaQuery(ConstantsColumn.allegati, "allegati"), new ColonnaQuery(ConstantsColumn.storia, "storia"));
			Iterator<Appuntamento> it = appuntamenti.iterator();
			while (it.hasNext()) {
				Appuntamento o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return appuntamenti;
	}
	public List<Appuntamento> impostaConteggioAppuntamento(Utente utente, List<Appuntamento> appuntamenti) {
		if(ApplicationUtil.notEmpty(appuntamenti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			List<Long> ll = ApplicationUtil.prendiIdAppuntamento(appuntamenti);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(null, ll, Appuntamento.class, new ColonnaQuery("allegati", "allegati"));
			Iterator<Appuntamento> it = appuntamenti.iterator();
			while (it.hasNext()) {
				Appuntamento o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
		}
		return appuntamenti;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoOperatore(Utente user, AppuntamentoCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoOperatoreCriteria(user, app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaAppuntamentoOperatoreSize(Utente user,  AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoOperatoreCriteria(user, app);
		return getListSize(crit);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoPartner(Utente user, AppuntamentoCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoPartnerCriteria(user, app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaAppuntamentoPartnerSize(Utente user,  AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoPartnerCriteria(user, app);
		return getListSize(crit);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoSupervisore(Utente user, AppuntamentoCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoSupervisoreCriteria(user, app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaAppuntamentoSupervisoreSize(Utente user,  AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoSupervisoreCriteria(user, app);
		return getListSize(crit);
	}
	public int ricercaAppuntamentoCoordinatoreSize(Utente user,  AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoCoordinatoreCriteria(user, app);
		return getListSize(crit);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoCoordinatore(Utente user, AppuntamentoCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoCoordinatoreCriteria(user, app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaAppuntamentoManagerSize(AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoManagerCriteria(app);
		return getListSize(crit);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoManager(AppuntamentoCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoManagerCriteria(app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaAppuntamentoBackofficeSize(AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoBackofficeCriteria(app);
		return getListSize(crit);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoBackoffice(AppuntamentoCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoBackofficeCriteria(app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaAppuntamentoSupervisoreCriteria(Utente user, AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		impostazioneDao.subCriteriaRoleSupervisore(user.getId(), criteria, "campagna", "operatore");
		impostazioneDao.subCriteriaSupervisore(filtro, user.getId(), criteriaOp, "id",app.getTeam(), app.getTeams());
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteriaCA, "id", filtro,app.getTeam(), app.getTeams());
		}
		impostazioneDao.ricercaIndirizzo(criteriaA, app, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(app, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(app, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaAppuntamentoI(app.getStorico_operatoreId(), app, criteria);

		ApplicationUtil.proprieta(filtro, criteria, "stato", app.getStato(), app.getStatos());

		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, app.getCamp_sup(), criteriaCA, "nome");
		}
		if (ApplicationUtil.isNotBlank(app.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(app.getRicerca_anagraficaV(), criteriaA, app.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(app.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(app.getOperatoreId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteriaC, "id", app.getEsito(), app.getEsitos());
		if (ApplicationUtil.gtZero(app.getConfermato())){
			if(ApplicationUtil.eq(app.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	private ExtendedCriteria ricercaAppuntamentoOperatoreCriteria(Utente user, AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("esito");
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "campagna");
		criteriaC.add(Restrictions.eq("visibile", Boolean.TRUE));
		if(ApplicationUtil.isTrue(app.getAzione())) {
			impostazioneDao.subCriteriaRoleOperatore(user.getId(), criteria, "campagna", "operatore");
			impostazioneDao.subCriteriaOperatore(user.getId(), criteriaOp, "id", filtro, app.getTeam(), app.getTeams());
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaOperatore(user.getId(), criteriaCA, "id", filtro, app.getTeam(), app.getTeams());
			}
			if (ApplicationUtil.gtZero(app.getOperatoreId())) {
				criteriaOp.add(Restrictions.idEq(app.getOperatoreId()));
			}
		} else {
			criteriaOp.add(Restrictions.idEq(user.getId()));
		}

		impostazioneDao.ricercaIndirizzo(criteriaA, app, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(app, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(app, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaAppuntamentoI(null, app, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "stato", app.getStato(), app.getStatos());
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getCamp_sup(), criteriaCA, "nome");
		}
		if (ApplicationUtil.isNotBlank(app.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(app.getRicerca_anagraficaV(), criteriaA, app.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteriaC, "id", app.getEsito(), app.getEsitos());
		if (ApplicationUtil.gtZero(app.getConfermato())){
			if(ApplicationUtil.eq(app.getConfermato(), 1) ) {
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			} else { 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
			}
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	private ExtendedCriteria ricercaAppuntamentoPartnerCriteria(Utente user, AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore");
		DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCa = criteriaCo.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteriaCa, "id");
		impostazioneDao.subCriteriaRicercaTeam(filtro, app.getTeam(), app.getTeams(), criteriaOp, "id");
		if (ApplicationUtil.gtZero(app.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(app.getOperatoreId()));
		}
		impostazioneDao.ricercaIndirizzo(criteriaA, app, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(app, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(app, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaAppuntamentoI(null, app, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "stato", app.getStato(), app.getStatos());
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, app.getCamp_sup(), criteriaCa, "nome");
		}
		if (ApplicationUtil.isNotBlank(app.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(app.getRicerca_anagraficaV(), criteriaA, app.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteriaC, "id", app.getEsito(), app.getEsitos());
		if (ApplicationUtil.gtZero(app.getConfermato())){
			if(ApplicationUtil.eq(app.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	private ExtendedCriteria ricercaAppuntamentoCoordinatoreCriteria(Utente user, AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "agente");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		impostazioneDao.subCriteriaRoleCoordinatore(user.getId(), criteria, "campagna", "agente");
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteriaAg.add(Restrictions.idEq(app.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCoordinatore(filtro, criteriaAg, user.getId(), app.getZona(), app.getZonas());
		}
		
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCoordinatoreZoneCampagne(filtro, criteriaCA, user.getId(), app.getZona(), app.getZonas());
		}
		
		if (ApplicationUtil.isNotBlank(app.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, app.getCamp_sup(), criteriaCA, "nome");
		}
		impostazioneDao.ricercaIndirizzo(criteriaA, app, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(app, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(app, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaAppuntamentoS(app.getStorico_operatore(), app, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "stato", app.getStato(), app.getStatos());
		if (ApplicationUtil.isNotBlank(app.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(app.getRicerca_anagraficaV(), criteriaA, app.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteriaC, "id", app.getEsito(), app.getEsitos());
		if (ApplicationUtil.gtZero(app.getConfermato())){
			if(ApplicationUtil.eq(app.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.isNotBlank(app.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, app.getOperatore());
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	private ExtendedCriteria ricercaAppuntamentoManagerCriteria(AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna");
		DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		impostazioneDao.ricercaIndirizzo(criteriaA, app, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(app, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(app, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaAppuntamentoI(app.getStorico_operatoreId(), app, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "stato", app.getStato(), app.getStatos());
		impostazioneDao.subCriteriaRicercaTeam(filtro, app.getTeam(), app.getTeams(), criteriaOp, "id");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaRicercaCampagna(filtro, app.getTeam(), app.getTeams(), criteriaCA, "id");
		}
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, app.getCamp_sup(), criteriaCA, "nome");
		}
		if (ApplicationUtil.isNotBlank(app.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(app.getRicerca_anagraficaV(), criteriaA, app.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(app.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(app.getOperatoreId()));
		}
		if (ApplicationUtil.gtZero(app.getUtenteId())) {
			criteria.add(Restrictions.eq("utente.id", app.getUtenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteriaC, "id", app.getEsito(), app.getEsitos());
		if (ApplicationUtil.gtZero(app.getConfermato())){
			if(ApplicationUtil.eq(app.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	private ExtendedCriteria ricercaAppuntamentoBackofficeCriteria(AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaCo.createCriteria("campagna");
		DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteriaCo.createCriteria("esitoArchiviazione").add(Restrictions.eq("backoffice", Boolean.TRUE));
		impostazioneDao.ricercaIndirizzo(criteriaA, app, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(app, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(app, criteriaA);
		}
		impostazioneDao.sottoCriteriaStoriaAppuntamentoI(app.getStorico_operatoreId(), app, criteria);
		ApplicationUtil.proprieta(filtro, criteria, "stato", app.getStato(), app.getStatos());
		impostazioneDao.subCriteriaRicercaTeam(filtro, app.getTeam(), app.getTeams(), criteriaOp, "id");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaRicercaCampagna(filtro, app.getTeam(), app.getTeams(), criteriaCA, "id");
		}
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getCamp_sup(),criteriaCA, "nome");
		}
		if (ApplicationUtil.isNotBlank(app.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(app.getRicerca_anagraficaV(), criteriaA, app.getRicerca_anagrafica());
		}
		if (ApplicationUtil.gtZero(app.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(app.getOperatoreId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteriaC, "id", app.getEsito(), app.getEsitos());
		if (ApplicationUtil.gtZero(app.getConfermato())){
			if(ApplicationUtil.eq(app.getConfermato(), 1))
				criteriaC.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteriaC.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaAppuntamentoAgenteSize(Utente utente,AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoAgenteCriteria(utente, app);
		return getListSize(crit);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoAgente(Utente utente,AppuntamentoCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoAgenteCriteria(utente, app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaAppuntamentoAgenteCriteria(Utente utente, AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
		DetachedCriteria criteriaEs = criteria.createCriteria("esito");
		criteriaEs.add(Restrictions.eq("confermato", Boolean.TRUE));
		if(ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaAgente(utente.getId(), criteria, "agente.id");
		}
		impostazioneDao.ricercaIndirizzo(criteriaA, app, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(app, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(app, criteriaA);
		}
		if (ApplicationUtil.isNotBlank(app.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, app.getCamp_sup(),  criteriaCo.createCriteria("campagna"), "nome");
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", app.getStato(), app.getStatos());
		ApplicationUtil.proprieta(false, filtro, criteriaEs, "id", app.getEsito(), app.getEsitos());
		if (ApplicationUtil.isNotBlank(app.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(app.getRicerca_anagraficaV(), criteriaA, app.getRicerca_anagrafica());
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaAppuntamentoReteSize(Utente utente,AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoReteCriteria(utente, app);
		return getListSize(crit);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoRete(Utente utente,AppuntamentoCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoReteCriteria(utente, app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaAppuntamentoReteCriteria(Utente utente, AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Integer max = userDao.getMaxLivelloRete();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
		DetachedCriteria criteriaEs = criteria.createCriteria("esito");
		DetachedCriteria critA = criteria.createCriteria("agente");
		criteriaEs.add(Restrictions.eq("confermato", Boolean.TRUE));
		if (ApplicationUtil.gtZero(app.getRete())) {
			if(ApplicationUtil.eq(app.getRete(), 1)) {
				if (ApplicationUtil.gtZero(app.getAgenteId())) {
					critA.createCriteria("agente_padre").add(Restrictions.idEq(app.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(app.getRete(), 2)) {
				if (ApplicationUtil.gtZero(app.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(app.getAgenteId(), true, max, critA, "id");
				}
			}
			if(ApplicationUtil.eq(app.getRete(), 3)) {
				if (ApplicationUtil.gtZero(app.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(app.getAgenteId(), true, max, critA, "id");
					critA.add(Restrictions.ne("id", app.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(app.getRete(), 4)) {
				if (ApplicationUtil.gtZero(app.getAgenteId())) {
					critA.add(Restrictions.idEq(app.getAgenteId()));
				}
			}
		} else {
			impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 0, max, critA, "id");
		}
		if (ApplicationUtil.isNotBlank(app.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, app.getCamp_sup(), criteriaCo.createCriteria("campagna"), "nome");
		}
		impostazioneDao.ricercaIndirizzo(criteriaA, app, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(app, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(app, criteriaA);
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", app.getStato(), app.getStatos());
		ApplicationUtil.proprieta(false, filtro, criteriaEs, "id", app.getEsito(), app.getEsitos());
		if (ApplicationUtil.isNotBlank(app.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(app.getRicerca_anagraficaV(), criteriaA, app.getRicerca_anagrafica());
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaAppuntamentoProssimiAgenteSize(Long utente, AppuntamentoCriteria app) {
		ExtendedCriteria crit = ricercaAppuntamentoProssimiAgenteCriteria(utente, app);
		return getListSize(crit);
	}
	public String trovaAppuntamentiAgenteMap(List<Appuntamento> appunta) {
		List<String> lista = new ArrayList<String>();
		if(ApplicationUtil.notEmpty(appunta)) {
			Iterator<Appuntamento> appIter = appunta.iterator();
			while (appIter.hasNext()) {
				Appuntamento appuntamento = (Appuntamento) appIter.next();
				if(appuntamento != null && appuntamento.getId() != null) {
					String a = appuntamento.getId().toString();
					if(!lista.contains(a)) {
						lista.add(a);
					}
				}
			}
		}
		return ApplicationUtil.detoken(lista, ConstantsGlobal.pipeSeparator);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> ricercaAppuntamentoProssimiAgente(Long utente, AppuntamentoCriteria app,  int start, int size) {
		ExtendedCriteria criteria = ricercaAppuntamentoProssimiAgenteCriteria(utente, app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.asc("data_inizio"), Order.asc("data_fine")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaAppuntamentoProssimiAgenteCriteria(Long utente, AppuntamentoCriteria app) {
		if(app != null)  app = (AppuntamentoCriteria) app.loadCriteria();
		Date now = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria critEsito = criteria.createCriteria("esito");
		critEsito.add(Restrictions.eq("confermato", Boolean.TRUE));
		if(ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaAgente(utente, criteria,"agente.id");
		}
		if (ApplicationUtil.gtZero(app.getStato())) {
			criteria.add(Restrictions.eq("stato", app.getStato()));
		}
		if(ApplicationUtil.gtZero(app.getEsito()))
			critEsito.add(Restrictions.idEq(app.getEsito()));
		if(ApplicationUtil.gtZero(app.getGiorni())) {
			Date a = ApplicationUtil.addDays(now,  app.getGiorni());
			ApplicationUtil.dataDate(now, a, "data_inizio", criteria);
		} else {
			criteria.add(Restrictions.gt("data_inizio", now));
		}
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	private ExtendedCriteria trovaAgentePadreCriteria(Long agenteId, AgenteCriteria ana) {
		if(ana != null)  ana = (AgenteCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if (ApplicationUtil.gtZero(agenteId)) {
			impostazioneDao.subCriteriaAgentiNotInSottoRete(agenteId,criteria, "id");
		}
		if (ApplicationUtil.isNotBlank(ana.getRicerca_denominazione())) {
			ApplicationUtil.denominazione(criteria, ana.getRicerca_denominazione());
		}
		if (ApplicationUtil.isNotBlank(ana.getRicerca_fiscale())) {
			ApplicationUtil.fiscale(criteria, ana.getRicerca_fiscale());
		}
		if (ApplicationUtil.isNotBlank(ana.getNumero())) {
			ApplicationUtil.telefonoA(criteria, ana.getNumero());
		}
		if (ApplicationUtil.isNotBlank(ana.getCodice())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  ana.getCodice(), criteria, "codice");
		}
		if (ApplicationUtil.gtZero(ana.getZona())) {
			criteria.createCriteria("zone").add(Restrictions.idEq(ana.getZona()));
		}
		if (ApplicationUtil.gtZero(ana.getQualifica())) {
			criteria.createCriteria("qualifica").add(Restrictions.idEq(ana.getQualifica()));
		}
		if (ApplicationUtil.gtZero(ana.getUfficio())) {
			criteria.createCriteria("ufficio").add(Restrictions.idEq(ana.getUfficio()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaAgentePadre(Long agenteId, AgenteCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaAgentePadreCriteria(agenteId, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), 	new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaAgentePadreSize(Long agenteId, AgenteCriteria ana) {
		ExtendedCriteria criteria = trovaAgentePadreCriteria(agenteId, ana);
		return getListSize(criteria);
	}
	
	
	public List<Agente> impostaConteggioAgente(GenericColumn c, List<Agente> agenti) {
		if(ApplicationUtil.notEmpty(agenti))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdAgente(agenti);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Agente.class, new ColonnaQuery(ConstantsColumn.allegati, "allegati"), new ColonnaQuery(ConstantsColumn.zone, "zone"), new ColonnaQuery(ConstantsColumn.utenti, "user"));
			Iterator<Agente> it = agenti.iterator();
			while (it.hasNext()) {
				Agente o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return agenti;
	}
	@SuppressWarnings("unchecked")
	public List<Agente> trovaAgente(AgenteCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaAgenteCriteria(ana);
		criteria.getCriteria().setFetchMode("agente_padre", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaAgenteSize(AgenteCriteria ana) {
		ExtendedCriteria criteria = trovaAgenteCriteria(ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaAgenteCriteria(AgenteCriteria ana) {
		if(ana != null)  ana = (AgenteCriteria) ana.loadCriteria();
		Integer max = userDao.getMaxLivelloRete();
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if (ApplicationUtil.gtZero(ana.getRete())) {
			if(ApplicationUtil.eq(ana.getRete(), 1)) {
				if (ApplicationUtil.gtZero(ana.getAgentePadreId())) {
					criteria.createCriteria("agente_padre").add(Restrictions.idEq(ana.getAgentePadreId()));
				}
			}
			if(ApplicationUtil.eq(ana.getRete(), 2)) {
				if (ApplicationUtil.gtZero(ana.getAgentePadreId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(ana.getAgentePadreId(), false, max, criteria, "id");
				}
			}
			if(ApplicationUtil.eq(ana.getRete(), 3)) {
				if (ApplicationUtil.gtZero(ana.getAgentePadreId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(ana.getAgentePadreId(), false, max, criteria, "id");
					criteria.add(Restrictions.ne("id", ana.getAgentePadreId()));
				}
			}
		}
		if (ApplicationUtil.isNotBlank(ana.getRicerca_denominazione())) {
			ApplicationUtil.denominazione(criteria, ana.getRicerca_denominazione());
		}
		if (ApplicationUtil.isNotBlank(ana.getRicerca_fiscale())) {
			ApplicationUtil.fiscale(criteria, ana.getRicerca_fiscale());
		}
		if (ApplicationUtil.isNotBlank(ana.getNumero())) {
			ApplicationUtil.telefonoA(criteria, ana.getNumero());
		}
		if (ApplicationUtil.isNotBlank(ana.getCodice())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getCodice(), criteria, "codice");
		}
		if (ApplicationUtil.gtZero(ana.getZona())) {
			criteria.createCriteria("zone").add(Restrictions.idEq(ana.getZona()));
		}
		if (ApplicationUtil.gtZero(ana.getQualifica())) {
			criteria.createCriteria("qualifica").add(Restrictions.idEq(ana.getQualifica()));
		}
		if (ApplicationUtil.gtZero(ana.getUfficio())) {
			criteria.createCriteria("ufficio").add(Restrictions.idEq(ana.getUfficio()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	public void completaCalendario(List<GiornoCalendarioAgenteVO> calendario, List<Appuntamento> appuntamenti, List<Nota> note, List<Orario> orari, int oramin, int oramax, Date attuale)  {
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			Iterator<Appuntamento> objIter = appuntamenti.iterator();
			while(objIter.hasNext()) {
				Appuntamento app = objIter.next();
				app.setOraMin(oramin);
				app.setOraMax(oramax);
				if(ApplicationUtil.isTrue(app.getCentrato())) {
					boolean trovato = false;
					Iterator<GiornoCalendarioAgenteVO> it = calendario.iterator();
					while (trovato == false && it.hasNext()) {
						GiornoCalendarioAgenteVO gc = it.next();
						if (ApplicationUtil.isSameDay(app.getData_inizio(), gc.getData())	|| ApplicationUtil.isSameDay(app.getData_fine(), gc.getData())) {
							gc.getAppuntamenti().add(app);
							trovato = true;
						}
					}
				}
			}
		}
		if(ApplicationUtil.notEmpty(note)) {
			Iterator<Nota> ntIter = note.iterator();
			while(ntIter.hasNext()) {
				Nota nota = ntIter.next();
				nota.setOraMin(oramin);
				nota.setOraMax(oramax);
				if(ApplicationUtil.isTrue(nota.getCentrato())) {
					if(ApplicationUtil.gtZero(nota.getRipetizioni())) {
						Iterator<GiornoCalendarioAgenteVO> it = calendario.iterator();
						while (it.hasNext()) {
							GiornoCalendarioAgenteVO gc = it.next();
							Date fine = ApplicationUtil.settaOrarioAData(gc.getData(), nota.getData_fine());
							Annotazione nn = new Annotazione();
							nn.setNota(nota);
							nn.setPassato(fine.before(attuale));
							if(nota.getRipetizioni().equals(ConstantsMap.ripeti_giorno)) {
								if(ApplicationUtil.giornoCorretto(nota, gc.getData())) {
									gc.getNote().add(nn);
								}
							}
							if(nota.getRipetizioni().equals(ConstantsMap.ripeti_settimana)) {
								if(ApplicationUtil.settimanaCorretto(nota, gc.getData())) {
									gc.getNote().add(nn);
								}
							}
							if(nota.getRipetizioni().equals(ConstantsMap.ripeti_mese)) {
								if(ApplicationUtil.meseCorretto(nota, gc.getData())) {
									gc.getNote().add(nn);
								}
							}
							if(nota.getRipetizioni().equals(ConstantsMap.ripeti_anno)) {
								if(ApplicationUtil.annoCorretto(nota, gc.getData())) {
									gc.getNote().add(nn);
								}
							}
						}
					} else {
						boolean trovato = false;
						Iterator<GiornoCalendarioAgenteVO> it = calendario.iterator();
						while (trovato == false && it.hasNext()) {
							GiornoCalendarioAgenteVO gc = it.next();
							Annotazione nn = new Annotazione();
							nn.setNota(nota);
							nn.setPassato(nota.getData_fine().before(attuale));
							if (ApplicationUtil.isSameDay(nota.getData_inizio(), gc.getData()) || ApplicationUtil.isSameDay(nota.getData_fine(), gc.getData())) {
								gc.getNote().add(nn);
								trovato = true;
							}
						}
					}
				}
			}
		}
		if(ApplicationUtil.notEmpty(orari)) {
			Iterator<Orario> objIter = orari.iterator();
			while(objIter.hasNext()) {
				Orario app = objIter.next();
				app.setOraMin(oramin);
				app.setOraMax(oramax);
				boolean trovato = false;
				Iterator<GiornoCalendarioAgenteVO> it = calendario.iterator();
				while (trovato == false && it.hasNext()) {
					GiornoCalendarioAgenteVO gc = it.next();
					if (ApplicationUtil.isSameDay(app.getData(), gc.getData())) {
						int time = ApplicationUtil.getTime(app.getData());
						if(time >= oramin && time <= oramax) {
							gc.getAdesso().add(app);
						}
						trovato = true;
					}
				}
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<Nota> trovaListaNote(Date inizio, Date fine, Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		criteria.add(Restrictions.eq("agente.id", agenteId));
		ApplicationUtil.dataNota(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria);
	}
	public List<CalendarioAgenteVO> getAppuntamentiList(List<Agente> agentiSel, Contatto contatto,  Date giornoPartenza, AppuntamentoCriteria appCr, Utente utente) {
		int inizioA = appCr.getInizioCC().intValue();
		int fineA = appCr.getFineCC().intValue();
		int giorni = appCr.getGiorniCC().intValue();
		List<String> sis = new ArrayList<String>();
		for(int i = inizioA; i <= fineA; i++) {
			sis.add(String.format("%02d", i, 0));
		}
		List<Agente> agenti = null;
		if (ApplicationUtil.gtZero(appCr.getAgenteId())) {
			agenti = new ArrayList<Agente>();
			agenti.add(campagnaDao.getAgente(appCr.getAgenteId()));
		} else {
			agenti = agentiSel;
		}
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		List<CalendarioAgenteVO> calendari = new ArrayList<CalendarioAgenteVO>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			List<GiornoCalendarioAgenteVO> calendario = completaListAppuntamenti(appuntamenti, contatto, giornoPartenza, agente, giorni, appCr, utente.getId(), inizioA, fineA);
			CalendarioAgenteVO cvo = new CalendarioAgenteVO();
			cvo.setAgente(agente);
			cvo.setGiorni(calendario);
			cvo.setOrari(sis);
			cvo.setGiorniCalendario(giorni);
			calendari.add(cvo);
		}
		if(contatto!=null) {
			if(ApplicationUtil.isTrue(impostazioneDao.getValidaCalendario())) {
				if(contatto.isOk()) {
					contatto.setTrovato(true);
				} else {
					if(ApplicationUtil.notEmpty(contatto.getMappaAppuntamenti())) {
						contatto.setTrovato(true);
					} else {
						contatto.setTrovato(false);
					}
				}
			}
			Integer g = Integer.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_GEOCODE_CONTROL));
			if(ApplicationUtil.gtZero(g)) {
				aggiustaDistanza(appuntamenti, contatto, g);
			}
			if(ApplicationUtil.notEmpty(appuntamenti))  {
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_APPUNTAMENTO);
			}
		}
		return calendari;
	}
	public void aggiustaDistanza(List<Appuntamento> appuntamenti, Contatto contatto, Integer giorni) {
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			Iterator<Appuntamento> it = appuntamenti.iterator();
			while (it.hasNext()) {
				Appuntamento appuntamento = it.next();
				if(ApplicationUtil.gtZero(appuntamento.getId()) && appuntamento.getAgente() != null) {
					Date fine = appuntamento.getData_fine();
					Date attuale = ApplicationUtil.date();
					if(fine.after(attuale) && fine.before(ApplicationUtil.fineGiornata(ApplicationUtil.addDays(attuale,  giorni)))) {
						Indirizzo i = ApplicationUtil.isTrue(appuntamento.getLocalizzazione()) ? appuntamento.getContatto().getAnagrafica().getIndirizzo_operativo() : appuntamento.getContatto().getAnagrafica().getIndirizzo_legale();
						Double d1 = ApplicationUtil.calcolaDistanza(contatto.getAnagrafica().getIndirizzo_legale(), i);
						Double d2 = ApplicationUtil.calcolaDistanza(contatto.getAnagrafica().getIndirizzo_operativo(), i);
						Double d = null;
						if(d1 != null && d2 == null) {
							d = d1;
						}
						if(d1 == null && d2 != null) {
							d = d2;
						}
						if(d1 != null && d2 != null) {
							if(d1.doubleValue() > d2.doubleValue()) {
								d = d2; 
							} else {
								d = d1;
							}
						}
						if(d != null) {
							AppuntamentoDistanza da = contatto.getPosizioneAppuntamento(appuntamento.getAgente().getId());
							Double distanza = da.getDistanza();
							if(distanza == null || (distanza != null && distanza.doubleValue() > d.doubleValue())) {
								da.setDistanza(d);
								da.setAppuntamento(appuntamento.getId());
							} 
						}
					}
				}
			}
		}
	}
	private List<GiornoCalendarioAgenteVO> completaListAppuntamenti(List<Appuntamento> appuntamentiTot, Contatto contatto, Date giornoPartenza, Agente agente, int giorni, AppuntamentoCriteria appCr, Long userId, int oramin, int oramax) {
		List<GiornoCalendarioAgenteVO> calendario = new ArrayList<GiornoCalendarioAgenteVO>();
		Date attuale = ApplicationUtil.date();
		if (agente != null) {
			Date inizio = ApplicationUtil.inizioGiornata(giornoPartenza);
			Date fine = ApplicationUtil.fineGiornata(ApplicationUtil.addDays(inizio, giorni));
			for(int i = 0; i < giorni; i++) {
				Date giorno_calendario = ApplicationUtil.addDays(inizio, i);
				GiornoCalendarioAgenteVO gc = new GiornoCalendarioAgenteVO();
				gc.setAppuntamenti(new ArrayList<Appuntamento>());
				gc.setNote(new ArrayList<Annotazione>());
				gc.setData(giorno_calendario);
				gc.setAgente(agente);
				calendario.add(gc);
			}
			List<Appuntamento> appuntamenti = trovaListaAppuntamenti(contatto, inizio, fine, agente.getId(),  appCr, userId);
			if(ApplicationUtil.empty(appuntamenti)) {
				appuntamenti = new ArrayList<Appuntamento>();
			}
			Map<Long, Appuntamento> map = contatto.getMappaAppuntamenti();
			if(ApplicationUtil.notEmpty(map)) {
				Iterator<Long> it = map.keySet().iterator();
				while (it.hasNext()) {
					Appuntamento appuntamento = map.get(it.next());
					if(appuntamento.getAgente()!= null && appuntamento.getAgente().equals(agente)) {
						appuntamenti.add(appuntamento);
					}
				}
			}
			List<Nota> note = trovaListaNote(inizio, fine, agente.getId());
			List<Orario> orari = new ArrayList<Orario>();
			Orario orario = new Orario();
			orario.setData(attuale);
			orari.add(orario);
			completaCalendario(calendario, appuntamenti, note, orari, oramin, oramax, attuale);
			appuntamentiTot.addAll(appuntamenti);
		}
		return calendario;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamenti(Contatto contatto, Date inizio, Date fine, Long agenteId, AppuntamentoCriteria appCr, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaO = criteria.createCriteria("operatore");
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaC = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteria.add(Restrictions.eq("agente.id", agenteId));
		if (ApplicationUtil.gtZero(appCr.getEsito())) {
			criteriaC.add(Restrictions.idEq(appCr.getEsito()));
		} 
		if (ApplicationUtil.gtZero(appCr.getStato())) {
			criteria.add(Restrictions.eq("stato", appCr.getStato()));
		}
		if (ApplicationUtil.gtZero(appCr.getOperatoreId())) {
			criteriaO.add(Restrictions.idEq(appCr.getOperatoreId()));
		} 
		if (appCr.getSoloContatto()) {
			criteriaCo.add(Restrictions.idEq(contatto.getId()));
		} 
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> trovaListaAppuntamenti(Date inizio, Date fine, Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		criteria.createCriteria("esito").add(Restrictions.eq("confermato", Boolean.TRUE));
		criteria.add(Restrictions.eq("agente.id", agenteId));
		ApplicationUtil.dataDate(inizio, fine, "data_inizio", criteria);
		criteria.addOrder(Order.desc("id"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> getAppuntamentiByContattoId(Boolean supervisore, Long contattoId,  AppuntamentoCriteria app, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto");
		DetachedCriteria criteriaO = criteria.createCriteria("operatore");
		DetachedCriteria criteriaA = criteria.createCriteria("agente", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaEs = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteriaCo.add(Restrictions.idEq(contattoId));
		if(ApplicationUtil.isFalse(supervisore)) { 
			criteriaEs.add(Restrictions.eq("visibile", Boolean.TRUE));
		}
		if (ApplicationUtil.gtZero(app.getEsito())) {
			criteriaEs.add(Restrictions.idEq(app.getEsito()));
		}
		if (ApplicationUtil.gtZero(app.getOperatoreId())) {
			criteriaO.add(Restrictions.idEq(app.getOperatoreId()));
		} 
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(app.getAgenteId()));
		} 
		criteria.addOrder(Order.asc("data_creazione"));
		return getList(criteria);
	}
	public void getRiepilogo(Utente utente, List<Agente> agentiPan, AppuntamentoCriteria app, DownloadBean downloadBean) {
		String html = impostazioneDao.getFormatoDocumento();
		List<Agente> agenti = new ArrayList<Agente>();
		if(ApplicationUtil.gtZero(app.getAgenteId())) {			
			agenti.add(userDao.getAgente(app.getAgenteId()));
		} else {
			agenti.addAll(agentiPan);
		}
		Date ora = ApplicationUtil.date();
		List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
		Iterator<Agente> objIter = agenti.iterator();
		while(objIter.hasNext()) {
			Agente agente = objIter.next();
			Date a = null;
			if(ApplicationUtil.gtZero(app.getGiorni())) {
				a = ApplicationUtil.addDays(ora,  app.getGiorni());
			}
			appuntamenti.addAll(trovaListaAppuntamenti(ora, a, agente.getId()));
		}
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			String pp = ConstantsGlobal.secretL;
			String p = impostazioneDao.getPasswordZIP();
			String extM = (pp == null || (html.equalsIgnoreCase(ConstantsMap.pdf))) ? html : ConstantsMap.zip;
			Iterator<Appuntamento> objAppIter = appuntamenti.iterator();
			while(objAppIter.hasNext()) {
				Appuntamento appuntamento = objAppIter.next();
				try {
					String nomeFile = ApplicationUtil.reportFile(appuntamento.getData_inizio(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione())  + "." + extM;
					boolean vs = appuntamento.getAgente() != null && ApplicationUtil.isTrue(appuntamento.getAgente().getVisualizzaStoria());
					ByteArrayOutputStream result = downloadDao.appuntamentoToHTMLString(utente, appuntamento, vs, true, html, true, false, pp);
					result.close();
					ApplicationUtil.addFile(files, nomeFile, result.toByteArray(), appuntamento.getData_creazione().getTime());
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
			String pg = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, pg ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
			if (ApplicationUtil.empty(files)) {
				ApplicationUtil.fileWriter(files, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			if (ApplicationUtil.notEmpty(files)) {
				try {
					String fileName = ApplicationUtil.reportFile(ora, ConstantsLanguage.APPUNTAMENTI.stampa()) + "." + ConstantsMap.zip;
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(fileName, files);
					ApplicationUtil.zip(null,files, out, p);
					downloadBean.close();
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			} 
		} 
	}
	public void revertStoriaAppuntamento(Appuntamento  appuntamento, StoriaAppuntamento storia) {
		appuntamento.setData_lavorazione(storia.getData());
		appuntamento.setUtente(storia.getUtente());
		appuntamento.setBloccato(storia.getBloccato());
		appuntamento.setOperatore(storia.getOperatore());
		appuntamento.setAgente(storia.getAgente());
		appuntamento.setData_conferma(storia.getData_conferma());
		appuntamento.setData_esito(storia.getData_esito());
		appuntamento.setData_stato(storia.getData_stato());
		appuntamento.setLocalizzazione(storia.getLocalizzazione());
		appuntamento.setStato(storia.getStato());
		appuntamento.setData_inizio(storia.getData_inizio());
		appuntamento.setData_fine(storia.getData_fine());
		appuntamento.setNote(storia.getNote());
		appuntamento.setEsito(storia.getEsito());
	}
	public void revertAppuntamento(List<Appuntamento> appuntamenti, Utente utente, Date data, Integer step, Boolean modo) {
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			List<Appuntamento> appuntamentiList = new ArrayList<Appuntamento>();
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Appuntamento> objIter = appuntamenti.iterator();
			while(objIter.hasNext()) {
				Appuntamento old = objIter.next();
				Appuntamento appuntamento = (Appuntamento) session.get(Appuntamento.class, old.getId());
				Hibernate.initialize(appuntamento.getStoria());
				List<StoriaAppuntamento> storia = appuntamento.getStoria(); 
				if(ApplicationUtil.notEmpty(storia)) {
					List<StoriaAppuntamento> removeList = new ArrayList<StoriaAppuntamento>();
					Iterator<StoriaAppuntamento> objSt = storia.iterator();
					int j = 0;
					while(objSt.hasNext()) {
						StoriaAppuntamento st = objSt.next();
						if(ApplicationUtil.isTrue(modo)) {
							if(j+step > storia.size()-1) {
								removeList.add(st);
							}
							j++;
						} else {
							if(st.getData().after(data)) {
								removeList.add(st);
							}
						}
					}
					if(ApplicationUtil.notEmpty(removeList)){
						boolean b = storia.size() == removeList.size();
						if(b) {
							removeList.remove(0);
						} 
						storia.removeAll(removeList);
					}
					if(ApplicationUtil.notEmpty(storia)) {
						if(ApplicationUtil.notEmpty(removeList)){
							Iterator<StoriaAppuntamento> objRm = removeList.iterator();
							while(objRm.hasNext()) {
								session.delete(objRm.next());
							}
						}
						StoriaAppuntamento el1 = storia.remove(storia.size()-1);
						revertStoriaAppuntamento(appuntamento, el1);
						session.update(appuntamento);
						appuntamentiList.add(appuntamento);
						if (i++ % ConstantsGlobal.flushSize == 0) {
							session.flush();
							session.clear();
						}
					}
				}
			}
			tx.commit();
			closeSession(session);

			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamentiList), appuntamentiList);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_REVERT_APPUNTAMENTO);
		}
	}
	public void revertAppuntamento(Appuntamento contatto, Utente utente, Date data, Integer step, Boolean modo) {
		if(contatto != null) {
			List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
			appuntamenti.add(contatto);
			revertAppuntamento(appuntamenti, utente, data, step, modo);
		}
	}
	private ExtendedCriteria ricercaNotaManagerCriteria(NotaCriteria app) {
		if(app != null)  app = (NotaCriteria) app.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} 
		if (ApplicationUtil.gtZero(app.getUtenteId())) {
			criteria.add(Restrictions.eq("operatore.id", app.getUtenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getNota(), criteria, "note");
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaNotaManagerSize(NotaCriteria app) {
		ExtendedCriteria criteria = ricercaNotaManagerCriteria(app);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> ricercaNotaManager(NotaCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaNotaManagerCriteria(app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaNotaBackofficeCriteria(Utente utente,NotaCriteria app) {
		if(app != null)  app = (NotaCriteria) app.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", app.getAgenteId()));
		} 
		if (ApplicationUtil.gtZero(app.getUtenteId())) {
			criteria.add(Restrictions.eq("operatore.id", app.getUtenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getNota(), criteria, "note");
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaNotaBackofficeSize(Utente utente,NotaCriteria app) {
		ExtendedCriteria criteria = ricercaNotaBackofficeCriteria(utente,app);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> ricercaNotaBackoffice(Utente utente,NotaCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaNotaBackofficeCriteria(utente,app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaNotaSupervisoreCriteria(Utente utente,NotaCriteria app) {
		if(app != null)  app = (NotaCriteria) app.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		DetachedCriteria criteriaA = criteria.createCriteria("operatore");
		impostazioneDao.subCriteriaSupervisore(utente.getId(), criteriaA, "id");
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(app.getAgenteId()));
		} 
		if (ApplicationUtil.gtZero(app.getUtenteId())) {
			criteria.add(Restrictions.eq("operatore.id", app.getUtenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getNota(), criteria, "note");
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> ricercaNotaSupervisore(Utente utente,NotaCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaNotaSupervisoreCriteria(utente,app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaNotaSupervisoreSize(Utente utente,NotaCriteria app) {
		ExtendedCriteria criteria = ricercaNotaSupervisoreCriteria(utente,app);
		return getListSize(criteria);
	}

	private ExtendedCriteria ricercaNotaCoordinatoreCriteria(Utente utente,NotaCriteria app) {
		if(app != null)  app = (NotaCriteria) app.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		DetachedCriteria criteriaA = criteria.createCriteria("agente");
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(app.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCoordinatore(utente.getId(), criteriaA, "id");
		}
		if(ApplicationUtil.isNotBlank(app.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, app.getUtente());
		}
		if (ApplicationUtil.isNotBlank(app.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getNota(), criteria, "note");
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaNotaCoordinatoreSize(Utente utente,NotaCriteria app) {
		ExtendedCriteria criteria = ricercaNotaCoordinatoreCriteria(utente,app);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> ricercaNotaCoordinatore(Utente utente,NotaCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaNotaCoordinatoreCriteria(utente,app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaNotaPartnerCriteria(Utente utente,NotaCriteria app) {
		if(app != null)  app = (NotaCriteria) app.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		DetachedCriteria criteriaA = criteria.createCriteria("agente");
		DetachedCriteria criteriaC = criteriaA.createCriteria("zone").createCriteria("campagne");
		if (ApplicationUtil.gtZero(app.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(app.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCampagnaPartner(utente.getId(), criteriaC, "id");
		}
		if (ApplicationUtil.gtZero(app.getUtenteId())) {
			criteria.add(Restrictions.eq("operatore.id", app.getUtenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getNota(), criteria, "note");
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaNotaPartnerSize(Utente utente,NotaCriteria app) {
		ExtendedCriteria criteria = ricercaNotaPartnerCriteria(utente,app);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> ricercaNotaPartner(Utente utente,NotaCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaNotaPartnerCriteria(utente,app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaNotaAgenteCriteria(Utente utente,NotaCriteria app) {
		if(app != null)  app = (NotaCriteria) app.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		DetachedCriteria criteriaA = criteria.createCriteria("agente");
		if(ApplicationUtil.gtZero(app.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(app.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaAgente(utente.getId(), criteriaA, "id");
		}
		if (ApplicationUtil.isNotBlank(app.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getNota(), criteria, "note");
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaNotaAgenteSize(Utente utente,NotaCriteria app) {
		ExtendedCriteria criteria = ricercaNotaAgenteCriteria(utente,app);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> ricercaNotaAgente(Utente utente,NotaCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaNotaAgenteCriteria(utente,app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaNotaReteCriteria(Utente utente,NotaCriteria app) {
		if(app != null)  app = (NotaCriteria) app.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		DetachedCriteria critA = criteria.createCriteria("agente");
		Integer max = userDao.getMaxLivelloRete();
		if (ApplicationUtil.gtZero(app.getRete())) {
			if(ApplicationUtil.eq(app.getRete(), 1)) {
				if (ApplicationUtil.gtZero(app.getAgenteId())) {
					critA.createCriteria("agente_padre").add(Restrictions.idEq(app.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(app.getRete(), 2)) {
				if (ApplicationUtil.gtZero(app.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(app.getAgenteId(), true, max, critA, "id");
				}
			}
			if(ApplicationUtil.eq(app.getRete(), 3)) {
				if (ApplicationUtil.gtZero(app.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(app.getAgenteId(), true, max, critA, "id");
					critA.add(Restrictions.ne("id", app.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(app.getRete(), 4)) {
				if (ApplicationUtil.gtZero(app.getAgenteId())) {
					critA.add(Restrictions.idEq(app.getAgenteId()));
				}
			}
		} else {
			impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 0, max, critA, "id");
		}
		if (ApplicationUtil.isNotBlank(app.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getNota(), criteria, "note");
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaNotaReteSize(Utente utente,NotaCriteria app) {
		ExtendedCriteria criteria = ricercaNotaReteCriteria(utente,app);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> ricercaNotaRete(Utente utente,NotaCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaNotaReteCriteria(utente,app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	private ExtendedCriteria ricercaNotaOperatoreCriteria(Utente utente,NotaCriteria app) {
		if(app != null)  app = (NotaCriteria) app.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Nota.class);
		DetachedCriteria criteriaA = criteria.createCriteria("agente");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore");
		if(ApplicationUtil.isTrue(app.getAzione())) {
			impostazioneDao.subCriteriaOperatore(utente.getId(), criteriaOp, "id");
			if (ApplicationUtil.gtZero(app.getOperatoreId())) {
				criteriaOp.add(Restrictions.idEq(app.getOperatoreId()));
			}
		} else {
			criteriaOp.add(Restrictions.idEq(utente.getId()));
		}
		if(ApplicationUtil.gtZero(app.getAgenteId())) {
			criteriaA.add(Restrictions.idEq(app.getAgenteId()));
		} 
		if (ApplicationUtil.isNotBlank(app.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  app.getNota(), criteria, "note");
		}
		app.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, app);
	}
	public int ricercaNotaOperatoreSize(Utente utente,NotaCriteria app) {
		ExtendedCriteria criteria = ricercaNotaOperatoreCriteria(utente,app);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Nota> ricercaNotaOperatore(Utente utente,NotaCriteria app, int start, int size) {
		ExtendedCriteria criteria = ricercaNotaOperatoreCriteria(utente,app);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), app.getO_tipo(), app.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	
	public List<Nota> impostaConteggioNota(Utente utente, GenericColumn c, List<Nota> note) {
		if(ApplicationUtil.notEmpty(note))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.NOTA.stampa() +": " + ApplicationUtil.size(note), note);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_NOTA);
			List<Long> ll = ApplicationUtil.prendiIdNota(note);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Nota.class, new ColonnaQuery(ConstantsColumn.storia, "storia"));
			Iterator<Nota> it = note.iterator();
			while (it.hasNext()) {
				Nota o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return note;
	}
	public List<AppuntamentoVO>  nuovaListaAppuntamentoVO() {
		return new ArrayList<AppuntamentoVO>();
	}
	public List<AppuntamentoVO>  salvaAppuntamenti(TemplateImportAppuntamento ti, ImportModelAppuntamento tiM, Utente utente, Integer stato, Long esitoNuovoId, Integer modifica, boolean forza, Date data, Integer step, Boolean modo) {
		List<AppuntamentoVO> nonTrovati = new ArrayList<AppuntamentoVO>();
		if(ti != null) {
			List<Appuntamento> appuntamentiNew = tiM.getElementi();
			if(ApplicationUtil.eq(modifica, 1)) {
				if (ApplicationUtil.notEmpty(appuntamentiNew)) {
					EsitoAppuntamento es = null;
					if(ApplicationUtil.gtZero(esitoNuovoId)) {
						es = campagnaDao.getEsitoAppuntamento(esitoNuovoId);
					}
					List<String> valori = ti.getValore_multiplo();
					List<Appuntamento>  contr = prendiEsistenti(appuntamentiNew, tiM.getTipo());
					for (Appuntamento appNew : appuntamentiNew) {
						List<Appuntamento> ctrs = trovaAppuntamento(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Appuntamento ctr : ctrs) {
								if (!appNew.getDuplicato()) {
									if(es != null) {
										ctr.setEsito(es);
									}
									if(valori.indexOf(ConstantsMap.appuntamento_nota) > -1) {
										ctr.setNote(appNew.getNote());
									}
									if(valori.indexOf(ConstantsMap.appuntamento_data_appuntamento) > -1) {
										ctr.imposta(appNew.getData_appuntamento());
									}
									if (ApplicationUtil.gtZero(stato)) {
										if(stato.equals(ConstantsGlobal.in_attesa)){
											if(!ctr.getStato().equals(ConstantsGlobal.stornato)) {
												nonTrovati.add(new AppuntamentoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										}
										if(stato.equals(ConstantsGlobal.in_pagamento)){
											if(!ctr.getStato().equals(ConstantsGlobal.in_attesa)) {
												nonTrovati.add(new AppuntamentoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										}
										if(stato.equals(ConstantsGlobal.pagato)){
											if(!ctr.getStato().equals(ConstantsGlobal.in_pagamento)) {
												nonTrovati.add(new AppuntamentoVO(appNew, stato,ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										}
										if(stato.equals(ConstantsGlobal.in_storno)){
											if(!ctr.getStato().equals(ConstantsGlobal.pagato)) {
												nonTrovati.add(new AppuntamentoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										} 
										if(stato.equals(ConstantsGlobal.stornato)){
											if(!ctr.getStato().equals(ConstantsGlobal.in_storno)) {
												nonTrovati.add(new AppuntamentoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										}
									}
									pannelloDao.salvaAppuntamento(ctr, utente, true);
									String p = ApplicationUtil.asString( ConstantsLanguage.APPUNTAMENTO.stampa(), ctr);
									operazioneDao.creaOperazione(ConstantsMap.LOG3, p , utente, ConstantsMap.OPERAZIONE_MODIFICA_APPUNTAMENTO);
								} else {
									nonTrovati.add(new AppuntamentoVO(appNew, stato, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), es));
								}
							}
						} else {
							nonTrovati.add(new AppuntamentoVO(appNew, stato, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), es));
						}
					}
				}
			} 
			if(ApplicationUtil.eq(modifica, 2)) {
				if (ApplicationUtil.notEmpty(appuntamentiNew)) {
					List<Appuntamento>  contr = prendiEsistenti(appuntamentiNew, tiM.getTipo());
					for (Appuntamento appNew : appuntamentiNew) {
						List<Appuntamento> ctrs = trovaAppuntamento(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Appuntamento ctr : ctrs) {
								if (!appNew.getDuplicato()) {
									pannelloDao.eliminaAppuntamento(utente, ctr);
								} else {
									nonTrovati.add(new AppuntamentoVO(appNew, null, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), null));
								}
							}
						} else {
							nonTrovati.add(new AppuntamentoVO(appNew, null, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), null));
						}
					}
				}
			}
			if(ApplicationUtil.eq(modifica, 3)) {
				if (ApplicationUtil.notEmpty(appuntamentiNew)) {
					List<Appuntamento>  contr = prendiEsistenti(appuntamentiNew, tiM.getTipo());
					for (Appuntamento appNew : appuntamentiNew) {
						List<Appuntamento> ctrs = trovaAppuntamento(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Appuntamento ctr : ctrs) {
								if (!appNew.getDuplicato()) {
									revertAppuntamento(ctr, utente, data, step, modo);
								} else {
									nonTrovati.add(new AppuntamentoVO(appNew, null, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), null));
								}
							}
						} else {
							nonTrovati.add(new AppuntamentoVO(appNew, null, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), null));
						}
					}
				}
			}
		}
		return nonTrovati;
	}
	private List<Appuntamento> trovaAppuntamento( List<Appuntamento> appuntamenti, Object seriale, String tipo) {
		List<Appuntamento> o = new ArrayList<Appuntamento>();
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			for (Appuntamento appuntamento: appuntamenti) {
				Object serial = appuntamento.getChiaveO(tipo);
				if (serial != null && seriale != null && serial.equals(seriale)) {
					o.add(appuntamento);
				}
			}
		}
		return o;
	}
	@SuppressWarnings("unchecked")
	public List<Appuntamento> prendiEsistenti(List<Appuntamento> conts, String tipo) {
		List<Object> appuntamentiL = new ArrayList<Object>();
		if(ApplicationUtil.notEmpty(conts)) {
			for (Appuntamento cont : conts) {
				Object seriale = cont.getChiaveL(tipo);
				if(seriale!= null && !appuntamentiL.contains(seriale) && ApplicationUtil.isTrue(cont.getTrovato())) {
					appuntamentiL.add(seriale);
				}
			}
		}
		if(ApplicationUtil.notEmpty(appuntamentiL))		{
			DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
			if(tipo != null && !ConstantsMap.appuntamento_id.equals(tipo)) {
				String tipoS = tipo.substring((ConstantsMap.anagrafica + "_").length());
				criteria.createCriteria("contatto").createCriteria("anagrafica").add(Restrictions.in(tipoS, appuntamentiL)); 
			} else {
				criteria.add(Restrictions.in("id", appuntamentiL)); 
			}
			return getList(criteria);
		}
		return null;
	}
	public List<SelectItem> getEsitiAppuntamento() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		List<EsitoAppuntamentoSI> lis = getEsitoAppuntamentoSI(criteria);
		return impostazioneDao.getEsitoAppuntamento(lis, false, null);
	}
}