package it.callite.dao;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.sip.SipBuddies;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.ChiamanteUtil;
@SuppressWarnings("serial")
@Repository("chiamanteDao")
public class ChiamanteDao extends GenericDao {
	
	
	
	public String getSito()  {
		//https://www.chistachiamando.it/numero-telefono/0266988483
		return ConstantsGlobal.chiamante;
	}
	
	public List<ChiamanteUtil> controlla(SipBuddies sp)  {
		List<ChiamanteUtil> cu = new ArrayList<ChiamanteUtil>();
		List<String> ls = ApplicationUtil.tokenU(sp.getCallers(), ConstantsGlobal.colonSeparator);
		Set<String> numeri = new HashSet<String>();
		if(ApplicationUtil.notEmpty(ls)) {
			Iterator<String> it = ls.iterator();
			while (it.hasNext()) {
				String n = (String) it.next();
				String numero = ApplicationUtil.trimToNull(n);
				if(numero != null) {
					if(!numero.equals(ConstantsGlobal.callerid)) {
						if(!numero.contains(ConstantsGlobal.pipeSeparator)) {
							numeri.add(numero);
						} else {
							List<String> a = ApplicationUtil.generateList(numero);
							if(ApplicationUtil.notEmpty(a)) {
								numeri.addAll(a);
							}
						}
					}
				}
			}
		}
		if(ApplicationUtil.notEmpty(numeri)) {
			String prefix = "";
			Iterator<String> it = numeri.iterator();
			while (it.hasNext()) {
				String n = ApplicationUtil.trimToNull((String) it.next());
				if(n != null) {
					ChiamanteUtil c = null;
					try {
						c = parse(ApplicationUtil.substringAfter(n, prefix));
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
					if(c != null) {
						cu.add(c);
					}
				}
			}
		}
		return cu;
	}
	
	private ChiamanteUtil parse(String numero) throws Exception  {
		Document doc = Jsoup.connect(getSito() + "/numero-telefono/" + numero).userAgent(ConstantsGlobal.browser).timeout(ConstantsGlobal.timeout).ignoreContentType(true).ignoreHttpErrors(true).followRedirects(true).get();
		List<String> l1 = new ArrayList<String>();
		List<String> l2 = new ArrayList<String>();
		Elements el1 = doc.select("div.ratings > ul > li");
		Iterator<Element> it1 = el1.iterator();
		while (it1.hasNext()) {
			Element figli = (Element) it1.next();	
			l1.add(figli.text());
		}
		Elements el2 = doc.select("div.categories > ul > li");
		Iterator<Element> it2 = el2.iterator();
		while (it2.hasNext()) {
			Element figli = (Element) it2.next();	
			l2.add(figli.text());
		}
		String t1 = null;
		String t2 = null;
		if(ApplicationUtil.notEmpty(l1)) {
			t1 = ApplicationUtil.trimToNull(ApplicationUtil.detoken(l1, ConstantsGlobal.colonSeparator + " "));
		}
		if(ApplicationUtil.notEmpty(l2)) {
			t2 = ApplicationUtil.trimToNull(ApplicationUtil.detoken(l2, ConstantsGlobal.colonSeparator + " "));
		}
		return new ChiamanteUtil(numero, t1, t2);
	}
}