package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.FetchMode;
import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.column.generic.GenericColumn;
import it.callite.constants.ConstantsColumn;
import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.ContrattoCriteria;
import it.callite.criteria.DomandaContrattoCriteria;
import it.callite.criteria.FileCriteria;
import it.callite.criteria.SondaggioContrattoCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Agente;
import it.callite.model.CampoAggiuntivoContratto;
import it.callite.model.Contatto;
import it.callite.model.Contratto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoContratto;
import it.callite.model.DomandaContratto;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoContratto;
import it.callite.model.SondaggioContratto;
import it.callite.model.StoriaContratto;
import it.callite.model.TemplateImportContratto;
import it.callite.model.Utente;
import it.callite.model.VocalOrder;
import it.callite.model.allegato.AllegatoContratto;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.CampoSpeciale;
import it.callite.utility.ContrattoProdotto;
import it.callite.utility.ContrattoVO;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.DatoAggiuntivoContrattoRicerca;
import it.callite.utility.DatoSondaggioAnagrafica;
import it.callite.utility.DatoSondaggioContratto;
import it.callite.utility.FileUpload;
import it.callite.utility.LoggerColonna;
import it.callite.utility.MyFile;
import it.callite.utility.ProdottoCheckbox;
import it.callite.utility.ProdottoSelezionato;
import it.callite.utility.ValoreMappa;
import it.callite.utility.VocalProdotto;
import it.callite.utility.importa.ImportModelContratto;
import it.callite.utility.thread.ContrattoMessaggioThread;
import it.callite.utility.thread.ContrattoNotiziaThread;
import it.callite.utility.thread.SalvaStoriaContrattoThread;
import it.callite.utility.utils.ModuloFileFilter;
@SuppressWarnings("serial")
@Repository("contrattoDao")
public class ContrattoDao extends GenericDao {
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private UserDao userDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private ProdottoDao prodottoDao;
	@Autowired
	private MessaggioDao messaggioDao;
	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private NotiziaDao notiziaDao;
	@Autowired
	private DataDao dataDao;
	public void scaricaAllegato(Contratto contratto, DownloadBean downloadBean) {
		List<AllegatoContratto> allegati = getListaAllegati(contratto);
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_contratto);
				String file_header = ApplicationUtil.reportFile(contratto.getData_creazione(), ConstantsLanguage.CONTRATTO.stampa(), contratto.getId(), contratto.getContatto().getAnagrafica().getDenominazione()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoContratto> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoContratto allegato = (AllegatoContratto) itA.next();
					ApplicationUtil.addFile(all, allegato.getId() + "_" +allegato.getNome(), allegato.getContentFile(s), allegato.getData().getTime());
				}
				ApplicationUtil.zip(null, all, out, p);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	
	public void scaricaAllegati(List <Contratto> contratti, Utente utente, DownloadBean downloadBean) {
		List<AllegatoContratto> allegati = getListaAllegati(contratti);
		String p = impostazioneDao.getPasswordZIP();
		try {
			String pc = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,pc ,utente, ConstantsMap.OPERAZIONE_EXPORT_CONTRATTO);
			String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_contratto);
			String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ApplicationUtil.lowerCase(ConstantsLanguage.ALLEGATI.stampa() + "_" + ConstantsLanguage.CONTRATTI.stampa())) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file_header, contratti, allegati);
			List<Allegato> all = new ArrayList<Allegato>();
			if(ApplicationUtil.notEmpty(allegati)) {
				Iterator<AllegatoContratto> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoContratto allegato = (AllegatoContratto) itA.next();
					ApplicationUtil.addFile(all, allegato.getContratto().getId() + "_" + allegato.getId() + "_" +allegato.getNome(), allegato.getContentFile(s), allegato.getData().getTime());
				}
			}
			if (ApplicationUtil.empty(all)) {
				ApplicationUtil.fileWriter(all, ConstantsLanguage.NO_CONTRATTO.stampa(), ConstantsLanguage.NO_CONTRATTI.stampa());
			}
			ApplicationUtil.zip(null, all, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	 
	public void scaricaAllegato(AllegatoContratto allegato, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_contratto);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());
			InputStream is = new ByteArrayInputStream(allegato.getContentFile(s));
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaAllegato(FileUpload fileUpload, Contratto contratto) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				String s = impostazioneDao.getPathBaseW(ConstantsGlobal.path_contratto);
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					AllegatoContratto allegato = new AllegatoContratto();
					allegato.setContratto(contratto);
					allegato.setDimensione(ffB.length);
					allegato.setNome(ApplicationUtil.getNameN(ff.getName()));
					allegato.setContentFile(ffB, s);
					salvaAggiorna(allegato);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto> getStoriaContratto(Contratto contratto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		criteria.add(Restrictions.eq("contratto.id", contratto.getId()));
		criteria.addOrder(Order.asc("data"));
		criteria.setFetchMode("contratto", FetchMode.JOIN);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoContratto> getListaAllegati(Contratto contratto) {
		if(contratto==null)
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoContratto.class);
		criteria.add(Restrictions.eq("contratto.id", contratto.getId()));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<AllegatoContratto> getListaAllegati(List<Contratto> contratti) {
		if(ApplicationUtil.empty(contratti))
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoContratto.class);
		criteria.add(Restrictions.in("contratto.id", ApplicationUtil.prendiIdContratto(contratti)));
		return getList(criteria);
	}
	public void deleteAllegato(AllegatoContratto allegato)  {
		String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_contratto);
		allegato.delete(s);
		cancellaAggiorna(allegato);
	}
	public void deleteAllegati(Contratto contratto)  {
		List<AllegatoContratto> allegati = getListaAllegati(contratto);
		if(ApplicationUtil.notEmpty(allegati)) {
			Iterator<AllegatoContratto> itAn = allegati.iterator();
			while (itAn.hasNext()) {
				AllegatoContratto allegato = (AllegatoContratto) itAn.next();
				deleteAllegato(allegato);
			}
		}
	}
	private ExtendedCriteria trovaDomandeCriteria(boolean modulo, DomandaContrattoCriteria dom) {
		if(dom != null)  dom = (DomandaContrattoCriteria) dom.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		if(ApplicationUtil.isNotBlank(dom.getCampo())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, dom.getCampo(), criteria, "descrizione", "etichetta");
		}
		if(ApplicationUtil.gtZero(dom.getTipo())) {
			criteria.add(Restrictions.eq("tipo",dom.getTipo()));
		}
		if(ApplicationUtil.gtZero(dom.getSondaggio())) {
			criteria.createCriteria("domande").createCriteria("sondaggio").add(Restrictions.eq("id",dom.getSondaggio()));
		}
		if(ApplicationUtil.isNotBlank(dom.getGruppo())) {
			criteria.add(Restrictions.eq("gruppo",dom.getGruppo()));
		}
		if(!modulo) {
			criteria.add(Restrictions.ne("tipo",ConstantsMap.TIPO_FORM));
		}
		dom.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, dom);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> trovaDomande(List<DatoAggiuntivoContratto> campiDaRimuovere,  Boolean modulo, DomandaContrattoCriteria dom, int start, int size) {
		ExtendedCriteria criteria = trovaDomandeCriteria(modulo, dom);
		List<Long> ids = ApplicationUtil.prendiIdDatoAggiuntivoContratto(campiDaRimuovere);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> getAllCampiAggiuntivi() {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.ne("tipo",ConstantsMap.TIPO_FORM));
		criteria.addOrder(Order.asc("etichetta"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> getAllCampiAggiuntiviAgente() {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.ne("tipo",ConstantsMap.TIPO_FORM));
		criteria.add(Restrictions.eq("visibile_agente", Boolean.TRUE));
		criteria.addOrder(Order.asc("etichetta"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> getAllCampiAggiuntiviOperatore() {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.ne("tipo",ConstantsMap.TIPO_FORM));
		criteria.add(Restrictions.eq("visibile_operatore", Boolean.TRUE));
		criteria.addOrder(Order.asc("etichetta"));
		return getList(criteria);
	}
	public void aggiornaCampi(List<CampoAggiuntivoContratto> cas) {
		if(ApplicationUtil.notEmpty(cas)) {
			Iterator<CampoAggiuntivoContratto> caIt = cas.iterator();
			while (caIt.hasNext()) {
				CampoAggiuntivoContratto ca = (CampoAggiuntivoContratto) caIt.next();
				aggiornaCampo(ca);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public void aggiornaCampo(CampoAggiuntivoContratto ca) {
		if(ca != null && !ca.getForm()) {
			DetachedCriteria criteria1 = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
			criteria1.add(Restrictions.eq("campo_aggiuntivo.id", ca.getId()));
			criteria1.setProjection(Projections.distinct(Projections.groupProperty("contratto.id")));
			DetachedCriteria criteria2 = DetachedCriteria.forClass(Contratto.class);
			impostazioneDao.subPropertyNotInS(criteria2, criteria1, "id");
			List<Contratto> a = getList(criteria2);
			if(ApplicationUtil.notEmpty(a)) {
				Session session = openSession();
				Transaction tx = session.beginTransaction();
				int i = 0;
				Iterator<Contratto> it = a.iterator();
				while(it.hasNext()) {
					Contratto c = (Contratto) it.next();
					DatoAggiuntivoContratto da = new DatoAggiuntivoContratto();
					da.setContratto(c);
					da.setCampo_aggiuntivo(ca);
					session.saveOrUpdate(da);
					if (i++ % ConstantsGlobal.flushSize == 0) {
						session.flush();
						session.clear();
					}
				}
				tx.commit();
				closeSession(session);
			}
		}
	}
	public int trovaDomandeSize(List<DatoAggiuntivoContratto> campiDaRimuovere, Boolean modulo, DomandaContrattoCriteria campo) {
		ExtendedCriteria criteria = trovaDomandeCriteria(modulo, campo);
		List<Long> ids = ApplicationUtil.prendiIdDatoAggiuntivoContratto(campiDaRimuovere);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> trovaDomande(Boolean modulo, DomandaContrattoCriteria dom, List<DomandaContratto> campiDaRimuovere, int start, int size) {
		ExtendedCriteria criteria = trovaDomandeCriteria(modulo, dom);
		List<Long> ids = ApplicationUtil.prendiIdDomandaContratto(campiDaRimuovere);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaDomandeSize(Boolean modulo, DomandaContrattoCriteria dom, List<DomandaContratto> campiDaRimuovere) {
		ExtendedCriteria criteria = trovaDomandeCriteria(modulo, dom);
		List<Long> ids = ApplicationUtil.prendiIdDomandaContratto(campiDaRimuovere);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteria(SondaggioContrattoCriteria snd) {
		if(snd != null)  snd = (SondaggioContrattoCriteria) snd.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		if (ApplicationUtil.isNotBlank(snd.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, snd.getNome(), criteria, "nome", "descrizione");
		}
		if (ApplicationUtil.gtZero(snd.getProdottoId())) {
			criteria.createCriteria("prodotti").add(Restrictions.eq("id",snd.getProdottoId()));
		}
		snd.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, snd);
	}
	@SuppressWarnings("unchecked")
	public List<SondaggioContratto> trova(SondaggioContrattoCriteria snd, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(snd);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(SondaggioContrattoCriteria snd) {
		ExtendedCriteria criteria = trovaCriteria(snd);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> getCampiEsistenti(Long sondaggioId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaContratto.class);
		criteria.add(Restrictions.eq("sondaggio.id",sondaggioId));
		criteria.addOrder(Order.asc("posizione"));
		criteria.setProjection(Projections.distinct(Projections.property("campo_aggiuntivo.id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<DomandaContratto> getDomandeSondaggio(Long sondaggioId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaContratto.class);
		criteria.add(Restrictions.eq("sondaggio.id",sondaggioId));
		criteria.addOrder(Order.asc("posizione"));
		return getList(criteria);
	}
	public DomandaContratto salva(DomandaContratto sondaggio) {
		return (DomandaContratto) salvaAggiorna(sondaggio);
	}
	public SondaggioContratto salva(SondaggioContratto sondaggio) {
		return (SondaggioContratto) salvaAggiorna(sondaggio);
	}
	public void elimina(SondaggioContratto sondaggio) {
		cancellaAggiorna(sondaggio);
	}
	public void eliminaContratto(Utente utente, Contratto contratto) {
		if(contratto!=null) {
			String p = ApplicationUtil.asString( ConstantsLanguage.CONTRATTO.stampa(), contratto);
			operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_CONTRATTO);
			cancellaAggiorna(contratto);
		}
	}
	public SondaggioContratto get(Long id) {
		return (SondaggioContratto) get(SondaggioContratto.class, id);
	}
	@SuppressWarnings("unchecked")
	public List<DomandaContratto> getDomandeSondaggioByProdotto(Integer tipo, Contatto contatto, Long prodotto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaContratto.class);
		DetachedCriteria criteriaS = criteria.createCriteria("sondaggio");
		if(!contatto.isOk()) {
			criteriaS.add(Restrictions.eq("solo_archiviati", Boolean.FALSE));
		}
		if(ApplicationUtil.eq(tipo, 1)) {
			criteriaS.add(Restrictions.eq("solo_operatore", Boolean.TRUE));
		}
		if(ApplicationUtil.eq(tipo, 2)) {
			criteriaS.add(Restrictions.eq("solo_supervisore", Boolean.TRUE));
		}
		if(ApplicationUtil.eq(tipo, 3)) {
			criteriaS.add(Restrictions.eq("solo_backoffice", Boolean.TRUE));
		}
		criteriaS.createCriteria("prodotti").add(Restrictions.eq("id",prodotto));
		criteriaS.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.addOrder(Order.asc("posizione"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<VocalOrder> getVocalOrder(Long prodotto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
		criteria.createCriteria("prodotti").add(Restrictions.eq("id",prodotto));
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		return getList(criteria);
	}
	private ContrattoProdotto getSondaggioForm(Utente utente, Integer tipo, EsitoArchiviazione esito, ProdottoSelezionato prd, Contatto contatto, Long esitoC, List<DatoSondaggioAnagrafica> dS) {
		ContrattoProdotto cr = new ContrattoProdotto();
		List<DatoSondaggioContratto> dati = getDatiSondaggio(tipo, contatto, esito, prd, dS);
		if(ApplicationUtil.eq(esito.getContratto_imposta(), 3) || ApplicationUtil.eq(esito.getContratto_imposta(), 4) || ApplicationUtil.eq(esito.getContratto_imposta(), 5)) {
			Contratto contratto = prd.getContratto();
			cr.setDatoSondaggioContratto(dati);
			cr.setContratto(contratto);
			cr.setOperatoreId(contratto.getOperatore() != null ? contratto.getOperatore().getId() : null);
			cr.setSeriale(contratto.getSeriale());
			cr.setCosto_scontato(contratto.getCosto_scontato());
			cr.setQuantita(contratto.getQuantita());
			cr.setNota(contratto.getNota());
			cr.setStatoId(contratto.getEsito() != null ? contratto.getEsito().getId() : null);
			cr.setProdotto(contratto.getProdotto());
			cr.setAgenteId(contratto.getAgente() != null ? contratto.getAgente().getId() : null);
			cr.setData_stipula(contratto.getData_stipula());
			cr.setData_contabile(contratto.getData_contabile());
			cr.setData_attivazione(contratto.getData_attivazione());
			cr.setData_caricamento(contratto.getData_caricamento());
		} else {
			cr.setContratto(new Contratto());
			cr.setDatoSondaggioContratto(dati);
			cr.setStatoId(esitoC);
			cr.setProdotto(prd.getProdotto());
			if(contatto!=null && contatto.getAgente()!=null) {
				cr.setAgenteId(contatto.getAgente().getId());
			}
			cr.setOperatoreId(utente.getId());
			Integer i = impostazioneDao.getGiorniContratti();
			cr.setData_stipula(i != null ? ApplicationUtil.inizioGiornata(i) : ApplicationUtil.date());
			cr.setData_contabile(null);
			cr.setData_attivazione(null);
			cr.setData_caricamento(null);
		}
		return cr;
	}
	@SuppressWarnings("unchecked")
	public List<DomandaContratto> getTutteDomandeSondaggio(Integer tipo, Contatto contatto, Long prodotto) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(DomandaContratto.class);
		DetachedCriteria criteriaS = criteriaC.createCriteria("sondaggio");
		if(!contatto.isOk()) {
			criteriaS.add(Restrictions.eq("solo_archiviati", Boolean.FALSE));
		}
		if(ApplicationUtil.eq(tipo, 1)) {
			criteriaS.add(Restrictions.eq("solo_operatore", Boolean.TRUE));
		}
		if(ApplicationUtil.eq(tipo, 2)) {
			criteriaS.add(Restrictions.eq("solo_supervisore", Boolean.TRUE));
		}
		if(ApplicationUtil.eq(tipo, 3)) {
			criteriaS.add(Restrictions.eq("solo_backoffice", Boolean.TRUE));
		}
		criteriaS.createCriteria("prodotti").add(Restrictions.eq("id", prodotto));
		criteriaS.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteriaC.addOrder(Order.asc("posizione"));
		return getList(criteriaC);
	}
	@SuppressWarnings("unchecked")
	public List<DomandaContratto> getRisposteDomandeSondaggio(EsitoArchiviazione esito, Long contratto) {
		List<DomandaContratto> dms = new ArrayList<DomandaContratto>();
		DetachedCriteria criteriaC = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
		criteriaC.add(Restrictions.eq("contratto.id", contratto));
		List<DatoAggiuntivoContratto> das = getList(criteriaC);
		if(ApplicationUtil.notEmpty(das)) {
			Iterator<DatoAggiuntivoContratto> objIter = das.iterator();
			while(objIter.hasNext()) {
				DatoAggiuntivoContratto da = objIter.next();
				DomandaContratto dm = new DomandaContratto();
				dm.setCampo_aggiuntivo(da.getCampo_aggiuntivo());
				dm.setRisposta(da.getValore());
				dm.setObbligatoria(da.getCampo_aggiuntivo().getObbligatorio());
				dm.setVisibile(true);
				dms.add(dm);
			} 
		}
		return dms;
	}
	public List<DatoSondaggioContratto> getDatiSondaggio(Integer tipo, Contatto contatto, EsitoArchiviazione esito, ProdottoSelezionato prd, List<DatoSondaggioAnagrafica> dS) {
		List<DatoSondaggioContratto> dati = new ArrayList<DatoSondaggioContratto>();
		if(esito != null && ApplicationUtil.eq(esito.getContratto_tipo(), 2)) {
			Map<String, DatoAggiuntivo> valori = new HashMap<String, DatoAggiuntivo>();
			List<DatoAggiuntivoAnagrafica> dA = new ArrayList<DatoAggiuntivoAnagrafica>();
			if(contatto != null) {
				List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
				valori.putAll(ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi));
				if(ApplicationUtil.notEmpty(contatto.getDettagli())) {
					dA.addAll(contatto.getDettagli());
				}
			}
			if(ApplicationUtil.notEmpty(dA)) {
				valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
			}
			if(ApplicationUtil.notEmpty(dS)) {
				valori.putAll(ApplicationUtil.convertiSondaggioA(dS));
			}
			if(ApplicationUtil.eq(esito.getContratto_imposta(), 3) || ApplicationUtil.eq(esito.getContratto_imposta(), 4) || ApplicationUtil.eq(esito.getContratto_imposta(), 5)) {
				List<DomandaContratto> domandeR = getRisposteDomandeSondaggio(esito, prd.getContratto().getId());
				if(ApplicationUtil.eq(esito.getContratto_imposta(), 3) || ApplicationUtil.eq(esito.getContratto_imposta(), 4)) {
					Map<String,Object> valoriS = new HashMap<String,Object>();
					Map<String,List<String>> valoriM = new HashMap<String,List<String>>();
					List<DomandaContratto> domandeT = getTutteDomandeSondaggio(tipo, contatto, prd.getProdotto().getId());
					if(ApplicationUtil.notEmpty(domandeR)) {
						Iterator<DomandaContratto> objDocIter = domandeR.iterator();
						while(objDocIter.hasNext()) {
							DomandaContratto domanda = objDocIter.next();
							if(ApplicationUtil.isNotBlank(domanda.getCampo_aggiuntivo().getGruppo())) {
								String risposta = domanda.getRisposta();
								if(domanda.getCampo_aggiuntivo().getManyCheckbox() || domanda.getCampo_aggiuntivo().getManyMenu() || domanda.getCampo_aggiuntivo().getPickList()) {
									valoriM.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.token(risposta, ConstantsGlobal.colonSeparator));
								} else if(domanda.getCampo_aggiuntivo().getData()) {
									valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(domanda.getCampo_aggiuntivo().getValoriT(), risposta));
								} else if(domanda.getCampo_aggiuntivo().getDataora()) {
									valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(domanda.getCampo_aggiuntivo().getValoriT(), risposta));
								} else if(domanda.getCampo_aggiuntivo().getOra()) {
									valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(ConstantsFormat.time.stampa(), risposta));
								} else {
									valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), risposta);
								}
							}
						}
					}
					if(ApplicationUtil.notEmpty(domandeT)) {
						Iterator<DomandaContratto> objDocIter = domandeT.iterator();
						while(objDocIter.hasNext()) {
							DomandaContratto domanda = objDocIter.next();
							DatoSondaggioContratto datoS = null;
							if(domandeR.contains(domanda)){
								int index = domandeR.indexOf(domanda);
								if(index > -1) {
									DomandaContratto d = domandeR.get(index);
									String risposta = d.getRisposta();
									datoS = ApplicationUtil.creaDomanda(domanda, risposta);
								}
							} else {
								String risposta = ApplicationUtil.rispondi(domanda,valori);
								datoS = ApplicationUtil.creaDomanda(domanda, risposta);
							}
							if(datoS != null) {
								addDomanda(dati, datoS);
								if(domanda.getCampo_aggiuntivo().getForm()) {
									datoS.setValoriM(valoriM);
									datoS.setValoriS(valoriS);
								}
							}
						}
					}
				}
				if(ApplicationUtil.eq(esito.getContratto_imposta(), 4) || ApplicationUtil.eq(esito.getContratto_imposta(), 5)) {
					if(ApplicationUtil.notEmpty(domandeR)) {
						Iterator<DomandaContratto> objDocIter = domandeR.iterator();
						while(objDocIter.hasNext()) {
							DomandaContratto domanda = objDocIter.next();
							if((ApplicationUtil.eq(esito.getContratto_imposta(), 4) && ApplicationUtil.isBlank(domanda.getCampo_aggiuntivo().getGruppo())) || ApplicationUtil.eq(esito.getContratto_imposta(), 5)) {
								String risposta = domanda.getRisposta();
								addDomanda(dati, ApplicationUtil.creaDomanda(domanda, risposta));
							}
						}
					}
				}
			} else {
				List<DomandaContratto> domande = getDomandeSondaggioByProdotto(tipo, contatto, prd.getProdotto().getId());
				if(ApplicationUtil.notEmpty(domande)) {
					Iterator<DomandaContratto> objIter = domande.iterator();
					while(objIter.hasNext()) {
						DomandaContratto domanda = objIter.next();
						String risposta = ApplicationUtil.rispondi(domanda,valori);
						addDomanda(dati, ApplicationUtil.creaDomanda(domanda, risposta));
					}
				}
			}
		}
		return dati;
	}
	private void addDomanda(List<DatoSondaggioContratto> dati, DatoSondaggioContratto domanda) {
		if(dati!=null && !dati.contains(domanda)) {
			dati.add(domanda);
		}
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoContratto> getDomandeRisposte(Long contrattoId) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
		criteriaC.add(Restrictions.eq("contratto.id", contrattoId));
		return getList(criteriaC);
	}

	public List<ProdottoSelezionato> getProdottiSelezionati(EsitoArchiviazione esito, List<ProdottoCheckbox> prodottoCheckbox) {
		List<ProdottoSelezionato> mappa = new ArrayList<ProdottoSelezionato>();
		if (ApplicationUtil.notEmpty(prodottoCheckbox)) {
			Iterator<ProdottoCheckbox> objIter = prodottoCheckbox.iterator();
			while(objIter.hasNext()) {
				ProdottoCheckbox prodottoc = objIter.next();
				if (ApplicationUtil.isTrue(prodottoc.getSelezionato())) {
					if(ApplicationUtil.eq(esito.getContratto_imposta(), 3) || ApplicationUtil.eq(esito.getContratto_imposta(), 4) || ApplicationUtil.eq(esito.getContratto_imposta(), 5)) {
						mappa.add(new ProdottoSelezionato(prodottoc.getContratto()));
					} else {
						int quantita = prodottoc.getQuantita();
						for(int i = 0; i <quantita; i++) {
							mappa.add(new ProdottoSelezionato(prodottoc.getProdotto(), i));
						}
					}
				}
			}
		}
		return mappa;
	}
	public List<ProdottoSelezionato> getProdottiSelezionato(EsitoArchiviazione esito, ProdottoCheckbox prodottoCheckbox) {
		List<ProdottoSelezionato> mappa = new ArrayList<ProdottoSelezionato>();
		if(ApplicationUtil.eq(esito.getContratto_imposta(), 3) || ApplicationUtil.eq(esito.getContratto_imposta(), 4) || ApplicationUtil.eq(esito.getContratto_imposta(), 5)) {
			mappa.add(new ProdottoSelezionato(prodottoCheckbox.getContratto()));
		} else {
			int quantita = prodottoCheckbox.getQuantita();
			for(int i = 0; i <quantita; i++) {
				mappa.add(new ProdottoSelezionato(prodottoCheckbox.getProdotto(), i));
			}
		}
		return mappa;
	}
	public Map<ProdottoSelezionato, ContrattoProdotto> getSondaggioFormMap(List<DatoSondaggioAnagrafica> dS, Utente utente, Integer tipo, EsitoArchiviazione esito, List<ProdottoSelezionato> prodottoCheckbox, Contatto contatto) {
		Map<ProdottoSelezionato, ContrattoProdotto> mappa = new HashMap<ProdottoSelezionato, ContrattoProdotto>();
		if (ApplicationUtil.notEmpty(prodottoCheckbox)) {
			Long ep = null;
			if(ApplicationUtil.eq(esito.getContratto_imposta(), 1) || ApplicationUtil.eq(esito.getContratto_imposta(), 2) ) {
				ep = getEsitoContrattoPredefinito();
			}
			Iterator<ProdottoSelezionato> objIter = prodottoCheckbox.iterator();
			while(objIter.hasNext()) {
				ProdottoSelezionato prodottoc = objIter.next();
				ContrattoProdotto cr = getSondaggioForm(utente, tipo, esito, prodottoc, contatto, ep, dS);
				mappa.put(prodottoc, cr);
			}
		}
		return mappa;
	}
	public Map<ProdottoSelezionato, VocalProdotto> vocalOrderMap(Contatto contatto, List<DatoAggiuntivoAnagrafica> dA, List<DatoSondaggioAnagrafica> dS, Map<ProdottoSelezionato, ContrattoProdotto> mapCr) {
		Map<ProdottoSelezionato, VocalProdotto> mappa = new HashMap<ProdottoSelezionato, VocalProdotto>();
		if (ApplicationUtil.notEmpty(mapCr)) {
			Iterator<ProdottoSelezionato> objIter = mapCr.keySet().iterator();
			while(objIter.hasNext()) {
				ProdottoSelezionato prodottoc = objIter.next();
				VocalProdotto cr = vocalOrder(contatto, dA, dS, mapCr.get(prodottoc));
				mappa.put(prodottoc, cr);
			}
		}
		return mappa;
	}

	public List<ProdottoSelezionato> contrattoPdaMap(Contatto contatto, List<DatoAggiuntivoAnagrafica> dA, List<DatoSondaggioAnagrafica> dS, Map<ProdottoSelezionato, ContrattoProdotto> mapCr) {
		List<ProdottoSelezionato> mappa = new ArrayList<ProdottoSelezionato>();
		if (ApplicationUtil.notEmpty(mapCr)) {
			Iterator<ProdottoSelezionato> objIter = mapCr.keySet().iterator();
			while(objIter.hasNext()) {
				ProdottoSelezionato prodottoc = objIter.next();
				boolean ok = prodottoDao.getTemplateByProdottoS(prodottoc.getProdotto().getId());
				if(ok) {
					prodottoc.setNome(contatto.getAnagrafica().getNome());
					prodottoc.setCognome(contatto.getAnagrafica().getCognome());
					prodottoc.setEmail(contatto.getAnagrafica().getEmail());
					prodottoc.setTelefono(contatto.getAnagrafica().getTelefono_mobile());
					mappa.add(prodottoc);
				}
			}
		}
		return mappa;
	}

	public VocalProdotto vocalOrder(Contatto contatto,  List<DatoAggiuntivoAnagrafica> dA, List<DatoSondaggioAnagrafica> dS, ContrattoProdotto lsCrM) {
		VocalProdotto cr = new VocalProdotto();
		List<VocalOrder> dati = new ArrayList<VocalOrder>();
		List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori = ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi);
		if(ApplicationUtil.notEmpty(dA)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
		}
		if(ApplicationUtil.notEmpty(dS)) {
			valori.putAll(ApplicationUtil.convertiSondaggioA(dS));
		}
		if(lsCrM !=null & ApplicationUtil.notEmpty(lsCrM.getDatoSondaggioContratto())) {
			valori.putAll(ApplicationUtil.convertiSondaggioC(lsCrM.getDatoSondaggioContratto()));
		}
		List<VocalOrder> domande = getVocalOrder(lsCrM.getProdotto().getId());
		Iterator<VocalOrder> objIter = domande.iterator();
		while(objIter.hasNext()) {
			VocalOrder domanda = objIter.next();
			domanda.setTestoString(ApplicationUtil.replaceInString(domanda.getTesto(), valori));
			if(dati != null && !dati.contains(domanda)) {
				dati.add(domanda);
			}
		}
		cr.setTesto(dati);
		return cr;
	}
	public boolean addDomandaToSondaggio(SondaggioContratto sondaggio, DomandaContratto domanda) {
		boolean risposta = false;
		if (sondaggio.getDomande() == null)
			sondaggio.setDomande(new ArrayList<DomandaContratto>());
		if (!sondaggio.getDomande().contains(domanda))
			risposta = sondaggio.getDomande().add(domanda);
		return risposta;
	}
	public CampoAggiuntivoContratto getCampoAggiuntivoByChiave(String chiave) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.eq("chiave", chiave));
		return (CampoAggiuntivoContratto) getUniqueResult(criteria);
	}
	public CampoAggiuntivoContratto salvaCampoAggiuntivo(CampoAggiuntivoContratto campoAggiuntivoContratto) {
		campoAggiuntivoContratto.imposta();
		return (CampoAggiuntivoContratto) salvaAggiorna(campoAggiuntivoContratto);
	}
	public CampoAggiuntivoContratto salvaCampoAggiuntivo(CampoAggiuntivoContratto campoAggiuntivoContratto, List<DomandaContratto> domande) {
		if(ApplicationUtil.notEmpty(domande)) {
			Iterator<DomandaContratto> dmIt = domande.iterator();
			while (dmIt.hasNext()) {
				DomandaContratto dm = (DomandaContratto) dmIt.next();
				salvaAggiorna(dm);
			}
		}
		return salvaCampoAggiuntivo(campoAggiuntivoContratto);
	}
	public DatoAggiuntivoContratto salvaDatoAggiuntivo(DatoAggiuntivoContratto datoAggiuntivoContratto) {
		datoAggiuntivoContratto.imposta();
		return (DatoAggiuntivoContratto) salvaAggiorna(datoAggiuntivoContratto);
	}
	public void saveSondaggioToContratto(EsitoArchiviazione esito, List<DatoSondaggioContratto> dati, Contratto contratto) {
		if(contratto != null && esito != null && esito.has_sceltaCampi() && ApplicationUtil.notEmpty(dati)) {
			List<DatoAggiuntivoContratto> dr = new ArrayList<DatoAggiuntivoContratto>();
			if(ApplicationUtil.gtZero(contratto.getId())) {
				dr.addAll(getDomandeRisposte(contratto.getId()));
			} 
			Iterator<DatoSondaggioContratto> objIter = dati.iterator();
			while(objIter.hasNext()) {
				DatoSondaggioContratto dato = objIter.next();
				if (dato.getCampo() != null) {
					if (dato.getCampo().getForm()) {
						Map<CampoSpeciale,ValoreMappa> mappa = dato.getMappa();
						if (ApplicationUtil.notEmpty(mappa)) {
							Iterator<CampoSpeciale> ssIt =  mappa.keySet().iterator();
							while(ssIt.hasNext()) {
								CampoSpeciale cc = ssIt.next();
								ValoreMappa val = mappa.get(cc);
								String chiave = ApplicationUtil.trimToEmpty(cc.getChiave()).replaceAll(ConstantsGlobal.colonSeparator, " ");
								if(val!=null) {
									if(ApplicationUtil.isBlank(cc.getEtichetta())) {
										cc.setEtichetta(chiave);
									}
									CampoAggiuntivoContratto campo = dato.getCampo();
									String nuovaChiave = ApplicationUtil.trimToEmpty(ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getChiave(), chiave, ConstantsGlobal.ereditaChiave));
									if(!ConstantsMap.anagraficaMap.contains(nuovaChiave)){
										CampoAggiuntivoContratto caa = getCampoAggiuntivoByChiave(nuovaChiave);
										if (caa == null) {
											caa = new CampoAggiuntivoContratto();
											caa.setChiave(nuovaChiave);
											String gruppo = ApplicationUtil.impostaGruppo(campo.getEtichetta(), campo.getColore());
											caa.setGruppo(gruppo);
											caa.setEtichetta(ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getEtichetta(), cc.getEtichetta(), ConstantsGlobal.ereditaEtichetta));
											caa.setDescrizione(ApplicationUtil.impostaCampoC(cc.getEredita(),campo.getDescrizione(), cc.getDescrizione()));
											caa.setModificabile_operatore(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_operatore(), cc.getModificabile_operatore()));
											caa.setVisibile_operatore(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_operatore(), cc.getVisibile_operatore()));
											caa.setModificabile_agente(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_agente(), cc.getModificabile_agente()));
											caa.setVisibile_agente(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_agente(), cc.getVisibile_agente()));
											caa.setObbligatorio(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getObbligatorio(), cc.getObbligatorio()));
											caa.setColore(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getColore(), cc.getColore()));
											caa.setStile(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getStile(), cc.getStile()));
											caa.setTipo(ApplicationUtil.gtZero(caa.getTipo()) ? cc.getTipo() : ConstantsMap.TIPO_APERTO);
											caa.setDefaults(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDefaults(), cc.getDefaults()));
											if(caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento() || caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu() || caa.getCombo()) {
												Set<String> valori = new HashSet<String>();
												valori.addAll(cc.getValuesList());
												if(val.getMultiplo() != null && (caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu())) {
													valori.addAll(val.getMultiplo());
												} 
												if(val.getSingolo() != null && (caa.getCombo() || caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento())) {
													valori.add(ApplicationUtil.trimToEmpty((String) val.getSingolo()));
												} 
												valori.removeAll(ApplicationUtil.asList("", null));
												caa.setValori(ApplicationUtil.detoken(new ArrayList<String>(valori), ConstantsGlobal.colonSeparator));
											}
											if(caa.getData() || caa.getDataora() || caa.getRegulare() || caa.getUnivoco()) {
												caa.setValori(cc.getValues());
											} 
											caa = (CampoAggiuntivoContratto) salvaCampoAggiuntivo(caa);
											dataDao.addCampo(caa);
										} else {
											boolean save = false;
											if(caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento() || caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu() || caa.getCombo()) {
												Set<String> vn = new HashSet<String>();
												vn.addAll(caa.getValoriList());
												vn.addAll(cc.getValuesList());
												if(val.getMultiplo() != null && (caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu())) {
													vn.addAll(val.getMultiplo());
												}
												if(val.getSingolo() != null && (caa.getCombo() || caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento())) {
													vn.add(ApplicationUtil.trimToEmpty((String) val.getSingolo()));
												}
												vn.removeAll(ApplicationUtil.asList("", null));
												Set<String> vl = new HashSet<String>(caa.getValoriList());
												if(!vl.containsAll(vn)) {
													vl.addAll(vn);
													caa.setValori(ApplicationUtil.detoken( new ArrayList<String>(vl), ConstantsGlobal.colonSeparator));
													save = true;
												}
											}
											if(caa.getData() || caa.getDataora() || caa.getRegulare() || caa.getUnivoco()) {
												String v = ApplicationUtil.trimToEmpty(cc.getValues());
												if(!ApplicationUtil.equals(caa.getValori(), v)) {
													caa.setValori(v);
													save = true;
												}
											} 
											if(!ApplicationUtil.equals(caa.getTipo(), cc.getTipo())) {
												caa.setTipo(cc.getTipo());
												save = true;
											}
											String gruppo = ApplicationUtil.impostaGruppo(campo.getEtichetta(), campo.getColore());
											if(!ApplicationUtil.equals(caa.getGruppo(), gruppo)) {
												caa.setGruppo(gruppo);
												save = true;
											}
											Boolean b1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getObbligatorio(), cc.getObbligatorio());
											if(!ApplicationUtil.equals(caa.getObbligatorio(), b1)) {
												caa.setObbligatorio(b1);
												save = true;
											}
											Boolean o1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_operatore(), cc.getModificabile_operatore());
											if(!ApplicationUtil.equals(caa.getModificabile_operatore(), o1)) {
												caa.setModificabile_operatore(o1);
												save = true;
											}
											Boolean o2 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_operatore(), cc.getVisibile_operatore());
											if(!ApplicationUtil.equals(caa.getVisibile_operatore(), o2)) {
												caa.setVisibile_operatore(o2);
												save = true;
											}
											Boolean a1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_agente(), cc.getModificabile_agente());
											if(!ApplicationUtil.equals(caa.getModificabile_agente(), a1)) {
												caa.setModificabile_agente(a1);
												save = true;
											}
											Boolean a2 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_agente(), cc.getVisibile_agente());
											if(!ApplicationUtil.equals(caa.getVisibile_agente(), a2)) {
												caa.setVisibile_agente(a2);
												save = true;
											}
											String e = ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getEtichetta(), cc.getEtichetta(),  ConstantsGlobal.ereditaEtichetta);
											if(!ApplicationUtil.equals(caa.getEtichetta(), e)) {
												caa.setEtichetta(e);
												save = true;
											}
											String h = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDescrizione(), cc.getDescrizione());
											if(!ApplicationUtil.equals(caa.getDescrizione(), h)) {
												caa.setDescrizione(h);
												save = true;
											}
											String c = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getColore(), cc.getColore());
											if(!ApplicationUtil.equals(caa.getColore(), c)) {
												caa.setColore(c);
												save = true;
											}
											String s = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getStile(), cc.getStile());
											if(!ApplicationUtil.equals(caa.getStile(), s)) {
												caa.setStile(s);
												save = true;
											}
											String d = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDefaults(), cc.getDefaults());
											if(!ApplicationUtil.equals(caa.getDefaults(), d)) {
												caa.setDefaults(d);
												save = true;
											}
											if(save) {
												caa = (CampoAggiuntivoContratto) salvaCampoAggiuntivo(caa);
											}
										}
										DatoAggiuntivoContratto daa = new DatoAggiuntivoContratto();
										daa.setCampo_aggiuntivo(caa);
										if(!dr.contains(daa)){
											daa.setContratto(contratto);
										} else {
											int index = dr.indexOf(daa);
											daa = dr.get(index);
										}
										if(caa.getData()) {
											daa.setValore_data(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getDataora()) {
											daa.setValore_datatime(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getOra()) {
											daa.setValore_time(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getBooleano()) {
											daa.setValore(ApplicationUtil.castBoolean(val.getSingolo()));
										} else if(caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu()) {
											daa.setValore(ApplicationUtil.detoken(val.getMultiplo(), ConstantsGlobal.colonSeparator));
										} else {
											daa.setValore(ApplicationUtil.castString(val.getSingolo()));
										}
										salvaDatoAggiuntivo(daa);
									}
								}
							}
						}
					} else {
						DatoAggiuntivoContratto daa = new DatoAggiuntivoContratto();
						daa.setCampo_aggiuntivo(dato.getCampo());
						if(!dr.contains(daa)){
							daa.setContratto(contratto);
						} else {
							int index = dr.indexOf(daa);
							daa = dr.get(index);
						}
						if(dato.getCampo().getManyCheckbox() || dato.getCampo().getManyMenu() || dato.getCampo().getPickList()) {
							daa.setValore(ApplicationUtil.detoken(dato.getValore_multiplo(), ConstantsGlobal.colonSeparator));
						} else if(dato.getCampo().getData()) {
							daa.setValore_data(dato.getValore_data());
						} else if(dato.getCampo().getDataora()) {
							daa.setValore_datatime(dato.getValore_datatime());
						} else if(dato.getCampo().getOra()) {
							daa.setValore_time(dato.getValore_time());
						} else {
							daa.setValore(dato.getValore());
						}
						salvaDatoAggiuntivo(daa);
					}
				}
			}
		}
	}
	public DomandaContratto creaDomanda(SondaggioContratto sondaggio, CampoAggiuntivoContratto campoAggiuntivo, String posizione, Boolean obbligatoria, Boolean visibile) {
		DomandaContratto domanda = new DomandaContratto();
		domanda.setCampo_aggiuntivo(campoAggiuntivo);
		Short posizioneShort = 1;
		try {
			posizioneShort = Short.parseShort(posizione);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		domanda.setPosizione(posizioneShort);
		domanda.setObbligatoria(obbligatoria);
		domanda.setVisibile(visibile);
		domanda.setSondaggio(sondaggio);
		salvaDomanda(domanda);
		return domanda;
	}
	public void creaDomande(SondaggioContratto sondaggio, List<CampoAggiuntivoContratto> cas, Short last) {
		if(ApplicationUtil.notEmpty(cas)) {
			Iterator<CampoAggiuntivoContratto> itC = cas.iterator();
			while (itC.hasNext()) {
				CampoAggiuntivoContratto campoAggiuntivo = (CampoAggiuntivoContratto) itC.next();
				DomandaContratto domanda = new DomandaContratto();
				domanda.setCampo_aggiuntivo(campoAggiuntivo);
				domanda.setPosizione(last++);
				domanda.setObbligatoria(false);
				domanda.setVisibile(true);
				domanda.setSondaggio(sondaggio);
				salvaDomanda(domanda);
			}
		}
	}
	public CampoAggiuntivoContratto getCampoAggiuntivoById(Long campoAggiuntivoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.idEq(campoAggiuntivoId));
		return (CampoAggiuntivoContratto) getUniqueResult(criteria);
	}
	public DomandaContratto salvaDomanda(DomandaContratto domanda) {
		return domanda = (DomandaContratto) salvaAggiorna(domanda);
	}
	public void eliminaDomanda(DomandaContratto domanda) {
		if (domanda != null)
			cancellaAggiorna(domanda);
	}
	public void eliminaDomande(List<DomandaContratto> domande) {
		if(ApplicationUtil.notEmpty(domande)) {
			Iterator<DomandaContratto> idD = domande.iterator();
			while (idD.hasNext()) {
				DomandaContratto domanda = (DomandaContratto) idD.next();
				cancellaAggiorna(domanda);
			}
		}
	}
	public CampoAggiuntivoContratto getNuovoCampoAggiuntivoContratto() {
		CampoAggiuntivoContratto campo = new CampoAggiuntivoContratto();
		return campo;
	}
	public List<DomandaContratto> copiaDomande(SondaggioContratto sondaggioDst,Long sondaggioIdSrc) {
		List<DomandaContratto> domande = getDomandeSondaggio(sondaggioIdSrc);
		List<Long> domandeE = getCampiEsistenti(sondaggioDst.getId());
		List<DomandaContratto> domande_copia = new ArrayList<DomandaContratto>();
		Iterator<DomandaContratto> objIter = domande.iterator();
		while(objIter.hasNext()) {
			DomandaContratto domanda = objIter.next();
			if(!domandeE.contains(domanda.getCampo_aggiuntivo().getId())) {
				//Domanda nuovo = (Domanda) SerializationUtils.clone(domanda);
				DomandaContratto nuovo = domanda;
				nuovo.setId(null);
				nuovo.setSondaggio(sondaggioDst);
				salvaDomanda(nuovo);
				domande_copia.add(nuovo);
			}
		}
		return domande_copia;
	}
	public void eliminaCampoAggiuntivo(CampoAggiuntivoContratto campoAggiuntivo) {
		cancellaAggiorna(campoAggiuntivo);
	}
	public int countCampoInDomanda(Long idCampoAggiuntivo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaContratto.class);
		criteria.createCriteria("campo_aggiuntivo").add(Restrictions.idEq(idCampoAggiuntivo));
		return getListSize(criteria);
	}
	
	public List<Contratto> impostaConteggioContratto(Utente utente, GenericColumn c,List<Contratto> contratti) {
		if(ApplicationUtil.notEmpty(contratti))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_CONTRATTO);
			if(impostazioneDao.getTabellaDatiAggiuntivi() && (c.getAggiuntiviA() || c.getAggiuntiviC()))  {
				Iterator<Contratto> objIter = contratti.iterator();
				Session session = openSession();
				while(objIter.hasNext()) {
					Contratto contratto = objIter.next();
					Contratto a = (Contratto) session.get(Contratto.class, contratto.getId());
					if(c.getAggiuntiviA()) {
						Hibernate.initialize(a.getContatto().getAnagrafica().getDettagli());
						Iterator<DatoAggiuntivoAnagrafica> objdA = a.getContatto().getAnagrafica().getDettagli().iterator();
						while(objdA.hasNext()) {
							DatoAggiuntivoAnagrafica da = objdA.next();
							contratto.getContatto().getAnagrafica().addDatoM(da.getCampo_aggiuntivo().getId(), da);
						}
					}
					if(c.getAggiuntiviC()) {
						Hibernate.initialize(a.getDettagli());
						Iterator<DatoAggiuntivoContratto> objdC = a.getDettagli().iterator();
						while(objdC.hasNext()) {
							DatoAggiuntivoContratto da = objdC.next();
							contratto.addDatoM(da.getCampo_aggiuntivo().getId(), da);
						}
					}
				}
				closeSession(session);
			}
			List<Long> ll = ApplicationUtil.prendiIdContratto(contratti);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Contratto.class, new ColonnaQuery(ConstantsColumn.allegati, "allegati"), new ColonnaQuery(ConstantsColumn.storia, "storia"));
			Iterator<Contratto> it = contratti.iterator();
			while (it.hasNext()) {
				Contratto o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return contratti;
	}
	public List<Contratto> impostaConteggioContratto(Utente utente, List<Contratto> contratti) {
		if(ApplicationUtil.notEmpty(contratti))  {
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_CONTRATTO);
			List<Long> ll = ApplicationUtil.prendiIdContratto(contratti);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(null, ll, Contratto.class, new ColonnaQuery("allegati", "allegati"));
			Iterator<Contratto> it = contratti.iterator();
			while (it.hasNext()) {
				Contratto o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
		}
		return contratti;
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> ricercaContrattoBackoffice(ContrattoCriteria crti, int start, int size) {
		ExtendedCriteria criteria = ricercaContrattoBackofficeCriteria(crti);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), crti.getO_tipo(), crti.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContrattoBackofficeSize(ContrattoCriteria crti) {
		ExtendedCriteria criteria = ricercaContrattoBackofficeCriteria(crti);
		return getListSize(criteria);
	}
	public BigDecimal ricercaContrattoBackofficeSum(ContrattoCriteria crti) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = ricercaContrattoBackofficeCriteria(crti);
			return getSommaDecimale(criteria , "costo_scontato");
		} else {
			return new BigDecimal(0);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> ricercaContrattoManager(ContrattoCriteria crti, int start, int size) {
		ExtendedCriteria criteria = ricercaContrattoManagerCriteria(crti);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), crti.getO_tipo(), crti.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContrattoManagerSize(ContrattoCriteria crti) {
		ExtendedCriteria criteria = ricercaContrattoManagerCriteria(crti);
		return getListSize(criteria);
	}
	public BigDecimal ricercaContrattoManagerSum(ContrattoCriteria crti) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = ricercaContrattoManagerCriteria(crti);
			return getSommaDecimale(criteria , "costo_scontato");
		} else {
			return new BigDecimal(0);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> ricercaContrattoAgente(Utente utente, ContrattoCriteria crti, int start, int size) {
		ExtendedCriteria criteria = ricercaContrattoAgenteCriteria(utente, crti);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), crti.getO_tipo(), crti.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContrattoAgenteSize(Utente utente, ContrattoCriteria crti) {
		ExtendedCriteria criteria = ricercaContrattoAgenteCriteria(utente, crti);
		return getListSize(criteria);
	}
	public BigDecimal ricercaContrattoAgenteSum(Utente utente, ContrattoCriteria crti) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = ricercaContrattoAgenteCriteria(utente, crti);
			return getSommaDecimale(criteria , "costo_scontato");
		} else {
			return new BigDecimal(0);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> ricercaContrattoRete(Utente utente, ContrattoCriteria crti, int start, int size) {
		ExtendedCriteria criteria = ricercaContrattoReteCriteria(utente, crti);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), crti.getO_tipo(), crti.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContrattoReteSize(Utente utente, ContrattoCriteria crti) {
		ExtendedCriteria criteria = ricercaContrattoReteCriteria(utente, crti);
		return getListSize(criteria);
	}
	public BigDecimal ricercaContrattoReteSum(Utente utente, ContrattoCriteria crti) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = ricercaContrattoReteCriteria(utente, crti);
			return getSommaDecimale(criteria , "costo_scontato");
		} else {
			return new BigDecimal(0);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> ricercaContrattoCoordinatore(Utente utente, ContrattoCriteria crti, int start, int size) {
		ExtendedCriteria criteria = ricercaContrattoCoordinatoreCriteria(utente, crti);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), crti.getO_tipo(), crti.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContrattoCoordinatoreSize(Utente utente, ContrattoCriteria crti) {
		ExtendedCriteria criteria = ricercaContrattoCoordinatoreCriteria(utente, crti);
		return getListSize(criteria);
	}
	public BigDecimal ricercaContrattoCoordinatoreSum(Utente utente, ContrattoCriteria crti) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = ricercaContrattoCoordinatoreCriteria(utente, crti);
			return getSommaDecimale(criteria , "costo_scontato");
		} else {
			return new BigDecimal(0);
		}
	}
	private ExtendedCriteria ricercaContrattoManagerCriteria(ContrattoCriteria crti) {
		if(crti != null)  crti = (ContrattoCriteria) crti.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna");
		impostazioneDao.ricercaIndirizzo(criteriaA, crti, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(crti, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(crti, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(crti, criteria);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(crti, criteria);
		}
		impostazioneDao.sottoCriteriaStoriaContrattoI(crti.getStorico_operatoreId(), crti, criteria);
		if (ApplicationUtil.isNotBlank(crti.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getCamp_sup(), criteriaCA, "nome");
		}
		impostazioneDao.subCriteriaRicercaTeam(filtro, crti.getTeam(), crti.getTeams(), criteriaOp, "id");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaRicercaCampagna(filtro, crti.getTeam(), crti.getTeams(), criteriaCA, "id");
		}
		
		ApplicationUtil.proprieta(filtro, criteria, "stato", crti.getStato(), crti.getStatos());
		if (ApplicationUtil.isNotBlank(crti.getSeriale())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getSeriale(), criteria, "seriale");
		}
		if (ApplicationUtil.isNotBlank(crti.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getNota(), criteria, "nota");
		}
		ApplicationUtil.proprieta(false, filtro, criteriaP, "id", crti.getProdotto(), crti.getProdottos());
		ApplicationUtil.proprieta(false, filtro, criteriaP, "categoria.id", crti.getCategoria(), crti.getCategorias());
		
		if (ApplicationUtil.gtZero(crti.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",crti.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", crti.getEsito(), crti.getEsitos());
		if (ApplicationUtil.gtZero(crti.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(crti.getOperatoreId()));
		}
		if (ApplicationUtil.gtZero(crti.getUtenteId())) {
			criteria.add(Restrictions.eq("utente.id", crti.getUtenteId()));
		}
		if (ApplicationUtil.isNotBlank(crti.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(crti.getRicerca_anagraficaV(), criteriaA, crti.getRicerca_anagrafica());
		}
		crti.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crti);
	}
	private ExtendedCriteria ricercaContrattoBackofficeCriteria(ContrattoCriteria crti) {
		if(crti != null)  crti = (ContrattoCriteria) crti.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna");
		criteriaC.createCriteria("esitoArchiviazione").add(Restrictions.eq("backoffice", Boolean.TRUE));
		impostazioneDao.ricercaIndirizzo(criteriaA, crti, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(crti, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(crti, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(crti, criteria);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(crti, criteria);
		}
		impostazioneDao.sottoCriteriaStoriaContrattoI(crti.getStorico_operatoreId(), crti, criteria);
		if (ApplicationUtil.isNotBlank(crti.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getCamp_sup(), criteriaCA, "nome");
		}
		impostazioneDao.subCriteriaRicercaTeam(filtro, crti.getTeam(), crti.getTeams(), criteriaOp, "id");
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaRicercaCampagna(filtro, crti.getTeam(), crti.getTeams(), criteriaCA, "id");
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", crti.getStato(), crti.getStatos());
		if (ApplicationUtil.isNotBlank(crti.getSeriale())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  crti.getSeriale(), criteria, "seriale");
		}
		if (ApplicationUtil.isNotBlank(crti.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getNota(), criteria, "nota");
		}
		ApplicationUtil.proprieta(false, filtro, criteriaP, "id", crti.getProdotto(), crti.getProdottos());
		ApplicationUtil.proprieta(false, filtro, criteriaP, "categoria.id", crti.getCategoria(), crti.getCategorias());
		if (ApplicationUtil.gtZero(crti.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",crti.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", crti.getEsito(), crti.getEsitos());
		if (ApplicationUtil.gtZero(crti.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(crti.getOperatoreId()));
		}
		if (ApplicationUtil.isNotBlank(crti.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(crti.getRicerca_anagraficaV(), criteriaA, crti.getRicerca_anagrafica());
		}
		crti.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crti);
	}
	private ExtendedCriteria ricercaContrattoAgenteCriteria(Utente utente, ContrattoCriteria crti) {
		if(crti != null)  crti = (ContrattoCriteria) crti.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaE = criteria.createCriteria("esito");
		criteriaE.add(Restrictions.eq("confermato", Boolean.TRUE));
		if (ApplicationUtil.gtZero(crti.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id", crti.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaAgente(utente.getId(), criteria, "agente.id");
		}
		impostazioneDao.ricercaIndirizzo(criteriaA, crti, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(crti, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(crti, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(crti, criteria);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(crti, criteria);
		}
		if (ApplicationUtil.isNotBlank(crti.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getCamp_sup(), criteriaC.createCriteria("campagna"), "nome");
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", crti.getStato(), crti.getStatos());
		if (ApplicationUtil.isNotBlank(crti.getSeriale())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  crti.getSeriale(), criteria, "seriale");
		}
		if (ApplicationUtil.isNotBlank(crti.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getNota(), criteria, "nota");
		}
		ApplicationUtil.proprieta(false, filtro, criteriaP, "id", crti.getProdotto(), crti.getProdottos());
		ApplicationUtil.proprieta(false, filtro, criteriaP, "categoria.id", crti.getCategoria(), crti.getCategorias());
		ApplicationUtil.proprieta(false, filtro, criteriaE, "id", crti.getEsito(), crti.getEsitos());
		if (ApplicationUtil.isNotBlank(crti.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, crti.getOperatore());
		}
		if (ApplicationUtil.isNotBlank(crti.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(crti.getRicerca_anagraficaV(), criteriaA, crti.getRicerca_anagrafica());
		}
		crti.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crti);
	}
	private ExtendedCriteria ricercaContrattoReteCriteria(Utente utente, ContrattoCriteria crti) {
		if(crti != null)  crti = (ContrattoCriteria) crti.loadCriteria();
		Integer max = userDao.getMaxLivelloRete();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaAA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaE = criteria.createCriteria("esito");
		DetachedCriteria criteriaA = criteria.createCriteria("agente");
		criteriaE.add(Restrictions.eq("confermato", Boolean.TRUE));
		if (ApplicationUtil.gtZero(crti.getRete())) {
			if(ApplicationUtil.eq(crti.getRete(),1)) {
				if (ApplicationUtil.gtZero(crti.getAgenteId())) {
					criteriaA.createCriteria("agente_padre").add(Restrictions.idEq(crti.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(crti.getRete(),2)) {
				if (ApplicationUtil.gtZero(crti.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(crti.getAgenteId(), true, max, criteriaA, "id");
				}
			}
			if(ApplicationUtil.eq(crti.getRete(),3)) {
				if (ApplicationUtil.gtZero(crti.getAgenteId())) {
					impostazioneDao.subCriteriaAgentiSottoRete(crti.getAgenteId(), true, max, criteriaA, "id");
					criteriaA.add(Restrictions.ne("id", crti.getAgenteId()));
				}
			}
			if(ApplicationUtil.eq(crti.getRete(),4)) {
				if (ApplicationUtil.gtZero(crti.getAgenteId())) {
					criteriaA.add(Restrictions.idEq(crti.getAgenteId()));
				}
			}
		} else {
			impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 0, max, criteriaA, "id");
		}
		impostazioneDao.ricercaIndirizzo(criteriaAA, crti, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(crti, criteriaAA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(crti, criteriaAA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(crti, criteria);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(crti, criteria);
		}
		if (ApplicationUtil.isNotBlank(crti.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getCamp_sup(), criteriaC.createCriteria("campagna"), "nome");
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", crti.getStato(), crti.getStatos());
		if (ApplicationUtil.isNotBlank(crti.getSeriale())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  crti.getSeriale(), criteria, "seriale");
		}
		if (ApplicationUtil.isNotBlank(crti.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  crti.getNota(), criteria, "nota");
		}
		ApplicationUtil.proprieta(false, filtro, criteriaP, "id", crti.getProdotto(), crti.getProdottos());
		ApplicationUtil.proprieta(false, filtro, criteriaP, "categoria.id", crti.getCategoria(), crti.getCategorias());
		ApplicationUtil.proprieta(false, filtro, criteriaE, "id", crti.getEsito(), crti.getEsitos());
		if (ApplicationUtil.isNotBlank(crti.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, crti.getOperatore());
		}
		if (ApplicationUtil.isNotBlank(crti.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(crti.getRicerca_anagraficaV(), criteriaAA, crti.getRicerca_anagrafica());
		}
		crti.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crti);
	}
	private ExtendedCriteria ricercaContrattoCoordinatoreCriteria(Utente utente, ContrattoCriteria crti) {
		if(crti != null)  crti = (ContrattoCriteria) crti.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaAA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaAg = criteria.createCriteria("agente", "agente");

		impostazioneDao.subCriteriaRoleCoordinatore(utente.getId(), criteria, "campagna", "agente");
		if (ApplicationUtil.gtZero(crti.getAgenteId())) {
			criteriaAg.add(Restrictions.idEq(crti.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCoordinatore(filtro, criteriaAg, utente.getId(), crti.getZona(), crti.getZonas());
		}
		
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCoordinatoreZoneCampagne(filtro, criteriaCA, utente.getId(), crti.getZona(), crti.getZonas());
		}

		impostazioneDao.ricercaIndirizzo(criteriaAA, crti, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(crti, criteriaAA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(crti, criteriaAA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(crti, criteria);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(crti, criteria);
		}
		impostazioneDao.sottoCriteriaStoriaContrattoS(crti.getStorico_operatore(), crti, criteria);
		if (ApplicationUtil.isNotBlank(crti.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getCamp_sup(), criteriaCA, "nome");
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", crti.getStato(), crti.getStatos());
		if (ApplicationUtil.isNotBlank(crti.getSeriale())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getSeriale(), criteria, "seriale");
		}
		if (ApplicationUtil.isNotBlank(crti.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  crti.getNota(), criteria, "nota");
		}
		ApplicationUtil.proprieta(false, filtro, criteriaP, "id", crti.getProdotto(), crti.getProdottos());
		ApplicationUtil.proprieta(false, filtro, criteriaP, "categoria.id", crti.getCategoria(), crti.getCategorias());
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", crti.getEsito(), crti.getEsitos());
		if (ApplicationUtil.isNotBlank(crti.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, crti.getOperatore());
		}
		if (ApplicationUtil.isNotBlank(crti.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(crti.getRicerca_anagraficaV(), criteriaAA, crti.getRicerca_anagrafica());
		}
		crti.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crti);
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoContratto> getDatiAggiuntiviContratto(Long contrattoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
		criteria.add(Restrictions.eq("contratto.id",contrattoId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoContratto> getDatiAggiuntiviContrattoAgente(Long contrattoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
		criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("visibile_agente", Boolean.TRUE));
		criteria.add(Restrictions.eq("contratto.id",contrattoId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoContratto> getDatiAggiuntiviContrattoOperatore(Long contrattoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
		criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("visibile_operatore", Boolean.TRUE));
		criteria.add(Restrictions.eq("contratto.id",contrattoId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoContratto> getDatiAggiuntiviContratto(Boolean supervisore, Long contrattoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
		if(!ApplicationUtil.isTrue(supervisore)) {
			criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("visibile_operatore", Boolean.TRUE));
		}
		criteria.add(Restrictions.eq("contratto.id",contrattoId));
		return getList(criteria);
	}
	public Contratto salva(Contratto contratto, Utente utente, List<DatoAggiuntivoContratto> dativo) {
		if(ApplicationUtil.notEmpty(dativo)){
			Iterator<DatoAggiuntivoContratto> objIter = dativo.iterator();
			while(objIter.hasNext()) {
				DatoAggiuntivoContratto davo = objIter.next();
				salvaDatoAggiuntivo(davo);
			}
		}
		contratto.setDettagli(dativo);
		salvaContratto(contratto, utente, true);
		return contratto;
	}
	public Contratto salva(Contratto contratto, Utente utente) {
		salvaContratto(contratto, utente, true);
		return contratto;
	}
	public void salvaContratto(Contratto contratto, Utente utente, boolean f) {
		Date data = ApplicationUtil.date();
		boolean modificato = modificaEsitoStatoContratto(contratto, data, f);
		contratto.setData_lavorazione(data);
		contratto.setUtente(utente);
		contratto.impostaVendita();
		contratto.impostaUserfield();
		boolean inserito = (contratto.getId() == null);
		if(inserito) {
			contratto.setData_esito(data);
			contratto.setData_stato(data);
		}
		salvaAggiorna(contratto);
		new SalvaStoriaContrattoThread(this, contratto, utente, data, inserito, modificato).start();
	}
	public void salvaStoriaContratto(Contratto contratto, Utente utente, Date data, boolean inserito, boolean modificato) {
		StoriaContratto storia = creaStoriaContratto(contratto, utente, data);
		salvaAggiorna(storia);
		if((modificato || inserito) && contratto.getEsito() != null) {
			if (ApplicationUtil.isNotBlank(contratto.getEsito().getMessaggio_notifica()) || ApplicationUtil.isTrue(contratto.getEsito().getMessaggio_operatore()) || ApplicationUtil.isTrue(contratto.getEsito().getMessaggio_agente()) ) {
				new ContrattoMessaggioThread(contratto, messaggioDao, downloadDao, userDao, utente).start();
			}
			if (ApplicationUtil.isNotBlank(contratto.getEsito().getNotizia_notifica())  || ApplicationUtil.isTrue(contratto.getEsito().getNotizia_operatore()) || ApplicationUtil.isTrue(contratto.getEsito().getNotizia_agente()) ) {
				new ContrattoNotiziaThread(contratto, notiziaDao, downloadDao, userDao, utente).start();
			}
		}
	}
	public void salvaContrattoE(Contratto contratto, Utente utente, boolean f) {
		Date data = ApplicationUtil.date();
		boolean modificato = modificaEsitoStatoContratto(contratto, data, f);
		contratto.setData_lavorazione(data);
		contratto.setUtente(utente);
		contratto.impostaVendita();
		contratto.impostaUserfield();
		boolean inserito = (contratto.getId() == null);
		if(inserito) {
			contratto.setData_esito(data);
			contratto.setData_stato(data);
		}
		salvaAggiorna(contratto);
		StoriaContratto storia = creaStoriaContratto(contratto, utente, data);
		salvaAggiorna(storia);
		if((modificato || inserito) && contratto.getEsito() != null) {
			if (ApplicationUtil.isNotBlank(contratto.getEsito().getMessaggio_notifica()) || ApplicationUtil.isTrue(contratto.getEsito().getMessaggio_operatore()) || ApplicationUtil.isTrue(contratto.getEsito().getMessaggio_agente()) ) {
				new ContrattoMessaggioThread(contratto, messaggioDao, downloadDao, userDao, utente).start();
			}
			if (ApplicationUtil.isNotBlank(contratto.getEsito().getNotizia_notifica()) || ApplicationUtil.isTrue(contratto.getEsito().getNotizia_operatore()) || ApplicationUtil.isTrue(contratto.getEsito().getNotizia_agente()) ) {
				new ContrattoNotiziaThread(contratto, notiziaDao, downloadDao, userDao, utente).start();
			}
		}
	}
	public StoriaContratto creaStoriaContratto(Contratto  contratto, Utente utente, Date data) {
		StoriaContratto storia = new StoriaContratto();
		storia.setData(data);
		storia.setUtente(utente);
		storia.setBloccato(contratto.getBloccato());
		storia.setContratto(contratto);
		storia.setOperatore(contratto.getOperatore());
		storia.setEsito(contratto.getEsito());
		storia.setData_stipula(contratto.getData_stipula());
		storia.setData_contabile(contratto.getData_contabile());
		storia.setData_attivazione(contratto.getData_attivazione());
		storia.setData_caricamento(contratto.getData_caricamento());
		storia.setData_esito(contratto.getData_esito());
		storia.setData_stato(contratto.getData_stato());
		storia.setSeriale(contratto.getSeriale());
		storia.setQuantita(contratto.getQuantita());
		storia.setCosto_scontato(contratto.getCosto_scontato());
		storia.setStato(contratto.getStato());
		storia.setNota(contratto.getNota());
		storia.setProdotto(contratto.getProdotto());
		storia.setAgente(contratto.getAgente());
		return storia;
	}
	public boolean modificaEsitoStatoContratto(Contratto  contratto, Date data, boolean f) {
		boolean modificato = false;
		if(contratto!=null && contratto.getId() != null && f) {
			Contratto crt = (Contratto) get(Contratto.class, contratto.getId());
			if(crt!=null) {
				int esito = ApplicationUtil.compareTo(contratto.getEsito() != null ? contratto.getEsito().getId() : null, crt.getEsito() != null ? crt.getEsito().getId() : null);
				if (esito != 0) {
					contratto.setData_esito(data);
					modificato = true;
				}
				int stato = ApplicationUtil.compareTo(contratto.getStato(), crt.getStato());
				if (stato != 0) {
					contratto.setData_stato(data);
					modificato = true;
				}
			}
		}
		return modificato;
	}
	public DatoAggiuntivoContratto creaDatoAggiuntivo(Contratto contratto, CampoAggiuntivoContratto caa, Object val) {
		DatoAggiuntivoContratto daa = new DatoAggiuntivoContratto();
		daa.setContratto(contratto);
		daa.setCampo_aggiuntivo(caa);
		if(caa.getData()) {
			daa.setValore_data(ApplicationUtil.castDate(val));
		} else if(caa.getDataora()) {
			daa.setValore_datatime(ApplicationUtil.castDate(val));
		} else if(caa.getOra()) {
			daa.setValore_time(ApplicationUtil.castDate(val));
		} else if(caa.getBooleano()) {
			daa.setValore(ApplicationUtil.castBoolean(val));
		} else if(caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu()) {
			daa.setValore(ApplicationUtil.castList(val, ConstantsGlobal.colonSeparator));
		} else {
			daa.setValore(ApplicationUtil.castString(val));
		}
		salvaDatoAggiuntivo(daa);
		return daa;
	}
	public void creaDatoAggiuntivoList(Contratto anagrafica, List<CampoAggiuntivoContratto> campi) {
		if(ApplicationUtil.notEmpty(campi)) {
			Iterator<CampoAggiuntivoContratto> dmIt = campi.iterator();
			while (dmIt.hasNext()) {
				CampoAggiuntivoContratto campoAggiuntivo = (CampoAggiuntivoContratto) dmIt.next();
				DatoAggiuntivoContratto datoAggiuntivo = new DatoAggiuntivoContratto();
				datoAggiuntivo.setContratto(anagrafica);
				datoAggiuntivo.setCampo_aggiuntivo(campoAggiuntivo);
				salvaDatoAggiuntivo(datoAggiuntivo);
			}
		}
	}
	public void eliminaDatoAggiuntivo(DatoAggiuntivoContratto datoAggiuntivo) {
		cancellaAggiorna(datoAggiuntivo);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> getCampiAggiuntiviContratto(List<String> cc) {
		if(ApplicationUtil.notEmpty(cc)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
			criteria.add(Restrictions.in("chiave",cc));
			return getList(criteria);
		} else return new ArrayList<CampoAggiuntivoContratto>();
	}
	
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> getAllCampiAggiuntiviSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.ne("tipo",ConstantsMap.TIPO_FORM));
		criteria.addOrder(Order.asc("etichetta"));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> getAllCampiAggiuntiviSIAgente() {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.eq("visibile_agente", Boolean.TRUE));
		criteria.add(Restrictions.ne("tipo",ConstantsMap.TIPO_FORM));
		criteria.addOrder(Order.asc("etichetta"));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> getAllCampiAggiuntiviSIOperatore() {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.eq("visibile_operatore", Boolean.TRUE));
		criteria.add(Restrictions.ne("tipo",ConstantsMap.TIPO_FORM));
		criteria.addOrder(Order.asc("etichetta"));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Contratto> getContrattiByContattoId(Boolean supervisore, Long contattoId, ContrattoCriteria crti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaEs = criteria.createCriteria("esito", ApplicationUtil.leftJoin());
		criteria.add(Restrictions.eq("contatto.id",contattoId));
		if(ApplicationUtil.isFalse(supervisore)) { 
			criteriaEs.add(Restrictions.eq("visibile", Boolean.TRUE));
		}
		if (ApplicationUtil.gtZero(crti.getEsito())) {
			criteriaEs.add(Restrictions.idEq(crti.getEsito()));
		}
		if (ApplicationUtil.gtZero(crti.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",crti.getAgenteId()));
		}
		if (ApplicationUtil.gtZero(crti.getOperatoreId())) {
			criteria.add(Restrictions.eq("operatore.id",crti.getOperatoreId()));
		}
		criteria.addOrder(Order.asc("data_creazione"));
		return getList(criteria);
	}
	public void eliminaContratti(Utente utente, List<Contratto> contratti) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
		operazioneDao.creaOperazione(ConstantsMap.LOG3 ,p, utente, ConstantsMap.OPERAZIONE_ELIMINA_CONTRATTO);
		cancellaLista(contratti);
	}
	public void modificaEsitoSelezionati( Utente utente,  List<Contratto> contratti, Long esitoNuovoId) {
		EsitoContratto esitoNuovo = campagnaDao.getEsitoContratto(esitoNuovoId);
		modificaEsito(utente, contratti, esitoNuovo);
	}
	public void modificaAgenteSelezionati( Utente utente,  List<Contratto> contratti,  Long agenteNuovoId) {
		Agente agenteNuovo =  campagnaDao.getAgente(agenteNuovoId);
		modificaAgente(utente, contratti, agenteNuovo);
	}
	public void modificaStatoSelezionati( Utente utente,  List<Contratto> contratti,  Integer stato) {
		modificaStato(utente, contratti, stato);
	}
	public void modificaOperatoreSelezionati( Utente utente,  List<Contratto> contratti, Long proprietarioID) {
		Utente operatore = userDao.getUser(proprietarioID);
		modificaOperatore(utente, contratti, operatore);
	}
	public void modificaOperatore(Utente utente, List<Contratto> contratti, Utente operatoreNuovo) {
		Iterator<Contratto> objIter = contratti.iterator();
		while(objIter.hasNext()) {
			Contratto contratto = objIter.next();
			contratto.setOperatore(operatoreNuovo);
			salvaContratto(contratto, utente, false);
		}
	}
	public void modificaStato(Utente utente, List<Contratto> contratti, Integer stato ) {
		Iterator<Contratto> objIter = contratti.iterator();
		while(objIter.hasNext()) {
			Contratto contratto = objIter.next();
			contratto.setStato(stato);
			salvaContratto(contratto, utente, true);
		}
	}
	public void modificaAgente(Utente utente, List<Contratto> contratti, Agente agenteNuovo ) {
		Iterator<Contratto> objIter = contratti.iterator();
		while(objIter.hasNext()) {
			Contratto contratto = objIter.next();
			contratto.setAgente(agenteNuovo);
			salvaContratto(contratto, utente, false);
		}
	}
	public void modificaEsito(Utente utente, List<Contratto> contratti, EsitoContratto esitoNuovo ) {
		Iterator<Contratto> objIter = contratti.iterator();
		while(objIter.hasNext()) {
			Contratto contratto = objIter.next();
			contratto.setEsito(esitoNuovo);
			salvaContratto(contratto, utente, true);
		}
	}
	public void modificaEsito(Utente utente, Contratto contratto, Long esitoNuovoId ) {
		EsitoContratto esitoNuovo = campagnaDao.getEsitoContratto(esitoNuovoId);
		contratto.setEsito(esitoNuovo);
		salvaContratto(contratto, utente, true);
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> ricercaContrattoSupervisore(Utente utente, ContrattoCriteria crti, int start, int size) {
		ExtendedCriteria criteria = ricercaContrattoSupervisoreCriteria(utente, crti);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), crti.getO_tipo(), crti.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContrattoSupervisoreSize(Utente utente, ContrattoCriteria crti) {
		ExtendedCriteria criteria = ricercaContrattoSupervisoreCriteria(utente, crti);
		return getListSize(criteria);
	}	
	public BigDecimal ricercaContrattoSupervisoreSum(Utente utente, ContrattoCriteria crti) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = ricercaContrattoSupervisoreCriteria(utente, crti);
			return getSommaDecimale(criteria , "costo_scontato");
		} else {
			return new BigDecimal(0);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> ricercaContrattoOperatore(Utente utente, ContrattoCriteria crti, int start, int size) {
		ExtendedCriteria criteria = ricercaContrattoOperatoreCriteria(utente, crti);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), crti.getO_tipo(), crti.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContrattoOperatoreSize(Utente utente, ContrattoCriteria crti) {
		ExtendedCriteria criteria = ricercaContrattoOperatoreCriteria(utente, crti);
		return getListSize(criteria);
	}
	public BigDecimal ricercaContrattoOperatoreSum(Utente utente, ContrattoCriteria crti) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = ricercaContrattoOperatoreCriteria(utente, crti);
			return getSommaDecimale(criteria , "costo_scontato");
		} else {
			return new BigDecimal(0);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> ricercaContrattoPartner(Utente utente, ContrattoCriteria crti, int start, int size) {
		ExtendedCriteria criteria = ricercaContrattoPartnerCriteria(utente, crti);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), crti.getO_tipo(), crti.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaContrattoPartnerSize(Utente utente, ContrattoCriteria crti) {
		ExtendedCriteria criteria = ricercaContrattoPartnerCriteria(utente, crti);
		return getListSize(criteria);
	}
	public BigDecimal ricercaContrattoPartnerSum(Utente utente, ContrattoCriteria crti) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = ricercaContrattoPartnerCriteria(utente, crti);
			return getSommaDecimale(criteria , "costo_scontato");
		} else {
			return new BigDecimal(0);
		}
	}
	private ExtendedCriteria ricercaContrattoSupervisoreCriteria(Utente utente, ContrattoCriteria crti) {
		if(crti != null)  crti = (ContrattoCriteria) crti.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		impostazioneDao.subCriteriaRoleSupervisore(utente.getId(), criteria, "campagna", "operatore");
		impostazioneDao.subCriteriaSupervisore(filtro, utente.getId(), criteriaOp, "id",crti.getTeam(), crti.getTeams());
		
		if(ApplicationUtil.isTrue(sd)) {
			impostazioneDao.subCriteriaCampagnaSupervisore(utente.getId(), criteriaCA, "id", filtro,crti.getTeam(), crti.getTeams());
		}
		impostazioneDao.ricercaIndirizzo(criteriaA, crti, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(crti, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(crti, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(crti, criteria);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(crti, criteria);
		}
		impostazioneDao.sottoCriteriaStoriaContrattoI(crti.getStorico_operatoreId(), crti, criteria);
		if (ApplicationUtil.isNotBlank(crti.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  crti.getCamp_sup(), criteriaCA, "nome");
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", crti.getStato(), crti.getStatos());
		if (ApplicationUtil.isNotBlank(crti.getSeriale())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getSeriale(), criteria, "seriale");
		}
		if (ApplicationUtil.isNotBlank(crti.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getNota(), criteria, "nota");
		}
		ApplicationUtil.proprieta(false, filtro, criteriaP, "id", crti.getProdotto(), crti.getProdottos());
		ApplicationUtil.proprieta(false, filtro, criteriaP, "categoria.id", crti.getCategoria(), crti.getCategorias());
		if (ApplicationUtil.gtZero(crti.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",crti.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", crti.getEsito(), crti.getEsitos());
		if (ApplicationUtil.gtZero(crti.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(crti.getOperatoreId()));
		}
		if (ApplicationUtil.isNotBlank(crti.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(crti.getRicerca_anagraficaV(), criteriaA, crti.getRicerca_anagrafica());
		}
		crti.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crti);
	}
	private ExtendedCriteria ricercaContrattoOperatoreCriteria(Utente utente, ContrattoCriteria crti) {
		if(crti != null)  crti = (ContrattoCriteria) crti.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaE = criteria.createCriteria("esito");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "campagna");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		criteriaE.add(Restrictions.eq("visibile", Boolean.TRUE));
		if(ApplicationUtil.isTrue(crti.getAzione())) {
			impostazioneDao.subCriteriaRoleOperatore(utente.getId(), criteria, "campagna", "operatore");
			impostazioneDao.subCriteriaOperatore(utente.getId(), criteriaOp, "id", filtro, crti.getTeam(), crti.getTeams());
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaOperatore(utente.getId(), criteriaCA, "id",filtro, crti.getTeam(), crti.getTeams());
			}
			if (ApplicationUtil.gtZero(crti.getOperatoreId())) {
				criteriaOp.add(Restrictions.idEq(crti.getOperatoreId()));
			}
		} else {
			criteriaOp.add(Restrictions.idEq(utente.getId()));
		}
		impostazioneDao.sottoCriteriaStoriaContrattoI(null, crti, criteria);
		impostazioneDao.ricercaIndirizzo(criteriaA, crti, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(crti, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(crti, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(crti, criteria);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(crti, criteria);
		}
		if (ApplicationUtil.isNotBlank(crti.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getCamp_sup(), criteriaCA, "nome");
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", crti.getStato(), crti.getStatos());
		if (ApplicationUtil.isNotBlank(crti.getSeriale())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getSeriale(), criteria, "seriale");
		}
		if (ApplicationUtil.isNotBlank(crti.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getNota(), criteria, "nota");
		}
		ApplicationUtil.proprieta(false, filtro, criteriaP, "id", crti.getProdotto(), crti.getProdottos());
		ApplicationUtil.proprieta(false, filtro, criteriaP, "categoria.id", crti.getCategoria(), crti.getCategorias());
		if (ApplicationUtil.gtZero(crti.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",crti.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteriaE, "id", crti.getEsito(), crti.getEsitos());
		if (ApplicationUtil.isNotBlank(crti.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(crti.getRicerca_anagraficaV(), criteriaA, crti.getRicerca_anagrafica());
		}
		crti.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crti);
	}
	private ExtendedCriteria ricercaContrattoPartnerCriteria(Utente utente, ContrattoCriteria crti) {
		if(crti != null)  crti = (ContrattoCriteria) crti.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		String indirizzo = impostazioneDao.getRicercaIndirizzo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaP = criteria.createCriteria("prodotto");
		DetachedCriteria criteriaCa = criteriaC.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaPartner(utente.getId(), criteriaCa, "id");
		impostazioneDao.subCriteriaRicercaTeam(filtro, crti.getTeam(), crti.getTeams(), criteriaOp, "id");
		if (ApplicationUtil.gtZero(crti.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(crti.getOperatoreId()));
		}
		impostazioneDao.ricercaIndirizzo(criteriaA, crti, indirizzo);
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(crti, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(crti, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(crti, criteria);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(crti, criteria);
		}
		impostazioneDao.sottoCriteriaStoriaContrattoI(null, crti, criteria);
		if (ApplicationUtil.isNotBlank(crti.getCamp_sup())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  crti.getCamp_sup(), criteriaCa, "nome");
		}
		ApplicationUtil.proprieta(filtro, criteria, "stato", crti.getStato(), crti.getStatos());
		if (ApplicationUtil.isNotBlank(crti.getSeriale())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crti.getSeriale(), criteria, "seriale");
		}
		if (ApplicationUtil.isNotBlank(crti.getNota())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,  crti.getNota(),criteria, "nota");
		}
		ApplicationUtil.proprieta(false, filtro, criteriaP, "id", crti.getProdotto(), crti.getProdottos());
		ApplicationUtil.proprieta(false, filtro, criteriaP, "categoria.id", crti.getCategoria(), crti.getCategorias());
		if (ApplicationUtil.gtZero(crti.getAgenteId())) {
			criteria.add(Restrictions.eq("agente.id",crti.getAgenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", crti.getEsito(), crti.getEsitos());
		if (ApplicationUtil.isNotBlank(crti.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(crti.getRicerca_anagraficaV(), criteriaA, crti.getRicerca_anagrafica());
		}
		crti.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crti);
	}
	public 	List<MyFile> listaPDA(Contratto contratto) {
		List<MyFile> ff = new ArrayList<MyFile>();
		File perc = new File(impostazioneDao.getPathBase(ConstantsGlobal.path_pda));
		if(perc.exists() && perc.isDirectory()) {
			File [] files = perc.listFiles(new ModuloFileFilter(ConstantsGlobal.pda + "_" +contratto.getId()+"_", true));
			if(ApplicationUtil.notEmpty(files)) {
				for(File file : files) {
					MyFile f = new MyFile(file);
					f.setName(file.getName());
					f.setModified(file.lastModified());
					ff.add(f);
				}
			}
		}
		return  ff;
	}
	public 	void eliminaPDAFtp(Contratto contratto) {
		List<MyFile> ff = listaPDA(contratto);
		if(ApplicationUtil.notEmpty(ff)) {
			for(MyFile file : ff) {
				eliminaPDAFtp(file);
			}
		}
	}
	public 	List<MyFile> listaPDA(FileCriteria mm) {
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			File perc = new File(impostazioneDao.getPathBase(ConstantsGlobal.path_pda));
			if(perc.exists() && perc.isDirectory()) {
				File [] files = perc.listFiles();
				if(ApplicationUtil.notEmpty(files)) {
					List<String> rs = ApplicationUtil.token(mm.getFullText(), ConstantsGlobal.colonSeparator);
					for(File file : files) {
						if(mm.isRicerca(file.getName(), file.length(), file.lastModified()) && (ApplicationUtil.textInPDF(file, rs) || ApplicationUtil.textInXLS(file, rs))) {
							MyFile f = new MyFile(file);
							f.setName(file.getName());
							f.setModified(file.lastModified());
							String a = ApplicationUtil.substringAfter(file.getName(), "_");
							String b = ApplicationUtil.trimToNull(ApplicationUtil.substringBefore(a, "_"));
							if(b != null) {
								Long id = null;
								try {
									id = Long.parseLong(b);
								} catch (Exception nfe) {
								}
								if(id != null) {
									f.setId(id);
								}
							}
							ff.add(f);
						}
					}
				}
			}
		}
		return  ff;
	}
	public void eliminaPDAFtp(MyFile pda) {
		File ff = new File(impostazioneDao.getPathBase(ConstantsGlobal.path_pda) +"/"+ pda.getName());
		ApplicationUtil.deleteFile(ff);
	}
	public void eliminaPDAFtp(List<MyFile> pdas) {
		if(ApplicationUtil.notEmpty(pdas)) {
			Iterator<MyFile> eIt = pdas.iterator();
			while (eIt.hasNext()) {
				MyFile file = (MyFile) eIt.next();
				eliminaPDAFtp(file);
			}
		}
	}
	public void eliminaPDAFtp() {
		File perc = new File(impostazioneDao.getPathBase(ConstantsGlobal.path_pda));
		if(perc.exists() && perc.isDirectory()) {
			File [] files = perc.listFiles();
			if(ApplicationUtil.notEmpty(files)) {
				for(File file : files) {
					MyFile f = new MyFile(file);
					f.setName(file.getName());
					f.setModified(file.lastModified());
					eliminaPDAFtp(f);
				}
			}
		}
	}
	public void scaricaPDA(Contratto contratto, MyFile file, DownloadBean downloadBean) {
		try {
			String path = impostazioneDao.getPathBase(ConstantsGlobal.path_pda);
			MyFile pda = campagnaDao.reloadFileP(contratto != null ? contratto.getId() : file.getId(), path + "/" + ApplicationUtil.getNameN(file.getName()));
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(pda.getName());
			if(pda.getData() == null) {
				InputStream istream = new FileInputStream(pda.getName());
				ApplicationUtil.copy(istream, out);
				istream.close();
				downloadBean.close();
			} else {
				out.write(pda.getData());
				downloadBean.close();
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} finally {
		}
	}
	public String openPDA(Contratto contratto, MyFile file) {
		try {
			String path = impostazioneDao.getPathBase(ConstantsGlobal.path_pda);
			MyFile pda = campagnaDao.reloadFileP(contratto != null ? contratto.getId() : file.getId(), path + "/" + ApplicationUtil.getNameN(file.getName()));
			if(pda.getData() == null) {
				byte[] bytesArray = new byte[(int) pda.getLength()];
				FileInputStream fileInputStream = new FileInputStream(pda.getName());
				fileInputStream.read(bytesArray);
				fileInputStream.close();
				return ApplicationUtil.codificato(ApplicationUtil.getContentType(pda.getName()), bytesArray);
			} else {
				return ApplicationUtil.codificato(ApplicationUtil.getContentType(pda.getName()), pda.getData());
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} finally {
		}
		return null;
	}
	public void eliminaDatiAggiuntiviContratto(List<DatoAggiuntivoContratto> dati) {
		cancellaLista(dati);
	}
	public void salvaDatiAggiuntiviContratto(List<DatoAggiuntivoContratto> dati) {
		if(ApplicationUtil.notEmpty(dati)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<DatoAggiuntivoContratto> objIter = dati.iterator();
			while(objIter.hasNext()) {
				DatoAggiuntivoContratto dato = objIter.next();
				session.saveOrUpdate(dato);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	public Long getContrattoBySeriale(String enteredName) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		criteria.add(Restrictions.eq("seriale", enteredName));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return (Long) getUniqueResult(criteria);
	}
	public void salvaAllegatoContratto(FileUpload fileUpload, ContrattoProdotto contratto) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					Allegato all = new Allegato(ff.getName(), ffB);
					all.setData_modifica(ApplicationUtil.date());
					all.setDimensione(ffB.length);
					if(!contratto.getAllegato().contains(all)){
						contratto.getAllegato().add(all);
					}
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void deleteAllegatoContratto(Allegato allegato, ContrattoProdotto contratto) {
		contratto.getAllegato().remove(allegato);
	}
	public void deleteAllegatoContratto(ContrattoProdotto contratto) {
		contratto.getAllegato().clear();
	}
	public void scaricaAllegatoContratto(Allegato allegato, DownloadBean downloadBean) {
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());
			InputStream is = new ByteArrayInputStream(allegato.getFile());
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void scaricaAllegatoContratto(ContrattoProdotto contratto, DownloadBean downloadBean) {
		List<Allegato> allegati = contratto.getAllegato();
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CONTRATTO.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				ApplicationUtil.zip(null,allegati, out, p);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public List<ContrattoVO>  nuovaListaContrattoVO() {
		return new ArrayList<ContrattoVO>();
	}
	public List<ContrattoVO>  salvaContratti(TemplateImportContratto ti, ImportModelContratto tiM, Utente utente, Integer stato, Long esitoNuovoId, Integer modifica, boolean forza, Date data, Integer step, Boolean modo) {
		List<ContrattoVO> nonTrovati = new ArrayList<ContrattoVO>();
		if(ti != null) {
			List<Contratto> contrattiNew = tiM.getElementi();

			if(ApplicationUtil.eq(modifica, 1)) {
				if (ApplicationUtil.notEmpty(contrattiNew)) {
					EsitoContratto es = null;
					if(ApplicationUtil.gtZero(esitoNuovoId)) {
						es = campagnaDao.getEsitoContratto(esitoNuovoId);
					}
					List<String> valori = ti.getValore_multiplo();
					List<Contratto>  contr = prendiEsistenti(contrattiNew, tiM.getTipo());
					for (Contratto appNew : contrattiNew) {
						List<Contratto> ctrs = trovaContratto(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Contratto ctr : ctrs) {
								if (!appNew.getDuplicato()) {
									if(es != null) {
										ctr.setEsito(es);
									}
									if(valori.indexOf(ConstantsMap.contratto_nota) > -1) {
										ctr.setNota(appNew.getNota());
									}
									if(valori.indexOf(ConstantsMap.contratto_data_contabile) > -1) {
										ctr.setData_contabile(appNew.getData_contabile());
									}
									if(valori.indexOf(ConstantsMap.contratto_data_stipula) > -1) {
										ctr.setData_stipula(appNew.getData_stipula());
									}
									if(valori.indexOf(ConstantsMap.contratto_data_attivazione) > -1) {
										ctr.setData_attivazione(appNew.getData_attivazione());
									}
									if(valori.indexOf(ConstantsMap.contratto_data_caricamento) > -1) {
										ctr.setData_caricamento(appNew.getData_caricamento());
									}
									if(valori.indexOf(ConstantsMap.contratto_costo_scontato) > -1) {
										ctr.setCosto_scontato(appNew.getCosto_scontatoU());
									}
									if(valori.indexOf(ConstantsMap.contratto_quantita) > -1) {
										ctr.setQuantita(appNew.getQuantitaU());
									}
									if (ApplicationUtil.gtZero(stato)) {
										if(stato.equals(ConstantsGlobal.in_attesa)){
											if(!ctr.getStato().equals(ConstantsGlobal.stornato)) {
												nonTrovati.add(new ContrattoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										}
										if(stato.equals(ConstantsGlobal.in_pagamento)){
											if(!ctr.getStato().equals(ConstantsGlobal.in_attesa)) {
												nonTrovati.add(new ContrattoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										}
										if(stato.equals(ConstantsGlobal.pagato)){
											if(!ctr.getStato().equals(ConstantsGlobal.in_pagamento)) {
												nonTrovati.add(new ContrattoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										}
										if(stato.equals(ConstantsGlobal.in_storno)){
											if(!ctr.getStato().equals(ConstantsGlobal.pagato)) {
												nonTrovati.add(new ContrattoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										} 
										if(stato.equals(ConstantsGlobal.stornato)){
											if(!ctr.getStato().equals(ConstantsGlobal.in_storno)) {
												nonTrovati.add(new ContrattoVO(appNew, stato, ConstantsLanguage.ANOMALIA.stampa() +": "  + ctr.getStatoTesto(), es));
												if(forza) {
													ctr.setStato(stato);
												}
											} else {
												ctr.setStato(stato);
											}
										}
									}
									salvaContratto(ctr, utente, true);
									String p = ApplicationUtil.asString( ConstantsLanguage.CONTRATTO.stampa(), ctr);
									operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_MODIFICA_CONTRATTO);
								} else {
									nonTrovati.add(new ContrattoVO(appNew, stato, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), es));
								}
							}
						} else {
							nonTrovati.add(new ContrattoVO(appNew, stato, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), es));
						}
					}
				}
			} 
			if(ApplicationUtil.eq(modifica, 2)) {
				if (ApplicationUtil.notEmpty(contrattiNew)) {
					List<Contratto>  contr = prendiEsistenti(contrattiNew, tiM.getTipo());
					for (Contratto appNew : contrattiNew) {
						List<Contratto> ctrs = trovaContratto(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Contratto ctr : ctrs) {
								if (!appNew.getDuplicato()) {
									eliminaContratto(utente, ctr);
								} else {
									nonTrovati.add(new ContrattoVO(appNew, null, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), null));
								}
							}
						} else {
							nonTrovati.add(new ContrattoVO(appNew, null, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), null));
						}
					}
				}
			}
			if(ApplicationUtil.eq(modifica, 3)) {
				if (ApplicationUtil.notEmpty(contrattiNew)) {
					List<Contratto>  contr = prendiEsistenti(contrattiNew, tiM.getTipo());
					for (Contratto appNew : contrattiNew) {
						List<Contratto> ctrs = trovaContratto(contr, appNew.getChiaveL(tiM.getTipo()), tiM.getTipo());
						if (ApplicationUtil.notEmpty(ctrs)) {
							for (Contratto ctr : ctrs) {
								if (!appNew.getDuplicato()) {
									revertContratto(ctr, utente, data, step, modo);
								} else {
									nonTrovati.add(new ContrattoVO(appNew, null, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.DUPLICATO.stampa()), null));
								}
							}
						} else {
							nonTrovati.add(new ContrattoVO(appNew, null, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NON_TROVATO.stampa()), null));
						}
					}
				}
			}
		}
		return nonTrovati;
	}
	private List<Contratto> trovaContratto( List<Contratto> contratti, Object seriale, String tipo) {
		List<Contratto> o = new ArrayList<Contratto>();
		if(ApplicationUtil.notEmpty(contratti)) {
			for (Contratto contratto: contratti) {
				Object serial = contratto.getChiaveO(tipo);
				if (serial != null && seriale != null && serial.equals(seriale))
					o.add(contratto);
			}
		}
		return o;
	}
	@SuppressWarnings("unchecked")
	public List<Contratto> prendiEsistenti(List<Contratto> conts, String tipo) {
		List<Object> contrattiL = new ArrayList<Object>();
		if(ApplicationUtil.notEmpty(conts)) {
			for (Contratto cont : conts) {
				Object seriale = cont.getChiaveL(tipo);
				if(seriale!= null && !contrattiL.contains(seriale) && ApplicationUtil.isTrue(cont.getTrovato())) {
					contrattiL.add(seriale);
				}
			}
		}
		if(ApplicationUtil.notEmpty(contrattiL))		{
			DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
			if(tipo != null && !ConstantsMap.contratto_id.equals(tipo)) {
				if(ConstantsMap.contratto_seriale.equals(tipo)) {
					criteria.add(Restrictions.in("seriale", contrattiL)); 
				} else {
					String tipoS = tipo.substring((ConstantsMap.anagrafica + "_").length());
					criteria.createCriteria("contatto").createCriteria("anagrafica").add(Restrictions.in(tipoS, contrattiL));
				}
			} else {
				criteria.add(Restrictions.in("id", contrattiL)); 
			}
			return getList(criteria);
		}
		return null;
	}
	public List<CampoAggiuntivoContratto> getCampiAggiuntiviRicerca() {
		Boolean s = impostazioneDao.getRicercaDomandaContrattoSingola();
		Boolean m = impostazioneDao.getRicercaDomandaContrattoMultipla();
		if(ApplicationUtil.isTrue(s) || ApplicationUtil.isTrue(m)) {
			return getAllCampiAggiuntivi();
		}
		return null;
	}
	public List<CampoAggiuntivoContratto> getCampiAggiuntiviTabella() {
		Boolean m = impostazioneDao.getTabellaDatiAggiuntivi();
		if(ApplicationUtil.isTrue(m)) {
			return getAllCampiAggiuntivi();
		}
		return null;
	}
	public Map<Long, CampoAggiuntivoContratto> preparaMappa(List<CampoAggiuntivoContratto> campi) {
		if(ApplicationUtil.notEmpty(campi)) {
			Map<Long, CampoAggiuntivoContratto> mappa = new LinkedHashMap<Long, CampoAggiuntivoContratto>();
			Iterator<CampoAggiuntivoContratto> itA = campi.iterator();
			while (itA.hasNext()) {
				CampoAggiuntivoContratto a = (CampoAggiuntivoContratto) itA.next();
				mappa.put(a.getId(), a);
			}
			return mappa;
		}
		return null;
	}
	public List<CampoAggiuntivoContratto> getCampiAggiuntiviTabellaAgente() {
		Boolean m = impostazioneDao.getTabellaDatiAggiuntivi();
		if(ApplicationUtil.isTrue(m)) {
			return getAllCampiAggiuntiviAgente();
		}
		return null;
	}
	public List<CampoAggiuntivoContratto> getCampiAggiuntiviTabellaOperatore() {
		Boolean m = impostazioneDao.getTabellaDatiAggiuntivi();
		if(ApplicationUtil.isTrue(m)) {
			return getAllCampiAggiuntiviOperatore();
		}
		return null;
	}
	public List<CampoAggiuntivoContratto> getCampiAggiuntiviRicercaAgente() {
		Boolean s = impostazioneDao.getRicercaDomandaContrattoSingola();
		Boolean m = impostazioneDao.getRicercaDomandaContrattoMultipla();
		if(ApplicationUtil.isTrue(s) || ApplicationUtil.isTrue(m)) {
			return getAllCampiAggiuntiviAgente();
		}
		return null;
	}
	public List<CampoAggiuntivoContratto> getCampiAggiuntiviRicercaOperatore() {
		Boolean s = impostazioneDao.getRicercaDomandaContrattoSingola();
		Boolean m = impostazioneDao.getRicercaDomandaContrattoMultipla();
		if(ApplicationUtil.isTrue(s) || ApplicationUtil.isTrue(m)) {
			return getAllCampiAggiuntiviOperatore();
		}
		return null;
	}
	public DatoAggiuntivoContrattoRicerca createDatoAggiuntivo(Long campoId) {
		if(ApplicationUtil.gtZero(campoId)) {
			CampoAggiuntivoContratto ca = getCampoAggiuntivoById(campoId);
			DatoAggiuntivoContrattoRicerca daa = new DatoAggiuntivoContrattoRicerca();
			daa.setId(System.currentTimeMillis());
			daa.setCampo_aggiuntivo(ca);
			return daa;
		} else return null;
	}
	public void salvaPredefinito(EsitoContratto esitoPredefinito) {
		List<EsitoContratto> esiti = getAllEsitiContrattoPredefinito();
		Iterator<EsitoContratto> objIter = esiti.iterator();
		while(objIter.hasNext()) {
			EsitoContratto esito = objIter.next();
			esito.setPredefinito(Boolean.FALSE);
			salvaEsito(esito);
		}
		esitoPredefinito.setPredefinito(Boolean.TRUE);
		salvaEsito(esitoPredefinito);
	}
	public void removePredefinito(EsitoContratto esitoPredefinito) {
		esitoPredefinito.setPredefinito(Boolean.FALSE);
		salvaEsito(esitoPredefinito);
	}
	public EsitoContratto salvaEsito(EsitoContratto esito) {
		return (EsitoContratto) salvaAggiorna(esito);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoContratto> getAllEsitiContrattoPredefinito() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("predefinito", Boolean.TRUE));
		return getList(criteria);
	}
	public Long getEsitoContrattoPredefinito() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("predefinito", Boolean.TRUE));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return (Long) getUniqueResult(criteria);
	}
	public void revertStoriaContratto(Contratto  contratto, StoriaContratto storia) {
		contratto.setData_lavorazione(storia.getData());
		contratto.setUtente(storia.getUtente());
		contratto.setBloccato(storia.getBloccato());
		contratto.setOperatore(storia.getOperatore());
		contratto.setEsito(storia.getEsito());
		contratto.setData_stipula(storia.getData_stipula());
		contratto.setData_contabile(storia.getData_contabile());
		contratto.setData_attivazione(storia.getData_attivazione());
		contratto.setData_caricamento(storia.getData_caricamento());
		contratto.setData_esito(storia.getData_esito());
		contratto.setData_stato(storia.getData_stato());
		contratto.setSeriale(storia.getSeriale());
		contratto.setQuantita(storia.getQuantita());
		contratto.setCosto_scontato(storia.getCosto_scontato());
		contratto.setStato(storia.getStato());
		contratto.setNota(storia.getNota());
		contratto.setProdotto(storia.getProdotto());
		contratto.setAgente(storia.getAgente());
	}
	public void revertContratto(List<Contratto> contratti, Utente utente, Date data, Integer step, Boolean modo) {
		if(ApplicationUtil.notEmpty(contratti)) {
			List<Contratto> contrattiList = new ArrayList<Contratto>();
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Contratto> objIter = contratti.iterator();
			while(objIter.hasNext()) {
				Contratto old = objIter.next();
				Contratto contratto = (Contratto) session.get(Contratto.class, old.getId());
				Hibernate.initialize(contratto.getStoria());
				List<StoriaContratto> storia = contratto.getStoria(); 
				if(ApplicationUtil.notEmpty(storia)) {
					List<StoriaContratto> removeList = new ArrayList<StoriaContratto>();
					Iterator<StoriaContratto> objSt = storia.iterator();
					int j = 0;
					while(objSt.hasNext()) {
						StoriaContratto st = objSt.next();
						if(ApplicationUtil.isTrue(modo)) {
							if(j+step > storia.size()-1) {
								removeList.add(st);
							}
							j++;
						} else {
							if(st.getData().after(data)) {
								removeList.add(st);
							}
						}
					}
					if(ApplicationUtil.notEmpty(removeList)){
						boolean b = storia.size() == removeList.size();
						if(b) {
							removeList.remove(0);
						} 
						storia.removeAll(removeList);
					}
					if(ApplicationUtil.notEmpty(storia)) {
						if(ApplicationUtil.notEmpty(removeList)){
							Iterator<StoriaContratto> objRm = removeList.iterator();
							while(objRm.hasNext()) {
								session.delete(objRm.next());
							}
						}
						StoriaContratto el1 = storia.remove(storia.size()-1);
						revertStoriaContratto(contratto, el1);
						session.update(contratto);
						contrattiList.add(contratto);
						if (i++ % ConstantsGlobal.flushSize == 0) {
							session.flush();
							session.clear();
						}
					}
				}
			}
			tx.commit();
			closeSession(session);

			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contrattiList), contrattiList);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p, utente, ConstantsMap.OPERAZIONE_REVERT_CONTRATTO);
		}
	}
	public void revertContratto(Contratto contatto, Utente utente, Date data, Integer step, Boolean modo) {
		if(contatto != null) {
			List<Contratto> contratti = new ArrayList<Contratto>();
			contratti.add(contatto);
			revertContratto(contratti, utente, data, step, modo);
		}
	}
	public List<DatoSondaggioContratto> getSondaggioForm(SondaggioContratto sondaggio) {
		List<DomandaContratto> domande = getDomandeSondaggio(sondaggio.getId());
		List<DatoSondaggioContratto> dati = new ArrayList<DatoSondaggioContratto>();
		Iterator<DomandaContratto> objIter = domande.iterator();
		while(objIter.hasNext()) {
			DomandaContratto domanda = objIter.next();
			addDomanda(dati, new DatoSondaggioContratto(domanda.getCampo_aggiuntivo(), domanda.getObbligatoria(), domanda.getVisibile()));
		}
		return dati;
	}
	public Contratto getContratto(Utente utente, MyFile rec) {
		Contratto contratto = (Contratto) get(Contratto.class, rec.getId());
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTRATTO.stampa(), contratto);
		operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_CONTRATTO);
		return contratto;
	}
	public Contratto getContrattoId(Utente utente, Long id) {
		Contratto contratto = (Contratto) get(Contratto.class, id);
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTRATTO.stampa(), contratto);
		operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_CONTRATTO);
		return contratto;
	}
	public void bloccaContratto(Utente utente, Contratto contratto, boolean blocca) {
		if (contratto != null ) {
			contratto.setBloccato(blocca);
			salvaContratto(contratto, utente, false);
		} 
	}
	public void bloccaContrattiSelezionati(Utente utente, List<Contratto> contratti, boolean blocca) {
		if (ApplicationUtil.notEmpty(contratti)) {
			Iterator<Contratto> it = contratti.iterator();
			while (it.hasNext()) {
				Contratto contratto = (Contratto) it.next();
				bloccaContratto(utente, contratto, blocca);
			}
		} 
	}
}