package it.callite.dao;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.lang.reflect.Method;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import javax.faces.model.SelectItem;
import javax.faces.model.SelectItemGroup;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.net.ftp.FTPClient;
import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.json.JSONArray;
import org.json.JSONObject;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.stereotype.Repository;

import com.warrenstrange.googleauth.GoogleAuthenticatorConfig;
import com.warrenstrange.googleauth.GoogleAuthenticatorConfig.GoogleAuthenticatorConfigBuilder;
import com.warrenstrange.googleauth.KeyRepresentation;

import it.callite.action.DownloadBean;
import it.callite.chiamate.Chiamate;
import it.callite.column.generic.GenericColumn;
import it.callite.comparator.FileComparator;
import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsPannello;
import it.callite.criteria.AvvioCriteria;
import it.callite.criteria.ClientAPICriteria;
import it.callite.criteria.ClientFTPCriteria;
import it.callite.criteria.ColonnaCriteria;
import it.callite.criteria.CriterioCriteria;
import it.callite.criteria.FileCriteria;
import it.callite.criteria.RulesCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.message.Message;
import it.callite.model.Accesso;
import it.callite.model.Agente;
import it.callite.model.Anagrafica;
import it.callite.model.Appuntamento;
import it.callite.model.AutoContatto;
import it.callite.model.Autodialer;
import it.callite.model.Avvio;
import it.callite.model.Campagna;
import it.callite.model.CampoAggiuntivoAnagrafica;
import it.callite.model.CampoAggiuntivoContratto;
import it.callite.model.ClientAPI;
import it.callite.model.ClientFTP;
import it.callite.model.Colonna;
import it.callite.model.Command;
import it.callite.model.Comune;
import it.callite.model.Comunicazione;
import it.callite.model.Contatto;
import it.callite.model.Contratto;
import it.callite.model.Criterio;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoContratto;
import it.callite.model.DomandaAnagrafica;
import it.callite.model.DomandaContratto;
import it.callite.model.Email;
import it.callite.model.EsitoAppuntamento;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoChiamata;
import it.callite.model.EsitoContratto;
import it.callite.model.Indirizzo;
import it.callite.model.Invoice;
import it.callite.model.Lista;
import it.callite.model.Marcatura;
import it.callite.model.Nota;
import it.callite.model.Notifica;
import it.callite.model.Notizia;
import it.callite.model.Operazione;
import it.callite.model.Pausa;
import it.callite.model.Prenotazione;
import it.callite.model.Prodotto;
import it.callite.model.Provincia;
import it.callite.model.Qualifica;
import it.callite.model.Ricezione;
import it.callite.model.Rmail;
import it.callite.model.Rules;
import it.callite.model.Ruolo;
import it.callite.model.Sondaggio;
import it.callite.model.StoriaAppuntamento;
import it.callite.model.StoriaContatto;
import it.callite.model.StoriaContratto;
import it.callite.model.StoriaNota;
import it.callite.model.StoriaPrenotazione;
import it.callite.model.Telecom;
import it.callite.model.Tema;
import it.callite.model.Ufficio;
import it.callite.model.Utente;
import it.callite.model.VocalOrder;
import it.callite.model.Webhook;
import it.callite.model.Zona;
import it.callite.model.allegato.AllegatoAgente;
import it.callite.model.allegato.AllegatoAnagrafica;
import it.callite.model.allegato.AllegatoAppuntamento;
import it.callite.model.allegato.AllegatoContatto;
import it.callite.model.allegato.AllegatoContratto;
import it.callite.model.allegato.AllegatoNotizia;
import it.callite.model.messaggio.Messaggio;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.model.report.ConteggioModel;
import it.callite.model.sip.Cdr;
import it.callite.model.sip.Cel;
import it.callite.model.sip.Coda;
import it.callite.model.sip.SipBuddies;
import it.callite.service.config.ConfigurazionePannelloService;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Campo;
import it.callite.utility.Coordinata;
import it.callite.utility.FileUpload;
import it.callite.utility.Firewall;
import it.callite.utility.GCoder;
import it.callite.utility.InvocaAPI;
import it.callite.utility.Label;
import it.callite.utility.LabelQuery;
import it.callite.utility.LoggerLogin;
import it.callite.utility.Modulo;
import it.callite.utility.MyFile;
import it.callite.utility.Parameter;
import it.callite.utility.QueryElement;
import it.callite.utility.QueryList;
import it.callite.utility.QueryUtils;
import it.callite.utility.StatoIntervallo;
import it.callite.utility.TemaUtil;
import it.callite.utility.UtenteUtil;
import it.callite.utility.UtilChat;
import it.callite.utility.UtilFTP;
import it.callite.utility.item.GenericSI;
import it.callite.utility.item.UtenteSI;
import it.callite.utility.massivo.AgenteMassivo;
import it.callite.utility.massivo.AnagraficaMassivo;
import it.callite.utility.massivo.CampagnaMassivo;
import it.callite.utility.massivo.CampoAggiuntivoAnagraficaMassivo;
import it.callite.utility.massivo.CampoAggiuntivoContrattoMassivo;
import it.callite.utility.massivo.CodaMassivo;
import it.callite.utility.massivo.EsitoAppuntamentoMassivo;
import it.callite.utility.massivo.EsitoArchiviazioneMassivo;
import it.callite.utility.massivo.EsitoChiamataMassivo;
import it.callite.utility.massivo.EsitoContrattoMassivo;
import it.callite.utility.massivo.InternoMassivo;
import it.callite.utility.massivo.ListaMassivo;
import it.callite.utility.massivo.TrunkMassivo;
import it.callite.utility.massivo.UtenteMassivo;
import it.callite.utility.sincronizzazione.InfoSincroniaAppuntamentoThread;
import it.callite.utility.sincronizzazione.InfoSincroniaContattoThread;
import it.callite.utility.sincronizzazione.InfoSincroniaContrattoThread;
import it.callite.utility.sincronizzazione.InfoSincroniaNotaThread;
import it.callite.utility.sincronizzazione.InfoSincroniaPrenotazioneThread;
import it.callite.utility.thread.RicaricaCalendarioThread;
import it.callite.utility.utils.ModuloFileFilter;
@SuppressWarnings("serial")
@Repository("dataDao")
public class DataDao extends GenericDao  implements InitializingBean, DisposableBean {
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private TemaDao temaDao;
	@Autowired
	private AutodialerDao autodialerDao;
	@Autowired
	private UserDao userDao;
	@Autowired
	private SipDao sipDao;
	@Autowired
	private ContattoDao contattoDao;
	@Autowired
	private DownloadDao downloadDao;
	
	@Autowired
	private ConfigurazionePannelloService configurazionePannelloService;
	
	@Autowired
	private ServletContext servletContext;
	
	private GoogleAuthenticatorConfig googleAuthenticatorConfig;
	
	public Integer getRigheSize() {
		return ConstantsGlobal.righeSize * ConstantsGlobal.righeM;
	}
	public Integer getRigheMaxSize() {
		return ConstantsGlobal.righeMaxSize * ConstantsGlobal.righeM;
	}
	public Integer getRigheMinSize() {
		return ConstantsGlobal.righeMinSize * ConstantsGlobal.righeM;
	}
	public Integer getRigheStep() {
		return ConstantsGlobal.righeStep * ConstantsGlobal.righeM;
	}
	public Integer getDefaultPageSize() {
		return ConstantsGlobal.righePagina;
	}
	public Integer getSizeOTP() {
		return ConstantsGlobal.cifre_otp;
	}
	public String getManuale() {
		return ConstantsGlobal.manuale;
	}

	public String getPipeSeparator() {
		return ConstantsGlobal.pipeSeparator;
	}
	public String getColonSeparator() {
		return ConstantsGlobal.colonSeparator;
	}
	public long getAltro() {
		return ConstantsGlobal.ALTRO;
	}

	public Set<String> getKeyImportAnagrafica() {
		return ConstantsMap.anagraficaIOMap;
	}
	public Set<String> getKeyImportOpposizione() {
		return ConstantsMap.opposizioneIMap;
	}
	public List<String> getKeyImportAppuntamento() {
		List<String> a = new ArrayList<String>(ConstantsMap.appuntamentoIMap);
		a.remove(ConstantsMap.chiave);
		return a;
	}
	public List<String> getKeyImportContratto() {
		List<String> a = new ArrayList<String>(ConstantsMap.contrattoIMap);
		a.remove(ConstantsMap.chiave);
		return a;
	}
	public List<String> getKeyImportContatto() {
		List<String> a = new ArrayList<String>(ConstantsMap.contattoIMap);
		a.remove(ConstantsMap.chiave);
		return a;
	}
	public TemaUtil getTema() {
		TemaUtil tu = new TemaUtil();
		Tema t = temaDao.getTema();
		if(t != null) {
			Date scadenza = ApplicationUtil.aggiustaScadenza(t.getScadenza());
			Integer preavviso = t.getPreavviso();
			tu.setScadenza(scadenza);
			tu.setPreavviso(preavviso);
			if(scadenza != null) {
				Date oggi = ApplicationUtil.date();
				tu.setScaduta(oggi.after(scadenza));
				if(ApplicationUtil.colcolaScadenza(oggi, scadenza, preavviso)) {
					tu.setVersione(Message.addCatalogText("scadenza") + ": " + ApplicationUtil.formatDate(ConstantsFormat.dateTime.stampa(), scadenza));
				}
			} 
		}
		return tu;
	}

	public String getTest() {
		return ConstantsGlobal.test;
	}
	public String getSite() {
		return ConstantsGlobal.site;
	}
	public String getSiteBase(){
		return ApplicationUtil.getUrlBase(getSite());
	}
	public List<SelectItem> getQuoteList() {
		return ConstantsMap.getQuoteList();
	}
	public List<SelectItem> getSeparatoreList() {
		return ConstantsMap.getSeparatoreList();
	}
	public int getZetaIndex() {
		return ConstantsGlobal.zetaIndex + 1;
	}
	public int getMaxValue() {
		return ConstantsGlobal.maxValue;
	}
	public Integer getCalendarSize() {
		return ConstantsGlobal.calendarSize;
	}
	
	public Integer getCalendarWidth() {
		return ConstantsGlobal.calendarWidth;
	}
	public Integer getCalendarShift() {
		return ConstantsGlobal.calendarLeft;
	}
	
	public int getAppuntamentoFine() {
		return ConstantsGlobal.appuntamentoFine;
	}
	public int getAppuntamentoInizio() {
		return ConstantsGlobal.appuntamentoInizio;
	}
	public Integer getAppuntamentoGiorni() {
		return Integer.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_LIMITA_GIORNI_CALENDARIO));
	}
	
	public Integer getPannelloLeft() {
		Boolean a = Boolean.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_CALENDARIO));
		return ApplicationUtil.isTrue(a) ? ConstantsGlobal.calendarLeft : 0;
	}
	
	public Integer getFiltriCalendario() {
		String s = configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_LIMITA_FILTRI_CALENDARIO);
		if(s != null) {
			if(s.equals(ConstantsMap.calendario0)) return ConstantsMap.value0;
			if(s.equals(ConstantsMap.calendario1)) return ConstantsMap.value1;
			if(s.equals(ConstantsMap.calendario2)) return ConstantsMap.value2;
			if(s.equals(ConstantsMap.calendario3)) return ConstantsMap.value3;
		}
		return ConstantsMap.value0;
	}
	
	public Integer getMaxAggressivita1() {
		return ConstantsGlobal.maxAggressivita1;
	}
	public Integer getMaxAggressivita2() {
		return ConstantsGlobal.maxAggressivita2;
	}
	public String getAudio() {
		return ConstantsGlobal.path_web_audio;
	}
	public String getMaxImportSize() {
		long maxSize = impostazioneDao.maxImport();
		if(maxSize > 0 ) {
			return " " + "(" + Message.addCatalogText("massimo") + ": " + ApplicationUtil.byteCount(maxSize) + ")";
		}
		return "";
	}
	
	public String getMaxAllegatoSize() {
		int maxSize = getMaxFileLimit();
		if(maxSize > 0 ) {
			return " " + "(" + Message.addCatalogText("massimo") + ": " + maxSize + " " + Message.addCatalogText("file")+")";
		}
		return "";
	}
	
	public String getMaxQuantitySize() {
		int maxSize = impostazioneDao.getMaxQuantity();
		if(maxSize > 0 ) {
			return " " + "(" + Message.addCatalogText("massimo") + ": " + maxSize + " " + Message.addCatalogText("file")+")";
		}
		return "";
	}
	public int getMaxExportSize() {
		Integer a = Integer.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_MASSIMO_VALORE_EXPORT));
		return (a != null) ? a.intValue() : 0;
	}
	public int getMaxWorkSize() {
		Integer a = Integer.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_MASSIMO_VALORE_LAVORO));
		return (a != null) ? a.intValue() : 0;
	}
	public String getImportXml() {
		boolean consenti = ConstantsGlobal.consenti_tutto;
		if(consenti || userDao.getFlash()) {
			return "*";
		} else {
			return getAcceptedXml();
		}
	}
	public String getImportExcel() {
		boolean consenti = ConstantsGlobal.consenti_tutto;
		if(consenti || userDao.getFlash()) {
			return "*";
		} else {
			return getAcceptedExcel();
		}
	}
	public String getImportTemplate() {
		boolean consenti = ConstantsGlobal.consenti_tutto;
		if(consenti || userDao.getFlash()) {
			return "*";
		} else {
			return getAcceptedTemplate();
		}
	}
	public String getImportConvert() {
		boolean consenti = ConstantsGlobal.consenti_tutto;
		if(consenti || userDao.getFlash()) {
			return "*";
		} else {
			return getAcceptedConvert();
		}
	}
	
	public String getAcceptedCommand() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_command);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	
	public String getAcceptedAiuto() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_aiuto);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedBackup() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_backup);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedSql() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_sql);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedImmagini() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_immagini);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedQuery() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_query);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedExcel() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_excel);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedFattura() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_fattura);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedTemplate() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_template);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedConvert() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_convert);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedAudio() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_audio1);
		if(ApplicationUtil.getPath(ConstantsGlobal.path_ffmpeg, true)) {
			list.addAll(ApplicationUtil.asList(ConstantsMap.a_audio2));
		} 
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedCompress() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_compress);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedReport() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_report);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedXml() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_xml);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedBusta() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_busta);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	
	public String getAcceptedMessaggio() {
		List<String> list = new ArrayList<String>();
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedFile() {
		List<String> list = new ArrayList<String>();
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedPda() {
		List<String> list = new ArrayList<String>();
		list.addAll(ApplicationUtil.asList(ConstantsMap.a_pda1));
		list.addAll(ApplicationUtil.asList(ConstantsMap.a_pda2));
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedXhtml() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_xhtml);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedLingua() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_properties);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	public String getAcceptedHtml() {
		List<String> list = ApplicationUtil.asList(ConstantsMap.a_html);
		return ApplicationUtil.detoken(list, ConstantsGlobal.commaSeparator + " ");
	}
	
	public String getAcceptedAllegato() {
		String a = configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_ALLEGATI_VALIDI);
		List<String> t = ApplicationUtil.token(a, ConstantsGlobal.colonSeparator);
		return ApplicationUtil.detoken(t, ConstantsGlobal.commaSeparator + " ");
	}
	
	public Boolean getVisualizzaImmagine() {
		return ConstantsGlobal.defaultImage;
	}

	public long getMaxFileSize() {
		return ConstantsGlobal.SIZE;
	}
	
	public Integer getMaxFileLimit() {
		String i = ApplicationUtil.trimToNull(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_MAX_ALLEGATO));
		return ApplicationUtil.asInteger(i);
	}
	
	public String getRevisione() {
		return ConstantsGlobal.REVISIONE;
	}
	public Date getTimeAlive() {
		return ConstantsGlobal.TIMEALIVE;
	}
	public String getAnno() {
		return ApplicationUtil.formatDate(ConstantsFormat.year.stampa(), ApplicationUtil.date());
	}
	public String getCurrency() {
		return ConstantsFormat.currency.stampa();
	}
	public String getPercentuale() {
		return ConstantsFormat.percentuale.stampa();
	}
	public String getNumber() {
		return ConstantsFormat.number.stampa();
	}
	public String getTime() {
		return ConstantsFormat.time.stampa();
	}
	public String getDateTime() {
		return ConstantsFormat.dateTime.stampa();
	}
	public String getDate() {
		return ConstantsFormat.date.stampa();
	}
	public String getDateSeconds() {
		return ConstantsFormat.dateSeconds.stampa();
	}
	public String getDayDate() {
		return ConstantsFormat.dayDate.stampa();
	}
	public String getDayDateSeconds() {
		return ConstantsFormat.dayDateSeconds.stampa();
	}
	public String getTimeZone() {
		return ConstantsGlobal.TIMEZONE;
	}
	public String getServer() {
		return ApplicationUtil.ip();
	}
	public String getHostname() {
		return ApplicationUtil.hostname();
	}
	public List<Label> getRegioni() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Provincia.class);
		List<Label> lis = getLabelSI(criteria, "regione", "regione");
		return impostazioneDao.getLabel(lis, true);
	}
	public List<Label> getProvince(String regione) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Provincia.class);
		if(ApplicationUtil.isNotBlank(regione)) {
			criteria.add(Restrictions.eq("regione", regione));
		}
		List<Label> lis = getLabelSI(criteria, "codice", "nome");
		return impostazioneDao.getLabel(lis, true);
	}
	public List<Label> getComuni(String codiceProvincia) {
		if(ApplicationUtil.isNotBlank(codiceProvincia)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Comune.class);
			criteria.add(Restrictions.eq("codice_provincia", codiceProvincia));
			List<Label> lis = getLabelSI(criteria, "nome", "nome");
			return impostazioneDao.getLabel(lis, true);
		} else {
			List<Label> lis = new ArrayList<Label>();
			return impostazioneDao.getLabel(lis, true);
		}
	}
	public String getRegione(String codiceProvincia) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Provincia.class);
		criteria.add(Restrictions.eq("codice", codiceProvincia));
		criteria.setProjection(Projections.distinct(Projections.property("regione")));
		return (String) getUniqueResult(criteria);
	}
	public String getLastInterno(String format) {
		Integer num = 0;
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		criteria.setProjection(Projections.distinct(Projections.property("name")));
		criteria.addOrder(Order.desc("name"));
		String result =  (String) getFirstResult(criteria);
		if(ApplicationUtil.isNotBlank(result)) {
			if(ApplicationUtil.isNumeric(result)) {
				num = Integer.parseInt(result);
			}
		}
		num++;
		NumberFormat formatter = ApplicationUtil.getDecimalFormat(format);
		return formatter.format(num);
	}
	public Short getLastPosizioneContratto(Long sondaggioId) {
		Short num = 0;
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaContratto.class);
		criteria.add(Restrictions.eq("sondaggio.id", sondaggioId));
		criteria.setProjection(Projections.distinct(Projections.property("posizione")));
		criteria.addOrder(Order.desc("posizione"));
		Short result =  (Short) getFirstResult(criteria);
		if(ApplicationUtil.gtZero(result)) {
			num = result;
		}
		num++;
		return num;
	}
	public Short getLastPosizioneAnagrafica(Long sondaggioId) {
		Short num = 0;
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaAnagrafica.class);
		criteria.add(Restrictions.eq("sondaggio.id", sondaggioId));
		criteria.setProjection(Projections.distinct(Projections.property("posizione")));
		criteria.addOrder(Order.desc("posizione"));
		Short result =  (Short) getFirstResult(criteria);
		if(ApplicationUtil.gtZero(result)) {
			num = result;
		}
		num++;
		return num;
	}
	@SuppressWarnings("rawtypes")
	public String getLast(Class classe, String preambolo, String proprieta, String format) {
		Integer num = 0;
		DetachedCriteria criteria = DetachedCriteria.forClass(classe);
		criteria.add(Restrictions.sqlRestriction(proprieta + " REGEXP '^"+preambolo+"[0-9]{4}$'"));
		criteria.setProjection(Projections.distinct(Projections.property(proprieta)));
		criteria.addOrder(Order.desc(proprieta));
		String result =  (String) getFirstResult(criteria);
		if(ApplicationUtil.isNotBlank(result)) {
			String numero = ApplicationUtil.substringAfter(result, preambolo);
			if(ApplicationUtil.isNotBlank(numero)) {
				if(ApplicationUtil.isNumeric(numero)) {
					num = Integer.parseInt(numero);
				}
			}
		} 
		num++;
		NumberFormat formatter = ApplicationUtil.getDecimalFormat(format);
		return preambolo+formatter.format(num);
	}
	public String getLastOperatore() {
		return getLast(Utente.class, impostazioneDao.getPreamboloOperatore(), "username", ConstantsGlobal.format);
	}
	public String getLastUtente() {
		return getLast(Utente.class, impostazioneDao.getPreamboloUtente(), "username",  ConstantsGlobal.format);
	}
	public String getLastAgente() {
		return getLast(Agente.class, impostazioneDao.getPreamboloAgente(), "codice",  ConstantsGlobal.format);
	}
	public String getLastInterno() {
		return getLastInterno(ConstantsGlobal.format);
	}
	public String getLastPassword() {
		return ApplicationUtil.randomString(ConstantsGlobal.lunghezza_password);
	}
	public List<Label>  cambiaRegione(Indirizzo ind) {
		ind.setComune("");
		ind.setProvincia("");
		return getComuni(null);
	}
	public List<Label> cambiaRegioneProvincia(Indirizzo ind) {
		return getProvince(ind.getRegione());
	}
	public List<Label>  cambiaProvincia(Indirizzo ind) {
		ind.setComune("");
		if(ApplicationUtil.isNotBlank(ind.getProvincia())) {
			ind.setRegione(getRegione(ind.getProvincia()));
		} 
		return getComuni(ind.getProvincia());
	}
	public List<Label>  cambiaRegione(Zona zona) {
		zona.setComune("");
		zona.setProvincia("");
		return getComuni(null);
	}
	public List<Label> cambiaRegioneProvincia(Zona zona) {
		return getProvince(zona.getRegione());
	}
	public List<Label>  cambiaProvincia(Zona zona) {
		zona.setComune("");
		if(ApplicationUtil.isNotBlank(zona.getProvincia())) {
			zona.setRegione(getRegione(zona.getProvincia()));
		} 
		return getComuni(zona.getProvincia());
	}
	public List<Label>  cambiaRegione(Ufficio ufficio) {
		ufficio.setComune("");
		ufficio.setProvincia("");
		return getComuni(null);
	}
	public List<Label> cambiaRegioneProvincia(Ufficio ufficio) {
		return getProvince(ufficio.getRegione());
	}
	public List<Label>  cambiaProvincia(Ufficio ufficio) {
		ufficio.setComune("");
		if(ApplicationUtil.isNotBlank(ufficio.getProvincia())) {
			ufficio.setRegione(getRegione(ufficio.getProvincia()));
		} 
		return getComuni(ufficio.getProvincia());
	}
	public List<SelectItem> getProvinceSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Provincia.class);
		List<GenericSI> lis = getGenericSI(criteria, "codice", "codice");
		return impostazioneDao.getGenericNoID(lis, false);
	}
	public Date getDataCorrente() {
		return ApplicationUtil.date();
	}
	public Long getTimeCorrente() {
		return ApplicationUtil.date().getTime();
	}
	public List<String> getFormats() {
		return ConstantsMap.audioSupportatoL();
	}	
	public	String  getCatastoByComuneNome(String comune){
		DetachedCriteria criteria = DetachedCriteria.forClass(Comune.class);
		criteria.add(Restrictions.eq("nome", comune));
		criteria.setProjection(Projections.distinct(Projections.property("codice_catastale")));
		return (String) getUniqueResult(criteria);
	}
	public Date impostaData(Integer operazione) {
		if(ApplicationUtil.eq(operazione, 1) || ApplicationUtil.eq(operazione, 3)) {
			Integer giorni = impostazioneDao.getGiorniContatti();
			return giorni != null ? ApplicationUtil.dataProgrammata(giorni) : null;
		} else {
			return getAdesso();
		}
	}
	public Date getDataProgrammata() {
		Integer giorni = impostazioneDao.getGiorniContatti();
		return giorni != null ? ApplicationUtil.dataProgrammata(giorni) : null;
	}
	public Date getFineGiornata() {
		Date date = ApplicationUtil.date();
		return ApplicationUtil.fineGiornata(date);
	}
	public Date getAdesso() {
		Date date = ApplicationUtil.date();
		GregorianCalendar gc = ApplicationUtil.getGregorianCalendar();
		if(date != null) gc.setTime(date);
		gc.set(GregorianCalendar.SECOND, 0);
		gc.set(GregorianCalendar.MILLISECOND, 0);
		return gc.getTime();
	}
	public List<SelectItem> getStatiSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.stato_attivo, Message.addCatalogText("ts_stato." + ConstantsGlobal.stato_attivo)));
		sis.add(new SelectItem(ConstantsGlobal.stato_pausa, Message.addCatalogText("ts_stato." + ConstantsGlobal.stato_pausa)));
		sis.add(new SelectItem(ConstantsGlobal.stato_chiuso, Message.addCatalogText("ts_stato." + ConstantsGlobal.stato_chiuso)));
		return sis;
	}
	public List<SelectItem> getStatiAutodialerSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.ts_attivo, Message.addCatalogText("ta_stato." + ConstantsGlobal.ts_attivo)));
		sis.add(new SelectItem(ConstantsGlobal.ts_auto, Message.addCatalogText("ta_stato." + ConstantsGlobal.ts_auto)));
		sis.add(new SelectItem(ConstantsGlobal.ts_chiuso, Message.addCatalogText("ta_stato." + ConstantsGlobal.ts_chiuso)));
		return sis;
	}
	public List<SelectItem> getOrdineListaSI() {
		String asc = Message.addCatalogText(ConstantsGlobal.asc);
		String desc = Message.addCatalogText(ConstantsGlobal.desc);
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		sisGroup.add(new SelectItem("", ConstantsLanguage.TRATTINI.stampa()));
		sisGroup.add(new SelectItem(ConstantsGlobal.randomize, ConstantsLanguage.RANDOM.stampa()));
		List<SelectItem> geocodifica = new ArrayList<SelectItem>();
		String t_geocodifica = Message.addCatalogText("geocodifica");
		geocodifica.add(new SelectItem(ConstantsMap.indirizzo_legale + "_" + ConstantsMap.geocodifica + ConstantsGlobal.colonSeparator + ConstantsGlobal.asc, Message.addCatalogText(ConstantsMap.indirizzo_legale) + " (" + asc + ")"));
		geocodifica.add(new SelectItem(ConstantsMap.indirizzo_legale + "_" + ConstantsMap.geocodifica + ConstantsGlobal.colonSeparator + ConstantsGlobal.desc, Message.addCatalogText(ConstantsMap.indirizzo_legale) + " (" + desc + ")"));
		geocodifica.add(new SelectItem(ConstantsMap.indirizzo_operativo + "_" + ConstantsMap.geocodifica + ConstantsGlobal.colonSeparator + ConstantsGlobal.asc, Message.addCatalogText(ConstantsMap.indirizzo_operativo) + " (" + asc + ")"));
		geocodifica.add(new SelectItem(ConstantsMap.indirizzo_operativo + "_" + ConstantsMap.geocodifica + ConstantsGlobal.colonSeparator + ConstantsGlobal.desc, Message.addCatalogText(ConstantsMap.indirizzo_operativo) + " (" + desc + ")"));
		SelectItemGroup geocodificaG = new SelectItemGroup(t_geocodifica); 
		geocodificaG.setSelectItems(geocodifica.toArray(new SelectItem[geocodifica.size()]));
		sisGroup.add(geocodificaG);
		List<SelectItem> normale = new ArrayList<SelectItem>();
		String t_normale = Message.addCatalogText("normale");
		for(String c : ConstantsMap.ordineCampoMap) {
			String msg = Message.addCatalogText(c);
			normale.add(new SelectItem(c + ConstantsGlobal.colonSeparator + ConstantsGlobal.asc,  msg + " (" + asc + ")"));
			normale.add(new SelectItem(c + ConstantsGlobal.colonSeparator + ConstantsGlobal.desc,  msg + " (" + desc + ")"));
		}
		SelectItemGroup normaleG = new SelectItemGroup(t_normale); 
		normaleG.setSelectItems(normale.toArray(new SelectItem[normale.size()]));
		sisGroup.add(normaleG);
		return sisGroup;
	}
	public List<SelectItem> getOrdineCampagnaSI() {
		String asc = Message.addCatalogText(ConstantsGlobal.asc);
		String desc = Message.addCatalogText(ConstantsGlobal.desc);
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		sisGroup.add(new SelectItem("", ConstantsLanguage.TRATTINI.stampa()));
		List<SelectItem> geocodifica = new ArrayList<SelectItem>();
		String t_geocodifica = Message.addCatalogText("geocodifica");
		geocodifica.add(new SelectItem(ConstantsMap.indirizzo_legale + "_" + ConstantsMap.geocodifica + ConstantsGlobal.colonSeparator + ConstantsGlobal.asc, Message.addCatalogText(ConstantsMap.indirizzo_legale) + " (" + asc + ")"));
		geocodifica.add(new SelectItem(ConstantsMap.indirizzo_legale + "_" + ConstantsMap.geocodifica + ConstantsGlobal.colonSeparator + ConstantsGlobal.desc, Message.addCatalogText(ConstantsMap.indirizzo_legale) + " (" + desc + ")"));
		geocodifica.add(new SelectItem(ConstantsMap.indirizzo_operativo + "_" + ConstantsMap.geocodifica + ConstantsGlobal.colonSeparator + ConstantsGlobal.asc, Message.addCatalogText(ConstantsMap.indirizzo_operativo) + " (" + asc + ")"));
		geocodifica.add(new SelectItem(ConstantsMap.indirizzo_operativo + "_" + ConstantsMap.geocodifica + ConstantsGlobal.colonSeparator + ConstantsGlobal.desc, Message.addCatalogText(ConstantsMap.indirizzo_operativo) + " (" + desc + ")"));
		SelectItemGroup geocodificaG = new SelectItemGroup(t_geocodifica); 
		geocodificaG.setSelectItems(geocodifica.toArray(new SelectItem[geocodifica.size()]));
		sisGroup.add(geocodificaG);
		List<SelectItem> normale = new ArrayList<SelectItem>();
		String t_normale = Message.addCatalogText("normale");
		for(String c : ConstantsMap.ordineCampoMap) {
			String msg = Message.addCatalogText(c);
			normale.add(new SelectItem(c + ConstantsGlobal.colonSeparator + ConstantsGlobal.asc,  msg + " (" + asc + ")"));
			normale.add(new SelectItem(c + ConstantsGlobal.colonSeparator + ConstantsGlobal.desc,  msg + " (" + desc + ")"));
		}
		SelectItemGroup normaleG = new SelectItemGroup(t_normale); 
		normaleG.setSelectItems(normale.toArray(new SelectItem[normale.size()]));
		sisGroup.add(normaleG);
		return sisGroup;
	}
	public List<SelectItem> getProprietaCampagnaSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem("", ConstantsLanguage.TRATTINI.stampa()));
		for(String c : ConstantsMap.ordineCampoMap) {
			String msg = Message.addCatalogText(c);
			sis.add(new SelectItem(c,  msg));
		}
		return sis;
	}
	public List<SelectItem> getTipiRichiami() {
		List<SelectItem> tipo = new ArrayList<SelectItem>();
		tipo.add(new SelectItem(0, Message.addCatalogText("nuovo")));
		tipo.add(new SelectItem(1, Message.addCatalogText("pubblico")));
		tipo.add(new SelectItem(2, Message.addCatalogText("privato")));
		return tipo;
	}
	public List<SelectItem> getModalitaAutodialerSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(Integer c : ConstantsMap.getAutoModalita()) {
			sis.add(new SelectItem(c, Message.addCatalogText("autodialer_modalita" + c)));
		}
		return sis;
	}
	public List<SelectItem> getTipoAccontoSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(Integer c : ConstantsMap.getTipoAcconto()) {
			sis.add(new SelectItem(c, Message.addCatalogText("tipo_acconto" + c)));
		}
		return sis;
	}
	public List<SelectItem> getTipoDate() {
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		sisGroup.add(new SelectItem("reset", Message.addCatalogText("reset")));
		List<SelectItem> giorno = new ArrayList<SelectItem>();
		String t_giorno = Message.addCatalogText("giorno");
		giorno.add(new SelectItem(ConstantsMap.p_g + ConstantsGlobal.colonSeparator + "+0", t_giorno + ": " +  Message.addCatalogText("attuale")));
		giorno.add(new SelectItem(ConstantsMap.p_g + ConstantsGlobal.colonSeparator + "+1", t_giorno + ": " +  Message.addCatalogText("prossimo")));
		giorno.add(new SelectItem(ConstantsMap.p_g + ConstantsGlobal.colonSeparator + "-1", t_giorno + ": " +  Message.addCatalogText("precedente")));
		SelectItemGroup giornoG = new SelectItemGroup(t_giorno); 
		giornoG.setSelectItems(giorno.toArray(new SelectItem[giorno.size()]));
		sisGroup.add(giornoG);
		List<SelectItem> settimana = new ArrayList<SelectItem>();
		String t_settimana = Message.addCatalogText("settimana");
		settimana.add(new SelectItem(ConstantsMap.p_s + ConstantsGlobal.colonSeparator + "+0", t_settimana + ": " +  Message.addCatalogText("attuale")));
		settimana.add(new SelectItem(ConstantsMap.p_s + ConstantsGlobal.colonSeparator + "+1", t_settimana + ": " +  Message.addCatalogText("prossimo")));
		settimana.add(new SelectItem(ConstantsMap.p_s + ConstantsGlobal.colonSeparator + "-1", t_settimana + ": " +  Message.addCatalogText("precedente")));
		SelectItemGroup settimanaG = new SelectItemGroup(t_settimana); 
		settimanaG.setSelectItems(settimana.toArray(new SelectItem[settimana.size()]));
		sisGroup.add(settimanaG);
		List<SelectItem> mese = new ArrayList<SelectItem>();
		String t_mese = Message.addCatalogText("mese");
		mese.add(new SelectItem(ConstantsMap.p_m + ConstantsGlobal.colonSeparator + "+0", t_mese + ": " +  Message.addCatalogText("attuale")));
		mese.add(new SelectItem(ConstantsMap.p_m + ConstantsGlobal.colonSeparator + "+1", t_mese + ": " +  Message.addCatalogText("prossimo")));
		mese.add(new SelectItem(ConstantsMap.p_m + ConstantsGlobal.colonSeparator + "-1", t_mese + ": " +  Message.addCatalogText("precedente")));
		SelectItemGroup meseG = new SelectItemGroup(t_mese); 
		meseG.setSelectItems(mese.toArray(new SelectItem[mese.size()]));
		sisGroup.add(meseG);
		List<SelectItem> anno = new ArrayList<SelectItem>();
		String t_anno = Message.addCatalogText("anno");
		anno.add(new SelectItem(ConstantsMap.p_a + ConstantsGlobal.colonSeparator + "+0",  t_anno + ": " +  Message.addCatalogText("attuale")));
		anno.add(new SelectItem(ConstantsMap.p_a + ConstantsGlobal.colonSeparator + "+1",  t_anno + ": " +  Message.addCatalogText("prossimo")));
		anno.add(new SelectItem(ConstantsMap.p_a + ConstantsGlobal.colonSeparator + "-1",  t_anno + ": " +  Message.addCatalogText("precedente")));
		SelectItemGroup annoG = new SelectItemGroup(t_anno); 
		annoG.setSelectItems(anno.toArray(new SelectItem[anno.size()]));
		sisGroup.add(annoG);
		return sisGroup;
	}
	public List<SelectItem> getStatiRicercaSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.stato_attivo, Message.addCatalogText("ts_stato." + ConstantsGlobal.stato_attivo)));
		sis.add(new SelectItem(ConstantsGlobal.stato_pausa, Message.addCatalogText("ts_stato." + ConstantsGlobal.stato_pausa)));
		sis.add(new SelectItem(ConstantsGlobal.stato_chiuso, Message.addCatalogText("ts_stato." + ConstantsGlobal.stato_chiuso)));
		return sis;
	}
	public List<SelectItem> getAttiviRicercaSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.attivo_si, Message.addCatalogText("tm_stato." + ConstantsGlobal.attivo_si)));
		sis.add(new SelectItem(ConstantsGlobal.attivo_no, Message.addCatalogText("tm_stato." + ConstantsGlobal.attivo_no)));
		return sis;
	}
	public List<SelectItem> getTipiRicercaSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.tipo_residenziale, ConstantsLanguage.TIPO_RESIDENZIALE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.tipo_business, ConstantsLanguage.TIPO_BUSINESS.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.tipo_misto, ConstantsLanguage.TIPO_MISTO.stampa()));
		return sis;
	}
	public List<SelectItem> getModuliSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(File mod : listaFile()) {
			String modd = ApplicationUtil.getBaseNameN(mod.getName());
			sis.add(new SelectItem(modd + getExtensionModules(), modd));
		}
		return sis;
	}
	public List<SelectItem> getModuliCampagnaSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		for(File mod : listaFile()) {
			String modd = ApplicationUtil.getBaseNameN(mod.getName());
			sis.add(new SelectItem(modd + getExtensionModules(), modd));
		}
		return sis;
	}
	public List<String> getModuli() {
		List<String> sis = new ArrayList<String>();
		for(File mod : listaFile()) {
			if(mod.exists()) {
				sis.add(ApplicationUtil.getBaseNameN(mod.getName()) + getExtensionModules());
			}
		}
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<Modulo> getModuliAnagrafica() {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoAnagrafica.class);
		criteria.add(Restrictions.eq("tipo", ConstantsMap.TIPO_FORM));
		criteria.createCriteria("domande").add(Restrictions.isNotNull("sondaggio.id"));
		criteria.add(Restrictions.isNotNull("valori"));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("chiave").as("chiave"))
				.add(Projections.property("valori").as("valori")));
		List<Modulo> moduli = getList(criteria, Modulo.class);
		List<Modulo> sis = new ArrayList<Modulo>();
		for(Modulo modulo : moduli) {
			if(!sis.contains(modulo)) {
				File file = new File(getPathFile().getAbsolutePath() + "/" + modulo.getPath());
				if(file.exists()) {
					sis.add(modulo);
				}
			}
		}
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<Modulo> getModuliContratto() {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.eq("tipo", ConstantsMap.TIPO_FORM));
		criteria.createCriteria("domande").add(Restrictions.isNotNull("sondaggio.id"));
		criteria.add(Restrictions.isNotNull("valori"));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("chiave").as("chiave"))
				.add(Projections.property("valori").as("valori")));
		List<Modulo> moduli = getList(criteria, Modulo.class);
		List<Modulo> sis = new ArrayList<Modulo>();
		for(Modulo modulo : moduli) {
			if(!sis.contains(modulo)) {
				File file = new File(getPathFile().getAbsolutePath() + "/" + modulo.getPath());
				if(file.exists()) {
					sis.add(modulo);
				}
			}
		}
		return sis;
	}
	public List<MyFile> listaModuli(FileCriteria mm) { 
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> sis = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			List<File> ls = listaFile();
			if(ApplicationUtil.notEmpty(ls)) {
				List<String> rs = ApplicationUtil.token(mm.getFullText(), ConstantsGlobal.colonSeparator);
				for(File mod : ls) {
					String nameFile = ApplicationUtil.getBaseNameN(mod.getName()) + getExtensionModules();
					if(mod.isFile() && mm.isRicerca(mod.getName(), mod.length(), mod.lastModified()) && ApplicationUtil.textInTXT(mod, rs)) {
						MyFile f = new MyFile(mod);
						f.setName(nameFile);
						f.setModified(mod.lastModified());
						sis.add(f);
					}
				}
			}
		}
		return sis;
	}
	public List<File> listaFile() {
		File perc = getPathFile();
		if(perc != null) {
			ApplicationUtil.creaFile(true, perc);
			List<File> files =  new ArrayList<File>(Arrays.asList(perc.listFiles(new ModuloFileFilter(getExtensionModules(), false))));
			Collections.sort(files, new FileComparator());
			return files;
		}
		return null;
	}
	public File getPathFile() {
		String path = servletContext.getRealPath("/");
		FileSystemResource resource=new FileSystemResource(path + getPathModules());
		return resource.getFile();
	}
	public String getPathModules() {
		return ConstantsGlobal.path_modules + "/";
	}

	public String getExtensionModules() {
		return "." + ConstantsMap.xhtml;
	}

	public void scaricaModulo(MyFile file, DownloadBean downloadBean) {
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file.getName());
			InputStream stream = new FileInputStream(getPathFile() + "/" + file.getName());  
			ApplicationUtil.copy(stream, out);
			stream.close();
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public Long getAllegatoByNome(Class<?> classe, String nome) {
		DetachedCriteria criteria = DetachedCriteria.forClass(classe);
		criteria.add(Restrictions.eq("nome", nome));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return (Long) getUniqueResult(criteria);
	}
	public void getModuli(List<MyFile> mFiles, DownloadBean downloadBean) {
		try {
			File path = getPathFile();
			List<Allegato> files = new ArrayList<Allegato>();
			if (ApplicationUtil.notEmpty(mFiles)) {
				String p = ConstantsGlobal.secretZ;
				Iterator<MyFile> itS = mFiles.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					ApplicationUtil.addFile(files, file.getName(), path+"/"+file.getName(), file.getModified());
				}
				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.MODULI.stampa()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);
				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();
			}
		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public String readModulo(MyFile ff) throws Exception {
		File file = new File(getPathFile() + "/" + ff.getName());
		if (file.exists()) {
			BufferedReader input = new BufferedReader(new FileReader(file));
			StringBuffer buffer = new StringBuffer();
			String text;
			while ((text = input.readLine()) != null)
				buffer.append(text + System.lineSeparator());
			input.close();
			return buffer.toString();
		}
		return null;
	}
	public void salvaModulo(MyFile ff, String content) throws Exception {
		File file = new File(getPathFile() + "/" + ff.getName());
		FileOutputStream f = new FileOutputStream(file);
		PrintStream scrivi = new PrintStream(f);
		scrivi.println(content != null ? content.replaceAll(ConstantsGlobal.EOLINE, "") : content);
		scrivi.close();
		String home = impostazioneDao.getHome();
		ApplicationUtil.copyFile(file, home + ConstantsGlobal.path_modules + "/" + ff.getName());
	}
	public String apriTable() throws Exception {
		File file = new File(ConstantsGlobal.path_firewall);
		if (file.exists()) {
			BufferedReader input = new BufferedReader(new FileReader(file));
			StringBuffer buffer = new StringBuffer();
			String text;
			while ((text = input.readLine()) != null)
				buffer.append(text + System.lineSeparator());
			input.close();
			return buffer.toString();
		}
		return null;
	}
	public void salvaTable(String content) throws Exception {
		File file = new File(ConstantsGlobal.path_firewall);
		FileOutputStream f = new FileOutputStream(file);
		PrintStream scrivi = new PrintStream(f);
		scrivi.println(content != null ? content.replaceAll(ConstantsGlobal.EOLINE, "") : content);
		scrivi.close();
		ricaricaRules();
	}
	public void salvaModuloS(String nome, String content) throws Exception {
		File file = new File(getPathFile() + "/" + nome +  getExtensionModules());
		FileOutputStream f = new FileOutputStream(file);
		PrintStream scrivi = new PrintStream(f);
		scrivi.println(content != null ? content.replaceAll(ConstantsGlobal.EOLINE, "") : content);
		scrivi.close();
		String home = impostazioneDao.getHome();
		ApplicationUtil.copyFile(file, home + ConstantsGlobal.path_modules + "/" + nome +  getExtensionModules());
	}
	public void eliminaModuli(List<MyFile> file) {
		if(ApplicationUtil.notEmpty(file)) {
			Iterator<MyFile> it = file.iterator();
			while (it.hasNext()) {
				MyFile myFile = (MyFile) it.next();
				eliminaModulo(myFile);
			}
		}
	}
	public void eliminaModulo(MyFile file) {
		Session session = openSession();
		Transaction tx = session.beginTransaction();
		String hql1 = "delete from CampoAggiuntivoAnagrafica where tipo = "+ConstantsMap.TIPO_FORM+" and valori = :campo";
		String hql2 = "delete from CampoAggiuntivoContratto where  tipo = "+ConstantsMap.TIPO_FORM+" and valori = :campo";
		String hql3 = "update Campagna set modulo = null where modulo = :campo";
		String hql4 = "update VocalOrder set modulo = null where modulo = :campo";
		execute(false, session, hql1, new Parameter("campo", file.getName()));
		execute(false, session, hql2, new Parameter("campo", file.getName()));
		execute(false, session, hql3, new Parameter("campo", file.getName()));
		execute(false, session, hql4, new Parameter("campo", file.getName()));
		session.flush();
		session.clear();
		tx.commit();
		closeSession(session);

		ApplicationUtil.deleteFile(getPathFile() + "/" + file);
		String home = impostazioneDao.getHome();
		ApplicationUtil.deleteFile(home + ConstantsGlobal.path_modules + "/" + file);
	}
	public void salvaModulo(FileUpload fileUpload) {
		List<MyFile> fls = fileUpload.getFiles();
		if(ApplicationUtil.notEmpty(fls)) {
			try {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					String ext = ApplicationUtil.getExtensionLower(ff.getName());
					if(ext.equalsIgnoreCase(ConstantsMap.xhtml)) {
						String rN = ApplicationUtil.trimToNull(ApplicationUtil.getBaseNameN(ff.getName()));
						if(rN != null) {
							String rName = ApplicationUtil.replaceFileName(rN);
							File f = new File(getPathFile() + "/" + rName + getExtensionModules());
							FileOutputStream fis = new FileOutputStream(f);
							fis.write(ff.getData());
							fis.flush();
							fis.close();
							ApplicationUtil.copyFile(f, home + ConstantsGlobal.path_modules + "/" + rName + getExtensionModules());
						}
					} else {
						ApplicationUtil.unzip(ff.getData(), getPathFile(), home, ConstantsGlobal.path_modules, getExtensionModules());
					}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public List<SelectItem> getCampiSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(Integer c : ConstantsMap.getTipoCampo()) {
			sis.add(new SelectItem(c,  Message.addCatalogText("tipocampo_" + c)));
		}
		return sis;
	}
	public List<SelectItem> getCampiSIFORM() {
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		SelectItemGroup singoloG = new SelectItemGroup(Message.addCatalogText("singolo")); 
		List<SelectItem> singolo = new ArrayList<SelectItem>();
		singolo.addAll(getCampiSI());
		singoloG.setSelectItems(singolo.toArray(new SelectItem[singolo.size()]));
		sisGroup.add(singoloG);
		SelectItemGroup multiploG = new SelectItemGroup(Message.addCatalogText("multiplo")); 
		List<SelectItem> multiplo = new ArrayList<SelectItem>();
		multiplo.add(new SelectItem(ConstantsMap.TIPO_FORM,  Message.addCatalogText("tipocampo_" + ConstantsMap.TIPO_FORM)));
		multiploG.setSelectItems(multiplo.toArray(new SelectItem[multiplo.size()]));
		sisGroup.add(multiploG);
		return sisGroup;
	}
	public List<SelectItem> getCampiSIRicerca() {
		List<SelectItem> sis = getCampiSI();
		sis.add(0, new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		return sis;
	}
	public List<SelectItem> getCampiSIFORMRicerca() {
		List<SelectItem> sis = getCampiSIFORM();
		sis.add(0, new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		return sis;
	}
	public List<SelectItem> getTipo() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(String c : ConstantsMap.telefonoFMap) {
			sis.add(new SelectItem(c, Message.addCatalogText(c)));
		}
		return sis;
	}
	
	
	public List<SelectItem> getPhone() {
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		SelectItemGroup p1 = new SelectItemGroup(ConstantsMap.phone1); 
		List<SelectItem> s1 = new ArrayList<SelectItem>();
		s1.add(new SelectItem(ConstantsMap.phone1, ConstantsMap.phone1));
		p1.setSelectItems(s1.toArray(new SelectItem[s1.size()]));
		sisGroup.add(p1);
		SelectItemGroup p2 = new SelectItemGroup(ConstantsMap.phone2); 
		List<SelectItem> s2 = new ArrayList<SelectItem>();
		for(String p : ConstantsMap.phoneA) {
			s2.add(new SelectItem(p, p));
		}
		p2.setSelectItems(s2.toArray(new SelectItem[s2.size()]));
		sisGroup.add(p2);
		return sisGroup;
	}
	public List<SelectItem> getTastierino() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsMap.t0, ConstantsLanguage.NO.stampa()));
		sis.add(new SelectItem(ConstantsMap.t1, ConstantsLanguage.TONO.stampa()));
		sis.add(new SelectItem(ConstantsMap.t2, ConstantsLanguage.NORMALE.stampa()));
		sis.add(new SelectItem(ConstantsMap.t3, ConstantsLanguage.VALIDA.stampa()));
		return sis;
	}
	public List<SelectItem> getNat() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.nat1, ConstantsLanguage.SI.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.nat2, ConstantsLanguage.COMEDIA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.nat3, ConstantsLanguage.NO.stampa()));
		return sis;
	}
	public List<SelectItem> getTipiContatto() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.inserito, ConstantsLanguage.INSERITO.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.inviato, ConstantsLanguage.INVIATO.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.respinto, ConstantsLanguage.RESPINTO.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.bloccato, ConstantsLanguage.BLOCCATO.stampa()));
		return sis;
	}
	public List<SelectItem> getStati() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.in_attesa, ConstantsLanguage.STATO_IN_ATTESA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.in_pagamento, ConstantsLanguage.STATO_IN_PAGAMENTO.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.pagato, ConstantsLanguage.STATO_PAGATO.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.in_storno, ConstantsLanguage.STATO_IN_STORNO.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.stornato, ConstantsLanguage.STATO_STORNATO.stampa()));
		return sis;
	}
	public List<SelectItem> getStatoPosta() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.INVIATA, ConstantsLanguage.TESTO_INVIATA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.TESTO_NON_INVIATA.stampa()));
		return sis;
	}
	public List<SelectItem> getStatoComunicazione() {
		List<SelectItem> sis = getStatoComunicazioneMassivo();
		sis.add(new SelectItem(ConstantsGlobal.SCHEDULATA, ConstantsLanguage.TESTO_SCHEDULATA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.PAUSE, ConstantsLanguage.TESTO_PAUSE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.ANNULLATA, ConstantsLanguage.TESTO_ANNULLATA.stampa()));
		return sis;
	}
	
	public List<SelectItem> getStatoComunicazioneA() {
		List<SelectItem> sis = getStatoComunicazioneMassivo();
		sis.add(new SelectItem(ConstantsGlobal.SCHEDULATA, ConstantsLanguage.TESTO_SCHEDULATA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.PAUSE, ConstantsLanguage.TESTO_PAUSE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.ANNULLATA, ConstantsLanguage.TESTO_ANNULLATA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.APERTO, ConstantsLanguage.TESTO_APERTO.stampa()));
		return sis;
	}
	public List<SelectItem> getStatoComunicazioneMassivo() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.IN_INVIO, ConstantsLanguage.TESTO_IN_INVIO.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.INVIATA, ConstantsLanguage.TESTO_INVIATA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.TESTO_NON_INVIATA.stampa()));
		return sis;
	}
	public List<SelectItem> getTipoComunicazioneMassivo() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.tipo_email, ConstantsLanguage.EMAIL.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.tipo_rmail, ConstantsLanguage.RMAIL.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.tipo_telecom, ConstantsLanguage.TELECOM.stampa()));
		return sis;
	}
	public List<SelectItem> getTipiContattoSI() {
		return getTipiContatto();
	}
	public List<SelectItem> getStatiRicerca() {
		return getStati();
	}
	public List<SelectItem> getEncryption() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(String e : ConstantsMap.getEncryptionList()) {
			sis.add(new SelectItem(e, e));
		}
		return sis;
	}
	public List<SelectItem> getRicezione_type() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(0, ConstantsMap.NO));
		sis.add(new SelectItem(1, ConstantsMap.IMAP));
		sis.add(new SelectItem(2, ConstantsMap.POP));
		return sis;
	}

	public List<SelectItem> getProtocol() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(String e : ConstantsMap.getProtocolList()) {
			sis.add(new SelectItem(e, Message.addCatalogText(e)));
		}
		return sis;
	}
	public List<SelectItem> getLimiteCampagna() {
		int def = ConstantsGlobal.defLimiteCampagna;
		int max = ConstantsGlobal.maxLimiteCampagna;
		int step = ConstantsGlobal.stepLimiteCampagna;
		boolean tutti = ConstantsGlobal.tuttiLimiteCampagna;
		return ApplicationUtil.getLimite(step, max, def, tutti);
	}
	public List<SelectItem> getLimiteOperatore() {
		int def = ConstantsGlobal.defLimiteOperatore;
		int max = ConstantsGlobal.maxLimiteOperatore;
		int step = ConstantsGlobal.stepLimiteOperatore;
		boolean tutti = ConstantsGlobal.tuttiLimiteOperatore;
		return ApplicationUtil.getLimite(step, max, def, tutti);
	}
	public List<SelectItem> getLimiteAgente() {
		int def = ConstantsGlobal.defLimiteAgente;
		int max = ConstantsGlobal.maxLimiteAgente;
		int step = ConstantsGlobal.stepLimiteAgente;
		boolean tutti = ConstantsGlobal.tuttiLimiteAgente;
		return ApplicationUtil.getLimite(step, max, def, tutti);
	}
	public List<SelectItem> getLimiteTeam() {
		int def = ConstantsGlobal.defLimiteTeam;
		int max = ConstantsGlobal.maxLimiteTeam;
		int step = ConstantsGlobal.stepLimiteTeam;
		boolean tutti = ConstantsGlobal.tuttiLimiteTeam;
		return ApplicationUtil.getLimite(step, max, def, tutti);
	}
	public List<SelectItem> getLimiteUfficio() {
		int def = ConstantsGlobal.defLimiteUfficio;
		int max = ConstantsGlobal.maxLimiteUfficio;
		int step = ConstantsGlobal.stepLimiteUfficio;
		boolean tutti = ConstantsGlobal.tuttiLimiteUfficio;
		return ApplicationUtil.getLimite(step, max, def, tutti);
	}
	public List<SelectItem> getLimiteQualifica() {
		int def = ConstantsGlobal.defLimiteQualifica;
		int max = ConstantsGlobal.maxLimiteQualifica;
		int step = ConstantsGlobal.stepLimiteQualifica;
		boolean tutti = ConstantsGlobal.tuttiLimiteQualifica;
		return ApplicationUtil.getLimite(step, max, def, tutti);
	}
	public List<SelectItem> getLimiteZona() {
		int def = ConstantsGlobal.defLimiteZona;
		int max = ConstantsGlobal.maxLimiteZona;
		int step = ConstantsGlobal.stepLimiteZona;
		boolean tutti = ConstantsGlobal.tuttiLimiteZona;
		return ApplicationUtil.getLimite(step, max, def, tutti);
	}
	public List<SelectItem> getVelocita() {
		int max = ConstantsGlobal.maxVelocita;
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(0, new SelectItem(0, ConstantsLanguage.TRATTINI.stampa()));
		for (int i = 0; i < max; i++) {
			sis.add(new SelectItem((i+1), ConstantsLanguage.VELOCITA.stampa() + " " + (i+1)));
		}
		return sis;
	}
	public List<SelectItem> getPreambolo() {
		int max = ConstantsGlobal.maxPreambolo;
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for (int i = 0; i < max; i++) {
			sis.add(new SelectItem(i, i + ""));
		}
		return sis;
	}
	public List<SelectItem> getGiorni() {
		int max = ConstantsGlobal.maxGiorni;
		boolean tutti = ConstantsGlobal.tuttiGiorni;
		List<SelectItem> sis = new ArrayList<SelectItem>();
		if(tutti) {
			sis.add(new SelectItem(0,  ConstantsLanguage.TUTTI.stampa()));
		}
		for (int i = 0; i < max; i++) {
			sis.add(new SelectItem((i+1), (i+1) + " (" + ((i+1)*24) + " "+Message.addCatalogText("ore")+")"));
		}
		return sis;
	}
	public List<SelectItem> getMonitor() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(Short c : ConstantsMap.getStato()) {
			sis.add(new SelectItem(c, Message.addCatalogText("stato." + c)));
		}
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<SelectItem> getAutoContatto() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		List<Integer> ls = new ArrayList<Integer>();
		if(ApplicationUtil.isTrue(impostazioneDao.getStatoPresente())) {
			ls.addAll(ConstantsMap.getAutoContatto());
		} else {
			DetachedCriteria criteria = DetachedCriteria.forClass(AutoContatto.class);
			criteria.setProjection(Projections.distinct(Projections.property("stato")));
			criteria.addOrder(Order.asc("stato"));
			ls.addAll(getList(criteria));
		}
		if(ApplicationUtil.notEmpty(ls)) {
			for(Integer c : ls) {
				sis.add(new SelectItem(c , Message.addCatalogText("stato.chiamata." + c)));
			}
		}
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<SelectItem> getOperazione() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		List<Short> ls = new ArrayList<Short>();
		if(ApplicationUtil.isTrue(impostazioneDao.getStatoPresente())) {
			ls.addAll(ConstantsMap.getOperazione());
		} else {
			DetachedCriteria criteria = DetachedCriteria.forClass(Operazione.class);
			criteria.setProjection(Projections.distinct(Projections.property("operazione")));
			criteria.addOrder(Order.asc("operazione"));
			ls.addAll(getList(criteria));
		}
		if(ApplicationUtil.notEmpty(ls)) {
			for(Short c : ls) {
				sis.add(new SelectItem(c , Message.addCatalogText("operazione." + c)));
			}
		}
		return sis;
	}
	public List<SelectItem> getPertinenza() {
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		List<SelectItem> a1 = new ArrayList<SelectItem>();
		String t_a1 = Message.addCatalogText("normale");
		a1.add(new SelectItem(ConstantsMap.agenti_tutti , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_tutti)));
		a1.add(new SelectItem(ConstantsMap.agenti_autonomo , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_autonomo)));
		a1.add(new SelectItem(ConstantsMap.agenti_nessuno , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_nessuno)));
		SelectItemGroup a1G = new SelectItemGroup(t_a1); 
		a1G.setSelectItems(a1.toArray(new SelectItem[a1.size()]));
		sisGroup.add(a1G);
		List<SelectItem> a2 = new ArrayList<SelectItem>();
		String t_a2 = Message.addCatalogText("localizzazione");
		a2.add(new SelectItem(ConstantsMap.agenti_pertinenza_regione , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_pertinenza_regione)));
		a2.add(new SelectItem(ConstantsMap.agenti_pertinenza_provincia , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_pertinenza_provincia)));
		a2.add(new SelectItem(ConstantsMap.agenti_pertinenza_comune , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_pertinenza_comune)));
		a2.add(new SelectItem(ConstantsMap.agenti_pertinenza_cap , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_pertinenza_cap)));
		SelectItemGroup a2G = new SelectItemGroup(t_a2); 
		a2G.setSelectItems(a2.toArray(new SelectItem[a2.size()]));
		sisGroup.add(a2G);
		List<SelectItem> a3 = new ArrayList<SelectItem>();
		String t_a3 = Message.addCatalogText("agente");
		a3.add(new SelectItem(ConstantsMap.agenti_personali , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_personali)));
		a3.add(new SelectItem(ConstantsMap.agenti_rete , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_rete)));
		a3.add(new SelectItem(ConstantsMap.agenti_sottorete , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_sottorete)));
		a3.add(new SelectItem(ConstantsMap.agenti_primalinea , Message.addCatalogText("pertinenza." + ConstantsMap.agenti_primalinea)));
		SelectItemGroup a3G = new SelectItemGroup(t_a3); 
		a3G.setSelectItems(a3.toArray(new SelectItem[a3.size()]));
		sisGroup.add(a3G);
		return sisGroup;
	}
	public List<SelectItem> getGeocoding() {
		List<SelectItem> a = new ArrayList<SelectItem>();
		a.add(new SelectItem(0 , ConstantsLanguage.TRATTINI.stampa()));
		a.add(new SelectItem(1 , Message.addCatalogText(ConstantsMap.geocodifica) + " " + "("+  Message.addCatalogText(ConstantsGlobal.asc) + ")"));
		a.add(new SelectItem(2 , Message.addCatalogText(ConstantsMap.geocodifica) + " " + "("+  Message.addCatalogText(ConstantsGlobal.desc) + ")"));
		return a;
	}
	public List<SelectItem> getRipetizioni() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(Integer c : ConstantsMap.getRipetizioni()) {
			sis.add(new SelectItem(c , Message.addCatalogText("ripetizione." + c)));
		}
		return sis;
	}
	public void deleteCampo(Object campoAggiuntivo) {
		Campo.updateCampo(campoAggiuntivo, this, null);
		Campo.updateCampoCA(campoAggiuntivo, this, false);
	}
	public void deleteLista(List<?> objs) {
		Iterator<?> objIter = objs.iterator();
		while(objIter.hasNext()) {
			Object obj = objIter.next();
			deleteCampo(obj);
		}
	}
	public void updateCampo(Object campoAggiuntivo, String ch) {
		Campo.updateCampo(campoAggiuntivo, this, ch);
		Campo.updateCampoCA(campoAggiuntivo, this, true);
	}
	public void addCampo(Object campoAggiuntivo) {
	}
	public String getAzione_sceltaData() {
		return ConstantsMap.sceltaData;
	}
	public String getAzione_sceltaModificaContatto() {
		return ConstantsMap.sceltaModificaContatto;
	}
	public String getAzione_sceltaInserisciContatto() {
		return ConstantsMap.sceltaInserisciContatto;
	}
	public String getAzione_sceltaTemplateEmail() {
		return ConstantsMap.sceltaTemplateEmail;
	}
	
	public String getAzione_sceltaTemplateRmail() {
		return ConstantsMap.sceltaTemplateRmail;
	}
	
	public String getAzione_sceltaTemplateTelecom() {
		return ConstantsMap.sceltaTemplateTelecom;
	}
	public String getAzione_sceltaProdotti() {
		return ConstantsMap.sceltaProdotti;
	}
	public String getAzione_sceltaCampi() {
		return ConstantsMap.sceltaCampi;
	}
	public String getAzione_sceltaVocalOrder() {
		return ConstantsMap.sceltaVocalOrder;
	}
	public String getAzione_sceltaContrattoPda() {
		return ConstantsMap.sceltaContrattoPda;
	}
	public String getAzione_sceltaCalendarioForm() {
		return ConstantsMap.sceltaCalendarioForm;
	}
	public String getAzione_sceltaRegistrazioni() {
		return ConstantsMap.sceltaRegistrazioni;
	}
	public String getAzione_sceltaCalendario() {
		return ConstantsMap.sceltaCalendario;
	}
	public String getAzione_sceltaSondaggio() {
		return ConstantsMap.sceltaSondaggio;
	}
	public String getAzione_sceltaFinale() {
		return ConstantsMap.sceltaFinale;
	}
	public String getAzione_sceltaChiudi() {
		return ConstantsMap.sceltaChiudi;
	}
	public String getAzione_sceltaOk() {
		return ConstantsMap.sceltaOk;
	}
	public Date getInizioGiornata() {
		return ApplicationUtil.inizioGiornata(ApplicationUtil.date());
	}
	public Date getInizioSettimana() {
		return ApplicationUtil.inizioSettimana(ApplicationUtil.date());
	}
	public Date getInizioMese() {
		return ApplicationUtil.inizioMese(ApplicationUtil.date());
	}
	public Date getInizioAnno() {
		return ApplicationUtil.inizioAnno(ApplicationUtil.date());
	}
	public int getGiorniMese() {
		return ConstantsGlobal.giorniMese;
	}
	@SuppressWarnings("unchecked")
	public List<Criterio> getCriterio(Long utenteId, String categoria, boolean tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Criterio.class);
		if(tutti) {
			criteria.add(Restrictions.or(Restrictions.eq("utente.id", utenteId), Restrictions.eq("tipo", ConstantsGlobal.pubblico)));
		} else {
			criteria.add(Restrictions.eq("utente.id", utenteId));
		}
		criteria.add(Restrictions.eq("categoria", categoria));
		criteria.addOrder(Order.desc("data_creazione"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Colonna> getColonna(Long utenteId, String categoria, boolean tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Colonna.class);
		if(tutti) {
			criteria.add(Restrictions.or(Restrictions.eq("utente.id", utenteId), Restrictions.eq("tipo", ConstantsGlobal.pubblico)));
		} else {
			criteria.add(Restrictions.eq("utente.id", utenteId));
		}
		criteria.add(Restrictions.eq("categoria", categoria));
		criteria.addOrder(Order.desc("data_creazione"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Colonna> loadColonnePredefinite(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Colonna.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.predefinito));
		criteria.add(Restrictions.eq("utente.id", utenteId));
		return getList(criteria);
	}
	public Criterio loadCriterioPredefinito(Long utenteId, String categoria) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Criterio.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.predefinito));
		criteria.add(Restrictions.eq("utente.id", utenteId));
		criteria.add(Restrictions.eq("categoria", categoria));
		return (Criterio) getFirstResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Colonna> getColonnePredefinite(Colonna crto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Colonna.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.predefinito));
		criteria.add(Restrictions.eq("utente.id", crto.getUtente().getId()));
		criteria.add(Restrictions.eq("categoria", crto.getCategoria()));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Criterio> getCriteriPredefinite(Criterio crto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Criterio.class);
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.predefinito));
		criteria.add(Restrictions.eq("utente.id", crto.getUtente().getId()));
		criteria.add(Restrictions.eq("categoria", crto.getCategoria()));
		return getList(criteria);
	}
	public List<SelectItem> getTipiLoad() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.normale , Message.addCatalogText("normale")));
		sis.add(new SelectItem(ConstantsGlobal.pubblico , Message.addCatalogText("pubblico")));
		sis.add(new SelectItem(ConstantsGlobal.predefinito , Message.addCatalogText("predefinito")));
		return sis;
	}
	public Colonna salva(Colonna obj) {
		if(ApplicationUtil.gtZero(obj.getTipo()) && obj.getTipo().equals(ConstantsGlobal.predefinito) ) {
			List<Colonna> colonne = getColonnePredefinite(obj);
			if(ApplicationUtil.notEmpty(colonne)) {
				Iterator<Colonna> objIter = colonne.iterator();
				while(objIter.hasNext()) {
					Colonna colonna = objIter.next();
					colonna.setTipo(ConstantsGlobal.normale);
					salvaAggiorna(colonna);
				}
			}
		}
		return (Colonna) salvaAggiorna(obj);
	}
	public void elimina(Colonna obj) {
		cancellaAggiorna(obj);
	}
	public ExtendedCriteria getColonnaCriteria(ColonnaCriteria crt) {
		if(crt != null)  crt = (ColonnaCriteria) crt.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Colonna.class);
		if (ApplicationUtil.isNotBlank(crt.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crt.getNome(), criteria, "nome");
		}
		if (ApplicationUtil.isNotBlank(crt.getCategoria())) {
			criteria.add(Restrictions.eq("categoria", crt.getCategoria()));
		}
		if (crt.getTipo() != null) {
			criteria.add(Restrictions.eq("tipo", crt.getTipo()));
		}
		if (ApplicationUtil.gtZero(crt.getUtente())) {
			criteria.add(Restrictions.eq("utente.id", crt.getUtente()));
		}
		crt.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crt);
	}
	@SuppressWarnings("unchecked")
	public List<Colonna> trova(ColonnaCriteria crt, int start, int size) {
		ExtendedCriteria criteria = getColonnaCriteria(crt);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(ColonnaCriteria prd) {
		ExtendedCriteria criteria = getColonnaCriteria(prd);
		return getListSize(criteria);
	}
	public List<SelectItem> getTipiCategoriaColonna() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Colonna.class);
		List<GenericSI> lis = getGenericSI(criteria, "categoria", "categoria");
		return impostazioneDao.getGenericNoID(lis, true);
	}
	public List<SelectItem> getUtentiSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	

	public UtenteUtil creaUtenti(Utente o) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(o != null) {
			criteria.add(Restrictions.ne("id", o.getId()));
		}
		return new UtenteUtil(getUtenteSI(criteria));
	}
	
	@SuppressWarnings("unchecked")
	public void processaUtenti(UtenteUtil uu, Colonna o) {
		if(ApplicationUtil.notEmpty(uu.getDestinazione())) {
			List<Long> ln = ApplicationUtil.prendiIdUserSI(uu.getDestinazione());
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
			criteria.add(Restrictions.in("id",ln));
			List<Utente> users = getList(criteria);
			for(Utente u : users) {
				Colonna copia = o.getCopia();
				copia.setUtente(u);
				salva(copia);
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	public void processaUtenti(UtenteUtil uu, Criterio o) {
		if(ApplicationUtil.notEmpty(uu.getDestinazione())) {
			List<Long> ln = ApplicationUtil.prendiIdUserSI(uu.getDestinazione());
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
			criteria.add(Restrictions.in("id",ln));
			List<Utente> users = getList(criteria);
			for(Utente u : users) {
				Criterio copia = o.getCopia();
				copia.setUtente(u);
				salva(copia);
			}
		}
	}


	public Criterio salva(Criterio obj) {
		if(ApplicationUtil.gtZero(obj.getTipo()) && obj.getTipo().equals(ConstantsGlobal.predefinito) ) {
			List<Criterio> criteri = getCriteriPredefinite(obj);
			if(ApplicationUtil.notEmpty(criteri)) {
				Iterator<Criterio> objIter = criteri.iterator();
				while(objIter.hasNext()) {
					Criterio criterio = objIter.next();
					criterio.setTipo(ConstantsGlobal.normale);
					salvaAggiorna(criterio);
				}
			}
		}
		return (Criterio) salvaAggiorna(obj);
	}
	public void elimina(Criterio obj) {
		cancellaAggiorna(obj);
	}
	public ExtendedCriteria getCriterioCriteria(CriterioCriteria crt) {
		if(crt != null)  crt = (CriterioCriteria) crt.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Criterio.class);
		if (ApplicationUtil.isNotBlank(crt.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, crt.getNome(), criteria, "nome");
		}
		if (ApplicationUtil.isNotBlank(crt.getCategoria())) {
			criteria.add(Restrictions.eq("categoria", crt.getCategoria()));
		}
		if (crt.getTipo() != null) {
			criteria.add(Restrictions.eq("tipo", crt.getTipo()));
		}
		if (ApplicationUtil.gtZero(crt.getUtente())) {
			criteria.add(Restrictions.eq("utente.id", crt.getUtente()));
		}
		crt.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, crt);
	}
	@SuppressWarnings("unchecked")
	public List<Criterio> trova(CriterioCriteria crt, int start, int size) {
		ExtendedCriteria criteria = getCriterioCriteria(crt);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(CriterioCriteria prd) {
		ExtendedCriteria criteria = getCriterioCriteria(prd);
		return getListSize(criteria);
	}
	public List<SelectItem> getTipiCategoriaCriterio() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Criterio.class);
		List<GenericSI> lis = getGenericSI(criteria, "categoria", "categoria");
		return impostazioneDao.getGenericNoID(lis, true);
	}
	public Colonna getColonna(Long id) {
		return (Colonna) get(Colonna.class, id);
	}
	public Criterio getCriterio(Long id) {
		return (Criterio) get(Criterio.class, id);
	}
	public List<SelectItem> getRegistrazioni() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsMap.REC_NO, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.NO.stampa())));
		sis.add(new SelectItem(ConstantsMap.REC_SI,  ApplicationUtil.upperCaseEmpty(ConstantsLanguage.SI.stampa())));
		sis.add(new SelectItem(ConstantsMap.REC_AUTO,  ApplicationUtil.upperCaseEmpty(ConstantsLanguage.AUTOMATICA.stampa())));
		return sis;
	}
	public List<SelectItem> getStatiPrenotazione() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.prenotazione_dainviare, ConstantsLanguage.DAINVIARE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.prenotazione_inviata, ConstantsLanguage.INVIATA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.prenotazione_duplicata, ConstantsLanguage.DUPLICATA.stampa()));
		return sis;
	}
	public List<SelectItem> getTelefoni() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.f_fisso, ConstantsLanguage.TELEFONO_FISSO.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.f_mobile, ConstantsLanguage.TELEFONO_MOBILE.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.f_fax, ConstantsLanguage.TELEFONO_FAX.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.f_altro, ConstantsLanguage.TELEFONO_ALTRO.stampa()));
		return sis;
	}
	public List<SelectItem> getAvvioAutomatico() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(0, Message.addCatalogText("no")));
		sis.add(new SelectItem(1, Message.addCatalogText("pubbliche")));
		sis.add(new SelectItem(2, Message.addCatalogText("personali")));
		sis.add(new SelectItem(3, Message.addCatalogText("tutte")));
		return sis;
	}
	public List<SelectItem> getScelte() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(false, ApplicationUtil.upperCaseEmpty(ConstantsLanguage.INALTERATO.stampa())));
		sis.add(new SelectItem(true,  ApplicationUtil.upperCaseEmpty(ConstantsLanguage.MODIFICA.stampa())));
		return sis;
	}
	public String getIp() {
		HttpServletRequest request = ApplicationUtil.getRequest(null);
		return ApplicationUtil.ip(request);
	}
	public String getCaptchaKey() {
		return impostazioneDao.getKeyCaptcha_public();
	}
	public void phone(Utente load, boolean ricarica) {
		if(!ricarica) {
			load.setPhone(ApplicationUtil.parameter("phone"));
		}
		loadPhone(load);
	}
	
	public void loadPhone(Utente load) {
		String pn = ApplicationUtil.trimToNull(load.getPhone());
		if(pn != null) {
			String [] s = pn.split(ConstantsGlobal.pipeSeparator);
			String ph = ApplicationUtil.trimToNull(s.length > 0 ? s[0] : null);
			String wb = ApplicationUtil.trimToNull(s.length > 1 ? s[1] : null);
			String cd = ApplicationUtil.trimToNull(s.length > 2 ? s[2] : null);
			SipBuddies sb = ph != null ? getInterno(ph) : load.getInterno();
			if(sb != null) {
				if(Arrays.asList(ConstantsMap.phoneA).contains(wb) || ApplicationUtil.equals(ConstantsMap.phone1, wb)) {
					sb.setPhone(wb);
				}
				if(ConstantsMap.getValoriStatici().contains(cd)) {
					sb.setAllow(cd);
				}
			}
			load.setInterno(sb);
		}
	}
	
	public SipBuddies getInterno(String name){
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("name", name));
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		return  (SipBuddies) getUniqueResult(criteria);
	}

	public boolean captcha(String ipAddress) {
		Integer maxLogin = impostazioneDao.getMassimoLogin1();
		boolean blocca = false;
		if(ApplicationUtil.gtZero(maxLogin)) {
			int i = Chiamate.getCodaLogin().get(ipAddress);
			if(i >= maxLogin.intValue()) {
				blocca = true;
				String g = ApplicationUtil.parameter("g-recaptcha-response");
				if(g != null) {
					String key =  impostazioneDao.getKeyCaptcha_private();
					String rc = String.format("https://www.google.com/recaptcha/api/siteverify?secret=%s&response=%s", key, g);
					try {
						Document doc = Jsoup.connect(rc).ignoreContentType(true).ignoreHttpErrors(true).followRedirects(true).get();
						Elements body = doc.select("body");
						JSONObject obj = new JSONObject(body.text());
						Boolean s = (Boolean) obj.get("success");
						if(ApplicationUtil.isTrue(s)) {
							blocca = false;
						}
					} catch (IOException e) {
					}
				}
			}
		}
		return blocca;
	}
	public boolean ipAddress(String ip_consentiti, String ip_bloccati, String ipAddress) {
		return ApplicationUtil.ipAddress(ip_consentiti, ip_bloccati, ipAddress);
	}
	
	public List<SelectItem> getServers() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		if(ApplicationUtil.isTrue(impostazioneDao.getServers())) {
			try {
				Document doc = Jsoup.connect(ConstantsGlobal.admin + ConstantsGlobal.path_web_controlla).ignoreContentType(true).ignoreHttpErrors(true).followRedirects(true).get();
				Elements body = doc.select("body");
				JSONArray obj = new JSONArray(body.text());
				Iterator<Object> lnt = obj.iterator();
				while (lnt.hasNext()) {
					JSONObject e = (JSONObject) lnt.next();
					for(String s : JSONObject.getNames(e)) {
						String url = e.getString(s);
						if(ApplicationUtil.isSSL(url)) {
							list.add(new SelectItem(url, s));
						}
					}
				}
			} catch (Exception e) {	}
		}
		if(list.size() <= 1) {
			list = new ArrayList<SelectItem>();
		}
		return list;
	}
	public AgenteMassivo nuovoAgente() {
		return new AgenteMassivo();
	}
	public UtenteMassivo nuovoUtente() {
		return new UtenteMassivo();
	}
	public EsitoChiamataMassivo nuovoEsitoChiamata() {
		return new EsitoChiamataMassivo();
	}
	public EsitoAppuntamentoMassivo nuovoEsitoAppuntamento() {
		return new EsitoAppuntamentoMassivo();
	}
	public EsitoContrattoMassivo nuovoEsitoContratto() {
		return new EsitoContrattoMassivo();
	}
	public EsitoArchiviazioneMassivo nuovoEsitoArchiviazione() {
		return new EsitoArchiviazioneMassivo();
	}
	public TrunkMassivo nuovoTrunk() {
		return new TrunkMassivo();
	}
	public InternoMassivo nuovoInterno() {
		return new InternoMassivo();
	}
	public CampagnaMassivo nuovoCampagna() {
		return new CampagnaMassivo();
	}
	public ListaMassivo nuovoLista() {
		return new ListaMassivo();
	}
	public AnagraficaMassivo nuovoAnagrafica() {
		return new AnagraficaMassivo();
	}
	public CampoAggiuntivoAnagraficaMassivo nuovoCampoAnagrafica() {
		return new CampoAggiuntivoAnagraficaMassivo();
	}
	public CampoAggiuntivoContrattoMassivo nuovoCampoContratto() {
		return new CampoAggiuntivoContrattoMassivo();
	}
	public CodaMassivo nuovoCoda() {
		return new CodaMassivo();
	}
	public void pannelloModificaAgenteSelezionati(List<Agente> obj,  AgenteMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Ufficio ufficio = (Ufficio) get(Ufficio.class, massivo.get("ufficio").getLungo());
			Qualifica qualifica = (Qualifica) get(Qualifica.class, massivo.get("qualifica").getLungo());
			Iterator<Agente> objIter = obj.iterator();
			while(objIter.hasNext()) {
				Agente agente = objIter.next();
				massivo.modifica(agente, qualifica, ufficio);
				salvaAggiorna(agente);
			}
		}
	}
	public void pannelloModificaUtenteSelezionati(List<Utente> obj,  UtenteMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			SipBuddies trunk = (SipBuddies) get(SipBuddies.class,  massivo.get("trunk").getLungo());
			List<Ruolo> ruoli = userDao.getRuoliByNome(massivo.get("ruoli").getMultiplo()); 
			Iterator<Utente> objIter = obj.iterator();
			while(objIter.hasNext()) {
				Utente utente = objIter.next();
				if(utente.getSelezionabile()) {
					massivo.modifica(utente, trunk, ruoli);
					salvaAggiorna(utente);
				}
			}
		}
	}
	public void pannelloModificaEsitoAppuntamentoSelezionati(List<EsitoAppuntamento> obj,  EsitoAppuntamentoMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<EsitoAppuntamento> objIter = obj.iterator();
			while(objIter.hasNext()) {
				EsitoAppuntamento esito = objIter.next();
				massivo.modifica(esito);
				salvaAggiorna(esito);
			}
		}
	}
	public void pannelloModificaEsitoChiamataSelezionati(List<EsitoChiamata> obj,  EsitoChiamataMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<EsitoChiamata> objIter = obj.iterator();
			while(objIter.hasNext()) {
				EsitoChiamata esito = objIter.next();
				if(esito.getSelezionabile()) {
					massivo.modifica(esito);
					contattoDao.salvaEsito(esito);
				}
			}
		}
	}
	public void pannelloModificaEsitoArchiviazioneSelezionati(List<EsitoArchiviazione> obj,  EsitoArchiviazioneMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<EsitoArchiviazione> objIter = obj.iterator();
			while(objIter.hasNext()) {
				EsitoArchiviazione esito = objIter.next();
				massivo.modifica(esito);
				salvaAggiorna(esito);
			}
		}
	}
	public void pannelloModificaEsitoContrattoSelezionati(List<EsitoContratto> obj,  EsitoContrattoMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<EsitoContratto> objIter = obj.iterator();
			while(objIter.hasNext()) {
				EsitoContratto esito = objIter.next();
				massivo.modifica(esito);
				salvaAggiorna(esito);
			}
		}
	}
	public void pannelloModificaListaSelezionati(List<Lista> obj,  ListaMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<Lista> objIter = obj.iterator();
			while(objIter.hasNext()) {
				Lista o = objIter.next();
				massivo.modifica(o);
				salvaAggiorna(o);
			}
		}
	}
	public void pannelloModificaCampagnaSelezionati(List<Campagna> obj,  CampagnaMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Autodialer autodialer = (Autodialer) get(Autodialer.class,  massivo.get("autodialer").getLungo());
			EsitoChiamata esito = (EsitoChiamata) get(EsitoChiamata.class,  massivo.get("esito").getLungo());
			Iterator<Campagna> objIter = obj.iterator();
			while(objIter.hasNext()) {
				Campagna o = objIter.next();
				massivo.modifica(o, autodialer, esito);
				salvaAggiorna(o);
			}
		}
	}
	public void pannelloModificaTrunkSelezionati(List<SipBuddies> obj,  TrunkMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<SipBuddies> objIter = obj.iterator();
			while(objIter.hasNext()) {
				SipBuddies o = objIter.next();
				massivo.modifica(o);
				o.impostaTrunk();
				salvaAggiorna(o);
			}
		}
	}
	public void pannelloModificaInternoSelezionati(List<SipBuddies> obj,  InternoMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<SipBuddies> objIter = obj.iterator();
			while(objIter.hasNext()) {
				SipBuddies o = objIter.next();
				massivo.modifica(o);
				o.impostaInterno();
				salvaAggiorna(o);
			}
		}
	}
	
	public void pannelloModificaCodaSelezionati(List<Coda> obj,  CodaMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<Coda> objIter = obj.iterator();
			while(objIter.hasNext()) {
				Coda o = objIter.next();
				massivo.modifica(o);
				salvaAggiorna(o);
			}
		}
	}
	public void pannelloModificaAnagraficaSelezionati(List<Anagrafica> obj,  AnagraficaMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<Anagrafica> objIter = obj.iterator();
			while(objIter.hasNext()) {
				Anagrafica anagrafica = objIter.next();
				massivo.modifica(anagrafica);
				salvaAggiorna(anagrafica);
			}
		}
	}

	public void pannelloModificaContattoSelezionati(List<Contatto> obj,  AnagraficaMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<Contatto> objIter = obj.iterator();
			while(objIter.hasNext()) {
				Contatto contatto = objIter.next();
				massivo.modifica(contatto.getAnagrafica());
				salvaAggiorna(contatto.getAnagrafica());
			}
		}
	}
	public List<SelectItem> getValoriCodec() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		for(String valore : ConstantsMap.getValoriStatici()) {
			list.add(new SelectItem(valore, valore));
		}
		return list;
	}
	public void pannelloModificaCampoAnagraficaSelezionati(List<CampoAggiuntivoAnagrafica> obj,  CampoAggiuntivoAnagraficaMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<CampoAggiuntivoAnagrafica> objIter = obj.iterator();
			while(objIter.hasNext()) {
				CampoAggiuntivoAnagrafica o = objIter.next();
				massivo.modifica(o);
				salvaAggiorna(o);
			}
		}
	}
	public void pannelloModificaCampoContrattoSelezionati(List<CampoAggiuntivoContratto> obj,  CampoAggiuntivoContrattoMassivo massivo) {
		if (ApplicationUtil.notEmpty(obj)) {
			Iterator<CampoAggiuntivoContratto> objIter = obj.iterator();
			while(objIter.hasNext()) {
				CampoAggiuntivoContratto o = objIter.next();
				massivo.modifica(o);
				salvaAggiorna(o);
			}
		}
	}
	public List<SelectItem> getTipoTelefoni() {
		List<SelectItem> normale = new ArrayList<SelectItem>();
		normale.add(new SelectItem(1,Message.addCatalogText("telefono_1")));
		normale.add(new SelectItem(2,Message.addCatalogText("telefono_2")));
		normale.add(new SelectItem(3,Message.addCatalogText("telefono_3")));
		normale.add(new SelectItem(4,Message.addCatalogText("telefono_4")));
		return normale;
	}
	
	public List<SelectItem> getTipoTelefoniC() {
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		List<SelectItem> normale = new ArrayList<SelectItem>();
		String t_normale = Message.addCatalogText("normale");
		normale.add(new SelectItem(1,Message.addCatalogText("telefono_1")));
		normale.add(new SelectItem(2,Message.addCatalogText("telefono_2")));
		normale.add(new SelectItem(3,Message.addCatalogText("telefono_3")));
		normale.add(new SelectItem(4,Message.addCatalogText("telefono_4")));
		SelectItemGroup normaleG = new SelectItemGroup(t_normale); 
		normaleG.setSelectItems(normale.toArray(new SelectItem[normale.size()]));
		sisGroup.add(normaleG);
		List<SelectItem> speciale = new ArrayList<SelectItem>();
		String t_speciale = Message.addCatalogText("speciale");
		speciale.add(new SelectItem(5, Message.addCatalogText("telefono_5")));
		speciale.add(new SelectItem(6, Message.addCatalogText("telefono_6")));
		speciale.add(new SelectItem(7, Message.addCatalogText("telefono_7")));
		speciale.add(new SelectItem(8, Message.addCatalogText("telefono_8")));
		SelectItemGroup specialeG = new SelectItemGroup(t_speciale); 
		specialeG.setSelectItems(speciale.toArray(new SelectItem[speciale.size()]));
		sisGroup.add(specialeG);
		return sisGroup;
	}
	public List<SelectItem> getTipoTelefoniL() {
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		sisGroup.add(new SelectItem(0, Message.addCatalogText("trattini")));
		List<SelectItem> normale = new ArrayList<SelectItem>();
		String t_normale = Message.addCatalogText("normale");
		normale.add(new SelectItem(1,Message.addCatalogText("telefono_1")));
		normale.add(new SelectItem(2,Message.addCatalogText("telefono_2")));
		normale.add(new SelectItem(3,Message.addCatalogText("telefono_3")));
		normale.add(new SelectItem(4,Message.addCatalogText("telefono_4")));
		SelectItemGroup normaleG = new SelectItemGroup(t_normale); 
		normaleG.setSelectItems(normale.toArray(new SelectItem[normale.size()]));
		sisGroup.add(normaleG);
		List<SelectItem> speciale = new ArrayList<SelectItem>();
		String t_speciale = Message.addCatalogText("speciale");
		speciale.add(new SelectItem(5, Message.addCatalogText("telefono_5")));
		speciale.add(new SelectItem(6, Message.addCatalogText("telefono_6")));
		speciale.add(new SelectItem(7, Message.addCatalogText("telefono_7")));
		speciale.add(new SelectItem(8, Message.addCatalogText("telefono_8")));
		SelectItemGroup specialeG = new SelectItemGroup(t_speciale); 
		specialeG.setSelectItems(speciale.toArray(new SelectItem[speciale.size()]));
		sisGroup.add(specialeG);
		return sisGroup;
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoAnagrafica> getCampoAggiuntivoAnagraficaByModulo(MyFile path) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoAnagrafica.class);
		criteria.add(Restrictions.eq("tipo", ConstantsMap.TIPO_FORM));
		criteria.add(Restrictions.eq("valori", path.getName()));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoContratto> getCampoAggiuntivoContrattoByModulo(MyFile path) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.eq("tipo", ConstantsMap.TIPO_FORM));
		criteria.add(Restrictions.eq("valori", path.getName()));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagnaByModulo(MyFile path) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.eq("modulo", path.getName()));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<VocalOrder> getVocalOrderByModulo(MyFile path) {
		DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
		criteria.add(Restrictions.eq("modulo", path.getName()));
		return getList(criteria);
	}
	public List<SelectItem> getTipoProgressoSI() {
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		sisGroup.add(new SelectItem(7, Message.addCatalogText("progresso")));
		List<SelectItem> personali = new ArrayList<SelectItem>();
		String t_personali = Message.addCatalogText("personali");
		personali.add(new SelectItem(1, t_personali + ": " + Message.addCatalogText("disponibili")));
		personali.add(new SelectItem(2, t_personali + ": " + Message.addCatalogText("prossimi")));
		personali.add(new SelectItem(3, t_personali + ": " + Message.addCatalogText("tutti")));
		SelectItemGroup personaliG = new SelectItemGroup(t_personali); 
		personaliG.setSelectItems(personali.toArray(new SelectItem[personali.size()]));
		sisGroup.add(personaliG);
		List<SelectItem> pubblici = new ArrayList<SelectItem>();
		String t_pubblici = Message.addCatalogText("pubbliche");
		pubblici.add(new SelectItem(4, t_pubblici + ": " + Message.addCatalogText("disponibili")));
		pubblici.add(new SelectItem(5, t_pubblici + ": " + Message.addCatalogText("prossimi")));
		pubblici.add(new SelectItem(6, t_pubblici + ": " + Message.addCatalogText("tutti")));
		SelectItemGroup pubbliciG = new SelectItemGroup(t_pubblici); 
		pubbliciG.setSelectItems(pubblici.toArray(new SelectItem[pubblici.size()]));
		sisGroup.add(pubbliciG);
		return sisGroup;
	}
	public List<SelectItem> getTipoDashboardSI() {
		List<SelectItem> sisGroup = new ArrayList<SelectItem>();
		sisGroup.add(new SelectItem(-1, Message.addCatalogText("trattini")));
		sisGroup.add(new SelectItem(0, Message.addCatalogText("progresso")));
		List<SelectItem> personali = new ArrayList<SelectItem>();
		String t_personali = Message.addCatalogText("personali");
		personali.add(new SelectItem(5, t_personali + ": " + Message.addCatalogText("disponibili")));
		personali.add(new SelectItem(6, t_personali + ": " + Message.addCatalogText("prossimi")));
		personali.add(new SelectItem(4, t_personali + ": " + Message.addCatalogText("tutti")));
		SelectItemGroup personaliG = new SelectItemGroup(t_personali); 
		personaliG.setSelectItems(personali.toArray(new SelectItem[personali.size()]));
		sisGroup.add(personaliG);
		List<SelectItem> pubblici = new ArrayList<SelectItem>();
		String t_pubblici = Message.addCatalogText("pubbliche");
		pubblici.add(new SelectItem(8, t_pubblici + ": " + Message.addCatalogText("disponibili")));
		pubblici.add(new SelectItem(9, t_pubblici + ": " + Message.addCatalogText("prossimi")));
		pubblici.add(new SelectItem(7, t_pubblici + ": " + Message.addCatalogText("tutti")));
		SelectItemGroup pubbliciG = new SelectItemGroup(t_pubblici); 
		pubbliciG.setSelectItems(pubblici.toArray(new SelectItem[pubblici.size()]));
		sisGroup.add(pubbliciG);
		List<SelectItem> tutte = new ArrayList<SelectItem>();
		String t_tutte = Message.addCatalogText("tutte");
		tutte.add(new SelectItem(2, t_tutte + ": " + Message.addCatalogText("disponibili")));
		tutte.add(new SelectItem(3, t_tutte + ": " + Message.addCatalogText("prossimi")));
		tutte.add(new SelectItem(1, t_tutte + ": " + Message.addCatalogText("tutti")));
		SelectItemGroup tutteG = new SelectItemGroup(t_tutte); 
		tutteG.setSelectItems(tutte.toArray(new SelectItem[tutte.size()]));
		sisGroup.add(tutteG);
		return sisGroup;
	}
	public List<SelectItem> getEstrazioneSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem( 1, Message.addCatalogText("sequenziale")));
		sis.add(new SelectItem( 2, Message.addCatalogText("casuale")));
		sis.add(new SelectItem( 0, Message.addCatalogText("normale")));
		sis.add(new SelectItem(-1, Message.addCatalogText("statica")));
		return sis;
	}
	public List<SelectItem> getModalitaImportA() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(1, Message.addCatalogText("modifica")));
		sis.add(new SelectItem(2, Message.addCatalogText("elimina")));
		if(ApplicationUtil.isTrue(impostazioneDao.getRevert())) {
			sis.add(new SelectItem(3, Message.addCatalogText("ripristina")));
		}
		return sis;
	}
	public List<SelectItem> getModalitaImportU() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(1, Message.addCatalogText("modifica")));
		sis.add(new SelectItem(2, Message.addCatalogText("elimina")));
		if(ApplicationUtil.isTrue(impostazioneDao.getResetContatto())) {
			sis.add(new SelectItem(3, Message.addCatalogText("reset")));
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRevert())) {
			sis.add(new SelectItem(4, Message.addCatalogText("ripristina")));
		}
		sis.add(new SelectItem(5, Message.addCatalogText("sposta")));
		return sis;
	}
	public List<SelectItem> getModalitaImportS() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(1, Message.addCatalogText("modifica")));
		//sis.add(new SelectItem(2, Message.addCatalogText("elimina")));
		if(ApplicationUtil.isTrue(impostazioneDao.getRevert())) {
			sis.add(new SelectItem(3, Message.addCatalogText("ripristina")));
		}
		return sis;
	}
	public List<SelectItem> getSceltaTasti() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(0, Message.addCatalogText("no")));
		sis.add(new SelectItem(1, Message.addCatalogText("si")));
		sis.add(new SelectItem(2, Message.addCatalogText("sempre")));
		return sis;
	}
	public List<SelectItem> getPeriodo() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(1, Message.addCatalogText("giornaliero")));
		sis.add(new SelectItem(2, Message.addCatalogText("settimanale")));
		sis.add(new SelectItem(3, Message.addCatalogText("mensile")));
		sis.add(new SelectItem(4, Message.addCatalogText("annuale")));
		return sis;
	}
	public Integer getTipo_telefonico() {
		return ConstantsMap.TIPO_TELEFONICO;
	}
	public Integer getTipo_regulare() { 
		return ConstantsMap.TIPO_REGULARE;
	}
	public Integer getTipo_univoco() { 
		return ConstantsMap.TIPO_UNIVOCO;
	}
	public Integer getTipo_suggerimento() { 
		return ConstantsMap.TIPO_SUGGERIMENTO;
	}
	public Integer getTipo_combo() { 
		return ConstantsMap.TIPO_COMBOBOX;
	}
	public Integer getTipo_spinner() { 
		return ConstantsMap.TIPO_SPINNER;
	}
	public Integer getTipo_slider() { 
		return ConstantsMap.TIPO_SLIDER;
	}
	public Integer getTipo_intero() { 
		return ConstantsMap.TIPO_INTERO;
	}
	public Integer getTipo_file() { 
		return ConstantsMap.TIPO_FILE;
	}
	public Integer getTipo_numero() { 
		return ConstantsMap.TIPO_NUMERO;
	}
	public Integer getTipo_link() { 
		return ConstantsMap.TIPO_LINK;
	}
	public Integer getTipo_aperto() { 
		return ConstantsMap.TIPO_APERTO;
	}
	public Integer getTipo_form() { 
		return ConstantsMap.TIPO_FORM;
	}
	public Integer getTipo_binario() { 
		return ConstantsMap.TIPO_BINARIO;
	}
	public Integer getTipo_editor() { 
		return ConstantsMap.TIPO_EDITOR;
	}
	public Integer getTipo_inplaceInput() { 
		return ConstantsMap.TIPO_INPLACEINPUT;
	}
	public Integer getTipo_inplaceSelect() { 
		return ConstantsMap.TIPO_INPLACESELECT;
	}
	public Integer getTipo_color() { 
		return ConstantsMap.TIPO_COLOR;
	}
	public Integer getTipo_textarea() { 
		return ConstantsMap.TIPO_TEXTAREA;
	}
	public Integer getTipo_booleano() { 
		return ConstantsMap.TIPO_BOOLEANO;
	}
	public Integer getTipo_dataora() { 
		return ConstantsMap.TIPO_DATAORA;
	}
	public Integer getTipo_ora() { 
		return ConstantsMap.TIPO_ORA;
	}
	public Integer getTipo_data() { 
		return ConstantsMap.TIPO_DATA;
	}
	public Integer getTipo_oneMenu() { 
		return ConstantsMap.TIPO_ONEMENU;
	}
	public Integer getTipo_oneRadio() { 
		return ConstantsMap.TIPO_ONERADIO;
	}
	public Integer getTipo_manyCheckbox() { 
		return ConstantsMap.TIPO_MANYCHECKBOX;
	}
	public Integer getTipo_pickList() { 
		return ConstantsMap.TIPO_PICKLIST;
	}
	public Integer getTipo_manyMenu() {
		return ConstantsMap.TIPO_MANYMENU;
	}
	public GoogleAuthenticatorConfig getGoogleAuthenticatorConfig() {
		return googleAuthenticatorConfig;
	}
	public void afterPropertiesSet() throws Exception {
		if(ConstantsGlobal.TIMEALIVE == null) {
			ConstantsGlobal.TIMEALIVE = ApplicationUtil.date();
		}
		googleAuthenticatorConfig = new GoogleAuthenticatorConfigBuilder().setKeyRepresentation(KeyRepresentation.BASE32).setCodeDigits(ConstantsGlobal.cifre_otp).setTimeStepSizeInMillis(TimeUnit.SECONDS.toMillis(ConstantsGlobal.secondi_otp)).setWindowSize(5).build();
		ApplicationUtil.caricaService();
		loadFW(true);
		loadAvvio();
		String home = impostazioneDao.getHome();
		File fd = new File(home + ConstantsGlobal.path_modules);
		ApplicationUtil.creaFile(true, fd);
		File fs = getPathFile();
		ApplicationUtil.creaFile(true, fs);
		ApplicationUtil.cleanDirectory(fs); 
		File [] ldk = fd.listFiles();
		for(File f : ldk) {
			ApplicationUtil.copyFile(f, fs.getAbsolutePath() + "/" +  f.getName());
		}
		File logs = new File(ConstantsGlobal.path_log_callite);
		ApplicationUtil.creaFile(true, logs);
		File [] lck = logs.listFiles(new ModuloFileFilter(ConstantsMap.lck, false));
		for(File dd : lck) {
			dd.delete();
		}
		sipDao.sincronizzaInbound();
		new InfoSincroniaContattoThread(impostazioneDao).start();
		new InfoSincroniaAppuntamentoThread(impostazioneDao).start();
		new InfoSincroniaContrattoThread(impostazioneDao).start();
		new InfoSincroniaNotaThread(impostazioneDao).start();
		new InfoSincroniaPrenotazioneThread(impostazioneDao).start();
		ConstantsGlobal.PHONE = ApplicationUtil.checkPhone();
	}
	
	public void loadFW(boolean clear)  {
		boolean ok = ApplicationUtil.caricaFW();
		if(ok) {
			if(!clear) {
				List<Firewall> fw = new ArrayList<Firewall>(operazioneDao.utenteRules());
				operazioneDao.clearRules();
				for(Firewall r : fw) {
					operazioneDao.aggiungiPorta(r);
				}
			} else {
				operazioneDao.clearRules();
			}
			for(Rules r : getRules()) {
				operazioneDao.aggiungiPorta(r);
			}
			for(SipBuddies r : getConnessione()) {
				operazioneDao.aggiungiPorta(r);
			}
		}
	}
	public void loadAvvio()  {
		for(Avvio r : getAvvio()) {
			if(ApplicationUtil.isNotBlank(r.getCommand())) {
				ApplicationUtil.lanciaCommand(null, r.getCommand(), r.getWrites());
			}
			if(ApplicationUtil.isTrue(r.getCancellabile())) {
				cancellaAggiorna(r);
			}
		}
	}
	public void getCentralino(List<MyFile> mFiles, DownloadBean downloadBean) {
		try {
			List<Allegato> files = new ArrayList<Allegato>();
			if (ApplicationUtil.notEmpty(mFiles)) {
				String p = ConstantsGlobal.secretZ;
				Iterator<MyFile> itS = mFiles.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					ApplicationUtil.addFile(files, file.getName(), ConstantsGlobal.path_asterisk + "/" +file.getName(), file.getModified());
				}
				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.CENTRALINO.stampa()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);
				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();
			}
		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	public String readCentralino(MyFile ff) throws Exception {
		File file = new File(ConstantsGlobal.path_asterisk + "/" + ff.getName());
		if (file.exists()) {
			BufferedReader input = new BufferedReader(new FileReader(file));
			StringBuffer buffer = new StringBuffer();
			String text;
			while ((text = input.readLine()) != null)
				buffer.append(text + System.lineSeparator());
			input.close();
			return buffer.toString();
		}
		return null;
	}

	public void ricaricaCentralino() {
		autodialerDao.reloadModule(null);
	}
	
	public void salvaCentralinoS(String nome, String content) {
		boolean	change = ApplicationUtil.scriviFile(content, ConstantsGlobal.path_asterisk + "/" + nome);
		if(change) {
			autodialerDao.reloadModule(null);
		}
	}
	public List<MyFile> listaCentralino(FileCriteria mm) { 
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> sis = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			File perc = new File(ConstantsGlobal.path_asterisk); 
			if(perc.exists() && perc.isDirectory()) {
				File [] ls = perc.listFiles();
				if(ApplicationUtil.notEmpty(ls)) {
					List<String> rs = ApplicationUtil.token(mm.getFullText(), ConstantsGlobal.colonSeparator);
					for(File mod : ls) {
						if(mod.isFile() && mm.isRicerca(mod.getName(), mod.length(), mod.lastModified()) && ApplicationUtil.textInTXT(mod, rs)) {
							MyFile f = new MyFile(mod);
							f.setName(mod.getName());
							f.setModified(mod.lastModified());
							sis.add(f);
						}
					}
				}
			}
		}
		return sis;
	}
	public void eliminaCentralino(MyFile file) {
		ApplicationUtil.deleteFile(ConstantsGlobal.path_asterisk + "/" + file.getName());
		autodialerDao.reloadModule(null);
	}
	public void eliminaCentralini(List<MyFile> file) {
		if(ApplicationUtil.notEmpty(file)) {
			Iterator<MyFile> it = file.iterator();
			while (it.hasNext()) {
				MyFile myFile = (MyFile) it.next();
				ApplicationUtil.deleteFile(ConstantsGlobal.path_asterisk + "/" + myFile.getName());
			}
			autodialerDao.reloadModule(null);
		}
	}
	public void scaricaCentralino(MyFile file, DownloadBean downloadBean) {
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file.getName());
			InputStream stream = new FileInputStream(ConstantsGlobal.path_asterisk + "/" + file.getName());  
			ApplicationUtil.copy(stream, out);
			stream.close();
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaCentralino(MyFile ff, String content) {	
		boolean	change = ApplicationUtil.scriviFile(content, ConstantsGlobal.path_asterisk + "/" + ff.getName());
		if(change) {
			autodialerDao.reloadModule(null);
		}
	}
	public void salvaCentralino(FileUpload fileUpload) {
		List<MyFile> fls = fileUpload.getFiles();
		if(ApplicationUtil.notEmpty(fls)) {
			try {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					if(!ApplicationUtil.getExtensionLower(ff.getName()).equalsIgnoreCase(ConstantsMap.zip)) {
						if(ff.getName() != null) {
							File f = new File(ConstantsGlobal.path_asterisk + "/" + ff.getName());
							FileOutputStream fis = new FileOutputStream(f);
							fis.write(ff.getData());
							fis.flush();
							fis.close();
						}
					} else {
						ApplicationUtil.unzipA(ff.getData());
					}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
			autodialerDao.reloadModule(null);
		}
	}
	
	public void destroy() throws Exception {
	}
	
	public Object loadSingolo(Object o) {
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			Session session = openSession();
			session.refresh(o);
			closeSession(session);
		}
		return getSingolo(o);
	}
	public Object loadSingolo(String property, Object o) {
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			Session session = openSession();
			session.refresh(o);
			initialize(o, property);
			closeSession(session);
		}
		return getSingolo(o);
	}
	
	public void initialize(Object o, String property)  {
		if(ApplicationUtil.isNotBlank(property)) {
			try {
				Method m = o.getClass().getMethod("get" + ApplicationUtil.capitalize(property));
				Hibernate.initialize(m.invoke(o));
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	
	public Object getSingolo(Object o) {
		return o;
	}
	public List<?> loadMultiplo(List<?> lista) {
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			if(ApplicationUtil.notEmpty(lista)) {
				Session session = openSession();
				Iterator<?> objIter = lista.iterator();
				while(objIter.hasNext()) {
					Object o = objIter.next();
					session.refresh(o);
				}
				closeSession(session);
			}
		}
		return getMultiplo(lista);
	}
	public List<?> loadMultiplo(String property, List<?> lista) {
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			if(ApplicationUtil.notEmpty(lista)) {
				Session session = openSession();
				Iterator<?> objIter = lista.iterator();
				while(objIter.hasNext()) {
					Object o = objIter.next();
					session.refresh(o);
					initialize(o, property);
				}
				closeSession(session);
			}
		}
		return getMultiplo(lista);
	}
	public List<SelectItem> getAnagraficaTotale() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		List<String> as = ApplicationUtil.getAnagraficaTotale();
		for(String a :  as) {
			list.add(new SelectItem(a, Message.addCatalogText(a)));
		}
		return list;
	}
	public List<?> getMultiplo(List<?> lista) {
		return (List<?>) lista;
	}
	public void exceptionHandler(Object a, Object b) {
		ErrorLog.log(null, getClass(), a, b);
	}

	public List<LoggerLogin> getTentativiLogin() {
		return Chiamate.getCodaLogin().table();
	}
	public void removeTentativoLogin(LoggerLogin l) {
		Chiamate.getCodaLogin().remove(l);
	}
	public void removeTentativiLogin() {
		Chiamate.getCodaLogin().clear();
	}
	public void ricercaCoordinataC(String indirizzo, Campagna campagna) {
		Coordinata coord = null;
		try {
			String key = impostazioneDao.getKeyGeocodificaMap();
			coord = GCoder.geocode(ApplicationUtil.trimToEmpty(indirizzo), key);
		} catch (Exception e) {
		}
		if(coord != null) {
			campagna.setLatitudine(coord.getLatitudine());
			campagna.setLongitudine(coord.getLongitudine());
		}
	}
	public void ricercaCoordinataL(String indirizzo, Lista lista) {
		Coordinata coord = null;
		try {
			String key = impostazioneDao.getKeyGeocodificaMap();
			coord = GCoder.geocode(ApplicationUtil.trimToEmpty(indirizzo), key);
		} catch (Exception e) {
		}
		if(coord != null) {
			lista.setLatitudine(coord.getLatitudine());
			lista.setLongitudine(coord.getLongitudine());
		}
	}
	public Boolean getPermitInterval() {
		return ConstantsGlobal.permitInterval;
	}
	public void scaricaRegistrazione(Contatto contatto, MyFile rec) {
		String b = impostazioneDao.audioSupportato();
		String monitor = ConstantsGlobal.path_monitor;
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			try {
				String home = impostazioneDao.getHome();
				ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
				String n_name = ApplicationUtil.outFileRCodec(contatto != null ? contatto.getId() : rec.getId(), outputStream, home + monitor +"/"+ rec.getName(), b);
				outputStream.close();
				rec.setData(outputStream.toByteArray());
				rec.setName(n_name);
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
					String n_name = ApplicationUtil.outFileRCodec(vmftp, contatto != null ? contatto.getId() : rec.getId(), outputStream, monitor + "/" + rec.getName(), b);
					outputStream.close();
					rec.setData(outputStream.toByteArray());
					rec.setName(n_name);
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}

	public void uploadFile(Contatto contatto, MyFile ff, String name, Long clientId) {
		ClientFTP c = getValueFTP(clientId);
		if(c != null) {
			scaricaRegistrazione(contatto, ff);
			String nameN = ApplicationUtil.trimToNull(name);
			String baseNameN = (nameN != null ? nameN : ApplicationUtil.getBaseNameN(ff.getName()));
			ApplicationUtil.clientFTP(c,ff.getData(), baseNameN  + "." + ApplicationUtil.getExtensionLower(ff.getName()));
		}
	}
	public ClientFTP nuovoFTP() {
		return new ClientFTP();
	}
	public ClientFTP getValueFTP(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ClientFTP.class);
		criteria.add(Restrictions.eq("id", id));
		return (ClientFTP) getUniqueResult(criteria);
	}
	public void elimina(ClientFTP config) {
		cancellaAggiorna(config);
	}
	public ClientFTP salva(ClientFTP config) {
		return (ClientFTP) salvaAggiorna(config);
	}
	public List<SelectItem> getClientFTPs() {
		DetachedCriteria criteria = DetachedCriteria.forClass(ClientFTP.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	private ExtendedCriteria getConfigByCriteria(ClientFTPCriteria conf) {
		if(conf != null)  conf = (ClientFTPCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ClientFTP.class);
		if (ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, conf.getNome(), criteria, "nome");
		}
		conf.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ClientFTP> trova(ClientFTPCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("nome")});
		return getList(criteria, start, size);
	}
	public int trovaSize(ClientFTPCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public List<SelectItem> getRestSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsMap.r_type1));
		sis.add(new SelectItem(ConstantsMap.r_type2));
		sis.add(new SelectItem(ConstantsMap.r_type3));
		sis.add(new SelectItem(ConstantsMap.r_type4));
		sis.add(new SelectItem(ConstantsMap.r_type5));
		sis.add(new SelectItem(ConstantsMap.r_type6));
		sis.add(new SelectItem(ConstantsMap.r_type7));
		return sis;
	}
	public List<SelectItem> getSystemSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsMap.r_system1, Message.addCatalogText(ConstantsMap.r_system1)));
		sis.add(new SelectItem(ConstantsMap.r_system2, Message.addCatalogText(ConstantsMap.r_system2)));
		sis.add(new SelectItem(ConstantsMap.r_system3, Message.addCatalogText(ConstantsMap.r_system3)));
		sis.add(new SelectItem(ConstantsMap.r_system4, Message.addCatalogText(ConstantsMap.r_system4)));
		return sis;
	}
	public List<Label> getCampiAggiuntiviAnagraficaByChiave(String searchString, Long cid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
		if(ApplicationUtil.gtZero(cid)) {
			criteria.createCriteria("campo_aggiuntivo", "ca").add(Restrictions.idEq(cid));
		}
		ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, searchString, criteria, "valore");
		List<Label> lis = getLabelSI(criteria, "ca.chiave", "valore");
		return impostazioneDao.getLabel(lis, false);
	}
	public List<Label> getCampiAggiuntiviContrattoByChiave(String searchString, Long cid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DatoAggiuntivoContratto.class);
		if(ApplicationUtil.gtZero(cid)) {
			criteria.createCriteria("campo_aggiuntivo", "ca").add(Restrictions.idEq(cid));
		}
		ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, searchString, criteria, "valore");
		List<Label> lis = getLabelSI(criteria, "ca.chiave", "valore");
		return impostazioneDao.getLabel(lis, false);
	}
	public ClientAPI nuovoAPI() {
		return new ClientAPI();
	}
	public Invoice nuovoInvoice() {
		return new Invoice();
	}
	
	public Command nuovoCommand() {
		return new Command();
	}
	
	public ClientAPI getAPI(Long id) {
		if(id != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ClientAPI.class);
			criteria.add(Restrictions.eq("id", id));
			return (ClientAPI) getUniqueResult(criteria);
		}
		return null;
	}
	public ClientAPI getValueAPI(Long id) {
		ClientAPI c = null;
		if(id != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ClientAPI.class);
			criteria.add(Restrictions.eq("id", id));
			c = (ClientAPI) getUniqueResult(criteria);
			c.setLabels(ApplicationUtil.readText(c.getInputs()));
		}
		return c;
	}
	public String invia(ClientAPI c) {
		List<Label> labels = c.getLabels();
		if(ApplicationUtil.notEmpty(labels)) {
			Map<String, Label> inputs = ApplicationUtil.createMap(labels);
			c.setDriver(ApplicationUtil.replaceInString(c.getDriver(), inputs));
			c.setCommands(ApplicationUtil.replaceInString(c.getCommands(), inputs));
			c.setLink(ApplicationUtil.replaceInString(c.getLink(), inputs));
			c.setBody(ApplicationUtil.replaceInString(c.getBody(), inputs));
			c.setHeaders(ApplicationUtil.replaceInString(c.getHeaders(), inputs));
			c.setFields(ApplicationUtil.replaceInString(c.getFields(), inputs));
			c.setOutputs(ApplicationUtil.replaceInString(c.getOutputs(), inputs));
			c.setParameters(ApplicationUtil.replaceInString(c.getParameters(), inputs));
			c.setUsername(ApplicationUtil.replaceInString(c.getUsername(), inputs));
			c.setPassword(ApplicationUtil.replaceInString(c.getPassword(), inputs));
		}
		try {
			return InvocaAPI.invoke(c.getSistema(), c.getTipo(), c.getLink(), c.getHeaders(), c.getFields(), c.getOutputs(), c.getParameters(), c.getBody(), c.getUsername(), c.getPassword(), c.getDriver(), c.getCommands());
		} catch (Exception e) {
		}
		return null;
	}
	public void elimina(ClientAPI config) {
		cancellaAggiorna(config);
	}
	public ClientAPI salva(ClientAPI config) {
		return (ClientAPI) salvaAggiorna(config);
	}
	@SuppressWarnings("unchecked")
	public List<ClientAPI> getClientAPIs() {
		DetachedCriteria criteria = DetachedCriteria.forClass(ClientAPI.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	private ExtendedCriteria getConfigByCriteria(ClientAPICriteria conf) {
		if(conf != null)  conf = (ClientAPICriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ClientAPI.class);
		if (ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, conf.getNome(), criteria, "nome");
		}
		conf.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ClientAPI> trova(ClientAPICriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("nome")});
		return getList(criteria, start, size);
	}
	public int trovaSize(ClientAPICriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public List<SelectItem> getWeekSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		for (int i = 1; i <= 7; i++) {
			sis.add(new SelectItem((long) i, Message.addCatalogText("giorno." + i)));
		}
		return sis;
	}
	public Nota getNota(Utente utente, Nota o) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.NOTA.stampa(), o);
		operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_NOTA);
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			return (Nota) get(Nota.class, o.getId());
		}
		return o;
	}
	public Prenotazione getPrenotazione(Utente utente, Prenotazione o) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.PRENOTAZIONE.stampa(), o);
		operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_PRENOTAZIONE);
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			return (Prenotazione) get(Prenotazione.class, o.getId());
		}
		return o;
	}
	public Appuntamento getAppuntamento(Utente utente, Appuntamento o) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.APPUNTAMENTO.stampa(), o);
		operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_APPUNTAMENTO);
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			return (Appuntamento) get(Appuntamento.class, o.getId());
		}
		return o;
	}
	public Contratto getContratto(Utente utente, Contratto o) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTRATTO.stampa(), o);
		operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_CONTRATTO);
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			return (Contratto) get(Contratto.class, o.getId());
		}
		return o;
	}
	public Contatto getContatto(Utente utente, Contatto o) {
		String pg = ApplicationUtil.asString(ConstantsLanguage.CONTATTO.stampa(), o);
		operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_CONTATTO);
		if(ApplicationUtil.isTrue(impostazioneDao.getSincronia())) {
			return (Contatto) get(Contatto.class, o.getId());
		}
		return o;
	}
	public void esitoAppuntamento(Utente utente, Appuntamento o) {
		//String pg = ApplicationUtil.asString(ConstantsLanguage.APPUNTAMENTO.stampa(), o);
		//operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_APPUNTAMENTO);
	}
	public void esitoContratto(Utente utente, Contratto o) {
		//String pg = ApplicationUtil.asString(ConstantsLanguage.CONTRATTO.stampa(), o);
		//operazioneDao.creaOperazione(ConstantsMap.LOG7, pg ,utente, ConstantsMap.OPERAZIONE_ELEMENTO_CONTRATTO);
	}
	public Rules nuovoRules() {
		return new Rules();
	}
	public void elimina(Rules config) {
		cancellaAggiorna(config);
		ricaricaRules();
	}
	public Rules salva(Rules config) {
		Rules r = (Rules) salvaAggiorna(config);
		ricaricaRules();
		return r;
	}
	
	public void  eliminaRulesSelezionati(List<Rules> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<Rules> itR = lst.iterator();
			while (itR.hasNext()) {
				Rules rules = (Rules) itR.next();
				cancellaAggiorna(rules);
			}
		}
		ricaricaRules();
	}
	
	
	@SuppressWarnings("unchecked")
	public List<SipBuddies> getConnessione() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("firewall",  ApplicationUtil.convertBS(true)));
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(false)));
		return getList(criteria);
	}
	
	
	@SuppressWarnings("unchecked")
	public List<Rules> getRules() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Rules.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<SipBuddies> getInterno() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<SipBuddies> getTrunks() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(false)));
		return getList(criteria);
	}
	
	private ExtendedCriteria getConfigByCriteria(RulesCriteria conf) {
		if(conf != null)  conf = (RulesCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Rules.class);
		if (ApplicationUtil.isNotBlank(conf.getIp())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, conf.getIp(), criteria, "ip");
		}
		if (ApplicationUtil.isNotBlank(conf.getService())) {
			criteria.add(Restrictions.eq("service", conf.getService()));
		}
		if (ApplicationUtil.isNotBlank(conf.getProtocol())) {
			criteria.add(Restrictions.eq("protocol", conf.getProtocol()));
		}
		conf.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	public boolean validaRules(Rules rl) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Rules.class);
		criteria.add(Restrictions.eq("ip", rl.getIp()));
		if(ApplicationUtil.isBlank(rl.getService())) {
			criteria.add(Restrictions.eq("protocol", rl.getProtocol()));
			criteria.add(Restrictions.eq("porta", rl.getPorta()));
		} else {	
			criteria.add(Restrictions.eq("service", rl.getService()));
		}
		criteria.setProjection(Projections.distinct(Projections.property("id").as("id")));
		Long id =  (Long) getUniqueResult(criteria);
		if(id != null) {
			if(rl.getId() == null) {
				return false;
			} else {
				if(!ApplicationUtil.eq(rl.getId(), id)) {
					return false;
				}
			}
		}
		return true;
	}
	
	@SuppressWarnings("unchecked")
	public List<Rules> trova(RulesCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("ip")});
		return getList(criteria, start, size);
	}
	public int trovaSize(RulesCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	
	
	public List<SelectItem> getServices() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(String s : ConstantsGlobal.SERVICE) {
			if(ApplicationUtil.getService(s)) {
				sis.add(new SelectItem(s,s));
			}
		}
		return sis;
	}
	public List<SelectItem> getProtocols() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.tcp));
		sis.add(new SelectItem(ConstantsGlobal.udp));
		return sis;
	}

	public Avvio nuovoAvvio() {
		return new Avvio();
	}
	public void elimina(Avvio config) {
		cancellaAggiorna(config);
	}
	public Avvio salva(Avvio config) {
		return (Avvio) salvaAggiorna(config);
	}
	@SuppressWarnings("unchecked")
	public List<Avvio> getAvvio() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Avvio.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		return getList(criteria);
	}
	private ExtendedCriteria getConfigByCriteria(AvvioCriteria conf) {
		if(conf != null)  conf = (AvvioCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Avvio.class);
		if (ApplicationUtil.isNotBlank(conf.getCommand())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, conf.getCommand(), criteria, "command");
		}
		conf.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<Avvio> trova(AvvioCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(AvvioCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public void  eliminaAvvioSelezionati(List<Avvio> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<Avvio> itR = lst.iterator();
			while (itR.hasNext()) {
				Avvio avvio = (Avvio) itR.next();
				elimina(avvio);
			}
		}
	}

	public List<SelectItem> getTipiNotifica() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(1, ConstantsLanguage.NOTIFICA0.stampa()));
		sis.add(new SelectItem(0, ConstantsLanguage.NOTIFICA1.stampa()));
		return sis;
	}

	public List<SelectItem> getMessaggistica() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(0, ConstantsLanguage.NO.stampa()));
		sis.add(new SelectItem(1, ConstantsLanguage.MENU.stampa()));
		sis.add(new SelectItem(2, ConstantsLanguage.TUTTO.stampa()));
		return sis;
	}
	public List<SelectItem> getContextSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsMap.context_u));
		sis.add(new SelectItem(ConstantsMap.context_a));
		sis.add(new SelectItem(ConstantsMap.context_b));
		return sis;
	}

	public List<SelectItem> getApplicazioni() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(String s : ConstantsMap.getLastapp()) {
			sis.add(new SelectItem(s));
		}
		return sis;
	}

	public boolean isValid() {
		return !ApplicationUtil.isWindows();
	}

	public String getDaccapo() {
		return ConstantsGlobal.RETURN;
	}

	public List<SelectItem> getTemplateSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		List<String> files = getTemplateFile();
		if (ApplicationUtil.notEmpty(files)) {
			Collections.sort(files);
			for(String file : files) {
				sis.add(new SelectItem(ApplicationUtil.getBaseNameN(file)));
			}
		}
		return sis;
	}
	
	
	public String getTemplateByName(String name) {
		try {
			String path = servletContext.getRealPath("/WEB-INF/classes" + ConstantsGlobal.path_template + "/" + name + "." + ConstantsMap.html);
			if(path != null) {
				FileSystemResource resource = new FileSystemResource(path);
				if(resource != null) {
					File file = resource.getFile();
					if (file.exists()) {
						BufferedReader input = new BufferedReader(new FileReader(file));
						StringBuffer buffer = new StringBuffer();
						String text;
						while ((text = input.readLine()) != null) {
							buffer.append(text + System.lineSeparator());
						}
						input.close();
						return buffer.toString();
					}
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return null;
	}

	public List<String> getTemplateFile() {
		String path = servletContext.getRealPath("/WEB-INF/classes" + ConstantsGlobal.path_template);
		FileSystemResource resource=new FileSystemResource(path);
		File file = resource.getFile();
		return file != null ? ApplicationUtil.asList(resource.getFile().list()) : null;
	}

	public List<LabelQuery> query(String proprieta, String alias, String tabella, boolean order, Integer limit, QueryUtils cv)  {
		if(proprieta != null && tabella != null) {
			String query = ApplicationUtil.trimToNull(cv.getQuery());
			List<Parameter> lines = cv.getLines();
			if(alias == null) alias = proprieta;
			String condition = "";
			String limite = "";
			if(query != null) {
				condition = " where " + query;
			}
			if(limit != null) {
				limite = " limit " + limit;
			}
			String ql =  ConstantsGlobal.select + " distinct concat(" + proprieta + ") as value, concat(" + alias + ") as label from " + tabella + condition + " order by label " + (order ? "asc" : "desc") + limite;
			Parameter [] pl = (lines != null && query != null) ? lines.toArray(new Parameter[lines.size()]) : null;
			return list(ql, pl);
		}
		return new ArrayList<LabelQuery>();
	}
	
	public List<LabelQuery> query(String gruppo, String valore, String quantity, String tabella, boolean order, Integer limit, QueryUtils cv)  {
		if(quantity != null && tabella != null && (gruppo != null || valore != null)) {
			String query = ApplicationUtil.trimToNull(cv.getQuery());
			List<Parameter> lines = cv.getLines();
			if(gruppo == null) {
				gruppo = "''";
			}
			if(valore == null) {
				valore = "''";
			}
			String limite = "";
			String condition = "";
			if(query != null) {
				condition = " where " + query;
			}
			if(limit != null) {
				limite = " limit " + limit;
			}
			String ql =  ConstantsGlobal.select + " distinct concat(" + gruppo + ") as value, concat(" + valore + ") as label, " + quantity + " as quantity from " + tabella + condition + " group by value, label order by value " + (order ? "asc" : "desc") + ", label " + (order ? "asc" : "desc") + limite;
			Parameter [] pl = (lines != null && query != null) ? lines.toArray(new Parameter[lines.size()]) : null;
			return list(ql, pl);
		}
		return new ArrayList<LabelQuery>();
	}

	@SuppressWarnings("unchecked")
	public List<QueryList> queryL(QueryUtils cv)  {
		List<QueryList> qk = new ArrayList<QueryList>();
		String query = ApplicationUtil.trimToNull(cv.getQuery());
		if(query != null) {
			List<Parameter> lines = cv.getLines();
			Parameter [] pl = (lines != null) ? lines.toArray(new Parameter[lines.size()]) : null;
			String test = ApplicationUtil.lowerCase(query);
			if(test.startsWith(ConstantsGlobal.select) || test.startsWith(ConstantsGlobal.show)) {
				List<Object[]> pls = list(true, query, pl);
				if(ApplicationUtil.notEmpty(pls)) {
					for(Object[] colonna : pls) {
						List<QueryElement> valori = new ArrayList<QueryElement>();
						if(ApplicationUtil.notEmpty(colonna)) {
							for(Object riga : colonna) {
								valori.add(new QueryElement(riga));
							}
						}
						QueryList qlst = new QueryList(valori);
						qk.add(qlst);
					}
				}
			} else {
				execute(true, query, pl);
			}
		}
		return qk;
	}

	public StatoIntervallo getStatoIntervallo() {
		return  new StatoIntervallo();
	}


	public void deleteOperazione(Integer intervallo) {
		deleteIntervallo(intervallo, Operazione.class, "data_creazione");
	}

	public void deletePausa(Integer intervallo) {
		deleteIntervallo(intervallo, Pausa.class, "data_creazione");
	}

	public void deleteMarcatura(Integer intervallo) {
		deleteIntervallo(intervallo, Marcatura.class, "data_creazione");
	}

	public void deleteAccesso(Integer intervallo) {
		deleteIntervallo(intervallo, Accesso.class, "data_creazione");
	}
	public void deleteAutoContatto(Integer intervallo) {
		deleteIntervallo(intervallo, AutoContatto.class, "data_creazione");
	}
	
	public void deleteEmail(Integer intervallo) {
		deleteIntervallo(intervallo, Email.class, "data_creazione");
	}
	public void deleteNotifica(Integer intervallo) {
		deleteIntervallo(intervallo, Notifica.class, "data_creazione");
	}
	public void deleteSondaggio(Integer intervallo) {
		deleteIntervallo(intervallo, Sondaggio.class, "data_creazione");
	}
	public void deleteComunicazione(Integer intervallo) {
		deleteIntervallo(intervallo, Comunicazione.class, "data_creazione");
	}

	public void deleteRmail(Integer intervallo) {
		deleteIntervallo(intervallo, Rmail.class, "data_creazione");
	}
	
	public void deleteWebhook(Integer intervallo) {
		deleteIntervallo(intervallo, Webhook.class, "data_creazione");
	}
	public void deleteRicezione(Integer intervallo) {
		deleteIntervallo(intervallo, Ricezione.class, "data_creazione");
	}
	
	public void deleteNotizia(Integer intervallo) {
		deleteIntervallo(intervallo, Notizia.class, "data_creazione");
	}
	public void deleteMessaggio(Integer intervallo) {
		deleteIntervallo(intervallo, Messaggio.class, "data_creazione");
	}
	public void deleteTelecom(Integer intervallo) {
		deleteIntervallo(intervallo, Telecom.class, "data_creazione");
	}
	public void deleteCdr(Integer intervallo) {
		deleteIntervallo(intervallo, Cdr.class, "calldate");
	}
	public void deleteCel(Integer intervallo) {
		deleteIntervallo(intervallo, Cel.class, "eventtime");
	}
	public void deleteStoriaContatto(Integer intervallo) {
		deleteIntervallo(intervallo, StoriaContatto.class, "data");
	}
	public void deleteStoriaContratto(Integer intervallo) {
		deleteIntervallo(intervallo, StoriaContratto.class, "data");
	}
	public void deleteStoriaAppuntamento(Integer intervallo) {
		deleteIntervallo(intervallo, StoriaAppuntamento.class, "data");
	}
	public void deleteStoriaNota(Integer intervallo) {
		deleteIntervallo(intervallo, StoriaNota.class, "data");
	}
	public void deleteStoriaPrenotazione(Integer intervallo) {
		deleteIntervallo(intervallo, StoriaPrenotazione.class, "data");
	}
	public void deleteAllegatoContatto(Integer intervallo) {
		deleteIntervallo(intervallo, AllegatoContatto.class, "data");
	}
	public void deleteAllegatoContratto(Integer intervallo) {
		deleteIntervallo(intervallo, AllegatoContratto.class, "data");
	}
	public void deleteAllegatoAppuntamento(Integer intervallo) {
		deleteIntervallo(intervallo, AllegatoAppuntamento.class, "data");
	}
	public void deleteAllegatoAnagrafica(Integer intervallo) {
		deleteIntervallo(intervallo, AllegatoAnagrafica.class, "data");
	}
	public void deleteAllegatoNotizia(Integer intervallo) {
		deleteIntervallo(intervallo, AllegatoNotizia.class, "data");
	}
	public void deleteAllegatoAgente(Integer intervallo) {
		deleteIntervallo(intervallo, AllegatoAgente.class, "data");
	}
	
	public void reloadCalendar() {
		new RicaricaCalendarioThread(this).start();
	}
	public void ricaricaCalendar() {
		StringBuffer sb = new StringBuffer();
		for(Utente r : userDao.getUtentiCalendar()) {
			if(r.isLocale() && ApplicationUtil.isTrue(r.getPromemoria())) {
				sb.append(downloadDao.calendarToHTMLString(r));
				sb.append(System.lineSeparator());
			}
		}
		boolean	change = ApplicationUtil.scriviFile(sb.toString(), ConstantsGlobal.path_calendars);
		if(change) {
			autodialerDao.reloadModule(null);
		}
	}

	public void ricaricaInterno() {
		StringBuffer sb = new StringBuffer();
		int rt = impostazioneDao.getRealtime();
		if(rt == 0) {
			for(SipBuddies r : getInterno()) {
				sb.append(downloadDao.internoToHTMLString(r));
				sb.append(System.lineSeparator());
			}
		}
		boolean	change = ApplicationUtil.scriviFile(sb.toString(), ConstantsGlobal.path_interni);
		if(change || rt == 1) {
			autodialerDao.reloadModule(null);
		}
	}
	public void ricaricaBuddy() {
		StringBuffer sb = new StringBuffer();
		if(ApplicationUtil.isTrue(impostazioneDao.chatCentralino())) {
			UtilChat uc = impostazioneDao.getUtilChat();
			for(SipBuddies r : getInterno()) {
				sb.append(downloadDao.buddyToHTMLString(r, uc.getHost(), uc.getPort()));
				sb.append(System.lineSeparator());
			}
		}
		boolean	change = ApplicationUtil.scriviFile(sb.toString(), ConstantsGlobal.path_buddy);
		if(change) {
			autodialerDao.reloadModule(null);
		}
	}

	public void ricaricaTrunk() {
		StringBuffer sb = new StringBuffer();
		int rt = impostazioneDao.getRealtime();
		if(rt == 0) {
			for(SipBuddies r : getTrunks()) {
				sb.append(downloadDao.trunkToHTMLString(r));
				sb.append(System.lineSeparator());
			}
		}
		boolean	change = ApplicationUtil.scriviFile(sb.toString(), ConstantsGlobal.path_trunks);
		if(change || rt == 1) {
			autodialerDao.reloadModule(null);
		}
		if(change || rt > 0) {
			ricaricaRules();
		}
	}

	
	public void ricaricaPeer() {
		StringBuffer sb1 = new StringBuffer();
		StringBuffer sb2 = new StringBuffer();
		StringBuffer sb3 = new StringBuffer();
		int rt = impostazioneDao.getRealtime();
		if(rt == 0) {
			for(SipBuddies r : getInterno()) {
				sb1.append(downloadDao.internoToHTMLString(r));
				sb1.append(System.lineSeparator());
			}
			for(SipBuddies r : getTrunks()) {
				sb2.append(downloadDao.trunkToHTMLString(r));
				sb2.append(System.lineSeparator());
			}
		}
		if(ApplicationUtil.isTrue(impostazioneDao.chatCentralino())) {
			UtilChat uc = impostazioneDao.getUtilChat();
			for(SipBuddies r : getInterno()) {
				sb3.append(downloadDao.buddyToHTMLString(r, uc.getHost(), uc.getPort()));
				sb3.append(System.lineSeparator());
			}
		}
		boolean	change1 = ApplicationUtil.scriviFile(sb1.toString(), ConstantsGlobal.path_interni);
		boolean	change2 = ApplicationUtil.scriviFile(sb2.toString(), ConstantsGlobal.path_trunks);
		boolean	change3 = ApplicationUtil.scriviFile(sb3.toString(), ConstantsGlobal.path_buddy);
		if(change1 || change2 || change3 || rt == 1) {
			autodialerDao.reloadModule(null);
		}
	}

	public void ricaricaRules() {
		loadFW(false);
	}
	
	public Anagrafica getAnagrafica() {
		return new Anagrafica();
	}
	
	public Contatto getContatto() {
		return new Contatto();
	}
	
	public Prodotto getProdotto() {
		return new Prodotto();
	}
	
	public List<SelectItem> getStrategy() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem("rrmemory"));
		sis.add(new SelectItem("ringall"));
		sis.add(new SelectItem("roundrobin"));
		sis.add(new SelectItem("leastrecent"));
		sis.add(new SelectItem("fewestcalls"));
		sis.add(new SelectItem("random"));
		return sis;
	}
	
	
	public List<SelectItem> getTipoRmail() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(ConstantsGlobal.a_sms, ConstantsLanguage.SMS.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.a_fax, ConstantsLanguage.FAX.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.a_lettera, ConstantsLanguage.LETTERA.stampa()));
		sis.add(new SelectItem(ConstantsGlobal.a_telegramma, ConstantsLanguage.TELEGRAMMA.stampa()));
		return sis;
	}
	
	
	public Map<Long, ConteggioModel> getConteggio1(Boolean esegui, List<Long> ids, String proprieta, Class<?> classe) {
		if(ApplicationUtil.isTrue(esegui)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(classe, "cont");
			criteria.createCriteria(proprieta, "st");
			criteria.add(Restrictions.in("id", ids));
			return evaluateConteggio(criteria, "cont.id", "st.id");
		}
		return new HashMap<Long, ConteggioModel>();
	}
	
	public Map<Long, ConteggioModel> getConteggio2(Boolean esegui, List<Long> ids, String proprieta, Class<?> classe) {
		if(ApplicationUtil.isTrue(esegui)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(classe, "cont");
			criteria.add(Restrictions.in(proprieta + "." + "id", ids));
			return evaluateConteggio(criteria, "cont" + "." + proprieta + "." + "id", "cont.id");
		}
		return new HashMap<Long, ConteggioModel>();
	}

	
	public Map<Long, ConteggioModel> evaluateConteggio(DetachedCriteria criteria, String p1, String p2) {
		Map<Long, ConteggioModel> map = new HashMap<Long, ConteggioModel>();
		List<ConteggioModel> l = getListConteggio(criteria, p1, p2);
		if(ApplicationUtil.notEmpty(l))  {
			Iterator<ConteggioModel> it = l.iterator();
			while (it.hasNext()) {
				ConteggioModel c = (ConteggioModel) it.next();
				map.put(c.getId(), c);
			}
		}
		return map;
	}
	
	@SuppressWarnings({ "unchecked" })
	public Map<Long, ColonnaModel> getConteggioColonna(GenericColumn cc, List<Long> ids, Class<?> classe, ColonnaQuery ... columns) {
		Map<Long, ColonnaModel> map = new HashMap<Long, ColonnaModel>();
		boolean esegui = false;
		if(ApplicationUtil.notEmpty(columns)) {
			for(ColonnaQuery column : columns) {
				esegui = esegui || (cc != null ? cc.getColonna(column.getColonna()) : true);
			}
		}
		if(esegui) {
			String main = "m" + classe.getSimpleName();
			DetachedCriteria criteria = DetachedCriteria.forClass(classe, main);
			criteria.add(Restrictions.in("id", ids));
			ProjectionList p = Projections.projectionList();
			p.add(Projections.distinct(Projections.groupProperty(main+".id").as("id")));
			for(ColonnaQuery column : columns) {
				if(cc != null ? cc.getColonna(column.getColonna()) : true) {
					String alias = column.getColonna();
					String table = column.getTabella();
					String altro = column.getAltro();
					if(altro == null) {
						ApplicationUtil.projectionList(p, criteria, table, alias, "id");
					} else {
						ApplicationUtil.projectionList(true, p, criteria, table, alias, "id", altro);
					}
					
				}
			}
			criteria.setProjection(p);
			List<ColonnaModel> l = getList(criteria, ColonnaModel.class);
			if(ApplicationUtil.notEmpty(l))  {
				Iterator<ColonnaModel> it = l.iterator();
				while (it.hasNext()) {
					ColonnaModel c = (ColonnaModel) it.next();
					map.put(c.getId(), c);
				}
			}
		}
		return map;
	}


	
	
}	