package it.callite.dao;
import java.io.ByteArrayOutputStream;
import java.util.Arrays;
import java.util.List;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Anagrafica;
import it.callite.model.anagrafica.AnagraficaXML;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("esportaDao")
public class EsportaDao extends GenericDao implements InitializingBean {
	@Autowired
	private AnagraficaDao anagraficaDao;
	
	private static EsportaDao instance;

	@Override
	public void afterPropertiesSet() throws Exception {
		instance = this;
	}

	public static EsportaDao get() {
		return instance;
	}
	
	
	public String inviaAnagrafica(String url, Long ... id) {
		String u = ApplicationUtil.trimToNull(url);
		if(u != null) {
			boolean xml = u.endsWith(ConstantsMap.xml);
			boolean json = u.endsWith(ConstantsMap.json);
			if(xml || json) {
				String content = null;
				String body = null;
				if(xml) {
					content = ConstantsGlobal.contentXml;
					body = preparaAnagraficaXML(id);
				} 
				if(json) {
					content = ConstantsGlobal.contentJson;
					body = preparaAnagraficaJson(id);
				}
				if(content != null && body != null) {
					try {
						HttpResponse<String> response = Unirest.post(u)
								.header("Content-Type", content)
								.header("cache-control", "no-cache")
								.body(body)
								.asString();
						return ApplicationUtil.trimToNull(response.getBody());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		}
		return null;
	}
	
	
	public String inviaContatto(String url, Long ... id) {
		String u = ApplicationUtil.trimToNull(url);
		if(u != null) {
			boolean xml = u.endsWith(ConstantsMap.xml);
			boolean json = u.endsWith(ConstantsMap.json);
			if(xml || json) {
				String content = null;
				String body = null;
				if(xml) {
					content = ConstantsGlobal.contentXml;
					body = preparaContattoXML(id);
				} 
				if(json) {
					content = ConstantsGlobal.contentJson;
					body = preparaContattoJson(id);
				}
				if(content != null && body != null) {
					try {
						HttpResponse<String> response = Unirest.post(u)
								.header("Content-Type", content)
								.header("cache-control", "no-cache")
								.body(body)
								.asString();
						return ApplicationUtil.trimToNull(response.getBody());
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		}
		return null;
	}
	
	public String preparaAnagraficaXML(Long ... id) {
		List<Long> ids = Arrays.asList(id);
		if(ApplicationUtil.notEmpty(ids)) {
			List<Anagrafica> anagrafiche = anagraficaDao.getAnagraficheById(ids);
			if(ApplicationUtil.notEmpty(anagrafiche)) {
				try {
					List<AnagraficaXML> anagraficheXML = anagraficaDao.convertToAnagrafiche(anagrafiche);
					ByteArrayOutputStream result = new ByteArrayOutputStream();
					ApplicationUtil.xmlEncoder(anagraficheXML, result);
					result.close();
					return ApplicationUtil.trimToNull(new String(result.toByteArray(), ConstantsGlobal.CHARSET));
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		return null;
	}

	public String preparaAnagraficaJson(Long ... id) {
		List<Long> ids = Arrays.asList(id);
		if(ApplicationUtil.notEmpty(ids)) {
			List<Anagrafica> anagrafiche = anagraficaDao.getAnagraficheById(ids);
			if(ApplicationUtil.notEmpty(anagrafiche)) {
				try {
					List<AnagraficaXML> anagraficheXML = anagraficaDao.convertToAnagrafiche(anagrafiche);
					ByteArrayOutputStream result = new ByteArrayOutputStream();
					ApplicationUtil.jsonEncoder(anagraficheXML, result);
					result.close();
					return ApplicationUtil.trimToNull(new String(result.toByteArray(), ConstantsGlobal.CHARSET));
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		return null;
	}

	
	public String preparaContattoXML(Long ... id) {
		List<Long> ids = Arrays.asList(id);
		if(ApplicationUtil.notEmpty(ids)) {
			List<Anagrafica> anagrafiche = anagraficaDao.getAnagraficheByContattoId(ids);
			if(ApplicationUtil.notEmpty(anagrafiche)) {
				try {
					List<AnagraficaXML> anagraficheXML = anagraficaDao.convertToAnagrafiche(anagrafiche);
					ByteArrayOutputStream result = new ByteArrayOutputStream();
					ApplicationUtil.xmlEncoder(anagraficheXML, result);
					result.close();
					return ApplicationUtil.trimToNull(new String(result.toByteArray(), ConstantsGlobal.CHARSET));
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		return null;
	}

	public String preparaContattoJson(Long ... id) {
		List<Long> ids = Arrays.asList(id);
		if(ApplicationUtil.notEmpty(ids)) {
			List<Anagrafica> anagrafiche = anagraficaDao.getAnagraficheByContattoId(ids);
			if(ApplicationUtil.notEmpty(anagrafiche)) {
				try {
					List<AnagraficaXML> anagraficheXML = anagraficaDao.convertToAnagrafiche(anagrafiche);
					ByteArrayOutputStream result = new ByteArrayOutputStream();
					ApplicationUtil.jsonEncoder(anagraficheXML, result);
					result.close();
					return ApplicationUtil.trimToNull(new String(result.toByteArray(), ConstantsGlobal.CHARSET));
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		}
		return null;
	}

}