package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projection;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.criterion.Subqueries;
import org.hibernate.type.BigDecimalType;
import org.hibernate.type.Type;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.AccontoCriteria;
import it.callite.criteria.EstrattoCriteria;
import it.callite.criteria.FatturaContrattoCriteria;
import it.callite.criteria.FatturaEstrattoCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Acconto;
import it.callite.model.Appuntamento;
import it.callite.model.Categoria;
import it.callite.model.Contest;
import it.callite.model.Contratto;
import it.callite.model.Estratto;
import it.callite.model.EstrattoDettaglio;
import it.callite.model.FatturaContratto;
import it.callite.model.FatturaEstratto;
import it.callite.model.Gettone;
import it.callite.model.Prodotto;
import it.callite.model.Qualifica;
import it.callite.model.Utente;
import it.callite.model.report.ContestModel;
import it.callite.model.report.EstrattoModel;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
import it.callite.utility.item.GenericSI;
@SuppressWarnings("serial")
@Repository("estrattoDao")
public class EstrattoDao extends GenericDao {
	@Autowired
	private UserDao userDao;
	@Autowired
	private ContrattoDao contrattoDao;
	@Autowired
	private PannelloDao pannelloDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	public Estratto getEstratto(Long id) {
		return (Estratto) get(Estratto.class, id);
	}
	public Acconto getAcconto(Long id) {
		return (Acconto) get(Acconto.class, id);
	}
	public List<SelectItem> getFatturaEstrattoSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(FatturaEstratto.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	public List<SelectItem> getFatturaContrattoSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(FatturaContratto.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getTemplateByIdUtente(Long fatturaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.eq("fattura.id", fatturaId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public ExtendedCriteria trovaFatturaEstrattoCriteria(FatturaEstrattoCriteria ema) {
		if(ema != null)  ema = (FatturaEstrattoCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(FatturaEstratto.class);
		if(ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ema.getNomeOggetto(), criteria, "nome");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<FatturaEstratto> trovaFatturaEstratto(FatturaEstrattoCriteria ema, int start, int size) {
		ExtendedCriteria criteria = trovaFatturaEstrattoCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaFatturaEstrattoSize(FatturaEstrattoCriteria ema) {
		ExtendedCriteria criteria = trovaFatturaEstrattoCriteria(ema);
		return getListSize(criteria);
	}
	public void getFatturaEstratto(FatturaEstratto fattura, DownloadBean downloadBean) {


		OutputStream out = downloadBean.outputStream();
		downloadBean.filename(fattura.getNomeFileS());
		try {
			InputStream bais = new ByteArrayInputStream(fattura.getContent());

			ApplicationUtil.copy(bais, out);
			downloadBean.close();
			bais.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public FatturaEstratto getFatturaEstratto(Long id) {
		return (FatturaEstratto) get(FatturaEstratto.class, id);
	}
	public void eliminaFatturaEstratto(FatturaEstratto fattura) {
		cancellaAggiorna(fattura);
	}
	public FatturaEstratto nuovoFatturaEstratto() {
		return new FatturaEstratto();
	}
	public void inserisciFileInFatturaEstratto(FileUpload fileUpload, FatturaEstratto fattura) {
		List<MyFile> fls = fileUpload.getFiles();
		if(ApplicationUtil.notEmpty(fls)) {
			try { 
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					fattura.setNomeFile(ApplicationUtil.getNameN(ff.getName()));
					fattura.setContent(ff.getData());
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public FatturaEstratto salvaFatturaEstratto(FatturaEstratto fattura) {
		if (fattura != null) {
			fattura = (FatturaEstratto) salvaAggiorna(fattura);
		}
		return fattura;
	}
	private ExtendedCriteria trovaCriteria(AccontoCriteria ana) {
		if(ana != null)  ana = (AccontoCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Acconto.class);
		DetachedCriteria criteriau =  criteria.createCriteria("user");
		if (ApplicationUtil.isNotBlank(ana.getUser())) {
			ApplicationUtil.operatore(criteriau, ana.getUser());
		}
		if (ApplicationUtil.gtZero(ana.getTipo())) {
			criteria.add(Restrictions.eq("tipo", ana.getTipo()));
		}
		if (ApplicationUtil.isNotBlank(ana.getDescrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getDescrizione(), criteria, "descrizione");
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	private ExtendedCriteria trovaCriteria(Utente user, AccontoCriteria ana) {
		if(ana != null)  ana = (AccontoCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Acconto.class);
		criteria.createCriteria("user").add(Restrictions.idEq(user.getId()));
		//criteria.add(Restrictions.or(Restrictions.eq("tipo", ConstantsMap.tipo_acconto1), Restrictions.eq("tipo", ConstantsMap.tipo_acconto3)));
		if (ApplicationUtil.isNotBlank(ana.getDescrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getDescrizione(), criteria, "descrizione");
		}
		if (ApplicationUtil.gtZero(ana.getTipo())) {
			criteria.add(Restrictions.eq("tipo", ana.getTipo()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Acconto> trova(AccontoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(AccontoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public BigDecimal trovaSum(AccontoCriteria ana) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = trovaCriteria(ana);
			return getSommaDecimale(criteria , "importo");
		} else {
			return new BigDecimal(0);
		}
	}
	public BigDecimal sumEstratto(ExtendedCriteria criteria) {
		criteria.getCriteria().createCriteria("estrattoDettagli");
		criteria.getCriteria().setProjection(Projections.sqlProjection("sum(valore * gettone) as somma", new String [] {"somma"}, new Type []{BigDecimalType.INSTANCE}));
		DetachedCriteria criteriaS = criteria.getCriteria();
		if(criteria.getRicerca()) {
			BigDecimal imp = (BigDecimal) getUniqueResult(criteriaS);
			if(imp == null) {
				imp = new BigDecimal(0); 
			}
			return imp;
		} else {
			return new BigDecimal(0);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Acconto> trova(Utente user, AccontoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(user, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(Utente user, AccontoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(user, ana);
		return getListSize(criteria);
	}
	public BigDecimal trovaSum(Utente user, AccontoCriteria ana) {
		boolean esegui = impostazioneDao.getSommaRicerca();
		if(esegui) {
			ExtendedCriteria criteria = trovaCriteria(user, ana);
			return getSommaDecimale(criteria , "importo");
		} else {
			return new BigDecimal(0);
		}
	}
	public Estratto salvaEstratto(Estratto estratto) {
		return (Estratto) salvaAggiorna(estratto);
	}
	public Acconto salvaAcconto(Acconto acconto) {
		return (Acconto) salvaAggiorna(acconto);
	}
	public EstrattoDettaglio salvaDettaglio(EstrattoDettaglio dettaglio) {
		return (EstrattoDettaglio) salvaAggiorna(dettaglio);
	}
	public EstrattoDettaglio nuovoDettaglio() {
		return new EstrattoDettaglio();
	}
	public void eliminaEstratto(Estratto estratto) {
		cancellaAggiorna(estratto);
	}
	public void eliminaAcconto(Acconto acconto) {
		cancellaAggiorna(acconto);
	}
	public void eliminaDettaglio(EstrattoDettaglio dettaglio) {
		cancellaAggiorna(dettaglio);
	}
	public void eliminaDettagli(List<EstrattoDettaglio> dettagli) {
		if(ApplicationUtil.notEmpty(dettagli)) {
			Iterator<EstrattoDettaglio>	it = dettagli.iterator();
			while (it.hasNext()) {
				EstrattoDettaglio dettaglio = (EstrattoDettaglio) it.next();
				eliminaDettaglio(dettaglio);
			}
		}
	}
	private ExtendedCriteria trovaCriteria(EstrattoCriteria ana) {
		if(ana != null)  ana = (EstrattoCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Estratto.class);
		DetachedCriteria criteriau =  criteria.createCriteria("user");
		if (ApplicationUtil.isNotBlank(ana.getUser())) {
			ApplicationUtil.operatore(criteriau, ana.getUser());
		}
		if (ApplicationUtil.isNotBlank(ana.getDescrizione())) {
			DetachedCriteria c = DetachedCriteria.forClass(EstrattoDettaglio.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getDescrizione(), c, "descrizione");
			c.setProjection(Projections.distinct(Projections.property("estratto.id")));
			impostazioneDao.subPropertyInS(criteria, c, "id");
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	private ExtendedCriteria trovaCriteria(Utente user, EstrattoCriteria ana) {
		if(ana != null)  ana = (EstrattoCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Estratto.class);
		criteria.createCriteria("user").add(Restrictions.idEq(user.getId()));
		if (ApplicationUtil.isNotBlank(ana.getDescrizione())) {
			DetachedCriteria c = DetachedCriteria.forClass(EstrattoDettaglio.class);
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getDescrizione(), c, "descrizione");
			c.setProjection(Projections.distinct(Projections.property("estratto.id")));
			impostazioneDao.subPropertyInS(criteria, c, "id");
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Estratto> trova(EstrattoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(EstrattoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public BigDecimal trovaSum(EstrattoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return sumEstratto(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Estratto> trova(Utente user, EstrattoCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(user, ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(Utente user, EstrattoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(user, ana);
		return getListSize(criteria);
	}
	public BigDecimal trovaSum(Utente user, EstrattoCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(user, ana);
		return sumEstratto(criteria);
	}
	public Estratto get(Long id) {
		return (Estratto) get(Estratto.class, id);
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoDettaglio> trovaDettagli(Long estratto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EstrattoDettaglio.class);
		criteria.createCriteria("estratto").add(Restrictions.idEq(estratto));
		return getList(criteria);
	} 
	public int contaFatturePerEstrattoAnno(Estratto estratto) {
		Date data = estratto.getData_estratto();
		DetachedCriteria criteria = DetachedCriteria.forClass(Estratto.class);
		ApplicationUtil.dataDate(ApplicationUtil.inizioAnno(data), data, "data_estratto", criteria);
		criteria.createCriteria("user").add(Restrictions.idEq(estratto.getUser().getId()));
		return getListSize(criteria);
	} 
	public int contaFatturePerContrattoAnno(Contratto contratto) {
		Date data = contratto.getData_stipula();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contratto.class);
		criteria.add(Restrictions.isNotNull("costo_scontato"));
		ApplicationUtil.dataDate(ApplicationUtil.inizioAnno(data), data, "data_stipula", criteria);
		criteria.createCriteria("prodotto").add(Restrictions.eq("vendita", Boolean.TRUE));
		return getListSize(criteria);
	} 
	public List<EstrattoDettaglio> trovaDettagliAnteprima(Estratto estratto) {
		return new ArrayList<EstrattoDettaglio>(estratto.getEstrattoDettagli());
	} 
	@SuppressWarnings("unchecked")
	public List<Acconto> trovaAccontoAttivi(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Acconto.class);
		criteria.add(Restrictions.or(Restrictions.eq("tipo", ConstantsMap.tipo_acconto1), Restrictions.eq("tipo", ConstantsMap.tipo_acconto3)));
		criteria.createCriteria("user").add(Restrictions.idEq(user));
		return getList(criteria);
	} 
	@SuppressWarnings("unchecked")
	public void generaEstratti(Boolean avanza) {
		Boolean b = impostazioneDao.getConsideraQualifica();
		int statoInPagamento = ConstantsGlobal.in_pagamento;
		int statoInStorno = ConstantsGlobal.in_storno;
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		estratti.addAll(generaEstrattiGettoneAppuntamenti(b, statoInPagamento, statoInStorno, null));
		estratti.addAll(generaEstrattiGettoneContrattiProdotto(b, statoInPagamento, statoInStorno, null));
		estratti.addAll(generaEstrattiGettoneContrattiCategoria(b, statoInPagamento, statoInStorno, null));
		SortedMap<Long, ArrayList<EstrattoModel>>  mappaCompleta = ApplicationUtil.collectionToHashEstratto(estratti, "getUtente_id");
		Set<Long> ss = mappaCompleta.keySet();
		if (ApplicationUtil.notEmpty(ss)) {
			Iterator<Long> itE = ss.iterator();
			while (itE.hasNext()) {
				Long uid = (Long) itE.next();
				Estratto estratto = new Estratto();
				Utente utente = userDao.getUser(uid);
				if(utente.getGeneraEstratto()) {
					estratto.setUser(utente);
					salvaAggiorna(estratto);
					List<EstrattoModel> tutti = ApplicationUtil.getEstratto(mappaCompleta, uid);
					Iterator<EstrattoModel> itT = tutti.iterator();
					while (itT.hasNext()) {
						EstrattoModel estrattoModel = (EstrattoModel) itT.next();
						EstrattoDettaglio dettaglio = new EstrattoDettaglio();
						dettaglio.setEstratto(estratto);
						dettaglio.setDescrizione(estrattoModel.getDescrizione());
						if(ApplicationUtil.equals(estrattoModel.getStato(), statoInPagamento)) {
							dettaglio.setGettone(estrattoModel.getGettone().multiply(new BigDecimal(1)));
						} else {
							dettaglio.setGettone(estrattoModel.getGettone().multiply(new BigDecimal(-1)));
						}
						dettaglio.setValore(estrattoModel.getValore());
						salvaAggiorna(dettaglio);
					}
					List<Acconto> acconti = trovaAccontoAttivi(utente.getId());
					if(ApplicationUtil.notEmpty(acconti)) {
						Iterator<Acconto> itA = acconti.iterator();
						while (itA.hasNext()) {
							Acconto accontoModel = (Acconto) itA.next();
							EstrattoDettaglio dettaglio = new EstrattoDettaglio();
							dettaglio.setEstratto(estratto);
							dettaglio.setDescrizione(accontoModel.getDescrizione());
							dettaglio.setGettone(accontoModel.getImporto());
							dettaglio.setValore(new BigDecimal(1));
							salvaAggiorna(dettaglio);
							if(ApplicationUtil.isTrue(avanza)) {
								if(accontoModel.getTipo().equals(ConstantsMap.tipo_acconto1)) {
									accontoModel.setTipo(ConstantsMap.tipo_acconto2);
									salvaAcconto(accontoModel);
								}
							}
						}
					}
				}
			}
		}
		if(ApplicationUtil.isTrue(avanza)) {
			DetachedCriteria criteriaC = DetachedCriteria.forClass(Contratto.class);
			criteriaC.add(Restrictions.or(Restrictions.eq("stato", statoInPagamento), Restrictions.eq("stato", statoInStorno)));
			List<Contratto> contratti =  getList(criteriaC);
			if(ApplicationUtil.notEmpty(contratti)) {
				Iterator<Contratto> itC = contratti.iterator();
				while (itC.hasNext()) {
					Contratto contratto = (Contratto) itC.next();
					if(contratto.getStato().equals(statoInPagamento)) {
						contratto.setStato(ConstantsGlobal.pagato);
					}
					if(contratto.getStato().equals(statoInStorno)) {
						contratto.setStato(ConstantsGlobal.stornato);
					}
					contrattoDao.salvaContratto(contratto, null, true);
				}
			}
			DetachedCriteria criteriaA = DetachedCriteria.forClass(Appuntamento.class);
			criteriaA.add(Restrictions.or(Restrictions.eq("stato", statoInPagamento), Restrictions.eq("stato", statoInStorno)));
			List<Appuntamento> appuntamenti =  getList(criteriaA);
			if(ApplicationUtil.notEmpty(appuntamenti)) {
				Iterator<Appuntamento> itA = appuntamenti.iterator();
				while (itA.hasNext()) {
					Appuntamento appuntamento = (Appuntamento) itA.next();
					if(appuntamento.getStato().equals(statoInPagamento)) {
						appuntamento.setStato(ConstantsGlobal.pagato);
					}
					if(appuntamento.getStato().equals(statoInStorno)) {
						appuntamento.setStato(ConstantsGlobal.stornato);
					}
					pannelloDao.salvaAppuntamento(appuntamento, null, true);
				}
			}
		}
	}
	public List<Estratto> anteprimaEstratti(Utente utente) {
		Boolean b = impostazioneDao.getConsideraQualifica();
		int statoInPagamento = ConstantsGlobal.in_pagamento;
		int statoInStorno = ConstantsGlobal.in_storno;
		List<Estratto> estrattiModel = new ArrayList<Estratto>();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		estratti.addAll(generaEstrattiGettoneAppuntamenti(b, statoInPagamento, statoInStorno, utente));
		estratti.addAll(generaEstrattiGettoneContrattiProdotto(b, statoInPagamento, statoInStorno, utente));
		estratti.addAll(generaEstrattiGettoneContrattiCategoria(b, statoInPagamento, statoInStorno, utente));
		SortedMap<Long, ArrayList<EstrattoModel>>  mappaCompleta = ApplicationUtil.collectionToHashEstratto(estratti, "getUtente_id");
		Set<Long> ss = mappaCompleta.keySet();
		if (ApplicationUtil.notEmpty(ss)) {
			Iterator<Long> itE = ss.iterator();
			while (itE.hasNext()) {
				Long uid = (Long) itE.next();
				Estratto estratto = new Estratto();
				if(utente.getGeneraEstratto()) {
					estratto.setUser(utente);
					Set<EstrattoDettaglio> dettagli = new HashSet<EstrattoDettaglio>();
					int nd = 0;
					BigDecimal importo = new BigDecimal(0);
					List<EstrattoModel> tutti = ApplicationUtil.getEstratto(mappaCompleta, uid);
					Iterator<EstrattoModel> itT = tutti.iterator();
					while (itT.hasNext()) {
						EstrattoModel estrattoModel = (EstrattoModel) itT.next();
						EstrattoDettaglio dettaglio = new EstrattoDettaglio();
						dettaglio.setEstratto(estratto);
						dettaglio.setDescrizione(estrattoModel.getDescrizione());
						if(ApplicationUtil.equals(estrattoModel.getStato(), statoInPagamento)) {
							dettaglio.setGettone(estrattoModel.getGettone().multiply(new BigDecimal(1)));
						} else {
							dettaglio.setGettone(estrattoModel.getGettone().multiply(new BigDecimal(-1)));
						}
						dettaglio.setValore(estrattoModel.getValore());
						dettagli.add(dettaglio);
						BigDecimal imp = dettaglio.getGettone().multiply(dettaglio.getValore());
						importo = importo.add(imp);
						nd++;
					}
					List<Acconto> acconti = trovaAccontoAttivi(utente.getId());
					if(ApplicationUtil.notEmpty(acconti)) {
						Iterator<Acconto> itA = acconti.iterator();
						while (itA.hasNext()) {
							Acconto accontoModel = (Acconto) itA.next();
							EstrattoDettaglio dettaglio = new EstrattoDettaglio();
							dettaglio.setEstratto(estratto);
							dettaglio.setDescrizione(accontoModel.getDescrizione());
							dettaglio.setGettone(accontoModel.getImporto());
							dettaglio.setValore(new BigDecimal(1));
							dettagli.add(dettaglio);
							BigDecimal imp = dettaglio.getGettone().multiply(dettaglio.getValore());
							importo = importo.add(imp);
							nd++;
						}
					}
					estratto.setEstrattoDettagli(dettagli);
					estratto.setNdettagli(Long.valueOf(nd));
					estratto.setTotale(importo);
					estrattiModel.add(estratto);
				}
			}
		}
		return estrattiModel;
	}
	public List<Estratto> anteprimaEstratti() {
		Boolean b = impostazioneDao.getConsideraQualifica();
		int statoInPagamento = ConstantsGlobal.in_pagamento;
		int statoInStorno = ConstantsGlobal.in_storno;
		List<Estratto> estrattiModel = new ArrayList<Estratto>();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		estratti.addAll(generaEstrattiGettoneAppuntamenti(b, statoInPagamento, statoInStorno, null));
		estratti.addAll(generaEstrattiGettoneContrattiProdotto(b, statoInPagamento, statoInStorno, null));
		estratti.addAll(generaEstrattiGettoneContrattiCategoria(b, statoInPagamento, statoInStorno, null));
		SortedMap<Long, ArrayList<EstrattoModel>>  mappaCompleta = ApplicationUtil.collectionToHashEstratto(estratti, "getUtente_id");
		Set<Long> ss = mappaCompleta.keySet();
		if (ApplicationUtil.notEmpty(ss)) {
			Iterator<Long> itE = ss.iterator();
			while (itE.hasNext()) {
				Long uid = (Long) itE.next();
				Estratto estratto = new Estratto();
				Utente utente = userDao.getUser(uid);
				if(utente.getGeneraEstratto()) {
					estratto.setUser(utente);
					Set<EstrattoDettaglio> dettagli = new HashSet<EstrattoDettaglio>();
					int nd = 0;
					BigDecimal importo = new BigDecimal(0);
					List<EstrattoModel> tutti = ApplicationUtil.getEstratto(mappaCompleta, uid);
					Iterator<EstrattoModel> itT = tutti.iterator();
					while (itT.hasNext()) {
						EstrattoModel estrattoModel = (EstrattoModel) itT.next();
						EstrattoDettaglio dettaglio = new EstrattoDettaglio();
						dettaglio.setEstratto(estratto);
						dettaglio.setDescrizione(estrattoModel.getDescrizione());
						if(ApplicationUtil.equals(estrattoModel.getStato(), statoInPagamento)) {
							dettaglio.setGettone(estrattoModel.getGettone().multiply(new BigDecimal(1)));
						} else {
							dettaglio.setGettone(estrattoModel.getGettone().multiply(new BigDecimal(-1)));
						}
						dettaglio.setValore(estrattoModel.getValore());
						dettagli.add(dettaglio);
						BigDecimal imp = dettaglio.getGettone().multiply(dettaglio.getValore());
						importo = importo.add(imp);
						nd++;
					}
					List<Acconto> acconti = trovaAccontoAttivi(utente.getId());
					if(ApplicationUtil.notEmpty(acconti)) {
						Iterator<Acconto> itA = acconti.iterator();
						while (itA.hasNext()) {
							Acconto accontoModel = (Acconto) itA.next();
							EstrattoDettaglio dettaglio = new EstrattoDettaglio();
							dettaglio.setEstratto(estratto);
							dettaglio.setDescrizione(accontoModel.getDescrizione());
							dettaglio.setGettone(accontoModel.getImporto());
							dettaglio.setValore(new BigDecimal(1));
							dettagli.add(dettaglio);
							BigDecimal imp = dettaglio.getGettone().multiply(dettaglio.getValore());
							importo = importo.add(imp);
							nd++;
						}
					}
					estratto.setEstrattoDettagli(dettagli);
					estratto.setNdettagli(Long.valueOf(nd));
					estratto.setTotale(importo);
					estrattiModel.add(estratto);
				}
			}
		}
		return estrattiModel;
	}
	public DetachedCriteria subCriteria(Class<?> classe, String pro, Integer tipo) {
		DetachedCriteria c = DetachedCriteria.forClass(classe);
		c.add(Restrictions.eq("attivo", Boolean.TRUE));
		c.add(Restrictions.eq("tipo", tipo));
		DetachedCriteria cd = c.createCriteria(pro);
		cd.setProjection(Projections.distinct(Projections.property("id")));
		return c;
	}
	public DetachedCriteria subCriteria(Class<?> classe, String pro) {
		DetachedCriteria c = DetachedCriteria.forClass(classe);
		c.add(Restrictions.eq("attivo", Boolean.TRUE));
		DetachedCriteria cd = c.createCriteria(pro);
		cd.setProjection(Projections.distinct(Projections.property("id")));
		return c;
	}
	@SuppressWarnings("unchecked")
	public List<Contest> trovaContestByQualificaId(Long qualificaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contest.class);
		criteria.add(Restrictions.eq("qualifica.id", qualificaId));
		List<Contest> ctl = getList(criteria);
		DetachedCriteria criteriaD = DetachedCriteria.forClass(Contest.class);
		criteriaD.add(Restrictions.eq("differenziale.id", qualificaId));
		List<Contest> diff = getList(criteriaD);
		if(ApplicationUtil.notEmpty(diff)) {
			Iterator<Contest> itC = diff.iterator();
			while (itC.hasNext()) {
				Contest contest = (Contest) itC.next();
				if(!ctl.contains(contest)){
					ctl.add(contest);
				}
			}
		}
		return ctl;
	}
	@SuppressWarnings("unchecked")
	public List<Gettone> trovaGettoneByQualificaId(Long qualificaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Gettone.class);
		criteria.add(Restrictions.eq("qualifica.id", qualificaId));
		List<Gettone> ctl = getList(criteria);
		DetachedCriteria criteriaD = DetachedCriteria.forClass(Gettone.class);
		criteriaD.add(Restrictions.eq("differenziale.id", qualificaId));
		List<Gettone> diff = getList(criteriaD);
		if(ApplicationUtil.notEmpty(diff)) {
			Iterator<Gettone> itC = diff.iterator();
			while (itC.hasNext()) {
				Gettone contest = (Gettone) itC.next();
				if(!ctl.contains(contest)){
					ctl.add(contest);
				}
			}
		}
		return ctl;
	}
	@SuppressWarnings("unchecked")
	public List<Categoria> getCategorieModel() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Categoria.class);
		DetachedCriteria c1 = subCriteria(Gettone.class, "categoria", 2);
		DetachedCriteria c2 = subCriteria(Contest.class, "categoria", 2);
		criteria.add(Restrictions.or(Subqueries.propertyIn("id", c1), Subqueries.propertyIn("id", c2)));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottiModel() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria c1 = subCriteria(Gettone.class, "prodotto", 1);
		DetachedCriteria c2 = subCriteria(Contest.class, "prodotto", 1);
		criteria.add(Restrictions.or(Subqueries.propertyIn("id", c1), Subqueries.propertyIn("id", c2)));
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Qualifica> getQualificheModel(String tipo) { 
		DetachedCriteria criteria = DetachedCriteria.forClass(Qualifica.class);
		DetachedCriteria c1 = subCriteria(Gettone.class, tipo);
		DetachedCriteria c2 = subCriteria(Contest.class, tipo);
		criteria.add(Restrictions.or(Subqueries.propertyIn("id", c1), Subqueries.propertyIn("id", c2)));
		List<Qualifica> qualifiche =  getList(criteria);
		qualifiche.add(0, null);
		return qualifiche;
	}
	public List<EstrattoModel> generaEstrattiGettoneContrattiProdotto(Boolean b, int statoInPagamento, int statoInStorno, Utente utente) {
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		List<Prodotto> prodottiList = getProdottiModel();
		List<Qualifica> qualificheList = getQualificheModel("qualifica");
		List<Qualifica> differenzialeList = getQualificheModel("differenziale");
		Iterator<Prodotto> itP = prodottiList.iterator();
		while (itP.hasNext()) {
			Prodotto prodotto = (Prodotto) itP.next();
			estratti.addAll(generaGettoneContrattiProdottoOperatore(prodotto,statoInPagamento, statoInStorno, utente));
			estratti.addAll(generaGettoneContrattiProdottoSupervisore(prodotto,statoInPagamento, statoInStorno, utente));
			Iterator<Qualifica> itQ = qualificheList.iterator();
			while (itQ.hasNext()) {
				Qualifica qualifica = (Qualifica) itQ.next();
				estratti.addAll(generaGettoneContrattiProdottoCoordinatore(b, qualifica, prodotto,statoInPagamento, statoInStorno, utente));
				estratti.addAll(generaGettoneContrattiProdottoAgente(b, qualifica, prodotto,statoInPagamento, statoInStorno, utente));
				estratti.addAll(generaGettoneContrattiProdottoRete(b, qualifica, null, prodotto,statoInPagamento, statoInStorno, utente, ConstantsGlobal.rete));
				estratti.addAll(generaGettoneContrattiProdottoRete(b, qualifica, null, prodotto,statoInPagamento, statoInStorno, utente, ConstantsGlobal.sotto));
				estratti.addAll(generaGettoneContrattiProdottoRete(b, qualifica, null, prodotto,statoInPagamento, statoInStorno, utente, ConstantsGlobal.prima));
				Iterator<Qualifica> itQDiff = differenzialeList.iterator();
				while (itQDiff.hasNext()) {
					Qualifica differenziale = (Qualifica) itQDiff.next();
					estratti.addAll(generaGettoneContrattiProdottoRete(b, qualifica, differenziale, prodotto,statoInPagamento, statoInStorno, utente, ConstantsGlobal.orete));
					estratti.addAll(generaGettoneContrattiProdottoRete(b, qualifica, differenziale, prodotto,statoInPagamento, statoInStorno, utente, ConstantsGlobal.osotto));
					estratti.addAll(generaGettoneContrattiProdottoRete(b, qualifica, differenziale, prodotto,statoInPagamento, statoInStorno, utente, ConstantsGlobal.oprima));
				}
			}
		}
		return estratti;
	}
	public List<EstrattoModel> generaEstrattiGettoneContrattiCategoria(Boolean b, int statoInPagamento, int statoInStorno, Utente utente) {
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		List<Categoria> categorieList = getCategorieModel();
		List<Qualifica> qualificheList = getQualificheModel("qualifica");
		List<Qualifica> differenzialeList = getQualificheModel("differenziale");
		Iterator<Categoria> itP = categorieList.iterator();
		while (itP.hasNext()) {
			Categoria categoria = (Categoria) itP.next();
			estratti.addAll(generaGettoneContrattiCategoriaOperatore(categoria,statoInPagamento, statoInStorno, utente));
			estratti.addAll(generaGettoneContrattiCategoriaSupervisore(categoria,statoInPagamento, statoInStorno, utente));
			Iterator<Qualifica> itQ = qualificheList.iterator();
			while (itQ.hasNext()) {
				Qualifica qualifica = (Qualifica) itQ.next();
				estratti.addAll(generaGettoneContrattiCategoriaCoordinatore(b, qualifica, categoria,statoInPagamento, statoInStorno, utente));
				estratti.addAll(generaGettoneContrattiCategoriaAgente(b, qualifica, categoria,statoInPagamento, statoInStorno, utente));
				estratti.addAll(generaGettoneContrattiCategoriaRete(b, qualifica, null, categoria,statoInPagamento, statoInStorno, utente, ConstantsGlobal.rete));
				estratti.addAll(generaGettoneContrattiCategoriaRete(b, qualifica, null, categoria,statoInPagamento, statoInStorno, utente, ConstantsGlobal.sotto));
				estratti.addAll(generaGettoneContrattiCategoriaRete(b, qualifica, null, categoria,statoInPagamento, statoInStorno, utente, ConstantsGlobal.prima));
				Iterator<Qualifica> itQDiff = differenzialeList.iterator();
				while (itQDiff.hasNext()) {
					Qualifica differenziale = (Qualifica) itQDiff.next();
					estratti.addAll(generaGettoneContrattiCategoriaRete(b, qualifica, differenziale, categoria,statoInPagamento, statoInStorno, utente, ConstantsGlobal.orete));
					estratti.addAll(generaGettoneContrattiCategoriaRete(b, qualifica, differenziale, categoria,statoInPagamento, statoInStorno, utente, ConstantsGlobal.osotto));
					estratti.addAll(generaGettoneContrattiCategoriaRete(b, qualifica, differenziale, categoria,statoInPagamento, statoInStorno, utente, ConstantsGlobal.oprima));
				}
			}
		}
		return estratti;
	}
	public List<EstrattoModel> generaEstrattiGettoneAppuntamenti(Boolean b, int statoInPagamento, int statoInStorno, Utente utente) {
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		estratti.addAll(generaGettoneAppuntamentiOperatore(statoInPagamento, statoInStorno, utente));
		estratti.addAll(generaGettoneAppuntamentiSupervisore(statoInPagamento, statoInStorno, utente));
		List<Qualifica> qualificheList = getQualificheModel("qualifica");
		List<Qualifica> differenzialeList = getQualificheModel("differenziale");
		Iterator<Qualifica> itQ = qualificheList.iterator();
		while (itQ.hasNext()) {
			Qualifica qualifica = (Qualifica) itQ.next();
			estratti.addAll(generaGettoneAppuntamentiCoordinatore(b, qualifica,statoInPagamento, statoInStorno, utente));
			estratti.addAll(generaGettoneAppuntamentiAgente(b, qualifica,statoInPagamento, statoInStorno, utente));
			estratti.addAll(generaGettoneAppuntamentiRete(b, qualifica, null, statoInPagamento, statoInStorno, utente, ConstantsGlobal.rete));
			estratti.addAll(generaGettoneAppuntamentiRete(b, qualifica, null, statoInPagamento, statoInStorno, utente, ConstantsGlobal.sotto));
			estratti.addAll(generaGettoneAppuntamentiRete(b, qualifica, null, statoInPagamento, statoInStorno, utente, ConstantsGlobal.prima));
			Iterator<Qualifica> itQDiff = differenzialeList.iterator();
			while (itQDiff.hasNext()) {
				Qualifica differenziale = (Qualifica) itQDiff.next();
				estratti.addAll(generaGettoneAppuntamentiRete(b, qualifica, differenziale, statoInPagamento, statoInStorno, utente, ConstantsGlobal.orete));
				estratti.addAll(generaGettoneAppuntamentiRete(b, qualifica, differenziale, statoInPagamento, statoInStorno, utente, ConstantsGlobal.osotto));
				estratti.addAll(generaGettoneAppuntamentiRete(b, qualifica, differenziale, statoInPagamento, statoInStorno, utente, ConstantsGlobal.oprima));
			}
		}
		return estratti;
	}
	private DetachedCriteria getInterno(Class<?> classe, Integer tipoUtente, Categoria categoria, Prodotto prodotto, boolean contratto) {
		DetachedCriteria criteriaG = DetachedCriteria.forClass(classe, "g");
		criteriaG.add(Restrictions.eq("attivo", Boolean.TRUE ));
		criteriaG.add(Restrictions.eq("utente", tipoUtente ));
		criteriaG.add(Restrictions.eq("contratto", contratto));
		if(contratto) {
			if(prodotto!=null) {
				criteriaG.add(Restrictions.eq("prodotto.id", prodotto.getId() ));
			}
			if(categoria!=null) {
				criteriaG.add(Restrictions.eq("categoria.id", categoria.getId() ));
			}
		}
		return criteriaG;
	}
	private DetachedCriteria getEsterno(Boolean b, Class<?> classe, Integer tipoUtente, Qualifica qualifica, Qualifica differenziale, Categoria categoria, Prodotto prodotto, boolean contratto) {
		DetachedCriteria criteriaG = DetachedCriteria.forClass(classe, "g");
		criteriaG.add(Restrictions.eq("attivo", Boolean.TRUE ));
		criteriaG.add(Restrictions.eq("utente", tipoUtente ));
		if(qualifica!=null) {
			criteriaG.add(Restrictions.eq("qualifica.id", qualifica.getId() ));
		} else {
			if(ApplicationUtil.isTrue(b)) {
				criteriaG.add(Restrictions.isNull("qualifica.id"));
			}
		}
		if(tipoUtente.equals(ConstantsGlobal.orete) || tipoUtente.equals(ConstantsGlobal.oprima) || tipoUtente.equals(ConstantsGlobal.osotto)) {
			if(differenziale!=null) {
				criteriaG.add(Restrictions.eq("differenziale.id", differenziale.getId() ));
			} else {
				boolean c = true;
				if(ApplicationUtil.isTrue(c)) {
					criteriaG.add(Restrictions.isNull("differenziale.id"));
				}
			}
		}
		criteriaG.add(Restrictions.eq("contratto",  contratto));
		if(contratto) {
			if(prodotto!=null) {
				criteriaG.add(Restrictions.eq("prodotto.id", prodotto.getId() ));
			}
			if(categoria!=null) {
				criteriaG.add(Restrictions.eq("categoria.id", categoria.getId() ));
			}
		}
		return criteriaG;
	}
	public boolean raggiuntoContest(EstrattoModel estrattoModel, ContestModel cm) {
		BigDecimal b1 = estrattoModel.getValore();
		BigDecimal b2 = new BigDecimal(cm.getQuota());
		//return  b1 >= b2;
		return  b1.compareTo(b2) >= 0;
	}
	public void sistemaEstratti(List<EstrattoModel> estratti, List<ContestModel> contests, Integer tipoUtente, String tipo, BigDecimal gettone, Qualifica qualifica, Qualifica differenziale, Categoria categoria, Prodotto prodotto) {
		List<EstrattoModel> estrattiNew =  new ArrayList<EstrattoModel>();
		Iterator<EstrattoModel> itE = estratti.iterator();
		while (itE.hasNext()) {
			EstrattoModel estrattoModel = (EstrattoModel) itE.next();
			estrattoModel.setTipoUtente(tipoUtente);
			estrattoModel.setTipo(tipo);
			estrattoModel.setQualifica(qualifica);
			estrattoModel.setDifferenziale(differenziale);
			estrattoModel.setProdotto(prodotto);
			estrattoModel.setCategoria(categoria);
			BigDecimal gett = new BigDecimal(gettone.doubleValue());
			if(ApplicationUtil.notEmpty(contests)) {
				for(ContestModel cm : contests) {
					if(ApplicationUtil.eq(cm.getMoltiplicatore(),1) && raggiuntoContest(estrattoModel, cm)) {
						gett = gett.multiply(cm.getImporto());
						estrattoModel.setGettone(gett);
						estrattoModel.setContest(true);
					} else if(ApplicationUtil.eq(cm.getMoltiplicatore(),2) && raggiuntoContest(estrattoModel, cm)) {
						gett = gett.add(cm.getImporto());
						estrattoModel.setGettone(gett);
						estrattoModel.setContest(true);
					} else if(ApplicationUtil.eq(cm.getMoltiplicatore(),3) && raggiuntoContest(estrattoModel, cm)) {
						EstrattoModel ems = new EstrattoModel();
						ems.setUtente_id(estrattoModel.getUtente_id());
						ems.setC_id(estrattoModel.getC_id());
						ems.setP_id(estrattoModel.getP_id());
						ems.setQ_id(estrattoModel.getQ_id());
						ems.setD_id(estrattoModel.getD_id());
						ems.setValore(new BigDecimal(1));
						ems.setQualifica(estrattoModel.getQualifica());
						ems.setProdotto(estrattoModel.getProdotto());
						ems.setStato(estrattoModel.getStato());
						ems.setTipoUtente(tipoUtente);
						ems.setTipo(tipo);
						ems.setGettone(cm.getImporto());
						ems.setContest(true);
						estrattiNew.add(ems);
						estrattoModel.setGettone(gett);
					} else {
						estrattoModel.setGettone(gett);
					}
				}
			} else {
				estrattoModel.setGettone(gett);
			}
		}
		estratti.addAll(estrattiNew);
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneAppuntamentiOperatore(int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.operatore;
		String tipo = ConstantsLanguage.APPUNTAMENTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getInterno(Gettone.class, tipoUtente, null, null, Boolean.FALSE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getInterno(Contest.class, tipoUtente, null, null, Boolean.FALSE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaA = criteria.createCriteria("appuntamenti", "ap");
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			criteria.setProjection(Projections.projectionList()
					.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")))
					.add(Projections.groupProperty("ap.stato").as("stato"))
					.add(Projections.countDistinct("ap.id").as("valore"))
					);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti,contests, tipoUtente, tipo, gettone, null, null, null, null);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneAppuntamentiSupervisore(int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.supervisore;
		String tipo = ConstantsLanguage.APPUNTAMENTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getInterno(Gettone.class, tipoUtente, null,null, Boolean.FALSE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getInterno(Contest.class, tipoUtente, null,null, Boolean.FALSE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaA =  criteria.createCriteria("teams" , "t").createCriteria("user", "o").createCriteria("appuntamenti", "ap");
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			criteria.setProjection(Projections.projectionList()
					.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")))
					.add(Projections.groupProperty("ap.stato").as("stato"))
					.add(Projections.countDistinct("ap.id").as("valore"))
					);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, null, null, null, null);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneAppuntamentiAgente(Boolean b, Qualifica qualifica, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.agente;
		String tipo = ConstantsLanguage.APPUNTAMENTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica, null, null,null, Boolean.FALSE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica,null, null,null, Boolean.FALSE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("agenti" , "a");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q", ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			DetachedCriteria criteriaA =  criteriaAg.createCriteria("appuntamenti", "ap");
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(Projections.countDistinct("ap.id").as("valore"));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica, null, null,null);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneAppuntamentiRete(Boolean b, Qualifica qualifica, Qualifica differenziale, int statoInPagamento, int statoInStorno, Utente utente, Integer tipoUtente) {
		String tipo = ConstantsLanguage.APPUNTAMENTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica, differenziale, null,null, Boolean.FALSE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica, differenziale, null,null, Boolean.FALSE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("agenti" , "a");
			DetachedCriteria criteriaCl = criteriaAg.createCriteria("closure_agenti_padre", "cl");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q", ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			if(tipoUtente.equals(ConstantsGlobal.rete) || tipoUtente.equals(ConstantsGlobal.orete)) {
				criteriaCl.add(Restrictions.ge("distanza", 0));
			} 
			if(tipoUtente.equals(ConstantsGlobal.sotto) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				criteriaCl.add(Restrictions.ge("distanza", 1));
			} 
			if(tipoUtente.equals(ConstantsGlobal.prima) || tipoUtente.equals(ConstantsGlobal.oprima)) {
				criteriaCl.add(Restrictions.eq("distanza", 1));
			} 
			Integer max = userDao.getMaxLivelloRete();
			if(ApplicationUtil.gtZero(max)) {
				criteriaCl.add(Restrictions.le("distanza", max));
			}
			DetachedCriteria criteriaClF =  criteriaCl.createCriteria("agente", "agf");
			DetachedCriteria criteriaA =  criteriaClF.createCriteria("appuntamenti", "ap");
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			if(tipoUtente.equals(ConstantsGlobal.orete) || tipoUtente.equals(ConstantsGlobal.oprima) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				DetachedCriteria criteriaQQ = criteriaClF.createCriteria("qualifica" , "d", ApplicationUtil.leftJoin());
				if(differenziale!=null) {
					criteriaQQ.add(Restrictions.eq("id", differenziale.getId() ));
				} else {
					boolean c = true;
					if(ApplicationUtil.isTrue(c)) {
						criteriaQQ.add(Restrictions.isNull("id"));
					}
				}
			}
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			if(tipoUtente.equals(ConstantsGlobal.orete) || tipoUtente.equals(ConstantsGlobal.oprima) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				if(differenziale!=null) {
					pl.add(Projections.groupProperty("d.id").as("d_id"));
				}
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(Projections.countDistinct("ap.id").as("valore"));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica, differenziale, null, null);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneAppuntamentiCoordinatore(Boolean b, Qualifica qualifica, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.coordinatore;
		String tipo = ConstantsLanguage.APPUNTAMENTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica,null, null,null, Boolean.FALSE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica,null, null,null, Boolean.FALSE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("zone" , "z").createCriteria("agente" , "a");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q", ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			DetachedCriteria criteriaA =  criteriaAg.createCriteria("appuntamenti", "ap");
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(Projections.countDistinct("ap.id").as("valore"));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica, null, null, null);
		}
		return estratti;
	}
	public Projection modalitaPagamento(Integer tipologia) {
		if(tipologia.equals(ConstantsGlobal.gpercentuale)) {
			return Projections.sum("ap.costo_scontato").as("valore");
		} else if(tipologia.equals(ConstantsGlobal.gsomma)) {
			return Projections.sum("ap.quantita").as("valore");
		} else {
			return Projections.countDistinct("ap.id").as("valore");
		}
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiProdottoOperatore(Prodotto prodotto, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.operatore;
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getInterno(Gettone.class, tipoUtente, null,prodotto, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getInterno(Contest.class, tipoUtente, null,prodotto, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaA = criteria.createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			criteriaP.add(Restrictions.idEq(prodotto.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			criteria.setProjection(Projections.projectionList()
					.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")))
					.add(Projections.groupProperty("p.id").as("p_id"))
					.add(Projections.groupProperty("ap.stato").as("stato"))
					.add(modalitaPagamento(prodotto.getTipologia()))
					);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, null, null, null,prodotto);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiProdottoSupervisore(Prodotto prodotto, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.supervisore;
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getInterno(Gettone.class, tipoUtente, null,prodotto, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getInterno(Contest.class, tipoUtente, null,prodotto, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaA =  criteria.createCriteria("teams" , "t").createCriteria("user", "o").createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			criteriaP.add(Restrictions.idEq(prodotto.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			criteria.setProjection(Projections.projectionList()
					.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")))
					.add(Projections.groupProperty("p.id").as("p_id"))
					.add(Projections.groupProperty("ap.stato").as("stato"))
					.add(modalitaPagamento(prodotto.getTipologia()))
					);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, null, null, null,prodotto);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiProdottoAgente(Boolean b, Qualifica qualifica, Prodotto prodotto, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.agente;
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica, null,null,prodotto, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica,null, null,prodotto, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("agenti" , "a");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q",  ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			DetachedCriteria criteriaA =  criteriaAg.createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			criteriaP.add(Restrictions.idEq(prodotto.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			pl.add(Projections.groupProperty("p.id").as("p_id"));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(modalitaPagamento(prodotto.getTipologia()));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica, null, null, prodotto);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiProdottoRete(Boolean b, Qualifica qualifica, Qualifica differenziale, Prodotto prodotto, int statoInPagamento, int statoInStorno, Utente utente, Integer tipoUtente) {
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica, differenziale, null,prodotto, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica, differenziale, null,prodotto, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("agenti" , "a");
			DetachedCriteria criteriaCl = criteriaAg.createCriteria("closure_agenti_padre", "cl");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q", ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			if(tipoUtente.equals(ConstantsGlobal.rete) || tipoUtente.equals(ConstantsGlobal.orete)) {
				criteriaCl.add(Restrictions.ge("distanza", 0));
			} 
			if(tipoUtente.equals(ConstantsGlobal.sotto) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				criteriaCl.add(Restrictions.ge("distanza", 1));
			} 
			if(tipoUtente.equals(ConstantsGlobal.prima) || tipoUtente.equals(ConstantsGlobal.oprima)) {
				criteriaCl.add(Restrictions.eq("distanza", 1));
			} 
			Integer max = userDao.getMaxLivelloRete();
			if(ApplicationUtil.gtZero(max)) {
				criteriaCl.add(Restrictions.le("distanza", max));
			}
			DetachedCriteria criteriaClF =  criteriaCl.createCriteria("agente", "agf");
			DetachedCriteria criteriaA =  criteriaClF.createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			criteriaP.add(Restrictions.idEq(prodotto.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			if(tipoUtente.equals(ConstantsGlobal.orete) || tipoUtente.equals(ConstantsGlobal.oprima) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				DetachedCriteria criteriaQQ = criteriaClF.createCriteria("qualifica" , "d", ApplicationUtil.leftJoin());
				if(differenziale!=null) {
					criteriaQQ.add(Restrictions.eq("id", differenziale.getId() ));
				} else {
					boolean c = true;
					if(ApplicationUtil.isTrue(c)) {
						criteriaQQ.add(Restrictions.isNull("id"));
					}
				}
			}
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			pl.add(Projections.groupProperty("p.id").as("p_id"));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			if(tipoUtente.equals(ConstantsGlobal.orete) || tipoUtente.equals(ConstantsGlobal.oprima) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				if(differenziale!=null) {
					pl.add(Projections.groupProperty("d.id").as("d_id"));
				}
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(modalitaPagamento(prodotto.getTipologia()));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica, differenziale, null, prodotto);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiProdottoCoordinatore(Boolean b, Qualifica qualifica, Prodotto prodotto, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.coordinatore;
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica, null,null,prodotto, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica, null,null,prodotto, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("zone" , "z").createCriteria("agente" , "a");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q",  ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			DetachedCriteria criteriaA =  criteriaAg.createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			criteriaP.add(Restrictions.idEq(prodotto.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			pl.add(Projections.groupProperty("p.id").as("p_id"));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(modalitaPagamento(prodotto.getTipologia()));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica,null,null, prodotto);
		}
		return estratti;
	}
	public void aggiungiUtenteAFatturaEstratto(Long fatturaId, Long utenteId) {
		FatturaEstratto fattura = getFullFatturaEstrattoUtente(fatturaId);
		Utente utente = userDao.getUser(utenteId);
		if (fattura != null && utente != null) {
			if (!fattura.getUtenti().contains(utente)) {
				fattura.getUtenti().add(utente);
				salvaAggiorna(fattura);
			}
		}
	}
	public void rimuoviUtenteDaFatturaEstratto(Long fatturaId, Long utenteId) {
		FatturaEstratto fattura = getFullFatturaEstrattoUtente(fatturaId);
		Utente utente = userDao.getUser(utenteId);
		if (fattura != null && utente != null) {
			if (fattura.getUtenti().contains(utente)) {
				fattura.getUtenti().remove(utente);
				salvaAggiorna(fattura);
			}
		}
	}
	public void aggiungiUtenteTuttiAFatturaEstratto(Long fatturaId, Integer tutti) {
		FatturaEstratto fattura = getFullFatturaEstrattoUtente(fatturaId);
		List<Utente> utenti = getUtentiNotInFatturaEstratto(fatturaId, tutti);
		if (fattura != null && ApplicationUtil.notEmpty(utenti)) {
			fattura.getUtenti().addAll(utenti);
			salvaAggiorna(fattura);
		}
	}
	public void rimuoviUtenteTuttiDaFatturaEstratto(Long fatturaId) {
		FatturaEstratto fattura = getFullFatturaEstrattoUtente(fatturaId);
		List<Utente> utenti = getTemplateByIdUtente(fatturaId);
		if (fattura != null && ApplicationUtil.notEmpty(utenti)) {
			fattura.getUtenti().removeAll(utenti);
			salvaAggiorna(fattura);
		}
	}
	public FatturaEstratto getFullFatturaEstrattoUtente(Long fatturaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(FatturaEstratto.class);
		criteria.add(Restrictions.idEq(fatturaId));
		criteria.setFetchMode("utenti", FetchMode.JOIN);
		return  (FatturaEstratto) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiNotInFatturaEstratto(Long fatturaId, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("fattura", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
				criteriaU.createCriteria("fattura").add(Restrictions.idEq(fatturaId));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("fattura", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
			criteriaU.createCriteria("fattura").add(Restrictions.idEq(fatturaId));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}

	public void  cambiaTipoSelezionati(List<Acconto> lst, Integer tipo) {
		Iterator<Acconto> itCont = lst.iterator();
		while (itCont.hasNext()) {
			Acconto acconto = itCont.next();
			acconto.setTipo(tipo);
			salvaAcconto(acconto);
		}
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiCategoriaOperatore(Categoria categoria, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.operatore;
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getInterno(Gettone.class, tipoUtente, categoria,null, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getInterno(Contest.class, tipoUtente, categoria,null, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaA = criteria.createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			DetachedCriteria criteriaCa =  criteriaP.createCriteria("categoria", "c");
			criteriaCa.add(Restrictions.idEq(categoria.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			criteria.setProjection(Projections.projectionList()
					.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")))
					.add(Projections.groupProperty("c.id").as("c_id"))
					.add(Projections.groupProperty("ap.stato").as("stato"))
					.add(modalitaPagamento(categoria.getTipologia()))
					);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, null, null, categoria, null);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiCategoriaSupervisore(Categoria categoria, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.supervisore;
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getInterno(Gettone.class, tipoUtente, categoria,null, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getInterno(Contest.class, tipoUtente, categoria,null, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaA =  criteria.createCriteria("teams" , "t").createCriteria("user", "o").createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			DetachedCriteria criteriaCa =  criteriaP.createCriteria("categoria", "c");
			criteriaCa.add(Restrictions.idEq(categoria.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			criteria.setProjection(Projections.projectionList()
					.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")))
					.add(Projections.groupProperty("c.id").as("c_id"))
					.add(Projections.groupProperty("ap.stato").as("stato"))
					.add(modalitaPagamento(categoria.getTipologia()))
					);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, null, null, categoria, null);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiCategoriaAgente(Boolean b, Qualifica qualifica, Categoria categoria, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.agente;
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica, null,categoria, null,Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica,null, categoria, null,Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("agenti" , "a");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q",  ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			DetachedCriteria criteriaA =  criteriaAg.createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			DetachedCriteria criteriaCa =  criteriaP.createCriteria("categoria", "c");
			criteriaCa.add(Restrictions.idEq(categoria.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			pl.add(Projections.groupProperty("c.id").as("c_id"));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(modalitaPagamento(categoria.getTipologia()));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica, null,  categoria, null);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiCategoriaRete(Boolean b, Qualifica qualifica, Qualifica differenziale, Categoria categoria, int statoInPagamento, int statoInStorno, Utente utente, Integer tipoUtente) {
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica, differenziale, categoria,null, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica, differenziale, categoria,null, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("agenti" , "a");
			DetachedCriteria criteriaCl = criteriaAg.createCriteria("closure_agenti_padre", "cl");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q", ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			if(tipoUtente.equals(ConstantsGlobal.rete) || tipoUtente.equals(ConstantsGlobal.orete)) {
				criteriaCl.add(Restrictions.ge("distanza", 0));
			} 
			if(tipoUtente.equals(ConstantsGlobal.sotto) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				criteriaCl.add(Restrictions.ge("distanza", 1));
			} 
			if(tipoUtente.equals(ConstantsGlobal.prima) || tipoUtente.equals(ConstantsGlobal.oprima)) {
				criteriaCl.add(Restrictions.eq("distanza", 1));
			} 
			Integer max = userDao.getMaxLivelloRete();
			if(ApplicationUtil.gtZero(max)) {
				criteriaCl.add(Restrictions.le("distanza", max));
			}
			DetachedCriteria criteriaClF =  criteriaCl.createCriteria("agente", "agf");
			DetachedCriteria criteriaA =  criteriaClF.createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			DetachedCriteria criteriaCa =  criteriaP.createCriteria("categoria", "c");
			criteriaCa.add(Restrictions.idEq(categoria.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			if(tipoUtente.equals(ConstantsGlobal.orete) || tipoUtente.equals(ConstantsGlobal.oprima) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				DetachedCriteria criteriaQQ = criteriaClF.createCriteria("qualifica" , "d", ApplicationUtil.leftJoin());
				if(differenziale!=null) {
					criteriaQQ.add(Restrictions.eq("id", differenziale.getId() ));
				} else {
					boolean c = true;
					if(ApplicationUtil.isTrue(c)) {
						criteriaQQ.add(Restrictions.isNull("id"));
					}
				}
			}
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			pl.add(Projections.groupProperty("c.id").as("c_id"));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			if(tipoUtente.equals(ConstantsGlobal.orete) || tipoUtente.equals(ConstantsGlobal.oprima) || tipoUtente.equals(ConstantsGlobal.osotto)) {
				if(differenziale!=null) {
					pl.add(Projections.groupProperty("d.id").as("d_id"));
				}
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(modalitaPagamento(categoria.getTipologia()));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica, differenziale, categoria, null);
		}
		return estratti;
	}
	@SuppressWarnings("unchecked")
	public List<EstrattoModel> generaGettoneContrattiCategoriaCoordinatore(Boolean b, Qualifica qualifica, Categoria categoria, int statoInPagamento, int statoInStorno, Utente utente) {
		Integer tipoUtente = ConstantsGlobal.coordinatore;
		String tipo = ConstantsLanguage.CONTRATTO.stampa();
		List<EstrattoModel> estratti =  new ArrayList<EstrattoModel>();
		DetachedCriteria criteriaG = getEsterno(b, Gettone.class, tipoUtente, qualifica, null,categoria,null, Boolean.TRUE);
		criteriaG.setProjection(Projections.sum("g.importo").as("importo"));
		BigDecimal gettone =  (BigDecimal) getUniqueResult(criteriaG);
		DetachedCriteria criteriaC = getEsterno(b, Contest.class, tipoUtente, qualifica, null,categoria, null, Boolean.TRUE);
		criteriaC.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("g.quota").as("quota")))
				.add(Projections.property("g.moltiplicatore").as("moltiplicatore"))
				.add(Projections.property("g.importo").as("importo"))
				);
		List<ContestModel> contests =  getList(criteriaC, ContestModel.class);
		if(gettone!=null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class, "u");
			if(utente!=null) {
				criteria.add(Restrictions.idEq(utente.getId()));
			}
			criteria.add(Restrictions.eq("generaEstratto", Boolean.TRUE));
			DetachedCriteria criteriaAg =  criteria.createCriteria("zone" , "z").createCriteria("agente" , "a");
			DetachedCriteria criteriaQ =  criteriaAg.createCriteria("qualifica" , "q",  ApplicationUtil.leftJoin());
			if(qualifica!=null) {
				criteriaQ.add(Restrictions.eq("id", qualifica.getId() ));
			} else {
				if(ApplicationUtil.isTrue(b)) {
					criteriaQ.add(Restrictions.isNull("id"));
				}
			}
			DetachedCriteria criteriaA =  criteriaAg.createCriteria("contratti", "ap");
			DetachedCriteria criteriaP =  criteriaA.createCriteria("prodotto", "p");
			DetachedCriteria criteriaCa =  criteriaP.createCriteria("categoria", "c");
			criteriaCa.add(Restrictions.idEq(categoria.getId() ));
			criteriaA.add(Restrictions.or(Restrictions.eq("stato",  statoInPagamento), Restrictions.eq("stato",  statoInStorno)));
			ProjectionList pl = Projections.projectionList();
			pl.add(Projections.distinct(Projections.groupProperty("u.id").as("utente_id")));
			pl.add(Projections.groupProperty("c.id").as("c_id"));
			if(qualifica!=null) {
				pl.add(Projections.groupProperty("q.id").as("q_id"));
			}
			pl.add(Projections.groupProperty("ap.stato").as("stato"));
			pl.add(modalitaPagamento(categoria.getTipologia()));
			criteria.setProjection(pl);
			estratti.addAll(getList(criteria, EstrattoModel.class));
			sistemaEstratti(estratti, contests, tipoUtente, tipo, gettone, qualifica,null, categoria, null);
		}
		return estratti;
	}
	public ExtendedCriteria trovaFatturaContrattoCriteria(FatturaContrattoCriteria ema) {
		if(ema != null)  ema = (FatturaContrattoCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(FatturaContratto.class);
		if(ApplicationUtil.isNotBlank(ema.getNomeOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ema.getNomeOggetto(), criteria, "nome");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<FatturaContratto> trovaFatturaContratto(FatturaContrattoCriteria ema, int start, int size) {
		ExtendedCriteria criteria = trovaFatturaContrattoCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaFatturaContrattoSize(FatturaContrattoCriteria ema) {
		ExtendedCriteria criteria =  trovaFatturaContrattoCriteria(ema);
		return getListSize(criteria);
	}
	public void getFatturaContratto(FatturaContratto fattura, DownloadBean downloadBean) {


		OutputStream out = downloadBean.outputStream();
		downloadBean.filename(fattura.getNomeFileS());
		try {
			InputStream bais = new ByteArrayInputStream(fattura.getContent());

			ApplicationUtil.copy(bais, out);
			downloadBean.close();
			bais.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public FatturaContratto getFatturaContratto(Long id) {
		return (FatturaContratto) get(FatturaContratto.class, id);
	}
	public void eliminaFatturaContratto(FatturaContratto fattura) {
		cancellaAggiorna(fattura);
	}
	public FatturaContratto nuovoFatturaContratto() {
		return new FatturaContratto();
	}
	public void inserisciFileInFatturaContratto(FileUpload fileUpload, FatturaContratto fattura) {
		List<MyFile> fls = fileUpload.getFiles();
		if(ApplicationUtil.notEmpty(fls)) {
			try { 
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					fattura.setNomeFile(ApplicationUtil.getNameN(ff.getName()));
					fattura.setContent(ff.getData());
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public FatturaContratto salvaFatturaContratto(FatturaContratto fattura) {
		if (fattura != null) {
			fattura = (FatturaContratto) salvaAggiorna(fattura);
		}
		return fattura;
	}
}