package it.callite.dao;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.criteria.AppuntamentoCriteria;
import it.callite.criteria.ContattoCriteria;
import it.callite.criteria.ContrattoCriteria;
import it.callite.criteria.NotaCriteria;
import it.callite.criteria.PrenotazioneCriteria;
import it.callite.criteria.StoriaAppuntamentoCriteria;
import it.callite.criteria.StoriaContattoCriteria;
import it.callite.criteria.StoriaContrattoCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Utente;
import it.callite.model.report.ReportBackground;
import it.callite.service.SmtpService;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.thread.export.EBaseThread;
import it.callite.utility.thread.export.EsportaAppuntamentoThread;
import it.callite.utility.thread.export.EsportaContattoThread;
import it.callite.utility.thread.export.EsportaContrattoThread;
import it.callite.utility.thread.export.EsportaNotaThread;
import it.callite.utility.thread.export.EsportaPrenotazioneThread;
import it.callite.utility.thread.export.EsportaStoriaAppuntamentoThread;
import it.callite.utility.thread.export.EsportaStoriaContattoThread;
import it.callite.utility.thread.export.EsportaStoriaContrattoThread;
@SuppressWarnings("serial")
@Repository("exportDao")
public class ExportDao extends GenericDao {
	@Autowired
	private AppuntamentoDao appuntamentoDao;
	@Autowired
	private ContattoDao contattoDao;
	@Autowired
	private ContrattoDao contrattoDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private PrenotazioneDao prenotazioneDao;
	@Autowired
	private StoriaDao storiaDao;
	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private SmtpService smtpService;

	private Map<String, EBaseThread> processi = new HashMap<String, EBaseThread>();

	public void exportAppuntamento(Boolean excel, Utente utente, AppuntamentoCriteria app) {
		String t = ConstantsLanguage.APPUNTAMENTI.stampa();
		EsportaAppuntamentoThread v = new EsportaAppuntamentoThread(t, "" + System.currentTimeMillis(), excel, smtpService, appuntamentoDao, operazioneDao, downloadDao, app, utente, ConstantsGlobal.recordSize);
		v.start();
		processi.put(v.getIdentificativo(), v);
	}
	public void exportContatto(Boolean excel, Utente utente, ContattoCriteria cont) {
		String t = ConstantsLanguage.CONTATTI.stampa();
		EsportaContattoThread v = new EsportaContattoThread(t, "" + System.currentTimeMillis(), excel, smtpService, contattoDao, operazioneDao, downloadDao, cont, utente, ConstantsGlobal.recordSize);
		v.start();
		processi.put(v.getIdentificativo(), v);
	}
	public void exportContratto(Boolean excel, Utente utente, ContrattoCriteria cont) {
		String t = ConstantsLanguage.CONTRATTI.stampa();
		EsportaContrattoThread v = new EsportaContrattoThread(t, "" + System.currentTimeMillis(), excel, smtpService, contrattoDao, operazioneDao, downloadDao, cont, utente, ConstantsGlobal.recordSize);
		v.start();
		processi.put(v.getIdentificativo(), v);
	}
	public void exportNota(Boolean excel, Utente utente, NotaCriteria cont) {
		String t = ConstantsLanguage.NOTE.stampa();
		EsportaNotaThread v = new EsportaNotaThread(t, "" + System.currentTimeMillis(), excel,  smtpService, appuntamentoDao, operazioneDao, downloadDao, cont, utente, ConstantsGlobal.recordSize);
		v.start();
		processi.put(v.getIdentificativo(), v);
	}
	public void exportPrenotazione(Boolean excel, Utente utente, PrenotazioneCriteria cont) {
		String t = ConstantsLanguage.PRENOTAZIONI.stampa();
		EsportaPrenotazioneThread v = new EsportaPrenotazioneThread(t, "" + System.currentTimeMillis(), excel, smtpService, prenotazioneDao, operazioneDao, downloadDao, cont, utente, ConstantsGlobal.recordSize);
		v.start();
		processi.put(v.getIdentificativo(), v);
	}
	public void exportStoriaAppuntamento(Boolean excel, Utente utente, StoriaAppuntamentoCriteria cont) {
		String t = ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTI.stampa();
		EsportaStoriaAppuntamentoThread v = new EsportaStoriaAppuntamentoThread(t, "" + System.currentTimeMillis(), excel, smtpService, storiaDao, operazioneDao, downloadDao, cont, utente, ConstantsGlobal.recordSize);
		v.start();
		processi.put(v.getIdentificativo(), v);
	}
	public void exportStoriaContatto(Boolean excel, Utente utente, StoriaContattoCriteria cont) {
		String t = ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTI.stampa();
		EsportaStoriaContattoThread v = new EsportaStoriaContattoThread(t, "" + System.currentTimeMillis(), excel, smtpService, storiaDao, operazioneDao, downloadDao, cont, utente, ConstantsGlobal.recordSize);
		v.start();
		processi.put(v.getIdentificativo(), v);
	}
	public void exportStoriaContratto(Boolean excel, Utente utente, StoriaContrattoCriteria cont) {
		String t = ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTI.stampa();
		EsportaStoriaContrattoThread v = new EsportaStoriaContrattoThread(t, "" + System.currentTimeMillis(), excel, smtpService, storiaDao, operazioneDao, downloadDao, cont, utente, ConstantsGlobal.recordSize);
		v.start();
		processi.put(v.getIdentificativo(), v);
	}
	public synchronized List<ReportBackground> getProcessi() {
		List<ReportBackground> processiList = new ArrayList<ReportBackground>();
		Set<String> remove = new HashSet<String>(); 
		Set<String> lngList = processi.keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<String> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				String ln = lnt.next();
				EBaseThread avv = processi.get(ln);
				if(avv!= null) {
					if(ApplicationUtil.isTrue(avv.getCompletato())) {
						remove.add(ln);
					} else {
						ReportBackground r = new ReportBackground(avv.getIdentificativo());
						r.setSize(avv.getSize());
						r.setCompletato(avv.getCompletato());
						r.setInterrompi(avv.getInterrompi());
						r.setScrittura(avv.getScrittura());
						r.setTipologia(avv.getTipologia());
						r.setCorrente(avv.getCorrente());
						r.setInizio(avv.getInizio());
						r.setFine(avv.getFine());
						r.setPercentuale(avv.getPercentuale());
						r.setTempo(avv.getTempo());
						r.setTipo(avv.getTipo());
						r.setTempoTotale(avv.getTempoTotale());
						r.setTotale(avv.getTotale());
						r.setUtente(avv.getUtente());
						processiList.add(r);
					}
				}
			}
		}
		processi.keySet().removeAll(remove);
		return processiList;
	}

	public synchronized void interrompi(ReportBackground v) {
		EBaseThread avvContatto = processi.get(v.getIdentificativo());
		if(avvContatto != null) {
			avvContatto.setInterrompi(true);
		}
	}
}