package it.callite.dao;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.faces.model.SelectItem;
import javax.servlet.ServletContext;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.comparator.FileComparator;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.FileCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
import it.callite.utility.utils.LanguageFileFilter;
@SuppressWarnings("serial")
@Repository("linguaDao")
public class LinguaDao extends GenericDao {
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private ServletContext servletContext;

	public List<MyFile> listaLingua(FileCriteria mm) { 
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> sis = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			List<File> ls = listaFile();
			if(ApplicationUtil.notEmpty(ls)) {
				List<String> rs = ApplicationUtil.token(mm.getFullText(), ConstantsGlobal.colonSeparator);
				for(File mod : ls) {
					String nameFile = ApplicationUtil.getBaseNameN(mod.getName()) + LanguageFileFilter.end;
					if(mod.isFile() && mm.isRicerca(mod.getName(), mod.length(), mod.lastModified()) && ApplicationUtil.textInTXT(mod, rs)) {
						MyFile f = new MyFile(mod);
						f.setName(nameFile);
						f.setModified(mod.lastModified());
						sis.add(f);
					}
				}
			}
		}
		return sis;
	}
	public List<File> listaFile() {
		File perc = getPathFile();
		if(perc != null) {
			ApplicationUtil.creaFile(true, perc);
			List<File> files =  new ArrayList<File>(Arrays.asList(perc.listFiles(new LanguageFileFilter())));
			Collections.sort(files, new FileComparator());
			return files;
		}
		return null;
	}
	public File getPathFile() {
		String path = servletContext.getRealPath("/WEB-INF/classes" + ConstantsGlobal.path_language);
		FileSystemResource resource=new FileSystemResource(path);
		return resource.getFile();
	}

	public File getPathAudio() {
		String path = servletContext.getRealPath("/sounds" + ConstantsGlobal.path_error);
		FileSystemResource resource=new FileSystemResource(path);
		return resource.getFile();
	}

	public void scaricaLingua(MyFile file, DownloadBean downloadBean) {
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file.getName());
			InputStream stream = new FileInputStream(getPathFile() + "/" + file.getName());  
			ApplicationUtil.copy(stream, out);
			stream.close();
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	public void getLingua(List<MyFile> mFiles, DownloadBean downloadBean) {
		try {
			File path = getPathFile();
			List<Allegato> files = new ArrayList<Allegato>();
			if (ApplicationUtil.notEmpty(mFiles)) {
				String p = ConstantsGlobal.secretZ;
				Iterator<MyFile> itS = mFiles.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					ApplicationUtil.addFile(files, file.getName(), path+"/"+file.getName(), file.getModified());
				}
				String fileName = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.LINGUA.stampa()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(fileName, files);
				ApplicationUtil.zip(null, files, out, p);
				downloadBean.close();
			}
		} catch(Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public String readLingua(MyFile ff) throws Exception {
		File file = new File(getPathFile() + "/" + ff.getName());
		if (file.exists()) {
			BufferedReader input = new BufferedReader(new FileReader(file));
			StringBuffer buffer = new StringBuffer();
			String text;
			while ((text = input.readLine()) != null)
				buffer.append(text + System.lineSeparator());
			input.close();
			return buffer.toString();
		}
		return null;
	}
	public void salvaLingua(MyFile ff, String content) throws Exception {
		File file = new File(getPathFile() + "/" + ff.getName());
		if (file.exists()) {
			FileOutputStream f = new FileOutputStream(file);
			PrintStream scrivi = new PrintStream(f);
			scrivi.println(content != null ? content.replaceAll(ConstantsGlobal.EOLINE, "") : content);
			scrivi.close();
		}
		String home = impostazioneDao.getHome();
		ApplicationUtil.copyFile(file, home + ConstantsGlobal.path_language + "/" + ff.getName());
		impostazioneDao.reloadBundle();
	}

	public void salvaLinguaS(String lingua, String paese, String content) throws Exception {
		File file = new File( getPathFile() + "/" + LanguageFileFilter.start + lingua + "_" + paese + LanguageFileFilter.end);
		//ApplicationUtil.creaFile(false, file);
		FileOutputStream f = new FileOutputStream(file);
		PrintStream scrivi = new PrintStream(f);
		scrivi.println(content != null ? content.replaceAll(ConstantsGlobal.EOLINE, "") : content);
		scrivi.close();
		String home = impostazioneDao.getHome();
		ApplicationUtil.copyFile(file, home + ConstantsGlobal.path_language + "/" + LanguageFileFilter.start +  lingua + "_" + paese + LanguageFileFilter.end);
		impostazioneDao.reloadBundle();
	}
	public void eliminaLingua(List<MyFile> file) {
		if(ApplicationUtil.notEmpty(file)) {
			Iterator<MyFile> it = file.iterator();
			while (it.hasNext()) {
				MyFile myFile = (MyFile) it.next();
				eliminaLingua(myFile);
			}
		}
	}
	public void eliminaLingua(MyFile file) {
		ApplicationUtil.deleteFile(getPathFile() + "/" + file);
		String home = impostazioneDao.getHome();
		ApplicationUtil.deleteFile(home + ConstantsGlobal.path_language + "/" + file);
		impostazioneDao.reloadBundle();
	}

	public void salvaLingua(FileUpload fileUpload) {
		List<MyFile> fls = fileUpload.getFiles();
		if(ApplicationUtil.notEmpty(fls)) {
			try {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					String rName = ApplicationUtil.trimToNull(ApplicationUtil.getNameN(ff.getName()));
					if(rName != null) {
						if(ApplicationUtil.linguaValida(rName, LanguageFileFilter.start, LanguageFileFilter.end)) {
							File f = new File(getPathFile() + "/" +  rName );
							FileOutputStream fis = new FileOutputStream(f);
							fis.write(ff.getData());
							fis.flush();
							fis.close();
							ApplicationUtil.copyFile(f, home + ConstantsGlobal.path_language + "/" +  rName);
						}
						else {
							ApplicationUtil.unzip(ff.getData(), getPathFile(), home, ConstantsGlobal.path_language, LanguageFileFilter.start, LanguageFileFilter.end);
						}
					}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
		impostazioneDao.reloadBundle();
	}

	public List<SelectItem> getLinguaSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for (String l : Locale.getISOLanguages()) {
			sis.add(new SelectItem(l, l));
		}
		return sis;
	}

	public List<SelectItem> getPaeseSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for (String l : Locale.getISOCountries()) {
			sis.add(new SelectItem(l, l));
		}
		return sis;
	}

}	