package it.callite.dao;
import java.util.List;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.criteria.QualificaCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Agente;
import it.callite.model.Qualifica;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.item.GenericSI;
@SuppressWarnings("serial")
@Repository("qualificaDao")
public class QualificaDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public Qualifica salvaQualifica(Qualifica qualifica) {
		return (Qualifica) salvaAggiorna(qualifica);
	}
	public void eliminaQualifica(Qualifica qualifica) {
		cancellaAggiorna(qualifica);
	}
	private ExtendedCriteria trovaCriteria(QualificaCriteria ana) {
		if(ana != null)  ana = (QualificaCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Qualifica.class);
		if (ApplicationUtil.isNotBlank(ana.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTesto(), criteria, "nome", "descrizione");
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Qualifica> trova(QualificaCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(QualificaCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public Qualifica get(Long id) {
		return (Qualifica) get(Qualifica.class, id);
	}
	public Agente getAgente(Long id) {
		return (Agente) get(Agente.class, id);
	}
	public List<SelectItem> getQualifiche() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Qualifica.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	
	public List<SelectItem> getQualificheN() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Qualifica.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, false);
	}
	
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiByQualificaId(Long qualificaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("qualifica").add(Restrictions.idEq(qualificaId));
		ApplicationUtil.orderA(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiNotInQualifica(Long qualifica, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("qualifica", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Agente.class);
				criteriaU.createCriteria("qualifica").add(Restrictions.idEq(qualifica));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("qualifica", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Agente.class);
			criteriaU.createCriteria("qualifica").add(Restrictions.idEq(qualifica));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderA(criteria);
		return getList(criteria);
	}
	public void aggiungiAgenteAQualifica(Long qualificaId, Long agenteId) {
		Qualifica qualifica = getFullQualificaAgente(qualificaId);
		Agente agente = getAgente(agenteId);
		if (qualifica != null && agente != null) {
			if (!qualifica.getAgenti().contains(agente)) {
				qualifica.getAgenti().add(agente);
				salvaAggiorna(qualifica);
			}
		}
	}
	public void rimuoviAgenteDaQualifica(Long qualificaId, Long agenteId) {
		Qualifica qualifica = getFullQualificaAgente(qualificaId);
		Agente agente = getAgente(agenteId);
		if (qualifica != null && agente != null) {
			if (qualifica.getAgenti().contains(agente)) {
				qualifica.getAgenti().remove(agente);
				salvaAggiorna(qualifica);
			}
		}
	}
	public void aggiungiAgenteTuttiAQualifica(Long qualificaId, Integer tutti) {
		Qualifica qualifica = getFullQualificaAgente(qualificaId);
		List<Agente> agenti = getAgentiNotInQualifica(qualificaId, tutti);
		if (qualifica != null && ApplicationUtil.notEmpty(agenti)) {
			qualifica.getAgenti().addAll(agenti);
			salvaAggiorna(qualifica);
		}
	}
	public void rimuoviAgenteTuttiDaQualifica(Long qualificaId) {
		Qualifica qualifica = getFullQualificaAgente(qualificaId);
		List<Agente> agenti = getAgentiByQualificaId(qualificaId);
		if (qualifica != null && ApplicationUtil.notEmpty(agenti)) {
			qualifica.getAgenti().removeAll(agenti);
			salvaAggiorna(qualifica);
		}
	}
	public Qualifica getFullQualificaAgente(Long qualificaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Qualifica.class);
		criteria.add(Restrictions.idEq(qualificaId));
		criteria.setFetchMode("agenti", FetchMode.JOIN);
		return  (Qualifica) getUniqueResult(criteria);
	}
}