package it.callite.dao;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.model.SelectItem;

import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsEsito;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.RiprogrammaContattoCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Campagna;
import it.callite.model.Contatto;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoChiamata;
import it.callite.model.Lista;
import it.callite.model.Utente;
import it.callite.model.report.ReportBackground;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.EsitoGenerico;
import it.callite.utility.RiprogrammaContatto;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.EsitoArchiviazioneSI;
import it.callite.utility.item.EsitoChiamataSI;
import it.callite.utility.item.ListaSI;
import it.callite.utility.thread.RiprogrammaContattoThread;
@SuppressWarnings("serial")
@Repository("riprogrammaDao")
public class RiprogrammaDao extends GenericDao {
	@Autowired
	private PannelloDao pannelloDao;
	@Autowired
	private ContattoDao contattoDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private AnagraficaDao anagraficaDao;
	public RiprogrammaContattoCriteria imposta(Boolean supervisore, Utente user, RiprogrammaContattoCriteria ta) {
		
		List<CampagnaSI> campagneSI = getAllCampagneSI(user, supervisore);
		campagneSI.removeAll(ta.getCampagne());
		ta.setCampagneSI(campagneSI);
		
		if(ApplicationUtil.notEmpty(ta.getCampagne())) {
			List<ListaSI> listeSI = getListeSI(ta);
			listeSI.removeAll(ta.getListe());
			ta.setListeSI(listeSI);
			List<EsitoGenerico> esitiSI = getAllEsitiSI(ta);
			esitiSI.removeAll(ta.getEsiti());
			ta.setEsitiSI(esitiSI);
		} else {
			ta.setListe(new ArrayList<ListaSI>());
			ta.setEsiti(new ArrayList<EsitoGenerico>());
			ta.setListeSI(new ArrayList<ListaSI>());
			ta.setEsitiSI(new ArrayList<EsitoGenerico>());
		}
		return ta;
	}
	public RiprogrammaContattoCriteria imposta(RiprogrammaContattoCriteria ta) {
		if(ApplicationUtil.notEmpty(ta.getCampagne())) {
			ta.setListe(new ArrayList<ListaSI>());
			ta.setEsiti(new ArrayList<EsitoGenerico>());
			ta.setListeSI(getListeSI(ta));
			ta.setEsitiSI(getAllEsitiSI(ta));
		} else {
			ta.setListe(new ArrayList<ListaSI>());
			ta.setEsiti(new ArrayList<EsitoGenerico>());
			ta.setListeSI(new ArrayList<ListaSI>());
			ta.setEsitiSI(new ArrayList<EsitoGenerico>());
		}
		return ta;
	}
	private List<CampagnaSI> getAllCampagneSI(Utente user, Boolean supervisore) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		if(user != null) {
			if(ApplicationUtil.isTrue(supervisore)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(user.getId(), criteria, "id");
			} else {
				impostazioneDao.subCriteriaCampagnaPartner(user.getId(), criteria, "id");
			}
		}
		List<CampagnaSI> ls = getCampagnaSI(criteria);
		if(ApplicationUtil.empty(ls)) {
			ls = new ArrayList<CampagnaSI>();
		}
		return ls;
	}
	
	public List<ListaSI> getListeSI(RiprogrammaContattoCriteria crit) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		if(ApplicationUtil.notEmpty(crit.getCampagne())) {
			List<Long> e1 = new ArrayList<Long>();
			Iterator<CampagnaSI> esC = crit.getCampagne().iterator();
			while (esC.hasNext()) {
				CampagnaSI ca = (CampagnaSI) esC.next();
				e1.add(ca.getId());
			}
			if(ApplicationUtil.notEmpty(e1)) {
				criteria.createCriteria("campagna").add(Restrictions.in("id",e1));
			}
		} 
		List<ListaSI> ls = getListaSI(criteria);
		if(ApplicationUtil.empty(ls)) {
			ls = new ArrayList<ListaSI>();
		}
		return ls;
	}
	@SuppressWarnings("unchecked")
	public List<EsitoGenerico> getAllEsitiSI(RiprogrammaContattoCriteria crit) {
		List<EsitoGenerico>  es = new ArrayList<EsitoGenerico>();
		DetachedCriteria criteria1 = DetachedCriteria.forClass(EsitoChiamata.class);
		List<Long> ids = new ArrayList<Long>();
		ids.add(ConstantsEsito.ID_OK);
		if(ApplicationUtil.notEmpty(ids)) {
			criteria1.add(Restrictions.not(Restrictions.in("id",ids) ));
		}
		ApplicationUtil.orderEsito(criteria1);
		List<EsitoChiamata> c = getList(criteria1);
		DetachedCriteria criteria2 = DetachedCriteria.forClass(EsitoArchiviazione.class);
		if(ApplicationUtil.notEmpty(crit.getCampagne())) {
			List<Long> e1 = new ArrayList<Long>();
			Iterator<CampagnaSI> esC = crit.getCampagne().iterator();
			while (esC.hasNext()) {
				CampagnaSI ca = (CampagnaSI) esC.next();
				e1.add(ca.getId());
			}
			if(ApplicationUtil.notEmpty(e1)) {
				criteria2.createCriteria("campagne").add(Restrictions.in("id",e1));
			}
		} 
		ApplicationUtil.orderEsito(criteria2);
		List<EsitoArchiviazione> a = getList(criteria2);
		if(ApplicationUtil.notEmpty(c)) {
			Iterator<EsitoChiamata> esC = c.iterator();
			while (esC.hasNext()) {
				EsitoChiamata esitoChiamata = (EsitoChiamata) esC.next();
				es.add(new EsitoGenerico(esitoChiamata, null));
			}
		}
		if(ApplicationUtil.notEmpty(a)) {
			Iterator<EsitoArchiviazione> esA = a.iterator();
			while (esA.hasNext()) {
				EsitoArchiviazione esitoArchiviazione = (EsitoArchiviazione) esA.next();
				es.add(new EsitoGenerico(null, esitoArchiviazione));
			}
		}
		if(ApplicationUtil.notEmpty(crit.getEsiti())) {
			es.removeAll(crit.getEsiti());
		}
		if(ApplicationUtil.empty(es)) {
			es = new ArrayList<EsitoGenerico>();
		}
		return es;
	}
	public void riprogrammaContatto(Boolean esistente, Lista listaM, Boolean sposta, Long campagnaId, Long listaId, boolean supervisore, Utente utente, Long operatoreId, RiprogrammaContattoCriteria cont, Long esitoId, Long archiviazioneId, Boolean elimina_storia, Boolean elimina_nota, Date data, Integer step, Boolean modo) {
		EsitoChiamata esito = pannelloDao.getEsitoChiamataById(esitoId);
		EsitoArchiviazione archiviazione = pannelloDao.getEsitoArchiviazioneById(archiviazioneId);
		Utente operatore = pannelloDao.getOperatore(operatoreId);
		if(esito != null) {
			RiprogrammaContatto c = new RiprogrammaContatto(cont);
			RiprogrammaContattoThread v = new RiprogrammaContattoThread("" + System.currentTimeMillis(), esistente, listaM, sposta, anagraficaDao, campagnaId, listaId, supervisore, this, c, utente, operatore, ConstantsGlobal.flushSize, elimina_storia, elimina_nota, esito, archiviazione, data, step, modo);
			v.start();
			processi.put(v.getIdentificativo(), v);
		}
	}
	public List<SelectItem> getEsitiChiamata(RiprogrammaContattoCriteria crit) {
		List<Long> ids = new ArrayList<Long>();
		if(!impostazioneDao.getResetContatto()) {
			ids.add(ConstantsEsito.ID_DA_CHIAMARE);
		} 
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		if(ApplicationUtil.notEmpty(ids)) {
			criteria.add(Restrictions.not(Restrictions.in("id",ids) ));
		}
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazione(Utente utente, RiprogrammaContattoCriteria crit) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		if(ApplicationUtil.notEmpty(crit.getCampagne())) {
			List<Long> e1 = new ArrayList<Long>();
			Iterator<CampagnaSI> esC = crit.getCampagne().iterator();
			while (esC.hasNext()) {
				CampagnaSI ca = (CampagnaSI) esC.next();
				e1.add(ca.getId());
			}
			if(ApplicationUtil.notEmpty(e1)) {
				criteria.createCriteria("campagne").add(Restrictions.in("id",e1));
			}
		} 
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	private Map<String, RiprogrammaContattoThread> processi = new HashMap<String, RiprogrammaContattoThread>();
	public synchronized List<ReportBackground> getProcessi() {
		List<ReportBackground> processiList = new ArrayList<ReportBackground>();
		Set<String> remove = new HashSet<String>(); 
		Set<String> lngList = processi.keySet();
		if(ApplicationUtil.notEmpty(lngList)) {
			Iterator<String> lnt = lngList.iterator();
			while (lnt.hasNext()) {
				String ln = lnt.next();
				RiprogrammaContattoThread avv = processi.get(ln);
				if(avv!= null) {
					if(ApplicationUtil.isTrue(avv.getCompletato())) {
						remove.add(ln);
					} else {
						ReportBackground r = new ReportBackground(avv.getIdentificativo());
						r.setSize(avv.getSize());
						r.setCompletato(avv.getCompletato());
						r.setInterrompi(avv.getInterrompi());
						r.setCorrente(avv.getCorrente());
						r.setInizio(avv.getInizio());
						r.setFine(avv.getFine());
						r.setPercentuale(avv.getPercentuale());
						r.setTempo(avv.getTempo());
						r.setProcesso(avv.getProcesso());
						r.setTempoTotale(avv.getTempoTotale());
						r.setTotale(avv.getTotale());
						r.setUtente(avv.getUtente());
						processiList.add(r);
					}
				}
			}
		}
		processi.keySet().removeAll(remove);
		return processiList;
	}
	public synchronized void interrompi(ReportBackground v) {
		RiprogrammaContattoThread avv = processi.remove(v.getIdentificativo());
		if(avv!= null) {
			avv.setInterrompi(true);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Contatto> riprogrammaContatto(boolean supervisore, List<Long> ln, Utente user, RiprogrammaContatto cont,  int start, int size) {
		DetachedCriteria criteria = riprogrammaContattoCriteria(supervisore, ln, user, cont);
		criteria.addOrder(Order.asc("id"));
		return getList(criteria, start, size);
	}
	public int riprogrammaContattoSize(boolean supervisore, List<Long> ln, Utente user, RiprogrammaContatto cont) {
		DetachedCriteria criteria = riprogrammaContattoCriteria(supervisore, ln, user, cont);
		return getListSize(criteria);
	}
	private DetachedCriteria riprogrammaContattoCriteria(boolean supervisore, List<Long> ln, Utente user, RiprogrammaContatto cont) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
		if(ApplicationUtil.notEmpty(ln)) {
			criteria.add(Restrictions.not(Restrictions.in("id",ln)));
		}
		if(ApplicationUtil.notEmpty(cont.getCampagne())) {
			List<Long> e1 = new ArrayList<Long>();
			Iterator<CampagnaSI> esC = cont.getCampagne().iterator();
			while (esC.hasNext()) {
				CampagnaSI c = (CampagnaSI) esC.next();
				e1.add(c.getId());
			}
			if(ApplicationUtil.notEmpty(e1)) {
				criteria.add(Restrictions.in("campagna.id",e1));
			}
		} 
		if(ApplicationUtil.notEmpty(cont.getListe())) {
			List<Long> e1 = new ArrayList<Long>();
			Iterator<ListaSI> esC = cont.getListe().iterator();
			while (esC.hasNext()) {
				ListaSI l = (ListaSI) esC.next();
				e1.add(l.getId());
			}
			if(ApplicationUtil.notEmpty(e1)) {
				criteria.add(Restrictions.in("lista.id",e1));
			}
		} 
		if(ApplicationUtil.notEmpty(cont.getEsiti())) {
			List<Long> e1 = new ArrayList<Long>();
			List<Long> e2 = new ArrayList<Long>();
			Iterator<EsitoGenerico> esC = cont.getEsiti().iterator();
			while (esC.hasNext()) {
				EsitoGenerico e = (EsitoGenerico) esC.next();
				if(e.isChiamata()) {
					e1.add(e.getId());
				} else {
					e2.add(e.getId());
				}
			}
			if(ApplicationUtil.notEmpty(e1) && ApplicationUtil.notEmpty(e2)) {
				Criterion c1 = Restrictions.in("esitoChiamata.id",e1);
				Criterion c2 = Restrictions.in("esitoArchiviazione.id",e2);
				criteria.add(Restrictions.or(c1, c2));
			}
			if(ApplicationUtil.notEmpty(e1) && ApplicationUtil.empty(e2)) {
				Criterion c1 = Restrictions.in("esitoChiamata.id",e1);
				criteria.add(c1);
			}
			if(ApplicationUtil.empty(e1) && ApplicationUtil.notEmpty(e2)) {
				Criterion c2 = Restrictions.in("esitoArchiviazione.id",e2);
				criteria.add(c2);
			}
			
			if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
				DetachedCriteria criteriaA = criteria.createCriteria("anagrafica");
				ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
			}
		}
		ApplicationUtil.intervallo(ApplicationUtil.isTrue(cont.getInserimento()), cont.getInizio(), cont.getFine(), criteria);
		return criteria;
	}
	public void riprogrammaPubblico(Boolean elimina_storia, Boolean elimina_nota, Utente utente, List<Contatto> contatti, Date data, EsitoChiamata esito) {
		if (ApplicationUtil.notEmpty(contatti)) {
			contattoDao.nascondiStoria(elimina_storia, contatti);
			Iterator<Contatto> objIter = contatti.iterator();
			while(objIter.hasNext()) {
				Contatto contatto = objIter.next();
				if(contatto != null) {
					contatto.imposta(data, esito, null);
					if(ApplicationUtil.isTrue(elimina_nota)) {
						contatto.setNota(null);
					}
					if(!ConstantsGlobal.massivo) {
						pannelloDao.salvaContatto(false, contatto, utente);
					}
				}
			}
			if(ConstantsGlobal.massivo) {
				contattoDao.salvaContatti(false, contatti, utente, false);
			}
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PUBBLICO);
		}
	}
	public void riprogrammaPrivato(Boolean elimina_storia, Boolean elimina_nota, Utente utente, Utente operatore, List<Contatto> contatti, Date data, EsitoChiamata esito) {
		if (ApplicationUtil.notEmpty(contatti)) {
			contattoDao.nascondiStoria(elimina_storia, contatti);
			Iterator<Contatto> objIter = contatti.iterator();
			while(objIter.hasNext()) {
				Contatto contatto = objIter.next();
				if(contatto != null) {
					contatto.imposta(data, esito, operatore);
					if(ApplicationUtil.isTrue(elimina_nota)) {
						contatto.setNota(null);
					}
					if(!ConstantsGlobal.massivo) {
						pannelloDao.salvaContatto(false, contatto, utente);
					}
				}
			}
			if(ConstantsGlobal.massivo) {
				contattoDao.salvaContatti(false, contatti, utente, false);
			}
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_PRIVATO);
		}
	}
	public void riprogrammaArchiviato(Utente operatore, Boolean elimina_storia, Boolean elimina_nota, Utente utente, List<Contatto> contatti, Date data, EsitoChiamata esito, EsitoArchiviazione archiviazione) {
		if (ApplicationUtil.notEmpty(contatti)) {
			contattoDao.nascondiStoria(elimina_storia, contatti);
			Iterator<Contatto> objIter = contatti.iterator();
			while(objIter.hasNext()) {
				Contatto contatto = objIter.next();
				if(contatto != null) {
					contatto.setEsitoChiamata(esito);
					contatto.setEsitoArchiviazione(archiviazione);
					if (archiviazione != null && ApplicationUtil.gtZero(archiviazione.getTipologiaS())) {
						if (ApplicationUtil.eq(archiviazione.getTipologiaS(), 1)) {
							contatto.proprietario(data, operatore);
							contatto.archivia(true);
						}
						if (ApplicationUtil.eq(archiviazione.getTipologiaS(), 2)) {
							contatto.proprietario(data, null);
							contatto.pubblico();
						}
						if (ApplicationUtil.eq(archiviazione.getTipologiaS(), 3)) {
							data = ApplicationUtil.date();
							contatto.proprietario(ApplicationUtil.addMinutes(data, archiviazione.getAttesa()), null);
							contatto.pubblico();
						}
					} else {
						contatto.archivia(false);
					}
					if(ApplicationUtil.isTrue(elimina_nota)) {
						contatto.setNota(null);
					}
					if(!ConstantsGlobal.massivo) {
						pannelloDao.salvaContatto(false, contatto, utente);
					}
				}
			}
			if(ConstantsGlobal.massivo) {
				contattoDao.salvaContatti(false, contatti, utente, false);
			}
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_GENERICO);
		}
	}
	public void resetContatto(Boolean elimina_storia, Boolean elimina_nota, List<Contatto> contatti,  Utente utente, Date data, EsitoChiamata esito) {
		if(ApplicationUtil.notEmpty(contatti)) {
			if(esito!=null) {
				contattoDao.nascondiStoria(elimina_storia, contatti);
				Iterator<Contatto> objIter = contatti.iterator();
				while(objIter.hasNext()) {
					Contatto contatto = objIter.next();
					if(contatto != null) {
						contatto.imposta(data);
						if(ApplicationUtil.isTrue(elimina_nota)) {
							contatto.setNota(null);
						}
						if(!ConstantsGlobal.massivo) {
							pannelloDao.salvaContattoC(false, null, contatto, utente, esito, false);
						} else {
							pannelloDao.esitaContattoC(contatto, esito, utente);
						}
					}
				}
				if(ConstantsGlobal.massivo) {
					contattoDao.salvaContatti(false, contatti, utente, false);
				}
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG5,  p, utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_RESET);
			}
		}
	}
	public void esitaContatto(Boolean elimina_storia, Boolean elimina_nota, List<Contatto> contatti, Utente utente, EsitoChiamata esito) {
		if(ApplicationUtil.notEmpty(contatti)) {
			if(esito!=null) {
				contattoDao.nascondiStoria(elimina_storia, contatti);
				Iterator<Contatto> objIter = contatti.iterator();
				while(objIter.hasNext()) {
					Contatto contatto = objIter.next();
					if(contatto != null) {
						if(ApplicationUtil.isTrue(elimina_nota)) {
							contatto.setNota(null);
						}
						if(!ConstantsGlobal.massivo) {
							pannelloDao.salvaContattoC(false, null, contatto, utente, esito, false);
						} else {
							pannelloDao.esitaContattoC(contatto, esito, utente);
						}
					}
				}
				if(ConstantsGlobal.massivo) {
					contattoDao.salvaContatti(false, contatti, utente, false);
				}
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
				operazioneDao.creaOperazione(ConstantsMap.LOG5, p,  utente, ConstantsMap.OPERAZIONE_RIPROGRAMMA_GENERICO);
			}
		}
	}
	public void eliminaContatto(List<Contatto> contatti, Utente utente) {
		if(ApplicationUtil.notEmpty(contatti)) {
			contattoDao.eliminaContatti(contatti);
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG3,  p, utente, ConstantsMap.OPERAZIONE_ELIMINA_CONTATTO);
		}
	}
	public void revertContatto(List<Contatto> contatti, Utente utente, Date data, Integer step, Boolean modo) {
		if(ApplicationUtil.notEmpty(contatti)) {
			contattoDao.revertContatto(false, contatti, utente, data, step, modo);
		}
	}
}