package it.callite.dao;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.hibernate.FetchMode;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.type.LongType;
import org.hibernate.type.StringType;
import org.hibernate.type.Type;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.column.generic.GenericColumn;
import it.callite.constants.ConstantsColumn;
import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.AutodialerCriteria;
import it.callite.criteria.CdrCriteria;
import it.callite.criteria.CelCriteria;
import it.callite.criteria.CodaCriteria;
import it.callite.criteria.EstensioneCriteria;
import it.callite.criteria.FileCriteria;
import it.callite.criteria.InboundCriteria;
import it.callite.criteria.IngressoCriteria;
import it.callite.criteria.RispostaCriteria;
import it.callite.criteria.SipCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.message.Message;
import it.callite.model.Autodialer;
import it.callite.model.Campagna;
import it.callite.model.Inbound;
import it.callite.model.Ingresso;
import it.callite.model.Risposta;
import it.callite.model.Ruolo;
import it.callite.model.Utente;
import it.callite.model.VocalOrder;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.model.report.RiepilogoCdr;
import it.callite.model.report.RiepilogoCdrInserimento;
import it.callite.model.sip.Cdr;
import it.callite.model.sip.Cel;
import it.callite.model.sip.Coda;
import it.callite.model.sip.ExtensionsTable;
import it.callite.model.sip.SipBuddies;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.LoggerColonna;
import it.callite.utility.MyFile;
import it.callite.utility.PathUtility;
import it.callite.utility.UtilChat;
import it.callite.utility.UtilFTP;
import it.callite.utility.ZipPassword;
import it.callite.utility.ZipUtils;
import it.callite.utility.chat.CreateBuddyThread;
import it.callite.utility.chat.DeleteBuddyThread;
import it.callite.utility.chat.SendBuddyThread;
import it.callite.utility.item.GenericSI;
import it.callite.utility.item.InternoSI;
import it.callite.utility.item.TrunkSI;
@SuppressWarnings("serial")
@Repository("sipDao")
public class SipDao extends GenericDao {
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private AutodialerDao autodialerDao;
	@Autowired
	private UserDao userDao;
	@Autowired
	private DataDao dataDao;
	
	public SipBuddies salva(SipBuddies sip_buddy) {
		return (SipBuddies) salvaAggiorna(sip_buddy);
	}
	public SipBuddies get(Long id){
		return (SipBuddies) get(SipBuddies.class, id);
	}
	public void eliminaInterno(SipBuddies idSip) {
		cancellaAggiorna(idSip);
		UtilChat uc = impostazioneDao.getUtilChat();
		new DeleteBuddyThread(uc, null, idSip).start();
		autodialerDao.sendInterno(idSip, false);
	}
	public void inviaInterno(SipBuddies idSip, String messaggio, Boolean invia) {
		UtilChat uc = impostazioneDao.getUtilChat();
		new SendBuddyThread(uc, null, idSip, messaggio, invia, autodialerDao).start();
	}

	public SipBuddies salvaInternoNuovo(SipBuddies nuovo) {
		nuovo =  salva(nuovo);
		UtilChat uc = impostazioneDao.getUtilChat();
		new CreateBuddyThread(uc, null, nuovo).start();
		autodialerDao.sendInterno(nuovo, true);
		return nuovo;
	}

	public SipBuddies salvaInternoModifica(SipBuddies nuovo) {
		SipBuddies vecchio = get(nuovo.getId());
		nuovo =  salva(nuovo);
		if(!nuovo.getName().equals(vecchio.getName())) {
			UtilChat uc = impostazioneDao.getUtilChat();
			new DeleteBuddyThread(uc, null, vecchio).start();
			new CreateBuddyThread(uc, null, nuovo).start();
			autodialerDao.sendInterno(vecchio, false);
			autodialerDao.sendInterno(nuovo, true);
		}
		return nuovo;
	}

	public SipBuddies salvaTrunk(SipBuddies nuovo) {
		return salva(nuovo);
	}
	public void inviaInterni(List<SipBuddies> lst, String messaggio, Boolean invia) {
		UtilChat uc = impostazioneDao.getUtilChat();
		new SendBuddyThread(uc, lst, null, messaggio, invia, autodialerDao).start();
	}
	public void elimina(SipBuddies idSip) {
		cancellaAggiorna(idSip);
	}
	public SipBuddies getByName(String name){
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("name", name));
		return  (SipBuddies) getUniqueResult(criteria);
	}
	
	public ExtensionsTable getById(Long id){
		DetachedCriteria criteria = DetachedCriteria.forClass(ExtensionsTable.class);
		criteria.add(Restrictions.idEq(id));
		return  (ExtensionsTable) getUniqueResult(criteria);
	}
	public List<SelectItem> getInterni() {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaI = criteriaU.createCriteria("interno");
		criteriaI.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		if(!impostazioneDao.getInternoMultiplo()) {
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		List<InternoSI> lis = getInternoSI(criteria);
		return impostazioneDao.getInterno(lis, false, false, null);
	}
	
	public List<SelectItem> getTrunkSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(false)));
		List<TrunkSI> lis = getTrunkSI(criteria);
		return impostazioneDao.getTrunk(true, lis, false, false, null);
	}

	public List<SelectItem> getInterniASI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		criteria.createCriteria("utenti").add(Restrictions.isNotNull("id"));
		List<InternoSI> lis = getInternoSI(criteria);
		return impostazioneDao.getInterno(lis, true, true, null);
	}
	public List<SelectItem> getTrunkASI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(false)));
		criteria.createCriteria("users").add(Restrictions.isNotNull("id"));
		List<TrunkSI> lis = getTrunkSI(criteria);
		return impostazioneDao.getTrunk(true, lis, true, true, null);
	}
	public List<SelectItem> getInterniModifica(Utente utente) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaI = criteriaU.createCriteria("interno");
		criteriaI.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		if(!impostazioneDao.getInternoMultiplo()) {
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		List<InternoSI> lis = getInternoSI(criteria);
		return impostazioneDao.getInterno(lis, false, false, utente.getInterno());
	}
	private ExtendedCriteria trovaCriteria(boolean b, SipCriteria ana) {
		if(ana != null)  ana = (SipCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		if(b) {
			criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		} else {
			criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(false)));
		}
		if (ApplicationUtil.isNotBlank(ana.getName())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getName(), criteria, "name", "username");
		}
		if (ApplicationUtil.isNotBlank(ana.getCallers())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getCallers(), criteria, "callers");
		}
		if (ApplicationUtil.gtZero(ana.getInterurbane())) {
			if (ApplicationUtil.eq(ana.getInterurbane(), 1))  {
				criteria.add(Restrictions.eq("interurbane", ApplicationUtil.convertBS(true)));
			} else {
				criteria.add(Restrictions.or(Restrictions.isNull("interurbane"), Restrictions.eq("interurbane", ApplicationUtil.convertBS(false))));
			} 
		}
		if (ApplicationUtil.gtZero(ana.getAttivo())) {
			if (ApplicationUtil.eq(ana.getAttivo(), 1))  {
				criteria.add(Restrictions.eq("attivo", ApplicationUtil.convertBS(true)));
			} else {
				criteria.add(Restrictions.or(Restrictions.isNull("attivo"), Restrictions.eq("attivo", ApplicationUtil.convertBS(false))));
			} 
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<SipBuddies> trova(boolean b, SipCriteria ana,int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(b,ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.asc("name")});
		return getList(criteria, start, size);
	}
	public int trovaSize(boolean b, SipCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(b,ana);
		return getListSize(criteria);
	}
	public Autodialer salva(Autodialer autodialer) {
		return (Autodialer) salvaAggiorna(autodialer);
	}
	public void elimina(Autodialer auto) {
		cancellaAggiorna(auto);
	}
	private ExtendedCriteria trovaCriteria(AutodialerCriteria auto) {
		if(auto != null)  auto = (AutodialerCriteria) auto.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		if (ApplicationUtil.isNotBlank(auto.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, auto.getNome(), criteria, "nome");
		}
		auto.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, auto);
	}
	@SuppressWarnings("unchecked")
	public List<Autodialer> trova(AutodialerCriteria auto,int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(auto);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), auto.getO_tipo(), auto.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(AutodialerCriteria auto) {
		ExtendedCriteria criteria = trovaCriteria(auto);
		return getListSize(criteria);
	}
	public Autodialer getAutodialer(Long id){
		return (Autodialer) get(Autodialer.class, id);
	}
	public SipBuddies interno(SipBuddies sc){
		SipBuddies sp = new SipBuddies(true, impostazioneDao.getDefaultCodec());
		if(sc != null) {
			sp.setAttivo(sc.getAttivo());
			sp.setCallers(sc.getCallers());
			sp.setInterne(sc.getInterne());
			sp.setInterurbane(sc.getInterurbane());
			sp.setOpposizione(sc.getOpposizione());
			sp.setRegistrazione(sc.getRegistrazione());
			sp.setPhone(sc.getPhone());
			sp.setTastierino(sc.getTastierino());
			sp.setAdd_prefisso(sc.getAdd_prefisso());
			sp.setMax_contacts(sc.getMax_contacts());
			sp.setPickup_group(sc.getPickup_group());
			sp.setCall_group(sc.getCall_group());
			sp.setContext(sc.getContext());
			sp.setDisallow(sc.getDisallow());
			sp.setAllow(sc.getAllow());
		}
		return sp;
	}
	public SipBuddies trunk(SipBuddies sc){
		SipBuddies sp = new SipBuddies(false, impostazioneDao.getDefaultCodec());
		if(sc != null) {
			sp.setAttivo(sc.getAttivo());
			sp.setCallers(sc.getCallers());
			sp.setMusicclass(sc.getMusicclass());
			sp.setVrx(sc.getVrx());
			sp.setVtx(sc.getVtx());
			sp.setInterurbane(sc.getInterurbane());
			sp.setOpposizione(sc.getOpposizione());
			sp.setRegistrazione(sc.getRegistrazione());
			sp.setAdd_prefisso(sc.getAdd_prefisso());
			sp.setAntecall(sc.getAntecall());
			sp.setMax_contacts(sc.getMax_contacts());
			sp.setHost(sc.getHost());
			sp.setPort(sc.getPort());
			sp.setPickup_group(sc.getPickup_group());
			sp.setContext(sc.getContext());
			sp.setCall_group(sc.getCall_group());
			sp.setDisallow(sc.getDisallow());
			sp.setAllow(sc.getAllow());
		}
		return sp;
	}
	@SuppressWarnings("unchecked")
	public List<ExtensionsTable> trova(EstensioneCriteria ext, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ext);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("context"),Order.asc("exten"),Order.asc("priority")});
		return getList(criteria, start, size);
	}
	public int trovaSize(EstensioneCriteria ext) {
		ExtendedCriteria criteria = trovaCriteria(ext);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteria(EstensioneCriteria ext) {
		if(ext != null)  ext = (EstensioneCriteria) ext.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ExtensionsTable.class);
		if (ApplicationUtil.isNotBlank(ext.getApp())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ext.getApp(), criteria, "app", "appdata");
		}
		if (ApplicationUtil.isNotBlank(ext.getEstensione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ext.getEstensione(), criteria, "exten");
		}
		if (ApplicationUtil.isNotBlank(ext.getContext())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ext.getContext(), criteria, "context");
		}
		ext.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ext);
	}
	
	public List<SelectItem> getExtenSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(ExtensionsTable.class);
		List<GenericSI> lis = getGenericSI(criteria, "exten", "exten");
		return impostazioneDao.getGeneric(lis, true);
	}
	
	public List<SelectItem> getContextSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(ExtensionsTable.class);
		List<GenericSI> lis = getGenericSI(criteria, "context", "context");
		return impostazioneDao.getGeneric(lis, true);
	}
	
	public void elimina(ExtensionsTable ext) {
		cancellaAggiorna(ext);
	}
	public ExtensionsTable salva(ExtensionsTable ext) {
		return (ExtensionsTable) salvaAggiorna(ext);
	}
	@SuppressWarnings("unchecked")
	public List<ExtensionsTable> getEstensioniByTrunk(Long trunk) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ExtensionsTable.class);
		criteria.createCriteria("trunk").add(Restrictions.idEq(trunk));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiByTrunkId(Long trunkId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("interno", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
		criteria.createCriteria("trunk").add(Restrictions.idEq(trunkId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiNotInTrunk(Long trunk, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaI = criteria.createCriteria("interno", ApplicationUtil.leftJoin());
		criteriaI.add(Restrictions.isNotNull("id"));
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("trunk", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
				criteriaU.createCriteria("trunk").add(Restrictions.idEq(trunk));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("trunk", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
			criteriaU.createCriteria("trunk").add(Restrictions.idEq(trunk));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public void aggiungiUtenteATrunk(Long trunkId, Long utenteId) {
		SipBuddies trunk = getFullTrunkUtente(trunkId);
		Utente utente = userDao.getUser(utenteId);
		if (trunk != null && utente != null) {
			if (!trunk.getUsers().contains(utente)) {
				trunk.getUsers().add(utente);
				salvaAggiorna(trunk);
			}
		}
	}
	public void rimuoviUtenteDaTrunk(Long trunkId, Long utenteId) {
		SipBuddies trunk = getFullTrunkUtente(trunkId);
		Utente utente = userDao.getUser(utenteId);
		if (trunk != null && utente != null) {
			if (trunk.getUsers().contains(utente)) {
				trunk.getUsers().remove(utente);
				salvaAggiorna(trunk);
			}
		}
	}
	public void aggiungiUtenteTuttiATrunk(Long trunkId, Integer tutti) {
		SipBuddies trunk = getFullTrunkUtente(trunkId);
		List<Utente> utenti = getUtentiNotInTrunk(trunkId, tutti);
		if (trunk != null && ApplicationUtil.notEmpty(utenti)) {
			trunk.getUsers().addAll(utenti);
			salvaAggiorna(trunk);
		}
	}
	public void rimuoviUtenteTuttiDaTrunk(Long trunkId) {
		SipBuddies trunk = getFullTrunkUtente(trunkId);
		List<Utente> utenti = getUtentiByTrunkId(trunkId);
		if (trunk != null && ApplicationUtil.notEmpty(utenti)) {
			trunk.getUsers().removeAll(utenti);
			salvaAggiorna(trunk);
		}
	}
	public SipBuddies getFullTrunkUtente(Long trunkId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.idEq(trunkId));
		criteria.setFetchMode("users", FetchMode.JOIN);
		return  (SipBuddies) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Autodialer> getAutodialerByTrunkId(Long trunkId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		criteria.createCriteria("trunk").add(Restrictions.idEq(trunkId));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Autodialer> getAutodialerNotInTrunk(Long trunk, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("trunk", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Autodialer.class);
				criteriaU.createCriteria("trunk").add(Restrictions.idEq(trunk));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("trunk", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Autodialer.class);
			criteriaU.createCriteria("trunk").add(Restrictions.idEq(trunk));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	public void aggiungiAutodialerATrunk(Long trunkId, Long autodialerId) {
		SipBuddies trunk = getFullTrunkAutodialer(trunkId);
		Autodialer autodialer = getAutodialer(autodialerId);
		if (trunk != null && autodialer != null) {
			if (!trunk.getAutodialer().contains(autodialer)) {
				trunk.getAutodialer().add(autodialer);
				salvaAggiorna(trunk);
			}
		}
	}
	public void rimuoviAutodialerDaTrunk(Long trunkId, Long autodialerId) {
		SipBuddies trunk = getFullTrunkAutodialer(trunkId);
		Autodialer autodialer = getAutodialer(autodialerId);
		if (trunk != null && autodialer != null) {
			if (trunk.getAutodialer().contains(autodialer)) {
				trunk.getAutodialer().remove(autodialer);
				salvaAggiorna(trunk);
			}
		}
	}
	public void aggiungiAutodialerTuttiATrunk(Long trunkId, Integer tutti) {
		SipBuddies trunk = getFullTrunkAutodialer(trunkId);
		List<Autodialer> autodialer = getAutodialerNotInTrunk(trunkId, tutti);
		if (trunk != null && ApplicationUtil.notEmpty(autodialer)) {
			trunk.getAutodialer().addAll(autodialer);
			salvaAggiorna(trunk);
		}
	}
	public void rimuoviAutodialerTuttiDaTrunk(Long trunkId) {
		SipBuddies trunk = getFullTrunkAutodialer(trunkId);
		List<Autodialer> autodialer = getAutodialerByTrunkId(trunkId);
		if (trunk != null && ApplicationUtil.notEmpty(autodialer)) {
			trunk.getAutodialer().removeAll(autodialer);
			salvaAggiorna(trunk);
		}
	}
	public SipBuddies getFullTrunkAutodialer(Long trunkId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.idEq(trunkId));
		criteria.setFetchMode("autodialer", FetchMode.JOIN);
		return  (SipBuddies) getUniqueResult(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiByInternoId(Long internoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("interno").add(Restrictions.idEq(internoId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiNotInInterno() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("interno", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public void aggiungiUtenteAInterno(Long internoId, Long utenteId) {
		SipBuddies interno = getFullInternoUtente(internoId);
		Utente utente = userDao.getUser(utenteId);
		if (interno != null && utente != null) {
			if (!interno.getUtenti().contains(utente)) {
				interno.getUtenti().add(utente);
				salvaAggiorna(interno);
			}
		}
	}
	public void rimuoviUtenteDaInterno(Long internoId, Long utenteId) {
		SipBuddies interno = getFullInternoUtente(internoId);
		Utente utente = userDao.getUser(utenteId);
		if (interno != null && utente != null) {
			if (interno.getUtenti().contains(utente)) {
				interno.getUtenti().remove(utente);
				salvaAggiorna(interno);
			}
		}
	}
	public void aggiungiUtenteTuttiAInterno(Long internoId) {
		SipBuddies interno = getFullInternoUtente(internoId);
		List<Utente> utenti = getUtentiNotInInterno();
		if (interno != null && ApplicationUtil.notEmpty(utenti)) {
			interno.getUtenti().addAll(utenti);
			salvaAggiorna(interno);
		}
	}
	public void rimuoviUtenteTuttiDaInterno(Long internoId) {
		SipBuddies interno = getFullInternoUtente(internoId);
		List<Utente> utenti = getUtentiByInternoId(internoId);
		if (interno != null && ApplicationUtil.notEmpty(utenti)) {
			interno.getUtenti().removeAll(utenti);
			salvaAggiorna(interno);
		}
	}
	public SipBuddies getFullInternoUtente(Long internoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.idEq(internoId));
		criteria.setFetchMode("utenti", FetchMode.JOIN);
		return  (SipBuddies) getUniqueResult(criteria);
	}
	public void  eliminaExtensionsTableSelezionati(List<ExtensionsTable> lst) {
		if (ApplicationUtil.notEmpty(lst)) {
			Iterator<ExtensionsTable> itCont = lst.iterator();
			while (itCont.hasNext()) {
				ExtensionsTable ex = (ExtensionsTable) itCont.next();
				if(ex.getSelezionabile()) {
					cancellaAggiorna(ex);
				}
			}
		}
	}
	public void eliminaInternoSelezionati(List<SipBuddies> lst) {
		Iterator<SipBuddies> itCont = lst.iterator();
		while (itCont.hasNext()) {
			SipBuddies sip = (SipBuddies) itCont.next();
			cancellaAggiorna(sip);
			autodialerDao.sendInterno(sip, false);
		}
		UtilChat uc = impostazioneDao.getUtilChat();
		new DeleteBuddyThread(uc, lst, null).start();
	}

	public List<SelectItem> getRadiceSIP() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(1, Message.addCatalogText("gruppo_cdr")));
		sis.add(new SelectItem(2, Message.addCatalogText("tutto")));
		return sis;
	}

	@SuppressWarnings("unchecked")
	public List<Cdr> trova(CdrCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(CdrCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<RiepilogoCdr> trovaSum(CdrCriteria crti) {
		if(ApplicationUtil.gtZero(crti.getRadice())) {
			String type = null; 
			if(ApplicationUtil.eq(crti.getRadice(), 1)) {
				type = "CASE WHEN {alias}.tipo in (7,8) THEN concat('"+Message.addCatalogText("gruppo_cdr")+": ',substring({alias}.callee,1,1)) ELSE {alias}.tipo END";
			}
			if(ApplicationUtil.eq(crti.getRadice(), 2)) {
				type = "{alias}.tipo";
			}
			if(type != null) {
				ExtendedCriteria criteria = trovaCriteria(crti);
				ProjectionList p = Projections.projectionList();
				p.add(Projections.sqlGroupProjection(type +" as radice", "radice order by radice asc" , new String [] {"radice"}, new Type []{StringType.INSTANCE}));
				p.add(Projections.sqlProjection("COUNT({alias}.id) as contatti", new String [] {"contatti"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("SUM({alias}.duration) as chiamata", new String [] {"chiamata"}, new Type []{LongType.INSTANCE}));
				p.add(Projections.sqlProjection("SUM({alias}.billsec) as conversazione", new String [] {"conversazione"}, new Type []{LongType.INSTANCE}));
				criteria.getCriteria().setProjection(p);
				return getList(criteria.getCriteria(), RiepilogoCdr.class);
			}
		} 
		return new ArrayList<RiepilogoCdr>();
	}
	
	@SuppressWarnings("unchecked")
	public RiepilogoCdrInserimento trovaChart(CdrCriteria crti) {
		RiepilogoCdrInserimento rpo = new RiepilogoCdrInserimento();
		ExtendedCriteria criteria = trovaCriteria(crti);
		ProjectionList p = Projections.projectionList();
		p.add(Projections.sqlGroupProjection("DATE_FORMAT({alias}.calldate, '"+ConstantsFormat.orarioQ.stampa()+"') as radice", "radice order by radice asc" , new String [] {"radice"}, new Type []{StringType.INSTANCE}));
		p.add(Projections.sqlProjection("COUNT({alias}.id) as contatti", new String [] {"contatti"}, new Type []{LongType.INSTANCE}));
		p.add(Projections.sqlProjection("SUM(IF({alias}.billsec > 0, 1, 0)) as chiamata", new String [] {"chiamata"}, new Type []{LongType.INSTANCE}));
		criteria.getCriteria().setProjection(p);
		rpo.setCampi(getList(criteria.getCriteria(), RiepilogoCdr.class)); 
		return rpo;
	}

	private ExtendedCriteria trovaCriteria(CdrCriteria ana) {
		if(ana != null)  ana = (CdrCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Cdr.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contatto", ApplicationUtil.leftJoin());
		if (ApplicationUtil.isNotBlank(ana.getCaller())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getCaller(), criteria, "caller");
		}
		if (ApplicationUtil.isNotBlank(ana.getCallee())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true,ana.getCallee(),  criteria, "callee");
		}
		if (ApplicationUtil.isNotBlank(ana.getDisposition())) {
			criteria.add(Restrictions.eq("disposition", ana.getDisposition()));
		}
		ApplicationUtil.lastapp(criteria, ana.getTipo(), ana.getLastapp());
		if (ApplicationUtil.isNotBlank(ana.getRicerca_anagrafica())) {
			DetachedCriteria criteriaA = criteriaCo.createCriteria("anagrafica");
			ApplicationUtil.anagrafica(ana.getRicerca_anagraficaV(), criteriaA, ana.getRicerca_anagrafica());
		}
		if (ApplicationUtil.isNotBlank(ana.getTrunk())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, "/" + ana.getTrunk()+ "-",  criteria, "channel", "dstchannel");
		}
		if (ApplicationUtil.gtZero(ana.getCampagna())) {
			criteriaCo.add(Restrictions.eq("campagna.id", ana.getCampagna()));
		}
		if (ApplicationUtil.isNotBlank(ana.getUtente())) {
			DetachedCriteria c = criteria.createCriteria("utente");
			ApplicationUtil.operatore(c, ana.getUtente());
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<Cdr> getCdrCollegato(Cdr cdr) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Cdr.class);
		criteria.add(Restrictions.eq("linkedid", cdr.getLinkedid()));
		ApplicationUtil.lastapp(criteria);
		criteria.addOrder(Order.asc("calldate"));
		return  getList(criteria);
	}
	public void elimina(Cdr cdr) {
		cancellaAggiorna(cdr);
	}

	@SuppressWarnings("unchecked")
	public List<SelectItem> getDisposition() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		List<String> ls = new ArrayList<String>();
		if(ApplicationUtil.isTrue(impostazioneDao.getStatoPresente())) {
			ls.addAll(ConstantsMap.getDisposition());
		} else {
			DetachedCriteria criteria = DetachedCriteria.forClass(Cdr.class);
			criteria.add(Restrictions.isNotNull(("tipo")));
			criteria.setProjection(Projections.distinct(Projections.property("disposition")));
			criteria.addOrder(Order.asc("disposition"));
			ls.addAll(getList(criteria));
		}
		if(ApplicationUtil.notEmpty(ls)) {
			for(String c : ls) {
				sis.add(new SelectItem(c));
			}
		}
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<SelectItem> getLastapp() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		List<String> ls = new ArrayList<String>();
		if(ApplicationUtil.isTrue(impostazioneDao.getStatoPresente())) {
			ls.addAll(ConstantsMap.getLastapp());
		} else {
			DetachedCriteria criteria = DetachedCriteria.forClass(Cdr.class);
			criteria.add(Restrictions.isNotNull(("tipo")));
			criteria.setProjection(Projections.distinct(Projections.property("lastapp")));
			criteria.addOrder(Order.asc("lastapp"));
			ls.addAll(getList(criteria));
		}
		if(ApplicationUtil.notEmpty(ls)) {
			for(String c : ls) {
				sis.add(new SelectItem(c));
			}
		}
		return sis;
	}
	public List<SelectItem> getTipo() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		for(Integer s : getTipoI()) {
			sis.add(new SelectItem(s , Message.addCatalogText("cdr." + s)));
		}
		return sis;
	}
	public List<Integer> getTipoI() {
		List<Integer> sis = new ArrayList<Integer>();
		sis.add(2);
		sis.add(3);
		sis.add(4);
		sis.add(5);
		sis.add(6);
		sis.add(7);
		sis.add(8);
		sis.add(9);
		sis.add(1);
		sis.add(0);
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<Cdr> trova(CelCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(CelCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteria(CelCriteria ana) {
		if(ana != null)  ana = (CelCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Cel.class);
		if (ApplicationUtil.isNotBlank(ana.getCid_num())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getCid_num(), criteria, "cid_num");
		}
		if (ApplicationUtil.isNotBlank(ana.getExten())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getExten(), criteria, "exten");
		}
		if (ApplicationUtil.isNotBlank(ana.getEventtype())) {
			criteria.add(Restrictions.eq("eventtype", ana.getEventtype()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	public void elimina(Cel cdr) {
		cancellaAggiorna(cdr);
	}
	public List<SelectItem> getEventtype() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Cel.class);
		List<GenericSI> lis = getGenericSI(criteria, "eventtype", "eventtype");
		return impostazioneDao.getGenericNoID(lis, true);
	}
	public List<TrunkSI> getTrunkWebSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(false)));
		criteria.add(Restrictions.eq("attivo", ApplicationUtil.convertBS(true)));
		return getTrunkSI(criteria);
	}
	

	public List<SipBuddies> impostaConteggioTrunk(GenericColumn c, List<SipBuddies> buddies) {
		if(ApplicationUtil.notEmpty(buddies))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdBuddies(buddies);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, SipBuddies.class, new ColonnaQuery(ConstantsColumn.utenti, "users"), new ColonnaQuery(ConstantsColumn.autodialer, "autodialer"), new ColonnaQuery(ConstantsColumn.aiuti, "vocals"), new ColonnaQuery(ConstantsColumn.risposte, "risposte"));
			Iterator<SipBuddies> it = buddies.iterator();
			while (it.hasNext()) {
				SipBuddies o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return buddies;
	}
	public List<SipBuddies> impostaConteggioInterno(GenericColumn c, List<SipBuddies> buddies) {
		if(ApplicationUtil.notEmpty(buddies))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdBuddies(buddies);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, SipBuddies.class, new ColonnaQuery(ConstantsColumn.utenti, "utenti"), new ColonnaQuery(ConstantsColumn.code, "code"));
			Iterator<SipBuddies> it = buddies.iterator();
			while (it.hasNext()) {
				SipBuddies o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return buddies;
	}
	@SuppressWarnings("unchecked")
	public List<VocalOrder> getAiutoNotInTrunk(Long trunk, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("trunk", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(VocalOrder.class);
				criteriaU.createCriteria("trunk").add(Restrictions.idEq(trunk));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("trunk", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(VocalOrder.class);
			criteriaU.createCriteria("trunk").add(Restrictions.idEq(trunk));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	public void aggiungiAiutoATrunk(Long trunkId, Long aiutoId) {
		SipBuddies trunk = getFullTrunkAiuto(trunkId);
		VocalOrder aiuto = getAiuto(aiutoId);
		if (trunk != null && aiuto != null) {
			if (!trunk.getVocals().contains(aiuto)) {
				trunk.getVocals().add(aiuto);
				salvaAggiorna(trunk);
			}
		}
	}
	public void rimuoviAiutoDaTrunk(Long trunkId, Long aiutoId) {
		SipBuddies trunk = getFullTrunkAiuto(trunkId);
		VocalOrder aiuto = getAiuto(aiutoId);
		if (trunk != null && aiuto != null) {
			if (trunk.getVocals().contains(aiuto)) {
				trunk.getVocals().remove(aiuto);
				salvaAggiorna(trunk);
			}
		}
	}
	public void aggiungiAiutoTuttiATrunk(Long trunkId, Integer tutti) {
		SipBuddies trunk = getFullTrunkAiuto(trunkId);
		List<VocalOrder> aiuto = getAiutoNotInTrunk(trunkId, tutti);
		if (trunk != null && ApplicationUtil.notEmpty(aiuto)) {
			trunk.getVocals().addAll(aiuto);
			salvaAggiorna(trunk);
		}
	}
	public void rimuoviAiutoTuttiDaTrunk(Long trunkId) {
		SipBuddies trunk = getFullTrunkAiuto(trunkId);
		List<VocalOrder> aiuto = getAiutoByTrunkId(trunkId);
		if (trunk != null && ApplicationUtil.notEmpty(aiuto)) {
			trunk.getVocals().removeAll(aiuto);
			salvaAggiorna(trunk);
		}
	}
	@SuppressWarnings("unchecked")
	public List<VocalOrder> getAiutoByTrunkId(Long trunkId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
		criteria.createCriteria("trunk").add(Restrictions.idEq(trunkId));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Risposta> getRispostaByTrunkId(Long trunkId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Risposta.class);
		criteria.createCriteria("trunk").add(Restrictions.idEq(trunkId));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	public SipBuddies getFullTrunkAiuto(Long trunkId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.idEq(trunkId));
		criteria.setFetchMode("vocals", FetchMode.JOIN);
		return  (SipBuddies) getUniqueResult(criteria);
	}
	public VocalOrder getAiuto(Long id){
		return (VocalOrder) get(VocalOrder.class, id);
	}

	public List<SelectItem> getTrunkSIP() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(false)));
		List<TrunkSI> lis = getTrunkSI(criteria);
		return impostazioneDao.getTrunk(false, lis, true, false, null);
	}

	@SuppressWarnings("unchecked")
	public List<Inbound> trova(InboundCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("nome")});
		return getList(criteria, start, size);
	}	
	public int trovaSize(InboundCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public void  eliminaInboundSelezionati(List<Inbound> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<Inbound> itR = lst.iterator();
			while (itR.hasNext()) {
				Inbound rules = (Inbound) itR.next();
				elimina(rules);
			}
		}
	}

	public void salvaDocumento(FileUpload fileUpload, Inbound inbound) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					inbound.setPdf(ff.getData());
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	private ExtendedCriteria getConfigByCriteria(InboundCriteria conf) {
		if(conf != null)  conf = (InboundCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Inbound.class);
		if (ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, conf.getNome(), criteria, "nome", "extension");
		}
		conf.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}

	public Inbound nuovoInbound() {
		return new Inbound();
	}
	public void elimina(Inbound config) {
		cancellaAggiorna(config);
		autodialerDao.sendInbound(config, false);
	}

	public void salva(Inbound config) {
		if(config.getId() != null) {
			Inbound vecchio = (Inbound) get(Inbound.class, config.getId());
			autodialerDao.sendInbound(vecchio, false);
		}
		config.save();
		salvaAggiorna(config);
		autodialerDao.sendInbound(config, true);
	}

	@SuppressWarnings("unchecked")
	public List<Inbound> getInboundList() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Inbound.class);
		return getList(criteria);
	}

	public void sincronizzaInbound() {
		List<Inbound> inbs = getInboundList();
		sincronizzaInbound(inbs);
	}
	public void sincronizzaInbound(List<Inbound> valori) {
		autodialerDao.sincronizzaInbound(valori);
	}

	public String root(String interno) {
		return ConstantsGlobal.path_voicemail + "/default/" + interno;
	}

	public void closeRoot(PathUtility p) {
		p.reset();
	}
	public String openRoot(PathUtility p) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			return home + p.getPath();
		} else {
			return p.getPath();
		}
	}
	public String openRoot(PathUtility p, MyFile m) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			return home + p.getPath() + "/" + m.getName();
		} else {
			return p.getPath() + "/" + m.getName();
		}
	}
	public void openFolder(PathUtility p, MyFile m) {
		String n = p.getPath( ) + "/" + m.getName();
		p.setPath(n);
	}
	public void closeFolder(PathUtility p) {
		String n = ApplicationUtil.substringBeforeLast(p.getPath(), "/");
		p.setPath(n);
	}
	public List<MyFile> listaFile(FileCriteria mm, PathUtility p) {
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
				String home = impostazioneDao.getHome();
				File perc = new File(home + p.getPath());
				ApplicationUtil.creaFile(true, perc);
				File [] files = perc.listFiles();
				if(ApplicationUtil.notEmpty(files)) {
					for(File file : files) {
						if(mm.isRicerca(file.getName(), file.length(), file.lastModified()) && (file.isDirectory() || ApplicationUtil.getExtensionN(file.getName()).equals(ConstantsMap.wav))) {
							MyFile f = new MyFile(file);
							f.setParent(ApplicationUtil.removeStart(p.getPath(), p.getRoot()));
							f.setName(file.getName());
							f.setModified(file.lastModified());
							ff.add(f);
						}
					}
				}
			} else {
				UtilFTP ut = impostazioneDao.getUtilFTP();
				FTPClient vmftp = ApplicationUtil.createClient();
				if (ApplicationUtil.connect(vmftp, ut)) {
					try {
						FTPFile[] wavFiles = ApplicationUtil.listFiles(vmftp,  p.getPath() + "/");
						if (ApplicationUtil.notEmpty(wavFiles)) {
							for(FTPFile file : wavFiles) {
								Long time = ApplicationUtil.time(file.getTimestamp());
								if(mm.isRicerca(file.getName(), file.getSize(), time) && (file.isDirectory() || ApplicationUtil.getExtensionN(file.getName()).equals(ConstantsMap.wav))) {
									MyFile f = new MyFile(file);
									f.setParent(ApplicationUtil.removeStart(p.getPath(), p.getRoot()));
									f.setName(file.getName());
									f.setModified(time);
									ff.add(f);
								}
							}
						}
					} catch (Exception e) {
						ErrorLog.log(null, getClass(), e);
					} finally {
						ApplicationUtil.disconnect(vmftp);
					}
				}	
			}
		}
		return ff;
	}
	public void eliminaFile(MyFile att, PathUtility p) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + p.getPath() + "/" + att.getName());
			if(perc.exists()) {
				try {
					if(!perc.isDirectory()) {
						for(String ext : ConstantsMap.a_segreterie) {
							ApplicationUtil.deleteFile(home + p.getPath() + "/" + ApplicationUtil.getBaseNameN(perc.getName()) + "." + ext);
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(!att.isFolder()) {
						for(String ext : ConstantsMap.a_segreterie) {
							ApplicationUtil.deleteFile(vmftp, p.getPath() + "/" + ApplicationUtil.getBaseNameN(att.getName()) + "." + ext);
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
	public void eliminaFile(List<MyFile> atts,  PathUtility p) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if(ApplicationUtil.notEmpty(atts)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> aIt = atts.iterator();
				while (aIt.hasNext()) {
					MyFile att = (MyFile) aIt.next();
					File perc = new File(home + p.getPath() + "/" + att.getName());
					if(perc.exists()) {
						try {
							if(!perc.isDirectory()) {
								for(String ext : ConstantsMap.a_segreterie) {
									ApplicationUtil.deleteFile(home + p.getPath() + "/" + ApplicationUtil.getBaseNameN(perc.getName()) + "." + ext);
								}
							}
						} catch (Exception e) {
							ErrorLog.log(null, getClass(), e);
						}
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(atts)) {
						Iterator<MyFile> aIt = atts.iterator();
						while (aIt.hasNext()) {
							MyFile att = (MyFile) aIt.next();
							if(!att.isFolder()) {
								if(!att.isFolder()) {
									for(String ext : ConstantsMap.a_segreterie) {
										ApplicationUtil.deleteFile(vmftp, p.getPath() + "/" + ApplicationUtil.getBaseNameN(att.getName()) + "." + ext);
									}
								}
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}

	public void scaricaFile(List<MyFile> fils,  PathUtility p, DownloadBean downloadBean) {
		List<Allegato> files = new ArrayList<Allegato>();
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			if (ApplicationUtil.notEmpty(fils)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> itS = fils.iterator();
				while (itS.hasNext()) {
					MyFile file = (MyFile) itS.next();
					String root = home + p.getRoot();
					if(file.isFolder()) {
						String base = home + p.getPath() + "/" + file.getName();
						ApplicationUtil.generateMyFileList(files, new File(base), root);
					} else {
						String base = home + p.getPath();
						ApplicationUtil.generateMyFileList(files, new File(base + "/" + file.getName()), root);
					}
				}
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(ApplicationUtil.notEmpty(fils)) {
						Iterator<MyFile> fIt = fils.iterator();
						while (fIt.hasNext()) {
							MyFile myFile = (MyFile) fIt.next();
							if(myFile.isFolder()) {
								ApplicationUtil.readDirectory(vmftp, files, p.getPath() + "/" + myFile.getName(), p.getRoot() + "/");
							} else {
								ApplicationUtil.addFile(vmftp, files, p.getPath() + "/" + myFile.getName(), myFile.getModified());
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}
		}
		try {
			if (ApplicationUtil.notEmpty(files)) {
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.FILE.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, files);
				String g = ConstantsGlobal.secretZ;
				ApplicationUtil.zip(null,files, out, g);
				downloadBean.close();
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	public void scaricaFile(MyFile att,  PathUtility p, DownloadBean downloadBean) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			try {
				String home = impostazioneDao.getHome();
				if(att.isFolder()) {
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(att.getName() + "." + ConstantsMap.zip);
					String g = ConstantsGlobal.secretZ;
					String ppath = home +  p.getPath() + "/" + att.getName();
					File ff = new File(ppath);
					if(g != null) {
						ZipPassword.zipIt(null, ff.getAbsolutePath(), out, g);
					} else {
						ZipUtils.zipIt(null, ff.getAbsolutePath(), out);
					}
					downloadBean.close();
				} else {
					OutputStream out = downloadBean.outputStream();
					downloadBean.filename(att.getName());
					String nome = home + p.getPath() +"/"+ att.getName();
					InputStream istream = new FileInputStream(nome);
					ApplicationUtil.copy(istream, out);
					istream.close();
					downloadBean.close();
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					if(att.isFolder()) {
						OutputStream out = downloadBean.outputStream();
						downloadBean.filename(att.getName() + "." + ConstantsMap.zip);
						List<Allegato> mFile = new ArrayList<Allegato>();
						ApplicationUtil.readDirectory(vmftp, mFile, p.getPath() + "/" + att.getName(), p.getRoot() + "/");
						String g = ConstantsGlobal.secretZ;
						ApplicationUtil.zip(null,mFile, out, g);
						downloadBean.close();
					} else {
						OutputStream out = downloadBean.outputStream();
						downloadBean.filename(att.getName());
						String path = p.getPath() + "/" + att.getName();
						ApplicationUtil.scaricaFile(vmftp, out, path);
						downloadBean.close();
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}

	public Boolean getEnabledFTP() {
		return ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled());
	}

	public String apriFile(MyFile f, PathUtility p) throws Exception {
		String home = impostazioneDao.getHome();
		File file = new File(home + p.getPath() + "/" + ApplicationUtil.getBaseNameN(f.getName()) + "." + ConstantsMap.txt);
		if (file.exists()) {
			BufferedReader input = new BufferedReader(new FileReader(file));
			StringBuffer buffer = new StringBuffer();
			String text;
			while ((text = input.readLine()) != null)
				buffer.append(text + System.lineSeparator());
			input.close();
			return buffer.toString();
		}
		return null;
	}


	@SuppressWarnings("unchecked")
	public List<Ingresso> trova(IngressoCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("nome")});
		return getList(criteria, start, size);
	}	
	public int trovaSize(IngressoCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public void  eliminaIngressoSelezionati(List<Ingresso> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<Ingresso> itR = lst.iterator();
			while (itR.hasNext()) {
				Ingresso rules = (Ingresso) itR.next();
				elimina(rules);
			}
		}
	}

	public void salvaDocumento(FileUpload fileUpload, Ingresso inbound) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					inbound.setPdf(ff.getData());
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	private ExtendedCriteria getConfigByCriteria(IngressoCriteria conf) {
		if(conf != null)  conf = (IngressoCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Ingresso.class);
		if (ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, conf.getNome(), criteria, "nome", "extension");
		}
		conf.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}

	public Ingresso nuovoIngresso() {
		return new Ingresso();
	}
	public void elimina(Ingresso config) {
		cancellaAggiorna(config);
	}

	public void salva(Ingresso config) {
		salvaAggiorna(config);
	}

	@SuppressWarnings("unchecked")
	public List<Ingresso> getIngressoList() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ingresso.class);
		return getList(criteria);
	}

	public void impostaTrunks(SipBuddies trunk) {
		if(trunk != null) {
			if(ApplicationUtil.isTrue(trunk.getFirewall())) {
				trunk.setPort(impostazioneDao.porta());
				trunk.setRtp1(impostazioneDao.portaRTP1());
				trunk.setRtp2(impostazioneDao.portaRTP2());
			} else {
				trunk.setPort(ConstantsGlobal.default_sip);
				trunk.setRtp1(ConstantsGlobal.default_port);
				trunk.setRtp2(ConstantsGlobal.default_port);
			}
		}
	}

	private ExtendedCriteria trovaCriteria(CodaCriteria ana) {
		if(ana != null)  ana = (CodaCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Coda.class);
		if (ApplicationUtil.isNotBlank(ana.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getNome(), criteria, "name");
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	
	@SuppressWarnings("unchecked")
	public List<Coda> trova(CodaCriteria ana,int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), ana.getO_tipo(), ana.getO_colonna(), new Order [] {Order.asc("name")});
		return getList(criteria, start, size);
	}
	
	public int trovaSize(CodaCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	
	public void elimina(Coda config) {
		cancellaAggiorna(config);
	}

	public Coda salva(Coda config) {
		config = (Coda) salvaAggiorna(config);
		return config;
	}
	
	public List<Coda> impostaConteggioCoda(GenericColumn c, List<Coda> buddies) {
		if(ApplicationUtil.notEmpty(buddies))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdCoda(buddies);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Coda.class, new ColonnaQuery(ConstantsColumn.interni, "interni"));
			Iterator<Coda> it = buddies.iterator();
			while (it.hasNext()) {
				Coda o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return buddies;
	}
	
	public List<Risposta> impostaConteggioRisposta(GenericColumn c, List<Risposta> buddies) {
		if(ApplicationUtil.notEmpty(buddies))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdRisposta(buddies);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Risposta.class, new ColonnaQuery(ConstantsColumn.campagne, "campagne"));
			Iterator<Risposta> it = buddies.iterator();
			while (it.hasNext()) {
				Risposta o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return buddies;
	}
	
	public Coda getByCoda(String name){
		DetachedCriteria criteria = DetachedCriteria.forClass(Coda.class);
		criteria.add(Restrictions.eq("name", name));
		return  (Coda) getUniqueResult(criteria);
	}
	
	public Coda getCoda(Long id){
		return (Coda) get(Coda.class, id);
	}
	
	@SuppressWarnings("unchecked")
	public List<SipBuddies> getInterniByCodaId(Long codaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.createCriteria("code").add(Restrictions.idEq(codaId));
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		criteria.addOrder(Order.asc("name"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<SipBuddies> getInterniNotInCoda(Long ut) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Coda.class);
		criteriaU.add(Restrictions.idEq(ut));
		DetachedCriteria criteriaA = criteriaU.createCriteria("interni");
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(SipBuddies.class);
		criteria.add(Restrictions.eq("tipologia", ApplicationUtil.convertBS(true)));
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		criteria.addOrder(Order.asc("name"));
		return getList(criteria);
	}
	public void aggiungiInternoACoda(Long codaId, Long internoId) {
		Coda coda = getFullCodaInterno(codaId);
		SipBuddies interno = get(internoId);
		if (coda != null && interno != null) {
			if (!coda.getInterni().contains(interno)) {
				coda.getInterni().add(interno);
				salvaAggiorna(coda);
			}
		}
	}
	public void rimuoviInternoDaCoda(Long codaId, Long internoId) {
		Coda coda = getFullCodaInterno(codaId);
		SipBuddies interno = get(internoId);
		if (coda != null && interno != null) {
			if (coda.getInterni().contains(interno)) {
				coda.getInterni().remove(interno);
				salvaAggiorna(coda);
			}
		}
	}
	public void aggiungiInternoTuttiACoda(Long codaId) {
		Coda coda = getFullCodaInterno(codaId);
		List<SipBuddies> interni = getInterniNotInCoda(codaId);
		if (coda != null && ApplicationUtil.notEmpty(interni)) {
			coda.getInterni().addAll(interni);
			salvaAggiorna(coda);
		}
	}
	public void rimuoviInternoTuttiDaCoda(Long codaId) {
		Coda coda = getFullCodaInterno(codaId);
		List<SipBuddies> interni = getInterniByCodaId(codaId);
		if (coda != null && ApplicationUtil.notEmpty(interni)) {
			coda.getInterni().removeAll(interni);
			salvaAggiorna(coda);
		}
	}
	public Coda getFullCodaInterno(Long codaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Coda.class);
		criteria.add(Restrictions.idEq(codaId));
		criteria.setFetchMode("interni", FetchMode.JOIN);
		return  (Coda) getUniqueResult(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Coda> getCode() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Coda.class);
		criteria.addOrder(Order.asc("name"));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Ruolo> trovaCodaPerInterno(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Coda.class);
		criteria.createCriteria("interni").add(Restrictions.idEq(id));
		criteria.addOrder(Order.asc("name"));
		return getList(criteria);
	}
	
	

	@SuppressWarnings("unchecked")
	public List<Risposta> trova(RispostaCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("nome")});
		return getList(criteria, start, size);
	}	
	public int trovaSize(RispostaCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public void  eliminaRispostaSelezionati(List<Risposta> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<Risposta> itR = lst.iterator();
			while (itR.hasNext()) {
				Risposta rules = (Risposta) itR.next();
				elimina(rules);
			}
		}
	}

	private ExtendedCriteria getConfigByCriteria(RispostaCriteria conf) {
		if(conf != null)  conf = (RispostaCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Risposta.class);
		if (ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, conf.getNome(), criteria, "nome", "extension");
		}
		conf.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}

	public Risposta nuovoRisposta() {
		return new Risposta();
	}
	public void elimina(Risposta config) {
		cancellaAggiorna(config);
		autodialerDao.sendRisposta(config, false);
	}

	public void salva(Risposta config) {
		if(config.getId() != null) {
			Risposta vecchio = (Risposta) get(Risposta.class, config.getId());
			autodialerDao.sendRisposta(vecchio, false);
		}
		config.save();
		salvaAggiorna(config);
		autodialerDao.sendRisposta(config, true);
	}

	@SuppressWarnings("unchecked")
	public List<Risposta> getRispostaList() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Risposta.class);
		return getList(criteria);
	}

	public void sincronizzaRisposta() {
		List<Risposta> inbs = getRispostaList();
		sincronizzaRisposta(inbs);
	}
	public void sincronizzaRisposta(List<Risposta> valori) {
		autodialerDao.sincronizzaRisposta(valori);
	}
	
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagnaByRispostaId(Long rispostaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("risposte").add(Restrictions.idEq(rispostaId));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagnaNotInRisposta(Long ut) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Risposta.class);
		criteriaU.add(Restrictions.idEq(ut));
		DetachedCriteria criteriaA = criteriaU.createCriteria("campagne");
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	public void aggiungiCampagnaARisposta(Long rispostaId, Long campagnaId) {
		Risposta risposta = getFullRispostaCampagna(rispostaId);
		Campagna campagna = getCampagna(campagnaId);
		if (risposta != null && campagna != null) {
			if (!risposta.getCampagne().contains(campagna)) {
				risposta.getCampagne().add(campagna);
				salvaAggiorna(risposta);
			}
		}
	}
	public void rimuoviCampagnaDaRisposta(Long rispostaId, Long campagnaId) {
		Risposta risposta = getFullRispostaCampagna(rispostaId);
		Campagna campagna = getCampagna(campagnaId);
		if (risposta != null && campagna != null) {
			if (risposta.getCampagne().contains(campagna)) {
				risposta.getCampagne().remove(campagna);
				salvaAggiorna(risposta);
			}
		}
	}
	public void aggiungiCampagnaTuttiARisposta(Long rispostaId) {
		Risposta risposta = getFullRispostaCampagna(rispostaId);
		List<Campagna> campagne = getCampagnaNotInRisposta(rispostaId);
		if (risposta != null && ApplicationUtil.notEmpty(campagne)) {
			risposta.getCampagne().addAll(campagne);
			salvaAggiorna(risposta);
		}
	}
	public void rimuoviCampagnaTuttiDaRisposta(Long rispostaId) {
		Risposta risposta = getFullRispostaCampagna(rispostaId);
		List<Campagna> campagne = getCampagnaByRispostaId(rispostaId);
		if (risposta != null && ApplicationUtil.notEmpty(campagne)) {
			risposta.getCampagne().removeAll(campagne);
			salvaAggiorna(risposta);
		}
	}
	public Risposta getFullRispostaCampagna(Long rispostaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Risposta.class);
		criteria.add(Restrictions.idEq(rispostaId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return  (Risposta) getUniqueResult(criteria);
	}
	
	public Campagna getCampagna(Long id){
		return (Campagna) get(Campagna.class, id);
	}
	
	@SuppressWarnings("unchecked")
	public List<Risposta> getRispostaByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Risposta.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Risposta> getRispostaNotInCampagna(Long ut) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.add(Restrictions.idEq(ut));
		DetachedCriteria criteriaA = criteriaU.createCriteria("risposte");
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Risposta.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	public void aggiungiRispostaACampagna(Long campagnaId, Long rispostaId) {
		Campagna campagna = getFullCampagnaRisposta(campagnaId);
		Risposta risposta = getRisposta(rispostaId);
		if (campagna != null && risposta != null) {
			if (!campagna.getRisposte().contains(risposta)) {
				campagna.getRisposte().add(risposta);
				salvaAggiorna(campagna);
			}
		}
	}
	public void rimuoviRispostaDaCampagna(Long campagnaId, Long rispostaId) {
		Campagna campagna = getFullCampagnaRisposta(campagnaId);
		Risposta risposta = getRisposta(rispostaId);
		if (campagna != null && risposta != null) {
			if (campagna.getRisposte().contains(risposta)) {
				campagna.getRisposte().remove(risposta);
				salvaAggiorna(campagna);
			}
		}
	}
	public void aggiungiRispostaTuttiACampagna(Long campagnaId) {
		Campagna campagna = getFullCampagnaRisposta(campagnaId);
		List<Risposta> campagne = getRispostaNotInCampagna(campagnaId);
		if (campagna != null && ApplicationUtil.notEmpty(campagne)) {
			campagna.getRisposte().addAll(campagne);
			salvaAggiorna(campagna);
		}
	}
	public void rimuoviRispostaTuttiDaCampagna(Long campagnaId) {
		Campagna campagna = getFullCampagnaRisposta(campagnaId);
		List<Risposta> campagne = getRispostaByCampagnaId(campagnaId);
		if (campagna != null && ApplicationUtil.notEmpty(campagne)) {
			campagna.getRisposte().removeAll(campagne);
			salvaAggiorna(campagna);
		}
	}
	public Campagna getFullCampagnaRisposta(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(campagnaId));
		criteria.setFetchMode("risposte", FetchMode.JOIN);
		return  (Campagna) getUniqueResult(criteria);
	}
		
	public Risposta getRisposta(Long id){
		return (Risposta) get(Risposta.class, id);
	}

	public void copiaRisposta(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Long> rispostas = getRispostaByCampagna(campagnaId_src);
		Iterator<Long> objIter = rispostas.iterator();
		while(objIter.hasNext()) {
			Long rispostaId = objIter.next();
			Risposta risposta = getFullRispostaCampagna(rispostaId);
			if(risposta.getCampagne() == null)
				risposta.setCampagne(new ArrayList<Campagna>());
			if(!risposta.getCampagne().contains(campagna)) {
				risposta.getCampagne().add(campagna);
				salvaAggiorna(risposta);
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<Long> getRispostaByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Risposta.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}

}
