package it.callite.dao;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.DomandaAnagraficaCriteria;
import it.callite.criteria.SondaggioAnagraficaCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.Anagrafica;
import it.callite.model.CampoAggiuntivoAnagrafica;
import it.callite.model.CampoAggiuntivoContratto;
import it.callite.model.Contatto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DomandaAnagrafica;
import it.callite.model.DomandaContratto;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.SondaggioAnagrafica;
import it.callite.model.SondaggioContratto;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.CampoSpeciale;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.DatoSondaggio;
import it.callite.utility.DatoSondaggioAnagrafica;
import it.callite.utility.ValoreMappa;
import it.callite.utility.item.GenericSI;
@SuppressWarnings("serial")
@Repository("sondaggioDao")
public class SondaggioDao extends GenericDao {
	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private CampagnaDao campagnaDao;
	@Autowired
	private DataDao dataDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	public DatoSondaggio nuovoDatoSondaggio() {
		return new DatoSondaggio();
	}
	private ExtendedCriteria trovaDomandeCriteria(Boolean modulo, DomandaAnagraficaCriteria dom) {
		if(dom != null)  dom = (DomandaAnagraficaCriteria) dom.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoAnagrafica.class);
		if(ApplicationUtil.isNotBlank(dom.getCampo())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, dom.getCampo(), criteria, "descrizione", "etichetta");
		}
		if(ApplicationUtil.gtZero(dom.getTipo())) {
			criteria.add(Restrictions.eq("tipo",dom.getTipo()));
		}
		if(ApplicationUtil.gtZero(dom.getSondaggio())) {
			criteria.createCriteria("domande").createCriteria("sondaggio").add(Restrictions.eq("id",dom.getSondaggio()));
		}
		if(ApplicationUtil.isNotBlank(dom.getGruppo())) {
			criteria.add(Restrictions.eq("gruppo",dom.getGruppo()));
		}
		if(!modulo) {
			criteria.add(Restrictions.ne("tipo",ConstantsMap.TIPO_FORM));
		}
		dom.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, dom);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoAnagrafica> trovaDomande(List<DatoAggiuntivoAnagrafica> campiDaRimuovere, Boolean modulo, DomandaAnagraficaCriteria dom, int start, int size) {
		ExtendedCriteria criteria = trovaDomandeCriteria(modulo, dom);
		List<Long> ids = ApplicationUtil.prendiIdDatoAggiuntivoAnagrafica(campiDaRimuovere);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}

	public List<SelectItem> getSondaggiAnagraficaSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioAnagrafica.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiContrattoSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiAnagraficaSI(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioAnagrafica.class);
		if(ApplicationUtil.gtZero(userId)) {
			DetachedCriteria c = criteria.createCriteria("esiti").createCriteria("campagne");
			impostazioneDao.subCriteriaCampagnaSupervisore(userId, c, "id");
		}
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiContrattoSI(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		if(ApplicationUtil.gtZero(userId)) {
			DetachedCriteria c = criteria.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
			impostazioneDao.subCriteriaCampagnaSupervisore(userId, c , "id.campagna.id");
		}
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiAnagraficaSIPartner(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioAnagrafica.class);
		DetachedCriteria c = criteria.createCriteria("esiti").createCriteria("campagne");
		impostazioneDao.subCriteriaCampagnaPartner(userId, c , "id");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiContrattoSIPartner(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		DetachedCriteria c = criteria.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
		impostazioneDao.subCriteriaCampagnaPartner(userId, c, "id.campagna.id");
		List<GenericSI> lis = getGenericSI(criteria, "id", "nome");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiDomandeAnagraficaSI(Long sondaggioId, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoAnagrafica.class);
		DetachedCriteria criteriaD = criteria.createCriteria("domande");
		criteria.add(Restrictions.ne("tipo", ConstantsMap.TIPO_FORM));
		if(ApplicationUtil.gtZero(sondaggioId)) {
			criteriaD .add(Restrictions.eq("sondaggio.id", sondaggioId));
		} else {
			DetachedCriteria criteriaS = criteriaD.createCriteria("sondaggio");
			criteriaS.add(Restrictions.isNotNull("id"));
			if(ApplicationUtil.gtZero(userId)) {
				DetachedCriteria c = criteriaS.createCriteria("esiti").createCriteria("campagne");
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, c, "id");
			} 
		}
		List<GenericSI> lis = getGenericSI(criteria, "id", "etichetta");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiDomandeContrattoSI(Long sondaggioId, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		DetachedCriteria criteriaD = criteria.createCriteria("domande");
		criteria.add(Restrictions.ne("tipo", ConstantsMap.TIPO_FORM));
		if(ApplicationUtil.gtZero(sondaggioId)) {
			criteriaD .add(Restrictions.eq("sondaggio.id", sondaggioId));
		} else {
			DetachedCriteria criteriaS = criteriaD.createCriteria("sondaggio");
			criteriaS.add(Restrictions.isNotNull("id"));
			if(ApplicationUtil.gtZero(userId)) {
				DetachedCriteria c = criteriaS.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
				impostazioneDao.subCriteriaCampagnaSupervisore(userId, c, "id.campagna.id");
			}
		}
		List<GenericSI> lis = getGenericSI(criteria, "id", "etichetta");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiDomandeContrattoSIPartner(Long sondaggioId, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		DetachedCriteria criteriaD = criteria.createCriteria("domande");
		criteria.add(Restrictions.ne("tipo", ConstantsMap.TIPO_FORM));
		if(ApplicationUtil.gtZero(sondaggioId)) {
			criteriaD.add(Restrictions.eq("sondaggio.id", sondaggioId));
		} else {
			DetachedCriteria criteriaS = criteriaD.createCriteria("sondaggio");
			criteriaS.add(Restrictions.isNotNull("id"));
			DetachedCriteria c = criteriaS.createCriteria("prodotti").createCriteria("prodotti_in_vendita");
			impostazioneDao.subCriteriaCampagnaPartner(userId, c, "id.campagna.id");
		}
		List<GenericSI> lis = getGenericSI(criteria, "id", "etichetta");
		return impostazioneDao.getGeneric(lis, true);
	}
	public List<SelectItem> getSondaggiDomandeAnagraficaSIPartner(Long sondaggioId, Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoAnagrafica.class);
		DetachedCriteria criteriaD = criteria.createCriteria("domande");
		criteria.add(Restrictions.ne("tipo", ConstantsMap.TIPO_FORM));
		if(ApplicationUtil.gtZero(sondaggioId)) {
			criteriaD .add(Restrictions.eq("sondaggio.id", sondaggioId));
		} else {
			DetachedCriteria criteriaS = criteriaD.createCriteria("sondaggio");
			criteriaS.add(Restrictions.isNotNull("id"));
			DetachedCriteria c = criteriaS.createCriteria("esiti").createCriteria("campagne");
			impostazioneDao.subCriteriaCampagnaPartner(userId, c, "id");
		}
		List<GenericSI> lis = getGenericSI(criteria, "id", "etichetta");
		return impostazioneDao.getGeneric(lis, true);
	}
	public void aggiornaCampi(List<CampoAggiuntivoAnagrafica> cas) {
		if(ApplicationUtil.notEmpty(cas)) {
			Iterator<CampoAggiuntivoAnagrafica> caIt = cas.iterator();
			while (caIt.hasNext()) {
				CampoAggiuntivoAnagrafica ca = (CampoAggiuntivoAnagrafica) caIt.next();
				aggiornaCampo(ca);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public void aggiornaCampo(CampoAggiuntivoAnagrafica ca) {
		if(ca != null && !ca.getForm()) {
			DetachedCriteria criteria1 = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
			criteria1.add(Restrictions.eq("campo_aggiuntivo.id", ca.getId()));
			criteria1.setProjection(Projections.distinct(Projections.groupProperty("anagrafica.id")));
			DetachedCriteria criteria2 = DetachedCriteria.forClass(Anagrafica.class);
			impostazioneDao.subPropertyNotInS(criteria2, criteria1, "id");
			List<Anagrafica> a = getList(criteria2);
			if(ApplicationUtil.notEmpty(a)) {
				Session session = openSession();
				Transaction tx = session.beginTransaction();
				int i = 0;
				Iterator<Anagrafica> it = a.iterator();
				while(it.hasNext()) {
					Anagrafica c = (Anagrafica) it.next();
					DatoAggiuntivoAnagrafica da = new DatoAggiuntivoAnagrafica();
					da.setAnagrafica(c);
					da.setCampo_aggiuntivo(ca);
					session.saveOrUpdate(da);
					if (i++ % ConstantsGlobal.flushSize == 0) {
						session.flush();
						session.clear();
					}
				}
				tx.commit();
				closeSession(session);
			}
		}
	}
	public int trovaDomandeSize(List<DatoAggiuntivoAnagrafica> campiDaRimuovere, Boolean modulo, DomandaAnagraficaCriteria dom) {
		ExtendedCriteria criteria = trovaDomandeCriteria(modulo, dom);
		List<Long> ids = ApplicationUtil.prendiIdDatoAggiuntivoAnagrafica(campiDaRimuovere);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<CampoAggiuntivoAnagrafica> trovaDomande(Boolean modulo, DomandaAnagraficaCriteria dom,List<DomandaAnagrafica> campiDaRimuovere, int start, int size) {
		ExtendedCriteria criteria = trovaDomandeCriteria(modulo, dom);
		List<Long> ids = ApplicationUtil.prendiIdDomandaAnagrafica(campiDaRimuovere);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaDomandeSize(Boolean modulo, DomandaAnagraficaCriteria dom, List<DomandaAnagrafica> campiDaRimuovere) {
		ExtendedCriteria criteria = trovaDomandeCriteria(modulo, dom);
		List<Long> ids = ApplicationUtil.prendiIdDomandaAnagrafica(campiDaRimuovere);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaCriteria(SondaggioAnagraficaCriteria snd) {
		if(snd != null)  snd = (SondaggioAnagraficaCriteria) snd.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioAnagrafica.class);
		if (ApplicationUtil.isNotBlank(snd.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, snd.getNome(), criteria, "nome", "descrizione");
		}
		if (ApplicationUtil.gtZero(snd.getEsitoId())) {
			criteria.createCriteria("esiti").add(Restrictions.eq("id",snd.getEsitoId()));
		}
		snd.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, snd);
	}
	@SuppressWarnings("unchecked")
	public List<SondaggioAnagrafica> trova(SondaggioAnagraficaCriteria snd, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(snd);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(SondaggioAnagraficaCriteria snd) {
		ExtendedCriteria criteria = trovaCriteria(snd);
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<DomandaAnagrafica> getDomandeSondaggio(Long sondaggioId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaAnagrafica.class);
		criteria.add(Restrictions.eq("sondaggio.id",sondaggioId));
		criteria.addOrder(Order.asc("posizione"));
		return getList(criteria);
	}
	public DomandaAnagrafica salva(DomandaAnagrafica sondaggio) {
		return (DomandaAnagrafica) salvaAggiorna(sondaggio);
	}
	public SondaggioAnagrafica salva(SondaggioAnagrafica sondaggio) {
		return (SondaggioAnagrafica) salvaAggiorna(sondaggio);
	}
	public void elimina(SondaggioAnagrafica sondaggio) {
		cancellaAggiorna(sondaggio);
	}
	public SondaggioAnagrafica get(Long id) {
		return (SondaggioAnagrafica) get(SondaggioAnagrafica.class, id);
	}
	public void addDomanda(List<DatoSondaggioAnagrafica> dati, DatoSondaggioAnagrafica domanda) {
		if(dati!=null && !dati.contains(domanda)) {
			dati.add(domanda);
		}
	}
	public List<DatoSondaggioAnagrafica> getSondaggioForm(Integer tipo, EsitoArchiviazione esito, Contatto contatto) {
		List<DatoSondaggioAnagrafica> dati = new ArrayList<DatoSondaggioAnagrafica>();
		if(esito != null) {
			Map<String, DatoAggiuntivo> valori = new HashMap<String, DatoAggiuntivo>();
			List<DatoAggiuntivoAnagrafica> dA = new ArrayList<DatoAggiuntivoAnagrafica>();
			if(contatto != null) {
				List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
				valori.putAll(ApplicationUtil.convertiAnagrafica(contatto.getAnagrafica(), valoriValidi));
				if(ApplicationUtil.notEmpty(contatto.getDettagli())) {
					dA.addAll(contatto.getDettagli());
				}
			}
			if(ApplicationUtil.notEmpty(dA)) {
				valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
			}
			List<DomandaAnagrafica> domandeR = getRisposteDomandeSondaggio(esito, contatto.getAnagrafica());
			if(ApplicationUtil.eq(esito.getAnagrafica_sondaggio(), 1) || ApplicationUtil.eq(esito.getAnagrafica_sondaggio(), 2)) {
				Map<String,Object> valoriS = new HashMap<String,Object>();
				Map<String,List<String>> valoriM = new HashMap<String,List<String>>();
				List<DomandaAnagrafica> domandeT = getTutteDomandeSondaggio(tipo, contatto, esito);
				if(ApplicationUtil.notEmpty(domandeR)) {
					Iterator<DomandaAnagrafica> objDocIter = domandeR.iterator();
					while(objDocIter.hasNext()) {
						DomandaAnagrafica domanda = objDocIter.next();
						if(ApplicationUtil.isNotBlank(domanda.getCampo_aggiuntivo().getGruppo())) {
							String risposta = domanda.getRisposta();
							if(domanda.getCampo_aggiuntivo().getManyCheckbox() || domanda.getCampo_aggiuntivo().getManyMenu() || domanda.getCampo_aggiuntivo().getPickList()) {
								valoriM.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.token(risposta, ConstantsGlobal.colonSeparator));
							} else if(domanda.getCampo_aggiuntivo().getData()) {
								valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(domanda.getCampo_aggiuntivo().getValoriT(), risposta));
							} else if(domanda.getCampo_aggiuntivo().getDataora()) {
								valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(domanda.getCampo_aggiuntivo().getValoriT(), risposta));
							} else if(domanda.getCampo_aggiuntivo().getOra()) {
								valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), ApplicationUtil.parseData(ConstantsFormat.time.stampa(), risposta));
							} else {
								valoriS.put(domanda.getCampo_aggiuntivo().getChiave(), risposta);
							}
						}
					}
				}
				if(ApplicationUtil.notEmpty(domandeT)) {
					Iterator<DomandaAnagrafica> objDocIter = domandeT.iterator();
					while(objDocIter.hasNext()) {
						DomandaAnagrafica domanda = objDocIter.next();
						DatoSondaggioAnagrafica datoS = null;
						if(domandeR.contains(domanda)){
							int index = domandeR.indexOf(domanda);
							if(index > -1) {
								DomandaAnagrafica d = domandeR.get(index);
								String risposta = d.getRisposta();
								datoS = ApplicationUtil.creaDomanda(domanda, risposta);
									
							}
						} else {
							String risposta = ApplicationUtil.rispondi(domanda,valori);
							datoS = ApplicationUtil.creaDomanda(domanda, risposta);
							
						}
						if(datoS != null) {
							addDomanda(dati, datoS);
							if(domanda.getCampo_aggiuntivo().getForm()) {
								datoS.setValoriM(valoriM);
								datoS.setValoriS(valoriS);
							}
						}
					}
				}
			}
			if(ApplicationUtil.eq(esito.getAnagrafica_sondaggio(), 2) || ApplicationUtil.eq(esito.getAnagrafica_sondaggio(), 3)) {
				if(ApplicationUtil.notEmpty(domandeR)) {
					Iterator<DomandaAnagrafica> objDocIter = domandeR.iterator();
					while(objDocIter.hasNext()) {
						DomandaAnagrafica domanda = objDocIter.next();
						if((ApplicationUtil.eq(esito.getAnagrafica_sondaggio(), 2) && ApplicationUtil.isBlank(domanda.getCampo_aggiuntivo().getGruppo())) || ApplicationUtil.eq(esito.getAnagrafica_sondaggio(), 3)) {
							String risposta = domanda.getRisposta();
							addDomanda(dati, ApplicationUtil.creaDomanda(domanda, risposta));
						}
					}
				}
			}
		}
		return dati;
	}
	@SuppressWarnings("unchecked")
	public List<DomandaAnagrafica> getTutteDomandeSondaggio(Integer tipo, Contatto contatto, EsitoArchiviazione esito) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(DomandaAnagrafica.class);
		DetachedCriteria criteriaS = criteriaC.createCriteria("sondaggio");
		if(!contatto.isOk()) {
			criteriaS.add(Restrictions.eq("solo_archiviati", Boolean.FALSE));
		}
		if(ApplicationUtil.eq(tipo, 1)) {
			criteriaS.add(Restrictions.eq("solo_operatore", Boolean.TRUE));
		}
		if(ApplicationUtil.eq(tipo, 2)) {
			criteriaS.add(Restrictions.eq("solo_supervisore", Boolean.TRUE));
		}
		if(ApplicationUtil.eq(tipo, 3)) {
			criteriaS.add(Restrictions.eq("solo_backoffice", Boolean.TRUE));
		}
		criteriaS.createCriteria("esiti").add(Restrictions.idEq(esito.getId()));
		criteriaS.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteriaC.addOrder(Order.asc("posizione"));
		return  getList(criteriaC);
	}
	@SuppressWarnings("unchecked")
	public List<DomandaAnagrafica> getRisposteDomandeSondaggio(EsitoArchiviazione esito, Anagrafica anagrafica) {
		List<DomandaAnagrafica> dms = new ArrayList<DomandaAnagrafica>();
		DetachedCriteria criteriaC = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
		criteriaC.add(Restrictions.eq("anagrafica.id", anagrafica.getId()));
		List<DatoAggiuntivoAnagrafica> das = getList(criteriaC);
		if(ApplicationUtil.notEmpty(das)) {
			Iterator<DatoAggiuntivoAnagrafica> objIter = das.iterator();
			while(objIter.hasNext()) {
				DatoAggiuntivoAnagrafica da = objIter.next();
				DomandaAnagrafica dm = new DomandaAnagrafica();
				dm.setCampo_aggiuntivo(da.getCampo_aggiuntivo());
				dm.setRisposta(da.getValore());
				dm.setObbligatoria(da.getCampo_aggiuntivo().getObbligatorio());
				dm.setVisibile(true);
				dms.add(dm);
			}
		}
		return dms;
	}
	@SuppressWarnings("unchecked")
	public List<DatoAggiuntivoAnagrafica> getDomandeRisposte(EsitoArchiviazione esito, Anagrafica anagrafica) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(DatoAggiuntivoAnagrafica.class);
		criteriaC.add(Restrictions.eq("anagrafica.id", anagrafica.getId()));
		criteriaC.createCriteria("campo_aggiuntivo").add(Restrictions.ne("tipo", ConstantsMap.TIPO_FORM));
		return getList(criteriaC);
	}
	public boolean addDomandaToSondaggio(SondaggioAnagrafica sondaggio, DomandaAnagrafica domanda) {
		boolean risposta = false;
		if (sondaggio.getDomande() == null)
			sondaggio.setDomande(new ArrayList<DomandaAnagrafica>());
		if (!sondaggio.getDomande().contains(domanda))
			risposta = sondaggio.getDomande().add(domanda);
		return risposta;
	}
	public void saveSondaggioToAnagrafica(EsitoArchiviazione esito, List<DatoSondaggioAnagrafica> dati, Anagrafica anagrafica) {
		if(anagrafica != null && esito != null && esito.has_sceltaSondaggio() && ApplicationUtil.notEmpty(dati)) {
			List<DatoAggiuntivoAnagrafica> dr = new ArrayList<DatoAggiuntivoAnagrafica>();
			dr.addAll(getDomandeRisposte(esito, anagrafica));
			Iterator<DatoSondaggioAnagrafica> objIter = dati.iterator();
			while(objIter.hasNext()) {
				DatoSondaggioAnagrafica dato = objIter.next();
				if (dato.getCampo() != null) {
					if (dato.getCampo().getForm()) {
						Map<CampoSpeciale,ValoreMappa> mappa = dato.getMappa();
						if (ApplicationUtil.notEmpty(mappa)) {
							Iterator<CampoSpeciale> ssIt =  mappa.keySet().iterator();
							while(ssIt.hasNext()) {
								CampoSpeciale cc = ssIt.next();
								ValoreMappa val = mappa.get(cc);
								String chiave = ApplicationUtil.trimToEmpty(cc.getChiave()).replaceAll(ConstantsGlobal.colonSeparator, " ");
								if(val!=null) {
									if(ApplicationUtil.isBlank(cc.getEtichetta())) {
										cc.setEtichetta(chiave);
									}
									CampoAggiuntivoAnagrafica campo = dato.getCampo();
									String nuovaChiave = ApplicationUtil.trimToEmpty(ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getChiave(), chiave, ConstantsGlobal.ereditaChiave));
									if(!ConstantsMap.anagraficaMap.contains(nuovaChiave)){
										CampoAggiuntivoAnagrafica caa = anagraficaDao.getCampoAggiuntivoByChiave(nuovaChiave);
										if (caa == null) {
											caa = new CampoAggiuntivoAnagrafica();
											caa.setChiave(nuovaChiave);
											String gruppo = ApplicationUtil.impostaGruppo(campo.getEtichetta(), campo.getColore());
											caa.setGruppo(gruppo);
											caa.setEtichetta(ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getEtichetta(), cc.getEtichetta(),  ConstantsGlobal.ereditaEtichetta));
											caa.setDescrizione(ApplicationUtil.impostaCampoC(cc.getEredita(),campo.getDescrizione(), cc.getDescrizione()));
											caa.setModificabile_operatore(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_operatore(), cc.getModificabile_operatore()));
											caa.setVisibile_operatore(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_operatore(), cc.getVisibile_operatore()));
											caa.setModificabile_agente(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_agente(), cc.getModificabile_agente()));
											caa.setVisibile_agente(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_agente(), cc.getVisibile_agente()));
											caa.setObbligatorio(ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getObbligatorio(), cc.getObbligatorio()));
											caa.setColore(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getColore(), cc.getColore()));
											caa.setStile(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getStile(), cc.getStile()));
											caa.setTipo(ApplicationUtil.gtZero(caa.getTipo()) ? cc.getTipo() : ConstantsMap.TIPO_APERTO);
											caa.setDefaults(ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDefaults(), cc.getDefaults()));
											if(caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento() || caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu() || caa.getCombo()) {
												Set<String> valori = new HashSet<String>();
												valori.addAll(cc.getValuesList());
												if(val.getMultiplo() != null && (caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu())) {
													valori.addAll(val.getMultiplo());
												} 
												if(val.getSingolo() != null && (caa.getCombo() || caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento())) {
													valori.add(ApplicationUtil.trimToEmpty((String) val.getSingolo()));
												} 
												valori.removeAll(ApplicationUtil.asList("", null));
												caa.setValori(ApplicationUtil.detoken(new ArrayList<String>(valori), ConstantsGlobal.colonSeparator));
											}
											if(caa.getData() ||caa.getDataora() || caa.getRegulare() || caa.getUnivoco()) {
												caa.setValori(cc.getValues());
											} 
											caa = (CampoAggiuntivoAnagrafica) anagraficaDao.salvaCampoAggiuntivo(caa);
											dataDao.addCampo(caa);
										} else {
											boolean save = false;
											if(caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento() || caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu() || caa.getCombo()) {
												Set<String> vn = new HashSet<String>();
												vn.addAll(caa.getValoriList());
												vn.addAll(cc.getValuesList());
												if(val.getMultiplo() != null && (caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu())) {
													vn.addAll(val.getMultiplo());
												}
												if(val.getSingolo() != null && (caa.getCombo() || caa.getOneMenu() || caa.getInplaceSelect() || caa.getOneRadio() || caa.getSuggerimento())) {
													vn.add(ApplicationUtil.trimToEmpty((String) val.getSingolo()));
												}
												vn.removeAll(ApplicationUtil.asList("", null));
												Set<String> vl = new HashSet<String>(caa.getValoriList());
												if(!vl.containsAll(vn)) {
													vl.addAll(vn);
													caa.setValori(ApplicationUtil.detoken( new ArrayList<String>(vl), ConstantsGlobal.colonSeparator));
													save = true;
												}
											}
											if(caa.getData() || caa.getDataora() || caa.getRegulare() || caa.getUnivoco()) {
												String v = ApplicationUtil.trimToEmpty(cc.getValues());
												if(!ApplicationUtil.equals(caa.getValori(), v)) {
													caa.setValori(v);
													save = true;
												}
											} 
											if(!ApplicationUtil.equals(caa.getTipo(), cc.getTipo())) {
												caa.setTipo(cc.getTipo());
												save = true;
											}
											String gruppo = ApplicationUtil.impostaGruppo(campo.getEtichetta(), campo.getColore());
											if(!ApplicationUtil.equals(caa.getGruppo(), gruppo)) {
												caa.setGruppo(gruppo);
												save = true;
											}
											Boolean b1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getObbligatorio(), cc.getObbligatorio());
											if(!ApplicationUtil.equals(caa.getObbligatorio(), b1)) {
												caa.setObbligatorio(b1);
												save = true;
											}
											Boolean o1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_operatore(), cc.getModificabile_operatore());
											if(!ApplicationUtil.equals(caa.getModificabile_operatore(), o1)) {
												caa.setModificabile_operatore(o1);
												save = true;
											}
											Boolean o2 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_operatore(), cc.getVisibile_operatore());
											if(!ApplicationUtil.equals(caa.getVisibile_operatore(), o2)) {
												caa.setVisibile_operatore(o2);
												save = true;
											}
											Boolean a1 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getModificabile_agente(), cc.getModificabile_agente());
											if(!ApplicationUtil.equals(caa.getModificabile_agente(), a1)) {
												caa.setModificabile_agente(a1);
												save = true;
											}
											Boolean a2 = ApplicationUtil.impostaCampoB(cc.getEredita(), campo.getVisibile_agente(), cc.getVisibile_agente());
											if(!ApplicationUtil.equals(caa.getVisibile_agente(), a2)) {
												caa.setVisibile_agente(a2);
												save = true;
											}
											String e = ApplicationUtil.impostaCampoS(cc.getEredita(),campo.getEtichetta(), cc.getEtichetta(),  ConstantsGlobal.ereditaEtichetta);
											if(!ApplicationUtil.equals(caa.getEtichetta(), e)) {
												caa.setEtichetta(e);
												save = true;
											}
											String h = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDescrizione(), cc.getDescrizione());
											if(!ApplicationUtil.equals(caa.getDescrizione(), h)) {
												caa.setDescrizione(h);
												save = true;
											}
											String c = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getColore(), cc.getColore());
											if(!ApplicationUtil.equals(caa.getColore(), c)) {
												caa.setColore(c);
												save = true;
											}
											String s = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getStile(), cc.getStile());
											if(!ApplicationUtil.equals(caa.getStile(), s)) {
												caa.setStile(s);
												save = true;
											}
											String d = ApplicationUtil.impostaCampoC(cc.getEredita(), campo.getDefaults(), cc.getDefaults());
											if(!ApplicationUtil.equals(caa.getDefaults(), d)) {
												caa.setDefaults(d);
												save = true;
											}
											if(save) {
												caa = (CampoAggiuntivoAnagrafica) anagraficaDao.salvaCampoAggiuntivo(caa);
											}
										}
										DatoAggiuntivoAnagrafica daa = new DatoAggiuntivoAnagrafica();
										daa.setCampo_aggiuntivo(caa);
										if(!dr.contains(daa)){
											daa.setAnagrafica(anagrafica);
										} else {
											int index = dr.indexOf(daa);
											if(index > -1) {
												daa = dr.get(index);
											}
										}
										if(caa.getData()) {
											daa.setValore_data(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getDataora()) {
											daa.setValore_datatime(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getOra()) {
											daa.setValore_time(ApplicationUtil.castDate(val.getSingolo()));
										} else if(caa.getBooleano()) {
											daa.setValore(ApplicationUtil.castBoolean(val.getSingolo()));
										} else if(caa.getPickList() || caa.getManyCheckbox() || caa.getManyMenu()) {
											daa.setValore(ApplicationUtil.detoken(val.getMultiplo(), ConstantsGlobal.colonSeparator));
										} else {
											daa.setValore(ApplicationUtil.castString(val.getSingolo()));
										}
										anagraficaDao.salvaDatoAggiuntivo(daa);
									}
								}
							}
						}
					} else {
						DatoAggiuntivoAnagrafica daa = new DatoAggiuntivoAnagrafica();
						daa.setCampo_aggiuntivo(dato.getCampo());
						if(!dr.contains(daa)){
							daa.setAnagrafica(anagrafica);
						} else {
							int index = dr.indexOf(daa);
							if(index > -1) {
								daa = dr.get(index);
							}
						}
						if(dato.getCampo().getManyCheckbox() || dato.getCampo().getManyMenu() || dato.getCampo().getPickList()) {
							daa.setValore(ApplicationUtil.detoken(dato.getValore_multiplo(), ConstantsGlobal.colonSeparator));
						} else if(dato.getCampo().getData()) {
							daa.setValore_data(dato.getValore_data());
						} else if(dato.getCampo().getDataora()) {
							daa.setValore_datatime(dato.getValore_datatime());
						} else if(dato.getCampo().getOra()) {
							daa.setValore_time(dato.getValore_time());
						} else {
							daa.setValore(dato.getValore());
						}
						anagraficaDao.salvaDatoAggiuntivo(daa);
					}
				}
			}
		}
	}
	public DomandaAnagrafica creaDomanda(SondaggioAnagrafica sondaggio, CampoAggiuntivoAnagrafica campoAggiuntivo, String posizione, Boolean obbligatoria, Boolean visibile) {
		DomandaAnagrafica domanda = new DomandaAnagrafica();
		domanda.setCampo_aggiuntivo(campoAggiuntivo);
		Short posizioneShort = 1;
		try {
			posizioneShort = Short.parseShort(posizione);
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		domanda.setPosizione(posizioneShort);
		domanda.setObbligatoria(obbligatoria);
		domanda.setVisibile(visibile);
		domanda.setSondaggio(sondaggio);
		salvaDomanda(domanda);
		return domanda;
	}
	public void creaDomande(SondaggioAnagrafica sondaggio, List<CampoAggiuntivoAnagrafica> cas, Short last) {
		if(ApplicationUtil.notEmpty(cas)) {
			Iterator<CampoAggiuntivoAnagrafica> itC = cas.iterator();
			while (itC.hasNext()) {
				CampoAggiuntivoAnagrafica campoAggiuntivo = (CampoAggiuntivoAnagrafica) itC.next();
				DomandaAnagrafica domanda = new DomandaAnagrafica();
				domanda.setCampo_aggiuntivo(campoAggiuntivo);
				domanda.setPosizione(last++);
				domanda.setObbligatoria(false);
				domanda.setVisibile(true);
				domanda.setSondaggio(sondaggio);
				salvaDomanda(domanda);
			}
		}
	}
	public DomandaAnagrafica salvaDomanda(DomandaAnagrafica domanda) {
		return domanda = (DomandaAnagrafica) salvaAggiorna(domanda);
	}
	public void eliminaDomanda(DomandaAnagrafica domanda) {
		if (domanda != null)
			cancellaAggiorna(domanda);
	}
	public void eliminaDomande(List<DomandaAnagrafica> domande) {
		if(ApplicationUtil.notEmpty(domande)) {
			Iterator<DomandaAnagrafica> idD = domande.iterator();
			while (idD.hasNext()) {
				DomandaAnagrafica domanda = (DomandaAnagrafica) idD.next();
				cancellaAggiorna(domanda);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<Long> getCampiEsistenti(Long sondaggioId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaAnagrafica.class);
		criteria.add(Restrictions.eq("sondaggio.id",sondaggioId));
		criteria.addOrder(Order.asc("posizione"));
		criteria.setProjection(Projections.distinct(Projections.property("campo_aggiuntivo.id")));
		return getList(criteria);
	}
	public List<DomandaAnagrafica> copiaDomande(SondaggioAnagrafica sondaggioDst,Long sondaggioIdSrc) {
		List<DomandaAnagrafica> domande = getDomandeSondaggio(sondaggioIdSrc);
		List<Long> domandeE = getCampiEsistenti(sondaggioDst.getId());
		List<DomandaAnagrafica> domande_copia = new ArrayList<DomandaAnagrafica>();
		Iterator<DomandaAnagrafica> objIter = domande.iterator();
		while(objIter.hasNext()) {
			DomandaAnagrafica domanda = objIter.next();
			if(!domandeE.contains(domanda.getCampo_aggiuntivo().getId())) {
				//Domanda nuovo = (Domanda) SerializationUtils.clone(domanda);
				DomandaAnagrafica nuovo = domanda;
				nuovo.setId(null);
				nuovo.setSondaggio(sondaggioDst);
				salvaDomanda(nuovo);
				domande_copia.add(nuovo);
			}
		}
		return domande_copia;
	}
	public void eliminaDatiAggiuntiviAnagrafica(List<DatoAggiuntivoAnagrafica> dati) {
		cancellaLista(dati);
	}
	public void salvaDatiAggiuntiviAnagrafica(List<DatoAggiuntivoAnagrafica> dati) {
		if(ApplicationUtil.notEmpty(dati)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<DatoAggiuntivoAnagrafica> objIter = dati.iterator();
			while(objIter.hasNext()) {
				DatoAggiuntivoAnagrafica dato = objIter.next();
				dato.imposta();
				session.saveOrUpdate(dato);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	public void aggiungiDomandaACampo(Long campoId, Long sondaId) {
		CampoAggiuntivoContratto campo = getFullCampoConDomande(campoId);
		SondaggioContratto sondaggio = getSondaggio(sondaId);
		if (sondaggio != null && campo != null) {
			DomandaContratto dom = new DomandaContratto();
			dom.setCampo_aggiuntivo(campo);
			dom.setSondaggio(sondaggio);
			dom.setObbligatoria(false);
			dom.setVisibile(true);
			dom.setPosizione((short) 1);
			salvaAggiorna(dom);
			campo.getDomande().add(dom);
			salvaAggiorna(campo);
		}
	}
	public void rimuoviDomandaACampo(Long campoId, Long sondaId) {
		DomandaContratto domanda = getDomanda(campoId, sondaId);
		if (domanda != null) {
			cancellaAggiorna(domanda);
		}
	}
	public void aggiungiDomandeTuttiACampo(Long campoId) {
		CampoAggiuntivoContratto campo = getFullCampoConDomande(campoId);
		List<SondaggioContratto> sondaggi = getDomandeNotInCampo(campoId);
		if (ApplicationUtil.notEmpty(sondaggi) && campo != null) {
			Iterator<SondaggioContratto> idSon = sondaggi.iterator();
			List<DomandaContratto> dml = new ArrayList<DomandaContratto>();
			while (idSon.hasNext()) {
				SondaggioContratto son = (SondaggioContratto) idSon.next();
				DomandaContratto dom = new DomandaContratto();
				dom.setCampo_aggiuntivo(campo);
				dom.setSondaggio(son);
				dom.setObbligatoria(false);
				dom.setVisibile(true);
				dom.setPosizione((short) 1);
				salvaAggiorna(dom);
				dml.add(dom);
			}
			if (ApplicationUtil.notEmpty(dml) ){
				campo.getDomande().addAll(dml);
				salvaAggiorna(campo);
			}
		}
	}
	public void rimuoviDomandeTuttiACampo(Long campoId) {
		List<DomandaContratto> domande = getDomande(campoId);
		if (ApplicationUtil.notEmpty(domande)) {
			Iterator<DomandaContratto> idSon = domande.iterator();
			while (idSon.hasNext()) {
				DomandaContratto son = (DomandaContratto) idSon.next();
				cancellaAggiorna(son);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<SondaggioContratto> getDomandeNotInCampo(Long campoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		DetachedCriteria criteriaU = DetachedCriteria.forClass(SondaggioContratto.class);
		criteriaU.createCriteria("domande").add(Restrictions.eq("campo_aggiuntivo.id", campoId));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<DomandaContratto> getDomandeInCampo(Long campoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaContratto.class);
		criteria.add(Restrictions.eq("campo_aggiuntivo.id", campoId));
		return getList(criteria);
	}
	private CampoAggiuntivoContratto getFullCampoConDomande(Long campoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
		criteria.add(Restrictions.eq("id", campoId));
		criteria.setFetchMode("domande", FetchMode.JOIN);
		return (CampoAggiuntivoContratto) getUniqueResult(criteria);
	}
	private SondaggioContratto getSondaggio(Long sondaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioContratto.class);
		criteria.add(Restrictions.eq("id", sondaId));
		return (SondaggioContratto) getUniqueResult(criteria);
	}
	private DomandaContratto getDomanda(Long campoId, Long sondaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaContratto.class);
		criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("id", campoId));
		criteria.createCriteria("sondaggio").add(Restrictions.eq("id", sondaId));
		return (DomandaContratto) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	private List<DomandaContratto> getDomande(Long campoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(DomandaContratto.class);
		criteria.createCriteria("campo_aggiuntivo").add(Restrictions.eq("id", campoId));
		return getList(criteria);
	}
	public void aggiungiSondaggioAnagraficaAdEsitoArchiviazione(Long sondaggioId, Long esitoId) {
		SondaggioAnagrafica sondaggio = getFullSondaggioAnagraficaEsitoArchiviazione(sondaggioId);
		EsitoArchiviazione esito = campagnaDao.getEsitoById(esitoId);
		if (sondaggio != null && esito != null) {
			if (!sondaggio.getEsiti().contains(esito)) {
				sondaggio.getEsiti().add(esito);
				salvaAggiorna(sondaggio);
			}
		}
	}
	public void rimuoviSondaggioAnagraficaAdEsitoArchiviazione(Long sondaggioId, Long esitoId) {
		SondaggioAnagrafica sondaggio = getFullSondaggioAnagraficaEsitoArchiviazione(sondaggioId);
		EsitoArchiviazione esito = campagnaDao.getEsitoById(esitoId);
		if (sondaggio != null && esito != null) {
			if (sondaggio.getEsiti().contains(esito)) {
				sondaggio.getEsiti().remove(esito);
				salvaAggiorna(sondaggio);
			}
		}
	}
	public void aggiungiEsitoArchiviazioneTuttiAdSondaggioAnagrafica(Long sondaggioId) {
		SondaggioAnagrafica sondaggio  = getFullSondaggioAnagraficaEsitoArchiviazione(sondaggioId);
		List<EsitoArchiviazione> esito = getEsitoArchiviazioneNotInSondaggioAnagrafica(sondaggioId);
		if (ApplicationUtil.notEmpty(esito) && sondaggio != null) {
			sondaggio.getEsiti().addAll(esito);
			salvaAggiorna(sondaggio);
		}
	}
	public void rimuoviEsitoArchiviazioneTuttiAdSondaggioAnagrafica(Long sondaggioId) {
		SondaggioAnagrafica sondaggio  = getFullSondaggioAnagraficaEsitoArchiviazione(sondaggioId);
		List<EsitoArchiviazione> esito = getEsitoArchiviazioneInSondaggioAnagrafica(sondaggioId);
		if (ApplicationUtil.notEmpty(esito) && sondaggio != null) {
			sondaggio.getEsiti().removeAll(esito);
			salvaAggiorna(sondaggio);
		}
	}
	public void aggiungiEsitoArchiviazioneAdSondaggioAnagrafica(Long esitoId, Long sondaggioId) {
		EsitoArchiviazione esito = getFullEsitoArchiviazioneSondaggioAnagrafica(esitoId);
		SondaggioAnagrafica sondaggio = get(sondaggioId);
		if (esito != null && sondaggio != null) {
			if (!esito.getSondaggi().contains(sondaggio)) {
				esito.getSondaggi().add(sondaggio);
				salvaAggiorna(esito);
			}
		}
	}
	public void rimuoviEsitoArchiviazioneAdSondaggioAnagrafica(Long esitoId, Long sondaggioId) {
		EsitoArchiviazione esito = getFullEsitoArchiviazioneSondaggioAnagrafica(esitoId);
		SondaggioAnagrafica sondaggio = get(sondaggioId);
		if (esito != null && sondaggio != null) {
			if (esito.getSondaggi().contains(sondaggio)) {
				esito.getSondaggi().remove(sondaggio);
				salvaAggiorna(esito);
			}
		}
	}
	public void aggiungiSondaggioAnagraficaTuttiAdEsitoArchiviazione(Long esitoId) {
		EsitoArchiviazione esito  = getFullEsitoArchiviazioneSondaggioAnagrafica(esitoId);
		List<SondaggioAnagrafica> sondaggio = getSondaggioAnagraficaNotInEsitoArchiviazione(esitoId);
		if (ApplicationUtil.notEmpty(sondaggio) && esito != null) {
			esito.getSondaggi().addAll(sondaggio);
			salvaAggiorna(esito);
		}
	}
	public void rimuoviSondaggioAnagraficaTuttiAdEsitoArchiviazione(Long esitoId) {
		EsitoArchiviazione esito  = getFullEsitoArchiviazioneSondaggioAnagrafica(esitoId);
		List<SondaggioAnagrafica> sondaggio = getSondaggioAnagraficaInEsitoArchiviazione(esitoId);
		if (ApplicationUtil.notEmpty(sondaggio) && esito != null) {
			esito.getSondaggi().removeAll(sondaggio);
			salvaAggiorna(esito);
		}
	}
	public EsitoArchiviazione getFullEsitoArchiviazioneSondaggioAnagrafica(Long esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esito));
		criteria.setFetchMode("sondaggi", FetchMode.JOIN);
		return  (EsitoArchiviazione) getUniqueResult(criteria);
	}
	public SondaggioAnagrafica getFullSondaggioAnagraficaEsitoArchiviazione(Long sondaggio) {
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioAnagrafica.class);
		criteria.add(Restrictions.idEq(sondaggio));
		criteria.setFetchMode("esiti", FetchMode.JOIN);
		return (SondaggioAnagrafica) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<SondaggioAnagrafica> getSondaggioAnagraficaInEsitoArchiviazione(Long esito) {
		if(esito != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioAnagrafica.class);
			criteria.createCriteria("esiti").add(Restrictions.idEq(esito));
			criteria.addOrder(Order.asc("nome"));
			return getList(criteria);
		}	else {
			return new ArrayList<SondaggioAnagrafica>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<SondaggioAnagrafica> getSondaggioAnagraficaNotInEsitoArchiviazione(Long esito) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(SondaggioAnagrafica.class);
		criteriaU.createCriteria("esiti").add(Restrictions.idEq(esito));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(SondaggioAnagrafica.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneNotInSondaggioAnagrafica(Long sondaggio) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteriaU.createCriteria("sondaggi").add(Restrictions.idEq(sondaggio));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitoArchiviazioneInSondaggioAnagrafica(Long sondaggio) {
		if(sondaggio != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			criteria.createCriteria("sondaggi").add(Restrictions.idEq(sondaggio));
			criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<EsitoArchiviazione>();
		}
	}
	public List<DatoSondaggioAnagrafica> getSondaggioForm(SondaggioAnagrafica sondaggio) {
		List<DomandaAnagrafica> domande = getDomandeSondaggio(sondaggio.getId());
		List<DatoSondaggioAnagrafica> dati = new ArrayList<DatoSondaggioAnagrafica>();
		Iterator<DomandaAnagrafica> objIter = domande.iterator();
		while(objIter.hasNext()) {
			DomandaAnagrafica domanda = objIter.next();
			addDomanda(dati, new DatoSondaggioAnagrafica(domanda.getCampo_aggiuntivo(), domanda.getObbligatoria(), domanda.getVisibile()));
		}
		return dati;
	}
}