package it.callite.dao;
import java.util.Iterator;
import java.util.List;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.column.generic.GenericColumn;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.StoriaAppuntamentoCriteria;
import it.callite.criteria.StoriaContattoCriteria;
import it.callite.criteria.StoriaContrattoCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Campagna;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoContratto;
import it.callite.model.EsitoAppuntamento;
import it.callite.model.EsitoChiamata;
import it.callite.model.EsitoContratto;
import it.callite.model.StoriaAppuntamento;
import it.callite.model.StoriaContatto;
import it.callite.model.StoriaContratto;
import it.callite.model.Utente;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.LoggerColonna;
import it.callite.utility.StatoIntervallo;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.EsitoAppuntamentoSI;
import it.callite.utility.item.EsitoChiamataSI;
import it.callite.utility.item.EsitoContrattoSI;
@SuppressWarnings("serial")
@Repository("storiaDao")
public class StoriaDao extends GenericDao {
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private OperazioneDao operazioneDao;
	public void eliminaStoriaContatto(Utente utente, StoriaContatto contatto) {
		String p = ApplicationUtil.asString(ConstantsLanguage.STORIA.stampa() + " " +  ConstantsLanguage.CONTATTO.stampa(), contatto);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_STORIA_CONTATTO);
		cancellaAggiorna(contatto);
	}
	public void eliminaStoriaAppuntamento(Utente utente, StoriaAppuntamento appuntamento) {
		String p = ApplicationUtil.asString(ConstantsLanguage.STORIA.stampa() + " " +  ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_STORIA_APPUNTAMENTO);
		cancellaAggiorna(appuntamento);
	}
	public void eliminaStoriaContratto(Utente utente, StoriaContratto contratto) {
		String p = ApplicationUtil.asString(ConstantsLanguage.STORIA.stampa() + " " +  ConstantsLanguage.CONTRATTO.stampa(), contratto);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_STORIA_CONTRATTO);
		cancellaAggiorna(contratto);
	}
	
	public void eliminaContattoSelezionati(Utente utente, List<StoriaContatto> lst) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(lst), lst);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_STORIA_CONTATTO);
		eliminaSelezionati(lst);
	}
	public void eliminaContrattoSelezionati(Utente utente, List<StoriaContratto> lst) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(lst), lst);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_STORIA_CONTRATTO);
		eliminaSelezionati(lst);
	}
	public void eliminaAppuntamentoSelezionati(Utente utente, List<StoriaAppuntamento> lst) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " +  ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(lst), lst);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_STORIA_APPUNTAMENTO);
		eliminaSelezionati(lst);
	}
	
	public void salvaStoriaContatto(StoriaContatto contatto) {
		salvaAggiorna(contatto);
	}
	public void mostraStoriaContattoSelezionati(List<StoriaContatto> storico) {
		Iterator<StoriaContatto> itCont = storico.iterator();
		while (itCont.hasNext()) {
			StoriaContatto storia = (StoriaContatto) itCont.next();
			storia.setInvisibile(false);
			salvaStoriaContatto(storia);
		}
	}
	public void nascondiStoriaContattoSelezionati(List<StoriaContatto> storico) {
		Iterator<StoriaContatto> itCont = storico.iterator();
		while (itCont.hasNext()) {
			StoriaContatto storia = (StoriaContatto) itCont.next();
			storia.setInvisibile(true);
			salvaStoriaContatto(storia);
		}
	}
	private ExtendedCriteria ricercaStoriaContattoManagerCriteria(Utente utente, StoriaContattoCriteria cont) {
		
		if(cont != null) cont = (StoriaContattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCL = criteria.createCriteria("lista", ApplicationUtil.leftJoin());
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA, cont.getRicerca_anagrafica());
		}
		
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		
		ApplicationUtil.proprieta(false, filtro, criteria, "archiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		
		if(ApplicationUtil.gtZero(cont.getOperatoreId())) {
			criteria.add(Restrictions.eq("operatore.id",cont.getOperatoreId()));
		}
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaCA, criteriaCL);
		ApplicationUtil.pannello(cont.getPannello(), criteria);
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> ricercaStoriaContattoManager(Utente utente, StoriaContattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContattoManagerCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contatto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContattoManagerSize(Utente utente, StoriaContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContattoManagerCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaAppuntamentoManagerCriteria(Utente utente, StoriaAppuntamentoCriteria cont) {
		if(cont != null) cont = (StoriaAppuntamentoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("appuntamento").createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.eq("campagna.id", cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getUtenteId())) {
			criteria.add(Restrictions.eq("utente.id",cont.getUtenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
	
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento> ricercaStoriaAppuntamentoManager(Utente utente, StoriaAppuntamentoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoManagerCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("appuntamento", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaAppuntamentoManagerSize(Utente utente, StoriaAppuntamentoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoManagerCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContrattoManagerCriteria(Utente utente, StoriaContrattoCriteria cont) {
		if(cont != null) cont = (StoriaContrattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contratto");
		DetachedCriteria criteriaC = criteriaCo.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(cont, criteriaCo);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(cont, criteriaCo);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.eq("campagna.id", cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getUtenteId())) {
			criteria.add(Restrictions.eq("utente.id",cont.getUtenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto> ricercaStoriaContrattoManager(Utente utente, StoriaContrattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContrattoManagerCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contratto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContrattoManagerSize(Utente utente, StoriaContrattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContrattoManagerCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContattoBackofficeCriteria(Utente utente, StoriaContattoCriteria cont) {
		if(cont != null) cont = (StoriaContattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCL = criteria.createCriteria("lista", ApplicationUtil.leftJoin());
		
		criteriaC.createCriteria("esitoArchiviazione").add(Restrictions.eq("backoffice", Boolean.TRUE));
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		
		ApplicationUtil.proprieta(false, filtro, criteria, "archiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		if(ApplicationUtil.gtZero(cont.getOperatoreId())) {
			criteria.add(Restrictions.eq("operatore.id",cont.getOperatoreId()));
		}
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaCA, criteriaCL);
		ApplicationUtil.pannello(cont.getPannello(), criteria);
		cont.impostaRicercaDate(criteria);
		
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> ricercaStoriaContattoBackoffice(Utente utente, StoriaContattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContattoBackofficeCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contatto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContattoBackofficeSize(Utente utente, StoriaContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContattoBackofficeCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaAppuntamentoBackofficeCriteria(Utente utente, StoriaAppuntamentoCriteria cont) {
		if(cont != null) cont = (StoriaAppuntamentoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("appuntamento").createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		
		criteriaC.createCriteria("esitoArchiviazione").add(Restrictions.eq("backoffice", Boolean.TRUE));
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.eq("campagna.id", cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getUtenteId())) {
			criteria.add(Restrictions.eq("utente.id",cont.getUtenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento> ricercaStoriaAppuntamentoBackoffice(Utente utente, StoriaAppuntamentoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoBackofficeCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("appuntamento", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaAppuntamentoBackofficeSize(Utente utente, StoriaAppuntamentoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoBackofficeCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContrattoBackofficeCriteria(Utente utente, StoriaContrattoCriteria cont) {
		if(cont != null) cont = (StoriaContrattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contratto");
		DetachedCriteria criteriaC = criteriaCo.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		
		criteriaC.createCriteria("esitoArchiviazione").add(Restrictions.eq("backoffice", Boolean.TRUE));
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(cont, criteriaCo);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(cont, criteriaCo);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaC.add(Restrictions.eq("campagna.id", cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getUtenteId())) {
			criteria.add(Restrictions.eq("utente.id",cont.getUtenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto> ricercaStoriaContrattoBackoffice(Utente utente, StoriaContrattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContrattoBackofficeCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contratto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContrattoBackofficeSize(Utente utente, StoriaContrattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContrattoBackofficeCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContattoSupervisoreCriteria(Utente utente, StoriaContattoCriteria cont) {
		if(cont != null) cont = (StoriaContattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "campagna", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCL = criteria.createCriteria("lista", ApplicationUtil.leftJoin());
		impostazioneDao.subCriteriaRoleSupervisore(utente.getId(), criteria, "campagna", "operatore");
		impostazioneDao.subCriteriaSupervisore(utente.getId(), criteriaOp, "id");
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(utente.getId(), criteriaCA, "id");
			}
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		
		ApplicationUtil.proprieta(false, filtro, criteria, "archiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		if(ApplicationUtil.gtZero(cont.getOperatoreId())) {
			criteriaOp.add(Restrictions.idEq(cont.getOperatoreId()));
		}
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaCA, criteriaCL);
		ApplicationUtil.pannello(cont.getPannello(), criteria);
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> ricercaStoriaContattoSupervisore(Utente utente, StoriaContattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContattoSupervisoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contatto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContattoSupervisoreSize(Utente utente, StoriaContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContattoSupervisoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaAppuntamentoSupervisoreCriteria(Utente utente, StoriaAppuntamentoCriteria cont) {
		if(cont != null) cont = (StoriaAppuntamentoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("utente", "operatore");
		DetachedCriteria criteriaC = criteria.createCriteria("appuntamento").createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "campagna");
		impostazioneDao.subCriteriaRoleSupervisore(utente.getId(), criteria, "campagna", "operatore");
		impostazioneDao.subCriteriaSupervisore(utente.getId(), criteriaOp, "id");
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(utente.getId(), criteriaCA, "id");
			}
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getUtenteId())) {
			criteriaOp.add(Restrictions.idEq(cont.getUtenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento> ricercaStoriaAppuntamentoSupervisore(Utente utente, StoriaAppuntamentoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoSupervisoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("appuntamento", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaAppuntamentoSupervisoreSize(Utente utente, StoriaAppuntamentoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoSupervisoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContrattoSupervisoreCriteria(Utente utente, StoriaContrattoCriteria cont) {
		if(cont != null) cont = (StoriaContrattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		DetachedCriteria criteriaOp = criteria.createCriteria("utente", "operatore");
		DetachedCriteria criteriaCo = criteria.createCriteria("contratto");
		DetachedCriteria criteriaC = criteriaCo.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "campagna");
		impostazioneDao.subCriteriaRoleSupervisore(utente.getId(), criteria, "campagna", "operatore");
		impostazioneDao.subCriteriaSupervisore(utente.getId(), criteriaOp, "id");
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(utente.getId(), criteriaCA, "id");
			}
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(cont, criteriaCo);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(cont, criteriaCo);
		}
		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if(ApplicationUtil.gtZero(cont.getUtenteId())) {
			criteriaOp.add(Restrictions.idEq(cont.getUtenteId()));
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto> ricercaStoriaContrattoSupervisore(Utente utente, StoriaContrattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContrattoSupervisoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contratto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContrattoSupervisoreSize(Utente utente, StoriaContrattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContrattoSupervisoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContattoOperatoreCriteria(Utente utente, StoriaContattoCriteria cont) {
		if(cont != null) cont = (StoriaContattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaOp = criteria.createCriteria("operatore", "operatore");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "campagna", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCL = criteria.createCriteria("lista", ApplicationUtil.leftJoin());
		
		if(ApplicationUtil.isTrue(cont.getAzione())) {
			impostazioneDao.subCriteriaRoleOperatore(utente.getId(), criteria, "campagna", "operatore");
			impostazioneDao.subCriteriaOperatore(utente.getId(), criteriaOp, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaOperatore(utente.getId(), criteriaC, "id");
			}
			if (ApplicationUtil.gtZero(cont.getOperatoreId())) {
				criteriaOp.add(Restrictions.idEq(cont.getOperatoreId()));
			}
		} else {
			criteriaOp.add(Restrictions.idEq(utente.getId()));
		}
		
		criteria.add(Restrictions.eq("invisibile", Boolean.FALSE));
		
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		
		ApplicationUtil.proprieta(false, filtro, criteria, "archiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaCA, criteriaCL);
		ApplicationUtil.pannello(cont.getPannello(), criteria);
		cont.impostaRicercaDate(criteria);
		
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> ricercaStoriaContattoOperatore(Utente utente, StoriaContattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContattoOperatoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contatto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContattoOperatoreSize(Utente utente, StoriaContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContattoOperatoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaAppuntamentoOperatoreCriteria(Utente utente, StoriaAppuntamentoCriteria cont) {
		if(cont != null) cont = (StoriaAppuntamentoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		DetachedCriteria criteriaCr = criteria.createCriteria("appuntamento");
		DetachedCriteria criteriaC = criteriaCr.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaOp = criteria.createCriteria("utente", "operatore");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "campagna");
		criteriaCr.createCriteria("esito").add(Restrictions.eq("visibile", Boolean.TRUE));
		if(ApplicationUtil.isTrue(cont.getAzione())) {
			impostazioneDao.subCriteriaRoleOperatore(utente.getId(), criteria, "campagna", "operatore");
			impostazioneDao.subCriteriaOperatore(utente.getId(), criteriaOp, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaOperatore(utente.getId(), criteriaCA, "id");
			}
			if (ApplicationUtil.gtZero(cont.getOperatoreId())) {
				criteriaOp.add(Restrictions.idEq(cont.getOperatoreId()));
			}
		} else {
			criteriaOp.add(Restrictions.idEq(utente.getId()));
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento> ricercaStoriaAppuntamentoOperatore(Utente utente, StoriaAppuntamentoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoOperatoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("appuntamento", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaAppuntamentoOperatoreSize(Utente utente, StoriaAppuntamentoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoOperatoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContrattoOperatoreCriteria(Utente utente, StoriaContrattoCriteria cont) {
		if(cont != null) cont = (StoriaContrattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		DetachedCriteria criteriaCr = criteria.createCriteria("contratto");
		DetachedCriteria criteriaC = criteriaCr.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaOp = criteria.createCriteria("utente", "operatore");
		
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "campagna");
		
		criteriaCr.createCriteria("esito").add(Restrictions.eq("visibile", Boolean.TRUE));
		if(ApplicationUtil.isTrue(cont.getAzione())) {
			impostazioneDao.subCriteriaRoleOperatore(utente.getId(), criteria, "campagna", "operatore");
			impostazioneDao.subCriteriaOperatore(utente.getId(), criteriaOp, "id");
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCampagnaOperatore(utente.getId(), criteriaCA, "id");
			}
			if (ApplicationUtil.gtZero(cont.getOperatoreId())) {
				criteriaOp.add(Restrictions.idEq(cont.getOperatoreId()));
			}
		} else {
			criteriaOp.add(Restrictions.idEq(utente.getId()));
		}
		
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(cont, criteriaCr);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(cont, criteriaCr);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto> ricercaStoriaContrattoOperatore(Utente utente, StoriaContrattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContrattoOperatoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contratto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContrattoOperatoreSize(Utente utente, StoriaContrattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContrattoOperatoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContattoPartnerCriteria(Utente utente, StoriaContattoCriteria cont) {
		if(cont != null) cont = (StoriaContattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCL = criteria.createCriteria("lista", ApplicationUtil.leftJoin());
		
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			impostazioneDao.subCriteriaCampagnaPartner(utente.getId(), criteriaCA, "id");
		}
		
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		
		ApplicationUtil.proprieta(false, filtro, criteria, "archiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaCA, criteriaCL);
		ApplicationUtil.pannello(cont.getPannello(), criteria);
		cont.impostaRicercaDate(criteria);
		
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> ricercaStoriaContattoPartner(Utente utente, StoriaContattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContattoPartnerCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contatto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContattoPartnerSize(Utente utente, StoriaContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContattoPartnerCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaAppuntamentoPartnerCriteria(Utente utente, StoriaAppuntamentoCriteria cont) {
		if(cont != null) cont = (StoriaAppuntamentoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		DetachedCriteria criteriaC = criteria.createCriteria("appuntamento").createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna");
		
		impostazioneDao.subCriteriaCampagnaPartner(utente.getId(), criteriaCA, "id");
		
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento> ricercaStoriaAppuntamentoPartner(Utente utente, StoriaAppuntamentoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoPartnerCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("appuntamento", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaAppuntamentoPartnerSize(Utente utente, StoriaAppuntamentoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoPartnerCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContrattoPartnerCriteria(Utente utente, StoriaContrattoCriteria cont) {
		if(cont != null) cont = (StoriaContrattoCriteria) cont.loadCriteria();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contratto");
		DetachedCriteria criteriaC = criteriaCo.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna");
		
		impostazioneDao.subCriteriaCampagnaPartner(utente.getId(), criteriaCA, "id");
		
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(cont, criteriaCo);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(cont, criteriaCo);
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		}
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto> ricercaStoriaContrattoPartner(Utente utente, StoriaContrattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContrattoPartnerCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contratto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContrattoPartnerSize(Utente utente, StoriaContrattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContrattoPartnerCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContattoCoordinatoreCriteria(Utente utente, StoriaContattoCriteria cont) {
		if(cont != null) cont = (StoriaContattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContatto.class);
		DetachedCriteria criteriaC = criteria.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaAg = criteriaC.createCriteria("agente", "agente");
		DetachedCriteria criteriaCA = criteria.createCriteria("campagna", "campagna", ApplicationUtil.leftJoin());
		DetachedCriteria criteriaCL = criteria.createCriteria("lista", ApplicationUtil.leftJoin());
		impostazioneDao.subCriteriaRoleCoordinatore(utente.getId(), criteria, "campagna", "agente");
		if (ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteriaAg.add(Restrictions.idEq(cont.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCoordinatore(utente.getId(), criteriaAg, "id");
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCoordinatoreCampagna(utente.getId(), criteriaCA, "id");
			}
		}
		
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsitoChiamataId(), cont.getEsitoChiamataIds());
		
		ApplicationUtil.proprieta(false, filtro, criteria, "archiviazione.id", cont.getEsitoArchiviazioneId(), cont.getEsitoArchiviazioneIds());
		
		if (ApplicationUtil.isNotBlank(cont.getOperatore())) {
			DetachedCriteria cOp = criteria.createCriteria("operatore");
			ApplicationUtil.operatore(cOp, cont.getOperatore());
		}
		ApplicationUtil.statoLC(cont.getStatoIntervallo(), criteriaCA, criteriaCL);
		ApplicationUtil.pannello(cont.getPannello(), criteria);
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContatto> ricercaStoriaContattoCoordinatore(Utente utente, StoriaContattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContattoCoordinatoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contatto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContattoCoordinatoreSize(Utente utente, StoriaContattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContattoCoordinatoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaAppuntamentoCoordinatoreCriteria(Utente utente, StoriaAppuntamentoCriteria cont) {
		if(cont != null) cont = (StoriaAppuntamentoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaAppuntamento.class);
		DetachedCriteria criteriaAp = criteria.createCriteria("appuntamento");
		DetachedCriteria criteriaC = criteriaAp.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaAg = criteriaAp.createCriteria("agente", "agente");
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "campagna");
		impostazioneDao.subCriteriaRoleCoordinatore(utente.getId(), criteria, "campagna", "agente");
		if (ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteriaAg.add(Restrictions.idEq(cont.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCoordinatore(utente.getId(), criteriaAg, "id");
		}
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCoordinatoreCampagna(utente.getId(), criteriaCA, "id");
			}
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if (ApplicationUtil.isNotBlank(cont.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("utente");
			ApplicationUtil.operatore(cOp, cont.getUtente());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaAppuntamento> ricercaStoriaAppuntamentoCoordinatore(Utente utente, StoriaAppuntamentoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoCoordinatoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("appuntamento", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaAppuntamentoCoordinatoreSize(Utente utente, StoriaAppuntamentoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaAppuntamentoCoordinatoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	private ExtendedCriteria ricercaStoriaContrattoCoordinatoreCriteria(Utente utente, StoriaContrattoCriteria cont) {
		if(cont != null) cont = (StoriaContrattoCriteria) cont.loadCriteria();
		Boolean sd = impostazioneDao.separaDati();
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(StoriaContratto.class);
		DetachedCriteria criteriaCo = criteria.createCriteria("contratto");
		DetachedCriteria criteriaC = criteriaCo.createCriteria("contatto");
		DetachedCriteria criteriaA = criteriaC.createCriteria("anagrafica");
		DetachedCriteria criteriaAg = criteriaCo.createCriteria("agente", "operatore");
		
		DetachedCriteria criteriaCA = criteriaC.createCriteria("campagna", "agente");
		impostazioneDao.subCriteriaRoleCoordinatore(utente.getId(), criteria, "campagna", "agente");
		if (ApplicationUtil.gtZero(cont.getAgenteId())) {
			criteriaAg.add(Restrictions.idEq(cont.getAgenteId()));
		} else {
			impostazioneDao.subCriteriaCoordinatore(utente.getId(), criteriaAg, "id");
		}
		
		if(ApplicationUtil.gtZero(cont.getCampagnaId())) {
			criteriaCA.add(Restrictions.idEq(cont.getCampagnaId()));
		} else {
			if(ApplicationUtil.isTrue(sd)) {
				impostazioneDao.subCriteriaCoordinatoreCampagna(utente.getId(), criteriaCA, "id");
			}
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaMultipla())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaMultipla(cont, criteriaA);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaAnagraficaSingola())) {
			impostazioneDao.ricercaAggiuntivaAnagraficaSingola(cont, criteriaA);
		}
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoMultipla())) {
			impostazioneDao.ricercaAggiuntivaContrattoMultiplo(cont, criteriaCo);
		} 
		if(ApplicationUtil.isTrue(impostazioneDao.getRicercaDomandaContrattoSingola())) {
			impostazioneDao.ricercaAggiuntivaContrattoSingola(cont, criteriaCo);
		}
		
		if (ApplicationUtil.isNotBlank(cont.getRicerca_anagrafica())) {
			ApplicationUtil.anagrafica(cont.getRicerca_anagraficaV(), criteriaA,cont.getRicerca_anagrafica());
		}
		if (ApplicationUtil.isNotBlank(cont.getUtente())) {
			DetachedCriteria cOp = criteria.createCriteria("utente");
			ApplicationUtil.operatore(cOp, cont.getUtente());
		}
		ApplicationUtil.proprieta(false, filtro, criteria, "esito.id", cont.getEsito(), cont.getEsitos());
		cont.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, cont);
	}
	@SuppressWarnings("unchecked")
	public List<StoriaContratto> ricercaStoriaContrattoCoordinatore(Utente utente, StoriaContrattoCriteria cont, int start, int size) {
		ExtendedCriteria criteria = ricercaStoriaContrattoCoordinatoreCriteria(utente, cont);
		criteria.getCriteria().setFetchMode("contratto", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), cont.getO_tipo(), cont.getO_colonna(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int ricercaStoriaContrattoCoordinatoreSize(Utente utente, StoriaContrattoCriteria cont) {
		ExtendedCriteria criteria = ricercaStoriaContrattoCoordinatoreCriteria(utente, cont);
		return getListSize(criteria);
	}
	public List<SelectItem> getEsitiAppuntamentoSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		List<EsitoAppuntamentoSI> lis = getEsitoAppuntamentoSI(criteria);
		return impostazioneDao.getEsitoAppuntamento(lis, false, null);
	}
	public List<SelectItem> getEsitiContattoSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<SelectItem> getEsitiContrattoSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}
	public List<SelectItem> getEsitiAppuntamentoOperatoreSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		criteria.add(Restrictions.eq("visibile", Boolean.TRUE));
		List<EsitoAppuntamentoSI> lis = getEsitoAppuntamentoSI(criteria);
		return impostazioneDao.getEsitoAppuntamento(lis, false, null);
	}
	public List<SelectItem> getEsitiContrattoOperatoreSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("visibile", Boolean.TRUE));
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}
	public List<SelectItem> getEsitiContattoOperatoreSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, false, null);
	}
	public List<StoriaAppuntamento> impostaConteggioAppuntamento(Utente utente,GenericColumn c,List<StoriaAppuntamento> appuntamenti) {
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_STORIA_APPUNTAMENTO);

			if(impostazioneDao.getTabellaDatiAggiuntivi() && c.getAggiuntiviA()) {
				Iterator<StoriaAppuntamento> objIter = appuntamenti.iterator();
				Session session = openSession();
				while(objIter.hasNext()) {
					StoriaAppuntamento appuntamento = objIter.next();
					StoriaAppuntamento a = (StoriaAppuntamento) session.get(StoriaAppuntamento.class, appuntamento.getId());
					Hibernate.initialize(a.getAppuntamento().getContatto().getAnagrafica().getDettagli());
					Iterator<DatoAggiuntivoAnagrafica> objd = a.getAppuntamento().getContatto().getAnagrafica().getDettagli().iterator();
					while(objd.hasNext()) {
						DatoAggiuntivoAnagrafica da = objd.next();
						appuntamento.getAppuntamento().getContatto().getAnagrafica().addDatoM(da.getCampo_aggiuntivo().getId(), da);
					}
				}
				closeSession(session);
			}
			l.log(c);
		}
		return appuntamenti;
	}
	public List<StoriaContratto> impostaConteggioContratto(Utente utente, GenericColumn c,List<StoriaContratto> contratti) {
		if(ApplicationUtil.notEmpty(contratti)) {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTRATTO.stampa() +": " + ApplicationUtil.size(contratti), contratti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_STORIA_CONTRATTO);

			if(impostazioneDao.getTabellaDatiAggiuntivi() && (c.getAggiuntiviA() || c.getAggiuntiviC())) {
				Iterator<StoriaContratto> objIter = contratti.iterator();
				Session session = openSession();
				while(objIter.hasNext()) {
					StoriaContratto contratto = objIter.next();
					StoriaContratto a = (StoriaContratto) session.get(StoriaContratto.class, contratto.getId());
					if(c.getAggiuntiviA()) {
						Hibernate.initialize(a.getContratto().getContatto().getAnagrafica().getDettagli());
						Iterator<DatoAggiuntivoAnagrafica> objdA = a.getContratto().getContatto().getAnagrafica().getDettagli().iterator();
						while(objdA.hasNext()) {
							DatoAggiuntivoAnagrafica da = objdA.next();
							contratto.getContratto().getContatto().getAnagrafica().addDatoM(da.getCampo_aggiuntivo().getId(), da);
						}
					}
					if(c.getAggiuntiviC()) {
						Hibernate.initialize(a.getContratto().getDettagli());
						Iterator<DatoAggiuntivoContratto> objdC = a.getContratto().getDettagli().iterator();
						while(objdC.hasNext()) {
							DatoAggiuntivoContratto da = objdC.next();
							contratto.getContratto().addDatoM(da.getCampo_aggiuntivo().getId(), da);
						}
					}
				}
				closeSession(session);
			}
			l.log(c);
		}
		return contratti;
	}
	public List<StoriaContatto> impostaConteggioContatto(Utente utente, GenericColumn c,List<StoriaContatto> contatti) {

		if(ApplicationUtil.notEmpty(contatti)) {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.STORIA.stampa() + " " + ConstantsLanguage.CONTATTO.stampa() +": " + ApplicationUtil.size(contatti), contatti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4, p ,utente, ConstantsMap.OPERAZIONE_VISUALIZZA_STORIA_CONTATTO);
			if(impostazioneDao.getTabellaDatiAggiuntivi() && c.getAggiuntiviA()) {
				Iterator<StoriaContatto> objIter = contatti.iterator();
				Session session = openSession();
				while(objIter.hasNext()) {
					StoriaContatto contatto = objIter.next();
					StoriaContatto a = (StoriaContatto) session.get(StoriaContatto.class, contatto.getId());
					Hibernate.initialize(a.getContatto().getAnagrafica().getDettagli());
					Iterator<DatoAggiuntivoAnagrafica> objd = a.getContatto().getAnagrafica().getDettagli().iterator();
					while(objd.hasNext()) {
						DatoAggiuntivoAnagrafica da = objd.next();
						contatto.getContatto().getAnagrafica().addDatoM(da.getCampo_aggiuntivo().getId(), da);
					}
				}
				closeSession(session);
			}
			l.log(c);
		}
		return contatti;
	}
	
	
	public List<SelectItem> getCampagneSI(StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		ApplicationUtil.statoLC(stato, criteria, null);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	
	public List<SelectItem> getCampagneSISupervisore(Utente utente, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(utente.getId()));
		ApplicationUtil.statoLC(stato, criteria, null);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	
	public List<SelectItem> getCampagneSICoordinatore(Utente utente, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(utente.getId()));
		ApplicationUtil.statoLC(stato, criteria, null);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	
	public List<SelectItem> getCampagneSIPartner(Utente utente, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("partner").add(Restrictions.idEq(utente.getId()));
		ApplicationUtil.statoLC(stato, criteria, null);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	
	public List<SelectItem> getCampagneSIOperatore(Utente utente, StatoIntervallo stato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(utente.getId()));
		ApplicationUtil.statoLC(stato, criteria, null);
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
}