package it.callite.dao;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;

import javax.faces.model.SelectItem;

import org.hibernate.criterion.DetachedCriteria;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.dao.generic.GenericDao;
import it.callite.model.Tema;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("temaDao")
public class TemaDao extends GenericDao implements ApplicationContextAware {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private LinguaDao linguaDao;
	
	private Tema tema;
	
	public Tema salvaTema(Tema tema) {
		this.tema = tema;
		return (Tema) salvaAggiorna(tema);
	}
	
	public Tema getTema() {
		if(this.tema == null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Tema.class);
			this.tema = (Tema) getUniqueResult(criteria);
		}
		return this.tema;
	}
	
	
	public void ricaricaTema() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Tema.class);
		this.tema = (Tema) getUniqueResult(criteria);
	}
	
	public List<SelectItem> getCountries() {
		List<SelectItem>  countryMap = new ArrayList<SelectItem>();
		for(String l : impostazioneDao.getLingue()) {
			countryMap.add(new SelectItem(l, l));
		}
		return countryMap;
	}
	
	public void setApplicationContext(ApplicationContext o)	{
		initializeLingua();
		
		Tema tm = getTema();
		if(tm == null) {
			tm = new Tema();
			tm.setScadenza(ApplicationUtil.addDays(ApplicationUtil.date(), ConstantsGlobal.scadenza));
			tm.setPreavviso(ConstantsGlobal.preavviso);
		}  
		if(ApplicationUtil.isEmpty(tm.getSkin())) {
			tm.setSkin(ConstantsMap.tema_A);
		}
		if(!tm.isLogo()) {
			tm.setContent(null);
		}
		if(ApplicationUtil.isEmpty(tm.getLingua())) {
			tm.setLingua(ConstantsGlobal.LOCALE);
		}
		if(tm.getZip() == null) {
			tm.setZip(ConstantsGlobal.GZIP);
		}
		if(tm.getFilesize() == null) {
			tm.setFilesize(ConstantsGlobal.SIZE);
		}
		if(tm.getColor() == null) {
			tm.setColor(ConstantsGlobal.COLOR);
		}
		salvaTema(tm);
		caricaTema(tm);
	}
	
	public void caricaTema(Tema tm) {
		valueChanged(tm);
		if(ApplicationUtil.isNotEmpty(tm.getLingua())) {
			ConstantsGlobal.LANGUAGE = ApplicationUtil.lingua(tm.getLingua());
			ConstantsGlobal.COUNTRY = ApplicationUtil.paese(tm.getLingua());
			Locale.setDefault(new Locale(tm.getLingua()));
		}
		if(ApplicationUtil.isBlank(ConstantsGlobal.TIMEZONE)) {
			ConstantsGlobal.TIMEZONE = TimeZone.getDefault().getID();
		}
		if(ApplicationUtil.isNotBlank(ConstantsGlobal.TIMEZONE)) {
			TimeZone.setDefault(TimeZone.getTimeZone(ConstantsGlobal.TIMEZONE));
		}
	}
	
	public void valueChanged(Tema tema){
		ConstantsGlobal.GZIP = ApplicationUtil.isTrue(tema.getZip());
		ConstantsGlobal.SIZE = ApplicationUtil.asZero(tema.getFilesize()) * ConstantsGlobal.kb;
		ConstantsGlobal.COLOR = tema.getColor();
	}
	public String getColor() {
		return ConstantsGlobal.COLOR;
	}
	
	public void initializeLingua() {
		String home = impostazioneDao.getHome();
		File fd = new File(home + ConstantsGlobal.path_language);
		File fs = linguaDao.getPathFile();

		File ad = new File(home + ConstantsGlobal.path_error);
		File as = linguaDao.getPathAudio();
		
		ApplicationUtil.copy(fd, fs);
		ApplicationUtil.copy(ad, as);
	}
	
	
	public List<SelectItem> getTemi() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		for(String tema : ConstantsMap.temi) {
			sis.add(new SelectItem(tema, tema));
		}
		return sis;
	}
}