package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.model.SelectItem;

import org.hibernate.FetchMode;
import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Repository;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.Unirest;
import com.mashape.unirest.request.GetRequest;
import com.warrenstrange.googleauth.GoogleAuthenticator;
import com.warrenstrange.googleauth.GoogleAuthenticatorConfig;
import com.warrenstrange.googleauth.GoogleAuthenticatorKey;
import com.warrenstrange.googleauth.GoogleAuthenticatorQRGenerator;

import it.callite.action.DownloadBean;
import it.callite.chiamate.Chiamate;
import it.callite.column.generic.GenericColumn;
import it.callite.constants.ConstantsColumn;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.ConstantsRole;
import it.callite.constants.config.DefaultsGeneric;
import it.callite.constants.config.DefaultsPannello;
import it.callite.criteria.AccessoCriteria;
import it.callite.criteria.MarcaturaCriteria;
import it.callite.criteria.TeamCriteria;
import it.callite.criteria.UserCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.message.Message;
import it.callite.model.Accesso;
import it.callite.model.Agente;
import it.callite.model.Campagna;
import it.callite.model.Marcatura;
import it.callite.model.Ruolo;
import it.callite.model.Team;
import it.callite.model.Utente;
import it.callite.model.Zona;
import it.callite.model.allegato.AllegatoAgente;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ColonnaQuery;
import it.callite.service.LoginAttemptService;
import it.callite.service.SmtpService;
import it.callite.service.config.ConfigurazioneGenericService;
import it.callite.service.config.ConfigurazionePannelloService;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.CodiceQR;
import it.callite.utility.Configurazione;
import it.callite.utility.FileUpload;
import it.callite.utility.LoggerColonna;
import it.callite.utility.MyFile;
import it.callite.utility.Parameter;
import it.callite.utility.ResponseIP;
import it.callite.utility.UtenteVO;
import it.callite.utility.item.AgenteSI;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.RuoloSI;
import it.callite.utility.item.TeamSI;
import it.callite.utility.item.UtenteSI;
import it.callite.utility.item.ZonaSI;
@SuppressWarnings("serial")
@Repository("userDao")
public class UserDao extends GenericDao implements UserDetailsService, InitializingBean, DisposableBean{
	@Autowired
	private ConfigurazioneGenericService configurazioneGenericService;
	@Autowired
	private ConfigurazionePannelloService configurazionePannelloService;
	@Autowired
	private SmtpService smtpService;
	@Autowired
	private DownloadDao downloadDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private AutodialerDao autodialerDao;
	@Autowired
	private DataDao dataDao;
	@Autowired
	private LoginAttemptService loginAttemptService;
	public void scaricaAllegato(Agente agente, DownloadBean downloadBean) {
		List<AllegatoAgente> allegati = getListaAllegati(agente);
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_agente);

				String file_header = ApplicationUtil.reportFile(agente.getData_creazione(), ConstantsLanguage.AGENTE.stampa(), agente.getId(), agente.getDenominazione()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoAgente> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoAgente allegato = (AllegatoAgente) itA.next();
					ApplicationUtil.addFile(all, allegato.getId() + "_" +allegato.getNome(), allegato.getContentFile(s), allegato.getData().getTime());
				}

				ApplicationUtil.zip(null, all, out, p);
				downloadBean.close();

			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void scaricaAllegato(AllegatoAgente allegato, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_agente);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());

			InputStream is = new ByteArrayInputStream(allegato.getContentFile(s));
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();

		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaAllegato(FileUpload fileUpload, Agente agente) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				String s = impostazioneDao.getPathBaseW(ConstantsGlobal.path_agente);
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					AllegatoAgente allegato = new AllegatoAgente();
					allegato.setAgente(agente);
					allegato.setDimensione(ffB.length);
					allegato.setNome(ApplicationUtil.getNameN(ff.getName()));
					allegato.setContentFile(ffB, s);
					salvaAggiorna(allegato);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoAgente> getListaAllegati(Agente agente) {
		if(agente==null)
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoAgente.class);
		criteria.add(Restrictions.eq("agente.id", agente.getId()));
		return getList(criteria);
	}
	public void deleteAllegato(AllegatoAgente allegato)  {
		String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_agente);
		allegato.delete(s);
		cancellaAggiorna(allegato);
	}
	public void deleteAllegati(Agente agente)  {
		List<AllegatoAgente> allegati = getListaAllegati(agente);
		Iterator<AllegatoAgente> itAn = allegati.iterator();
		while (itAn.hasNext()) {
			AllegatoAgente allegato = (AllegatoAgente) itAn.next();
			deleteAllegato(allegato);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByUserId(Long user){
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(user));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneBySuperOperatoreId(Long user){
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(user));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByAgenteId(Long user){
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("zone").createCriteria("agente").createCriteria("user").add(Restrictions.idEq(user));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByCoordinatoreId(Long user){
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(user));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Zona> getZoneBySupervisoreId(Long user){
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.createCriteria("supervisori").add(Restrictions.idEq(user));
		ApplicationUtil.orderZ(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiBySupervisoreId(Long user){
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(user));
		return getList(criteria);
	}
	public List<SelectItem> getAgentiByUtenteId(Long user){
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("user").add(Restrictions.idEq(user));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}
	public List<AgenteSI> getAgentiSIByUtenteId(Long user){
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("user").add(Restrictions.idEq(user));
		return getAgenteSI(criteria);
	}

	public void email(Utente user, String password1){
		if(ApplicationUtil.isNotEmpty(user.getEmail())) {
			if(ApplicationUtil.isNotEmpty(password1)) {
				Boolean cp = Boolean.valueOf(configurazioneGenericService.getValore(DefaultsGeneric.KEY_PASSWORD_EMAIL_CAMBIO));
				if(ApplicationUtil.isTrue(cp)) {
					user.setCambioPassword(Boolean.TRUE);
				}
				String testo =  downloadDao.getRecupero(user, password1);
				String oggetto = ConstantsLanguage.INVIO_ACCOUNT.stampa() + ": " + user.getDenominazione();
				String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
				smtpService.invia(percorsoReply, user.getEmail(), null, oggetto, testo, null, null);
			}
		}
	}
	public void mailSender(String numero, String destinazione, String e){
		String testo =  downloadDao.getRichiamo(numero, destinazione);
		String oggetto = ConstantsLanguage.RICHIESTA_CONTATTO.stampa() + ": " + numero;
		String email = ApplicationUtil.trimToNull(e);
		if(email == null) {
			email = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
		}
		if(email != null) {
			smtpService.invia(null, email, null, oggetto, testo, null, null);
		}
	}
	
	public void mailSender(String from, String to, String o, String tH, String tT, String e){
		String testo =  downloadDao.getRicezione(from, to, o, tH, tT);
		String oggetto = ConstantsLanguage.RICHIESTA_CONTATTO.stampa() + ": " + from;
		String email = ApplicationUtil.trimToNull(e);
		if(email == null) {
			email = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
		}
		if(email != null) {
			smtpService.invia(null, email, null, oggetto, testo, null, null);
		}
	}
	public void mailSender(String nome, String body, String param, String e){
		String testo =  downloadDao.getWebhook(nome, body, param);
		String oggetto = ConstantsLanguage.RICHIESTA_CONTATTO.stampa() + ": " + nome;
		String email = ApplicationUtil.trimToNull(e);
		if(email == null) {
			email = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
		}
		if(email != null) {
			smtpService.invia(null, email, null, oggetto, testo, null, null);
		}
	}
	
	
	public String linkSender(String link){
		return ApplicationUtil.downloadLink(link);
	}
	
	public void emailDaScheda(Utente user){
		if(ApplicationUtil.isNotEmpty(user.getEmail())) {
			String passwd_nn_codificata = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_password);
			user.setData_password(ApplicationUtil.date());
			user.setPassword(ApplicationUtil.encoder(passwd_nn_codificata));
			Boolean cp = Boolean.valueOf(configurazioneGenericService.getValore(DefaultsGeneric.KEY_PASSWORD_EMAIL_CAMBIO));
			if(ApplicationUtil.isTrue(cp)) {
				user.setCambioPassword(Boolean.TRUE);
			}
			salvaUser(user);
			String testo = downloadDao.getRecupero(user, passwd_nn_codificata);
			String oggetto = ConstantsLanguage.INVIO_ACCOUNT.stampa() + ": " + user.getDenominazione();
			String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
			smtpService.invia(percorsoReply, user.getEmail(), null, oggetto, testo, null, null);
		}
	}
	public void sendNotificaLoginA(Utente user,  String ip){
		String email = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_LOGIN));
		if(email != null) {
			sendNotificaLogin(email, user, ip);
		}
	}
	public void sendNotificaLoginU(Utente user,  String ip){
		String email = ApplicationUtil.trimToNull(user.getEmail());
		if(email != null) {
			sendNotificaLogin(email, user, ip);
		}
	}
	
	public void sendNotificaLogin(String email, Utente user,  String ip){
		Date data = ApplicationUtil.date();
		String testo = downloadDao.notificaLoginTemplate(user, ip, data);
		String oggetto = ApplicationUtil.report(ConstantsLanguage.NOTIFICA_LOGIN.stampa() + ": " +  user.getDenominazione() , ConstantsGlobal.no_periodo,  data);
		smtpService.invia(null, email, null, oggetto, testo, null, null);
	}
	
	public List<SelectItem> getAllRuoliSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ruolo.class);
		List<RuoloSI> lis = getRuoloSI(criteria);
		return impostazioneDao.getRuolo(lis, true, true);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaUser(UserCriteria criteUser, int start, int size)  {
		ExtendedCriteria criteria = trovaUserCriteria(criteUser);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), criteUser.getO_tipo(), criteUser.getO_colonna(),  new Order [] {Order.asc("id")});
		List<Utente> users = getList(criteria, start, size);
		return userRuoloA(users);
	}
	
	public List<Utente> userRuoloA(List<Utente> users) {
		if(ApplicationUtil.notEmpty(users))  {
			Utente logged = ApplicationUtil.getUsernameLoggato();
			boolean s = logged.isShell();
			List<String> su = logged.getSuperusers();
			Iterator<Utente> it = users.iterator();
			while (it.hasNext()) {
				Utente user = it.next();
				user.impostaRuoloA(s, su);
			}
		}
		return users;
	}
	public int trovaUserSize(UserCriteria criteUser) {
		ExtendedCriteria criteria = trovaUserCriteria(criteUser);
		return getListSize(criteria);
	}
	private ExtendedCriteria trovaUserCriteria(UserCriteria user) {
		if(user != null)  user = (UserCriteria) user.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		ApplicationUtil.utente(criteria, user);
		user.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, user);
	}
	public Utente getUserByEmail(String email) {
		if(ApplicationUtil.isNotBlank(email)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
			criteria.add(Restrictions.eq("email", email));
			return (Utente) getUniqueResult(criteria);
		}
		return null;
	}
	private ExtendedCriteria trovaUtentiSenzaAgenteCriteria(Long agenteId, UserCriteria user) {
		if(user != null)  user = (UserCriteria) user.loadCriteria();
		DetachedCriteria criteriaA = DetachedCriteria.forClass(Utente.class);
		criteriaA.createCriteria("agenti").add(Restrictions.idEq(agenteId));
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		ApplicationUtil.utente(criteria, user);
		user.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, user);
	}
	public Utente getUserByUsername(String username) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.eq("username", username));
		return (Utente) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiByAgenteId(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("agenti").add(Restrictions.idEq(agenteId));
		return getList(criteria);
	}
	public Boolean estratto() {
		return Boolean.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_ESTRATTO));
	}
	private String[] split(String username) {
		if(username != null && username.contains(ConstantsGlobal.pipeSeparator) && impostazioneDao.getModalitaSitter()) {
			String [] us = username.split(ConstantsGlobal.pipeSeparator);
			if(us != null && us.length == 2) {
				List<String> sitterList = getSitterListSlave(us[0]);
				if(ApplicationUtil.notEmpty(sitterList) && sitterList.contains(us[1])) {
					return us;
				} 
			}
		}
		return new String [] {username , username};
	}

	@SuppressWarnings("unchecked")
	public List<Ruolo> getRuoli() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ruolo.class);
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	public void bloccaReload(Utente utente){
		operazioneDao.bloccaReload(utente);
	}

	public void ricaricaCode(Utente utente){
		Chiamate.getCodaOperatori().replace(utente);
	}
	public Utente reloadUser(Utente utente){
		boolean sitter = utente.getRuoloSitter();
		Utente load = getFullUserById(utente.getId());
		if(load != null) {
			load.setPhone(utente.getPhone());
			impostaRuoli(load, sitter, true);
			if(ApplicationUtil.eq(impostazioneDao.getLoginMultiplo(), 4)) {
				load.setSessionId(utente.getSessionId());
			}
			SecurityContext sc = ApplicationUtil.getSecurityContext();
			if(sc != null) {
				Authentication au = sc.getAuthentication();
				if (au!=null) {
					Object principal =  au.getPrincipal();
					if (principal instanceof Utente) {
						sc.setAuthentication( new UsernamePasswordAuthenticationToken(load, au.getCredentials(), load.getAuthorities())); 
					}
				}
			}
			return load;
		} else {
			return utente;
		}
	}
	public Utente loadUserByUsername(String username) {
		String []  us = split(username); 
		String master = us[0];
		String sitter = us[1];
		Utente u = getFullUserByUsername(master);
		Utente s = null;
		if(!sitter.equals(master)) {
			s = getFullUserByUsername(sitter);
		} 
		Utente load = s != null ? s : u;
		if(load != null) {
			changePassword(u);
			impostaRuoli(load, s != null, false);
			if(u != null) {
				load.setPassword(u.getPassword());
			}
			return load;
		}
		throw new UsernameNotFoundException(Message.addCatalogText("loginError"));	
		
	}
	
	public void changePassword(Utente utente) {
		if(utente != null) {
			String password = ApplicationUtil.parameter("c_password");
			if(ApplicationUtil.isNotBlank(password) && ApplicationUtil.equals(ApplicationUtil.hash(password), utente.getPassword())) {
				String pwdStore = ApplicationUtil.encoder(password);
				utente.setPassword(pwdStore);
				String hql = "update Utente set password = :password where id = :id";
				execute(false, hql, new Parameter("password", pwdStore), new Parameter("id", utente.getId()));
			}
		}
	}

	public void impostaRuoli(Utente load, boolean sb, boolean ricarica){
		String ipAddress = ApplicationUtil.trimToNull(dataDao.getIp());
		if(!ricarica) {
			if(ipAddress != null) {
				boolean blocca1 = dataDao.ipAddress(impostazioneDao.getIndirizzoLogin1(), impostazioneDao.getIndirizzoLogin2(), ipAddress);
				boolean blocca2 = dataDao.ipAddress(load.getIp_consentiti(), load.getIp_bloccati(), ipAddress);
				boolean blocca3 = loginAttemptService.isBlocked(ipAddress, load.getUsername());
				boolean blocca4 = dataDao.captcha(ipAddress);
				boolean blocca5 = verificaOTP(load);
				if(blocca1 || blocca2 || blocca3 || blocca4 || blocca5) {
					load.setBlocca(true);
				} else {
					load.setBlocca(false);
				}
			} else {
				load.setBlocca(true);
			}
		} else {
			load.setBlocca(false);
		}
		boolean ruoloSupervisore = isSupervisore(load);
		boolean ruoloCoordinatore = isCoordinatore(load);
		boolean ruoloOperatore = isOperatore(load);
		boolean ruoloAgente = isAgente(load);
		boolean ruoloRete = isRete(load);
		boolean ruoloPartner = isPartner(load);
		boolean estratto = estratto();
		load.setSuperusers(impostazioneDao.getSuperusers());
		load.setRuoloSitter(sb);
		load.setRuoloAgente(ruoloAgente);
		load.setRuoloRete(ruoloRete);
		load.setRuoloPartner(ruoloPartner);
		load.setRuoloOperatore(ruoloOperatore);
		load.setRuoloSupervisore(ruoloSupervisore);
		load.setRuoloCoordinatore(ruoloCoordinatore);
		load.setRuoloPannello(ruoloSupervisore || ruoloOperatore);
		load.setRuoloEstratto(estratto);
		load.setBrowser(ApplicationUtil.browser());
		load.setMassimoUtenti(impostazioneDao.getMassimoUtenti());
		load.setLoginMultiplo(impostazioneDao.getLoginMultiplo());
		load.setInternoMultiplo(impostazioneDao.getInternoMultiplo());
		load.setTempoPassword(impostazioneDao.getScadenzaPassword());
		load.setForceReload(impostazioneDao.getForceReload());
		load.setAttivoControl(impostazioneDao.getAttivoControl());
		dataDao.phone(load, ricarica);
	}


	public Ruolo getRuoloById(Long ruoloId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ruolo.class);
		criteria.add(Restrictions.idEq(ruoloId));
		return (Ruolo) getUniqueResult(criteria);
	}
	public Ruolo getRuoloByNome(String nome) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ruolo.class);
		criteria.add(Restrictions.eq("nome", nome));
		return (Ruolo) getUniqueResult(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<Ruolo> getRuoliByNome(List<String> nomi) {
		if(ApplicationUtil.notEmpty(nomi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Ruolo.class);
			criteria.add(Restrictions.in("nome", nomi));
			return getList(criteria);
		}
		return null;
	}
	public Utente getFullUserById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("ruoli", FetchMode.JOIN);
		return (Utente) getUniqueResult(criteria);
	}
	public Utente getFullUserByUsername(String username) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.eq("username", username));
		criteria.setFetchMode("ruoli", FetchMode.JOIN);
		return (Utente) getUniqueResult(criteria);
	}
	public void elimina(Utente utente) {
		cancellaAggiorna(utente);
	}
	public List<SelectItem> trovaUtentiStorici(Utente utente, Boolean abilitato) {
		if(ApplicationUtil.isTrue(abilitato)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
			if(utente != null) {
				criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(utente.getId()));
			}
			List<UtenteSI> lis = getUtenteSI(criteria);
			return impostazioneDao.getUtente(lis, false, null);
		}
		return new ArrayList<SelectItem>();
	}
	public List<SelectItem> trovaUtentiByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(ApplicationUtil.gtZero(campagnaId)) { 
			criteria.createCriteria("team").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		}
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> getUtenti() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> trovaAgentiByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ =  criteria.createCriteria("zone");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		} else {
			criteriaZ.add(Restrictions.isNotNull("id"));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}
	public List<SelectItem> trovaUtentiByCampagnaId(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria critT = criteria.createCriteria("team");
		if(ApplicationUtil.gtZero(campagnaId)) {
			critT.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		} 
		critT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> trovaUtentiByCampagnaIdPartner(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria critC = criteria.createCriteria("team").createCriteria("campagne");
		if(ApplicationUtil.gtZero(campagnaId)) {
			critC.add(Restrictions.idEq(campagnaId));
		} 
		critC.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> trovaAgentiByCampagnaIdPartner(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone").createCriteria("campagne");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteriaZ.add(Restrictions.idEq(campagnaId));
		} else {
			criteriaZ.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}

	public List<SelectItem> trovaAgentiByCampagnaId(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone").createCriteria("campagne");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteriaZ.add(Restrictions.idEq(campagnaId));
		} else {
			criteriaZ.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}
	public List<SelectItem> trovaAgentiPartnerByCampagnaId(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone").createCriteria("campagne");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteriaZ.add(Restrictions.idEq(campagnaId));
		} else {
			criteriaZ.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}
	public List<SelectItem> trovaAgentiCoordinatoreByCampagnaId(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		}
		criteriaZ.createCriteria("supervisori").add(Restrictions.idEq(user.getId()));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}
	public List<SelectItem> trovaAgentiOperatoreByCampagnaId(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		DetachedCriteria criteriaC = criteriaZ.createCriteria("campagne");
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteriaC.add(Restrictions.idEq(campagnaId));
		}
		criteriaC.createCriteria("team").createCriteria("user").add(Restrictions.idEq(user.getId()));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}
	public List<SelectItem> trovaAgentiAgenteByCampagnaId(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		} 
		criteria.createCriteria("user").add(Restrictions.idEq(user.getId()));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}
	public List<SelectItem> trovaAgentiReteByCampagnaId(Utente user, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		} 
		impostazioneDao.subCriteriaAgentiSottoReteByUser(user.getId(),0,  getMaxLivelloRete(), criteria, "id");
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}
	public List<SelectItem> trovaUtentiOperatori() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("team").add(Restrictions.isNotNull("id"));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, true, null);
	}
	public List<SelectItem> trovaUtentiOperatori(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, true, null);
	}
	public List<SelectItem> trovaUtentiOperatoriSI(Utente user, Long team) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(user != null || ApplicationUtil.gtZero(team)) {
			DetachedCriteria criteriaT = criteria.createCriteria("team");
			if(user != null) {
				criteriaT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
			}
			if(ApplicationUtil.gtZero(team)) {
				criteriaT.add(Restrictions.idEq(team));
			} 
		}
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, true, null);
	}
	public List<SelectItem> getUtentiSI(Long campagna) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("team").createCriteria("campagne").add(Restrictions.idEq(campagna));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> getUtentiPrivatoCampagnaSI(Long campagna) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("team");
		if(ApplicationUtil.gtZero(campagna)) {
			criteriaT.createCriteria("campagne").add(Restrictions.idEq(campagna));
		} else {
			criteriaT.add(Restrictions.isNotNull("id"));
		}
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> getUtentiPrivatoCampagnaSI() {
		return getUtentiPrivatoCampagnaSI(null);
	}
	public List<SelectItem> getUtentiSI(Long campagna, Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria critT = criteria.createCriteria("team");
		critT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		critT.createCriteria("campagne").add(Restrictions.idEq(campagna));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> getUtentiPartnerPrivatoSI(Long campagna, Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria critT = criteria.createCriteria("team");
		DetachedCriteria critC = critT.createCriteria("campagne");
		if(ApplicationUtil.gtZero(campagna)) {
			critC.add(Restrictions.idEq(campagna));
		} 
		critC.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> getUtentiPrivatoSI(Boolean supervisore, Long campagna, Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria critT = criteria.createCriteria("team");
		if(ApplicationUtil.isTrue(supervisore)) {
			critT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
			if(ApplicationUtil.gtZero(campagna)) {
				critT.createCriteria("campagne").add(Restrictions.idEq(campagna));
			} 
			List<UtenteSI> lis = getUtenteSI(criteria);
			return impostazioneDao.getUtente(lis, false, null);
		} else {
			critT.createCriteria("user").add(Restrictions.idEq(user.getId()));
			if(ApplicationUtil.gtZero(campagna)) {
				critT.createCriteria("campagne").add(Restrictions.idEq(campagna));
			} 
			List<UtenteSI> lis = getUtenteSI(criteria);
			return impostazioneDao.getUtente(lis, false, user);
		}
	}
	public List<SelectItem> getUtentiPrivatoSIPartner(Long campagna, Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria critC = criteria.createCriteria("team").createCriteria("campagne");
		critC.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		if(ApplicationUtil.gtZero(campagna)) {
			critC.add(Restrictions.idEq(campagna));
		} 
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public Ruolo salvaRuolo(Ruolo ruolo) {
		if(ruolo != null) {
			ruolo = (Ruolo) salvaAggiorna(ruolo);
		}
		return ruolo;
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiByTeam(Long teamId) {
		if(teamId != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
			criteria.createCriteria("team").add(Restrictions.idEq(teamId));
			ApplicationUtil.orderU(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Utente>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByTeam(Long teamId) {
		if(teamId != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("team").add(Restrictions.idEq(teamId));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getSupervisoriByTeam(Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("teams").add(Restrictions.idEq(teamId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiNonInTeam(Long team, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti,1)) {
				criteria.createCriteria("team", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Team.class);
				criteriaU.add(Restrictions.idEq(team));
				DetachedCriteria criteriaA = criteriaU.createCriteria("user");
				criteriaA.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
				criteria.createCriteria("team", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Team.class);
			criteriaU.add(Restrictions.idEq(team));
			DetachedCriteria criteriaA = criteriaU.createCriteria("user");
			criteriaA.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		}
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiByInterno(Long interno) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("interno").add(Restrictions.idEq(interno));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiByTrunk(Long trunk) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("trunk").add(Restrictions.idEq(trunk));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public boolean isSupervisore(Utente utente) {
		if(utente == null || utente.getId() == null)
			return false;
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("supervisore").add(Restrictions.idEq(utente.getId()));
		int size = getListSize(criteria);
		return (size > 0);
	}
	public boolean isCoordinatore(Utente utente) {
		if(utente == null || utente.getId() == null)
			return false;
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.createCriteria("supervisori").add(Restrictions.idEq(utente.getId()));
		int size = getListSize(criteria);
		return (size > 0);
	}
	public boolean isOperatore(Utente utente) {
		if(utente == null || utente.getId() == null)
			return false;
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		int size = getListSize(criteria);
		return (size > 0);
	}
	public boolean isAgente(Utente utente) {
		if(utente == null || utente.getId() == null)
			return false;
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		int size = getListSize(criteria);
		return (size > 0);
	}
	public boolean isRete(Utente utente) {
		if(utente == null || utente.getId() == null)
			return false;
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
		criteria.createCriteria("agente_figli");
		int size = getListSize(criteria);
		return (size > 0);
	}
	public boolean isPartner(Utente utente) {
		if(utente == null || utente.getId() == null)
			return false;
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("partner").add(Restrictions.idEq(utente.getId()));
		int size = getListSize(criteria);
		return (size > 0);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNonInTeam(Long team) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Team.class);
		criteriaU.add(Restrictions.idEq(team));
		DetachedCriteria criteriaA = criteriaU.createCriteria("campagne");
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Team> getTeamNonInUtente(Long ut) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
		criteriaU.add(Restrictions.idEq(ut));
		DetachedCriteria criteriaA = criteriaU.createCriteria("team");
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Team> getTeamNonInSupervisore(Long ut) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
		criteriaU.add(Restrictions.idEq(ut));
		DetachedCriteria criteriaA = criteriaU.createCriteria("teams");
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Team> getTeamBySupervisore(Long ut) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("supervisore").add(Restrictions.idEq(ut));
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Team> getTeamByUtente(Long ut) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("user").add(Restrictions.idEq(ut));
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Ruolo> trovaRuoloPerUtente(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Ruolo.class);
		criteria.createCriteria("user").add(Restrictions.idEq(id));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	public Utente nuovoUtente() {
		return new Utente();
	}
	
	public List<SelectItem> getTeamSIAll() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, true, true, null);
	}

	public List<SelectItem> getPartnerSIAll() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("partner").add(Restrictions.isNotNull("id"));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, true, false, null);
	}

	public List<SelectItem> getZonaSIAll() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, true, true, null);
	}
	public List<SelectItem> getAgenteSIAll() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("user").add(Restrictions.isNotNull("id"));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, true, null);
	}
	public List<SelectItem> getTeam() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, false, false, null);
	}
	public Utente salva(Utente utente, String password1) {
		if(utente != null) {
			if(ApplicationUtil.isNotEmpty(password1)) {
				String password = ApplicationUtil.encoder(password1);
				utente.setData_password(ApplicationUtil.date());
				utente.setPassword(password);
			} 
			if(utente.getInterno() == null) {
				utente.esterno();
			}
			if(ApplicationUtil.isFalse(utente.getGeneraEstratto())) {
				utente.estratto();
			} else {
				if(utente.getFattura() == null) {
					utente.fattura();
				}
			}
			utente = salvaUser(utente);
		}
		return utente;
	}
	public void salvaHome(Utente utente) {
		if(utente != null) {
			utente = salvaUser(utente);
		}
	}
	
	public void salvaPasswordHome(Long user, String password1) {
		Utente utente = getUser(user);
		if(utente != null && ApplicationUtil.isNotEmpty(password1)) {
			String password = ApplicationUtil.encoder(password1);
			utente.setData_password(ApplicationUtil.date());
			utente.setPassword(password);
			utente.setCambioPassword(Boolean.FALSE);
			salvaUser(utente);
		}
	}

	public void salvaPassword(Utente utente, String password1) {
		if(utente != null) {
			if(ApplicationUtil.isNotEmpty(password1)) {
				String password = ApplicationUtil.encoder(password1);
				utente.setData_password(ApplicationUtil.date());
				utente.setPassword(password);
			}
			utente = salvaUser(utente);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaUtentiSenzaAgente(Long agenteId, UserCriteria criteUser, int start, int size) {
		ExtendedCriteria criteria = trovaUtentiSenzaAgenteCriteria(agenteId, criteUser);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), criteUser.getO_tipo(), criteUser.getO_colonna(),  new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaUtentiSenzaAgenteSize(Long agenteId, UserCriteria user) {
		ExtendedCriteria criteria = trovaUtentiSenzaAgenteCriteria(agenteId, user);
		return getListSize(criteria);
	}
	public void eliminaTeam(Team team) {
		elimina(team);
	}
	public void aggiungiUtenteATeam(Long teamId, Long utenteId) {
		Utente utente = getFullUserConTeamById(utenteId);
		Team team = get(teamId);
		if(utente != null && team != null) {
			if(utente.getTeam() == null)
				utente.setTeam(new ArrayList<Team>());
			if(!utente.getTeam().contains(team)) {
				utente.getTeam().add(team);
			}
			salvaUser(utente);
		}
	}
	public void aggiungiUtenteATeamTutti(Long utenteId) {
		Utente utente = getFullUserConTeamById(utenteId);
		List<Team> teams = getTeamNonInUtente(utenteId);
		if(utente != null && ApplicationUtil.notEmpty(teams)) {
			if(utente.getTeam() == null)
				utente.setTeam(new ArrayList<Team>());
			utente.getTeam().addAll(teams);
			salvaUser(utente);
		}
	}
	public void eliminaUtenteDaTeamTutti(Long utenteId) {
		Utente utente = getFullUserConTeamById(utenteId);
		List<Team> teams = getTeamByUtente(utenteId);
		if(utente != null && ApplicationUtil.notEmpty(teams)) {
			if(utente.getTeam() == null)
				utente.setTeam(new ArrayList<Team>());
			utente.getTeam().removeAll(teams);
			salvaUser(utente);
		}
	}
	public void aggiungiCampagnaATeam(Long teamId, Long campagnaId) {
		Campagna campagna = getFullCampagnaConTeamById(campagnaId);
		Team team = get(teamId);
		if(campagna != null && team != null) {
			if(campagna.getTeam() == null)
				campagna.setTeam(new ArrayList<Team>());
			if(!campagna.getTeam().contains(team)) {
				campagna.getTeam().add(team);
			}
			salvaAggiorna(campagna);
		}
	}
	public void aggiungiSupervisoreATeam(Long teamId, Long utenteId) {
		Utente utente = getFullUserConTeamsById(utenteId);
		Team team = get(teamId);
		if(utente != null && team != null) {
			if(utente.getTeams() == null)
				utente.setTeams(new ArrayList<Team>());
			if(!utente.getTeams().contains(team)) {
				utente.getTeams().add(team);
			}
			salvaUser(utente);
		}
	}
	public void eliminaSupervisoreDaTeam(Long teamId, Long utenteId) {
		Utente utente = getFullUserConTeamsById(utenteId);
		Team team = get(teamId);
		if(utente != null && team != null) {
			if(utente.getTeams() == null)
				utente.setTeams(new ArrayList<Team>());
			if(utente.getTeams().contains(team)) {
				utente.getTeams().remove(team);
			}
			salvaUser(utente);
		}
	}
	public void aggiungiSupervisoreTuttiATeam(Long teamId, Integer tutti) {
		Team team = getFullTeamSupervisore(teamId);
		List<Utente> utenti = getSupervisori(teamId, tutti);
		if(team != null && utenti != null) {
			if(team.getSupervisore() == null)
				team.setSupervisore(new ArrayList<Utente>());
			team.getSupervisore().addAll(utenti);
			salvaTeam(team);
		}
	}
	public void eliminaSupervisoreTuttiDaTeam(Long teamId) {
		Team team = getFullTeamSupervisore(teamId);
		List<Utente> utenti = getSupervisoriByTeam(teamId);
		if(team != null && utenti != null) {
			if(team.getSupervisore() == null)
				team.setSupervisore(new ArrayList<Utente>());
			team.getSupervisore().removeAll(utenti);
			salvaTeam(team);
		}
	}
	public void aggiungiCampagnaTuttiATeam(Long teamId) {
		Team team = getFullTeamCampagna(teamId);
		List<Campagna> campagne = getCampagneNonInTeam(teamId);
		if(team != null && campagne != null) {
			if(team.getCampagne() == null)
				team.setCampagne(new ArrayList<Campagna>());
			team.getCampagne().addAll(campagne);
			salvaTeam(team);
		}
	}
	public void eliminaCampagnaTuttiDaTeam(Long teamId) {
		Team team = getFullTeamCampagna(teamId);
		List<Campagna> campagne = getCampagneByTeam(teamId);
		if(team != null && campagne != null) {
			if(team.getCampagne() == null)
				team.setCampagne(new ArrayList<Campagna>());
			team.getCampagne().removeAll(campagne);
			salvaTeam(team);
		}
	}
	public void aggiungiUtenteTuttiATeam(Long teamId, Integer tutti) {
		Team team = getFullTeamUser(teamId);
		List<Utente> utenti = getUtentiNonInTeam(teamId, tutti);
		if(team != null && utenti != null) {
			if(team.getUser() == null)
				team.setUser(new ArrayList<Utente>());
			team.getUser().addAll(utenti);
			salvaTeam(team);
		}
	}
	public void eliminaUtenteTuttiDaTeam(Long teamId) {
		Team team = getFullTeamUser(teamId);
		List<Utente> utenti = getUtentiByTeam(teamId);
		if(team != null && utenti != null) {
			if(team.getUser() == null)
				team.setUser(new ArrayList<Utente>());
			team.getUser().removeAll(utenti);
			salvaTeam(team);
		}
	}
	public void aggiungiSupervisoreATeamTutti(Long utenteId) {
		Utente utente = getFullUserConTeamsById(utenteId);
		List<Team> teams = getTeamNonInSupervisore(utenteId);
		if(utente != null && ApplicationUtil.notEmpty(teams)) {
			if(utente.getTeams() == null)
				utente.setTeams(new ArrayList<Team>());
			utente.getTeams().addAll(teams);
			salvaUser(utente);
		}
	}
	public void eliminaSupervisoreDaTeamTutti( Long utenteId) {
		Utente utente = getFullUserConTeamsById(utenteId);
		List<Team> teams = getTeamBySupervisore(utenteId);
		if(utente != null && ApplicationUtil.notEmpty(teams)) {
			if(utente.getTeams() == null)
				utente.setTeams(new ArrayList<Team>());
			utente.getTeams().removeAll(teams);
			salvaUser(utente);
		}
	}
	public void eliminaUtenteDaTeam(Long teamId, Long utenteId) {
		Utente utente = getFullUserConTeamById(utenteId);
		Team team = get(teamId);
		if(utente != null && team != null) {
			if(utente.getTeam() == null)
				utente.setTeam(new ArrayList<Team>());
			if(utente.getTeam().contains(team)) {
				utente.getTeam().remove(team);
			}
			salvaUser(utente);
		}
	}
	public void eliminaCampagnaDaTeam(Long teamId, Long campagnaId) {
		Campagna campagna = getFullCampagnaConTeamById(campagnaId);
		Team team = getTeam(teamId);
		if(campagna != null && team != null) {
			if(campagna.getTeam() == null)
				campagna.setTeam(new ArrayList<Team>());
			if(campagna.getTeam().contains(team)) {
				campagna.getTeam().remove(team);
			}
			salvaAggiorna(campagna);
		}
	}
	public Utente getUser(Long utenteId) {
		return  (Utente) get(Utente.class, utenteId);
	}
	public Team getTeam(Long teamId) {
		return  (Team) get(Team.class, teamId);
	}
	public Utente salvaUser(Utente user) {
		return  (Utente) salvaAggiorna(user);
	}
	
	public void salvaLogin(Utente user, Date data, boolean login) {
		if(user != null) {
			String hql = "update Utente set " + (login ? "data_login" : "data_logout") + " = :data where id = :id";
			execute(false, hql, new Parameter("data", data), new Parameter("id", user.getId()));
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<Agente> getFullAgenteByUserId(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("user").add(Restrictions.idEq(userId));
		return  getList(criteria);
	}
	public List<SelectItem> getFullAgenteByUserIdSI(Long userId, boolean trattini) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("user").add(Restrictions.idEq(userId));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, trattini, false, null);
	}
	public List<SelectItem> getFullAgenteByReteIdSI(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		impostazioneDao.subCriteriaAgentiSottoReteByUser(userId, 0, getMaxLivelloRete(), criteria, "id");
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}
	public List<SelectItem> getFullAgenteByReteId(Long userId, Long campagnaId, Agente agente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		impostazioneDao.subCriteriaAgentiSottoReteByUser(userId,0,  getMaxLivelloRete(), criteria, "id");
		criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, agente);
	}
	public List<SelectItem> getFullAgenteByUserId(Long userId, Long campagnaId, Agente agente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.createCriteria("user").add(Restrictions.idEq(userId));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, agente);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getFullAgenteBySupervisoreId(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(userId));
		return  getList(criteria);
	}
	public Utente getFullUserConTeamsById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("teams", FetchMode.JOIN);
		return (Utente) getUniqueResult(criteria);
	}
	public Team getFullTeamSupervisore(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("supervisore", FetchMode.JOIN);
		return (Team) getUniqueResult(criteria);
	}
	public Team getFullTeamUser(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("user", FetchMode.JOIN);
		return (Team) getUniqueResult(criteria);
	}
	public Team getFullTeamCampagna(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return (Team) getUniqueResult(criteria);
	}
	public Utente getFullUserConTeamById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("team", FetchMode.JOIN);
		return (Utente) getUniqueResult(criteria);
	}
	public Campagna getFullCampagnaConTeamById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("team", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteria);
	}
	public Team salvaTeam(Team team) {
		return (Team) salvaAggiorna(team);
	}
	public Agente salvaAgente(Agente agente) {
		return (Agente) salvaAggiorna(agente);
	}
	public Zona salvaZona(Zona zona) {
		return (Zona) salvaAggiorna(zona);
	}
	public Campagna salvaCampagna(Campagna campagna) {
		return (Campagna) salvaAggiorna(campagna);
	}
	
	public List<Team> impostaConteggioTeam(GenericColumn c, List<Team> teams) {
		if(ApplicationUtil.notEmpty(teams))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdTeam(teams);
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Team.class, new ColonnaQuery(ConstantsColumn.supervisori, "supervisore"), new ColonnaQuery(ConstantsColumn.operatori, "user"), new ColonnaQuery(ConstantsColumn.campagne, "campagne"));
			Iterator<Team> it = teams.iterator();
			while (it.hasNext()) {
				Team o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return teams;
	}
	
	public List<Utente> impostaConteggioUser(GenericColumn c, List<Utente> users) {
		if(ApplicationUtil.notEmpty(users))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdUser(users);
			
			Map<Long, ColonnaModel> map = dataDao.getConteggioColonna(c, ll, Utente.class, new ColonnaQuery(ConstantsColumn.ruoli, "ruoli", "nome"), new ColonnaQuery(ConstantsColumn.supervisori, "teams", "nome"), new ColonnaQuery(ConstantsColumn.operatori, "team", "nome"), new ColonnaQuery(ConstantsColumn.agenti, "agenti", "codice"), new ColonnaQuery(ConstantsColumn.coordinatori, "zone", "nome"), new ColonnaQuery(ConstantsColumn.partner, "campagne", "nome"));
			Iterator<Utente> it = users.iterator();
			while (it.hasNext()) {
				Utente o = it.next();
				ColonnaModel lmap = map.get(o.getId());
				if(lmap != null) {
					o.setModel(lmap);
				}
			}
			l.log(c);
		}
		return users;
	}
		
	public List<SelectItem> getTeamSIPartner(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("campagne").createCriteria("partner").add(Restrictions.idEq(utente.getId()));
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, false, false, null);
	}

	public List<SelectItem> getTeamSI(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, true, false, null);
	}
	public List<SelectItem> getTeamSIRicerca(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, false, false, null);
	}
	public List<SelectItem> getTeamOperatoreSI(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("user").add(Restrictions.idEq(user.getId()));
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, true, false, null);
	}
	
	public List<SelectItem> getTeamOperatoreSIRicerca(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("user").add(Restrictions.idEq(user.getId()));
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, false, false, null);
	}
	public List<SelectItem> getZonaSIRicerca(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.createCriteria("supervisori").add(Restrictions.idEq(user.getId()));
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, false, false, null);
	}
	public List<SelectItem> getTeamSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, true, false, null);
	}
	
	public List<SelectItem> getTeamSIRicerca() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, false, false, null);
	}
	
	@SuppressWarnings("unchecked")
	public List<Team> trovaTeamSupervisionati(Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("supervisore").add(Restrictions.idEq(userid));
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaOperatoriSupervisionati(Long  userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(userid));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> trovaOperatoriMonitor(Long  userid, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(ApplicationUtil.gtZero(teamId)) {
			criteria.createCriteria("team").add(Restrictions.idEq(teamId));
		} else {
			if(ApplicationUtil.gtZero(userid)) {
				criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(userid));
			}
		}
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> trovaTeamMonitor(Long  userid, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(ApplicationUtil.gtZero(teamId)) {
			criteria.createCriteria("team").add(Restrictions.idEq(teamId));
		} else {
			if(ApplicationUtil.gtZero(userid)) {
				criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(userid));
			}
		}
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Team> trovaTeamUtente(Long userid) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("user").add(Restrictions.idEq(userid));
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> getTeamByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> getPartnerByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByTeamId(Long team){
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("team").add(Restrictions.idEq(team));
		return getList(criteriaU);
	}
	@SuppressWarnings("unchecked")
	public List<Team> trovaTeam() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	public ExtendedCriteria trovaTeamCriteria(TeamCriteria tm) {
		if(tm != null)  tm = (TeamCriteria) tm.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		if(ApplicationUtil.isNotBlank(tm.getNome())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, tm.getNome(), criteria, "nome");
		}
		if (ApplicationUtil.gtZero(tm.getCampagna())) {
			criteria.createCriteria("campagne").add(Restrictions.idEq(tm.getCampagna()));
		}
		tm.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, tm);
	}
	@SuppressWarnings("unchecked")
	public List<Team> trovaTeam(TeamCriteria tm, int start, int size) {
		ExtendedCriteria criteria = trovaTeamCriteria(tm);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), tm.getO_tipo(), tm.getO_colonna(), new Order [] {Order.asc("nome")});
		return getList(criteria, start, size);
	}
	public int trovaTeamSize(TeamCriteria tm) {
		ExtendedCriteria criteria = trovaTeamCriteria(tm);
		return getListSize(criteria);
	}
	public Team getTeamCampagne(Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.add(Restrictions.idEq(teamId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return (Team) getUniqueResult(criteria);
	}
	

	public Utente getPartnerCampagne(Long partnerId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(partnerId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		ApplicationUtil.orderU(criteria);
		return (Utente) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Team> trovaTeam(Long camp_id) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Team.class);
		criteriaU.createCriteria("campagne").add(Restrictions.idEq(camp_id));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaPartner(Long camp_id) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
		criteriaU.createCriteria("campagne").add(Restrictions.idEq(camp_id));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public Team get(Long id) {
		return (Team) get(Team.class, id);
	}
	public Agente getAgente(Long id) {
		return (Agente) get(Agente.class, id);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getSupervisori(Long teamId, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti,1)) {
				criteria.createCriteria("teams", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Team.class);
				criteriaU.add(Restrictions.idEq(teamId));
				criteriaU.createCriteria("supervisore").setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("teams", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Team.class);
			criteriaU.add(Restrictions.idEq(teamId));
			criteriaU.createCriteria("supervisore").setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiByTeamId(Team team) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.eq("team", team));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public void elimina(Team team) {
		cancellaAggiorna(team);
	}
	public Agente getFullAgente(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("user", FetchMode.JOIN);
		return  (Agente) getUniqueResult(criteria);
	}
	public void aggiungiAgenteAUtenteTutti(List<Utente> utenti, Long agenteId) {
		Agente agente = getFullAgente(agenteId);
		if (agente != null && ApplicationUtil.notEmpty(utenti)) {
			agente.getUser().addAll(utenti);
			salvaAggiorna(agente);
		}
	}
	public void rimuoviAgenteAUtenteTutti(List<Utente> utenti, Long agenteId) {
		Agente agente = getFullAgente(agenteId);
		if (agente != null && ApplicationUtil.notEmpty(utenti)) {
			agente.getUser().removeAll(utenti);
			salvaAggiorna(agente);
		}
	}
	public Long getAgenteByCodice(String enteredName) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.add(Restrictions.eq("codice", enteredName));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return (Long) getUniqueResult(criteria);
	}
	public List<SelectItem> getAgentiPadreSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		//criteria.createCriteria("agente_figli").add(Restrictions.isNotNull("id"));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}
	public List<AgenteSI> getAgentiFigli(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(agenteId!=null) {
			criteria.add(Restrictions.eq("agente_padre.id", agenteId));
		} else {
			criteria.createCriteria("agente_padre", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
		}
		return  getAgenteSI(criteria);
	}
	public List<Agente> getAllPadri(Agente ag, List<Agente> allPadri) {
		if (allPadri == null) {
			allPadri = new ArrayList<Agente>();
		}
		Session session = openSession();
		Agente agente = (Agente) session.get(Agente.class, ag.getId());
		Hibernate.initialize(agente.getAgente_padre());
		closeSession(session);
		if (agente.getAgente_padre() != null) {
			allPadri.add(agente.getAgente_padre());
			allPadri = getAllPadri(agente.getAgente_padre(), allPadri);
		}
		return allPadri;
	}

	public Agente getAllPadre(Agente ag) {
		Session session = openSession();
		Agente agente = (Agente) session.get(Agente.class, ag.getId());
		Hibernate.initialize(agente.getAgente_padre());
		closeSession(session);
		if (agente.getAgente_padre() != null) {
			return getAllPadre(agente.getAgente_padre());
		} else {
			return agente;
		}
	}
	
	public Marcatura salvaMarcatura(Marcatura marcatura) {
		return (Marcatura) salvaAggiorna(marcatura);
	}
	
	public void updateAccesso(Utente utente, String ip, String browser, Accesso accesso) {
		Integer l = impostazioneDao.loginAdmin();
		if(ApplicationUtil.gtZero(l)) {
			List<String> sus = impostazioneDao.getSuperusers();
			if(sus.contains(utente.getUsername())) {
				int conteggio = contaAccessi(utente, ip);
				if(conteggio == 0) {
					if(ApplicationUtil.eq(l, 1)) {
						sendNotificaLoginA(utente, ip);
					}
					if(ApplicationUtil.eq(l, 2)) {
						sendNotificaLoginU(utente, ip);
					}
				}
			}
		}
		salvaAccesso(accesso);
	}
	public Accesso salvaAccesso(Accesso accesso) {
		return (Accesso) salvaAggiorna(accesso);
	}

	public void eliminaAccesso(Accesso accesso) {
		cancellaAggiorna(accesso);
	}
	public ResponseIP controlla(Accesso accesso)  {
		return control(accesso.getIp());
	}

	public ResponseIP control(String ip)  {
		try {
			String key = impostazioneDao.getKeyGeocodificaIP();
			GetRequest r = Unirest.get(ConstantsGlobal.ipstack + "/"+ip+"?access_key="+key+"&format=1");
			HttpResponse<String> response = r.asString();
			ResponseIP ri = new ResponseIP(response.getBody());
			return (ri != null && ri.getIp() != null) ? ri : null;
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
		return null;
	}
	
	private int contaAccessi(Utente utente, String ip) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Accesso.class);
		criteria.add(Restrictions.eq("utente.id",utente.getId()));
		criteria.add(Restrictions.eq("ip",ip));
		return getListSize(criteria);
	}

	private ExtendedCriteria getCriteria(AccessoCriteria ema) {
		if(ema != null)  ema = (AccessoCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Accesso.class);
		if (ApplicationUtil.isNotBlank(ema.getUtente())) {
			DetachedCriteria c = criteria.createCriteria("utente");
			ApplicationUtil.operatore(c, ema.getUtente());
		}
		if(ApplicationUtil.isNotBlank(ema.getIp())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator, true, ema.getIp(), criteria, "ip");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Marcatura> trova(AccessoCriteria ema, int start, int size) {
		ExtendedCriteria criteria = getCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(AccessoCriteria ema) {
		ExtendedCriteria criteria = getCriteria(ema);
		return getListSize(criteria);
	}
	public void eliminaMarcatura(Marcatura marcatura) {
		cancellaAggiorna(marcatura);
	}
	private ExtendedCriteria getCriteria(Utente user, MarcaturaCriteria ema) {
		if(ema != null)  ema = (MarcaturaCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Marcatura.class);
		//criteria.add(Restrictions.isNotNull("data_fine"));
		DetachedCriteria criteriaU = criteria.createCriteria("user");
		if(ApplicationUtil.gtZero(ema.getOperatore())) {
			criteriaU.add(Restrictions.idEq(ema.getOperatore()));
		}
		if(ApplicationUtil.gtZero(ema.getTeam())) {
			impostazioneDao.subCriteriaOperatoreTeam(ema.getTeam(), criteriaU, "id");
		}
		impostazioneDao.subCriteriaSupervisore(user.getId(), criteriaU, "id");
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Marcatura> trova(Utente utente, MarcaturaCriteria ema, int start, int size) {
		ExtendedCriteria criteria = getCriteria(utente, ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(Utente utente, MarcaturaCriteria ema) {
		ExtendedCriteria criteria = getCriteria(utente, ema);
		return getListSize(criteria);
	}
	private ExtendedCriteria getCriteria(MarcaturaCriteria ema) {
		if(ema != null)  ema = (MarcaturaCriteria) ema.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Marcatura.class);
		//criteria.add(Restrictions.isNotNull("data_fine"));
		DetachedCriteria criteriaU = criteria.createCriteria("user");
		if(ApplicationUtil.gtZero(ema.getOperatore())) {
			criteriaU.add(Restrictions.idEq(ema.getOperatore()));
		}
		if(ApplicationUtil.gtZero(ema.getTeam())) {
			impostazioneDao.subCriteriaOperatoreTeam(ema.getTeam(), criteriaU, "id");
		}
		ema.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ema);
	}
	@SuppressWarnings("unchecked")
	public List<Marcatura> trova(MarcaturaCriteria ema, int start, int size) {
		ExtendedCriteria criteria = getCriteria(ema);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(MarcaturaCriteria ema) {
		ExtendedCriteria criteria = getCriteria(ema);
		return getListSize(criteria);
	}
	public Marcatura getMarcatura(Utente user,  Date oggi, boolean multiplo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Marcatura.class);
		criteria.add(Restrictions.eq("user.id",user.getId()));
		criteria.add(Restrictions.eq("data_creazione", oggi));
		if(multiplo) {
			criteria.add(Restrictions.isNull("data_fine"));
		}
		return (Marcatura) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByPartner(Long partnerId) {
		if(partnerId != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("partner").add(Restrictions.idEq(partnerId));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNonInPartner(Long partner) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
		criteriaU.add(Restrictions.idEq(partner));
		DetachedCriteria criteriaA = criteriaU.createCriteria("campagne");
		criteriaA.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaA, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	public void aggiungiCampagnaAPartner(Long partnerId, Long campagnaId) {
		Campagna campagna = getFullCampagnaConPartnerById(campagnaId);
		Utente partner = getUser(partnerId);
		if(campagna != null && partner != null) {
			if(campagna.getPartner() == null)
				campagna.setPartner(new ArrayList<Utente>());
			if(!campagna.getPartner().contains(partner)) {
				campagna.getPartner().add(partner);
			}
			salvaAggiorna(campagna);
		}
	}
	public void eliminaCampagnaDaPartner(Long partnerId, Long campagnaId) {
		Campagna campagna = getFullCampagnaConPartnerById(campagnaId);
		Utente partner = getUser(partnerId);
		if(campagna != null && partner != null) {
			if(campagna.getPartner() == null)
				campagna.setPartner(new ArrayList<Utente>());
			if(campagna.getPartner().contains(partner)) {
				campagna.getPartner().remove(partner);
			}
			salvaAggiorna(campagna);
		}
	}
	public void aggiungiCampagnaTuttiAPartner(Long partnerId) {
		Utente partner = getFullPartnerCampagna(partnerId);
		List<Campagna> campagne = getCampagneNonInPartner(partnerId);
		if(partner != null && campagne != null) {
			if(partner.getCampagne() == null)
				partner.setCampagne(new ArrayList<Campagna>());
			partner.getCampagne().addAll(campagne);
			salvaUser(partner);
		}
	}
	public void eliminaCampagnaTuttiDaPartner(Long partnerId) {
		Utente partner = getFullPartnerCampagna(partnerId);
		List<Campagna> campagne = getCampagneByPartner(partnerId);
		if(partner != null && campagne != null) {
			if(partner.getCampagne() == null)
				partner.setCampagne(new ArrayList<Campagna>());
			partner.getCampagne().removeAll(campagne);
			salvaUser(partner);
		}
	}
	public Utente getFullPartnerCampagna(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return (Utente) getUniqueResult(criteria);
	}
	public Campagna getFullCampagnaConPartnerById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("partner", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteria);
	}
	public Agente preparaAgente(Integer copia, Agente rif) {
		Agente agente = new Agente();
		if(ApplicationUtil.gtZero(copia) && rif != null) {
			agente = rif.getCopia();
			if(ApplicationUtil.eq(copia,1)) {
				agente.setAgente_padre(rif);
				return agente;
			}
			if(ApplicationUtil.eq(copia,2)) {
				agente.setAgente_padre(rif.getAgente_padre());
				return agente;
			}
		} 
		return agente;
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getReteAgentiByAgenteId(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		impostazioneDao.subCriteriaAgentiSottoRete(agenteId, false, getMaxLivelloRete(), criteria, "id");
		criteria.add(Restrictions.ne("id", agenteId));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getReteAgentiByUserId(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		impostazioneDao.subCriteriaAgentiSottoReteByUser(userId,0,  getMaxLivelloRete(), criteria, "id");
		return getList(criteria);
	}
	public Integer getMaxLivelloRete() {
		return Integer.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_MASSIMO_LIVELLO_RETE));
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getSitterMasterByUser(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("sitterMaster").add(Restrictions.idEq(user));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getSitterMasterNonInUser(Long user) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
		criteriaU.createCriteria("sitterMaster").add(Restrictions.idEq(user));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.not(Restrictions.idEq(user)));
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiCalendar() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.isNotNull("interno.id"));
		criteria.add(Restrictions.eq("promemoria", Boolean.TRUE));
		return getList(criteria);
	}
	
	public void aggiungiSitterMasterAUtente(Long utenteId, Long uMasterId) {
		Utente utente = getFullSitterMasterById(utenteId);
		Utente uMaster = getUser(uMasterId);
		if(utente != null && uMaster != null) {
			if(utente.getSitterSlave() == null)
				utente.setSitterSlave(new ArrayList<Utente>());
			if(!utente.getSitterSlave().contains(uMaster)) {
				utente.getSitterSlave().add(uMaster);
			}
			salvaAggiorna(utente);
		}
	}
	public void eliminaSitterMasterDaUtente(Long utenteId, Long uMasterId) {
		Utente utente = getFullSitterMasterById(utenteId);
		Utente uMaster = getUser(uMasterId);
		if(utente != null && uMaster != null) {
			if(utente.getSitterSlave() == null)
				utente.setSitterSlave(new ArrayList<Utente>());
			if(utente.getSitterSlave().contains(uMaster)) {
				utente.getSitterSlave().remove(uMaster);
			}
			salvaAggiorna(utente);
		}
	}
	public void aggiungiSitterMasterTuttiAUtente(Long utenteId) {
		Utente utente = getFullSitterMasterById(utenteId);
		List<Utente> sitters = getSitterMasterNonInUser(utenteId);
		if(utente != null && sitters != null) {
			if(utente.getSitterSlave() == null)
				utente.setSitterSlave(new ArrayList<Utente>());
			utente.getSitterSlave().addAll(sitters);
			salvaAggiorna(utente);
		}
	}
	public void eliminaSitterMasterTuttiDaUtente(Long utenteId) {
		Utente utente = getFullSitterMasterById(utenteId);
		List<Utente> sitters = getSitterMasterByUser(utenteId);
		if(utente != null && sitters != null) {
			if(utente.getSitterSlave() == null)
				utente.setSitterSlave(new ArrayList<Utente>());
			utente.getSitterSlave().removeAll(sitters);
			salvaAggiorna(utente);
		}
	}
	public Utente getFullSitterMasterById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("sitterSlave", FetchMode.JOIN);
		return (Utente) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<String> getSitterListSlave(String username) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("sitterSlave").add(Restrictions.eq("username", username));
		criteria.setProjection(Projections.distinct(Projections.property("username")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getSitterSlaveByUser(Long user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("sitterSlave").add(Restrictions.idEq(user));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getSitterSlaveNonInUser(Long user) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
		criteriaU.createCriteria("sitterSlave").add(Restrictions.idEq(user));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.not(Restrictions.idEq(user)));
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public void aggiungiSitterSlaveAUtente(Long utenteId, Long uSlaveId) {
		Utente utente = getFullSitterSlaveById(utenteId);
		Utente uSlave = getUser(uSlaveId);
		if(utente != null && uSlave != null) {
			if(utente.getSitterMaster() == null)
				utente.setSitterMaster(new ArrayList<Utente>());
			if(!utente.getSitterMaster().contains(uSlave)) {
				utente.getSitterMaster().add(uSlave);
			}
			salvaAggiorna(utente);
		}
	}
	public void eliminaSitterSlaveDaUtente(Long utenteId, Long uSlaveId) {
		Utente utente = getFullSitterSlaveById(utenteId);
		Utente uSlave = getUser(uSlaveId);
		if(utente != null && uSlave != null) {
			if(utente.getSitterMaster() == null)
				utente.setSitterMaster(new ArrayList<Utente>());
			if(utente.getSitterMaster().contains(uSlave)) {
				utente.getSitterMaster().remove(uSlave);
			}
			salvaAggiorna(utente);
		}
	}
	public void aggiungiSitterSlaveTuttiAUtente(Long utenteId) {
		Utente utente = getFullSitterSlaveById(utenteId);
		List<Utente> sitters = getSitterSlaveNonInUser(utenteId);
		if(utente != null && sitters != null) {
			if(utente.getSitterMaster() == null)
				utente.setSitterMaster(new ArrayList<Utente>());
			utente.getSitterMaster().addAll(sitters);
			salvaAggiorna(utente);
		}
	}
	public void eliminaSitterSlaveTuttiDaUtente(Long utenteId) {
		Utente utente = getFullSitterSlaveById(utenteId);
		List<Utente> sitters = getSitterSlaveByUser(utenteId);
		if(utente != null && sitters != null) {
			if(utente.getSitterMaster() == null)
				utente.setSitterMaster(new ArrayList<Utente>());
			utente.getSitterMaster().removeAll(sitters);
			salvaAggiorna(utente);
		}
	}
	public Utente getFullSitterSlaveById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("sitterMaster", FetchMode.JOIN);
		return (Utente) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public void eliminaRuoli(List<String> nomi) {
		if(ApplicationUtil.notEmpty(nomi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Ruolo.class);
			criteria.add(Restrictions.not(Restrictions.in("nome", nomi)));
			List<Ruolo> c = getList(criteria);
			cancellaLista(c);
		}
	}
	public void destroy() throws Exception {
	}
	public void afterPropertiesSet() throws Exception {
		creaRuoli();
		List<String> sus = impostazioneDao.getSuperusers();
		for(String su : sus) {
			String password = ApplicationUtil.trimToNull(Configurazione.getVariabile(ConstantsGlobal.APPLICATION_SECRET));
			creaAdmin(su, password);
		}
	}
	
	public void creaRuoli() {
		List<String> nomi = new ArrayList<String>(ConstantsRole.DEFAULTS.keySet());
		if(ApplicationUtil.notEmpty(nomi)) {
			for (String key : nomi) {
				Ruolo role =  getRuoloByNome(key);
				if(role == null) {
					role = ConstantsRole.DEFAULTS.get(key);
					salvaAggiorna(role);
				} 
			}
			eliminaRuoli(nomi);
		}
	}
	

	public void creaAdmin(String username, String password) {
		Boolean cp = Boolean.valueOf(configurazioneGenericService.getValore(DefaultsGeneric.KEY_PASSWORD_EMAIL_CAMBIO));
		String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.eq("username", username));
		Utente user = (Utente) getUniqueResult(criteria);
		if(user == null) {
			user = new Utente();
			user.setUsername(username);
			user.setData_password(ApplicationUtil.date());
			user.setEmail(ApplicationUtil.trimToNull(percorsoReply));
			if(ApplicationUtil.isTrue(cp)) {
				user.setCambioPassword(Boolean.TRUE);
			}
			if(percorsoReply == null || password != null) {
				if(password != null) {
					user.setPassword(ApplicationUtil.encoder(password));
				} else {
					user.setPassword(ApplicationUtil.encoder(user.getUsername()));
				}
			} else {
				String passwd_nn_codificata = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_password);
				user.setPassword(ApplicationUtil.encoder(passwd_nn_codificata));
				String testo =  downloadDao.getRecupero(user, passwd_nn_codificata);
				String oggetto = ConstantsLanguage.INVIO_ACCOUNT.stampa() + ": " + user.getDenominazione();
				smtpService.invia(null, percorsoReply, null, oggetto, testo, null, null);
			}
			List<Ruolo> rl = new ArrayList<Ruolo>();
			Ruolo role =  getRuoloByNome(ConstantsRole.ROLE_ADMIN);
			rl.add(role);
			user.setRuoli(rl);
			salvaUser(user);
		} else {
			if(user.getPassword() == null) {
				user.setData_password(ApplicationUtil.date());
				if(ApplicationUtil.isTrue(cp)) {
					user.setCambioPassword(Boolean.TRUE);
				}
				if(percorsoReply == null || password != null) {
					if(password != null) {
						user.setPassword(ApplicationUtil.encoder(password));
					} else {
						user.setPassword(ApplicationUtil.encoder(user.getUsername()));
					}
				} else {
					String passwd_nn_codificata = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_password);
					user.setPassword(ApplicationUtil.encoder(passwd_nn_codificata));
					String testo =  downloadDao.getRecupero(user, passwd_nn_codificata);
					String oggetto = ConstantsLanguage.INVIO_ACCOUNT.stampa() + ": " + user.getDenominazione();
					smtpService.invia(null, percorsoReply, null, oggetto, testo, null, null);
				}
				salvaUser(user);
			}
		}
	}
	
	public void chiama(Utente utente, String numero, String recall, String contatto, String ingresso, String context)  {
		try {
			autodialerDao.originateCommand(utente, numero, recall, contatto, ingresso, context);
		} catch (Exception e) {
		}
	}
	public String getEditor() {
		return configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_MODE_EDITOR);
	}
	public Boolean getFlash() {
		return Boolean.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_MODE_FLASH));
	}
	public CodiceQR generaCodice(Utente user) {
		String azienda = impostazioneDao.getApplicazione();
		String link = user.getUsername() + "@" + impostazioneDao.getUrlBase();
		String secret = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_codice);
		GoogleAuthenticatorConfig config = dataDao.getGoogleAuthenticatorConfig();
		String KEY_SECRET = ApplicationUtil.base32Encoder(secret);
		GoogleAuthenticatorKey key = new GoogleAuthenticatorKey.Builder(KEY_SECRET).setConfig(config).build();
		//String secretKey = key.getKey();
		String otpAuthURL = GoogleAuthenticatorQRGenerator.getOtpAuthURL(azienda, link, key);
		CodiceQR qr = new CodiceQR();
		qr.setLink(otpAuthURL);
		qr.setCodice(secret);
		try {
			qr.setImmagine(ApplicationUtil.paintFromUrl(otpAuthURL));
		} catch (Exception e) {
		}
		return qr;
	}
	public void salvaCodice(Long userId, CodiceQR qr){
		Utente user = getUser(userId);
		if(user != null) {
			user.setSecret(qr.getCodice());
			salvaUser(user);
		}
	}
	public void resettaCodice(Long userId){
		Utente user = getUser(userId);
		if(user != null) {
			user.setSecret(null);
			salvaUser(user);
		}
	}
	public void clearCodice(Utente user){
		if(user != null) {
			user.setSecret(null);
			salvaUser(user);
		}
	}
	public Integer generaOTP(Utente user)  {
		GoogleAuthenticatorConfig config = dataDao.getGoogleAuthenticatorConfig();
		String KEY_SECRET = ApplicationUtil.base32Encoder(user.getSecret());
		GoogleAuthenticatorKey key = new GoogleAuthenticatorKey.Builder(KEY_SECRET).setConfig(config).build();
		GoogleAuthenticator ga = new GoogleAuthenticator(config);
		String secretKey = key.getKey();
		return ga.getTotpPassword(secretKey);
	}
	public boolean verificaOTP(Utente user) {
		Integer a = impostazioneDao.getAbilitaOTP();
		boolean blocca = false;
		if(ApplicationUtil.gtZero(a)) {
			blocca = true;
			String s = ApplicationUtil.trimToNull(user.getSecret());
			String g = ApplicationUtil.parameter("otp");
			if(s == null) {
				if(g == null) {
					blocca = false;
				} else {
					blocca = true;
				}
			} else {
				if(g == null) {
					blocca = true;
				} else {
					String KEY_SECRET = ApplicationUtil.base32Encoder(s);
					GoogleAuthenticatorConfig config = dataDao.getGoogleAuthenticatorConfig();
					GoogleAuthenticator ga = new GoogleAuthenticator(config);
					Integer otp = ApplicationUtil.parseInteger(g);
					if(otp != null) {
						blocca = !ga.authorize(KEY_SECRET, otp);
					} else {
						blocca = true;
					}
				}
			}
		}
		return blocca;
	}
	
	public void codiceDaScheda(Utente user){
		if(ApplicationUtil.isNotEmpty(user.getEmail())) {
			String codice = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_codice);
			user.setSecret(codice);
			salvaUser(user);
			String testo = downloadDao.getRecupero(user, null);
			String oggetto = ConstantsLanguage.INVIO_OTP.stampa() + ": " + user.getDenominazione();
			try{
				String ext = ConstantsMap.png;
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				generaCodice(user, out, ext);
				String nomefile = ApplicationUtil.lowerCase(ConstantsLanguage.OTP.stampa()) + "." + ext;
				String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
				List<Allegato> allegati = ApplicationUtil.allegatiConvertO(out, nomefile);
				smtpService.invia(percorsoReply, user.getEmail(), null, oggetto, testo, allegati, null);
			} catch(Exception e) { 
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void generaCodice(Utente user, ByteArrayOutputStream out, String ext) throws Exception {
		String azienda = impostazioneDao.getApplicazione();
		String link = user.getUsername() + "@" + impostazioneDao.getUrlBase();
		GoogleAuthenticatorConfig config = dataDao.getGoogleAuthenticatorConfig();
		String KEY_SECRET = ApplicationUtil.base32Encoder(user.getSecret());
		GoogleAuthenticatorKey key = new GoogleAuthenticatorKey.Builder(KEY_SECRET).setConfig(config).build();
		//String secretKey = key.getKey();
		String otpAuthURL = GoogleAuthenticatorQRGenerator.getOtpAuthURL(azienda, link, key);
		InputStream stream = new URL(otpAuthURL).openStream();
		byte [] b = ApplicationUtil.toByteArray(stream);
		stream.close();
		ByteArrayInputStream fi = new ByteArrayInputStream(b);
		ApplicationUtil.copyImage(fi, ext, out);
		fi.close();
		out.close();
	}
	public void codiceDaScheda(Long userId, CodiceQR qr){
		Utente user = getUser(userId);
		if(user != null && ApplicationUtil.isNotEmpty(user.getEmail())) {
			user.setSecret(qr.getCodice());
			salvaUser(user);
			String testo = downloadDao.getRecupero(user, null);
			String oggetto = ConstantsLanguage.INVIO_OTP.stampa() + ": " + user.getDenominazione();
			try{
				String ext = ConstantsMap.png;
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				generaCodice(qr, out, ext);
				String nomefile = ApplicationUtil.lowerCase(ConstantsLanguage.OTP.stampa()) + "." + ext;
				String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
				List<Allegato> allegati = ApplicationUtil.allegatiConvertO(out, nomefile);
				smtpService.invia(percorsoReply, user.getEmail(), null, oggetto, testo, allegati, null);
			} catch(Exception e) { 
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void generaCodice(CodiceQR qr, ByteArrayOutputStream out, String ext) throws Exception {
		InputStream stream = new URL(qr.getLink()).openStream();
		byte [] b = ApplicationUtil.toByteArray(stream);
		stream.close();
		ByteArrayInputStream fi = new ByteArrayInputStream(b);
		ApplicationUtil.copyImage(fi, ext, out);
		fi.close();
		out.close();
	}
	public boolean emailRecuperoDati(String email){
		if(ApplicationUtil.isNotEmpty(email)) {
			String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
			Boolean cp = Boolean.valueOf(configurazioneGenericService.getValore(DefaultsGeneric.KEY_PASSWORD_EMAIL_CAMBIO));
			Utente user1 = getUserByEmail(email);
			if(user1!=null) {
				String passwd_nn_codificata = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_password);
				user1.setData_password(ApplicationUtil.date());
				if(ApplicationUtil.isTrue(cp)) {
					user1.setCambioPassword(Boolean.TRUE);
				}
				user1.setPassword(ApplicationUtil.encoder(passwd_nn_codificata));
				salvaUser(user1);
				String testo =  downloadDao.getRecupero(user1, passwd_nn_codificata);
				String oggetto = ConstantsLanguage.INVIO_ACCOUNT.stampa() + ": " + user1.getDenominazione();
				smtpService.invia(percorsoReply, user1.getEmail(), null, oggetto, testo, null, null);
				return true;
			}
			Utente user2 = getUserByUsername(email);
			if(user2!=null && ApplicationUtil.isNotEmpty(user2.getEmail())) {
				String passwd_nn_codificata = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_password);
				user2.setData_password(ApplicationUtil.date());
				if(ApplicationUtil.isTrue(cp)) {
					user2.setCambioPassword(Boolean.TRUE);
				}
				user2.setPassword(ApplicationUtil.encoder(passwd_nn_codificata));
				salvaUser(user2);
				String testo =  downloadDao.getRecupero(user2, passwd_nn_codificata);
				String oggetto = ConstantsLanguage.INVIO_ACCOUNT.stampa() + ": " + user2.getDenominazione();
				smtpService.invia(percorsoReply, user2.getEmail(), null, oggetto, testo, null, null);
				return true;
			}
		}
		return false;
	}
	public boolean emailRecuperoCodice(String email){
		if(ApplicationUtil.isNotEmpty(email)) {
			String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
			Utente user1 = getUserByEmail(email);
			if(user1!=null) {
				String secret = ApplicationUtil.trimToNull(user1.getSecret());
				if(secret != null) {
					Integer codice = generaOTP(user1);
					String cod = String.format("%0"+ConstantsGlobal.cifre_otp+"d", codice);
					String testo =  downloadDao.getCodice(user1, cod);
					String oggetto = ConstantsLanguage.RECUPERA_OTP.stampa() + ": " + user1.getDenominazione();
					smtpService.invia(percorsoReply, user1.getEmail(), null, oggetto, testo, null, null);
					return true;
				}
			}
			Utente user2 = getUserByUsername(email);
			if(user2!=null) {
				String secret = ApplicationUtil.trimToNull(user2.getSecret());
				if(secret != null) {
					Integer codice = generaOTP(user2);
					String cod = String.format("%0"+ConstantsGlobal.cifre_otp+"d", codice);
					String testo =  downloadDao.getCodice(user2, cod);
					String oggetto = ConstantsLanguage.RECUPERA_OTP.stampa() + ": " + user2.getDenominazione();
					smtpService.invia(percorsoReply, user2.getEmail(), null, oggetto, testo, null, null);
					return true;
				}
			}
		}
		return false;
	}
	public boolean emailRecuperoQR(String email){
		if(ApplicationUtil.isNotEmpty(email)) {
			String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
			Utente user1 = getUserByEmail(email);
			if(user1!=null) {
				String secret = ApplicationUtil.trimToNull(user1.getSecret());
				if(secret != null) {
					try{
						String codice = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_codice);
						user1.setSecret(codice);
						salvaUser(user1);
						String testo = downloadDao.getRecupero(user1, null);
						String oggetto = ConstantsLanguage.INVIO_OTP.stampa() + ": " + user1.getDenominazione();
						String ext = ConstantsMap.png;
						ByteArrayOutputStream out = new ByteArrayOutputStream();
						generaCodice(user1, out, ext);
						String nomefile = ApplicationUtil.lowerCase(ConstantsLanguage.OTP.stampa()) + "." + ext;
						List<Allegato> allegati = ApplicationUtil.allegatiConvertO(out, nomefile);
						smtpService.invia(percorsoReply, user1.getEmail(), null, oggetto, testo, allegati, null);
						return true;
					} catch(Exception e) { 
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
			Utente user2 = getUserByUsername(email);
			if(user2!=null) {
				String secret = ApplicationUtil.trimToNull(user2.getSecret());
				if(secret != null) {
					try{
						String codice = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_codice);
						user2.setSecret(codice);
						salvaUser(user2);
						String testo = downloadDao.getRecupero(user2, null);
						String oggetto = ConstantsLanguage.INVIO_OTP.stampa() + ": " + user2.getDenominazione();
						String ext = ConstantsMap.png;
						ByteArrayOutputStream out = new ByteArrayOutputStream();
						generaCodice(user2, out, ext);
						String nomefile = ApplicationUtil.lowerCase(ConstantsLanguage.OTP.stampa()) + "." + ext;
						List<Allegato> allegati = ApplicationUtil.allegatiConvertO(out, nomefile);
						smtpService.invia(percorsoReply, user2.getEmail(), null, oggetto, testo, allegati, null);
						return true;
					} catch(Exception e) { 
						ErrorLog.log(null, getClass(), e);
					}
				}
			}
		}
		return false;
	}

	private void prenota(Utente utente, UtenteVO u){
		utente.setData_inizio(ApplicationUtil.inizioGiornata(u.getData()));
		utente.setData_fine(ApplicationUtil.fineGiornata(u.getData()));
		utente.setOra_inizio(u.getOra_inizio());
		utente.setOra_fine(u.getOra_fine());
		utente.setWeek_inizio(null);
		utente.setWeek_fine(null);
		u.setInverti(false);
		//u.setIp_bloccati(null);
		//u.setIp_consentiti(null);
	}
	public boolean prenotaUser(String email, UtenteVO u){
		if(ApplicationUtil.isNotEmpty(email)) {
			String percorsoReply = ApplicationUtil.trimToNull(configurazioneGenericService.getValore(DefaultsGeneric.KEY_INVIO_MAIL_AMMINISTRAZIONE));
			Boolean cp = Boolean.valueOf(configurazioneGenericService.getValore(DefaultsGeneric.KEY_PASSWORD_EMAIL_CAMBIO));
			Utente user1 = getUserByEmail(email);
			if(user1!=null && user1.isEnabled()) {
				String passwd_nn_codificata = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_password);
				user1.setData_password(ApplicationUtil.date());
				if(ApplicationUtil.isTrue(cp)) {
					user1.setCambioPassword(Boolean.TRUE);
				}
				user1.setPassword(ApplicationUtil.encoder(passwd_nn_codificata));
				prenota(user1, u);
				salvaUser(user1);
				String testo =  downloadDao.getPrenotazione(user1, u.getData(), passwd_nn_codificata);
				String oggetto = ConstantsLanguage.INVIO_ACCOUNT.stampa() + ": " + user1.getDenominazione();
				smtpService.invia(percorsoReply, user1.getEmail(), null, oggetto, testo, null, null);
				if(percorsoReply != null) {
					smtpService.invia(null, percorsoReply, null, oggetto, testo, null, null);
				}
				return true;
			}
			Utente user2 = getUserByUsername(email);
			if(user2!=null && user2.isEnabled() && ApplicationUtil.isNotEmpty(user2.getEmail())) {
				String passwd_nn_codificata = ApplicationUtil.randomString(ConstantsGlobal.lunghezza_password);
				user2.setData_password(ApplicationUtil.date());
				if(ApplicationUtil.isTrue(cp)) {
					user2.setCambioPassword(Boolean.TRUE);
				}
				user2.setPassword(ApplicationUtil.encoder(passwd_nn_codificata));
				prenota(user2, u);
				salvaUser(user2);
				String testo =  downloadDao.getPrenotazione(user2, u.getData(), passwd_nn_codificata);
				String oggetto = ConstantsLanguage.INVIO_ACCOUNT.stampa() + ": " + user2.getDenominazione();
				smtpService.invia(percorsoReply, user2.getEmail(), null, oggetto, testo, null, null);
				if(percorsoReply != null) {
					smtpService.invia(null, percorsoReply, null, oggetto, testo, null, null);
				}
				return true;
			}
		}
		return false;
	}

	public UtenteVO getUtenteVO(){
		return new UtenteVO();
	}


	public void  eliminaUtenteSelezionati(List<Utente> lst) {
		if (ApplicationUtil.notEmpty(lst)) {
			Iterator<Utente> itCont = lst.iterator();
			while (itCont.hasNext()) {
				Utente utente = itCont.next();
				if(utente.getSelezionabile()) {
					cancellaAggiorna(utente);
				}
			}
		}
	}
	
	
	public void copiaOperatore(Utente user_dst, Utente user_src) {
		if(user_dst != null && user_src != null) {
			List<Long> teams = getTeamByOperatore(user_src);
			Iterator<Long> objIter = teams.iterator();
			while(objIter.hasNext()) {
				Long teamId = objIter.next();
				Team team = getTeamOperatore(teamId);
				if(team.getUser() == null)
					team.setUser(new ArrayList<Utente>());
				if(!team.getUser().contains(user_dst)) {
					team.getUser().add(user_dst);
					salvaTeam(team);
				}
			}
			user_dst.setPermessiOperatore(user_src.getPermessiOperatore());
			user_dst.setPannelloOperatore(user_src.getPannelloOperatore());
			user_dst.setLimitaRicercaOperatore(user_src.getLimitaRicercaOperatore());
			user_dst.setLimitaAzioniOperatore(user_src.getLimitaAzioniOperatore());
			user_dst.setLimitaDatiOperatore(user_src.getLimitaDatiOperatore());
			user_dst.setLimitaModificaOperatore(user_src.getLimitaModificaOperatore());
			user_dst.setLimitaPermessiOperatore(user_src.getLimitaPermessiOperatore());
			salvaUser(user_dst);
		}
	}

	@SuppressWarnings("unchecked")
	public List<Long> getTeamByOperatore(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("user").add(Restrictions.idEq(user.getId()));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}

	public Team getTeamOperatore(Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.add(Restrictions.idEq(teamId));
		criteria.setFetchMode("user", FetchMode.JOIN);
		return (Team) getUniqueResult(criteria);
	}
	
	public void copiaSupervisore(Utente user_dst, Utente user_src) {
		if(user_dst != null && user_src != null) {
			List<Long> teams = getTeamBySupervisore(user_src);
			Iterator<Long> objIter = teams.iterator();
			while(objIter.hasNext()) {
				Long teamId = objIter.next();
				Team team = getTeamSupervisore(teamId);
				if(team.getSupervisore() == null)
					team.setSupervisore(new ArrayList<Utente>());
				if(!team.getSupervisore().contains(user_dst)) {
					team.getSupervisore().add(user_dst);
					salvaTeam(team);
				}
			}
			user_dst.setPermessiSupervisore(user_src.getPermessiSupervisore());
			user_dst.setPannelloSupervisore(user_src.getPannelloSupervisore());
			user_dst.setLimitaRicercaSupervisore(user_src.getLimitaRicercaSupervisore());
			user_dst.setLimitaAzioniSupervisore(user_src.getLimitaAzioniSupervisore());
			user_dst.setLimitaDatiSupervisore(user_src.getLimitaDatiSupervisore());
			user_dst.setLimitaModificaSupervisore(user_src.getLimitaModificaSupervisore());
			user_dst.setLimitaCampagnaSupervisore(user_src.getLimitaCampagnaSupervisore());
			salvaUser(user_dst);
		}
	}

	@SuppressWarnings("unchecked")
	public List<Long> getTeamBySupervisore(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}

	public Team getTeamSupervisore(Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.add(Restrictions.idEq(teamId));
		criteria.setFetchMode("supervisore", FetchMode.JOIN);
		ApplicationUtil.orderT(criteria);
		return (Team) getUniqueResult(criteria);
	}
	
	
	public void copiaAgente(Utente user_dst, Utente user_src) {
		if(user_dst != null && user_src != null) {
			List<Long> agenti = getAgenteByUser(user_src);
			Iterator<Long> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Long agenteId = objIter.next();
				Agente agente = getAgenteUser(agenteId);
				if(agente.getUser() == null)
					agente.setUser(new ArrayList<Utente>());
				if(!agente.getUser().contains(user_dst)) {
					agente.getUser().add(user_dst);
					salvaAgente(agente);
				}
			}
			user_dst.setPermessiAgente(user_src.getPermessiAgente());
			user_dst.setLimitaRicercaAgente(user_src.getLimitaRicercaAgente());
			user_dst.setLimitaAzioniAgente(user_src.getLimitaAzioniAgente());
			user_dst.setLimitaDatiAgente(user_src.getLimitaDatiAgente());
			user_dst.setLimitaModificaAgente(user_src.getLimitaModificaAgente());
			salvaUser(user_dst);
		}
	}

	@SuppressWarnings("unchecked")
	public List<Long> getAgenteByUser(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("user").add(Restrictions.idEq(user.getId()));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}

	public Agente getAgenteUser(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("user", FetchMode.JOIN);
		ApplicationUtil.orderA(criteria);
		return (Agente) getUniqueResult(criteria);
	}
	
	
	public void copiaCoordinatore(Utente user_dst, Utente user_src) {
		if(user_dst != null && user_src != null) {
			List<Long> agenti = getCoordinatoreByUser(user_src);
			Iterator<Long> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Long agenteId = objIter.next();
				Zona zona = getCoordinatoreUser(agenteId);
				if(zona.getSupervisori() == null)
					zona.setSupervisori(new ArrayList<Utente>());
				if(!zona.getSupervisori().contains(user_dst)) {
					zona.getSupervisori().add(user_dst);
					salvaZona(zona);
				}
			}
			user_dst.setPermessiCoordinatore(user_src.getPermessiCoordinatore());
			user_dst.setLimitaRicercaCoordinatore(user_src.getLimitaRicercaCoordinatore());
			user_dst.setLimitaAzioniCoordinatore(user_src.getLimitaAzioniCoordinatore());
			user_dst.setLimitaDatiCoordinatore(user_src.getLimitaDatiCoordinatore());
			user_dst.setLimitaModificaCoordinatore(user_src.getLimitaModificaCoordinatore());
			salvaUser(user_dst);
		}
	}

	@SuppressWarnings("unchecked")
	public List<Long> getCoordinatoreByUser(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.createCriteria("supervisori").add(Restrictions.idEq(user.getId()));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}

	public Zona getCoordinatoreUser(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("supervisori", FetchMode.JOIN);
		return (Zona) getUniqueResult(criteria);
	}
	
	public void copiaPartner(Utente user_dst, Utente user_src) {
		if(user_dst != null && user_src != null) {
			List<Long> agenti = getPartnerByUser(user_src);
			Iterator<Long> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Long agenteId = objIter.next();
				Campagna campagna = getPartnerUser(agenteId);
				if(campagna.getPartner() == null)
					campagna.setPartner(new ArrayList<Utente>());
				if(!campagna.getPartner().contains(user_dst)) {
					campagna.getPartner().add(user_dst);
					salvaCampagna(campagna);
				}
			}
			user_dst.setPermessiPartner(user_src.getPermessiPartner());
			user_dst.setLimitaRicercaPartner(user_src.getLimitaRicercaPartner());
			user_dst.setLimitaAzioniPartner(user_src.getLimitaAzioniPartner());
			user_dst.setLimitaDatiPartner(user_src.getLimitaDatiPartner());
			user_dst.setLimitaModificaPartner(user_src.getLimitaModificaPartner());
			user_dst.setLimitaCampagnaPartner(user_src.getLimitaCampagnaPartner());
			salvaUser(user_dst);
		}
	}

	@SuppressWarnings("unchecked")
	public List<Long> getPartnerByUser(Utente user) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}

	public Campagna getPartnerUser(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("partner", FetchMode.JOIN);
		ApplicationUtil.orderCL(criteria);
		return (Campagna) getUniqueResult(criteria);
	}
	
	public Boolean getSegreteriaOK(String interno) {
		if(ApplicationUtil.isNotBlank(interno)) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + "/" + ConstantsGlobal.path_voicemail + "/default/" + interno);
			return perc.exists();
		}
		return false;
	}
	
	
	public List<SelectItem> getOperatoriSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, true, null);
	}
}