package it.callite.dao;
import java.util.Iterator;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.criteria.VocalOrderCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.model.VocalOrder;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.FileUpload;
import it.callite.utility.MyFile;
@SuppressWarnings("serial")
@Repository("vocalOrderDao")
public class VocalOrderDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public VocalOrder get(Long id) {
		return  (VocalOrder) get(VocalOrder.class, id);
	}
	public VocalOrder salvaVocalOrder(VocalOrder vocalOrder) {
		return (VocalOrder) salvaAggiorna(vocalOrder);
	}
	public void eliminaVocalOrder(VocalOrder vocalOrder) {
		cancellaAggiorna(vocalOrder);
	}
	private ExtendedCriteria trovaCriteria(VocalOrderCriteria ana) {
		if(ana != null)  ana = (VocalOrderCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(VocalOrder.class);
		if (ApplicationUtil.isNotBlank(ana.getTesto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getTesto(), criteria, "nome", "testo", "descrizione");
		}
		if (ApplicationUtil.gtZero(ana.getProdottoId())) {
			criteria.createCriteria("prodotti").add(Restrictions.eq("id",ana.getProdottoId()));
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<VocalOrder> trova(VocalOrderCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(VocalOrderCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	
	public void salvaDocumento(FileUpload fileUpload, VocalOrder vocalOrder) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					vocalOrder.setPdf(ff.getData());
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
}