package it.callite.dao.config;
import java.util.Iterator;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.config.DefaultsAnagrafica;
import it.callite.criteria.ConfigurazioneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.ImpostazioneDao;
import it.callite.dao.generic.GenericDao;
import it.callite.model.config.ConfigAnagrafica;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("configurazioneAnagraficaDao")
public class ConfigurazioneAnagraficaDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;

	public ConfigAnagrafica getValue(String key) {
		ConfigAnagrafica conf = getValoreByChiave(key);
		if (conf == null) {
			conf = DefaultsAnagrafica.DEFAULTS.get(key);
			try {
				salva(conf);
			} catch (Exception e) {	}
		} 
		return conf;
	}
	private ConfigAnagrafica getValoreByChiave(String chiave) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAnagrafica.class);
		criteria.add(Restrictions.eq("chiave", ApplicationUtil.trimToEmpty(chiave)));
		return (ConfigAnagrafica) getUniqueResult(criteria);
	}
	public void elimina(ConfigAnagrafica config) {
		cancellaAggiorna(config);
	}
	public ConfigAnagrafica salva(ConfigAnagrafica config) {
		return (ConfigAnagrafica) salvaAggiorna(config);
	}
	@SuppressWarnings("unchecked")
	public void eliminaNotInConfig(List<String> chiavi) {
		if(ApplicationUtil.notEmpty(chiavi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAnagrafica.class);
			criteria.add(Restrictions.not(Restrictions.in("chiave", chiavi)));
			List<ConfigAnagrafica> c = getList(criteria);
			cancellaLista(c);
		}
	}
	private ExtendedCriteria getConfigByCriteria(ConfigurazioneCriteria conf) {
		if(conf != null)  conf = (ConfigurazioneCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAnagrafica.class);
		if(ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.chiave(ConstantsGlobal.colonSeparator, conf.getNome(), criteria, "chiave");
		} 
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ConfigAnagrafica> trova(ConfigurazioneCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("chiave")});
		List<ConfigAnagrafica> lst = getList(criteria, start, size);
		return defaultS(lst);
	}
	public int trovaSize(ConfigurazioneCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public List<ConfigAnagrafica> defaultS(List<ConfigAnagrafica> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ConfigAnagrafica> it = lst.iterator();
			while (it.hasNext()) {
				ConfigAnagrafica configAnagrafica = (ConfigAnagrafica) it.next();
				configAnagrafica.setConfigure(DefaultsAnagrafica.DEFAULTS.get(configAnagrafica.getChiave()).getConfigure());
			}
		}
		return lst;
	}
}