package it.callite.dao.config;
import java.util.Iterator;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.config.DefaultsAutodialer;
import it.callite.criteria.ConfigurazioneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.ImpostazioneDao;
import it.callite.dao.generic.GenericDao;
import it.callite.model.config.ConfigAutodialer;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("configurazioneAutodialerDao")
public class ConfigurazioneAutodialerDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public ConfigAutodialer getValue(String key) {
		ConfigAutodialer conf = getValoreByChiave(key);
		if (conf == null) {
			conf = DefaultsAutodialer.DEFAULTS.get(key);
			try {
				salva(conf);
			} catch (Exception e) {	}
		} 
		return conf;
	}
	private ConfigAutodialer getValoreByChiave(String chiave) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAutodialer.class);
		criteria.add(Restrictions.eq("chiave", ApplicationUtil.trimToEmpty(chiave)));
		return (ConfigAutodialer) getUniqueResult(criteria);
	}
	public void elimina(ConfigAutodialer config) {
		cancellaAggiorna(config);
	}
	public ConfigAutodialer salva(ConfigAutodialer config) {
		return (ConfigAutodialer) salvaAggiorna(config);
	}
	@SuppressWarnings("unchecked")
	public void eliminaNotInConfig(List<String> chiavi) {
		if(ApplicationUtil.notEmpty(chiavi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAutodialer.class);
			criteria.add(Restrictions.not(Restrictions.in("chiave", chiavi)));
			List<ConfigAutodialer> c = getList(criteria);
			cancellaLista(c);
		}
	}
	private ExtendedCriteria getConfigByCriteria(ConfigurazioneCriteria conf) {
		if(conf != null)  conf = (ConfigurazioneCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigAutodialer.class);
		if(ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.chiave(ConstantsGlobal.colonSeparator, conf.getNome(), criteria, "chiave");
		} 
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ConfigAutodialer> trova(ConfigurazioneCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("chiave")});
		List<ConfigAutodialer> lst = getList(criteria, start, size);
		return defaultS(lst);
	}
	public int trovaSize(ConfigurazioneCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public List<ConfigAutodialer> defaultS(List<ConfigAutodialer> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ConfigAutodialer> it = lst.iterator();
			while (it.hasNext()) {
				ConfigAutodialer configAutodialer = (ConfigAutodialer) it.next();
				configAutodialer.setConfigure(DefaultsAutodialer.DEFAULTS.get(configAutodialer.getChiave()).getConfigure());
			}
		}
		return lst;
	}
}