package it.callite.dao.config;
import java.util.Iterator;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.config.DefaultsScheda;
import it.callite.criteria.ConfigurazioneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.ImpostazioneDao;
import it.callite.dao.generic.GenericDao;
import it.callite.model.config.ConfigScheda;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("configurazioneSchedaDao")
public class ConfigurazioneSchedaDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public ConfigScheda getValue(String key) {
		ConfigScheda conf = getValoreByChiave(key);
		if (conf == null) {
			conf = DefaultsScheda.DEFAULTS.get(key);
			try {
				salva(conf);
			} catch (Exception e) {	}
		} 
		return conf;
	}
	private ConfigScheda getValoreByChiave(String chiave) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigScheda.class);
		criteria.add(Restrictions.eq("chiave", ApplicationUtil.trimToEmpty(chiave)));
		return (ConfigScheda) getUniqueResult(criteria);
	}
	public void elimina(ConfigScheda config) {
		cancellaAggiorna(config);
	}
	public ConfigScheda salva(ConfigScheda config) {
		return (ConfigScheda) salvaAggiorna(config);
	}
	@SuppressWarnings("unchecked")
	public void eliminaNotInConfig(List<String> chiavi) {
		if(ApplicationUtil.notEmpty(chiavi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ConfigScheda.class);
			criteria.add(Restrictions.not(Restrictions.in("chiave", chiavi)));
			List<ConfigScheda> c = getList(criteria);
			cancellaLista(c);
		}
	}
	private ExtendedCriteria getConfigByCriteria(ConfigurazioneCriteria conf) {
		if(conf != null)  conf = (ConfigurazioneCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigScheda.class);
		if(ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.chiave(ConstantsGlobal.colonSeparator, conf.getNome(), criteria, "chiave");
		} 
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ConfigScheda> trova(ConfigurazioneCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("chiave")});
		List<ConfigScheda> lst = getList(criteria, start, size);
		return defaultS(lst);
	}
	public int trovaSize(ConfigurazioneCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public List<ConfigScheda> defaultS(List<ConfigScheda> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ConfigScheda> it = lst.iterator();
			while (it.hasNext()) {
				ConfigScheda configScheda = (ConfigScheda) it.next();
				configScheda.setConfigure(DefaultsScheda.DEFAULTS.get(configScheda.getChiave()).getConfigure());
			}
		}
		return lst;
	}
}