package it.callite.dao.config;
import java.util.Iterator;
import java.util.List;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.config.DefaultsValidatore;
import it.callite.criteria.ConfigurazioneCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.ImpostazioneDao;
import it.callite.dao.generic.GenericDao;
import it.callite.model.config.ConfigValidatore;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Repository("configurazioneValidatoreDao")
public class ConfigurazioneValidatoreDao extends GenericDao {
	
	@Autowired
	private ImpostazioneDao impostazioneDao;
	
	public ConfigValidatore getValue(String key) {
		ConfigValidatore conf = getValoreByChiave(key);
		if (conf == null) {
			conf = DefaultsValidatore.DEFAULTS.get(key);
			try {
				salva(conf);
			} catch (Exception e) {	}
		} 
		return conf;
	}
	private ConfigValidatore getValoreByChiave(String chiave) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigValidatore.class);
		criteria.add(Restrictions.eq("chiave", ApplicationUtil.trimToEmpty(chiave)));
		return (ConfigValidatore) getUniqueResult(criteria);
	}
	public void elimina(ConfigValidatore config) {
		cancellaAggiorna(config);
	}
	public ConfigValidatore salva(ConfigValidatore config) {
		return (ConfigValidatore) salvaAggiorna(config);
	}
	@SuppressWarnings("unchecked")
	public void eliminaNotInConfig(List<String> chiavi) {
		if(ApplicationUtil.notEmpty(chiavi)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(ConfigValidatore.class);
			criteria.add(Restrictions.not(Restrictions.in("chiave", chiavi)));
			List<ConfigValidatore> c = getList(criteria);
			cancellaLista(c);
		}
	}
	private ExtendedCriteria getConfigByCriteria(ConfigurazioneCriteria conf) {
		if(conf != null)  conf = (ConfigurazioneCriteria) conf.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(ConfigValidatore.class);
		if(ApplicationUtil.isNotBlank(conf.getNome())) {
			ApplicationUtil.chiave(ConstantsGlobal.colonSeparator, conf.getNome(), criteria, "chiave");
		} 
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, conf);
	}
	@SuppressWarnings("unchecked")
	public List<ConfigValidatore> trova(ConfigurazioneCriteria conf, int start, int size) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), new Order [] {Order.asc("chiave")});
		List<ConfigValidatore> lst = getList(criteria, start, size);
		return defaultS(lst);
	}
	public int trovaSize(ConfigurazioneCriteria conf) {
		ExtendedCriteria criteria = getConfigByCriteria(conf);
		return getListSize(criteria);
	}
	public List<ConfigValidatore> defaultS(List<ConfigValidatore> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			Iterator<ConfigValidatore> it = lst.iterator();
			while (it.hasNext()) {
				ConfigValidatore configValidatore = (ConfigValidatore) it.next();
				configValidatore.setConfigure(DefaultsValidatore.DEFAULTS.get(configValidatore.getChiave()).getConfigure());
			}
		}
		return lst;
	}
}