package it.callite.dao.generic;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;

import org.hibernate.FetchMode;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.query.Query;
import org.hibernate.transform.AliasToBeanResultTransformer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.faces.model.OneSelectionTrackingListDataModel;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.orm.hibernate5.SessionFactoryUtils;

import it.callite.constants.ConstantsGlobal;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.model.report.ConteggioModel;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.HibernateSupport;
import it.callite.utility.Label;
import it.callite.utility.LabelQuery;
import it.callite.utility.Parameter;
import it.callite.utility.item.AgenteSI;
import it.callite.utility.item.AutodialerSI;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.ContattoSI;
import it.callite.utility.item.EsitoAppuntamentoSI;
import it.callite.utility.item.EsitoArchiviazioneSI;
import it.callite.utility.item.EsitoChiamataSI;
import it.callite.utility.item.EsitoContrattoSI;
import it.callite.utility.item.GenericSI;
import it.callite.utility.item.InternoSI;
import it.callite.utility.item.ListaSI;
import it.callite.utility.item.ProdottoSI;
import it.callite.utility.item.RuoloSI;
import it.callite.utility.item.TeamSI;
import it.callite.utility.item.TrunkSI;
import it.callite.utility.item.UtenteSI;
import it.callite.utility.item.ZonaSI;
@SuppressWarnings("serial")
public class GenericDao implements Serializable {
	
	@Autowired
	private HibernateSupport hibernateSupport;
	private JdbcTemplate jdbcTemplate;
	private EntityManager entityManager;
	public EntityManager getEntityManager() {
		return entityManager;
	}
	public HibernateSupport getHibernateSupport() {
		return hibernateSupport;
	}
	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}
	public Session openSession(){
		return getHibernateSupport().openSession();
	}
	public void closeSession(Session session){
		session.close();
	}
	public Session currentSession(){
		return getHibernateSupport().currentSession();
	}
	//	public AuditReader getAuditReaderEntity() {
	//		return AuditReaderFactory.get(getEntityManager());
	//	}
	//	
	//	public AuditReader getAuditReaderSession() {
	//		return AuditReaderFactory.get(currentSession());
	//	}
	@Autowired
	public void sessionFactory(SessionFactory sessionFactory) {
		this.jdbcTemplate = new JdbcTemplate(SessionFactoryUtils.getDataSource(sessionFactory));
	}
	@Autowired
	public void setEntityManagerFactory(EntityManagerFactory entityManagerFactory) {
		this.entityManager = entityManagerFactory.createEntityManager();
	}
	public Object get(Class<?> classe, Long id) {
		if(id != null) { 
			DetachedCriteria criteria = DetachedCriteria.forClass(classe);
			criteria.add(Restrictions.idEq(id));
			return getUniqueResult(criteria);
		} else {
			return null;
		}
	}
	@SuppressWarnings("rawtypes")
	public Object getUniqueResult(DetachedCriteria criteria) {
		List ogg = getHibernateSupport().findByCriteria(criteria);
		if(ApplicationUtil.notEmpty(ogg)) {
			return ogg.get(0);
		} else {
			return null;
		}
	}

	@SuppressWarnings("rawtypes")
	public Object getFirstResult(DetachedCriteria criteria) {
		List ogg = getHibernateSupport().findByCriteria(criteria, 0, 1);
		if(ApplicationUtil.notEmpty(ogg)) {
			return ogg.get(0);
		} else {
			return null;
		}
	}
	@SuppressWarnings("rawtypes")
	public List getFirstResult(DetachedCriteria criteria, int size) {
		return getHibernateSupport().findByCriteria(criteria, 0, size);
	}
	public int getListSize(DetachedCriteria criteria) {
		criteria.setProjection(Projections.countDistinct("id"));
		Long size = (Long) getUniqueResult(criteria);
		return (size!=null) ? size.intValue() : 0;
	}
	public int getSommaIntero(DetachedCriteria criteria, String property) {
		criteria.setProjection(Projections.sum(property));
		Long imp = (Long) getUniqueResult(criteria);
		return (imp!=null) ? imp.intValue() : 0;
	}
	public BigDecimal getSommaDecimale(DetachedCriteria criteria, String property) {
		criteria.setProjection(Projections.sum(property));
		BigDecimal imp = (BigDecimal) getUniqueResult(criteria);
		return (imp!=null) ? imp : new BigDecimal(0);
	}
	@SuppressWarnings("rawtypes")
	public List getList(DetachedCriteria criteria) {
		criteria.setResultTransformer(ApplicationUtil.distinctRoot());
		return getHibernateSupport().findByCriteria(criteria);
	}
	@SuppressWarnings("rawtypes")
	public List getList(DetachedCriteria criteria, int start, int size) {
		criteria.setResultTransformer(ApplicationUtil.distinctRoot());
		return getHibernateSupport().findByCriteria(criteria, start, size);
	}
	@SuppressWarnings({ "unchecked" })
	public List<SelectItem> getListSI(DetachedCriteria criteria, String value, String label) {
		criteria.addOrder(Order.asc(label));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property(value).as("value")))
				.add(Projections.property(label).as("label")));
		return getList(criteria, SelectItem.class);
	}

	@SuppressWarnings({ "unchecked" })
	public List<Label> getLabelSI(DetachedCriteria criteria, String id, String label) {
		criteria.addOrder(Order.asc(label));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property(id).as("value")))
				.add(Projections.property(label).as("label")));
		return getList(criteria, Label.class);
	}

	@SuppressWarnings("rawtypes")
	public Object getUniqueResult(DetachedCriteria criteria, Class<?> classe) {
		criteria.setResultTransformer(ApplicationUtil.distinctRoot());
		criteria.setResultTransformer(new AliasToBeanResultTransformer(classe));
		List ogg = getHibernateSupport().findByCriteria(criteria);
		if(ApplicationUtil.notEmpty(ogg))
			return ogg.get(0);
		else return null;
	}
	@SuppressWarnings({ "unchecked" })
	public List<ConteggioModel> getListConteggio(DetachedCriteria criteria, String group, String count) {
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.groupProperty(group).as("id")))
				.add(Projections.countDistinct(count).as("conteggio"))
				);
		return getList(criteria, ConteggioModel.class);
	}
	@SuppressWarnings("rawtypes")
	public List getList(DetachedCriteria criteria, Class<?> classe) {
		criteria.setResultTransformer(ApplicationUtil.distinctRoot());
		criteria.setResultTransformer(new AliasToBeanResultTransformer(classe));
		return getHibernateSupport().findByCriteria(criteria);
	}
	public int getListSize(DetachedCriteria criteria, String property) {
		criteria.setProjection(Projections.countDistinct(property));
		Long size = (Long) getUniqueResult(criteria);
		return (size!=null) ? size.intValue() : 0;
	}
	@SuppressWarnings("rawtypes")
	public List getList(DetachedCriteria criteria, int start, int size, Class<?> classe) {
		criteria.setResultTransformer(ApplicationUtil.distinctRoot());
		criteria.setResultTransformer(new AliasToBeanResultTransformer(classe));
		return getHibernateSupport().findByCriteria(criteria, start, size);
	}
	@SuppressWarnings("unchecked")
	public List<CampagnaSI> getCampagnaSI(DetachedCriteria criteria) {
		ApplicationUtil.orderCL(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("nome").as("nome"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("stato").as("stato"))
				.add(Projections.property("inverti").as("inverti"))
				.add(Projections.property("data_inizio").as("data_inizio"))
				.add(Projections.property("data_fine").as("data_fine"))
				.add(Projections.property("week_inizio").as("week_inizio"))
				.add(Projections.property("week_fine").as("week_fine"))
				.add(Projections.property("ora_inizio").as("ora_inizio"))
				.add(Projections.property("ora_fine").as("ora_fine"))
				.add(Projections.property("tipo").as("tipo")));
		return getList(criteria, CampagnaSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<AgenteSI> getAgenteSI(DetachedCriteria criteria) {
		criteria.createCriteria("ufficio", "ufficio", ApplicationUtil.leftJoin());
		criteria.createCriteria("qualifica", "qualifica", ApplicationUtil.leftJoin());
		ApplicationUtil.orderA(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("autonomo").as("autonomo"))
				.add(Projections.property("nome").as("nome"))
				.add(Projections.property("cognome").as("cognome"))
				.add(Projections.property("pf").as("pf"))
				.add(Projections.property("nome_completo").as("nome_completo"))
				.add(Projections.property("ragione_sociale").as("ragione_sociale"))
				.add(Projections.property("ufficio.nome").as("ufficio"))
				.add(Projections.property("qualifica.nome").as("qualifica")));
		return getList(criteria, AgenteSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<ContattoSI> getContattoSI(DetachedCriteria criteria, DetachedCriteria criteriaA, String property) {
		criteriaA.addOrder(Order.asc(property));
		criteria.addOrder(Order.asc("id"));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("c.id").as("id")))
				.add(Projections.property("c.archiviato").as("archiviato"))
				.add(Projections.property("an.id").as("anagrafica_id"))
				.add(Projections.property("an.nome").as("anagrafica_nome"))
				.add(Projections.property("an.cognome").as("anagrafica_cognome"))
				.add(Projections.property("an.pf").as("anagrafica_pf"))
				.add(Projections.property("an.nome_completo").as("anagrafica_nome_completo"))
				.add(Projections.property("an.ragione_sociale").as("anagrafica_ragione_sociale"))
				.add(Projections.property("an.telefono_fisso").as("anagrafica_telefono_fisso"))
				.add(Projections.property("an.telefono_mobile").as("anagrafica_telefono_mobile"))
				.add(Projections.property("an.telefono_fax").as("anagrafica_telefono_fax"))
				.add(Projections.property("an.telefono_altro").as("anagrafica_telefono_altro"))
				.add(Projections.property("an.email").as("anagrafica_email"))
				.add(Projections.property("an.link").as("anagrafica_link"))
				.add(Projections.property("an.codice_fiscale").as("anagrafica_codice_fiscale"))
				.add(Projections.property("an.partita_iva").as("anagrafica_partita_iva"))
				.add(Projections.property("li.id").as("lista_id"))
				.add(Projections.property("li.nome").as("lista_nome"))
				.add(Projections.property("li.stato").as("lista_stato"))
				.add(Projections.property("li.codice").as("lista_codice"))
				.add(Projections.property("li.inverti").as("lista_inverti"))
				.add(Projections.property("li.data_inizio").as("lista_data_inizio"))
				.add(Projections.property("li.data_fine").as("lista_data_fine"))
				.add(Projections.property("li.week_inizio").as("lista_week_inizio"))
				.add(Projections.property("li.week_fine").as("lista_week_fine"))
				.add(Projections.property("li.ora_inizio").as("lista_ora_inizio"))
				.add(Projections.property("li.ora_fine").as("lista_ora_fine"))
				.add(Projections.property("ca.id").as("campagna_id"))
				.add(Projections.property("ca.nome").as("campagna_nome"))
				.add(Projections.property("ca.codice").as("campagna_codice"))
				.add(Projections.property("ca.stato").as("campagna_stato"))
				.add(Projections.property("ca.tipo").as("campagna_tipo"))
				.add(Projections.property("ca.inverti").as("campagna_inverti"))
				.add(Projections.property("ca.data_inizio").as("campagna_data_inizio"))
				.add(Projections.property("ca.data_fine").as("campagna_data_fine"))
				.add(Projections.property("ca.ora_inizio").as("campagna_ora_inizio"))
				.add(Projections.property("ca.week_inizio").as("campagna_week_inizio"))
				.add(Projections.property("ca.week_fine").as("campagna_week_fine"))
				.add(Projections.property("ca.ora_fine").as("campagna_ora_fine"))
				.add(Projections.property("ec.id").as("esito_chiamata_id"))
				.add(Projections.property("ec.etichetta").as("esito_chiamata_etichetta"))
				.add(Projections.property("ec.codice").as("esito_chiamata_codice"))
				.add(Projections.property("ec.richiamare").as("esito_chiamata_richiamare"))
				.add(Projections.property("ec.colore").as("esito_chiamata_colore"))
				.add(Projections.property("ea.id").as("esito_archiviazione_id"))
				.add(Projections.property("ea.etichetta").as("esito_archiviazione_etichetta"))
				.add(Projections.property("ea.codice").as("esito_archiviazione_codice"))
				.add(Projections.property("ea.utile").as("esito_archiviazione_utile"))
				.add(Projections.property("ea.colore").as("esito_archiviazione_colore"))
				);
		return getList(criteria, ContattoSI.class);
	}

	@SuppressWarnings("unchecked")
	public List<TeamSI> getTeamSI(DetachedCriteria criteria) {
		ApplicationUtil.orderT(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("nome").as("nome")));
		return getList(criteria, TeamSI.class);
	}

	@SuppressWarnings("unchecked")
	public List<ZonaSI> getZonaSI(DetachedCriteria criteria) {
		ApplicationUtil.orderZ(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("nome").as("nome")));
		return getList(criteria, ZonaSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<ProdottoSI> getProdottoSI(DetachedCriteria criteria) {
		ApplicationUtil.orderP(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("nome").as("nome"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("tipo").as("tipo"))
				.add(Projections.property("attivo").as("attivo"))
				.add(Projections.property("categoria.nome").as("categoria")));
		return getList(criteria, ProdottoSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<UtenteSI> getUtenteSI(DetachedCriteria criteria) {
		ApplicationUtil.orderU(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("nome").as("nome"))
				.add(Projections.property("cognome").as("cognome"))
				.add(Projections.property("username").as("username"))
				.add(Projections.property("enabled").as("enabled"))
				.add(Projections.property("inverti").as("inverti"))
				.add(Projections.property("data_inizio").as("data_inizio"))
				.add(Projections.property("data_fine").as("data_fine"))
				.add(Projections.property("week_inizio").as("week_inizio"))
				.add(Projections.property("week_fine").as("week_fine"))
				.add(Projections.property("ora_inizio").as("ora_inizio"))
				.add(Projections.property("ora_fine").as("ora_fine")));
		return getList(criteria, UtenteSI.class);
	}

	@SuppressWarnings("unchecked")
	public List<AutodialerSI> getAutodialerSI(DetachedCriteria criteria) {
		ApplicationUtil.orderAU(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("nome").as("nome"))
				.add(Projections.property("stato").as("stato"))
				.add(Projections.property("inverti").as("inverti"))
				.add(Projections.property("data_inizio").as("data_inizio"))
				.add(Projections.property("data_fine").as("data_fine"))
				.add(Projections.property("week_inizio").as("week_inizio"))
				.add(Projections.property("week_fine").as("week_fine"))
				.add(Projections.property("ora_inizio").as("ora_inizio"))
				.add(Projections.property("ora_fine").as("ora_fine")));
		return getList(criteria, AutodialerSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoChiamataSI> getEsitoChiamataSI(DetachedCriteria criteria) {
		ApplicationUtil.orderEsito(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("etichetta").as("etichetta"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("richiamare").as("richiamare")));
		return getList(criteria, EsitoChiamataSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazioneSI> getEsitoArchiviazioneSI(DetachedCriteria criteria) {
		ApplicationUtil.orderEsito(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("etichetta").as("etichetta"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("utile").as("utile")));
		return getList(criteria, EsitoArchiviazioneSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoContrattoSI> getEsitoContrattoSI(DetachedCriteria criteria) {
		ApplicationUtil.orderEsito(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("etichetta").as("etichetta"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("utile").as("utile")));
		return getList(criteria, EsitoContrattoSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamentoSI> getEsitoAppuntamentoSI(DetachedCriteria criteria) {
		ApplicationUtil.orderEsito(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("etichetta").as("etichetta"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("utile").as("utile")));
		return getList(criteria, EsitoAppuntamentoSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<RuoloSI> getRuoloSI(DetachedCriteria criteria) {
		ApplicationUtil.orderR(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("nome").as("nome")));
		return getList(criteria, RuoloSI.class);
	}
	@SuppressWarnings("unchecked")
	public List<ListaSI> getListaSI(DetachedCriteria criteria) {
		ApplicationUtil.orderCL(criteria);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("nome").as("nome"))
				.add(Projections.property("codice").as("codice"))
				.add(Projections.property("stato").as("stato"))
				.add(Projections.property("inverti").as("inverti"))
				.add(Projections.property("data_inizio").as("data_inizio"))
				.add(Projections.property("data_fine").as("data_fine"))
				.add(Projections.property("week_inizio").as("week_inizio"))
				.add(Projections.property("week_fine").as("week_fine"))
				.add(Projections.property("ora_inizio").as("ora_inizio"))
				.add(Projections.property("ora_fine").as("ora_fine")));
		return getList(criteria, ListaSI.class);
	}


	@SuppressWarnings("unchecked")
	public List<InternoSI> getInternoSI(DetachedCriteria criteria) {
		criteria.addOrder(Order.asc("name"));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("name").as("name"))
				.add(Projections.property("opposizione").as("opposizione"))
				.add(Projections.property("interurbane").as("interurbane"))
				.add(Projections.property("attivo").as("attivo"))
				.add(Projections.property("registrazione").as("registrazione"))
				.add(Projections.property("interne").as("interne")));
		return getList(criteria, InternoSI.class);
	}

	@SuppressWarnings("unchecked")
	public List<TrunkSI> getTrunkSI(DetachedCriteria criteria) {
		criteria.addOrder(Order.asc("name"));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("id").as("id")))
				.add(Projections.property("name").as("name"))
				.add(Projections.property("callers").as("callers"))
				.add(Projections.property("musicclass").as("musicclass"))
				.add(Projections.property("add_prefisso").as("add_prefisso"))
				.add(Projections.property("antecall").as("antecall"))
				.add(Projections.property("opposizione").as("opposizione"))
				.add(Projections.property("interurbane").as("interurbane"))
				.add(Projections.property("attivo").as("attivo"))
				.add(Projections.property("registrazione").as("registrazione"))
				.add(Projections.property("vrx").as("vrx"))
				.add(Projections.property("vtx").as("vtx")));
		return getList(criteria, TrunkSI.class);
	}

	@SuppressWarnings({ "unchecked" })
	public List<GenericSI> getGenericSI(DetachedCriteria criteria, String id, String label) {
		criteria.addOrder(Order.asc(label));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property(id).as("id")))
				.add(Projections.property(label).as("nome")));

		return getList(criteria, GenericSI.class);
	}

	public Object salvaAggiorna(Object oggetto) {
		return getHibernateSupport().saveOrUpdate(oggetto);
	}

	public void cancellaAggiorna(Object oggetto) {
		getHibernateSupport().delete(oggetto);
	}

	public void execute(boolean nq, String hql, Parameter ... params) {
		Session session = openSession();
		Transaction tx = session.beginTransaction();
		execute(nq, session, hql, params);
		session.flush();
		session.clear();
		tx.commit();
		closeSession(session);
	}


	@SuppressWarnings("rawtypes")
	public List list(boolean nq, String hql, Parameter ... params) {
		Session session = openSession();
		List<?> ls = list(nq, session, hql, params);
		closeSession(session);
		return ls;
	}

	public void execute(boolean nq, Session session, String hql, Parameter ... params) {
		Query<?> query = nq ? session.createNativeQuery(hql) : session.createQuery(hql);
		if(ApplicationUtil.notEmpty(params)) {
			for (Parameter param : params) {
				if(param.getSingle() != null) {
					query.setParameter(param.getLabel(), param.getSingle());
				} 
				if(param.getMultiple() != null) {
					query.setParameterList(param.getLabel(), param.getMultiple());
				}
			}
		}
		query.executeUpdate();
	}

	@SuppressWarnings("rawtypes")
	public List list(boolean nq, Session session, String hql, Parameter ... params) {
		Query<?> query = nq ? session.createNativeQuery(hql) : session.createQuery(hql);
		if(ApplicationUtil.notEmpty(params)) {
			for (Parameter param : params) {
				if(param.getSingle() != null) {
					query.setParameter(param.getLabel(), param.getSingle());
				} 
				if(param.getMultiple() != null) {
					query.setParameterList(param.getLabel(), param.getMultiple());
				}
			}
		}
		return query.list();
	}

	public List<LabelQuery> list(String hql, Parameter ... params) {
		Session session = openSession();
		Query<?> query = session.createNativeQuery(hql);
		if(ApplicationUtil.notEmpty(params)) {
			for (Parameter param : params) {
				if(param.getSingle() != null) {
					query.setParameter(param.getLabel(), param.getSingle());
				} 
				if(param.getMultiple() != null) {
					query.setParameterList(param.getLabel(), param.getMultiple());
				}
			}
		}
		List<LabelQuery> ls = ApplicationUtil.getResultList(query, LabelQuery.class);
		closeSession(session);
		return ls;
	}

	public void cancellaLista(List<?> objs) {
		if(ApplicationUtil.notEmpty(objs)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<?> objIter = objs.iterator();
			while(objIter.hasNext()) {
				Object obj = objIter.next();
				session.delete(obj);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}

	public void salvaLista(List<?> objs) {
		if(ApplicationUtil.notEmpty(objs)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<?> objIter = objs.iterator();
			while(objIter.hasNext()) {
				Object obj = objIter.next();
				session.save(obj);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	public void eliminaSelezionati(OneSelectionTrackingListDataModel selezionatoDaList) {
		if (selezionatoDaList != null) {
			Object lista = selezionatoDaList.getWrappedData();
			if (lista != null) {
				List<?> lst = (List<?>) lista;
				eliminaSelezionati(lst);
			}
		}
	}
	public void  eliminaSelezionati(List<?> lst) {
		if(ApplicationUtil.notEmpty(lst)) {
			cancellaLista(lst);
		}
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public List getList(DetachedCriteria criteria, Class<?> classe, String property, Order [] defaults, int start, int size) {
		String id = "id";
		criteria.setProjection((Projections.distinct(Projections.property(id))));
		List<Long> ids =  getList(criteria, start, size);
		DetachedCriteria crit = DetachedCriteria.forClass(classe);
		if(ApplicationUtil.notEmpty(ids)) {
			crit.add(Restrictions.in(id, ids));
		} else {
			crit.add(Restrictions.eq(id, ConstantsGlobal.NULL));
		}
		crit.setFetchMode(property, FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(crit, defaults);
		return getList(crit);
	}

	public BigDecimal getSommaDecimale(ExtendedCriteria c, String property) {
		DetachedCriteria criteria = c.getCriteria();
		if(c.getRicerca()) {
			BigDecimal ls = getSommaDecimale(criteria, property);
			c.log(null, null, ls);
			return ls;
		} else {
			return new BigDecimal(0);
		}
	}

	public int getListSize(ExtendedCriteria c) {
		DetachedCriteria criteria = c.getCriteria();
		if(c.getRicerca()) {
			int ls = getListSize(criteria);
			c.log(null, ls, null);
			return ls;
		} else {
			return 0;
		}
	}

	@SuppressWarnings("rawtypes")
	public List getList(ExtendedCriteria c, int start, int size) {
		DetachedCriteria criteria = c.getCriteria();
		if(c.getRicerca()) {
			List ls = getList(criteria, start, size);
			c.log(ls, null, null);
			return ls;
		} else {
			return null;
		}
	}

	@SuppressWarnings("rawtypes")
	public List getList(ExtendedCriteria c, Class<?> classe, String property, Order [] defaults, int start, int size) {
		DetachedCriteria criteria = c.getCriteria();
		if(c.getRicerca()) {
			List ls = getList(criteria, classe, property, defaults, start, size);
			return ls;
		} else {
			return null;
		}
	}

	public void deleteIntervallo(Integer intervallo, Class<?> classe, String campo) {
		Date data = ApplicationUtil.intervallo(intervallo);
		if(data != null) {
			String hql = "delete from "+classe.getSimpleName()+" where "+campo+" <= :data";
			execute(false, hql, new Parameter("data", data));
		}
	}

}