package it.callite.model;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.message.Message;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "acconto", indexes = { @Index(name= "acconto", columnList = "data_creazione, data_acconto" )})
public class Acconto  extends SelezionatoAbstract implements Serializable, Comparable<Acconto> {
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getUser().getDenominazione() + ": " + getImporto().toString());
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_acconto = ApplicationUtil.aggiustaDataT(ApplicationUtil.date(), true);
	public Date getData_acconto() {
		return ApplicationUtil.aggiustaDataT(data_acconto, true);
	}
	public void setData_acconto(Date data_acconto) {
		this.data_acconto = ApplicationUtil.aggiustaDataT(data_acconto, true);
	}
	@ManyToOne
	private Utente user;
	@Column(columnDefinition = "text")
	private String descrizione;
	private BigDecimal importo = new BigDecimal(0);
	public double getParziale() {
		BigDecimal i = getImporto();
		if(i == null)  {
			return 0;
		} else {
			return i.doubleValue();
		}
	}
	public BigDecimal getImporto() {
		return importo;
	}
	public void setImporto(BigDecimal importo) {
		this.importo = importo;
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public Utente getUser() {
		return user;
	}
	public void setUser(Utente user) {
		this.user = user;
	}
	public int compareTo(Acconto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Acconto other = (Acconto) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	private Integer tipo = ConstantsMap.tipo_acconto1;
	public Integer getTipo() {
		if(ApplicationUtil.ltZero(tipo)) {
			return ConstantsMap.tipo_acconto1;
		}
		return tipo;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	public String getTipoTesto() {
		return Message.addCatalogText("tipo_acconto" + tipo);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
}