package it.callite.model;
import java.io.Serializable;
import java.sql.Time;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.model.allegato.AllegatoAppuntamento;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.sincronizzazione.InfoAppuntamento;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "appuntamento", indexes = { @Index(name= "appuntamento", columnList = "data_creazione, data_inizio, data_fine, data_conferma, data_esito, data_stato, data_lavorazione, stato"), @Index(name = "id", columnList = "userfield") })
public class Appuntamento  extends SelezionatoAbstract implements Serializable, Comparable<Appuntamento> {
	@Transient
	private Long seq = Long.valueOf(0);
	
	public Long getSeq() {
		return seq;
	}

	public void setSeq(Long seq) {
		this.seq = seq;
	}
	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	@Transient
	private List<String> motivi = new ArrayList<String>();
	public void addMotivi(List<String> motivi) {
		this.motivi = motivi;
	}
	public String getMotivi() {
		return ApplicationUtil.detoken(motivi, ConstantsGlobal.colonSeparator);
	}
	@ManyToOne 
	private Agente agente;
	@OneToMany(mappedBy = "appuntamento")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AllegatoAppuntamento> allegati;
	@Transient
	private List<Allegato> allegato;
	@ManyToOne
	private Contatto contatto;
	@Transient
	private Date data_appuntamento;
	private Date data_conferma;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_esito;
	private Date data_fine;
	private Date data_inizio;
	private Date data_lavorazione;
	private Date data_stato;
	@ManyToOne
	private EsitoAppuntamento esito;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Boolean localizzazione = Boolean.FALSE;
	@Transient
	private Integer minTime;
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	@Column(columnDefinition = "text")
	private String note;
	@ManyToOne
	private Utente operatore;
	@Transient
	private Boolean dotted = Boolean.FALSE;
	@Transient
	private int oraMax;
	@Transient
	private int oraMin;
	private Integer stato = ConstantsGlobal.in_attesa;
	@OneToMany(mappedBy = "appuntamento", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaAppuntamento> storia;
	@ManyToOne
	private Utente utente;
	public int compareTo(Appuntamento o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Appuntamento other = (Appuntamento) obj;
		if (chiave == null) {
			if (other.chiave != null)
				return false;
		} else if (!chiave.equals(other.chiave))
			return false;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (duplicato == null) {
			if (other.duplicato != null)
				return false;
		} else if (!duplicato.equals(other.duplicato))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (seq == null) {
			if (other.seq != null)
				return false;
		} else if (!seq.equals(other.seq))
			return false;
		if (trovato == null) {
			if (other.trovato != null)
				return false;
		} else if (!trovato.equals(other.trovato))
			return false;
		return true;
	}
	public Agente getAgente() {
		return agente;
	}
	public String getAgente_denominazione() {
		return getAgente() != null ?  getAgente().getDenominazione() : "";
	}
	public String getAgente_codice() {
		return getAgente() != null ?  getAgente().getCodice() : "";
	}
	public String getOperatore_denominazione() {
		return getOperatore() != null ? getOperatore().getDenominazione() : "";
	}
	public String getOperatore_codice() {
		return getOperatore() != null ? getOperatore().getUsername() : "";
	}
	public List<AllegatoAppuntamento> getAllegati() {
		return allegati;
	}
	public List<Allegato> getAllegato() {
		if(allegato==null)
			allegato = new ArrayList<Allegato>();
		return allegato;
	}
	public int getAllegatoSize() {
		return ApplicationUtil.size(allegato);
	}
	public Boolean getCentrato() {
		GregorianCalendar gc_a = ApplicationUtil.getGregorianCalendar();
		gc_a.setTime(getData_inizio());
		GregorianCalendar gc_b = ApplicationUtil.getGregorianCalendar();
		gc_b.setTime(getData_fine());
		int oraInizio = gc_a.get(GregorianCalendar.HOUR_OF_DAY);
		int oraFine = gc_b.get(GregorianCalendar.HOUR_OF_DAY);
		boolean inizio_ok = oraInizio <=  oraMax;
		boolean fine_ok = oraFine >=  oraMin;
		return inizio_ok && fine_ok;
	}
	public String getColore() {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsGlobal.color_in_attesa;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsGlobal.color_in_pagamento;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsGlobal.color_pagato;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsGlobal.color_in_storno;
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsGlobal.color_stornato;
		} else {
			return ConstantsGlobal.color_in_attesa;
		} 
	}
	public Contatto getContatto() {
		return contatto;
	}
	public Date getData_appuntamento() {
		if(data_appuntamento==null)
			data_appuntamento = getData_inizio();
		return data_appuntamento;
	}
	public Date getData_conferma() {
		return data_conferma;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_esito() {
		return data_esito;
	}
	public Date getData_fine() {
		return ApplicationUtil.aggiustaDataT(data_fine, false);
	}
	public Date getData_inizio() {
		return ApplicationUtil.aggiustaDataT(data_inizio, true);
	}
	public Date getData_lavorazione() {
		return data_lavorazione;
	}
	public Date getData_stato() {
		return data_stato;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getContatto().getDenominazione());
	}
	public String getDenominazioneIndirizzo() {
		String den = getDenominazione();
		String ind = getIndirizzo();
		if(ApplicationUtil.isNotBlank(ind)) {
			den = den + " - " + ind;
		}
		return den;
	}
	public EsitoAppuntamento getEsito() {
		return esito;
	}
	public String getEsito_appuntamento() {
		return  getEsitoEtichetta();
	}
	public String getEsitoEtichetta() {
		return  getEsito() != null ? getEsito().getEtichetta() : "";
	}
	public String getEsitoEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(getEsitoEtichetta()));
		if(ApplicationUtil.isNotBlank(getEsitoCodice())) {
			sb.append(" (" + getEsitoCodice() + ")");
		}
		return sb.toString();
	}
	public String getEsitoCodice() {
		return getEsito() != null ? getEsito().getCodice() : "";
	}
	public String getCodice_appuntamento() {
		return getEsitoCodice();
	}
	public Long getId() {
		return id;
	}
	public Object getChiaveO(String tipo) {
		if(tipo != null && !tipo.equals(ConstantsMap.appuntamento_id)) {
			if(tipo.equals(ConstantsMap.anagrafica_telefono_fisso)) {
				return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getTelefono_fisso());
			} else if(tipo.equals(ConstantsMap.anagrafica_telefono_mobile)) {
				return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getTelefono_mobile());
			} else if(tipo.equals(ConstantsMap.anagrafica_telefono_fax)) {
				return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getTelefono_fax());
			} else if(tipo.equals(ConstantsMap.anagrafica_telefono_altro)) {
				return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getTelefono_altro());
			} else if(tipo.equals(ConstantsMap.anagrafica_email)) {
				return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getEmail());
			} else if(tipo.equals(ConstantsMap.anagrafica_link)) {
				return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getLink());
			} else if(tipo.equals(ConstantsMap.anagrafica_codice_fiscale)) {
				return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getCodice_fiscale());
			} else if(tipo.equals(ConstantsMap.anagrafica_partita_iva)) {
				return ApplicationUtil.trimToNull(getContatto().getAnagrafica().getPartita_iva());
			}
		} else {
			return getId();
		}
		return null;
	}
	public String getIdentificativo() {
		return id != null ? id.toString() : null;
	}
	public String getIndirizzo() {
		return getLocalizzazione() ?  ApplicationUtil.trimToEmpty(getContatto().getAnagrafica().getIndirizzo_operativo().getComune_completo()) : ApplicationUtil.trimToEmpty(getContatto().getAnagrafica().getIndirizzo_legale().getComune_completo());
	}
	public Boolean getLocalizzazione() {
		if(localizzazione==null)
			return false;
		return localizzazione;
	}
	public String getLocalizzazioneString() {
		if(getLocalizzazione()) {
			return ConstantsLanguage.INDIRIZZO_OPERATIVO.stampa();
		} else {
			return ConstantsLanguage.INDIRIZZO_LEGALE.stampa();
		}
	}
	public Integer getMinDurata() {
		if(ApplicationUtil.ltZero(minTime)) {
			return ApplicationUtil.timeDiff(getData_inizio(), getData_fine());
		}
		return minTime;
	}
	public Integer getMinTime() {
		return minTime;
	}

	public String getNote() {
		return note;
	}
	public String getNota() {
		return note;
	}
	public Utente getOperatore() {
		return operatore;
	}
	public int getOraMax() {
		return oraMax;
	}
	public int getOraMin() {
		return oraMin;
	}
	public Boolean getPassato() {
		return getData_fine().before(ApplicationUtil.date());		
	}
	public Integer getStato() {
		return stato;
	}
	public String getStatoTesto() {
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_attesa)) {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			return ConstantsLanguage.STATO_IN_PAGAMENTO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.pagato)) {
			return ConstantsLanguage.STATO_PAGATO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.in_storno)) {
			return ConstantsLanguage.STATO_IN_STORNO.stampa();
		} else if(ApplicationUtil.equals(stato, ConstantsGlobal.stornato)) {
			return ConstantsLanguage.STATO_STORNATO.stampa();
		} else {
			return ConstantsLanguage.STATO_IN_ATTESA.stampa();
		} 
	}
	public List<StoriaAppuntamento> getStoria() {
		return storia;
	}
	public String getTime() {
		return ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), getData_inizio()) + "-"  + ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), getData_fine());
	}
	public String getTop() {
		GregorianCalendar gc_a = ApplicationUtil.getGregorianCalendar();
		gc_a.setTime(getData_inizio());
		GregorianCalendar gc_b = ApplicationUtil.getGregorianCalendar();
		gc_b.setTime(getData_fine());
		int ooI = gc_a.get(GregorianCalendar.HOUR_OF_DAY);
		int omI = gc_a.get(GregorianCalendar.MINUTE);
		int ore = (ooI >= oraMin ? ooI : oraMin) -  oraMin;
		int minuti = (ooI >= oraMin ? omI : 0);
		int minuti_totali = (ore * 60) + minuti;
		int top = Math.round(Integer.valueOf(minuti_totali).floatValue() / (60 / ConstantsGlobal.calendarSize));
		return String.valueOf(top);
	}
	public Utente getUtente() {
		return utente;
	}
	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}
	public String getUtente_codice() {
		return getUtente() != null ? getUtente().getUsername() : "";
	}
	public String getWidth() {
		GregorianCalendar gc_a = ApplicationUtil.getGregorianCalendar();
		gc_a.setTime(getData_inizio());
		GregorianCalendar gc_b = ApplicationUtil.getGregorianCalendar();
		gc_b.setTime(getData_fine());
		int ooI = gc_a.get(GregorianCalendar.HOUR_OF_DAY);
		int omI = gc_a.get(GregorianCalendar.MINUTE);
		int ooF = gc_b.get(GregorianCalendar.HOUR_OF_DAY);
		int omF = gc_b.get(GregorianCalendar.MINUTE);
		int ore_a = (ooI > oraMin ? ooI : oraMin);
		int minuti_a = (ooI > oraMin ? omI : 0);
		int ore_b = (ooF > oraMax ?  oraMax : ooF);
		int minuti_b = (ooF > oraMax ? 60 : omF);
		int minuti_totali_a = (ore_a * 60) + minuti_a;
		int minuti_totali_b = (ore_b * 60) + minuti_b;
		int top = Math.round(Integer.valueOf(minuti_totali_b - minuti_totali_a).floatValue() / (60 / ConstantsGlobal.calendarSize));
		if(top < (ConstantsGlobal.calendarSize/2)) {
			top = ConstantsGlobal.calendarSize/2;
			dotted = Boolean.TRUE;
		}
		return String.valueOf(top);
	}
	public int getZetaIndex() {
		int zindex = ConstantsGlobal.zetaIndex;
		int i = esito != null ? esito.getOrdinamento().intValue() : zindex;
		int zi = zindex - i;
		return (zi > 0 ? zi : 1);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chiave == null) ? 0 : chiave.hashCode());
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((duplicato == null) ? 0 : duplicato.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((seq == null) ? 0 : seq.hashCode());
		result = prime * result + ((trovato == null) ? 0 : trovato.hashCode());
		return result;
	}
	public void setAgente(Agente agente) {
		this.agente = agente;
	}
	public void setAllegati(List<AllegatoAppuntamento> allegati) {
		this.allegati = allegati;
	}
	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}
	public void setData_appuntamento(Date data_appuntamento) {
		this.data_appuntamento = data_appuntamento;
	}
	public void setData_conferma(Date dataConferma) {
		this.data_conferma = dataConferma;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_esito(Date data_esito) {
		this.data_esito = data_esito;
	}
	public void setData_fine(Date dataFine) {
		data_fine = ApplicationUtil.settaOrarioAData(getData_appuntamento(), dataFine);
	}
	public void setData_inizio(Date dataInizio) {
		data_inizio = ApplicationUtil.settaOrarioAData(getData_appuntamento(), dataInizio);
	}
	public void setData_lavorazione(Date data_lavorazione) {
		this.data_lavorazione = data_lavorazione;
	}
	public void setData_stato(Date data_stato) {
		this.data_stato = data_stato;
	}
	public void setEsito(EsitoAppuntamento esito) {
		this.esito = esito;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setLocalizzazione(Boolean localizzazione) {
		this.localizzazione = localizzazione;
	}
	public void setMinDurata(Integer minTime) {
		data_fine = ApplicationUtil.addMinutes(getData_inizio(), minTime);
	}
	public void setMinTime(Integer minTime) {
		this.minTime = minTime;
	}
	
	public void setNote(String note) {
		this.note = note;
	}
	public void setOperatore(Utente operatore) {
		this.operatore = operatore;
	}
	public void setOraInizio(String ora) {
		GregorianCalendar cal = ApplicationUtil.getGregorianCalendar();
		cal.setTime(getData_appuntamento());
		if(ora != null) {
			cal.set(GregorianCalendar.HOUR_OF_DAY, Integer.parseInt(ora));
		} 
		cal.set(GregorianCalendar.MINUTE, 0);
		cal.set(GregorianCalendar.SECOND, 0);
		cal.set(GregorianCalendar.MILLISECOND, 0);
		data_inizio = cal.getTime();
		data_fine = ApplicationUtil.addMinutes(cal.getTime(), getMinDurata());
	}
	public void imposta(Date data) {
		data = ApplicationUtil.aggiustaDataT(data, true);
		int diff = ApplicationUtil.timeDiff(getData_inizio(), getData_fine());
		data_appuntamento = data;
		data_inizio = data;
		Date f1 = ApplicationUtil.fineGiornata(data);
		Date f2 = ApplicationUtil.addMinutes(data, diff);
		if(f2.after(f1)) {
			data_fine = f1;
		} else {
			data_fine = f2;
		}
	}
	public void setOraMax(int oraMax) {
		this.oraMax = oraMax;
	}
	public void setOraMin(int oraMin) {
		this.oraMin = oraMin;
	}
	public void setStato(Integer stato) {
		this.stato = stato;
	}
	public void setStoria(List<StoriaAppuntamento> storia) {
		this.storia = storia;
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public List<SelectItem> getTempi() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		List<Integer> tempi = ConstantsMap.getTimeAppuntamento();
		Integer min = getMinDurata();
		if(!tempi.contains(min)) {
			tempi.add(min);
		}
		Collections.sort(tempi);
		for(Integer c : tempi) {
			sis.add(new SelectItem(c, c + ""));
		}
		return sis;
	}
	public Boolean getDotted() {
		return dotted;
	}
	public void setDotted(Boolean dotted) {
		this.dotted = dotted;
	}
	public String getNotaShort() {
		return ApplicationUtil.tronca(note);
	}
	@Transient
	private Boolean trovato = Boolean.FALSE;
	public Boolean getTrovato() {
		return trovato;
	}
	@Transient
	private Boolean duplicato = Boolean.FALSE;
	@Transient
	private String chiave;
	
	public Object getChiaveL(String tipo) {
		String s = ApplicationUtil.trimToNull(getChiave());
		if((tipo == null || tipo.equals(ConstantsMap.appuntamento_id)) && s != null) {
			Long l = null;
			try {
				l = Long.parseLong(s);
			} catch (Exception nfe) {
			}
			return l;
		}
		return s;
	}
	public String getChiave() {
		return chiave;
	}
	public void setChiave(String chiave) {
		this.chiave = chiave;
	}
	public Boolean getDuplicato() {
		return duplicato;
	}
	public void setDuplicato(Boolean duplicato) {
		this.duplicato = duplicato;
	}
	public void setTrovato(Boolean trovato) {
		this.trovato = trovato;
	}
	public Time getOra_inizio() {
		return ApplicationUtil.dateToTime(getData_inizio());
	}
	public Time getOra_fine() {
		return ApplicationUtil.dateToTime(getData_fine());
	}
	
	public InfoAppuntamento getSincronia() {
		if(Chiamate.getCodaSincroniaAppuntamenti().contains(getId())) {
			return Chiamate.getCodaSincroniaAppuntamenti().get(getId());
		} 
		return null;
	}
	public String getBloccatoS() {
		return getBloccato() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	private Boolean bloccato = Boolean.FALSE;
	public Boolean getBloccato() {
		if(bloccato == null) {
			bloccato = Boolean.FALSE;
		}
		return bloccato;
	}
	public void setBloccato(Boolean bloccato) {
		this.bloccato = bloccato;
	}
	
}