package it.callite.model;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;


@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "busta", indexes = { @Index(name= "busta", columnList = "data_creazione, nome") })
public class Busta  extends SelezionatoAbstract implements Serializable, Comparable<Busta> {
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	private String nome;
	private Date data_creazione = ApplicationUtil.date();
	
	public Date getData_creazione() {
		return data_creazione;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getNome() {
		return nome;
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	public String getNomeFileS() {
		String n = ApplicationUtil.trimToNull(nome);
		if(n != null) {
			return n + "." + ApplicationUtil.getExtensionN(nomeFile);
		} else {
			return nomeFile;
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Busta other = (Busta) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	@Lob
	private byte[] content;

	public byte[] getContent() {
		return content;
	}

	public void setContent(byte[] content) {
		this.content = content;
	}
	
	@Column(columnDefinition = "text")
	private String nomeFile;

	public String getNomeFile() {
		return nomeFile;
	}

	public void setNomeFile(String nomeFile) {
		this.nomeFile = nomeFile;
	}

	public int compareTo(Busta o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	@OneToMany
	@JoinColumn(name = "busta_id", nullable = true, updatable=true, insertable=true)
	private List<RmailTemplate> rmail;
	
	
	@OneToMany(mappedBy = "busta", fetch = FetchType.LAZY, cascade =  CascadeType.ALL)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<BustaStore> bustaStore;


	public List<BustaStore> getBustaStore() {
		return bustaStore;
	}

	public void setBustaStore(List<BustaStore> bustaStore) {
		this.bustaStore = bustaStore;
	}

	public List<RmailTemplate> getRmail() {
		return rmail;
	}

	public void setRmail(List<RmailTemplate> rmail) {
		this.rmail = rmail;
	}
	
}