package it.callite.model;
import java.io.Serializable;
import java.sql.Time;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.ValidatorException;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Lob;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.collections4.ListUtils;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.message.Message;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.PannelloReport;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "campagna", indexes = { @Index(name= "campagna", columnList = "data_creazione, tipo, nome, inverti, data_inizio, data_fine, week_inizio, week_fine, ora_inizio, ora_fine, stato, priorita, ordine") })
public class Campagna extends SelezionatoAbstract implements Serializable, Comparable<Campagna>  {
	private Long week_fine;
	private Long week_inizio;
	private String suono;
	private String codice;
	
	private String invoke_link;
	private String invoke_username;
	private String invoke_password;
	
	public String getCodice() {
		return ApplicationUtil.normalizeToEmpty(codice);
	}
	public void setCodice(String codice) {
		this.codice = ApplicationUtil.normalizeToEmpty(codice);
	}

	private Boolean ritardoT = false;
	
	public Long getWeek_fine() {
		return week_fine;
	}
	public void setWeek_fine(Long week_fine) {
		this.week_fine = week_fine;
	}
	public Long getWeek_inizio() {
		return week_inizio;
	}
	public void setWeek_inizio(Long week_inizio) {
		this.week_inizio = week_inizio;
	}
	private String query_proprieta;
	@Column(columnDefinition = "text")
	private String query_valore;
	public String getQuery_proprieta() {
		return ApplicationUtil.trimToNull(query_proprieta);
	}
	public void setQuery_proprieta(String query_proprieta) {
		this.query_proprieta = ApplicationUtil.trimToNull(query_proprieta);
	}
	public String getQuery_valore() {
		return ApplicationUtil.trimToNull(query_valore);
	}
	public void setQuery_valore(String query_valore) {
		this.query_valore = ApplicationUtil.trimToNull(query_valore);
	}
	@Column(columnDefinition = "text")
	private String modulo;
	@Column(columnDefinition = "text")
	private String aiuto;
	private Short aiuto_velocita;
	@ManyToOne(fetch = FetchType.LAZY)
	private Autodialer autodialer;
	private Integer tempo = 0;
	private Boolean ferma_timer = false;
	public Boolean getFerma_timer() {
		return ApplicationUtil.isTrue(ferma_timer);
	}
	public void setFerma_timer(Boolean ferma_timer) {
		this.ferma_timer = ferma_timer;
	}
	public Integer getTempo() {
		if(tempo == null) {
			tempo = 0;
		}
		return tempo;
	}
	public void setTempo(Integer tempo) {
		this.tempo = tempo;
	}
	@ManyToOne
	private EsitoChiamata esito;
	public EsitoChiamata getEsito() {
		return esito;
	}
	public void setEsito(EsitoChiamata esito) {
		this.esito = esito;
	}
	@OneToMany(mappedBy = "campagna")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Opposizione> opposizioni;
	@OneToMany(mappedBy = "campagna")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AutoProcessEmail> autoEmails;
	
	@OneToMany(mappedBy = "campagna")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AutoProcessRmail> autoRmails;
	
	private Integer avvio_automatico = 3;
	
	private Boolean powerdialer = Boolean.TRUE;
	private Boolean seleziona_liste = Boolean.FALSE;
	
	@OneToMany(mappedBy = "campagna")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AutoProcessTelecom> autoTelecoms;
	@Column(columnDefinition = "text")
	private String callers;
	
	@Column(columnDefinition = "text")
	private String musicclass;
	
	@Column(columnDefinition = "text")
	private String inbound;
	@OneToMany(mappedBy = "campagna")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contatto> contatti;
	private Date data_creazione = ApplicationUtil.date();
	@Transient
	private String criterio;
	public List<Opposizione> getOpposizioni() {
		return opposizioni;
	}
	public void setOpposizioni(List<Opposizione> opposizioni) {
		this.opposizioni = opposizioni;
	}
	public String getCriterio() {
		return ApplicationUtil.trimToEmpty(criterio);
	}
	public void setCriterio(String criterio) {
		this.criterio = criterio;
	}
	public void criterio(String tipo, String proprieta, Integer limite, String query_proprieta, String query_valore, Boolean nuovo, Boolean producer) {
		criterio = ApplicationUtil.criterio(tipo, proprieta, limite, query_proprieta, query_valore, nuovo, producer);
	}
	public void criterio(String tipo) {
		criterio = ApplicationUtil.criterio(tipo);
	}
	private Date data_fine;
	private Date data_inizio;
	@Column(columnDefinition = "text")
	private String descrizione;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "email_campagna", joinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="email_id", nullable=false, updatable=true, insertable=true)    )
	private List<EmailTemplate> email;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esiti_campagna", joinColumns=@JoinColumn(name="campagna_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true)    )
	private List<EsitoArchiviazione> esiti_archiviazione;
	private Integer estrazione_lista = 1;
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Boolean inserisci_dati = Boolean.TRUE;
	private Boolean invio_email = Boolean.FALSE;
	private Boolean invio_rmail = Boolean.FALSE;
	private Boolean invio_telecom = Boolean.FALSE;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "rmail_campagna", joinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="rmail_id", nullable=false, updatable=true, insertable=true)    )
	private List<RmailTemplate> rmail;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "campagna_lista", joinColumns=@JoinColumn(name="campagna_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="liste_id", nullable=false, updatable=true, insertable=true)    )
	@OrderBy("stato ASC, priorita DESC")
	private List<Lista> liste;
	
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "campagna_risposta", joinColumns=@JoinColumn(name="campagna_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="risposta_id", nullable=false, updatable=true, insertable=true)    )
	private List<Risposta> risposte;
	
	public List<Risposta> getRisposte() {
		return risposte;
	}
	public void setRisposte(List<Risposta> risposte) {
		this.risposte = risposte;
	}

	private Integer modalita_scelta;
	private Integer agenti_scelta;
	private Boolean modifica_dati = Boolean.TRUE;
	
	@Column(nullable = false)
	private String nome;
	
	private Time ora_fine;
	private Time ora_inizio;
	private String ordine;
	private String proprieta;
	private Integer limite = 1;
	public String getProprieta() {
		return proprieta;
	}
	public void setProprieta(String proprieta) {
		this.proprieta = proprieta;
	}
	public Integer getLimite() {
		if(ApplicationUtil.ltZero(limite)) {
			limite = 1;
		}
		return limite;
	}
	public void setLimite(Integer limite) {
		this.limite = limite;
	}
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "campagne_utente", joinColumns=@JoinColumn(name="campagna_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="utente_id", nullable=false, updatable=true, insertable=true)    )
	private List<Utente> partner;
	private Boolean prenotabile_esterno = Boolean.FALSE;
	private Boolean prenotabile_interno = Boolean.TRUE;
	@OneToMany(mappedBy = "campagna")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Prenotazione> prenotazioni;
	private Integer priorita = ConstantsGlobal.defaultPriorita;
	@Transient
	private List<ProdottiInVendita> prodottiDaAggiungere;
	//@NotAudited
	@OneToMany(mappedBy = "id.campagna")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<ProdottiInVendita> prodottiInVendita;
	
	private Integer registrazione_locale = ConstantsMap.REC_NO;
	private Integer registrazione_remota = ConstantsMap.REC_NO;
	@Transient
	private PannelloReport report;
	private Boolean riepilogo_appuntamenti = Boolean.FALSE;
	private Boolean riepilogo_calendario = Boolean.FALSE;
	private Boolean riepilogo_contratti = Boolean.FALSE;
	private Boolean riepilogo_registrazioni = Boolean.FALSE;
	private Boolean riprogramma_privato = Boolean.TRUE;
	private Boolean riprogramma_pubblico = Boolean.TRUE; 
	private Boolean riprogramma_team = Boolean.FALSE;
	private Boolean riprogramma_esito = Boolean.TRUE;
	private Boolean scheda_dati = Boolean.TRUE;
	@Transient
	private AtomicInteger seq = new AtomicInteger();
	
	private Integer stato = ConstantsGlobal.stato_attivo;
	private Integer tasto_chiudi = 0;
	private Integer tasto_indietro = 2;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "team_campagna", joinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="team_id", nullable=false, updatable=true, insertable=true)    )
	private List<Team> team;
	private Short telefono = 1;
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "telecom_campagna", joinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="telecom_id", nullable=false, updatable=true, insertable=true)    )
	private List<TelecomTemplate> telecom;
	private Integer tipo;
	private String rem_prefisso;
	private String add_prefisso;
	private String append_prefisso;
	public String getAppend_prefisso() {
		return ApplicationUtil.normalizeToNull(append_prefisso);
	}
	public void setAppend_prefisso(String append_prefisso) {
		this.append_prefisso = ApplicationUtil.normalizeToNull(append_prefisso);
	}
	private Boolean consenso_telefono = Boolean.TRUE;
	
	public Boolean getConsenso_telefono() {
		if(consenso_telefono == null) {
			consenso_telefono = Boolean.TRUE;
		}
		return consenso_telefono;
	}
	public void setConsenso_telefono(Boolean consenso_telefono) {
		this.consenso_telefono = consenso_telefono;
	}
	public String getAdd_prefisso() {
		return ApplicationUtil.normalizeToEmpty(add_prefisso);
	}
	public String getRem_prefisso() {
		return ApplicationUtil.normalizeToEmpty(rem_prefisso);
	}
	public void setAdd_prefisso(String add_prefisso) {
		this.add_prefisso = ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, add_prefisso));
	}
	public void setRem_prefisso(String rem_prefisso) {
		this.rem_prefisso = ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, rem_prefisso));
	}
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "zona_campagna", joinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="zona_id", nullable=false, updatable=true, insertable=true)    )
	private List<Zona> zone;
	public int compareTo(Campagna o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Campagna other = (Campagna) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public String getAiuto() {
		return aiuto;
	}
	public Short getAiuto_velocita() {
		return aiuto_velocita;
	}
	
	public Boolean getStatoAttiva() {
		return ApplicationUtil.calcolaIntervallo(getInverti(), getData_inizio(), getData_fine(), getWeek_inizio(), getWeek_fine(), getTime_inizio(), getTime_fine());
	}
	public String getStatoAttivaT() {
		return getStatoAttiva() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public Autodialer getAutodialer() {
		return autodialer;
	}
	public List<AutoProcessEmail> getAutoEmails() {
		return autoEmails;
	}
	
	public List<AutoProcessRmail> getAutoRmails() {
		return autoRmails;
	}
	
	public Boolean getAutostart_reg_locale() {
		return ApplicationUtil.gtZero(getRegistrazione_locale()) && getRegistrazione_locale().equals(ConstantsMap.REC_AUTO);
	}
	public Boolean getAutostart_reg_remota() {
		return ApplicationUtil.gtZero(getRegistrazione_remota()) && getRegistrazione_remota().equals(ConstantsMap.REC_AUTO);
	}
	
	public List<AutoProcessTelecom> getAutoTelecoms() {
		return autoTelecoms;
	}
	public String getCallers() {
		return ApplicationUtil.trimToNull(callers);
	}
	public List<Contatto> getContatti() {
		return contatti;
	}
	
	public String getMusicclass() {
		return ApplicationUtil.normalizeToNull(musicclass);
	}
	public void setMusicclass(String musicclass) {
		this.musicclass = ApplicationUtil.normalizeToNull(musicclass);
	}
	private Boolean agente_notizia = false;
	private Boolean agente_messaggio = false;
	private Boolean proprietario_notizia = false;
	private Boolean proprietario_messaggio = false;
	private Boolean email_notifica = false;
	
	
	public Boolean getAgente_notizia() {
		return agente_notizia;
	}
	public void setAgente_notizia(Boolean agente_notizia) {
		this.agente_notizia = agente_notizia;
	}
	public Boolean getAgente_messaggio() {
		return agente_messaggio;
	}
	public void setAgente_messaggio(Boolean agente_messaggio) {
		this.agente_messaggio = agente_messaggio;
	}
	public Boolean getProprietario_notizia() {
		return proprietario_notizia;
	}
	public void setProprietario_notizia(Boolean proprietario_notizia) {
		this.proprietario_notizia = proprietario_notizia;
	}
	public Boolean getProprietario_messaggio() {
		return proprietario_messaggio;
	}
	public void setProprietario_messaggio(Boolean proprietario_messaggio) {
		this.proprietario_messaggio = proprietario_messaggio;
	}
	
	public Boolean getEmail_notifica() {
		return email_notifica;
	}
	public void setEmail_notifica(Boolean email_notifica) {
		this.email_notifica = email_notifica;
	}
	public Campagna getCopia() {
		//BeanUtils.copyProperties(this, copia);
		Campagna copia = new Campagna();
		copia.setNome(this.getNome() + " " + ConstantsLanguage.COPY.stampa());
		copia.setQuery_proprieta(this.getQuery_proprieta());
		copia.setQuery_valore(this.getQuery_valore());
		copia.setDescrizione(this.getDescrizione());
		copia.setProprietario_notizia(this.getProprietario_notizia());
		copia.setProprietario_messaggio(this.getProprietario_messaggio());
		copia.setEmail_notifica(this.getEmail_notifica());
		copia.setAgente_notizia(this.getAgente_notizia());
		copia.setAgente_messaggio(this.getAgente_messaggio());
		copia.setAiuto(this.getAiuto());
		copia.setAiuto_velocita(this.getAiuto_velocita());
		copia.setTelefono(this.getTelefono());
		copia.setData_fine(this.getData_fine());
		copia.setData_inizio(this.getData_inizio());
		copia.setAutodialer(this.getAutodialer());
		copia.setEsito(this.getEsito());
		copia.setTempo(this.getTempo());
		copia.setFerma_timer(this.getFerma_timer());
		copia.setStato(this.getStato());
		copia.setInvio_email(this.getInvio_email());
		copia.setInvio_rmail(this.getInvio_rmail());
		copia.setInvio_telecom(this.getInvio_telecom());
		copia.setContent(this.getContent());
		copia.setPrenotabile_esterno(this.getPrenotabile_esterno());
		copia.setPriorita(this.getPriorita());
		copia.setCodice(this.getCodice());
		copia.setPrenotabile_interno(this.getPrenotabile_interno());
		copia.setRegistrazione_locale(this.getRegistrazione_locale());
		copia.setRegistrazione_remota(this.getRegistrazione_remota());
		copia.setLimite(this.getLimite());
		copia.setProprieta(this.getProprieta());
		copia.setModifica_dati(this.getModifica_dati());
		copia.setInserisci_dati(this.getInserisci_dati());
		copia.setScheda_dati(this.getScheda_dati());
		copia.setRiepilogo_contratti(this.getRiepilogo_contratti());
		copia.setModalita_scelta(this.getModalita_scelta());
		copia.setAgenti_scelta(this.getAgenti_scelta());
		copia.setRiepilogo_appuntamenti(this.getRiepilogo_appuntamenti());
		copia.setEstrazione_lista(this.getEstrazione_lista());
		copia.setRiepilogo_calendario(this.getRiepilogo_calendario());
		copia.setRiepilogo_registrazioni(this.getRiepilogo_registrazioni());
		copia.setRiprogramma_team(this.getRiprogramma_team());
		copia.setRiprogramma_esito(this.getRiprogramma_esito());
		copia.setRiprogramma_privato(this.getRiprogramma_privato());
		copia.setRiprogramma_pubblico(this.getRiprogramma_pubblico());
		copia.setAvvio_automatico(this.getAvvio_automatico());
		copia.setPowerdialer(this.getPowerdialer());
		copia.setSeleziona_liste(this.getSeleziona_liste());
		copia.setTasto_indietro(this.getTasto_indietro());
		copia.setTasto_chiudi(this.getTasto_chiudi());
		copia.setTipo(this.getTipo());
		copia.setOra_inizio(this.getOra_inizio());
		copia.setOra_fine(this.getOra_fine());
		copia.setInverti(this.getInverti());
		copia.setOrdine(this.getOrdine());
		copia.setModulo(this.getModulo());
		copia.setPdf(this.getPdf());
		copia.setCallers(this.getCallers());
		copia.setMusicclass(this.getMusicclass());
		copia.setInbound(this.getInbound());
		copia.setConsenso_telefono(this.getConsenso_telefono());
		copia.setAdd_prefisso(this.getAdd_prefisso());
		copia.setRem_prefisso(this.getRem_prefisso());
		copia.setAppend_prefisso(this.getAppend_prefisso());
		copia.setOrdine_transizioni(this.getOrdine_transizioni());
		copia.setLatitudine(this.getLatitudine());
		copia.setLongitudine(this.getLongitudine());
		copia.setWeek_fine(this.getWeek_fine());
		copia.setWeek_inizio(this.getWeek_inizio());
		copia.setSuono(this.getSuono());
		copia.setRitardoT(this.getRitardoT());
		copia.setInvoke_link(this.getInvoke_link());
		copia.setInvoke_username(this.getInvoke_username());
		copia.setInvoke_password(this.getInvoke_password());
		return copia;
	}

	public Boolean getRitardoT() {
		if(ritardoT == null) {
			ritardoT = false;
		}
		return ritardoT;
	}
	public void setRitardoT(Boolean ritardoT) {
		this.ritardoT = ritardoT;
	}
	public String getSuono() {
		return suono;
	}
	public void setSuono(String suono) {
		this.suono = suono;
	}
	public String getModulo() {
		return ApplicationUtil.trimToNull(modulo);
	}
	public void setModulo(String modulo) {
		this.modulo = ApplicationUtil.trimToNull(modulo);
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_fine() {
		return ApplicationUtil.aggiustaDataT(data_fine, false);
	}
	public Date getData_inizio() {
		return ApplicationUtil.aggiustaDataT(data_inizio, true);
	}
	public String getDenominazione() {
		return getDenominazioneTipo() + " (" + getDescrizioneStato() +(getStatoAttiva() ? "" : "*") +")";
	}
	
	public String getDenominazioneTipo() {
		String codice = getCodice();
		String tipo = getDescrizioneTipo();
		if(ApplicationUtil.isBlank(tipo)) {
			return (ApplicationUtil.isNotBlank(codice) ? (codice + " - ") : "") + ApplicationUtil.trimToEmpty(nome);
		} else {
			return (ApplicationUtil.isNotBlank(codice) ? (codice + " - ") : "") + ApplicationUtil.trimToEmpty(nome) + " - " + tipo;
		}
	}
	
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public String getDescrizioneStato() {
		return Message.addCatalogText("ts_stato." + stato);
	}
	public String getDescrizioneTipo() {
		if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_residenziale)) {
			return ConstantsLanguage.TIPO_RESIDENZIALE.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_business)) {
			return ConstantsLanguage.TIPO_BUSINESS.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.tipo_misto)) {
			return ConstantsLanguage.TIPO_MISTO.stampa();
		} else {
			return "";
		} 
	}
	public List<EmailTemplate> getEmail() {
		return email;
	}
	public List<EsitoArchiviazione> getEsiti_archiviazione() {
		return esiti_archiviazione;
	}
	public Integer getEstrazione_lista() {
		return estrazione_lista;
	}
	
	public Long getId() {
		return id;
	}
	public Boolean getInserisci_dati() {
		return inserisci_dati;
	}
	public Boolean getInvio_email() {
		return invio_email;
	}
	
	public Boolean getInvio_rmail() {
		return invio_rmail;
	}
	
	public Boolean getInvio_telecom() {
		return invio_telecom;
	}
	public List<RmailTemplate> getRmail() {
		return rmail;
	}
	public List<Lista> getListe() {
		if (liste == null)
			liste = new ArrayList<Lista>();
		return liste;
	}
	public Integer getModalita_scelta() {
		if(modalita_scelta == null) {
			modalita_scelta = ConstantsMap.agenti_tutti;
		}
		return modalita_scelta;
	}
	public Integer getAgenti_scelta() {
		if(agenti_scelta == null) {
			agenti_scelta = 0;
		}
		return agenti_scelta;
	}
	public void setAgenti_scelta(Integer agenti_scelta) {
		this.agenti_scelta = agenti_scelta;
	}
	public Boolean getModifica_dati() {
		return modifica_dati;
	}
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	
	public String getNome() {
		return nome;
	}

	public Date getOra_fine() {
		return ApplicationUtil.timeToDate(ora_fine);
	}
	public Date getOra_inizio() {
		return  ApplicationUtil.timeToDate(ora_inizio);
	}
	private Boolean inverti = false;
	public Boolean getInverti() {
		if(inverti == null) {
			inverti = false;
		}
		return inverti;
	}
	public void setInverti(Boolean inverti) {
		this.inverti = inverti;
	}
	public String getOrdine() {
		return ApplicationUtil.trimToEmpty(ordine);
	}
	public Boolean getOrdineA() {
		return ConstantsMap.getOrdineList().contains(ApplicationUtil.trimToEmpty(ordine));
	}
	public List<Utente> getPartner() {
		return partner;
	}
	public Boolean getPrenotabile_esterno() {
		return prenotabile_esterno;
	}
	public Boolean getPrenotabile_interno() {
		return prenotabile_interno;
	}
	public List<Prenotazione> getPrenotazioni() {
		return prenotazioni;
	}
	public Integer getPriorita() {
		if(ApplicationUtil.ltZero(priorita)) {
			priorita = ConstantsGlobal.defaultPriorita;
		}
		return priorita;
	}
	public List<ProdottiInVendita> getProdottiDaAggiungere() {
		return prodottiDaAggiungere;
	}
	public List<ProdottiInVendita> getProdottiInVendita() {
		return prodottiInVendita;
	}
	
	public Boolean getReg_locale() {
		return ApplicationUtil.gtZero(getRegistrazione_locale()) && (getRegistrazione_locale().equals(ConstantsMap.REC_SI) || getRegistrazione_locale().equals(ConstantsMap.REC_AUTO));
	}
	public Boolean getReg_remota() {
		return ApplicationUtil.gtZero(getRegistrazione_remota()) && (getRegistrazione_remota().equals(ConstantsMap.REC_SI) || getRegistrazione_remota().equals(ConstantsMap.REC_AUTO));
	}
	public Integer getRegistrazione_locale() {
		if(ApplicationUtil.ltZero(registrazione_locale)) {
			registrazione_locale = ConstantsMap.REC_NO;
		}
		return registrazione_locale;
	}
	public Integer getRegistrazione_remota() {
		if(ApplicationUtil.ltZero(registrazione_remota)) {
			registrazione_remota = ConstantsMap.REC_NO;
		}
		return registrazione_remota;
	}
	public PannelloReport getReport() {
		return report;
	}
	public Boolean getRiepilogo_appuntamenti() {
		return riepilogo_appuntamenti;
	}
	public Boolean getRiepilogo_calendario() {
		return riepilogo_calendario;
	}
	public Boolean getRiepilogo_contratti() {
		return riepilogo_contratti;
	}
	public Boolean getRiepilogo_registrazioni() {
		return riepilogo_registrazioni;
	}
	public Boolean getRiprogramma() {
		return riprogramma_team || riprogramma_pubblico || riprogramma_privato;
	}
	public Boolean getRiprogramma_privato() {
		return riprogramma_privato;
	}
	public Boolean getRiprogramma_pubblico() {
		return riprogramma_pubblico;
	}
	public Boolean getRiprogramma_team() {
		return riprogramma_team;
	}
	public Boolean getScheda_dati() {
		return scheda_dati;
	}
	public AtomicInteger getSeq() {
		return seq;
	}
	
	public Integer getStato() {
		return stato;
	}
	public Integer getTasto_chiudi() {
		return tasto_chiudi;
	}
	public Integer getTasto_indietro() {
		return tasto_indietro;
	}
	public List<Team> getTeam() {
		return team;
	}
	public Short getTelefono() {
		if(ApplicationUtil.ltZero(telefono)) {
			telefono = 1;
		}
		return telefono;
	}
	public String getTelefonoTesto() {
		return "text.telefono_" + telefono;
	}

	public List<TelecomTemplate> getTelecom() {
		return telecom;
	}
	public Time getTime_fine() {
		return  ora_fine;
	}
	public Time getTime_inizio() {
		return  ora_inizio;
	}
	public Integer getTipo() {
		return tipo;
	}
	public List<Zona> getZone() {
		return zone;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAiuto(String aiuto) {
		this.aiuto = aiuto;
	}  
	public void setAiuto_velocita(Short aiuto_velocita) {
		this.aiuto_velocita = aiuto_velocita;
	}
	public void setAutodialer(Autodialer autodialer) {
		this.autodialer = autodialer;
	} 
	public void setAutoEmails(List<AutoProcessEmail> autoEmails) {
		this.autoEmails = autoEmails;
	}
	
	public void setAutoRmails(List<AutoProcessRmail> autoRmails) {
		this.autoRmails = autoRmails;
	}
	
	public Integer getAvvio_automatico() {
		return avvio_automatico;
	}
	public void setAvvio_automatico(Integer avvio_automatico) {
		this.avvio_automatico = avvio_automatico;
	}
	
	public Boolean getSeleziona_liste() {
		if(seleziona_liste == null) {
			seleziona_liste = Boolean.FALSE;
		}
		return seleziona_liste;
	}
	public void setSeleziona_liste(Boolean seleziona_liste) {
		this.seleziona_liste = seleziona_liste;
	}
	public Boolean getPowerdialer() {
		if(powerdialer == null) {
			powerdialer = Boolean.TRUE;
		}
		return powerdialer;
	}
	public void setPowerdialer(Boolean powerdialer) {
		this.powerdialer = powerdialer;
	}

	public void setAutoTelecoms(List<AutoProcessTelecom> autoTelecoms) {
		this.autoTelecoms = autoTelecoms;
	}
	public void setCallers(String callers) {
		this.callers = ApplicationUtil.aggiustaLista(false, ConstantsGlobal.colonSeparator, callers);
	}
	public String getInbound() {
		return ApplicationUtil.trimToNull(inbound);
	}
	public void setInbound(String inbound) {
		this.inbound = ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, inbound);
	}
	public void setContatti(List<Contatto> contatti) {
		this.contatti = contatti;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_fine(Date data_fine) {
		this.data_fine = ApplicationUtil.aggiustaDataT(data_fine, false);
	}
	public void setData_inizio(Date data_inizio) {
		this.data_inizio = ApplicationUtil.aggiustaDataT(data_inizio, true);
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setEmail(List<EmailTemplate> email) {
		this.email = email;
	}
	public void setEsiti_archiviazione(List<EsitoArchiviazione> esiti_archiviazione) {
		this.esiti_archiviazione = esiti_archiviazione;
	}
	public void setEstrazione_lista(Integer estrazione_lista) {
		this.estrazione_lista = estrazione_lista;
	}
	
	public void setId(Long id) {
		this.id = id;
	}
	public void setInserisci_dati(Boolean inserisci_dati) {
		this.inserisci_dati = inserisci_dati;
	}
	public void setInvio_email(Boolean invioEmail) {
		invio_email = invioEmail;
	}
	
	public void setInvio_rmail(Boolean invio_rmail) {
		this.invio_rmail = invio_rmail;
	} 
	
	public void setInvio_telecom(Boolean invio_telecom) {
		this.invio_telecom = invio_telecom;
	}
	public void setRmail(List<RmailTemplate> rmail) {
		this.rmail = rmail;
	}
	public void setListe(List<Lista> liste) {
		this.liste = liste;
	}
	public boolean isLogo() {
		return content != null && content.length > 0;
	}
	@Lob
	private byte[] content;
	public Byte[] getFileP() {
		if(isFile()) {
			return ApplicationUtil.toObject(pdf);
		}
		return null;
	}
	
	public boolean isFile() {
		return pdf != null && pdf.length > 0;
	}
	
	@Lob
	private byte[] pdf;
	public byte[] getPdf() {
		return pdf;
	}
	public void setPdf(byte[] pdf) {
		this.pdf = pdf;
	}
	public byte[] getContent() {
		return content;
	}
	public void setContent(byte[] content) {
		this.content = content;
	}
	public Allegato getAllegato() {
		return isLogo() ? new Allegato("", content) : null;
	}
	public void setModalita_scelta(Integer modalita_scelta) {
		this.modalita_scelta = modalita_scelta;
	}
	public void setModifica_dati(Boolean modifica_dati) {
		this.modifica_dati = modifica_dati;
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	public void setOra_fine(Date ora_fine) {
		this.ora_fine = ApplicationUtil.dateToTime(ora_fine);
	}
	public void setOra_inizio(Date ora_inizio) {
		this.ora_inizio = ApplicationUtil.dateToTime(ora_inizio);
	}
	public void setOrdine(String ordine) {
		this.ordine = ApplicationUtil.trimToNull(ordine);
	}
	public void setPartner(List<Utente> partner) {
		this.partner = partner;
	}
	public void setPrenotabile_esterno(Boolean prenotabile_esterno) {
		this.prenotabile_esterno = prenotabile_esterno;
	}
	public void setPrenotabile_interno(Boolean prenotabile_interno) {
		this.prenotabile_interno = prenotabile_interno;
	}
	public void setPrenotazioni(List<Prenotazione> prenotazioni) {
		this.prenotazioni = prenotazioni;
	}
	public void setPriorita(Integer priorita) {
		this.priorita = priorita;
	}
	public void setProdottiDaAggiungere(List<ProdottiInVendita> prodottiDaAggiungere) {
		this.prodottiDaAggiungere = prodottiDaAggiungere;
	}
	public void setProdottiInVendita(List<ProdottiInVendita> prodottiInVendita) {
		this.prodottiInVendita = prodottiInVendita;
	}
	
	public void setRegistrazione_locale(Integer registrazione_locale) {
		this.registrazione_locale = registrazione_locale;
	}
	public void setRegistrazione_remota(Integer registrazione_remota) {
		this.registrazione_remota = registrazione_remota;
	}
	public void setReport(PannelloReport report) {
		this.report = report;
	}
	public void setRiepilogo_appuntamenti(Boolean riepilogo_appuntamenti) {
		this.riepilogo_appuntamenti = riepilogo_appuntamenti;
	}
	public void setRiepilogo_calendario(Boolean riepilogo_calendario) {
		this.riepilogo_calendario = riepilogo_calendario;
	}
	public void setRiepilogo_contratti(Boolean riepilogo_contratti) {
		this.riepilogo_contratti = riepilogo_contratti;
	}
	public void setRiepilogo_registrazioni(Boolean riepilogo_registrazioni) {
		this.riepilogo_registrazioni = riepilogo_registrazioni;
	}
	public void setRiprogramma_privato(Boolean riprogramma_privato) {
		this.riprogramma_privato = riprogramma_privato;
	}
	public void setRiprogramma_pubblico(Boolean riprogramma_pubblico) {
		this.riprogramma_pubblico = riprogramma_pubblico;
	}
	public void setRiprogramma_team(Boolean riprogramma_team) {
		this.riprogramma_team = riprogramma_team;
	}
	public Boolean getRiprogramma_esito() {
		if(riprogramma_esito == null) {
			riprogramma_esito = Boolean.TRUE;
		}
		return riprogramma_esito;
	}
	public void setRiprogramma_esito(Boolean riprogramma_esito) {
		this.riprogramma_esito = riprogramma_esito;
	}
	public void setScheda_dati(Boolean scheda_dati) {
		this.scheda_dati = scheda_dati;
	}
	public void setSeq(AtomicInteger seq) {
		this.seq = seq;
	}

	public void setStato(Integer stato) {
		this.stato = stato;
	}
	public void setTasto_chiudi(Integer tasto_chiudi) {
		this.tasto_chiudi = tasto_chiudi;
	}
	public void setTasto_indietro(Integer tasto_indietro) {
		this.tasto_indietro = tasto_indietro;
	}
	public void setTeam(List<Team> team) {
		this.team = team;
	}
	public void setTelefono(Short telefono) {
		this.telefono = telefono;
	}
	public void setTelecom(List<TelecomTemplate> telecom) {
		this.telecom = telecom;
	}

	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	public void setZone(List<Zona> zone) {
		this.zone = zone;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	
	public String getInvoke_link() {
		return ApplicationUtil.normalizeToNull(invoke_link);
	}
	public void setInvoke_link(String invoke_link) {
		this.invoke_link = ApplicationUtil.normalizeToNull(invoke_link);
	}
	public String getInvoke_username() {
		return ApplicationUtil.normalizeToNull(invoke_username);
	}
	public void setInvoke_username(String invoke_username) {
		this.invoke_username = ApplicationUtil.normalizeToNull(invoke_username);
	}
	public String getInvoke_password() {
		return ApplicationUtil.normalizeToNull(invoke_password);
	}
	public void setInvoke_password(String invoke_password) {
		this.invoke_password = ApplicationUtil.normalizeToNull(invoke_password);
	}

	@OneToMany(mappedBy="sorgente")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Workflow> sorgenti; 
	@OneToMany(mappedBy="campagna")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Workflow> destinazioni;
	public List<Workflow> getSorgenti() {
		return sorgenti;
	}
	public void setSorgenti(List<Workflow> sorgenti) {
		this.sorgenti = sorgenti;
	}
	public List<Workflow> getDestinazioni() {
		return destinazioni;
	}
	public void setDestinazioni(List<Workflow> destinazioni) {
		this.destinazioni = destinazioni;
	} 
	@OneToMany
	@JoinColumn(name = "campagna_id", nullable = true, updatable=true, insertable=true)
	private List<StoriaContatto> storia_contatti;
	public List<StoriaContatto> getStoria_contatti() {
		return storia_contatti;
	}
	public void setStoria_contatti(List<StoriaContatto> storia_contatti) {
		this.storia_contatti = storia_contatti;
	}
	private String latitudine;
	private String longitudine;
	public String getLatitudine() {
		return ApplicationUtil.normalizeToNull(latitudine);
	}
	public void setLatitudine(String latitudine) {
		this.latitudine = ApplicationUtil.normalizeToNull(latitudine);
	}
	public String getLongitudine() {
		return ApplicationUtil.normalizeToNull(longitudine);
	}
	public void setLongitudine(String longitudine) {
		this.longitudine = ApplicationUtil.normalizeToNull(longitudine);
	}
	@Column(columnDefinition = "text")
	private String ordine_transizioni;
	public String getOrdine_transizioni() {
		return ordine_transizioni;
	}
	public void setOrdine_transizioni(String ordine_transizioni) {
		this.ordine_transizioni = ordine_transizioni;
	}
	public List<String> getTransizioniList() {
		List<String> a =  getTransizioni();
		a.add(ConstantsMap.sceltaFinale);
		return a;
	}
	public void resetTransizioni() {
		ordine_transizioni = null;
	}
	public List<String> getTransizioni() {
		List<String> a =  ApplicationUtil.token(ordine_transizioni, ConstantsGlobal.colonSeparator);
		if(ApplicationUtil.empty(a)) {
			a.addAll(ConstantsMap.TRANSIZIONI);
		}
		return a;
	}
	
	public void setTransizioni(List<String> valore_multiplo) {
		ordine_transizioni = ApplicationUtil.detoken(valore_multiplo, ConstantsGlobal.colonSeparator);
	}
	public boolean isTransizioniE() {
		return ordine_transizioni != null && !ListUtils.isEqualList(ApplicationUtil.token(ordine_transizioni, ConstantsGlobal.colonSeparator), ConstantsMap.TRANSIZIONI);
	}
	@SuppressWarnings("unchecked")
	public void validate_transizioni(FacesContext context, UIComponent component, Object input)  {
		List<String> os = (List<String>)input;
		if(ApplicationUtil.notEmpty(os) ) {
			int index1 = os.indexOf(ConstantsMap.sceltaProdotti);
			int index2 = os.indexOf(ConstantsMap.sceltaCampi);
			int index3 = os.indexOf(ConstantsMap.sceltaVocalOrder);
			int index4 = os.indexOf(ConstantsMap.sceltaContrattoPda);
			boolean b = (index1 < index2) && (index1 < index3) && (index1 < index4);
			if (!b) {
				throw new ValidatorException(Message.addValidatorMessage("non_valido" , null));
			}
		}
	}
	
	
}