package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.faces.validator.ValidatorException;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Lob;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.message.Message;
import it.callite.model.anagrafica.CampoAggiuntivoAnagraficaXML;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Ricerca;
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "campo_aggiuntivo_anagrafica", indexes = { @Index(name= "campo", columnList = "data_creazione, chiave, etichetta, tipo, visibile_agente, visibile_operatore, modificabile_agente, modificabile_operatore") })
public class CampoAggiuntivoAnagrafica  extends SelezionatoAbstract implements Serializable, Comparable<CampoAggiuntivoAnagrafica> {
	
	private static final long serialVersionUID = 1L;
	
	@Lob
	private String defaults;

	public String getDefaults() {
		return ApplicationUtil.trimToNull(defaults);
	}
	public void setDefaults(String defaults) {
		this.defaults = ApplicationUtil.trimToNull(defaults);
	}
	
	public Boolean getTipologia() {
		return true;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getChiave() + " - "  + getEtichetta());
	}
	private Date data_creazione = ApplicationUtil.date();
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getColoreString() {
		return ApplicationUtil.trimToEmpty(colore);
	}
	public String getStileString() {
		return ApplicationUtil.trimToEmpty(stile);
	}
	private String colore;
	public String getColore() {
		return colore;
	}
	public void setColore(String colore) {
		this.colore = colore;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	private String gruppo;
	private Boolean visibile_operatore = Boolean.TRUE;
	private Boolean visibile_agente = Boolean.TRUE;
	private Boolean modificabile_operatore = Boolean.FALSE;
	private Boolean modificabile_agente = Boolean.FALSE;
	@Column(columnDefinition = "text")
	private String stile;

	public String getStile() {
		return ApplicationUtil.trimToEmpty(stile);
	}
	public void setStile(String stile) {
		this.stile = ApplicationUtil.trimToEmpty(stile);
	}
	public Boolean getModificabile_operatore() {
		return modificabile_operatore;
	}
	public void setModificabile_operatore(Boolean modificabile_operatore) {
		this.modificabile_operatore = modificabile_operatore;
	}
	public Boolean getModificabile_agente() {
		return modificabile_agente;
	}
	public void setModificabile_agente(Boolean modificabile_agente) {
		this.modificabile_agente = modificabile_agente;
	}
	@Column(columnDefinition = "varbinary(255)")
	private String chiave;
	@Column(columnDefinition = "text")
	private String descrizione;
	@Transient
	private List<DatoAggiuntivoAnagrafica> dettagli;
	private String etichetta;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Boolean obbligatorio = Boolean.FALSE;
	public Boolean getObbligatorio() {
		return obbligatorio;
	}
	public void setObbligatorio(Boolean obbligatorio) {
		this.obbligatorio = obbligatorio;
	}
	private Integer tipo = ConstantsMap.TIPO_APERTO;
	@Column(columnDefinition = "text")
	private String valori;
	@OneToMany(mappedBy = "campo_aggiuntivo", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<DomandaAnagrafica> domande;
	@OneToMany(mappedBy = "campo_aggiuntivo")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<DatoAggiuntivoAnagrafica> dati_aggiuntivi;
	public List<DatoAggiuntivoAnagrafica> getDati_aggiuntivi() {
		return dati_aggiuntivi;
	}
	public void setDati_aggiuntivi(List<DatoAggiuntivoAnagrafica> dati_aggiuntivi) {
		this.dati_aggiuntivi = dati_aggiuntivi;
	}
	public List<DomandaAnagrafica> getDomande() {
		return domande;
	}
	public void setDomande(List<DomandaAnagrafica> domande) {
		this.domande = domande;
	}
	public CampoAggiuntivoAnagrafica() { }
	public CampoAggiuntivoAnagrafica(CampoAggiuntivoAnagraficaXML campo) {
		this.setData_creazione(ApplicationUtil.date());
		this.setChiave(campo.getChiave());
		this.setDescrizione(campo.getDescrizione());
		this.setEtichetta(campo.getEtichetta());
		this.setGruppo(campo.getGruppo());
		this.setColore(campo.getColore());
		this.setStile(campo.getStile());
		this.setDefaults(campo.getDefaults());
		this.setPath(campo.getPath());
		this.setTipo(campo.getTipo());
		this.setValori(campo.getValori());
		this.setObbligatorio(campo.getObbligatorio());
		this.setVisibile_operatore(campo.getVisibile_operatore());
		this.setVisibile_agente(campo.getVisibile_agente());
		this.setModificabile_agente(campo.getModificabile_agente());
		this.setModificabile_operatore(campo.getModificabile_operatore());
	}

	public int compareTo(CampoAggiuntivoAnagrafica o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CampoAggiuntivoAnagrafica other = (CampoAggiuntivoAnagrafica) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public String getChiave() {
		return ApplicationUtil.normalizeToEmpty(chiave);
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToNull(descrizione);
	}
	public CampoAggiuntivoAnagrafica getCopia() {
		CampoAggiuntivoAnagrafica pr = new CampoAggiuntivoAnagrafica();
		pr.setEtichetta(ApplicationUtil.trimToEmpty(this.getEtichetta()) + " " +  ConstantsLanguage.COPY.stampa());
		pr.setChiave(ApplicationUtil.trimToEmpty(this.getChiave()) + " " +  ConstantsLanguage.COPY.stampa());
		pr.setGruppo(null);
		pr.setDescrizione(this.getDescrizione());
		pr.setValori(this.getValori());
		pr.setTipo(this.getTipo());
		pr.setDefaults(this.getDefaults());
		pr.setObbligatorio(this.getObbligatorio());
		pr.setVisibile_agente(this.getVisibile_agente());
		pr.setVisibile_operatore(this.getVisibile_operatore());
		pr.setModificabile_agente(this.getModificabile_agente());
		pr.setModificabile_operatore(this.getModificabile_operatore());
		pr.setColore(this.getColore());
		pr.setStile(this.getStile());
		return pr;
	}
	public List<DatoAggiuntivoAnagrafica> getDettagli() {
		return dettagli;
	}
	public String getEtichetta() {
		return ApplicationUtil.trimToEmpty(etichetta);
	}
	public Long getId() {
		return id;
	}
	public String getPath() {
		return ApplicationUtil.trimToEmpty(valori);
	}
	public String[] getSuggerimenti() {
		return ApplicationUtil.split(valori, ConstantsGlobal.colonSeparator);
	}
	public Integer getTipo() {
		if (tipo == null)
			tipo = ConstantsMap.TIPO_APERTO;
		return tipo;
	}
	public String getTipoCatalogo() {
		return "text.tipocampo_"+tipo;
	}
	public String getValori() {
		return ApplicationUtil.trimToEmpty(valori);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setChiave(String chiave) {
		this.chiave = ApplicationUtil.normalizeToEmpty(chiave);
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setDettagli(List<DatoAggiuntivoAnagrafica> dettagli) {
		this.dettagli = dettagli;
	}
	public void setEtichetta(String etichetta) {
		this.etichetta = ApplicationUtil.normalizeToNull(etichetta);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setPath(String valori) {
		this.valori = ApplicationUtil.trimToEmpty(valori);
	}
	public void setSuggerimenti(String[] suggerimenti) {
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	public void setTipoCatalogo(String tipoCatalogo) { }

	public void setValori(String valori) {
		if(this.getCombo() || this.getOneMenu() || this.getInplaceSelect() || this.getOneRadio() || this.getPickList() || this.getManyCheckbox() || this.getManyMenu() || this.getSuggerimento()) {
			this.valori = ApplicationUtil.trimToEmpty(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, valori));
		} else {
			this.valori = valori;
		}
	}

	public Boolean getScelta() { 
		return this.getOneMenu() || this.getOneRadio() || this.getManyCheckbox() || this.getPickList() || this.getManyMenu() || this.getRegulare() || this.getUnivoco() || this.getSuggerimento() || this.getCombo()|| this.getInplaceSelect();
	}
	public Boolean getDatario() { 
		return this.getData() || this.getDataora();
	}
	public Boolean getGenerico () { 
		return tipo!=null && (!ConstantsMap.getTipoCampo().contains(tipo));
	}
	public Boolean getTelefonico() {
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_TELEFONICO));
	}
	public Boolean getRegulare() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_REGULARE));
	}
	public Boolean getUnivoco() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_UNIVOCO));
	}
	public Boolean getSuggerimento() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_SUGGERIMENTO));
	}
	public Boolean getCombo() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_COMBOBOX));
	}
	public Boolean getSpinner() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_SPINNER));
	}
	public Boolean getFile() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_FILE));
	}
	public Boolean getSlider() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_SLIDER));
	}
	public Boolean getNumero() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_NUMERO));
	}
	public Boolean getIntero() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_INTERO));
	}
	public Boolean getLink() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_LINK));
	}
	public Boolean getAperto() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_APERTO));
	}
	public Boolean getForm() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_FORM));
	}
	public Boolean getBinario() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_BINARIO));
	}
	public Boolean getEditor() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_EDITOR));
	}
	public Boolean getInplaceInput() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_INPLACEINPUT));
	}
	public Boolean getInplaceSelect() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_INPLACESELECT));
	}
	public Boolean getColor() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_COLOR));
	}
	public Boolean getTextarea() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_TEXTAREA));
	}
	public Boolean getBooleano() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_BOOLEANO));
	}
	public Boolean getDataora() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_DATAORA));
	}
	public Boolean getOra() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_ORA));
	}
	public Boolean getData() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_DATA));
	}
	public Boolean getOneMenu() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_ONEMENU));
	}
	public Boolean getOneRadio() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_ONERADIO));
	}
	public Boolean getManyCheckbox() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_MANYCHECKBOX));
	}
	public Boolean getPickList() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_PICKLIST));
	}
	public Boolean getManyMenu() {
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_MANYMENU));
	}
	public void validate(FacesContext context, UIComponent component, Object input)  {
		if(valori!=null) {
			String var = (String) input;
			Pattern p = Pattern.compile(valori);
			Matcher m = p.matcher(var);
			boolean b = m.matches();
			if (!b) {
				throw new ValidatorException(Message.addValidatorMessage("formato.errato" , valori));
			}
		}
	}
	public List<String> getValoriList() {
		return ApplicationUtil.tokenU(this.getValori(), ConstantsGlobal.colonSeparator);
	}
	public List<SelectItem> getValoriCampo() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		if(this.getBinario() || this.getOneMenu() || this.getInplaceSelect() ){
			if(ApplicationUtil.isFalse(this.getObbligatorio())) {
				list.add(new SelectItem("", ConstantsLanguage.TRATTINI.stampa()));
			}
		}
		if(this.getCombo() || this.getOneMenu() || this.getInplaceSelect() || this.getOneRadio() || this.getPickList() || this.getManyCheckbox() || this.getManyMenu()) {
			List<String> valori = ApplicationUtil.tokenU(this.getValori(), ConstantsGlobal.colonSeparator);
			for(String valore : valori) {
				list.add(new SelectItem(valore, valore));
			}
		} 
		if(this.getBinario()) {
			list.add(new SelectItem(ConstantsLanguage.SI.stampa(), ConstantsLanguage.SI.stampa()));
			list.add(new SelectItem(ConstantsLanguage.NO.stampa(), ConstantsLanguage.NO.stampa()));
		}
		return list;
	}
	public List<String> autocompleteRicercaValori(Object query) {
		String q = ApplicationUtil.upperCaseEmpty((String) query);
		List<String> list = new ArrayList<String>();
		if(this.getSuggerimento() && ApplicationUtil.isNotBlank(q)) {
			List<String> valori = ApplicationUtil.tokenU(this.getValori(), ConstantsGlobal.colonSeparator);
			for(String valore : valori) {
				Ricerca m = ApplicationUtil.match(q);
				if(m.getValore() == 1) {
					if(m.isInverti()) {
						if(m.getRicerca() != null && !ApplicationUtil.upperCase(valore).equals(m.getRicerca()) && !list.contains(valore)) {
							list.add(valore);
						}
					} else {
						if(m.getRicerca() != null && ApplicationUtil.upperCase(valore).equals(m.getRicerca()) && !list.contains(valore)) {
							list.add(valore);
						}
					}
				}
				if(m.getValore() == 2) {
					if(m.isInverti()) {
						if(m.getRicerca() != null && !ApplicationUtil.upperCase(valore).startsWith(m.getRicerca()) && !list.contains(valore)) {
							list.add(valore);
						}
					} else {
						if(m.getRicerca() != null && ApplicationUtil.upperCase(valore).startsWith(m.getRicerca()) && !list.contains(valore)) {
							list.add(valore);
						}
					}
				}
				if(m.getValore() == 3) {
					if(m.isInverti()) {
						if(m.getRicerca() != null && !ApplicationUtil.upperCase(valore).endsWith(m.getRicerca()) && !list.contains(valore)) {
							list.add(valore);
						}
					} else {
						if(m.getRicerca() != null && ApplicationUtil.upperCase(valore).endsWith(m.getRicerca()) && !list.contains(valore)) {
							list.add(valore);
						}
					}
				}
				if(m.getValore() == 4) {
					if(m.isInverti()) {
						if(m.getRicerca() != null && !ApplicationUtil.upperCase(valore).contains(m.getRicerca()) && !list.contains(valore)) {
							list.add(valore);
						}
					} else {
						if(m.getRicerca() != null && ApplicationUtil.upperCase(valore).contains(m.getRicerca()) && !list.contains(valore)) {
							list.add(valore);
						}
					}
				}
			}
		} 
		return list;
	}
	public Boolean getVisibile_operatore() {
		return visibile_operatore;
	}
	public void setVisibile_operatore(Boolean visibile_operatore) {
		this.visibile_operatore = visibile_operatore;
	}
	public Boolean getVisibile_agente() {
		return visibile_agente;
	}
	public void setVisibile_agente(Boolean visibile_agente) {
		this.visibile_agente = visibile_agente;
	}
	public String getGruppo() {
		return ApplicationUtil.trimToEmpty(gruppo);
	}

	@Transient
	private String etichettaS;

	public String getValoriT() {
		String f = ApplicationUtil.trimToNull(getValori()); 
		if(getData()) {
			return f != null ? f : ConstantsFormat.date.stampa();
		} 
		if(getDataora()) {
			return f != null ? f : ConstantsFormat.dateTime.stampa();
		} 
		return f;
	}

	public String getEtichettaS() {
		if(etichettaS == null) {
			return etichetta;
		}
		return etichettaS;
	}
	public void setEtichettaS(String etichettaS) {
		this.etichettaS = etichettaS;
	}
	public void setGruppo(String gruppo) {
		this.gruppo = ApplicationUtil.normalizeToNull(gruppo);
	}
	public Boolean getRicercaFile() {
		return  getFile();
	}
	public Boolean getRicercaBooleana() {
		return  (getBinario() || getBooleano() || getColor());
	}
	public Boolean getRicercaMultipla() {
		return (getManyCheckbox() || getManyMenu() || getPickList());
	}
	public Boolean getRicercaNumero() {
		return  (getDataora() || getData() || getOra() || getSlider() || getSpinner() || getNumero() || getIntero());
	}
	public Boolean getRicercaSingola() {
		return (getOneRadio()|| getInplaceSelect() || getOneMenu() || getCombo());
	}
	public Boolean getRicercaStringa() {
		return (getAperto() || getLink() || getRegulare() || getUnivoco() || getInplaceInput() || getTelefonico() || getTextarea() || getEditor() || getSuggerimento());
	}

	public void imposta() {
		if(tipo == null) {
			tipo = ConstantsMap.TIPO_APERTO;
		}
		if(obbligatorio == null) {
			obbligatorio = Boolean.FALSE;
		}
		if(visibile_operatore == null) {
			visibile_operatore = Boolean.TRUE;
		}
		if(visibile_agente == null) {
			visibile_agente = Boolean.TRUE;
		}
		if(modificabile_operatore == null) {
			modificabile_operatore = Boolean.FALSE;
		}
		if(modificabile_agente == null) {
			modificabile_agente = Boolean.FALSE;
		}
		if(ApplicationUtil.eq(tipo, ConstantsMap.TIPO_FORM)) {
			gruppo = null;
			defaults = null;
		}
	}
	
}