package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "categoria", indexes = { @Index(name= "categoria", columnList = "data_creazione, nome") })
public class Categoria  extends SelezionatoAbstract implements Serializable {
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	private Date data_creazione = ApplicationUtil.date();
	@Column(columnDefinition = "text")
	private String descrizione;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String nome;
	public Categoria getCopia() {
		Categoria pr = new Categoria();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setDescrizione(this.getDescrizione());
		pr.setTipologia(this.getTipologia());
		return pr;
	}
	@OneToMany(mappedBy = "categoria")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Gettone> gettoni = new ArrayList<Gettone>();
	@OneToMany(mappedBy = "categoria")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contest> contest = new ArrayList<Contest>();
	public List<Gettone> getGettoni() {
		return gettoni;
	}
	public void setGettoni(List<Gettone> gettoni) {
		this.gettoni = gettoni;
	}
	public List<Contest> getContest() {
		return contest;
	}
	public void setContest(List<Contest> contest) {
		this.contest = contest;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Categoria other = (Categoria) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	@OneToMany
	@JoinColumn(name = "categoria_id", nullable = true, updatable=true, insertable=true)
	private List<Prodotto> prodotti = new ArrayList<Prodotto>();
	public void addProdotto(Prodotto prodotto) {
		if(prodotti == null) {
			prodotti = new ArrayList<Prodotto>();
		}
		prodotto.setCategoria(this);
		this.prodotti.add(prodotto);
	}
	public List<Prodotto> getProdotti() {
		return prodotti;
	}
	public void setProdotti(List<Prodotto> prodotti) {
		this.prodotti = prodotti;
	}
	private Integer tipologia = ConstantsGlobal.gconteggio;
	public Integer getTipologia() {
		if(tipologia == null) {
			tipologia = ConstantsGlobal.gconteggio;
		}
		return tipologia;
	}
	public void setTipologia(Integer tipologia) {
		this.tipologia = tipologia;
	}
}