package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.Formula;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsEsito;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.message.Message;
import it.callite.model.allegato.AllegatoContatto;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.model.sip.Cdr;
import it.callite.utility.AiutoCampagna;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.AppuntamentoDistanza;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.sincronizzazione.InfoContatto;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "contatto", indexes = { @Index(name= "contatto", columnList = "data_creazione, data_archiviazione, data_presentazione, data_lavorazione, archiviato, tipo") })
public class Contatto extends SelezionatoAbstract  implements Serializable, Comparable<Contatto> {

	@Transient
	private Long tempoRisposta;
	
	
	public Long getTempoRisposta() {
		return tempoRisposta;
	}

	public void setTempoRisposta(Long tempoRisposta) {
		this.tempoRisposta = tempoRisposta;
	}

	@Transient
	private Long seq = Long.valueOf(0);
	
	public Long getSeq() {
		return seq;
	}

	public void setSeq(Long seq) {
		this.seq = seq;
	}
	
	@Transient
	private List<String> motivi = new ArrayList<String>();
	public void addMotivi(List<String> motivi) {
		this.motivi = motivi;
	}
	public String getMotivi() {
		return ApplicationUtil.detoken(motivi, ConstantsGlobal.colonSeparator);
	}
	@Transient
	private Integer time;
	@Transient
	private Boolean inbound = Boolean.FALSE;
	
	@Transient
	private List<String> telefoniS = new ArrayList<String>();
	private Boolean bloccato = Boolean.FALSE;

	public String getBloccatoS() {
		return getBloccato() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public Boolean getBloccato() {
		if(bloccato == null) {
			bloccato = Boolean.FALSE;
		}
		return bloccato;
	}
	public void setBloccato(Boolean bloccato) {
		this.bloccato = bloccato;
	}

	public List<String> getTelefoniS() {
		return telefoniS;
	}
	public void setTelefoniS(List<String> telefoniS) {
		this.telefoniS = telefoniS;
	}
	public Integer getTipoTelefono() {
		return ApplicationUtil.gtZero(this.getLista().getTelefono()) ? this.getLista().getTelefono().intValue() : this.getCampagna().getTelefono().intValue();
	}
	public Boolean getInbound() {
		return ApplicationUtil.isTrue(inbound);
	}
	public void setInbound(Boolean inbound) {
		this.inbound = inbound;
	}
	public String getTime() {
		if(time == null) {
			time = campagna.getTempo();
		} else {
			time--;
		}
		if(time.intValue() < 0) {
			time = 0;
		}
		return ApplicationUtil.format(time);
	}
	public Boolean getAutomaticoA() {
		return isPubblica() && campagna.getEsito() != null && ApplicationUtil.gtZero(campagna.getTempo());
	}
	public Boolean getAutomaticoC() {
		return isPubblica() && campagna.getEsito() != null && ApplicationUtil.gtZero(campagna.getTempo());
	}
	//@OneToMany
	//@JoinColumn(name = "contatto", nullable = true, updatable=true, insertable=true)
	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Cdr> cdr = new ArrayList<Cdr>();
	public List<Cdr> getCdr() {
		return cdr;
	}
	public void setCdr(List<Cdr> cdr) {
		this.cdr = cdr;
	}
	public String getCallers() {
		String recall = getRecall_callers();
		if (recall != null) {
			return recall;
		}
		String sl = lista.getCallers();
		if(sl == null) {
			sl = campagna.getCallers();
		}
		return sl;
	} 
	public void setCallers(String callers) {
	}
	
	@Transient
	private AutoContatto autoContatto;

	public AutoContatto getAutoContatto() {
		return autoContatto;
	}
	public void setAutoContatto(AutoContatto autoContatto) {
		this.autoContatto = autoContatto;
	}

	public String getMusicclass() {
		String recall = getRecall_musicclass();
		if (recall != null) {
			return recall;
		}
		String sl = lista.getMusicclass();
		if(sl == null) {
			sl = campagna.getMusicclass();
		}
		return sl;
	} 
	public void setMusicclass(String musicclass) {
	}
	@ManyToOne
	private Agente agente;
	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AllegatoContatto> allegati;
	@Transient
	private List<Allegato> allegato;
	@ManyToOne
	private Anagrafica anagrafica;
	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Appuntamento> appuntamenti;
	private Boolean archiviato = Boolean.FALSE;
	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AutoContatto> auto;
	@Transient
	private Boolean autodialer = Boolean.FALSE;
	@Transient
	private Boolean autodialerApertura = Boolean.FALSE;
	public Boolean getAutodialerApertura() {
		return autodialerApertura;
	}
	public void setAutodialerApertura(Boolean autodialerApertura) {
		this.autodialerApertura = autodialerApertura;
	}
	

	@ManyToOne
	private Campagna campagna;
	@Transient 
	private Date caricamento = ApplicationUtil.date();
	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contratto> contratti;
	private Date data_archiviazione;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_lavorazione;
	private Date data_presentazione = ApplicationUtil.aggiustaDataT(ApplicationUtil.date(), true);
	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Email> emails;
	@ManyToOne
	private EsitoArchiviazione esitoArchiviazione;
	@ManyToOne
	private EsitoChiamata esitoChiamata;
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Rmail> rmails;
	@ManyToOne
	private Lista lista;
	@Transient
	private Map<Long, Appuntamento> mappaAppuntamenti = new HashMap<Long, Appuntamento>();
	
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	@Column(columnDefinition = "text")
	private String nota;
	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Notifica> notifiche;

	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Sondaggio> sondaggi;
	//@NotAudited
	//@Formula("esitoChiamata_id")
	@Formula(value = "(SELECT a.ordine FROM " + ConstantsGlobal.tp + "esito_chiamata a WHERE a.id = esitoChiamata_id)")
	private Long ordine;
	//@NotAudited
	@Formula(value = "(SELECT a.priorita FROM " + ConstantsGlobal.tp + "lista a WHERE a.id = lista_id)")
	private Integer priorita;
	@ManyToOne
	private Utente proprietario;
	
	@Transient
	private String registrazione;
	
	@OneToMany(mappedBy = "contatto", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaContatto> storia;	
	

	@OneToMany(mappedBy = "contatto")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Telecom> telecoms;
	@Transient 
	private Date timestamp_apertura;		
	@Transient 
	private Date timestamp_chiamata; 
	@Transient 
	private Date timestamp_esito; 
	@Transient 
	private Date timestamp_fine; 
	@Transient 
	private Date timestamp_rec_fine;
	@Transient 
	private Date timestamp_rec_inizio;
	@Transient 
	private Date timestamp_risposta; 
	private Integer tipo = ConstantsGlobal.inviato; 
	@Transient 
	private Date timestamp_attesa_fine;
	@Transient 
	private Date timestamp_attesa_inizio;
	@Transient
	private Integer tipologia;
	@Transient
	private Boolean trovato = Boolean.FALSE;
	@Transient
	private Map<Long, AppuntamentoDistanza> posizione = new HashMap<Long, AppuntamentoDistanza>();
	public AppuntamentoDistanza getPosizioneAppuntamento(Long agenteId) {
		AppuntamentoDistanza d = posizione.get(agenteId);
		if(d == null) {
			d = new AppuntamentoDistanza();	
			posizione.put(agenteId, d);
		}
		return d;
	}
	public Map<Long, AppuntamentoDistanza> getPosizione() {
		return posizione;
	}
	public void setPosizione(Map<Long, AppuntamentoDistanza> posizione) {
		this.posizione = posizione;
	}
	@ManyToOne
	private Utente utente;
	
	public Contatto() {
		userfield =  "id_" + System.currentTimeMillis();
	}
	@Transient
	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public Contatto(Long contatto_id) {
		this.setId(contatto_id);
	}
	public void addAppuntamento(Appuntamento appuntamento) {
		this.mappaAppuntamenti.put(appuntamento.getId(), appuntamento);
	}
	public int compareTo(Contatto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Contatto other = (Contatto) obj;
		if (chiave == null) {
			if (other.chiave != null)
				return false;
		} else if (!chiave.equals(other.chiave))
			return false;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (duplicato == null) {
			if (other.duplicato != null)
				return false;
		} else if (!duplicato.equals(other.duplicato))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (seq == null) {
			if (other.seq != null)
				return false;
		} else if (!seq.equals(other.seq))
			return false;
		if (trovato == null) {
			if (other.trovato != null)
				return false;
		} else if (!trovato.equals(other.trovato))
			return false;
		return true;
	}
	public Agente getAgente() {
		return agente;
	}
	public String getAgente_denominazione() {
		return getAgente() != null ? getAgente().getDenominazione() : "";
	}
	public String getAgente_codice() {
		return getAgente() != null ? getAgente().getCodice() : "";
	}
	public String getAiuto() {
		List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
		Map<String, DatoAggiuntivo> valori = ApplicationUtil.convertiAnagrafica(getAnagrafica(), valoriValidi);
		if(ApplicationUtil.notEmpty(dettagli)) {
			valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dettagli));
		}
		return ApplicationUtil.replaceInString(script != null ? script.getScript() : getCampagna().getAiuto(), valori);
	}
	@Transient
	private AiutoCampagna script;

	public AiutoCampagna getScript() {
		return script;
	}
	public void setScript(AiutoCampagna script) {
		this.script = script;
	}
	public Byte[] getFileP() {
		return script != null ? script.getFileP() : getCampagna().getFileP();
	}

	public boolean isFile() {
		return script != null ? script.isFile() : getCampagna().isFile();
	}

	public String getModulo() {
		return script != null ? script.getModulo() : getCampagna().getModulo();
	}

	public Short getAiuto_velocita() {
		return script != null ? script.getScript_velocita() : getCampagna().getAiuto_velocita();
	}


	@Transient
	private List<DatoAggiuntivoAnagrafica> dettagli;
	public List<DatoAggiuntivoAnagrafica> getDettagli() {
		return dettagli;
	}
	public void setDettagli(List<DatoAggiuntivoAnagrafica> dettagli) {
		this.dettagli = dettagli;
	}
	public List<AllegatoContatto> getAllegati() {
		return allegati;
	}
	public List<Allegato> getAllegato() {
		if(allegato==null)
			allegato = new ArrayList<Allegato>();
		return allegato;
	}
	public int getAllegatoSize() {
		return ApplicationUtil.size(allegato);
	}
	public Anagrafica getAnagrafica() {
		return anagrafica;
	}
	public List<Appuntamento> getAppuntamenti() {
		return appuntamenti;
	}
	public Appuntamento getAppuntamento(Long id) {
		return mappaAppuntamenti.get(id);
	}
	public Boolean getArchiviato() {
		return ApplicationUtil.isTrue(archiviato);
	}
	public List<AutoContatto> getAuto() {
		return auto;
	}
	public Boolean getAutodialer() {
		return autodialer;
	}
	
	public Campagna getCampagna() {
		return campagna;
	}
	public Date getCaricamento() {
		return caricamento;
	}
	public String getColore() {
		if(ApplicationUtil.equals(tipo, ConstantsGlobal.inserito)) {
			return ConstantsGlobal.color_inserito;
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.inviato)) {
			return ConstantsGlobal.color_inviato;
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.respinto)) {
			return ConstantsGlobal.color_respinto;
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.bloccato)) {
			return ConstantsGlobal.color_bloccato;
		} else {
			return ConstantsGlobal.color_inviato;
		} 
	}
	public List<Contratto> getContratti() {
		return contratti;
	}
	public Date getData_archiviazione() {
		return data_archiviazione;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_lavorazione() {
		return data_lavorazione;
	}
	public Date getData_presentazione() {
		return ApplicationUtil.aggiustaDataT(data_presentazione, true);
	}
	@Transient
	public String getDenominazione() {
		StringBuilder sb = new StringBuilder();
		String s = ApplicationUtil.trimToNull(this.getDenominazioneTelefono());
		if(s != null) {
			sb.append(s);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public List<Email> getEmails() {
		return emails;
	}
	public EsitoArchiviazione getEsitoArchiviazione() {
		return esitoArchiviazione;
	}
	public EsitoChiamata getEsitoChiamata() {
		return esitoChiamata;
	}
	public String getEsitoGeneraleColore() {
		if(esitoArchiviazione != null) {
			return getEsitoArchiviazione().getColore();
		} else {
			return getEsitoChiamata().getColore();
		}
	}
	public String getEsitoGeneraleCodice() {
		if(esitoArchiviazione != null) {
			return getEsito_archiviazione_etichettaCodice();
		} else {
			return getEsito_chiamata_etichettaCodice();
		}
	}
	
	public Long getId() {
		return id;
	}
	public Object getChiaveO(String tipo) {
		if(tipo != null && !tipo.equals(ConstantsMap.contatto_id)) {
			if(tipo.equals(ConstantsMap.anagrafica_telefono_fisso)) {
				return ApplicationUtil.trimToNull(getAnagrafica().getTelefono_fisso());
			} else if(tipo.equals(ConstantsMap.anagrafica_telefono_mobile)) {
				return ApplicationUtil.trimToNull(getAnagrafica().getTelefono_mobile());
			} else if(tipo.equals(ConstantsMap.anagrafica_telefono_fax)) {
				return ApplicationUtil.trimToNull(getAnagrafica().getTelefono_fax());
			} else if(tipo.equals(ConstantsMap.anagrafica_telefono_altro)) {
				return ApplicationUtil.trimToNull(getAnagrafica().getTelefono_altro());
			} else if(tipo.equals(ConstantsMap.anagrafica_email)) {
				return ApplicationUtil.trimToNull(getAnagrafica().getEmail());
			} else if(tipo.equals(ConstantsMap.anagrafica_link)) {
				return ApplicationUtil.trimToNull(getAnagrafica().getLink());
			} else if(tipo.equals(ConstantsMap.anagrafica_codice_fiscale)) {
				return ApplicationUtil.trimToNull(getAnagrafica().getCodice_fiscale());
			} else if(tipo.equals(ConstantsMap.anagrafica_partita_iva)) {
				return ApplicationUtil.trimToNull(getAnagrafica().getPartita_iva());
			}
		} else {
			return getId();
		}
		return null;
	}
	public boolean getInviabile() {
		return getAgente() != null && (ApplicationUtil.equals(tipo, ConstantsGlobal.inserito) || ApplicationUtil.equals(tipo, ConstantsGlobal.respinto));
	}
	public boolean getInviato() {
		return getAgente() != null && ApplicationUtil.equals(tipo, ConstantsGlobal.inviato);
	}

	public List<Rmail> getRmails() {
		return rmails;
	}
	public Lista getLista() {
		return lista;
	}
	public Map<Long, Appuntamento> getMappaAppuntamenti() {
		return mappaAppuntamenti;
	}
	public Long getMappaId() {
		return Long.valueOf(-1 * (mappaAppuntamenti.size() + 1));
	}

	public String getNota() {
		return nota;
	}
	public String getNotaShort() {
		return ApplicationUtil.tronca(nota);
	}
	public List<Notifica> getNotifiche() {
		return notifiche;
	}
	public Long getOrdine() {
		if(ApplicationUtil.ltZero(ordine))
			return ConstantsGlobal.defaultOrdine;
		return ordine;
	}
	public Integer getPriorita() {
		if(ApplicationUtil.ltZero(priorita))
			return ConstantsGlobal.defaultPriorita;
		return priorita;
	}
	public Utente getProprietario() {
		return proprietario;
	}
	public String getProprietario_denominazione() {
		return getProprietario() != null ? getProprietario().getDenominazione() : "";
	}
	public String getProprietario_codice() {
		return getProprietario() != null ? getProprietario().getUsername() : "";
	}

	public String getRegistrazione() {
		return  ApplicationUtil.upperCaseEmpty(ApplicationUtil.replaceFileName(anagrafica.getDenominazione()));
	}
	public Boolean getRender() {
		String aiuto = ApplicationUtil.trimToNull(getCampagna().getAiuto());
		if(aiuto != null) {
			for(String s: ConstantsMap.anagraficaMap){
				if(aiuto.contains(ConstantsGlobal.pipeSeparator+s+ConstantsGlobal.pipeSeparator)) {
					return true;
				}
			}
		}
		return false;
	}
	public Boolean getRiprogrammabile() {
		if(esitoArchiviazione!=null && (ApplicationUtil.eq(esitoArchiviazione.getTipologiaS(), 1) || ApplicationUtil.eq(esitoArchiviazione.getTipologiaS(), 2))) {
			return true;
		}
		return false;
	}

	public List<StoriaContatto> getStoria() {
		if(storia==null)
			storia = new ArrayList<StoriaContatto>();
		return storia;
	}
	private Integer sequenziale;
	public Integer getSequenziale() {
		return sequenziale;
	}
	public void setSequenziale(Integer sequenziale) {
		this.sequenziale = sequenziale;
	}
	public String getTelefono_chiamata() {
		if(telefono == null) {
			String t = null;
			Integer tel = getTipoTelefono();
			if(ApplicationUtil.eq(tel, 1) || ApplicationUtil.eq(tel, 2) || ApplicationUtil.eq(tel, 3) || ApplicationUtil.eq(tel, 4)) {
				t = this.getTelefono_predefinito();
			} 
			if(ApplicationUtil.eq(tel, 5) || ApplicationUtil.eq(tel, 6)) {
				t = this.getTelefono_random();
			}
			if(ApplicationUtil.eq(tel, 7) || ApplicationUtil.eq(tel, 8)) {
				t = this.getTelefono_sequenziale();
			}
			t = ApplicationUtil.compila(t, this.getLista().getAppend_prefisso());
			t = ApplicationUtil.compila(t, this.getCampagna().getAppend_prefisso());
			if(ApplicationUtil.isNotBlank(t)) {
				t = ApplicationUtil.uniforma(t, this.getLista().getRem_prefisso(), this.getLista().getAdd_prefisso());
				t = ApplicationUtil.uniforma(t, this.getCampagna().getRem_prefisso(), this.getCampagna().getAdd_prefisso());
			}
			telefono = t;
		}
		return ApplicationUtil.trimToEmpty(telefono);
	}
	public String getTelefono_sequenziale() {
		if(this.sequenziale == null) {
			sequenziale = 0;
		} 
		List<String> telefoni = new ArrayList<String>();
		String fisso = this.getAnagrafica().getConsenso_tfissoT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_fisso()) : null;
		String mobile = this.getAnagrafica().getConsenso_tmobileT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_mobile()) : null;
		String fax = this.getAnagrafica().getConsenso_tfaxT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_fax()) : null;
		String altro = this.getAnagrafica().getConsenso_taltroT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_altro()) : null;
		if(fisso != null && !telefoni.contains(fisso)) {
			telefoni.add(fisso);
		}
		if(mobile != null && !telefoni.contains(mobile)) {
			telefoni.add(mobile);
		}
		if(fax != null && !telefoni.contains(fax)) {
			telefoni.add(fax);
		}
		if(altro != null && !telefoni.contains(altro)) {
			telefoni.add(altro);
		}
		for (String a : this.getTelefoniS()){
			String agg = ApplicationUtil.trimToNull(a);
			if (agg != null && !telefoni.contains(agg)) {
				telefoni.add(agg);
			}
		}
		if(ApplicationUtil.notEmpty(telefoni)) {
			if(this.sequenziale >= telefoni.size()) {
				this.sequenziale = 0;
			}
			String t = telefoni.get(this.sequenziale);
			this.sequenziale++;
			return t;
		}
		return "";
	}
	public String getTelefono_random() {
		List<String> telefoni = new ArrayList<String>();
		String fisso = this.getAnagrafica().getConsenso_tfissoT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_fisso()) : null;
		String mobile = this.getAnagrafica().getConsenso_tmobileT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_mobile()) : null;
		String fax = this.getAnagrafica().getConsenso_tfaxT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_fax()) : null;
		String altro = this.getAnagrafica().getConsenso_taltroT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_altro()) : null;
		if(fisso != null && !telefoni.contains(fisso)) {
			telefoni.add(fisso);
		}
		if(mobile != null && !telefoni.contains(mobile)) {
			telefoni.add(mobile);
		}
		if(fax != null && !telefoni.contains(fax)) {
			telefoni.add(fax);
		}
		if(altro != null && !telefoni.contains(altro)) {
			telefoni.add(altro);
		}
		for (String a : this.getTelefoniS()){
			String agg = ApplicationUtil.trimToNull(a);
			if (agg != null && !telefoni.contains(agg)) {
				telefoni.add(agg);
			}
		}
		if(ApplicationUtil.notEmpty(telefoni)) {
			return ApplicationUtil.trimToEmpty(telefoni.get(new Random().nextInt(telefoni.size())));
		}
		return "";
	}
	
	public Boolean getConsenso_telefono() {
		return ApplicationUtil.isTrue(this.getCampagna().getConsenso_telefono()) && ApplicationUtil.isTrue(this.getLista().getConsenso_telefono());
	}
	public Boolean getConsenso_tfissoT() {
		return this.getAnagrafica().getConsenso_tfissoT() && getConsenso_telefono();
	}
	public Boolean getConsenso_tmobileT() {
		return this.getAnagrafica().getConsenso_tmobileT() && getConsenso_telefono();
	}
	public Boolean getConsenso_tfaxT() {
		return this.getAnagrafica().getConsenso_tfaxT() && getConsenso_telefono();
	}
	public Boolean getConsenso_taltroT() {
		return this.getAnagrafica().getConsenso_taltroT() && getConsenso_telefono();
	}


	public String getTelefono_predefinito() {
		Integer tel = getTipoTelefono();
		if(ApplicationUtil.eq(tel, 1) || ApplicationUtil.eq(tel, 2) || ApplicationUtil.eq(tel, 3) || ApplicationUtil.eq(tel, 4)) {
			List<String> telefoni = new ArrayList<String>();
			telefoni.add(this.getAnagrafica().getConsenso_tfissoT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_fisso()) : null);
			telefoni.add(this.getAnagrafica().getConsenso_tmobileT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_mobile()) : null);
			telefoni.add(this.getAnagrafica().getConsenso_tfaxT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_fax()) : null);
			telefoni.add(this.getAnagrafica().getConsenso_taltroT() ? ApplicationUtil.trimToNull(this.getAnagrafica().getTelefono_altro()) : null);
			for(int i = 0; i < telefoni.size(); i++) {
				int index = (i + (tel-1)) % telefoni.size();
				String t = telefoni.get(index);
				if(t != null) {
					return ApplicationUtil.trimToEmpty(t);
				}
			}
		} 
		return "";
	}
	public String getFisso_predefinito() {
		String telefono = this.getAnagrafica().getTelefono_fisso();
		if(ApplicationUtil.isNotBlank(telefono)) {
			return (ApplicationUtil.equals(this.getTelefono_predefinito(), telefono)) ? "underline" : "none";
		} else {
			return "none";
		}
	}
	public String getMobile_predefinito() {
		String telefono = this.getAnagrafica().getTelefono_mobile();
		if(ApplicationUtil.isNotBlank(telefono)) {
			return (ApplicationUtil.equals(this.getTelefono_predefinito(), telefono)) ? "underline" : "none";
		} else {
			return "none";
		}
	}
	public String getFax_predefinito() {
		String telefono = this.getAnagrafica().getTelefono_fax();
		if(ApplicationUtil.isNotBlank(telefono)) {
			return (ApplicationUtil.equals(this.getTelefono_predefinito(), telefono)) ? "underline" : "none";
		} else {
			return "none";
		}
	}
	public String getAltro_predefinito() {
		String telefono = this.getAnagrafica().getTelefono_altro();
		if(ApplicationUtil.isNotBlank(telefono)) {
			return (ApplicationUtil.equals(this.getTelefono_predefinito(), telefono)) ? "underline" : "none";
		} else {
			return "none";
		}
	}
	
	public Date getTimestamp_apertura() {
		return timestamp_apertura;
	}
	public Date getTimestamp_chiamata() {
		return timestamp_chiamata;
	}
	public Date getTimestamp_esito() {
		return timestamp_esito;
	}
	public Date getTimestamp_fine() {
		return timestamp_fine;
	}
	public Date getTimestamp_rec_fine() {
		return timestamp_rec_fine;
	}
	public Date getTimestamp_rec_inizio() {
		return timestamp_rec_inizio;
	}
	public Date getTimestamp_risposta() {
		return timestamp_risposta;
	}
	public Integer getTipo() {
		return tipo;
	}
	public Integer getTipologia() {
		if(tipologia == null)
			tipologia = ConstantsGlobal.ID_SEZIONE_PUBBLICHE;
		return tipologia;
	}
	public String getTipologiaString() {
		return ApplicationUtil.testo(tipologia);
	}
	public String getTipologiaStyle() {
		return ApplicationUtil.style(tipologia);
	}
	public String getTipoTesto() {
		if(ApplicationUtil.equals(tipo, ConstantsGlobal.inserito)) {
			return ConstantsLanguage.INSERITO.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.inviato)) {
			return ConstantsLanguage.INVIATO.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.respinto)) {
			return ConstantsLanguage.RESPINTO.stampa();
		} else if(ApplicationUtil.equals(tipo, ConstantsGlobal.bloccato)) {
			return ConstantsLanguage.BLOCCATO.stampa();
		} else {
			return ConstantsLanguage.INVIATO.stampa();
		} 
	}
	public Boolean getTrovato() {
		return trovato;
	}
	public Utente getUtente() {
		return utente;
	}
	public String getOperatore_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}
	public String getOperatore_codice() {
		return getUtente() != null ? getUtente().getUsername() : "";
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chiave == null) ? 0 : chiave.hashCode());
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((duplicato == null) ? 0 : duplicato.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((seq == null) ? 0 : seq.hashCode());
		result = prime * result + ((trovato == null) ? 0 : trovato.hashCode());
		return result;
	}
	public void imposta(Date date) {
		this.archiviato = Boolean.FALSE;
		this.data_archiviazione = null;
		this.data_presentazione = ApplicationUtil.aggiustaDataT(date != null ? date : ApplicationUtil.date(), true); 
	}
	public void imposta(Date date, EsitoChiamata esitoChiamata, Utente proprietario) {
		imposta(date);
		this.esitoChiamata = esitoChiamata;
		this.esitoArchiviazione = null;
		this.proprietario = proprietario;
	}
	public void archivia(boolean richiama) {
		this.data_archiviazione = ApplicationUtil.date();
		this.archiviato = Boolean.TRUE;
		if(!richiama) {
			this.proprietario = null;
			this.data_presentazione = null;
		} 
	}
	public void archivia(EsitoChiamata esitoChiamata) {
		this.data_archiviazione = ApplicationUtil.date();
		this.archiviato = Boolean.TRUE;
		this.esitoChiamata = esitoChiamata;
		this.esitoArchiviazione = null;
		this.data_presentazione = null;
		this.proprietario = null;
	}
	public void proprietario(Date date, Utente proprietario) {
		this.data_presentazione = ApplicationUtil.aggiustaDataT(date != null ? date : ApplicationUtil.date(), true); 
		this.proprietario = proprietario;
	}
	public void pubblico() {
		this.data_archiviazione = null;
		this.archiviato = Boolean.FALSE;
	}

	public boolean isOk() {
		if(esitoChiamata!=null && esitoChiamata.getId().equals(ConstantsEsito.ID_OK))	{
			return true;
		}
		else return false;
	}
	public boolean isPubblica() {
		return proprietario == null;
	}
	public Appuntamento removeAppuntamento(Long id) {
		return mappaAppuntamenti.remove(id);
	}
	public void setAgente(Agente agente) {
		this.agente = agente;
	}
	public void setAllegati(List<AllegatoContatto> allegati) {
		this.allegati = allegati;
	}
	public void setAnagrafica(Anagrafica anagrafica) {
		this.anagrafica = anagrafica;
	}
	public void setAppuntamenti(List<Appuntamento> appuntamenti) {
		this.appuntamenti = appuntamenti;
	}
	public void setArchiviato(Boolean archiviato) {
		this.archiviato = archiviato;
	}
	public void setAuto(List<AutoContatto> auto) {
		this.auto = auto;
	}
	public void setAutodialer(Boolean autodialer) {
		this.autodialer = autodialer;
	}
	
	public void setCampagna(Campagna campagna) {
		this.campagna = campagna;
	}
	public void setCaricamento(Date caricamento) {
		this.caricamento = caricamento;
	}
	public void setContratti(List<Contratto> contratti) {
		this.contratti = contratti;
	}
	public void setData_archiviazione(Date dataArchiviazione) {
		data_archiviazione = dataArchiviazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_lavorazione(Date data_lavorazione) {
		this.data_lavorazione = data_lavorazione;
	}
	public void setData_presentazione(Date dataPresentazione) {
		data_presentazione = ApplicationUtil.aggiustaDataT(dataPresentazione, true);
	}
	public void setEmails(List<Email> emails) {
		this.emails = emails;
	}
	public void setEsitoArchiviazione(EsitoArchiviazione esitoArchiviazione) {
		this.esitoArchiviazione = esitoArchiviazione;
	}
	public void setEsitoChiamata(EsitoChiamata esitoChiamata) {
		this.esitoChiamata = esitoChiamata;
	}

	public void setId(Long id) {
		this.id = id;
	}
	public void setRmails(List<Rmail> rmails) {
		this.rmails = rmails;
	}
	public void setLista(Lista lista) {
		this.lista = lista;
	}
	
	public void setNota(String nota) {
		this.nota = nota;
	}
	public void setNotifiche(List<Notifica> notifiche) {
		this.notifiche = notifiche;
	}
	public void setProprietario(Utente proprietario) {
		this.proprietario = proprietario;
	}
	
	public void setRegistrazione(String registrazione) {
		this.registrazione = registrazione;
	}
	
	public void setStoria(List<StoriaContatto> storia) {
		this.storia = storia;
	}
	public void setTelefono_chiamata(String telefono_chiamata) {
	}
	@Transient
	private String destinazione;
	public String getDestinazione() {
		return destinazione;
	}
	public void setDestinazione(String destinazione) {
		this.destinazione = destinazione;
	}

	@Transient
	private String recall_callers;
	@Transient
	private String recall_musicclass;

	public String getRecall_callers() {
		return recall_callers;
	}
	public void setRecall_callers(String recall_callers) {
		this.recall_callers = recall_callers;
	}
	public String getRecall_musicclass() {
		return recall_musicclass;
	}
	public void setRecall_musicclass(String recall_musicclass) {
		this.recall_musicclass = recall_musicclass;
	}
	@Transient
	private String telefono;
	public String getTelefono() {
		return telefono;
	}
	public void setTelefono(String telefono) {
		this.telefono = telefono;
	}
	public String getEsito_archiviazione_etichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(getEsito_archiviazione()));
		if(ApplicationUtil.isNotBlank(getCodice_archiviazione())) {
			sb.append(" (" + getCodice_archiviazione() + ")");
		}
		return sb.toString();
	}
	public String getEsito_chiamata_etichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(getEsito_chiamata()));
		if(ApplicationUtil.isNotBlank(getCodice_chiamata())) {
			sb.append(" (" + getCodice_chiamata() + ")");
		}
		return sb.toString();
	}
	
	
	public List<Telecom> getTelecoms() {
		return telecoms;
	}
	public void setTelecoms(List<Telecom> telecoms) {
		this.telecoms = telecoms;
	}
	public void setTimestamp_apertura(Date timestamp_apertura) {
		this.timestamp_apertura = timestamp_apertura;
	}
	public void setTimestamp_chiamata(Date timestamp_chiamata) {
		this.timestamp_chiamata = timestamp_chiamata;
	}
	public void setTimestamp_esito(Date timestamp_esito) {
		this.timestamp_esito = timestamp_esito;
	}
	public void setTimestamp_fine(Date timestamp_fine) {
		this.timestamp_fine = timestamp_fine;
	}
	public void setTimestamp_rec_fine(Date timestamp_rec_fine) {
		this.timestamp_rec_fine = timestamp_rec_fine;
	}
	public void setTimestamp_rec_inizio(Date timestamp_rec_inizio) {
		this.timestamp_rec_inizio = timestamp_rec_inizio;
	}
	public void setTimestamp_risposta(Date timestamp_risposta) {
		this.timestamp_risposta = timestamp_risposta;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	public void setTipologia(Integer tipologia) {
		this.tipologia = tipologia;
	}
	public void setTrovato(Boolean trovato) {
		this.trovato = trovato;
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	public Date getTimestamp_attesa_fine() {
		return timestamp_attesa_fine;
	}
	public void setTimestamp_attesa_fine(Date timestamp_attesa_fine) {
		this.timestamp_attesa_fine = timestamp_attesa_fine;
	}
	public Date getTimestamp_attesa_inizio() {
		return timestamp_attesa_inizio;
	}
	public void setTimestamp_attesa_inizio(Date timestamp_attesa_inizio) {
		this.timestamp_attesa_inizio = timestamp_attesa_inizio;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public Boolean getCaricato() {
		return Chiamate.getCodaContatti().contains(getId()) && Chiamate.getCodaAutodialer().contains(getId());
	}
	@Transient
	private Boolean duplicato = Boolean.FALSE;
	@Transient
	private String chiave;

	public Object getChiaveL(String tipo) {
		String s = ApplicationUtil.trimToNull(getChiave());
		if((tipo == null || tipo.equals(ConstantsMap.contatto_id)) && s != null) {
			Long l = null;
			try {
				l = Long.parseLong(s);
			} catch (Exception nfe) {
			}
			return l;
		}
		return s;
	}
	public String getChiave() {
		return chiave;
	}
	public void setChiave(String chiave) {
		this.chiave = chiave;
	}
	public Boolean getDuplicato() {
		return duplicato;
	}
	public void setDuplicato(Boolean duplicato) {
		this.duplicato = duplicato;
	}
	public String getEsito_chiamata() {
		return getEsitoChiamata().getEtichetta();
	} 
	public String getEsito_archiviazione() {
		return getEsitoArchiviazione() != null ? getEsitoArchiviazione().getEtichetta() : "";
	} 
	public String getEsito_generale() {
		if(esitoArchiviazione != null) {
			return getEsito_archiviazione();
		} else {
			return getEsito_chiamata();
		}
	} 
	public String getCodice_chiamata() {
		return getEsitoChiamata().getCodice();
	} 
	public String getCodice_archiviazione() {
		return getEsitoArchiviazione() != null ? getEsitoArchiviazione().getCodice() : "";
	} 
	public String getCodice_generale() {
		if(esitoArchiviazione != null) {
			return getCodice_archiviazione();
		} else {
			return getCodice_chiamata();
		}
	} 
	public String getStato() {
		return getTipoTesto();
	} 
	public String getPositivo() {
		if(getEsitoArchiviazione()!=null && ApplicationUtil.isTrue(getEsitoArchiviazione().getPositivo()))
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	}
	public String getUtile() {
		if(getEsitoArchiviazione()!=null && ApplicationUtil.isTrue(getEsitoArchiviazione().getUtile()))
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	} 
	public String getChiuso() {
		if(ApplicationUtil.isTrue(getArchiviato()))
			return ConstantsLanguage.SI.stampa();
		else
			return ConstantsLanguage.NO.stampa();
	} 
	public String getLista_denominazione() {
		return getLista().getDenominazione();
	} 

	public String getLista_codice() {
		return getLista().getCodice();
	} 
	public String getCampagna_denominazione() {
		return getCampagna().getDenominazione();
	} 

	public String getCampagna_codice() {
		return getCampagna().getCodice();
	} 
	
	public String getTelefonoList() {
		List<String> telefoni = new ArrayList<String>();
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getTelefono_fisso())) {
			telefoni.add(ConstantsLanguage.FISSO.stampa() + ": " + this.getAnagrafica().getTelefono_fisso());
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getTelefono_mobile())) {
			telefoni.add(ConstantsLanguage.MOBILE.stampa() + ": " + this.getAnagrafica().getTelefono_mobile());
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getTelefono_fax())) {
			telefoni.add(ConstantsLanguage.FAX.stampa() + ": " + this.getAnagrafica().getTelefono_fax());
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getTelefono_altro())) {
			telefoni.add(ConstantsLanguage.ALTRO.stampa() + ": " + this.getAnagrafica().getTelefono_altro());
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getEmail())) {
			telefoni.add(ConstantsLanguage.EMAIL.stampa() + ": " + this.getAnagrafica().getEmail());
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getLink())) {
			telefoni.add(ConstantsLanguage.LINK.stampa() + ": " + this.getAnagrafica().getLink());
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getCodice_fiscale())) {
			telefoni.add(ConstantsLanguage.CODICEA.stampa() + ": " + this.getAnagrafica().getCodice_fiscale());
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getPartita_iva())) {
			telefoni.add(ConstantsLanguage.PARTITAA.stampa() + ": " + this.getAnagrafica().getPartita_iva());
		}
		return ApplicationUtil.detoken(telefoni, ConstantsGlobal.colonSeparator + " ");
	}
	public String getDenominazioneTelefono() {
		StringBuilder sb = new StringBuilder();
		sb.append(anagrafica.getDenominazione());
		String telefoni  = getTelefonoList();
		if(ApplicationUtil.isNotBlank(telefoni)) {
			sb.append(" (" + telefoni + ")");
		} 
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getTelefonoString() {
		StringBuilder sb = new StringBuilder();
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getTelefono_fisso())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.FISSO.stampa() + ConstantsGlobal.B2 +": "+ this.getAnagrafica().getTelefono_fisso());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getTelefono_mobile())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.MOBILE.stampa() + ConstantsGlobal.B2 +": " + this.getAnagrafica().getTelefono_mobile());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getTelefono_fax())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.FAX.stampa() + ConstantsGlobal.B2 +": "+ this.getAnagrafica().getTelefono_fax());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getTelefono_altro())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.ALTRO.stampa() + ConstantsGlobal.B2 +": "+ this.getAnagrafica().getTelefono_altro());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getEmail())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.EMAIL.stampa() + ConstantsGlobal.B2 +": "+ this.getAnagrafica().getEmail());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getLink())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.LINK.stampa() + ConstantsGlobal.B2 +": "+ this.getAnagrafica().getLink());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getCodice_fiscale())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.CODICEA.stampa() + ConstantsGlobal.B2 +": "+ this.getAnagrafica().getCodice_fiscale());
			sb.append(ConstantsGlobal.RETURN);
		}
		if(ApplicationUtil.isNotBlank(this.getAnagrafica().getPartita_iva())) {
			sb.append(ConstantsGlobal.B1 + ConstantsLanguage.PARTITAA.stampa() + ConstantsGlobal.B2 +": "+ this.getAnagrafica().getPartita_iva());
			sb.append(ConstantsGlobal.RETURN);
		}
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getPredefinito() {
		Integer tel = getTipoTelefono();
		return Message.addCatalogText("telefono_" + tel.intValue());
	}
	public String getCampagnaSpecifica() {
		return getCampagna_denominazione() + getSpecifica();
	}
	public String getSpecifica() {
		if(ApplicationUtil.isTrue(this.getInbound())) {
			return " [I]";
		}
		if(ApplicationUtil.isTrue(this.getAutodialer())) {
			return " [A]";
		}
		return "";
	}
	public String getSpecificaC() {
		return getTipologiaString() + getSpecifica();
	}
	public Boolean getAutostart() {
		Integer a = getCampagna().getAvvio_automatico();
		if(ApplicationUtil.ge(a, 0)) {
			boolean a1 = isPubblica() && ApplicationUtil.eq(a, 1);
			boolean a2 = !isPubblica() && ApplicationUtil.eq(a, 2);
			boolean a3 = ApplicationUtil.eq(a, 3);
			return a1 || a2 || a3;
		} 
		return true;
	}
	public InfoContatto getSincronia() {
		if(Chiamate.getCodaContatti().contains(getId())) {
			return Chiamate.getCodaContatti().get(getId());
		} 
		if(Chiamate.getCodaSincroniaContatti().contains(getId())) {
			return Chiamate.getCodaSincroniaContatti().get(getId());
		} 
		return null;
	}

	public String getSondaggio() {
		String suono = ApplicationUtil.trimToNull(campagna.getSuono());
		if(suono != null) {
			return ApplicationUtil.getBaseNameN(suono);
		} else {
			return "";
		}
	} 
	public void setSondaggio(String sondaggio) {
	}
	public List<Sondaggio> getSondaggi() {
		return sondaggi;
	}
	public void setSondaggi(List<Sondaggio> sondaggi) {
		this.sondaggi = sondaggi;
	}

}