package it.callite.model;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "contest", indexes = { @Index(name= "contest", columnList = "data_creazione") })
public class Contest  extends SelezionatoAbstract implements Serializable, Comparable<Contest> {
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getImporto().toString());
	}
	private Boolean attivo = Boolean.TRUE;
	private Boolean contratto = Boolean.FALSE;
	
	private Date data_creazione = ApplicationUtil.date();
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	@ManyToOne
	private Qualifica differenziale;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private BigDecimal importo = new BigDecimal(0);
	private Integer moltiplicatore = 1;
	private Integer tipo = 1;
	
	@ManyToOne
	private Categoria categoria;
	@ManyToOne
	private Prodotto prodotto;
	@ManyToOne
	private Qualifica qualifica;
	private Integer quota = 0;
	private Integer utente = ConstantsGlobal.operatore;
	
	public int compareTo(Contest o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Contest other = (Contest) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Boolean getAttivo() {
		return attivo;
	}
	public void reset() {
		if(!getVisualizzaQualifica()) {
			qualifica = null;
		}
		
		if(!getVisualizzaDifferenziale()) {
			differenziale = null;
		}
		
		if(!getVisualizzaContratto()) {
			prodotto = null;
			categoria = null;
		}
		if(!getVisualizzaProdotto()) {
			prodotto = null;
		}
		
		if(!getVisualizzaCategoria()) {
			categoria = null;
		}
	}
	
	
	public Boolean getContratto() {
		return contratto;
	}
	
	public Contest getCopia() {
		Contest pr = new Contest();
		pr.setAttivo(this.getAttivo());
		pr.setContratto(this.getContratto());
		pr.setDifferenziale(this.getDifferenziale());
		pr.setImporto(this.getImporto());
		pr.setMoltiplicatore(this.getMoltiplicatore());
		pr.setProdotto(this.getProdotto());
		pr.setQualifica(this.getQualifica());
		pr.setQuota(this.getQuota());
		pr.setTipo(this.getTipo());
		pr.setCategoria(this.getCategoria());
		pr.setUtente(this.getUtente());
		return pr;
	}
	
	public Integer getTipo() {
		return tipo;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	public Categoria getCategoria() {
		return categoria;
	}
	public void setCategoria(Categoria categoria) {
		this.categoria = categoria;
	}
	public Qualifica getDifferenziale() {
		return differenziale;
	}
	public Long getId() {
		return id;
	}
	public BigDecimal getImporto() {
		return importo;
	}
	
	public Integer getMoltiplicatore() {
		return moltiplicatore;
	}
	public Prodotto getProdotto() {
		return prodotto;
	}
	public Qualifica getQualifica() {
		return qualifica;
	}
	public Integer getQuota() {
		return quota;
	}
	public Integer getUtente() {
		return utente;
	}
	public Boolean getVisualizzaDifferenziale() {
		return  utente.equals(ConstantsGlobal.orete) || utente.equals(ConstantsGlobal.oprima)|| utente.equals(ConstantsGlobal.osotto);
	}
	public Boolean getVisualizzaContratto() {
		return ApplicationUtil.isTrue(contratto);
	}
	
	public Boolean getVisualizzaCategoria() {
		return ApplicationUtil.isTrue(contratto) &&  ApplicationUtil.eq(tipo,2);
	}
	public Boolean getVisualizzaProdotto() {
		return ApplicationUtil.isTrue(contratto) &&  ApplicationUtil.eq(tipo,1);
	}
	
	public Boolean getVisualizzaQualifica() {
		return utente.equals(ConstantsGlobal.agente) || utente.equals(ConstantsGlobal.coordinatore) || utente.equals(ConstantsGlobal.rete) || utente.equals(ConstantsGlobal.prima)|| utente.equals(ConstantsGlobal.sotto) || utente.equals(ConstantsGlobal.orete) || utente.equals(ConstantsGlobal.oprima)|| utente.equals(ConstantsGlobal.osotto);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	public void setContratto(Boolean contratto) {
		this.contratto = contratto;
	}
	public void setDifferenziale(Qualifica differenziale) {
		this.differenziale = differenziale;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setImporto(BigDecimal importo) {
		this.importo = importo;
	}
	public void setMoltiplicatore(Integer moltiplicatore) {
		this.moltiplicatore = moltiplicatore;
	}
	public void setProdotto(Prodotto prodotto) {
		this.prodotto = prodotto;
	}
	public void setQualifica(Qualifica qualifica) {
		this.qualifica = qualifica;
	}
	public void setQuota(Integer quota) {
		this.quota = quota;
	}
	
	public void setUtente(Integer utente) {
		this.utente = utente;
	}
	
	
	
}