package it.callite.model;
import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Time;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import javax.faces.model.SelectItem;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.anagrafica.DatoAggiuntivoAnagraficaXML;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "dato_aggiuntivo_anagrafica", indexes = { @Index(name= "dato_aggiuntivo", columnList = "data_creazione, data_lavorazione, val_s, val_d, val_t, val_i, val_f") })
public class DatoAggiuntivoAnagrafica implements Serializable, Comparable<DatoAggiuntivoAnagrafica> {

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime
				* result
				+ ((campo_aggiuntivo == null) ? 0 : campo_aggiuntivo.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DatoAggiuntivoAnagrafica other = (DatoAggiuntivoAnagrafica) obj;
		if (campo_aggiuntivo == null) {
			if (other.campo_aggiuntivo != null)
				return false;
		} else if (!campo_aggiuntivo.equals(other.campo_aggiuntivo))
			return false;
		return true;
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@ManyToOne
	private Anagrafica anagrafica;
	@ManyToOne
	private CampoAggiuntivoAnagrafica campo_aggiuntivo;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione;
	private Date data_lavorazione;
	
	public Date getData_lavorazione() {
		return data_lavorazione;
	}
	public void setData_lavorazione(Date data_lavorazione) {
		this.data_lavorazione = data_lavorazione;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	@Lob
	private String valore;


	private String val_s;
	private Date val_d;
	private Time val_t;
	private Integer val_i;
	private BigDecimal val_f;


	public String getVal_s() {
		return val_s;
	}
	public void setVal_s(String val_s) {
		this.val_s = val_s;
	}
	public Date getVal_d() {
		return val_d;
	}
	public void setVal_d(Date val_d) {
		this.val_d = val_d;
	}
	public Time getVal_t() {
		return val_t;
	}
	public void setVal_t(Time val_t) {
		this.val_t = val_t;
	}
	public Integer getVal_i() {
		return val_i;
	}
	public void setVal_i(Integer val_i) {
		this.val_i = val_i;
	}
	public BigDecimal getVal_f() {
		return val_f;
	}
	public void setVal_f(BigDecimal val_f) {
		this.val_f = val_f;
	}
	public DatoAggiuntivoAnagrafica() {
	}
	public CampoAggiuntivoAnagrafica getCampo_aggiuntivo() {
		return campo_aggiuntivo;
	}
	public void setCampo_aggiuntivo(CampoAggiuntivoAnagrafica campoAggiuntivo) {
		campo_aggiuntivo = campoAggiuntivo;
	}
	public Anagrafica getAnagrafica() {
		return anagrafica;
	}
	public void setAnagrafica(Anagrafica anagrafica) {
		this.anagrafica = anagrafica;
	}
	public String getValore() {
		return ApplicationUtil.trimToEmpty(valore);
	}
	public String getValore_formattato() {
		String v = null;
		if(campo_aggiuntivo.getSlider() || campo_aggiuntivo.getSpinner() || campo_aggiuntivo.getIntero()) {
			v = ApplicationUtil.formatNumber(getValore());
		} else if( campo_aggiuntivo.getNumero()) {
			v = ApplicationUtil.formatNumber(ApplicationUtil.parseBigDecimal(getValore()));
		} else if( campo_aggiuntivo.getData()) {
			v = ApplicationUtil.formatDate(campo_aggiuntivo.getValoriT(), getValore_data());
		} else if( campo_aggiuntivo.getDataora()) {
			v = ApplicationUtil.formatDate(campo_aggiuntivo.getValoriT(), getValore_datatime());
		} else if( campo_aggiuntivo.getOra()) {
			v = ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), getValore_time());
		} else if( campo_aggiuntivo.getFile()) {
			v = ApplicationUtil.filename(getValore());
		} else {
			v = getValore();
		}
		return ApplicationUtil.trimToEmpty(v);
	}

	public Object getOrdineDato() {
		if(campo_aggiuntivo.getBinario() || campo_aggiuntivo.getColor() || campo_aggiuntivo.getBooleano()) { 
			return val_s;
		} else if(campo_aggiuntivo.getNumero()) {
			return val_f;
		} else if(campo_aggiuntivo.getSpinner() || campo_aggiuntivo.getSlider() || campo_aggiuntivo.getIntero()) {
			return val_i;
		} else if(campo_aggiuntivo.getData()) {
			return val_d;
		} else if(campo_aggiuntivo.getDataora()) {
			return val_d;
		} else if(campo_aggiuntivo.getOra()) {
			return val_t;
		} else if(campo_aggiuntivo.getFile()) {
			return val_s;
		} else {
			return valore;
		}
	}

	public void imposta() {
		this.val_s = null;
		this.val_d = null;
		this.val_t = null;
		this.val_i = null;
		this.val_f = null;
		if(campo_aggiuntivo.getBinario() || campo_aggiuntivo.getColor() || campo_aggiuntivo.getBooleano()) { 
			this.val_s = ApplicationUtil.trimToNull(getValore());
		} else if(campo_aggiuntivo.getNumero()) {
			this.val_f = ApplicationUtil.parseBigDecimal(ApplicationUtil.trimToNull(getValore()));
		} else if(campo_aggiuntivo.getSpinner() || campo_aggiuntivo.getSlider() || campo_aggiuntivo.getIntero()) {
			this.val_i = ApplicationUtil.parseInteger(ApplicationUtil.trimToNull(getValore()));
		} else if(campo_aggiuntivo.getData()) {
			this.val_d = getValore_data();
		} else if(campo_aggiuntivo.getDataora()) {
			this.val_d = getValore_datatime();
		} else if(campo_aggiuntivo.getOra()) {
			this.val_t = ApplicationUtil.dateToTime(getValore_time());
		} else if(campo_aggiuntivo.getFile()) {
			this.val_s = ApplicationUtil.filename(getValore());
		}
		Date data = ApplicationUtil.date();
		if(data_creazione == null) {
			data_creazione = data;
		}
		data_lavorazione = data;
	}

	public String getOrdineEtichetta() {
		return campo_aggiuntivo != null ? campo_aggiuntivo.getEtichetta() : "";
	}

	public void setValore(String valore) {
		this.valore = ApplicationUtil.trimToEmpty(valore);
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public int compareTo(DatoAggiuntivoAnagrafica o) {
		if( o.campo_aggiuntivo != null) {
			return campo_aggiuntivo.compareTo(o.campo_aggiuntivo);
		} else if( o.campo_aggiuntivo == null) {
			return -1;
		} else if(campo_aggiuntivo == null && o.campo_aggiuntivo != null) {
			return 1;
		} else {
			return 0;
		}
	}
	public DatoAggiuntivoAnagrafica(DatoAggiuntivoAnagraficaXML dato, Anagrafica anagrafica, CampoAggiuntivoAnagrafica campo_aggiuntivo) {
		this.setData_creazione(ApplicationUtil.date());
		this.setAnagrafica(anagrafica);
		this.setCampo_aggiuntivo(new CampoAggiuntivoAnagrafica(dato.getCampo_aggiuntivo()));
		this.setValore(dato.getValore());
	}
	public List<String> getValore_multiplo() {
		return ApplicationUtil.tokenU(this.getValore(), ConstantsGlobal.colonSeparator);
	}
	public void setValore_multiplo(List<String> valori) {
		valore = ApplicationUtil.detoken(valori, ConstantsGlobal.colonSeparator);
	}
	public void setValore_data(Date data) {
		if (data != null) {
			valore =  ApplicationUtil.formatDate(campo_aggiuntivo.getValoriT(), data);
		} else {
			valore = null;
		}
	}
	public Date getValore_data() {
		String s = this.getValore();
		return ApplicationUtil.parseData(campo_aggiuntivo.getValoriT(), s);
	}

	public void setValore_datatime(Date data) {
		if (data != null) {
			valore =  ApplicationUtil.formatDate(campo_aggiuntivo.getValoriT(), data);
		} else {
			valore = null;
		}
	}
	public Date getValore_datatime() {
		String s = this.getValore();
		return ApplicationUtil.parseData(campo_aggiuntivo.getValoriT(), s);
	}
	public void setValore_time(Date data) {
		if (data != null) {
			valore =  ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), data);
		} else {
			valore = null;
		}
	}
	
	public void setValore_numero(Integer data) {
		if (data != null) {
			valore =  ApplicationUtil.formatNumber(data);
		} else {
			valore = null;
		}
	}

	public void setValore_numero(BigDecimal data) {
		if (data != null) {
			valore =  ApplicationUtil.formatNumber(data);
		} else {
			valore = null;
		}
	}
	public Date getValore_time() {
		String s = this.getValore();
		return ApplicationUtil.parseData(ConstantsFormat.time.stampa(), s);
	}
	public List<SelectItem> getValoriCampo() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		if(campo_aggiuntivo.getBinario() || campo_aggiuntivo.getOneMenu() || campo_aggiuntivo.getInplaceSelect() ){
			if(ApplicationUtil.isFalse(campo_aggiuntivo.getObbligatorio())) {
				list.add(new SelectItem("", ConstantsLanguage.TRATTINI.stampa()));
			}
		}
		if(campo_aggiuntivo.getCombo() || campo_aggiuntivo.getOneMenu() || campo_aggiuntivo.getInplaceSelect() || campo_aggiuntivo.getOneRadio() || campo_aggiuntivo.getPickList() || campo_aggiuntivo.getManyCheckbox() || campo_aggiuntivo.getManyMenu()) {
			Set<String> valori = new LinkedHashSet<String>(ApplicationUtil.tokenU(campo_aggiuntivo.getValori(), ConstantsGlobal.colonSeparator));
			if(campo_aggiuntivo.getCombo() || campo_aggiuntivo.getOneMenu() || campo_aggiuntivo.getInplaceSelect() || campo_aggiuntivo.getOneRadio()) {
				valori.addAll(ApplicationUtil.tokenU(getValore(), ConstantsGlobal.colonSeparator));
			}
			if(campo_aggiuntivo.getPickList() || campo_aggiuntivo.getManyCheckbox() || campo_aggiuntivo.getManyMenu()) {
				valori.addAll(getValore_multiplo());
			}
			for(String valore : valori) {
				list.add(new SelectItem(valore, valore));
			}
		} 
		if(campo_aggiuntivo.getBinario()) {
			list.add(new SelectItem(ConstantsLanguage.SI.stampa(), ConstantsLanguage.SI.stampa()));
			list.add(new SelectItem(ConstantsLanguage.NO.stampa(), ConstantsLanguage.NO.stampa()));
		}
		return list;
	}
	public List<String> autocompleteInputValori(Object query) {
		String q = ApplicationUtil.upperCaseEmpty((String) query);
		List<String> list = new ArrayList<String>();
		if(campo_aggiuntivo.getSuggerimento() && ApplicationUtil.isNotBlank(q)) {
			Set<String> valori = new LinkedHashSet<String>();
			valori.addAll(ApplicationUtil.tokenU(campo_aggiuntivo.getValori(), ConstantsGlobal.colonSeparator));
			valori.addAll(ApplicationUtil.tokenU(getValore(), ConstantsGlobal.colonSeparator));
			for(String valore : valori) {
				if(ApplicationUtil.upperCaseEmpty(valore).contains(q) && !list.contains(valore)) {
					list.add(valore);
				}
			}
		} 
		return list;
	}

	public boolean isLinkValid() {
		if(campo_aggiuntivo.getLink()) {
			String protocol = ApplicationUtil.protocol(getValore());
			return protocol != null;
		}
		return false;
	}

	@Override
	public String toString() {
		return getValore_formattato();
	}

}