package it.callite.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;


@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "email_template", indexes = { @Index(name= "email_template", columnList = "data_creazione, nome") })
public class EmailTemplate  extends SelezionatoAbstract implements Serializable {

	@Transient
	private List<Allegato> allegati;

	@OneToMany(mappedBy = "emailTemplate")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AutoProcessEmail> autoProcessEmails;

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "email_campagna", joinColumns=@JoinColumn(name="email_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true)    )
	private List<Campagna> campagne;

	private Date data_creazione = ApplicationUtil.date();

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "emailTemplate_documento", joinColumns=@JoinColumn(name="emailTemplate_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="documenti_id", nullable=false, updatable=true, insertable=true)    )
	private Set<Documento> documenti = new HashSet<Documento>();

	@OneToMany(mappedBy = "emailTemplate")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Email> emails;

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esito_email", joinColumns=@JoinColumn(name="email_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true)    )
	private List<EsitoArchiviazione> esiti;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	private Boolean invia_allegati = Boolean.FALSE;
	private Boolean attivo = Boolean.TRUE;
	private String nome;

	@Column(columnDefinition = "text")
	private String oggetto;
	@ManyToOne
	private Smtp smtp;

	@Column(columnDefinition = "text")
	private String testo;

	public Boolean getAttivo() {
		return attivo;
	}

	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}

	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EmailTemplate other = (EmailTemplate) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	@Transient
	public List<Allegato> getAllegati() {
		return allegati;
	}


	public List<AutoProcessEmail> getAutoProcessEmails() {
		return autoProcessEmails;
	}

	public List<Campagna> getCampagne() {
		return campagne;
	}

	public Date getData_creazione() {
		return data_creazione;
	}

	public String getDenominazione() {
		return nome;
	}

	public Set<Documento> getDocumenti() {
		if(documenti == null)
			documenti = new HashSet<Documento>();
		return documenti;
	}

	@Transient
	public List<Documento> getDocumentiList() {
		if(documenti != null) 
			return new ArrayList<Documento>(documenti);
		else return new ArrayList<Documento>();
	}

	public String getDocumentiNome() {
		if(documenti != null) {
			List<String> nomi = new ArrayList<String>();
			Iterator<Documento> iter = documenti.iterator();
			while (iter.hasNext()) {
				Documento doc = iter.next();
				nomi.add(doc.getNome());
			}
			return ApplicationUtil.detoken(nomi, ConstantsGlobal.colonSeparator);
		}
		else return "";
	}

	public List<Email> getEmails() {
		return emails;
	}

	public List<EsitoArchiviazione> getEsiti() {
		return esiti;
	}

	public Long getId() {
		return id;
	}

	public Boolean getInvia_allegati() {
		return invia_allegati;
	}

	public String getNome() {
		return ApplicationUtil.normalizeToNull(nome);
	}

	public String getOggetto() {
		return oggetto;
	}

	public Smtp getSmtp() {
		return smtp;
	}

	public String getTesto() {
		return testo;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	public void setAllegati(List<Allegato> allegati) {
		this.allegati = allegati;
	}

	public void setAutoProcessEmails(List<AutoProcessEmail> autoProcessEmails) {
		this.autoProcessEmails = autoProcessEmails;
	}

	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDocumenti(Set<Documento> documenti) {
		this.documenti = documenti;
	}

	public void setDocumentiList(List<Documento> documenti) { }

	public void setEmails(List<Email> emails) {
		this.emails = emails;
	}

	public void setEsiti(List<EsitoArchiviazione> esiti) {
		this.esiti = esiti;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setInvia_allegati(Boolean invia_allegati) {
		this.invia_allegati = invia_allegati;
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	public void setOggetto(String oggetto) {
		this.oggetto = oggetto;
	}

	public void setSmtp(Smtp smtp) {
		this.smtp = smtp;
	}

	public void setTesto(String testo) {
		this.testo = testo;
	}

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	
	public EmailTemplate getCopia() {
		EmailTemplate copia = new EmailTemplate();
		copia.setNome(this.getNome() + " " + ConstantsLanguage.COPY.stampa());
		copia.setInvia_allegati(this.getInvia_allegati());
		copia.setAttivo(this.getAttivo());
		copia.setOggetto(this.getOggetto());
		copia.setTesto(this.getTesto());
		copia.setLink(this.getLink());
		copia.setAnalytics(this.getAnalytics());
		return copia;
	}
	private String analytics;
	
	public String getAnalytics() {
		return ApplicationUtil.upperCaseNull(analytics);
	}

	public void setAnalytics(String analytics) {
		this.analytics = ApplicationUtil.upperCaseNull(analytics);
	}
	private Boolean link = Boolean.FALSE;

	public Boolean getLink() {
		return link;
	}

	public void setLink(Boolean link) {
		this.link = link;
	}
	

}