package it.callite.model;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;



@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "esito_contratto", indexes = { @Index(name= "esito_contratto", columnList = "data_creazione, etichetta, ordinamento, codice, colore, predefinito, utile, confermato, visibile, impostabile_operatore, impostabile_agente, modificabile_operatore, modificabile_agente, apribile_operatore, apribile_agente") })
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
public class EsitoContratto  extends SelezionatoAbstract implements Serializable, Comparable<EsitoContratto> {

	@Column(columnDefinition = "text")
	private String messaggio_notifica;	
	@Column(columnDefinition = "text")
	private String notizia_notifica;	
	private String codice;
	private Long ordinamento = Long.valueOf(0);
	private String colore = ConstantsGlobal.sfondo;
	private Boolean notizia_operatore = false;
	private Boolean messaggio_operatore = false;
	private Boolean notizia_agente = false;
	private Boolean messaggio_agente = false;

	private Boolean confermato = Boolean.FALSE;
	@OneToMany
	//@OnDelete(action = OnDeleteAction.CASCADE)
	@JoinColumn(name = "esito_id", nullable = true, updatable=true, insertable=true)
	private List<Contratto> contratti;

	private Date data_creazione = ApplicationUtil.date();


	@Column(columnDefinition = "text")
	private String descrizione;

	private String etichetta;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	private Boolean impostabile_agente = Boolean.FALSE;
	private Boolean impostabile_operatore = Boolean.FALSE;
	private Boolean modificabile_agente = Boolean.FALSE;

	private Boolean modificabile_operatore = Boolean.FALSE;
	
	private Boolean apribile_agente = Boolean.FALSE;

	private Boolean apribile_operatore = Boolean.FALSE;

	private Boolean predefinito = Boolean.FALSE;

	@OneToMany
	//@OnDelete(action = OnDeleteAction.CASCADE)
	@JoinColumn(name = "esito_id", nullable = true, updatable=true, insertable=true)
	private List<StoriaContratto> storia_contratti;

	private Boolean utile = Boolean.FALSE;

	private Boolean visibile = Boolean.FALSE;

	public EsitoContratto(){ }

	public int compareTo(EsitoContratto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EsitoContratto other = (EsitoContratto) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}



	public String getCodice() {
		return ApplicationUtil.trimToEmpty(codice);
	}

	public void setOrdinamento(Long ordinamento) {
		this.ordinamento = ordinamento;
	}
	public Long getOrdinamento() {
		if(ApplicationUtil.ltZero(ordinamento)) {
			ordinamento = Long.valueOf(0);
		}
		return ordinamento;
	}
	public String getColore() {
		return colore;
	}


	public Boolean getConfermato() {
		return confermato;
	}




	public List<Contratto> getContratti() {
		return contratti;
	}
	public EsitoContratto getCopia() {
		EsitoContratto copia = new EsitoContratto();
		copia.setCodice(this.getCodice());
		copia.setOrdinamento(this.getOrdinamento());
		copia.setEtichetta(this.getEtichetta()+ " " + ConstantsLanguage.COPY.stampa());
		copia.setDescrizione(this.getDescrizione());
		copia.setUtile(this.getUtile());
		copia.setNotizia_notifica(this.getNotizia_notifica());
		copia.setMessaggio_notifica(this.getMessaggio_notifica());
		copia.setImpostabile_operatore(this.getImpostabile_operatore());
		copia.setImpostabile_agente(this.getImpostabile_agente());
		copia.setModificabile_agente(this.getModificabile_agente());
		copia.setConfermato(this.getConfermato());
		copia.setModificabile_operatore(this.getModificabile_operatore());
		copia.setColore(this.getColore());
		copia.setPredefinito(false);
		copia.setVisibile(this.getVisibile());
		copia.setNotizia_operatore(this.getNotizia_operatore());
		copia.setMessaggio_operatore(this.getMessaggio_operatore());
		copia.setNotizia_agente(this.getNotizia_agente());
		copia.setMessaggio_agente(this.getMessaggio_agente());
		copia.setApribile_agente(this.getApribile_agente());
		copia.setApribile_operatore(this.getApribile_operatore());
		return copia;
	}

	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getEtichettaCodice());
	}

	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}

	public String getEtichetta() {
		return etichetta;
	}	

	public String getEtichettaCodice() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.trimToEmpty(etichetta));
		if(ApplicationUtil.isNotBlank(codice)) {
			sb.append(" (" + codice + ")");
		}
		return sb.toString();
	}

	public Long getId() {
		return id;
	}
	public Boolean getImpostabile_agente() {
		return impostabile_agente;
	}
	public Boolean getImpostabile_operatore() {
		return impostabile_operatore;
	}

	public Boolean getModificabile_agente() {
		return modificabile_agente;
	}
	public Boolean getModificabile_operatore() {
		return modificabile_operatore;
	}
	public Boolean getPredefinito() {
		return ApplicationUtil.isTrue(predefinito);
	}
	public List<StoriaContratto> getStoria_contratti() {
		return storia_contratti;
	}
	public Boolean getUtile() {
		return utile;
	}

	public Boolean getVisibile() {
		return visibile;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	public void setCodice(String codice) {
		this.codice = ApplicationUtil.normalizeToEmpty(codice);
	}

	public void setColore(String colore) {
		this.colore = colore;
	}

	public void setConfermato(Boolean confermato) {
		this.confermato = confermato;
	}

	public void setContratti(List<Contratto> contratti) {
		this.contratti = contratti;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}

	public void setEtichetta(String etichetta) {
		this.etichetta = ApplicationUtil.normalizeToNull(etichetta);
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setImpostabile_agente(Boolean impostabile_agente) {
		this.impostabile_agente = impostabile_agente;
	}

	public void setImpostabile_operatore(Boolean impostabile_operatore) {
		this.impostabile_operatore = impostabile_operatore;
	}

	public void setModificabile_agente(Boolean modificabile_agente) {
		this.modificabile_agente = modificabile_agente;
	}


	public void setModificabile_operatore(Boolean modificabile_operatore) {
		this.modificabile_operatore = modificabile_operatore;
	}

	public void setPredefinito(Boolean predefinito) {
		this.predefinito = predefinito;
	}
	public void setStoria_contratti(List<StoriaContratto> storia_contratti) {
		this.storia_contratti = storia_contratti;
	}


	public Boolean getApribile_agente() {
		return apribile_agente;
	}

	public void setApribile_agente(Boolean apribile_agente) {
		this.apribile_agente = apribile_agente;
	}

	public Boolean getApribile_operatore() {
		return apribile_operatore;
	}

	public void setApribile_operatore(Boolean apribile_operatore) {
		this.apribile_operatore = apribile_operatore;
	}

	public void setUtile(Boolean utile) {
		this.utile = utile;
	}

	public void setVisibile(Boolean visibile) {
		this.visibile = visibile;
	}
	public String getMessaggio_notifica() {
		return ApplicationUtil.trimToEmpty(messaggio_notifica);
	}



	public void setMessaggio_notifica(String messaggio_notifica) {
		this.messaggio_notifica = ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, messaggio_notifica);
	}


	public void setNotizia_notifica(String notizia_notifica) {
		this.notizia_notifica = ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, notizia_notifica);
	}



	public String getNotizia_notifica() {
		return ApplicationUtil.trimToEmpty(notizia_notifica);
	}



	public Boolean getNotizia_agente() {
		return notizia_agente;
	}

	public void setNotizia_agente(Boolean notizia_agente) {
		this.notizia_agente = notizia_agente;
	}

	public Boolean getMessaggio_agente() {
		return messaggio_agente;
	}

	public void setMessaggio_agente(Boolean messaggio_agente) {
		this.messaggio_agente = messaggio_agente;
	}

	public Boolean getNotizia_operatore() {
		return notizia_operatore;
	}

	public void setNotizia_operatore(Boolean notizia_operatore) {
		this.notizia_operatore = notizia_operatore;
	}

	public Boolean getMessaggio_operatore() {
		return messaggio_operatore;
	}

	public void setMessaggio_operatore(Boolean messaggio_operatore) {
		this.messaggio_operatore = messaggio_operatore;
	}

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

}
