package it.callite.model;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.Set;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import org.hibernate.annotations.Formula;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "estratto", indexes = { @Index(name= "estratto", columnList = "data_creazione, data_estratto") })
public class Estratto  extends SelezionatoAbstract implements Serializable, Comparable<Estratto> {
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getUser().getDenominazione() + ": " + getTotale().toString());
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_estratto = ApplicationUtil.date();
	public Date getData_estratto() {
		return ApplicationUtil.aggiustaDataT(data_estratto, true);
	}
	public void setData_estratto(Date data_estratto) {
		this.data_estratto = ApplicationUtil.aggiustaDataT(data_estratto, true);
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	@ManyToOne
	private Utente user;
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public Utente getUser() {
		return user;
	}
	public void setUser(Utente user) {
		this.user = user;
	}
	public Set<EstrattoDettaglio> getEstrattoDettagli() {
		return estrattoDettagli;
	}
	public void setEstrattoDettagli(Set<EstrattoDettaglio> estrattoDettagli) {
		this.estrattoDettagli = estrattoDettagli;
	}
	@OneToMany(mappedBy = "estratto", fetch = FetchType.LAZY)
	@OnDelete(action = OnDeleteAction.CASCADE)
	private Set<EstrattoDettaglio> estrattoDettagli;
	public int compareTo(Estratto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Estratto other = (Estratto) obj;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (user == null) {
			if (other.user != null)
				return false;
		} else if (!user.equals(other.user))
			return false;
		return true;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((user == null) ? 0 : user.hashCode());
		return result;
	}
	public double getParziale() {
		BigDecimal i = getTotale();
		if(i == null)  {
			return 0;
		} else {
			return i.doubleValue();
		}
	}
	//@NotAudited
	@Formula(value = "(SELECT sum(a.valore * a.gettone) FROM " + ConstantsGlobal.tp + "estratto_dettaglio a WHERE a.estratto_id = id)")
	private BigDecimal totale;
	public BigDecimal getTotale() {
		if(totale == null)
			return new BigDecimal(0);
		return totale;
	}
	public void setTotale(BigDecimal totale) {
		this.totale = totale;
	}
	//@NotAudited
	@Formula(value = "(SELECT count(a.id) FROM " + ConstantsGlobal.tp + "estratto_dettaglio a WHERE a.estratto_id = id)")
	private Long ndettagli;
	public Long getNdettagli() {
		if(ApplicationUtil.ltZero(ndettagli))
			return Long.valueOf(0);
		return ndettagli;
	}
	public void setNdettagli(Long ndettagli) {
		this.ndettagli = ndettagli;
	}
}