package it.callite.model;
import java.io.Serializable;
import java.math.BigDecimal;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "estratto_dettaglio")
public class EstrattoDettaglio implements Serializable, Comparable<EstrattoDettaglio> {
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@ManyToOne
	private Estratto estratto;
	private BigDecimal valore = new BigDecimal(0);
	private BigDecimal gettone = new BigDecimal(1);
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public Estratto getEstratto() {
		return estratto;
	}
	public void setEstratto(Estratto estratto) {
		this.estratto = estratto;
	}
	public BigDecimal getValore() {
		if(valore == null)
			return new BigDecimal(0);
		return valore;
	}
	public void setValore(BigDecimal valore) {
		this.valore = valore;
	}
	public BigDecimal getGettone() {
		if(gettone == null)
			return new BigDecimal(0);
		return gettone;
	}
	public void setGettone(BigDecimal gettone) {
		this.gettone = gettone;
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public BigDecimal getImporto() {
		return getGettone().multiply(getValore());
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	@Column(columnDefinition = "text")
	private String descrizione;
	public int compareTo(EstrattoDettaglio o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EstrattoDettaglio other = (EstrattoDettaglio) obj;
		if (descrizione == null) {
			if (other.descrizione != null)
				return false;
		} else if (!descrizione.equals(other.descrizione))
			return false;
		if (estratto == null) {
			if (other.estratto != null)
				return false;
		} else if (!estratto.equals(other.estratto))
			return false;
		if (gettone == null) {
			if (other.gettone != null)
				return false;
		} else if (!gettone.equals(other.gettone))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (valore == null) {
			if (other.valore != null)
				return false;
		} else if (!valore.equals(other.valore))
			return false;
		return true;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((descrizione == null) ? 0 : descrizione.hashCode());
		result = prime * result
				+ ((estratto == null) ? 0 : estratto.hashCode());
		result = prime * result + ((gettone == null) ? 0 : gettone.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((valore == null) ? 0 : valore.hashCode());
		return result;
	}
}