package it.callite.model;
import java.io.Serializable;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.anagrafica.IndirizzoXML;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Coordinata;
import it.callite.utility.GCoder;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "indirizzo", indexes = { @Index(name= "indirizzo1", columnList = "regione, provincia"), @Index(name= "indirizzo2", columnList = "comune, cap" ), @Index(name= "indirizzo3", columnList = "strada, civico" ), @Index(name= "indirizzo4", columnList = "latitudine, longitudine")})
public class Indirizzo implements Serializable {
	private String regione;
	private String provincia;
	private String cap;
	private String comune;
	private String strada;
	@Column(columnDefinition = "text")
	private String presso;
	private String civico;
	@Column(columnDefinition = "text")
	private String dettagli;
	private String latitudine;
	private String longitudine;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public String getRegione() {
		return regione;
	}
	public void setRegione(String regione) {
		this.regione = ApplicationUtil.normalizeToUpperNull(regione);
	}
	public String getProvincia() {
		return provincia;
	}
	public void setProvincia(String provincia) {
		this.provincia = ApplicationUtil.normalizeToUpperNull(provincia);
	}
	public String getCap() {
		return cap;
	}
	public void setCap(String cap) {
		this.cap = ApplicationUtil.normalizeToUpperNull(cap);
	}
	public String getComune() {
		return comune;
	}
	public void setComune(String comune) {
		this.comune = ApplicationUtil.normalizeToUpperNull(comune);
	}
	public List<Agente> getAgente_legale() {
		return agente_legale;
	}
	public void setAgente_legale(List<Agente> agente_legale) {
		this.agente_legale = agente_legale;
	}
	public List<Agente> getAgente_operativo() {
		return agente_operativo;
	}
	public void setAgente_operativo(List<Agente> agente_operativo) {
		this.agente_operativo = agente_operativo;
	}
	public List<Anagrafica> getAnagrafica_operativo() {
		return anagrafica_operativo;
	}
	public void setAnagrafica_operativo(List<Anagrafica> anagrafica_operativo) {
		this.anagrafica_operativo = anagrafica_operativo;
	}
	public List<Anagrafica> getAnagrafica_legale() {
		return anagrafica_legale;
	}
	public void setAnagrafica_legale(List<Anagrafica> anagrafica_legale) {
		this.anagrafica_legale = anagrafica_legale;
	}
	public String getStrada() {
		return ApplicationUtil.upperCaseEmpty(strada);
	}
	public void setStrada(String strada) {
		this.strada = ApplicationUtil.normalizeToUpperNull(strada);
	}
	public String getPresso() {
		return ApplicationUtil.trimToEmpty(presso);
	}
	public void setPresso(String presso) {
		this.presso =  ApplicationUtil.trimToNull(presso);
	}
	public String getCivico() {
		return  ApplicationUtil.upperCaseEmpty(civico);
	}
	public void setCivico(String civico) {
		this.civico =  ApplicationUtil.normalizeToUpperNull(civico);
	}
	public String getDettagli() {
		return  ApplicationUtil.trimToEmpty(dettagli);
	}
	public void setDettagli(String dettagli) {
		this.dettagli =  ApplicationUtil.trimToNull(dettagli);
	}
	@OneToMany(mappedBy="indirizzo_legale")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Agente> agente_legale; 
	@OneToMany(mappedBy="indirizzo_operativo")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Agente> agente_operativo; 
	@OneToMany(mappedBy="indirizzo_operativo")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Anagrafica> anagrafica_operativo; 
	@OneToMany(mappedBy="indirizzo_legale")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Anagrafica> anagrafica_legale; 
	@OneToMany(mappedBy="destinatario")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Rmail> rmail; 
	
	
	public List<Rmail> getRmail() {
		return rmail;
	}
	public void setRmail(List<Rmail> rmail) {
		this.rmail = rmail;
	}
	
	public String getComune_completo() {
		return getIndirizzo1(false);
	}
	
	public String getStrada_completo() {
		return getIndirizzo2(false);
	}
	public String getIndirizzo_completo() {
		StringBuilder sb = new StringBuilder();
		String comune_completo = ApplicationUtil.trimToNull(getIndirizzo1(false));
		String strada_completo = ApplicationUtil.trimToNull(getIndirizzo2(false));
		if(comune_completo != null) {
			sb.append(comune_completo);
		} 
		if(comune_completo != null && strada_completo != null) {
			sb.append(" - ");
		}
		if(strada_completo != null) {
			sb.append(strada_completo);
		} 
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public Boolean getValidoMap() {
		String comune = ApplicationUtil.trimToNull(getComune());
		String strada = ApplicationUtil.trimToNull(getStrada());
		return comune != null && strada != null;
	}
	
	public Coordinata addressMap(String key) {
		if(ApplicationUtil.isTrue(getValidoMap())) {
			StringBuilder sb = new StringBuilder();
			String comune_completo = ApplicationUtil.trimToNull(getIndirizzo1(true));
			String strada_completo = ApplicationUtil.trimToNull(getIndirizzo2(true));
			if(comune_completo != null) {
				sb.append(comune_completo);
			} 
			if(comune_completo != null && strada_completo != null) {
				sb.append(", ");
			}
			if(strada_completo != null) {
				sb.append(strada_completo);
			} 
			String indirizzo = ApplicationUtil.trimToEmpty(sb.toString());
			try {
				return GCoder.geocode(indirizzo, key);
			} catch (Exception e) {}
		}
		return null;
	}
	public void geocode(String key) {
		Coordinata coord = addressMap(key);
		if(coord != null) {
			this.setLatitudine(coord.getLatitudine());
			this.setLongitudine(coord.getLongitudine());
		}
	}
	public Indirizzo() {
	}
	public String getLatitudine() {
		return ApplicationUtil.normalizeToNull(latitudine);
	}
	public void setLatitudine(String latitudine) {
		this.latitudine = ApplicationUtil.normalizeToNull(latitudine);
	}
	
	public Boolean getCoordinateValide() {
		return getLatitudine() != null && getLongitudine() != null;
	}
	public String getLongitudine() {
		return ApplicationUtil.normalizeToNull(longitudine);
	}
	public void setLongitudine(String longitudine) {
		this.longitudine = ApplicationUtil.normalizeToNull(longitudine);
	}
	public Indirizzo(IndirizzoXML indirizzo) {
		this.setRegione(indirizzo.getRegione());
		this.setProvincia(indirizzo.getProvincia());
		this.setCap(indirizzo.getCap());
		this.setComune(indirizzo.getComune());
		this.setStrada(indirizzo.getStrada());
		this.setPresso(indirizzo.getPresso());
		this.setCivico(indirizzo.getCivico());
		this.setDettagli(indirizzo.getDettagli());
		this.setLatitudine(indirizzo.getLatitudine());
		this.setLongitudine(indirizzo.getLongitudine());
	}
	public String getIndirizzo1(boolean map) {
		StringBuilder sb = new StringBuilder();
		String cap = ApplicationUtil.trimToNull(getCap());
		String comune = ApplicationUtil.trimToNull(getComune());
		String provincia =  ApplicationUtil.trimToNull(getProvincia());
		if(cap != null) {
			sb.append(cap + " ");
		} 
		if(comune != null) {
			sb.append(comune + " ");
		} 
		if(provincia != null) {
			if(!map) {
				sb.append("(" + provincia + ")");
			} else {
				sb.append(provincia);
			}
		} 
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	public String getIndirizzo2(boolean map) {
		StringBuilder sb = new StringBuilder();
		String strada = ApplicationUtil.trimToNull(getStrada());
		String civico =  ApplicationUtil.trimToNull(getCivico());
		if(strada != null) {
			sb.append(strada);
		} 
		if(civico != null) {
			if(!map) {
				sb.append(", " + civico);
			} else {
				sb.append(" " + civico);
			}
		} 
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
	
	public String getIndirizzo_completoP() {
		StringBuilder sb = new StringBuilder();
		String regione = ApplicationUtil.trimToNull(getRegione());
		
		String comune_completo = ApplicationUtil.trimToNull(getIndirizzo1(false));
		String strada_completo = ApplicationUtil.trimToNull(getIndirizzo2(false));
		if(comune_completo != null) {
			sb.append(comune_completo);
		} 
		if(comune_completo != null && strada_completo != null) {
			sb.append(" - ");
		}
		if(strada_completo != null) {
			sb.append(strada_completo);
		} 
		String indirizzo = ApplicationUtil.trimToNull(sb.toString());
		if(indirizzo != null && regione != null) {
			sb.append(" - ");
		} 
		if(regione != null) {
			sb.append(regione);
		} 
		return ApplicationUtil.trimToEmpty(sb.toString());
	}
}