package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.message.Message;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited
@Table(name = ConstantsGlobal.tp + "invoice", indexes = { @Index(name= "invoice", columnList = "data_creazione, nome" )})
public class Invoice  extends SelezionatoAbstract implements Serializable, Comparable<Invoice> {


	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	public Invoice() {
	}
	public int compareTo(Invoice o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	private Date data_creazione = ApplicationUtil.date();
	private Boolean header = Boolean.FALSE;

	@Column(columnDefinition = "text")
	private String link;
	private String nome;
	@Column(columnDefinition = "text")
	private Boolean password = Boolean.FALSE;
	private String type = ConstantsMap.csv;

	private String orario = "20";
	@Column(columnDefinition = "text")
	private String queries;
	private Boolean started = Boolean.TRUE;
	private Integer tipo = 0;

	public Integer getTipo() {
		if(tipo == null) {
			tipo = 0;
		}
		return tipo;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	@Transient
	private List<Integer> valore_multiplo;

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Invoice other = (Invoice) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		if (started == null) {
			if (other.started != null)
				return false;
		} else if (!started.equals(other.started))
			return false;
		return true;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}

	public Long getId() {
		return id;
	}
	public String getLink() {
		return ApplicationUtil.trimToNull(link);
	};
	public String getNome() {
		return ApplicationUtil.trimToNull(nome);
	}
	public String getOrario() {
		return orario;
	}

	public String getQueries() {
		return ApplicationUtil.trimToNull(queries);
	}
	public String getOrarioString() {
		return ApplicationUtil.trimToEmpty(orario);
	}
	public Boolean getStarted() {
		return started;
	}
	public List<Integer> getValore_multiplo() {
		if(ApplicationUtil.eq(tipo, 0)) {
			return ApplicationUtil.tokenI(orario, ConstantsGlobal.commaSeparator);
		}
		return null;
	}
	public List<SelectItem> getValori() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		for(int i=0; i<24; i++){
			Integer ora = Integer.valueOf(i); 
			list.add(new SelectItem(ora, ora.toString()));
		}
		return list;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		result = prime * result + ((started == null) ? 0 : started.hashCode());
		return result;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public Boolean getHeader() {
		return header;
	}
	public void setHeader(Boolean header) {
		this.header = header;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setLink(String link) {
		this.link = ApplicationUtil.trimToNull(link);
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.trimToNull(nome);
	}
	public void setOrario(String orario) {
		this.orario = orario;
	}
	public void setQueries(String queries) {
		this.queries = ApplicationUtil.trimToNull(queries);
	}
	public void setStarted(Boolean started) {
		this.started = started;
	}
	public void setValore_multiplo(List<Integer> valoreMultiplo) {
		orario = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.commaSeparator);
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	private String separatore;
	private String quote;

	public Character getQuoteChar() {
		return ApplicationUtil.getChar(quote);
	}

	public Character getSeparatoreChar() {
		return ApplicationUtil.getChar(separatore);
	}

	public List<SelectItem> getQuoteList() {
		return ConstantsMap.getQuoteList();
	}

	public List<SelectItem> getSeparatoreList() {
		return ConstantsMap.getSeparatoreList();
	}
	public String getSeparatore() {
		return separatore;
	}
	public void setSeparatore(String separatore) {
		this.separatore = separatore;
	}
	public String getQuote() {
		return quote;
	}
	public void setQuote(String quote) {
		this.quote = quote;
	}
	public Boolean getPassword() {
		return password;
	}
	public void setPassword(Boolean password) {
		this.password = password;
	}
	public String getType() {
		return type;
	}
	public void setType(String type) {
		this.type = type;
	}
	public boolean isValid() {
		return type != null && type.equals(ConstantsMap.csv);
	}

	public List<SelectItem> getTipoList() {
		List<SelectItem> valore = new ArrayList<SelectItem>();
		valore.add(new SelectItem(ConstantsMap.csv, Message.addCatalogText("csv")));
		valore.add(new SelectItem(ConstantsMap.xlsx, Message.addCatalogText("excel")));
		valore.add(new SelectItem(ConstantsMap.pdf, Message.addCatalogText("pdf")));
		return valore;
	}
}