package it.callite.model;
import java.io.Serializable;
import java.sql.Time;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.message.Message;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "lista", indexes = { @Index(name= "lista", columnList = "data_creazione, nome, inverti, data_inizio, data_fine, week_inizio, week_fine, ora_inizio, ora_fine, stato, priorita, ordine") })
public class Lista extends SelezionatoAbstract implements Serializable, Comparable<Lista>  {
	@Column(columnDefinition = "text")
	private String callers;
	private String codice;
	public String getCodice() {
		return ApplicationUtil.trimToEmpty(codice);
	}
	public void setCodice(String codice) {
		this.codice = ApplicationUtil.normalizeToEmpty(codice);
	}
	public String getCallers() {
		return ApplicationUtil.trimToNull(callers);
	}
	public void setCallers(String callers) {
		this.callers = ApplicationUtil.aggiustaLista(false, ConstantsGlobal.colonSeparator, callers);
	}
	@Column(columnDefinition = "text")
	private String musicclass;
	public String getMusicclass() {
		return ApplicationUtil.normalizeToNull(musicclass);
	}
	public void setMusicclass(String musicclass) {
		this.musicclass = ApplicationUtil.normalizeToNull(musicclass);
	}
	
	private Short telefono = null;
	public Short getTelefono() {
		if(ApplicationUtil.ltZero(telefono)) {
			telefono = 0;
		}
		return telefono;
	}
	private Long week_fine;
	private Long week_inizio;
	
	public Long getWeek_fine() {
		return week_fine;
	}
	public void setWeek_fine(Long week_fine) {
		this.week_fine = week_fine;
	}
	public Long getWeek_inizio() {
		return week_inizio;
	}
	public void setWeek_inizio(Long week_inizio) {
		this.week_inizio = week_inizio;
	}
	public void setTelefono(Short telefono) {
		this.telefono = telefono;
	}
	private String latitudine;
	private String longitudine;
	public String getLatitudine() {
		return ApplicationUtil.normalizeToNull(latitudine);
	}
	public void setLatitudine(String latitudine) {
		this.latitudine = ApplicationUtil.normalizeToNull(latitudine);
	}
	public String getLongitudine() {
		return ApplicationUtil.normalizeToNull(longitudine);
	}
	public void setLongitudine(String longitudine) {
		this.longitudine = ApplicationUtil.normalizeToNull(longitudine);
	}
	private String rem_prefisso;
	private String add_prefisso;
	private String append_prefisso;
	public String getAppend_prefisso() {
		return ApplicationUtil.normalizeToNull(append_prefisso);
	}
	public void setAppend_prefisso(String append_prefisso) {
		this.append_prefisso = ApplicationUtil.normalizeToNull(append_prefisso);
	}
	public String getAdd_prefisso() {
		return ApplicationUtil.normalizeToEmpty(add_prefisso);
	}
	public String getRem_prefisso() {
		return ApplicationUtil.normalizeToEmpty(rem_prefisso);
	}
	public void setAdd_prefisso(String add_prefisso) {
		this.add_prefisso = ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, add_prefisso));
	}
	public void setRem_prefisso(String rem_prefisso) {
		this.rem_prefisso = ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, rem_prefisso));
	}

	private Boolean consenso_telefono = Boolean.TRUE;
	
	public Boolean getConsenso_telefono() {
		if(consenso_telefono == null) {
			consenso_telefono = Boolean.TRUE;
		}
		return consenso_telefono;
	}
	public void setConsenso_telefono(Boolean consenso_telefono) {
		this.consenso_telefono = consenso_telefono;
	}
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "campagna_lista", joinColumns=@JoinColumn(name="liste_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagna_id", nullable=false, updatable=true, insertable=true)    )
	private List<Campagna> campagna;
	@OneToMany(mappedBy = "lista")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contatto> chiamate;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "lista_anagrafica", joinColumns=@JoinColumn(name="lista_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="contatti_id", nullable=false, updatable=true, insertable=true)    )
	private List<Anagrafica> contatti;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_fine;
	private Date data_inizio;
	@Column(columnDefinition = "text")
	private String descrizione;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	
	private String nome;
	private Time ora_fine;
	private Time ora_inizio;
	private String ordine;
	@OneToMany(mappedBy = "lista")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Prenotazione> prenotazioni;
	private Integer priorita = ConstantsGlobal.defaultPriorita;
	private Integer stato = ConstantsGlobal.stato_attivo;
	public int compareTo(Lista o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Lista other = (Lista) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	
	public Boolean getStatoAttiva() {
		return ApplicationUtil.calcolaIntervallo(getInverti(), getData_inizio(), getData_fine(), getWeek_inizio(), getWeek_fine(), getTime_inizio(), getTime_fine());
	}
	
	public String getStatoAttivaT() {
		return getStatoAttiva() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public List<Campagna> getCampagna() {
		return campagna;
	}
	public List<Contatto> getChiamate() {
		return chiamate;
	}
	public List<Anagrafica> getContatti() {
		if(contatti == null) contatti = new ArrayList<Anagrafica>();
		return contatti;
	}
	public Lista getCopia() {
		Lista copia = new Lista();
		copia.setNome(this.getNome() + " " + ConstantsLanguage.COPY.stampa());
		copia.setDescrizione(this.getDescrizione());
		copia.setData_fine(this.getData_fine());
		copia.setData_inizio(this.getData_inizio());
		copia.setPriorita(this.getPriorita());
		copia.setCodice(this.getCodice());
		copia.setStato(this.getStato());
		copia.setOra_inizio(this.getOra_inizio());
		copia.setOra_fine(this.getOra_fine());
		copia.setInverti(this.getInverti());
		copia.setOrdine(this.getOrdine());
		copia.setCallers(this.getCallers());
		copia.setMusicclass(this.getMusicclass());
		copia.setTelefono(this.getTelefono());
		copia.setAdd_prefisso(this.getAdd_prefisso());
		copia.setRem_prefisso(this.getRem_prefisso());
		copia.setAppend_prefisso(this.getAppend_prefisso());
		copia.setConsenso_telefono(this.getConsenso_telefono());
		copia.setLatitudine(this.getLatitudine());
		copia.setLongitudine(this.getLongitudine());
		copia.setWeek_fine(this.getWeek_fine());
		copia.setWeek_inizio(this.getWeek_inizio());
		return copia;
	}
	private Boolean inverti = false;
	public Boolean getInverti() {
		if(inverti == null) {
			inverti = false;
		}
		return inverti;
	}
	public void setInverti(Boolean inverti) {
		this.inverti = inverti;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_fine() {
		return data_fine;
	}
	public Date getData_inizio() {
		return data_inizio;
	}
	public String getDenominazione() {
		String codice = getCodice();
		return (ApplicationUtil.isNotBlank(codice) ? (codice + " - ") : "") + ApplicationUtil.trimToEmpty(nome) + " (" + getDescrizioneStato() +(getStatoAttiva() ? "" : "*") + ")";
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public String getDescrizioneStato() {
		return Message.addCatalogText("ts_stato." + stato);
	}
	public Long getId() {
		return id;
	}
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	public String getNome() {
		return nome;
	}
	public Date getOra_fine() {
		return ApplicationUtil.timeToDate(ora_fine);
	}
	public Date getOra_inizio() {
		return  ApplicationUtil.timeToDate(ora_inizio);
	}
	public String getOrdine() {
		return ApplicationUtil.trimToEmpty(ordine);
	}
	public Boolean getOrdineA() {
		return ConstantsMap.getOrdineList().contains(ApplicationUtil.trimToEmpty(ordine));
	}
	public List<Prenotazione> getPrenotazioni() {
		return prenotazioni;
	}
	public Integer getPriorita() {
		if(ApplicationUtil.ltZero(priorita)) {
			priorita = ConstantsGlobal.defaultPriorita;
		}
		return priorita;
	}
	public Integer getStato() {
		return stato;
	}
	public Time getTime_fine() {
		return  ora_fine;
	}
	public Time getTime_inizio() {
		return  ora_inizio;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setCampagna(List<Campagna> campagna) {
		this.campagna = campagna;
	}
	public void setChiamate(List<Contatto> chiamate) {
		this.chiamate = chiamate;
	}
	public void setContatti(List<Anagrafica> contatti) {
		this.contatti = contatti;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_fine(Date data_fine) {
		this.data_fine = data_fine;
	}
	public void setData_inizio(Date data_inizio) {
		this.data_inizio = data_inizio;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setId(Long id) {
		this.id = id;
	}
	
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setOra_fine(Date ora_fine) {
		this.ora_fine = ApplicationUtil.dateToTime(ora_fine);
	}
	public void setOra_inizio(Date ora_inizio) {
		this.ora_inizio = ApplicationUtil.dateToTime(ora_inizio);
	}
	public void setOrdine(String ordine) {
		this.ordine = ApplicationUtil.trimToNull(ordine);
	}
	public void setPrenotazioni(List<Prenotazione> prenotazioni) {
		this.prenotazioni = prenotazioni;
	}
	public void setPriorita(Integer priorita) {
		this.priorita = priorita;
	}
	public void setStato(Integer stato) {
		this.stato = stato;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	@OneToMany
	@JoinColumn(name = "lista_id", nullable = true, updatable=true, insertable=true)
	private List<Workflow> destinazioni;
	public List<Workflow> getDestinazioni() {
		return destinazioni;
	}
	public void setDestinazioni(List<Workflow> destinazioni) {
		this.destinazioni = destinazioni;
	}
	@OneToMany
	@JoinColumn(name = "lista_id", nullable = true, updatable=true, insertable=true)
	private List<StoriaContatto> storia_contatti;
	public List<StoriaContatto> getStoria_contatti() {
		return storia_contatti;
	}
	public void setStoria_contatti(List<StoriaContatto> storia_contatti) {
		this.storia_contatti = storia_contatti;
	}
}