package it.callite.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;






@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "marcatura", indexes = { @Index(name= "marcatura", columnList = "data_creazione, data_inizio, data_fine") })
public class Marcatura extends SelezionatoAbstract  implements Serializable, Comparable<Marcatura> {

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	@ManyToOne
	private Utente user;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();
	public Date getData_creazione() {
		return data_creazione;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	private Date data_inizio;
	private Date data_fine;

	public int compareTo(Marcatura o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Marcatura other = (Marcatura) obj;
		if (data_fine == null) {
			if (other.data_fine != null)
				return false;
		} else if (!data_fine.equals(other.data_fine))
			return false;
		if (data_inizio == null) {
			if (other.data_inizio != null)
				return false;
		} else if (!data_inizio.equals(other.data_inizio))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (user == null) {
			if (other.user != null)
				return false;
		} else if (!user.equals(other.user))
			return false;
		return true;
	}

	public Date getData_fine() {
		return data_fine;
	}
	public Date getData_inizio() {
		return data_inizio;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getUser().getDenominazione());
	}
	public String getDurata() {
		if(data_fine != null) {
			Long secondi = getDurataTime();
			return ApplicationUtil.format(secondi);
		} 
		return null;
	}
	public Long getDurataTime() {
		Long secondi = null;
		if(data_fine!=null && data_inizio!=null) {
			secondi =  (data_fine.getTime() - data_inizio.getTime())/ConstantsGlobal.millis;
		}
		return secondi;
	}
	public Long getId() {
		return id;
	}
	public Utente getUser() {
		return user;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((data_fine == null) ? 0 : data_fine.hashCode());
		result = prime * result
				+ ((data_inizio == null) ? 0 : data_inizio.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((user == null) ? 0 : user.hashCode());
		return result;
	}

	public void setData_fine(Date data_fine) {
		this.data_fine = data_fine;
	}

	public void setData_inizio(Date data_inizio) {
		this.data_inizio = data_inizio;
	}


	public void setId(Long id) {
		this.id = id;
	}
	public void setUser(Utente user) {
		this.user = user;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
}