package it.callite.model;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.Allegato;
import it.callite.utility.AllegatoUtils;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.thread.Posta;



@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "notifica", indexes = { @Index(name= "notifica", columnList = "data_creazione, data_invio, codice"), @Index(name = "id", columnList = "userfield") })
public class Notifica extends SelezionatoAbstract implements Serializable, Posta, Comparable<Notifica> {


	private String userfield;
	public String getUserfield() {
		return userfield;
	}
	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}
	public void impostaUserfield() {
		if(contatto != null && userfield == null) {
			userfield = contatto.getUserfield();
		}
	}
	private Integer codice;

	@ManyToOne
	private Contatto contatto;

	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();


	private Date data_invio;
	
	@Column(columnDefinition = "text")
	private String destinatario;
	
	@Column(columnDefinition = "text")
	private String cc;
	@Column(columnDefinition = "text")
	private String sender;
	@Column(columnDefinition = "text")
	private String template;


	public String getCc() {
		return cc;
	}
	public void setCc(String cc) {
		this.cc = cc;
	}
	public String getSender() {
		return sender;
	}
	public void setSender(String sender) {
		this.sender = sender;
	}
	public String getTemplate() {
		return template;
	}
	public void setTemplate(String template) {
		this.template = template;
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	@Column(columnDefinition = "text")
	private String oggetto;

	@Column(columnDefinition = "text")
	private String stato;

	@Column(columnDefinition = "text")
	private String testo;

	@ManyToOne
	private Utente utente;

	public void addMotivo(Integer codice, String ragione) {
		setCodice(codice);
		String s = "";
		if(ApplicationUtil.equals(codice, ConstantsGlobal.IN_INVIO)) {
			s = ConstantsLanguage.TESTO_IN_INVIO.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.INVIATA)) {
			s = ConstantsLanguage.TESTO_INVIATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.NON_INVIATA)) {
			s = ConstantsLanguage.TESTO_NON_INVIATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.SCHEDULATA)) {
			s = ConstantsLanguage.TESTO_SCHEDULATA.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.PAUSE)) {
			s = ConstantsLanguage.TESTO_PAUSE.stampa();
		} else if(ApplicationUtil.equals(codice, ConstantsGlobal.ANNULLATA)) {
			s = ConstantsLanguage.TESTO_ANNULLATA.stampa();
		} else {
			s = ConstantsLanguage.TESTO_IN_INVIO.stampa();
		}
		if(ApplicationUtil.isNotBlank(ragione)) {
			setStato(s + ": " + ragione);
		} else {
			setStato(s);
		}
	}

	public int compareTo(Notifica o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Notifica other = (Notifica) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	@Lob
	private byte [] allegatiTemplates;

	public byte[] getAllegatiTemplates() {
		return allegatiTemplates;
	}

	public void setAllegatiTemplates(byte[] allegatiTemplates) {
		this.allegatiTemplates = allegatiTemplates;
	}

	public void setAllegatiTemplateList(List<Allegato> allegati) {
		this.allegatiTemplates = ApplicationUtil.setAllegatiTemplateList(allegati);
	}
	
	public List<AllegatoUtils> getAllegatiTemplateList() {
		return ApplicationUtil.getAllegatiTemplateList(allegatiTemplates);
	}

	public List<Allegato> getAttachments() {
		return ApplicationUtil.getAllegatiReinvioList(allegatiTemplates);
	}


	public Integer getCodice() {
		return codice;
	}

	public Contatto getContatto() {
		return contatto;
	}

	public Date getData_creazione() {
		return data_creazione;
	}

	public Date getData_invio() {
		return data_invio;
	}


	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(contatto.getDenominazione());
	}

	public String getDestinatario() {
		return destinatario;
	}


	public Long getId() {
		return id;
	}

	public String getOggetto() {
		return oggetto;
	}

	public String getStato() {
		return stato;
	}

	public String getTesto() {
		return testo;
	}

	public Utente getUtente() {
		return utente;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}


	public void setCodice(Integer codice) {
		this.codice = codice;
	}

	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public void setData_invio(Date dataInvio) {
		data_invio = dataInvio;
	}

	public void setDestinatario(String destinatario) {
		this.destinatario = ApplicationUtil.trimToNull(destinatario);
	}


	public void setId(Long id) {
		this.id = id;
	}

	public void setOggetto(String oggetto) {
		this.oggetto = oggetto;
	}

	public void setStato(String stato) {
		this.stato = stato;
	}


	public void setTesto(String testo) {
		this.testo = testo;
	}

	public void setUtente(Utente utente) {
		this.utente = utente;
	}


	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public boolean valido() {
		return ApplicationUtil.isNotBlank(this.getDestinatario());
	}
	
	public boolean inviabile() {
		return true;
	}
	public String getUtente_denominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}

	public Boolean getHtml() {
		return true;
	}
	
	public String getTo() {
		return destinatario;
	}

	public String getSubject() {
		return oggetto;
	}

	public String getText() {
		return testo;
	}
}