package it.callite.model;
import java.io.Serializable;
import java.sql.Time;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.allegato.AllegatoNotizia;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "notizia", indexes = { @Index(name= "notizia", columnList = "data_creazione, inverti, data_inizio, data_fine, week_inizio, week_fine, ora_inizio, ora_fine") })
public class Notizia  extends SelezionatoAbstract implements Serializable, Comparable<Notizia> {
	private Long week_fine;
	private Long week_inizio;
	
	public Long getWeek_fine() {
		return week_fine;
	}
	public void setWeek_fine(Long week_fine) {
		this.week_fine = week_fine;
	}
	public Long getWeek_inizio() {
		return week_inizio;
	}
	public void setWeek_inizio(Long week_inizio) {
		this.week_inizio = week_inizio;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getOggetto());
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Boolean attivo = Boolean.TRUE;
	private Boolean eliminabile = Boolean.TRUE;
	public Boolean getEliminabile() {
		return eliminabile;
	}
	public void setEliminabile(Boolean eliminabile) {
		this.eliminabile = eliminabile;
	}
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "notizia_utente", joinColumns=@JoinColumn(name="notizia_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="utente_id", nullable=false, updatable=true, insertable=true)    )
	private List<Utente> utenti;
	public List<Utente> getUtenti() {
		return utenti;
	}
	public void setUtenti(List<Utente> utenti) {
		this.utenti = utenti;
	}
	@OneToMany(mappedBy = "notizia")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AllegatoNotizia> allegati;
	public List<AllegatoNotizia> getAllegati() {
		return allegati;
	}
	public void setAllegati(List<AllegatoNotizia> allegati) {
		this.allegati = allegati;
	}
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	public Boolean getAttivo() {
		return attivo;
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}
	private Date data_creazione = ApplicationUtil.date();
	private Date data_inizio;
	private Date data_fine;
	@Column(columnDefinition = "text")
	private String oggetto;
	@Column(columnDefinition = "text")
	private String testo;
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public Date getData_inizio() {
		return ApplicationUtil.aggiustaDataT(data_inizio, true);
	}
	public void setData_inizio(Date data_inizio) {
		this.data_inizio = ApplicationUtil.aggiustaDataT(data_inizio, true);
	}
	private Time ora_inizio;
	private Time ora_fine;
	public Time getTime_inizio() {
		return  ora_inizio;
	}
	public Time getTime_fine() {
		return  ora_fine;
	}
	public Date getOra_inizio() {
		return  ApplicationUtil.timeToDate(ora_inizio);
	}
	public void setOra_inizio(Date ora_inizio) {
		this.ora_inizio = ApplicationUtil.dateToTime(ora_inizio);
	}
	public Date getOra_fine() {
		return ApplicationUtil.timeToDate(ora_fine);
	}
	private Boolean inverti = false;
	public Boolean getInverti() {
		if(inverti == null) {
			inverti = false;
		}
		return inverti;
	}
	public void setInverti(Boolean inverti) {
		this.inverti = inverti;
	}
	public void setOra_fine(Date ora_fine) {
		this.ora_fine = ApplicationUtil.dateToTime(ora_fine);
	}
	public Date getData_fine() {
		return ApplicationUtil.aggiustaDataT(data_fine, false);
	}
	public void setData_fine(Date data_fine) {
		this.data_fine = ApplicationUtil.aggiustaDataT(data_fine, false);
	}
	public String getOggetto() {
		return oggetto;
	}
	public void setOggetto(String oggetto) {
		this.oggetto = oggetto;
	}
	public String getTesto() {
		return testo;
	}
	public void setTesto(String testo) {
		this.testo = testo;
	}
	public int compareTo(Notizia o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result
				+ ((data_fine == null) ? 0 : data_fine.hashCode());
		result = prime * result
				+ ((data_inizio == null) ? 0 : data_inizio.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((oggetto == null) ? 0 : oggetto.hashCode());
		result = prime * result + ((testo == null) ? 0 : testo.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Notizia other = (Notizia) obj;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (data_fine == null) {
			if (other.data_fine != null)
				return false;
		} else if (!data_fine.equals(other.data_fine))
			return false;
		if (data_inizio == null) {
			if (other.data_inizio != null)
				return false;
		} else if (!data_inizio.equals(other.data_inizio))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (oggetto == null) {
			if (other.oggetto != null)
				return false;
		} else if (!oggetto.equals(other.oggetto))
			return false;
		if (testo == null) {
			if (other.testo != null)
				return false;
		} else if (!testo.equals(other.testo))
			return false;
		return true;
	}
	@Transient
	private List<Allegato> allegato;
	public List<Allegato> getAllegato() {
		if(allegato==null)
			allegato = new ArrayList<Allegato>();
		return allegato;
	}
	public int getAllegatoSize() {
		return ApplicationUtil.size(allegato);
	}
	public Boolean getStatoAttiva() {
		return ApplicationUtil.calcolaIntervallo(getInverti(), getData_inizio(), getData_fine(), getWeek_inizio(), getWeek_fine(), getTime_inizio(), getTime_fine());
	}
	
	public String getStatoAttivaT() {
		return getStatoAttiva() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	
	public Notizia getCopia() {
		Notizia pr = new Notizia();
		pr.setOggetto(this.getOggetto() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setTesto(this.getTesto());
		pr.setAttivo(this.getAttivo());
		pr.setEliminabile(this.getEliminabile());
		pr.setInverti(this.getInverti());
		pr.setOra_inizio(this.getOra_inizio());
		pr.setOra_fine(this.getOra_fine());
		pr.setData_fine(this.getData_fine());
		pr.setData_inizio(this.getData_inizio());
		pr.setWeek_fine(this.getWeek_fine());
		pr.setWeek_inizio(this.getWeek_inizio());
		return pr;
	}
}