package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "operazione", indexes = { @Index(name= "utente", columnList = "operazione, data_creazione") })
public class Operazione  extends SelezionatoAbstract implements Serializable, Comparable<Operazione> {
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getUser().getDenominazione() + " " + getDescrizioneShort());
	}
	
	public Operazione() {
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@ManyToOne
	private Utente user;
	public Utente getUser() {
		return user;
	}
	public void setUser(Utente user) {
		this.user = user;
	}
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	
	@Column(columnDefinition = "text")
	private String descrizione;
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public String getDescrizioneShort() {
		return ApplicationUtil.tronca(descrizione);
	}
	
	public String getDescrizioneLong() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	
	private Short operazione;
	public Operazione (Utente user) {
		this.user = user;
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	
	public Short getOperazione() {
		return operazione;
	}
	public void setOperazione(Short operazione) {
		this.operazione = operazione;
	}
	public String getOperazioneTesto() {
		return "text.operazione."+operazione;
	}
	public int compareTo(Operazione o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((operazione == null) ? 0 : operazione.hashCode());
		result = prime * result + ((user == null) ? 0 : user.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Operazione other = (Operazione) obj;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (operazione == null) {
			if (other.operazione != null)
				return false;
		} else if (!operazione.equals(other.operazione))
			return false;
		if (user == null) {
			if (other.user != null)
				return false;
		} else if (!user.equals(other.user))
			return false;
		return true;
	}
	
	
	
	
}