package it.callite.model;
import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "pausa", indexes = { @Index(name= "pausa", columnList = "chiave, positivo, data_creazione, data_inizio, data_fine") })
public class Pausa extends SelezionatoAbstract  implements Serializable, Comparable<Pausa> {
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getUser().getDenominazione());
	}
	
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String chiave;
	private Boolean positivo = Boolean.FALSE;
	
	@ManyToOne
	private Utente user;
	private Date data_inizio = ApplicationUtil.date();
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public Utente getUser() {
		return user;
	}
	public void setUser(Utente user) {
		this.user = user;
	}
	public Date getData_inizio() {
		return data_inizio;
	}
	public void setData_inizio(Date data_inizio) {
		this.data_inizio = data_inizio;
	}
	public Date getData_fine() {
		return data_fine;
	}
	public void setData_fine(Date data_fine) {
		this.data_fine = data_fine;
	}
	private Date data_fine;
	public String getDurata() {
		if(data_fine != null) {
			Long secondi = getDurataTime();
			return ApplicationUtil.format(secondi);
		} 
		return null;
	}
	public Long getDurataTime() {
		Long secondi = null;
		if(data_fine!=null && data_inizio!=null) {
			secondi =  (data_fine.getTime() - data_inizio.getTime())/ConstantsGlobal.millis;
		}
		return secondi;
	}
	
	public String getChiave() {
		return chiave;
	}
	public void setChiave(String chiave) {
		this.chiave = ApplicationUtil.normalizeToNull(chiave);
	}
	public Boolean getPositivo() {
		return positivo;
	}
	public void setPositivo(Boolean positivo) {
		this.positivo = positivo;
	}
	
	
	public String getChiaveS() {
		String c = ApplicationUtil.trimToNull(chiave);
		if(c != null) {
			return c + " " + (ApplicationUtil.isTrue(positivo) ? "[+]" : "[-]");
		}
		return "";
	}
	public int compareTo(Pausa o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((data_fine == null) ? 0 : data_fine.hashCode());
		result = prime * result + ((data_inizio == null) ? 0 : data_inizio.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((user == null) ? 0 : user.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Pausa other = (Pausa) obj;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (data_fine == null) {
			if (other.data_fine != null)
				return false;
		} else if (!data_fine.equals(other.data_fine))
			return false;
		if (data_inizio == null) {
			if (other.data_inizio != null)
				return false;
		} else if (!data_inizio.equals(other.data_inizio))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (user == null) {
			if (other.user != null)
				return false;
		} else if (!user.equals(other.user))
			return false;
		return true;
	}
}