package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.sincronizzazione.InfoPrenotazione;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "prenotazione", indexes = { @Index(name= "prenotazione", columnList = "data_creazione, telefono, stato") })
public class Prenotazione extends SelezionatoAbstract implements Serializable, Comparable<Prenotazione>  {
	@ManyToOne
	private  Agente  agente;
	@ManyToOne
	private Campagna campagna;
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	private Date data_creazione = ApplicationUtil.date();
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;
	@ManyToOne
	private Lista lista;
	@ManyToOne
	private  Utente operatore;
	private Integer stato = ConstantsGlobal.prenotazione_dainviare;
	private String telefono;
	private Integer tipo_telefono;
	public int compareTo(Prenotazione o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Prenotazione other = (Prenotazione) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Agente getAgente() {
		return agente;
	}
	public String getAgente_denominazione() {
		return  getAgente() != null ? getAgente().getDenominazione() : "";
	}
	public String getAgente_codice() {
		return getAgente() != null ? getAgente().getCodice() : "";
	}
	public Campagna getCampagna() {
		return campagna;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getStatoString() + " - " + getTipo_telefonoString() + " - " + getTelefono());
	}
	public Long getId() {
		return id;
	}
	public Boolean getInviabile() {
		return getStato().equals(ConstantsGlobal.prenotazione_dainviare);
	}
	public Lista getLista() {
		return lista;
	}
	
	public Utente getOperatore() {
		return operatore;
	}
	public Integer getStato() {
		return stato;
	}
	public String getStatoString() {
		if(getStato().equals(ConstantsGlobal.prenotazione_dainviare)) {
			return ConstantsLanguage.DAINVIARE.stampa();
		}
		if(getStato().equals(ConstantsGlobal.prenotazione_inviata)) {
			return ConstantsLanguage.INVIATA.stampa();
		}
		if(getStato().equals(ConstantsGlobal.prenotazione_duplicata)) {
			return ConstantsLanguage.DUPLICATA.stampa();
		}
		return "";
	}
	public String getTelefono() {
		return telefono;
	}
	public Integer getTipo_telefono() {
		return tipo_telefono;
	}
	public String getTipo_telefonoString() {
		if(ApplicationUtil.eq(getTipo_telefono(),1)) {
			return ConstantsLanguage.TELEFONO_MOBILE.stampa();
		} else if(ApplicationUtil.eq(getTipo_telefono(),2)) {
			return ConstantsLanguage.TELEFONO_FAX.stampa();
		} else if(ApplicationUtil.eq(getTipo_telefono(),3)) {
			return ConstantsLanguage.TELEFONO_ALTRO.stampa();
		} else {
			return ConstantsLanguage.TELEFONO_FISSO.stampa();
		}
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAgente(Agente agente) {
		this.agente = agente;
	}
	public void setCampagna(Campagna campagna) {
		this.campagna = campagna;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setLista(Lista lista) {
		this.lista = lista;
	}
	@OneToMany(mappedBy = "prenotazione")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<StoriaPrenotazione> prenotazioni;
	public List<StoriaPrenotazione> getPrenotazioni() {
		return prenotazioni;
	}
	public void setPrenotazioni(List<StoriaPrenotazione> prenotazioni) {
		this.prenotazioni = prenotazioni;
	}
	public void setOperatore(Utente operatore) {
		this.operatore = operatore;
	}
	public void setStato(Integer stato) {
		this.stato = stato;
	}
	public void setTelefono(String telefono) {
		this.telefono = ApplicationUtil.normalizeToNull(telefono);
	}
	public void setTipo_telefono(Integer tipo_telefono) {
		this.tipo_telefono = tipo_telefono;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public InfoPrenotazione getSincronia() {
		if(Chiamate.getCodaSincroniaPrenotazioni().contains(getId())) {
			return Chiamate.getCodaSincroniaPrenotazioni().get(getId());
		} 
		return null;
	}
	public String getLista_denominazione() {
		return getLista().getDenominazione();
	} 
	public String getCampagna_denominazione() {
		return getCampagna().getDenominazione();
	} 
	private Boolean bloccato = Boolean.FALSE;
	public String getBloccatoS() {
		return getBloccato() ? ConstantsLanguage.SI.stampa() : ConstantsLanguage.NO.stampa();
	}
	public Boolean getBloccato() {
		if(bloccato == null) {
			bloccato = Boolean.FALSE;
		}
		return bloccato;
	}
	public void setBloccato(Boolean bloccato) {
		this.bloccato = bloccato;
	}
	
	@ManyToOne
	private Utente utente;
	private Date data_lavorazione;
	
	public Utente getUtente() {
		return utente;
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	public Date getData_lavorazione() {
		return data_lavorazione;
	}
	public void setData_lavorazione(Date data_lavorazione) {
		this.data_lavorazione = data_lavorazione;
	}
}