package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "qualifica", indexes = { @Index(name= "qualifica", columnList = "data_creazione, nome") })
public class Qualifica extends SelezionatoAbstract implements Serializable, Comparable<Qualifica> {
	
	public Qualifica getCopia() {
		Qualifica pr = new Qualifica();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setDescrizione(this.getDescrizione());
		return pr;
	}
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String nome;
	@Column(columnDefinition = "text")
	private String descrizione;
	private Date data_creazione = ApplicationUtil.date();
	@OneToMany(mappedBy = "qualifica")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Gettone> gettoni = new ArrayList<Gettone>();
	
	@OneToMany(mappedBy = "qualifica")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contest> contest = new ArrayList<Contest>();
	
	@OneToMany(mappedBy = "differenziale")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Gettone> gettoniSotto = new ArrayList<Gettone>();
	
	@OneToMany(mappedBy = "differenziale")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Contest> contestSotto = new ArrayList<Contest>();
	public List<Gettone> getGettoniSotto() {
		return gettoniSotto;
	}
	public void setGettoniSotto(List<Gettone> gettoniSotto) {
		this.gettoniSotto = gettoniSotto;
	}
	public List<Contest> getContestSotto() {
		return contestSotto;
	}
	public void setContestSotto(List<Contest> contestSotto) {
		this.contestSotto = contestSotto;
	}
	public List<Contest> getContest() {
		return contest;
	}
	public void setContest(List<Contest> contest) {
		this.contest = contest;
	}
	public List<Gettone> getGettoni() {
		return gettoni;
	}
	public void setGettoni(List<Gettone> gettoni) {
		this.gettoni = gettoni;
	}
	public Qualifica() {
	}
	public int compareTo(Qualifica o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@OneToMany
	@JoinColumn(name = "qualifica_id", nullable = true, updatable=true, insertable=true)
	private List<Agente> agenti = new ArrayList<Agente>();
	public List<Agente> getAgenti() {
		return agenti;
	}
	public void setAgenti(List<Agente> agenti) {
		this.agenti = agenti;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Qualifica other = (Qualifica) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDescrizione() {
		return ApplicationUtil.trimToEmpty(descrizione);
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = ApplicationUtil.trimToNull(descrizione);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
}