package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.EmailUtils;
import it.callite.utility.RicezioneUtil;
import jakarta.activation.DataSource;
import jakarta.mail.Message;
import jakarta.mail.MessagingException;
import jodd.mail.EMLParser;
import jodd.mail.EmailAttachment;
import jodd.mail.EmailMessage;
import jodd.mail.ReceivedEmail;
@SuppressWarnings("serial")
@Entity
@Table(name = ConstantsGlobal.tp + "ricezione", indexes = { @Index(name= "ricezione", columnList = "data_creazione") })
public class Ricezione extends SelezionatoAbstract implements Serializable {
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;

	private Date data_creazione = ApplicationUtil.date();

	@Column(columnDefinition = "text")
	private String msgid;
	@Column(columnDefinition = "text")
	private String folder;
	@Column(columnDefinition = "text")
	private String mittente;
	@Column(columnDefinition = "text")
	private String destinatario;
	@Column(columnDefinition = "text")
	private String cc;
	@Column(columnDefinition = "text")
	private String reply;
	@Column(columnDefinition = "text")
	private String oggetto;
	private Integer priority;
	@Column(columnDefinition = "text")
	private String flags;
	private Boolean answered;
	private Boolean deleted;
	private Boolean draft;
	private Boolean flagged;
	private Boolean recent;
	private Boolean seen;
	private Date data_invio;
	private Date data_ricezione;
	private Boolean messages;
	private Integer attachments;

	private Boolean letto = false;
	private Boolean archiviato = false;

	@Lob
	private byte[] content;

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Ricezione other = (Ricezione) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	@ManyToOne
	private Smtp smtp;

	public Ricezione() {
	}

	public Ricezione(Smtp smtp, ReceivedEmail email) {
		Message message = email.originalMessage();

		this.smtp = smtp;
		this.msgid = email.messageId();
		this.content =  EmailUtils.message(message);
		this.folder = ApplicationUtil.trimToEmpty(message.getFolder() != null ? message.getFolder().getName() : "");
		this.mittente = EmailUtils.email(email.from());
		this.destinatario = EmailUtils.email(email.to());
		this.cc = EmailUtils.email(email.cc());
		this.reply = EmailUtils.email(email.replyTo());
		this.oggetto = email.subject();
		this.priority = email.priority();
		this.flags = ApplicationUtil.trimToEmpty(email.flags() != null ? email.flags().toString() : "");
		this.answered = email.isAnswered();
		this.deleted = email.isDeleted();
		this.draft = email.isDraft();
		this.flagged = email.isFlagged();
		this.recent = email.isRecent();
		this.seen = email.isSeen();
		this.data_invio = email.sentDate();
		this.data_ricezione = email.receivedDate();
		this.messages = ApplicationUtil.notEmpty(email.messages());
		this.attachments = ApplicationUtil.size(email.attachments());

		//try {
		//	message.setFlag(Flags.Flag.DELETED, true);
		//} catch (MessagingException e) {}
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public String getMsgid() {
		return msgid;
	}
	public void setMsgid(String msgid) {
		this.msgid = msgid;
	}
	public String getFolder() {
		return folder;
	}
	public void setFolder(String folder) {
		this.folder = folder;
	}

	public String getDestinatario() {
		return destinatario;
	}

	public void setDestinatario(String destinatario) {
		this.destinatario = destinatario;
	}

	public Smtp getSmtp() {
		return smtp;
	}

	public void setSmtp(Smtp smtp) {
		this.smtp = smtp;
	}

	public String getCc() {
		return cc;
	}
	public void setCc(String cc) {
		this.cc = cc;
	}
	public String getReply() {
		return reply;
	}
	public void setReply(String reply) {
		this.reply = reply;
	}

	public String getMittente() {
		return mittente;
	}

	public void setMittente(String mittente) {
		this.mittente = mittente;
	}

	public String getOggetto() {
		return oggetto;
	}

	public void setOggetto(String oggetto) {
		this.oggetto = oggetto;
	}

	public Integer getPriority() {
		return priority;
	}
	public void setPriority(Integer priority) {
		this.priority = priority;
	}
	public String getFlags() {
		return flags;
	}
	public void setFlags(String flags) {
		this.flags = flags;
	}
	public Boolean getAnswered() {
		return answered;
	}
	public void setAnswered(Boolean answered) {
		this.answered = answered;
	}
	public Boolean getDeleted() {
		return deleted;
	}
	public void setDeleted(Boolean deleted) {
		this.deleted = deleted;
	}
	public Boolean getDraft() {
		return draft;
	}
	public void setDraft(Boolean draft) {
		this.draft = draft;
	}
	public Boolean getFlagged() {
		return flagged;
	}
	public void setFlagged(Boolean flagged) {
		this.flagged = flagged;
	}
	public Boolean getRecent() {
		return recent;
	}
	public void setRecent(Boolean recent) {
		this.recent = recent;
	}
	public Boolean getSeen() {
		return seen;
	}
	public void setSeen(Boolean seen) {
		this.seen = seen;
	}

	public Date getData_invio() {
		return data_invio;
	}

	public void setData_invio(Date data_invio) {
		this.data_invio = data_invio;
	}

	public Date getData_ricezione() {
		return data_ricezione;
	}

	public void setData_ricezione(Date data_ricezione) {
		this.data_ricezione = data_ricezione;
	}


	public Boolean getMessages() {
		return messages;
	}

	public void setMessages(Boolean messages) {
		this.messages = messages;
	}

	public Integer getAttachments() {
		return attachments;
	}
	public void setAttachments(Integer attachments) {
		this.attachments = attachments;
	}
	public byte[] getContent() {
		return content;
	}
	public void setContent(byte[] content) {
		this.content = content;
	}

	@Transient
	private String testoH;
	@Transient
	private String testoT;

	public String getTestoH() {
		return testoH;
	}

	public void setTestoH(String testoH) {
		this.testoH = testoH;
	}

	public String getTestoT() {
		return testoT;
	}

	public void setTestoT(String testoT) {
		this.testoT = testoT;
	}

	public void elabora() {
		StringBuilder sbT = new StringBuilder();
		StringBuilder sbH = new StringBuilder();
		if(content != null) {
			try {
				ReceivedEmail eml = EMLParser.create().parse(content);
				List<EmailMessage> messages = eml.messages();
				if(ApplicationUtil.notEmpty(messages)) {
					for (EmailMessage msg : messages) {
						if(ApplicationUtil.equals(msg.getMimeType(), "text/plain")) {
							sbT.append(msg.getContent());
						}
						if(ApplicationUtil.equals(msg.getMimeType(), "text/html")) {
							sbH.append(msg.getContent());
						}
					}
				}
			} catch (MessagingException e) {
			}
		}
		testoT = ApplicationUtil.trimToNull(sbT.toString());
		testoH = ApplicationUtil.trimToNull(sbH.toString());
	}

	public List<RicezioneUtil> getAllegati() {
		List<RicezioneUtil> sb = new ArrayList<RicezioneUtil>();
		if(content != null) {
			try {
				ReceivedEmail eml = EMLParser.create().parse(content);
				List<EmailAttachment<? extends DataSource>> attachments = eml.attachments();
				if(ApplicationUtil.notEmpty(attachments)) {
					for (EmailAttachment<? extends DataSource> attachment : attachments) {
						sb.add(new RicezioneUtil(attachment));
					}
				}
			} catch (MessagingException e) {
			}
		}
		return sb;
	}

	public Boolean getLetto() {
		if(letto == null) {
			letto = false;
		}
		return letto;
	}

	public void setLetto(Boolean letto) {
		this.letto = letto;
	}

	public Boolean getArchiviato() {
		if(archiviato == null) {
			archiviato = false;
		}
		return archiviato;
	}

	public void setArchiviato(Boolean archiviato) {
		this.archiviato = archiviato;
	}

}