package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.model.sip.SipBuddies;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.IntervalloInboundUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "risposta", indexes = { @Index(name= "risposta", columnList = "data_creazione, nome") })
public class Risposta extends SelezionatoAbstract implements Serializable, Comparable<Risposta> {
	

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Date data_creazione = ApplicationUtil.date();
	
	@ManyToOne
	private SipBuddies trunk;

	private String nome;
	private String extension;
	private String add_prefisso;
	private String rem_prefisso;
	
	
	private Integer soloNumero = 1;
	private Boolean soloPubbliche = Boolean.FALSE;
	private Boolean soloAperte = Boolean.FALSE;
	private Boolean soloNuova = Boolean.FALSE;
	private Boolean soloHangup = Boolean.FALSE;
	private Boolean soloUpdate = Boolean.FALSE;
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "campagna_risposta", joinColumns=@JoinColumn(name="risposta_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagna_id", nullable=false, updatable=true, insertable=true)    )
	private List<Campagna> campagne;
	
	public List<Campagna> getCampagne() {
		return campagne;
	}
	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}
	public Integer getSoloNumero() {
		if(soloNumero == null) {
			soloNumero = 1;
		}
		return soloNumero;
	}
	public void setSoloNumero(Integer soloNumero) {
		this.soloNumero = soloNumero;
	}
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	
	
	public Boolean getSoloPubbliche() {
		return soloPubbliche;
	}
	public void setSoloPubbliche(Boolean soloPubbliche) {
		this.soloPubbliche = soloPubbliche;
	}
	public Boolean getSoloAperte() {
		return soloAperte;
	}
	public void setSoloAperte(Boolean soloAperte) {
		this.soloAperte = soloAperte;
	}
	public Boolean getSoloNuova() {
		return soloNuova;
	}
	public void setSoloNuova(Boolean soloNuova) {
		this.soloNuova = soloNuova;
	}
	public Boolean getSoloUpdate() {
		return soloUpdate;
	}
	public void setSoloUpdate(Boolean soloUpdate) {
		this.soloUpdate = soloUpdate;
	}
	@Column(columnDefinition = "text")
	private String musicclass;
	
	private String context;

	public int compareTo(Risposta o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Risposta other = (Risposta) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public String getAdd_prefisso() {
		return ApplicationUtil.trimToEmpty(add_prefisso);
	}
	public Risposta getCopia() {
		Risposta pr = new Risposta();
		pr.setNome(this.getNome());
		pr.setExtension(this.getExtension());
		pr.setAdd_prefisso(this.getAdd_prefisso());
		pr.setRem_prefisso(this.getRem_prefisso());
		pr.setMusicclass(this.getMusicclass());
		pr.setContext(this.getContext());
		pr.setSeriale(this.getSeriale());
		pr.setTrunk(this.getTrunk());
		pr.setUtil(this.getUtil());
		pr.setSoloNumero(this.getSoloNumero());
		pr.setSoloAperte(this.getSoloAperte());
		pr.setSoloNuova(this.getSoloNuova());
		pr.setSoloPubbliche(this.getSoloPubbliche());
		pr.setSoloHangup(this.getSoloHangup());
		pr.setSoloUpdate(this.getSoloUpdate());
		return pr;
	}
	
	public Boolean getSoloHangup() {
		return soloHangup;
	}
	public void setSoloHangup(Boolean soloHangup) {
		this.soloHangup = soloHangup;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public String getDenominazione() {
		return nome;
	}
	public String getExtension() {
		return ApplicationUtil.trimToEmpty(extension);
	}
	public Long getId() {
		return id;
	}

	public void load() {
		if(seriale != null) {
			util = (IntervalloInboundUtil) ApplicationUtil.deserialize(seriale);
		}
		if(util == null) {
			util = new IntervalloInboundUtil();
		}
	}

	public void save() {
		if(util != null) {
			seriale = (String) ApplicationUtil.serialize(util);
		}
	}

	public SipBuddies getTrunk() {
		return trunk;
	}
	public void setTrunk(SipBuddies trunk) {
		this.trunk = trunk;
	}

	@Column(columnDefinition = "text")
	private String seriale;

	public String getSeriale() {
		return seriale;
	}
	public void setSeriale(String seriale) {
		this.seriale = seriale;
	}

	@Transient
	private IntervalloInboundUtil util = new IntervalloInboundUtil();



	public IntervalloInboundUtil getUtil() {
		return util;
	}
	public void setUtil(IntervalloInboundUtil util) {
		this.util = util;
	}
	public String getNome() {
		return ApplicationUtil.trimToEmpty(nome);
	}

	public String getRem_prefisso() {
		return ApplicationUtil.trimToEmpty(rem_prefisso);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAdd_prefisso(String add_prefisso) {
		this.add_prefisso = ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, add_prefisso));
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public void setExtension(String extension) {
		this.extension = ApplicationUtil.normalizeToNull(extension);
	}
	public void setId(Long id) {
		this.id = id;
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	public void setRem_prefisso(String rem_prefisso) {
		this.rem_prefisso = ApplicationUtil.normalize(ApplicationUtil.aggiustaLista(true, ConstantsGlobal.colonSeparator, rem_prefisso));
	}

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	
	public String getMusicclass() {
		return ApplicationUtil.trimToNull(musicclass);
	}
	public void setMusicclass(String musicclass) {
		this.musicclass = ApplicationUtil.trimToNull(musicclass);
	}
	
	public String getContext() {
		return ApplicationUtil.trimToNull(context);
	}
	public void setContext(String context) {
		this.context = ApplicationUtil.normalizeToNull(context);
	}
	

}