package it.callite.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;


@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "rmail_template", indexes = { @Index(name= "rmail_template", columnList = "data_creazione, nome") })
public class RmailTemplate extends SelezionatoAbstract implements Serializable {

	@Transient
	private List<Allegato> allegati;

	private Integer tipo = ConstantsGlobal.a_sms;


	@OneToMany(mappedBy = "rmailTemplate")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AutoProcessRmail> autoProcessRmails;

	@ManyToOne
	private Busta busta;

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "rmail_campagna", joinColumns=@JoinColumn(name="rmail_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true)    )
	private List<Campagna> campagne;

	private Date data_creazione = ApplicationUtil.date();

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "rmailTemplate_documento", joinColumns=@JoinColumn(name="rmailTemplate_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="documenti_id", nullable=false, updatable=true, insertable=true)    )
	private Set<Documento> documenti = new HashSet<Documento>();

	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "esito_rmail", joinColumns=@JoinColumn(name="rmail_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="esito_id", nullable=false, updatable=true, insertable=true)    )
	private List<EsitoArchiviazione> esiti;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private Boolean invia_allegati = Boolean.FALSE;
	private Boolean genera_allegati = Boolean.TRUE;
	private Boolean attivo = Boolean.TRUE;
	@ManyToOne
	private Invio invio;

	@OneToMany(mappedBy = "rmailTemplate")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Rmail> rmails;
	private String nome;

	private Boolean residenza = Boolean.TRUE;

	@ManyToOne
	private Smtp smtp;

	@Column(columnDefinition = "text")
	private String testo;

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RmailTemplate other = (RmailTemplate) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	@Transient
	public List<Allegato> getAllegati() {
		return allegati;
	}

	public Boolean getAttivo() {
		return attivo;
	}

	public void setAttivo(Boolean attivo) {
		this.attivo = attivo;
	}

	public List<AutoProcessRmail> getAutoProcessRmails() {
		return autoProcessRmails;
	}

	public Busta getBusta() {
		return busta;
	}

	public List<Campagna> getCampagne() {
		return campagne;
	}

	public Date getData_creazione() {
		return data_creazione;
	}

	public Integer getTipo() {
		if(tipo == null) {
			tipo = ConstantsGlobal.a_sms;
		}
		return tipo;
	}

	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}

	public String getDenominazione() {
		return nome;
	}

	public Set<Documento> getDocumenti() {
		if(documenti == null) {
			documenti = new HashSet<Documento>();
		}
		return documenti;
	}

	@Transient
	public List<Documento> getDocumentiList() {
		if(documenti != null) 
			return new ArrayList<Documento>(documenti);
		else return new ArrayList<Documento>();
	}

	public String getDocumentiNome() {
		if(documenti != null) {
			List<String> nomi = new ArrayList<String>();
			Iterator<Documento> iter = documenti.iterator();
			while (iter.hasNext()) {
				Documento doc = iter.next();
				nomi.add(doc.getNome());
			}
			return ApplicationUtil.detoken(nomi, ConstantsGlobal.colonSeparator);
		}
		else return "";
	}

	public List<EsitoArchiviazione> getEsiti() {
		return esiti;
	}

	public Long getId() {
		return id;
	}

	public Boolean getInvia_allegati() {
		return invia_allegati;
	}

	public Invio getInvio() {
		return invio;
	}


	public Boolean getGenera_allegati() {
		return genera_allegati;
	}

	public void setGenera_allegati(Boolean genera_allegati) {
		this.genera_allegati = genera_allegati;
	}

	public List<Rmail> getRmails() {
		return rmails;
	}

	public String getNome() {
		return ApplicationUtil.normalizeToNull(nome);
	}

	public Boolean getResidenza() {
		if(residenza==null)
			return false;
		return residenza;
	}

	public Smtp getSmtp() {
		return smtp;
	}

	public String getTesto() {
		return testo;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	public void setAllegati(List<Allegato> allegati) {
		this.allegati = allegati;
	}


	public void setAutoProcessRmails(List<AutoProcessRmail> autoProcessRmails) {
		this.autoProcessRmails = autoProcessRmails;
	}

	public void setBusta(Busta busta) {
		this.busta = busta;
	}

	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public void setDocumenti(Set<Documento> documenti) {
		this.documenti = documenti;
	}

	public void setDocumentiList(List<Documento> documenti) { }

	public void setEsiti(List<EsitoArchiviazione> esiti) {
		this.esiti = esiti;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setInvia_allegati(Boolean invia_allegati) {
		this.invia_allegati = invia_allegati;
	}

	public void setInvio(Invio invio) {
		this.invio = invio;
	}

	public void setRmails(List<Rmail> rmails) {
		this.rmails = rmails;
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	public void setResidenza(Boolean residenza) {
		this.residenza = residenza;
	}	

	public void setSmtp(Smtp smtp) {
		this.smtp = smtp;
	}

	public void setTesto(String testo) {
		this.testo = testo;
	}

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public RmailTemplate getCopia() {
		RmailTemplate copia = new RmailTemplate();
		copia.setNome(this.getNome() + " " + ConstantsLanguage.COPY.stampa());
		copia.setInvia_allegati(this.getInvia_allegati());
		copia.setGenera_allegati(this.getGenera_allegati());
		copia.setAttivo(this.getAttivo());
		copia.setTesto(this.getTesto());
		copia.setResidenza(this.getResidenza());
		copia.setTipo(this.getTipo());
		return copia;
	}

	public boolean isLettera() {
		return ApplicationUtil.eq(tipo, ConstantsGlobal.a_lettera);
	}

	public boolean isSms() {
		return ApplicationUtil.eq(tipo, ConstantsGlobal.a_sms);
	}

	public boolean isFax() {
		return ApplicationUtil.eq(tipo, ConstantsGlobal.a_fax);
	}

	public boolean isTelegramma() {
		return ApplicationUtil.eq(tipo, ConstantsGlobal.a_telegramma);
	}

	private String analytics;

	public String getAnalytics() {
		return ApplicationUtil.upperCaseNull(analytics);
	}

	public void setAnalytics(String analytics) {
		this.analytics = ApplicationUtil.upperCaseNull(analytics);
	}
	
	private Boolean link = Boolean.FALSE;

	public Boolean getLink() {
		return link;
	}

	public void setLink(Boolean link) {
		this.link = link;
	}

}