package it.callite.model;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.message.Message;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "schedulatore", indexes = { @Index(name= "schedulatore", columnList = "data_creazione, data_schedulata, nome") })
public class Schedulatore extends SelezionatoAbstract implements Serializable {
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	private String nome;
	private Date data_schedulata;
	public Date getData_schedulata() {
		return ApplicationUtil.aggiustaDataT(data_schedulata, true);
	}
	public void setData_schedulata(Date data_schedulata) {
		this.data_schedulata = ApplicationUtil.aggiustaDataT(data_schedulata, true);
	}
	private Date data_creazione = ApplicationUtil.date();
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	private String orario = "20";
	private Boolean started = Boolean.FALSE;
	private Integer tipo = 0;
	@Transient
	private List<Integer> valore_multiplo;
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Schedulatore other = (Schedulatore) obj;
		if (tipo == null) {
			if (other.tipo != null)
				return false;
		} else if (!tipo.equals(other.tipo))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		if (orario == null) {
			if (other.orario != null)
				return false;
		} else if (!orario.equals(other.orario))
			return false;
		if (started == null) {
			if (other.started != null)
				return false;
		} else if (!started.equals(other.started))
			return false;
		if (valore_multiplo == null) {
			if (other.valore_multiplo != null)
				return false;
		} else if (!valore_multiplo.equals(other.valore_multiplo))
			return false;
		return true;
	}
	public Date getDateTime() {
		if(ApplicationUtil.gt(tipo, 0)) {
			return ApplicationUtil.date(Long.parseLong(orario));
		}
		return null;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	};
	public String getNomeTesto() {
		return Message.addCatalogMessage(nome);
	}
	public Integer getTipo() {
		if(tipo == null) {
			tipo = 0;
		}
		return tipo;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	public String getOrario() {
		return orario;
	}
	public String getOrarioString() {
		return ApplicationUtil.trimToEmpty(orario);
	}

	public Boolean getStarted() {
		return started;
	}
	public List<Integer> getValore_multiplo() {
		if(ApplicationUtil.eq(tipo, 0)) {
			return ApplicationUtil.tokenI(orario, ConstantsGlobal.commaSeparator);
		}
		return null;
	}
	public List<SelectItem> getValori() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		for(int i=0; i<24; i++){
			Integer ora = Integer.valueOf(i); 
			list.add(new SelectItem(ora, ora.toString()));
		}
		return list;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((tipo == null) ? 0 : tipo.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		result = prime * result + ((orario == null) ? 0 : orario.hashCode());
		result = prime * result + ((started == null) ? 0 : started.hashCode());
		result = prime * result + ((valore_multiplo == null) ? 0 : valore_multiplo.hashCode());
		return result;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setOrario(String orario) {
		this.orario = orario;
	}
	public void setStarted(Boolean started) {
		this.started = started;
	}
	public void setValore_multiplo(List<Integer> valoreMultiplo) {
		orario = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.commaSeparator);
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getTipoTesto() {
		if(isTipo_email()) {
			return ConstantsLanguage.EMAIL.stampa();
		} 
		if(isTipo_rmail()) {
			return ConstantsLanguage.RMAIL.stampa();
		} 
		if(isTipo_telecom()) {
			return ConstantsLanguage.TELECOM.stampa();
		}
		return null;
	}

	@OneToMany
	@JoinColumn(name = "scheduler_id", nullable = true, updatable=true, insertable=true)
	private List<Email> emails;

	@OneToMany
	@JoinColumn(name = "scheduler_id", nullable = true, updatable=true, insertable=true)
	private List<Rmail> lettaras;


	@OneToMany
	@JoinColumn(name = "scheduler_id", nullable = true, updatable=true, insertable=true)
	private List<Telecom> telecoms;
	public List<Email> getEmails() {
		return emails;
	}
	public void setEmails(List<Email> emails) {
		this.emails = emails;
	}

	public List<Rmail> getLettaras() {
		return lettaras;
	}
	public void setLettaras(List<Rmail> lettaras) {
		this.lettaras = lettaras;
	}
	public List<Telecom> getTelecoms() {
		return telecoms;
	}
	public void setTelecoms(List<Telecom> telecoms) {
		this.telecoms = telecoms;
	}

	public boolean isTipo_email() {
		return ApplicationUtil.eq(tipo, ConstantsGlobal.tipo_email);
	}

	public boolean isTipo_rmail() {
		return ApplicationUtil.eq(tipo, ConstantsGlobal.tipo_rmail);
	}

	public boolean isTipo_telecom() {
		return ApplicationUtil.eq(tipo, ConstantsGlobal.tipo_telecom);
	}



}