package it.callite.model;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;


@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "smtp", indexes = { @Index(name= "smtp", columnList = "data_creazione") })
public class Smtp extends SelezionatoAbstract implements Serializable, Comparable<Smtp> {

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getSmtp_mittente());
	}

	private Date data_creazione = ApplicationUtil.date();

	public Date getData_creazione() {
		return data_creazione;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	@OneToMany
	@JoinColumn(name = "smtp_id", nullable = true, updatable=true, insertable=true)
	private List<EmailTemplate> email;

	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@OneToMany
	@JoinColumn(name = "smtp_id", nullable = true, updatable=true, insertable=true)
	private List<RmailTemplate> rmail;

	@OneToMany
	@JoinColumn(name = "smtp_id", nullable = true, updatable=true, insertable=true)
	private List<Ricezione> ricezione;
	
	private Boolean smtp_auth;
	private String smtp_from;
	private String smtp_host;
	private String smtp_mittente;
	private String smtp_password;
	private Integer smtp_port;
	private String smtp_protocol;
	private String smtp_encryption;
	private String smtp_user;
	private Boolean smtp_debug;
	private Boolean smtp_pec;
	
	private Integer ricezione_type = 0;
	private String ricezione_host;
	private Integer ricezione_port;
	@Column(columnDefinition = "text")
	private String ricezione_email;
	
	@Column(columnDefinition = "text")
	private String ricezione_link;

	public String getSmtp_protocol() {
		return ApplicationUtil.trimToNull(smtp_protocol);
	}

	public void setSmtp_protocol(String smtp_protocol) {
		this.smtp_protocol = ApplicationUtil.trimToNull(smtp_protocol);
	}

	public int compareTo(Smtp o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Smtp other = (Smtp) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	public List<Ricezione> getRicezione() {
		return ricezione;
	}

	public void setRicezione(List<Ricezione> ricezione) {
		this.ricezione = ricezione;
	}

	public List<EmailTemplate> getEmail() {
		return email;
	}

	public Long getId() {
		return id;
	}

	public List<RmailTemplate> getRmail() {
		return rmail;
	}


	public Boolean getSmtp_auth() {
		return smtp_auth;
	}

	public String getSmtp_from() {
		return ApplicationUtil.trimToNull(smtp_from);
	}


	public String getSmtp_host() {
		return ApplicationUtil.trimToNull(smtp_host);
	}

	public String getSmtp_mittente() {
		return ApplicationUtil.trimToNull(smtp_mittente);
	}


	public String getSmtp_password() {
		return smtp_password;
	}

	public Integer getSmtp_port() {
		return smtp_port;
	}



	public String getSmtp_encryption() {
		return ApplicationUtil.trimToNull(smtp_encryption);
	}

	public void setSmtp_encryption(String smtp_encryption) {
		this.smtp_encryption = ApplicationUtil.trimToNull(smtp_encryption);
	}

	public String getSmtp_user() {
		return ApplicationUtil.trimToNull(smtp_user);
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	public void setEmail(List<EmailTemplate> email) {
		this.email = email;
	}


	public void setId(Long id) {
		this.id = id;
	}

	public void setRmail(List<RmailTemplate> rmail) {
		this.rmail = rmail;
	}


	public void setSmtp_auth(Boolean smtp_auth) {
		this.smtp_auth = smtp_auth;
	}


	public void setSmtp_from(String smtp_from) {
		this.smtp_from = ApplicationUtil.trimToNull(smtp_from);
	}

	public void setSmtp_host(String smtp_host) {
		this.smtp_host = ApplicationUtil.trimToNull(smtp_host);
	}

	public void setSmtp_mittente(String smtp_mittente) {
		this.smtp_mittente = ApplicationUtil.trimToNull(smtp_mittente);
	}

	public void setSmtp_password(String smtp_password) {
		this.smtp_password = smtp_password;
	}

	public void setSmtp_port(Integer smtp_port) {
		this.smtp_port = smtp_port;
	}


	public void setSmtp_user(String smtp_user) {
		this.smtp_user = ApplicationUtil.trimToNull(smtp_user);
	}


	public Smtp getCopia() {
		Smtp copia = new Smtp();
		copia.setSmtp_mittente(this.getSmtp_mittente());
		copia.setSmtp_auth(this.getSmtp_auth());
		copia.setSmtp_from(this.getSmtp_from());
		copia.setSmtp_host(this.getSmtp_host());
		copia.setSmtp_password(this.getSmtp_password());
		copia.setSmtp_port(this.getSmtp_port());
		copia.setSmtp_encryption(this.getSmtp_encryption());
		copia.setSmtp_protocol(this.getSmtp_protocol());
		copia.setSmtp_user(this.getSmtp_user());
		copia.setSmtp_pec(this.getSmtp_pec());
		copia.setSmtp_debug(this.getSmtp_debug());
		copia.setRicezione_type(this.getRicezione_type());
		copia.setRicezione_host(this.getRicezione_host());
		copia.setRicezione_port(this.getRicezione_port());
		copia.setRicezione_email(this.getRicezione_email());
		copia.setRicezione_link(this.getRicezione_link());
		return copia;
	}

	public Boolean getSmtp_debug() {
		return ApplicationUtil.isTrue(smtp_debug);
	}

	public void setSmtp_debug(Boolean smtp_debug) {
		this.smtp_debug = smtp_debug;
	}

	public Boolean getSmtp_pec() {
		return ApplicationUtil.isTrue(smtp_pec);
	}

	public void setSmtp_pec(Boolean smtp_pec) {
		this.smtp_pec = smtp_pec;
	}

	public Integer getRicezione_type() {
		if(ricezione_type == null) {
			ricezione_type = 0;
		}
		return ricezione_type;
	}

	public void setRicezione_type(Integer ricezione_type) {
		this.ricezione_type = ricezione_type;
	}

	public String getRicezione_host() {
		return ApplicationUtil.trimToNull(ricezione_host);
	}

	public void setRicezione_host(String ricezione_host) {
		this.ricezione_host = ApplicationUtil.trimToNull(ricezione_host);
	}

	public Integer getRicezione_port() {
		return ricezione_port;
	}

	public void setRicezione_port(Integer ricezione_port) {
		this.ricezione_port = ricezione_port;
	}
	
	public String getRicezione_email() {
		return ApplicationUtil.trimToNull(ricezione_email);
	}

	public void setRicezione_email(String ricezione_email) {
		this.ricezione_email = ApplicationUtil.trimToNull(ricezione_email);
	}

	public String getRicezione_link() {
		return ApplicationUtil.trimToNull(ricezione_link);
	}

	public void setRicezione_link(String ricezione_link) {
		this.ricezione_link = ApplicationUtil.trimToNull(ricezione_link);
	}
	
}