package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "webhook_template", indexes = { @Index(name= "webhook_template", columnList = "data_creazione") })
public class WebhookTemplate   extends SelezionatoAbstract implements Serializable, Comparable<WebhookTemplate> {
	
	
	private String nome;
	
	@Column(columnDefinition = "text")
	private String outputs;
	
	@Column(columnDefinition = "text")
	private String callback;
	
	@Column(columnDefinition = "text")
	private String link;
	
	@Column(columnDefinition = "text")
	private String email;
	
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	
	public WebhookTemplate() {
	}

	public int compareTo(WebhookTemplate o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		WebhookTemplate other = (WebhookTemplate) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public WebhookTemplate getCopia() {
		WebhookTemplate copia = new WebhookTemplate();
		copia.setNome(this.getNome() + " " + ConstantsLanguage.COPY.stampa());
		copia.setOutputs(this.getOutputs());
		copia.setCallback(this.getCallback());
		copia.setLink(this.getLink());
		copia.setEmail(this.getEmail());
		return copia;
	}
	
	@OneToMany(mappedBy = "webhookTemplate")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Webhook> webhooks;
	
	public List<Webhook> getWebhooks() {
		return webhooks;
	}

	public void setWebhooks(List<Webhook> webhooks) {
		this.webhooks = webhooks;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	public String getNome() {
		return ApplicationUtil.normalizeToNull(nome);
	}

	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}

	public String getOutputs() {
		return ApplicationUtil.trimToNull(outputs);
	}

	public void setOutputs(String outputs) {
		this.outputs = ApplicationUtil.trimToNull(outputs);
	}

	public String getCallback() {
		return ApplicationUtil.trimToNull(callback);
	}

	public void setCallback(String callback) {
		this.callback = ApplicationUtil.trimToNull(callback);
	}

	public String getLink() {
		return ApplicationUtil.trimToNull(link);
	}

	public void setLink(String link) {
		this.link = ApplicationUtil.trimToNull(link);
	}

	public String getEmail() {
		return ApplicationUtil.trimToNull(email);
	}

	public void setEmail(String email) {
		this.email = ApplicationUtil.trimToNull(email);
	}

	public Date getData_creazione() {
		return data_creazione;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}
	
	
	
}