package it.callite.model;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "zona", indexes = { @Index(name= "zona", columnList = "data_creazione, nome"), @Index(name= "indirizzo1", columnList = "regione, provincia") , @Index(name= "indirizzo2", columnList = "comune, cap") })
public class Zona extends SelezionatoAbstract implements Serializable, Comparable<Zona> {
	
	public Zona getCopia() {
		Zona pr = new Zona();
		pr.setNome(this.getNome() + " " +  ConstantsLanguage.COPY.stampa());
		pr.setRegione(this.getRegione());
		pr.setProvincia(this.getProvincia());
		pr.setComune(this.getComune());
		pr.setCap(this.getCap());
		return pr;
	}
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	private Date data_creazione = ApplicationUtil.date();
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	private String nome;
	private String regione;
	private String provincia;
	private String cap;
	private String comune;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "agente_zona", joinColumns=@JoinColumn(name="zone_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="agente_id", nullable=false, updatable=true, insertable=true)    )
	private List<Agente> agente;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "supervisore_zona", joinColumns=@JoinColumn(name="zone_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="supervisore_id", nullable=false, updatable=true, insertable=true)    )
	private List<Utente> supervisori;
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "zona_campagna", joinColumns=@JoinColumn(name="zona_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="campagne_id", nullable=false, updatable=true, insertable=true)    )
	private List<Campagna> campagne;
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Zona other = (Zona) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public List<Agente> getAgente() {
		return agente;
	}
	public List<Campagna> getCampagne() {
		return campagne;
	}
	public String getCap() {
		return cap;
	}
	public String getComune() {
		return comune;
	}
	public String getDenominazione() {
		String regioneS = ApplicationUtil.trimToEmpty(regione);
		String provinciaS = ApplicationUtil.trimToEmpty(provincia);
		String comuneS = ApplicationUtil.trimToEmpty(comune);
		String capS = ApplicationUtil.trimToEmpty(cap);
		String locazione = ApplicationUtil.trimToEmpty(regioneS + " " + provinciaS + " " + comuneS + " " + capS);
		if(ApplicationUtil.isNotBlank(locazione)) {
			return nome + " (" + locazione + ")";
		} else {
			return nome;
		}
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	}
	public String getProvincia() {
		return provincia;
	}
	public String getRegione() {
		return regione;
	}
	public List<Utente> getSupervisori() {
		return supervisori;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAgente(List<Agente> agente) {
		this.agente = agente;
	}
	public void setCampagne(List<Campagna> campagne) {
		this.campagne = campagne;
	}
	public void setCap(String cap) {
		this.cap = ApplicationUtil.normalizeToUpperNull(cap);
	}
	public void setComune(String comune) {
		this.comune = ApplicationUtil.normalizeToUpperNull(comune);
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setProvincia(String provincia) {
		this.provincia = ApplicationUtil.normalizeToUpperNull(provincia);
	}
	public void setRegione(String regione) {
		this.regione = ApplicationUtil.normalizeToUpperNull(regione);
	}
	public void setSupervisori(List<Utente> supervisori) {
		this.supervisori = supervisori;
	}
	public int compareTo(Zona o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	
	
}