package it.callite.model.allegato;
import java.io.File;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.Contatto;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.tp + "allegato_contatto")
public class AllegatoContatto extends SelezionatoAbstract implements Serializable, Comparable<AllegatoContatto> {
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	public int compareTo(AllegatoContatto o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(content);
		result = prime * result + ((data == null) ? 0 : data.hashCode());
		result = prime * result + (int) (dimensione ^ (dimensione >>> 32));
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AllegatoContatto other = (AllegatoContatto) obj;
		if (!Arrays.equals(content, other.content))
			return false;
		if (data == null) {
			if (other.data != null)
				return false;
		} else if (!data.equals(other.data))
			return false;
		if (dimensione != other.dimensione)
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		return true;
	}
	@ManyToOne(fetch = FetchType.LAZY)
	private Contatto contatto;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data = ApplicationUtil.date();
	@Lob
	private byte[] content;
	public byte[] getContent() {
		return content;
	}
	public void setContent(byte[] content) {
		this.content = content;
	}
	private long dimensione;
	public Contatto getContatto() {
		return contatto;
	}
	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}
	public Date getData() {
		return data;
	}
	public void setData(Date data) {
		this.data = data;
	}
	public long getDimensione() {
		return dimensione;
	}
	public void setDimensione(long dimensione) {
		this.dimensione = dimensione;
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public String getNome() {
		return nome;
	}
	public void setNome(String nome) {
		this.nome = nome;
	}
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@Column(columnDefinition = "text")
	private String nome;
	@Column(columnDefinition = "text")
	private String percorso;
	public String getPercorso() {
		return percorso;
	}
	public void setPercorso(String percorso) {
		this.percorso = percorso;
	}
	public void setContentFile(byte[] c, String path)  throws Exception  {
		if(path != null) {
			percorso  = "/" +System.currentTimeMillis();
			ApplicationUtil.writeByteArrayToFile(new File(path + percorso), c);
		} else {
			content = c;
		}
	}
	public byte[] getContentFile(String path) throws Exception  {
		if(percorso != null) {
			File f = new File(path + percorso);
			if(f.exists()) {
				return ApplicationUtil.readFileToByteArray(f);
			}
		}
		return content;
	}
	
	public void delete(String path) {
		if(percorso != null) {
			File f = new File(path + percorso);
			if(f.exists()) {
				ApplicationUtil.deleteFile(f);
			}
		}
	}
}