package it.callite.model.anagrafica;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import it.callite.model.Anagrafica;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.DatoSondaggioAnagrafica;
@SuppressWarnings("serial")
public class AnagraficaXML implements Serializable {
	
	private Long id;
	private Date data_creazione;
	private String sorgente;
	private String cognome;
	private String nome;
	private String nome_completo;
	private String ragione_sociale;
	private String codice_fiscale;
	private String partita_iva; 
	private String luogo_nascita;
	private Date data_nascita; 
	private String telefono_altro;
	private String telefono_fax;
	private String telefono_fisso;
	private String telefono_mobile;
	private String email;
	private String link;
	private Boolean tipo; 
	private Boolean sesso;
	private String settore;
	private String nota;
	private Date consenso_tfisso;
	private Date consenso_tmobile;
	private Date consenso_tfax;
	private Date consenso_taltro;
	private Date consenso_email;
	private Date consenso_rmail;
	private IndirizzoXML indirizzo_legale;
	private IndirizzoXML indirizzo_operativo;
	private List<DatoAggiuntivoAnagraficaXML> dettagli; 
	
	public AnagraficaXML() {
	}
	public AnagraficaXML(Anagrafica anagrafica) {
		this.setId(anagrafica.getId());
		this.setData_creazione(anagrafica.getData_creazione());
		this.setSesso(anagrafica.getMaschio());
		this.setTipo(anagrafica.getPf());
		this.setData_nascita(anagrafica.getDate_nascita());
		this.setSorgente(ApplicationUtil.trimToEmpty(anagrafica.getSorgente()));
		this.setNota(ApplicationUtil.trimToEmpty(anagrafica.getNota()));
		this.setCodice_fiscale(ApplicationUtil.trimToEmpty(anagrafica.getCodice_fiscale()));
		this.setCognome(ApplicationUtil.trimToEmpty(anagrafica.getCognome()));
		this.setRagione_sociale(ApplicationUtil.trimToEmpty(anagrafica.getRagione_sociale()));
		this.setLuogo_nascita(ApplicationUtil.trimToEmpty(anagrafica.getLuogo_nascita()));
		this.setSettore(ApplicationUtil.trimToEmpty(anagrafica.getSettore()));
		this.setEmail(ApplicationUtil.trimToEmpty(anagrafica.getEmail()));
		this.setNome(ApplicationUtil.trimToEmpty(anagrafica.getNome()));
		this.setNome_completo(ApplicationUtil.trimToEmpty(anagrafica.getNome_completo()));
		this.setPartita_iva(ApplicationUtil.trimToEmpty(anagrafica.getPartita_iva()));
		this.setTelefono_fax(ApplicationUtil.trimToEmpty(anagrafica.getTelefono_fax()));
		this.setTelefono_fisso(ApplicationUtil.trimToEmpty(anagrafica.getTelefono_fisso()));
		this.setTelefono_altro(ApplicationUtil.trimToEmpty(anagrafica.getTelefono_altro()));
		this.setTelefono_mobile(ApplicationUtil.trimToEmpty(anagrafica.getTelefono_mobile()));
		this.setLink(ApplicationUtil.trimToEmpty(anagrafica.getLink()));
		this.setConsenso_tfisso(anagrafica.getConsenso_tfisso());
		this.setConsenso_tmobile(anagrafica.getConsenso_tmobile());
		this.setConsenso_tfax(anagrafica.getConsenso_tfax());
		this.setConsenso_taltro(anagrafica.getConsenso_taltro());
		this.setConsenso_email(anagrafica.getConsenso_email());
		this.setConsenso_rmail(anagrafica.getConsenso_rmail());
		this.setIndirizzo_legale(new IndirizzoXML(anagrafica.getIndirizzo_legale()));
		this.setIndirizzo_operativo(new IndirizzoXML(anagrafica.getIndirizzo_operativo()));
		List<DatoAggiuntivoAnagrafica> dati =  anagrafica.getDettagli();
		List<DatoAggiuntivoAnagraficaXML> dettagli =  new ArrayList<DatoAggiuntivoAnagraficaXML>();
		if(ApplicationUtil.notEmpty(dati)) {
			Iterator<DatoAggiuntivoAnagrafica> iter = dati.iterator();
			while (iter.hasNext()) {
				DatoAggiuntivoAnagrafica datoAggiuntivoAnagrafica = (DatoAggiuntivoAnagrafica) iter.next();
				dettagli.add(new DatoAggiuntivoAnagraficaXML(datoAggiuntivoAnagrafica));
			}
		}
		this.setDettagli(dettagli);
	}
	
	public AnagraficaXML(Anagrafica anagrafica, List<DatoSondaggioAnagrafica> dati) {
		this.setId(anagrafica.getId());
		this.setData_creazione(anagrafica.getData_creazione());
		this.setSesso(anagrafica.getMaschio());
		this.setTipo(anagrafica.getPf());
		this.setData_nascita(anagrafica.getDate_nascita());
		this.setSorgente(ApplicationUtil.trimToEmpty(anagrafica.getSorgente()));
		this.setNota(ApplicationUtil.trimToEmpty(anagrafica.getNota()));
		this.setCodice_fiscale(ApplicationUtil.trimToEmpty(anagrafica.getCodice_fiscale()));
		this.setCognome(ApplicationUtil.trimToEmpty(anagrafica.getCognome()));
		this.setRagione_sociale(ApplicationUtil.trimToEmpty(anagrafica.getRagione_sociale()));
		this.setLuogo_nascita(ApplicationUtil.trimToEmpty(anagrafica.getLuogo_nascita()));
		this.setSettore(ApplicationUtil.trimToEmpty(anagrafica.getSettore()));
		this.setEmail(ApplicationUtil.trimToEmpty(anagrafica.getEmail()));
		this.setNome(ApplicationUtil.trimToEmpty(anagrafica.getNome()));
		this.setNome_completo(ApplicationUtil.trimToEmpty(anagrafica.getNome_completo()));
		this.setPartita_iva(ApplicationUtil.trimToEmpty(anagrafica.getPartita_iva()));
		this.setTelefono_fax(ApplicationUtil.trimToEmpty(anagrafica.getTelefono_fax()));
		this.setTelefono_fisso(ApplicationUtil.trimToEmpty(anagrafica.getTelefono_fisso()));
		this.setTelefono_altro(ApplicationUtil.trimToEmpty(anagrafica.getTelefono_altro()));
		this.setTelefono_mobile(ApplicationUtil.trimToEmpty(anagrafica.getTelefono_mobile()));
		this.setLink(ApplicationUtil.trimToEmpty(anagrafica.getLink()));
		this.setConsenso_tfisso(anagrafica.getConsenso_tfisso());
		this.setConsenso_tmobile(anagrafica.getConsenso_tmobile());
		this.setConsenso_tfax(anagrafica.getConsenso_tfax());
		this.setConsenso_taltro(anagrafica.getConsenso_taltro());
		this.setConsenso_email(anagrafica.getConsenso_email());
		this.setConsenso_rmail(anagrafica.getConsenso_rmail());
		this.setIndirizzo_legale(new IndirizzoXML(anagrafica.getIndirizzo_legale()));
		this.setIndirizzo_operativo(new IndirizzoXML(anagrafica.getIndirizzo_operativo()));
		List<DatoAggiuntivoAnagraficaXML> dettagli =  new ArrayList<DatoAggiuntivoAnagraficaXML>();
		if(ApplicationUtil.notEmpty(dati)) {
			Iterator<DatoSondaggioAnagrafica> iter = dati.iterator();
			while (iter.hasNext()) {
				DatoSondaggioAnagrafica datoAggiuntivoAnagrafica = (DatoSondaggioAnagrafica) iter.next();
				dettagli.add(new DatoAggiuntivoAnagraficaXML(datoAggiuntivoAnagrafica));
			}
		}
		this.setDettagli(dettagli);
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public String getSettore() {
		return settore;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	
	public String getSorgente() {
		return sorgente;
	}
	public void setSorgente(String sorgente) {
		this.sorgente = sorgente;
	}
	public String getCodice_fiscale() {
		return codice_fiscale;
	}
	public String getCognome() {
		return cognome;
	}

	
	public List<DatoAggiuntivoAnagraficaXML> getDettagli() {
		if (dettagli == null)
			dettagli = new ArrayList<DatoAggiuntivoAnagraficaXML>();
		return dettagli;
	}
	public String getEmail() {
		return email;
	}
	public IndirizzoXML getIndirizzo_legale() {
		if (indirizzo_legale == null)
			indirizzo_legale = new IndirizzoXML();
		return indirizzo_legale;
	}
	public IndirizzoXML getIndirizzo_operativo() {
		if (indirizzo_operativo == null)
			indirizzo_operativo = new IndirizzoXML();
		return indirizzo_operativo;
	}
	public String getLuogo_nascita() {
		return luogo_nascita;
	}
	
	public String getNome() {
		return nome;
	}
	public String getNome_completo() {
		return nome_completo;
	}
	public String getNota() {
		return nota;
	}
	public String getPartita_iva() {
		return partita_iva;
	}

	public String getRagione_sociale() {
		return ragione_sociale;
	}
	public String getTelefono_altro() {
		return telefono_altro;
	}
	public String getTelefono_fax() {
		return telefono_fax;
	}
	public String getTelefono_fisso() {
		return telefono_fisso;
	}
	public String getTelefono_mobile() {
		return telefono_mobile;
	}
	public void setSettore(String settore) {
		this.settore = settore;
	}
	public void setCodice_fiscale(String codice_fiscale) {
		this.codice_fiscale = codice_fiscale;
	}
	public void setCognome(String cognome) {
		this.cognome = cognome;
	}
	
	public void setDettagli(List<DatoAggiuntivoAnagraficaXML> dettagli) {
		this.dettagli = dettagli;
	}
	
	public void setEmail(String email) {
		this.email = email;
	}
	public void setIndirizzo_legale(IndirizzoXML indirizzo_legale) {
		this.indirizzo_legale = indirizzo_legale;
	}
	public void setIndirizzo_operativo(IndirizzoXML indirizzo_operativo) {
		this.indirizzo_operativo = indirizzo_operativo;
	}
	public void setLuogo_nascita(String luogo_nascita) {
		this.luogo_nascita = luogo_nascita;
	}
	public String getLink() {
		return link;
	}
	public void setLink(String link) {
		this.link = link;
	}
	
	public void setNome(String nome) {
		this.nome = nome;
	}
	public void setNome_completo(String nome_completo) {
		this.nome_completo = nome_completo;
	}
	public void setNota(String nota) {
		this.nota = nota;
	}
	public void setPartita_iva(String partita_iva) {
		this.partita_iva = partita_iva;
	} 
	
	public void setRagione_sociale(String ragione_sociale) {
		this.ragione_sociale = ragione_sociale;
	}
	public void setTelefono_altro(String telefono_altro) {
		this.telefono_altro = telefono_altro;
	}
	public void setTelefono_fax(String telefono_fax) {
		this.telefono_fax = telefono_fax;
	}
	public void setTelefono_fisso(String telefono_fisso) {
		this.telefono_fisso = telefono_fisso;
	}
	public void setTelefono_mobile(String telefono_mobile) {
		this.telefono_mobile = telefono_mobile;
	}
	
	public Date getConsenso_tfisso() {
		return consenso_tfisso;
	}
	public void setConsenso_tfisso(Date consenso_tfisso) {
		this.consenso_tfisso = consenso_tfisso;
	}
	public Date getConsenso_tmobile() {
		return consenso_tmobile;
	}
	public void setConsenso_tmobile(Date consenso_tmobile) {
		this.consenso_tmobile = consenso_tmobile;
	}
	public Date getConsenso_tfax() {
		return consenso_tfax;
	}
	public void setConsenso_tfax(Date consenso_tfax) {
		this.consenso_tfax = consenso_tfax;
	}
	public Date getConsenso_taltro() {
		return consenso_taltro;
	}
	public void setConsenso_taltro(Date consenso_taltro) {
		this.consenso_taltro = consenso_taltro;
	}
	public Date getConsenso_email() {
		return consenso_email;
	}
	public void setConsenso_email(Date consenso_email) {
		this.consenso_email = consenso_email;
	}
	public Date getConsenso_rmail() {
		return consenso_rmail;
	}
	public void setConsenso_rmail(Date consenso_rmail) {
		this.consenso_rmail = consenso_rmail;
	}
	
	public Date getData_nascita() {
		return data_nascita;
	}
	public void setData_nascita(Date data_nascita) {
		this.data_nascita = data_nascita;
	}
	public Boolean getSesso() {
		return sesso;
	}
	public void setSesso(Boolean sesso) {
		this.sesso = sesso;
	}
	public Boolean getTipo() {
		return tipo;
	}
	public void setTipo(Boolean tipo) {
		this.tipo = tipo;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((codice_fiscale == null) ? 0 : codice_fiscale.hashCode());
		result = prime * result + ((cognome == null) ? 0 : cognome.hashCode());
		result = prime * result + ((consenso_email == null) ? 0 : consenso_email.hashCode());
		result = prime * result + ((consenso_rmail == null) ? 0 : consenso_rmail.hashCode());
		result = prime * result + ((consenso_taltro == null) ? 0 : consenso_taltro.hashCode());
		result = prime * result + ((consenso_tfax == null) ? 0 : consenso_tfax.hashCode());
		result = prime * result + ((consenso_tfisso == null) ? 0 : consenso_tfisso.hashCode());
		result = prime * result + ((consenso_tmobile == null) ? 0 : consenso_tmobile.hashCode());
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((data_nascita == null) ? 0 : data_nascita.hashCode());
		result = prime * result + ((dettagli == null) ? 0 : dettagli.hashCode());
		result = prime * result + ((email == null) ? 0 : email.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((indirizzo_legale == null) ? 0 : indirizzo_legale.hashCode());
		result = prime * result + ((indirizzo_operativo == null) ? 0 : indirizzo_operativo.hashCode());
		result = prime * result + ((link == null) ? 0 : link.hashCode());
		result = prime * result + ((luogo_nascita == null) ? 0 : luogo_nascita.hashCode());
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		result = prime * result + ((nome_completo == null) ? 0 : nome_completo.hashCode());
		result = prime * result + ((nota == null) ? 0 : nota.hashCode());
		result = prime * result + ((sorgente == null) ? 0 : sorgente.hashCode());
		result = prime * result + ((partita_iva == null) ? 0 : partita_iva.hashCode());
		result = prime * result + ((ragione_sociale == null) ? 0 : ragione_sociale.hashCode());
		result = prime * result + ((sesso == null) ? 0 : sesso.hashCode());
		result = prime * result + ((settore == null) ? 0 : settore.hashCode());
		result = prime * result + ((telefono_altro == null) ? 0 : telefono_altro.hashCode());
		result = prime * result + ((telefono_fax == null) ? 0 : telefono_fax.hashCode());
		result = prime * result + ((telefono_fisso == null) ? 0 : telefono_fisso.hashCode());
		result = prime * result + ((telefono_mobile == null) ? 0 : telefono_mobile.hashCode());
		result = prime * result + ((tipo == null) ? 0 : tipo.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AnagraficaXML other = (AnagraficaXML) obj;
		if (codice_fiscale == null) {
			if (other.codice_fiscale != null)
				return false;
		} else if (!codice_fiscale.equals(other.codice_fiscale))
			return false;
		if (cognome == null) {
			if (other.cognome != null)
				return false;
		} else if (!cognome.equals(other.cognome))
			return false;
		if (consenso_email == null) {
			if (other.consenso_email != null)
				return false;
		} else if (!consenso_email.equals(other.consenso_email))
			return false;
		if (consenso_rmail == null) {
			if (other.consenso_rmail != null)
				return false;
		} else if (!consenso_rmail.equals(other.consenso_rmail))
			return false;
		if (consenso_taltro == null) {
			if (other.consenso_taltro != null)
				return false;
		} else if (!consenso_taltro.equals(other.consenso_taltro))
			return false;
		if (consenso_tfax == null) {
			if (other.consenso_tfax != null)
				return false;
		} else if (!consenso_tfax.equals(other.consenso_tfax))
			return false;
		if (consenso_tfisso == null) {
			if (other.consenso_tfisso != null)
				return false;
		} else if (!consenso_tfisso.equals(other.consenso_tfisso))
			return false;
		if (consenso_tmobile == null) {
			if (other.consenso_tmobile != null)
				return false;
		} else if (!consenso_tmobile.equals(other.consenso_tmobile))
			return false;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (data_nascita == null) {
			if (other.data_nascita != null)
				return false;
		} else if (!data_nascita.equals(other.data_nascita))
			return false;
		if (dettagli == null) {
			if (other.dettagli != null)
				return false;
		} else if (!dettagli.equals(other.dettagli))
			return false;
		if (email == null) {
			if (other.email != null)
				return false;
		} else if (!email.equals(other.email))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (indirizzo_legale == null) {
			if (other.indirizzo_legale != null)
				return false;
		} else if (!indirizzo_legale.equals(other.indirizzo_legale))
			return false;
		if (indirizzo_operativo == null) {
			if (other.indirizzo_operativo != null)
				return false;
		} else if (!indirizzo_operativo.equals(other.indirizzo_operativo))
			return false;
		if (link == null) {
			if (other.link != null)
				return false;
		} else if (!link.equals(other.link))
			return false;
		if (luogo_nascita == null) {
			if (other.luogo_nascita != null)
				return false;
		} else if (!luogo_nascita.equals(other.luogo_nascita))
			return false;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		if (nome_completo == null) {
			if (other.nome_completo != null)
				return false;
		} else if (!nome_completo.equals(other.nome_completo))
			return false;
		if (nota == null) {
			if (other.nota != null)
				return false;
		} else if (!nota.equals(other.nota))
			return false;
		if (sorgente == null) {
			if (other.sorgente != null)
				return false;
		} else if (!sorgente.equals(other.sorgente))
			return false;
		if (partita_iva == null) {
			if (other.partita_iva != null)
				return false;
		} else if (!partita_iva.equals(other.partita_iva))
			return false;
		if (ragione_sociale == null) {
			if (other.ragione_sociale != null)
				return false;
		} else if (!ragione_sociale.equals(other.ragione_sociale))
			return false;
		if (sesso == null) {
			if (other.sesso != null)
				return false;
		} else if (!sesso.equals(other.sesso))
			return false;
		if (settore == null) {
			if (other.settore != null)
				return false;
		} else if (!settore.equals(other.settore))
			return false;
		if (telefono_altro == null) {
			if (other.telefono_altro != null)
				return false;
		} else if (!telefono_altro.equals(other.telefono_altro))
			return false;
		if (telefono_fax == null) {
			if (other.telefono_fax != null)
				return false;
		} else if (!telefono_fax.equals(other.telefono_fax))
			return false;
		if (telefono_fisso == null) {
			if (other.telefono_fisso != null)
				return false;
		} else if (!telefono_fisso.equals(other.telefono_fisso))
			return false;
		if (telefono_mobile == null) {
			if (other.telefono_mobile != null)
				return false;
		} else if (!telefono_mobile.equals(other.telefono_mobile))
			return false;
		if (tipo == null) {
			if (other.tipo != null)
				return false;
		} else if (!tipo.equals(other.tipo))
			return false;
		return true;
	}
	
	
}