package it.callite.model.messaggio;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.Utente;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;

@SuppressWarnings("serial")
@Entity
//@Audited
@Table(name = ConstantsGlobal.tp + "destinatario")
public class Destinatario  extends SelezionatoAbstract implements Serializable, Comparable<Destinatario> {
	
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getUser().getDenominazione());
	}
	
	
	@ManyToOne
	private Utente user;

	private Boolean stato= Boolean.FALSE;
	private Boolean cestino = Boolean.FALSE;
	private Boolean eliminato = Boolean.FALSE;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	@ManyToOne
	private Messaggio messaggio;

	public Destinatario() {

	}

	public Destinatario(Utente user) {
		this.setUser(user);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Destinatario other = (Destinatario) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (user == null) {
			if (other.user != null)
				return false;
		} else if (!user.equals(other.user))
			return false;
		return true;
	}

	public Boolean getCestino() {
		return cestino;
	}

	public Destinatario getCopia() {
		Destinatario am = new Destinatario();
		am.setUser(this.getUser());
		return am;
	}

	public Boolean getEliminato() {
		return eliminato;
	}



	public Long getId() {
		return id;
	}

	public Messaggio getMessaggio() {
		return messaggio;
	}

	public Boolean getStato() {
		return stato;
	}

	public Utente getUser() {
		return user;
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((user == null) ? 0 : user.hashCode());
		return result;
	}

	public void setCestino(Boolean cestino) {
		this.cestino = cestino;
	}

	public void setEliminato(Boolean eliminato) {
		this.eliminato = eliminato;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setMessaggio(Messaggio messaggio) {
		this.messaggio = messaggio;
	}

	public void setStato(Boolean stato) {
		this.stato = stato;
	}



	public void setUser(Utente user) {
		this.user = user;
	}



	public int compareTo(Destinatario o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

}