package it.callite.model.messaggio;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.Utente;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
@SuppressWarnings("serial")
@Entity
//@Audited
@Table(name = ConstantsGlobal.tp + "documento_messaggio", indexes = { @Index(name= "documento_messaggio", columnList = "data_creazione, data_modifica, nome" )})
public class DocumentoMessaggio  extends SelezionatoAbstract implements Serializable, Comparable<DocumentoMessaggio> {
	
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date data_creazione = ApplicationUtil.date();
	private Date data_modifica;
	private long dimensione;
	@Lob
	private byte[] file;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;
	
	private String nome;
	@ManyToOne
	private Utente utente;
	public DocumentoMessaggio() {
	}
	public DocumentoMessaggio(long dimensione, Date data_modifica, byte[] file, String nome) {
		this.setDimensione(dimensione);
		this.setData_modifica(data_modifica);
		this.setFile(file);
		this.setNome(nome);
	}
	public int compareTo(DocumentoMessaggio o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DocumentoMessaggio other = (DocumentoMessaggio) obj;
		if (data_creazione == null) {
			if (other.data_creazione != null)
				return false;
		} else if (!data_creazione.equals(other.data_creazione))
			return false;
		if (data_modifica == null) {
			if (other.data_modifica != null)
				return false;
		} else if (!data_modifica.equals(other.data_modifica))
			return false;
		if (dimensione != other.dimensione)
			return false;
		if (!Arrays.equals(file, other.file))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nome == null) {
			if (other.nome != null)
				return false;
		} else if (!nome.equals(other.nome))
			return false;
		if (utente == null) {
			if (other.utente != null)
				return false;
		} else if (!utente.equals(other.utente))
			return false;
		return true;
	}
	public DocumentoMessaggio getCopia() {
		DocumentoMessaggio am = new DocumentoMessaggio();
		am.setDimensione(this.getDimensione());
		am.setData_modifica(this.getData_modifica());
		am.setFile(this.getFile());
		am.setNome(this.getNome());
		return am;
	}
	
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_modifica() {
		return data_modifica;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getNome());
	}
	public long getDimensione() {
		return dimensione;
	}
	public byte[] getFile() {
		return file;
	}
	public Long getId() {
		return id;
	}
	
	public String getNome() {
		return nome;
	}
	
	public Utente getUtente() {
		return utente;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((data_creazione == null) ? 0 : data_creazione.hashCode());
		result = prime * result + ((data_modifica == null) ? 0 : data_modifica.hashCode());
		result = prime * result + (int) (dimensione ^ (dimensione >>> 32));
		result = prime * result + Arrays.hashCode(file);
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nome == null) ? 0 : nome.hashCode());
		result = prime * result + ((utente == null) ? 0 : utente.hashCode());
		return result;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_modifica(Date data_modifica) {
		this.data_modifica = data_modifica;
	}
	public void setDimensione(long dimensione) {
		this.dimensione = dimensione;
	}
	public void setFile(byte[] file) {
		this.file = file;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = ApplicationUtil.normalizeToNull(nome);
	}
	public void setUtente(Utente utente) {
		this.utente = utente;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
}