package it.callite.model.messaggio;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import org.richfaces.event.UploadEvent;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.Utente;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.MyFile;
@SuppressWarnings("serial")
@Entity
//@Audited
@Table(name = ConstantsGlobal.tp + "messaggio", indexes = { @Index(name= "messaggio", columnList = "data_creazione, data_invio" )})
public class Messaggio  extends SelezionatoAbstract implements Serializable, Comparable<Messaggio> {
	
	@OneToMany(mappedBy = "messaggio")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<AllegatoMessaggio> allegati = new ArrayList<AllegatoMessaggio>();
	@Transient
	private List<Allegato> allegato;
	
	@Transient
	private String destinatariString;
	
	@Transient
	private String allegatiString;
	
	public String getDestinatariString() {
		return destinatariString;
	}
	public void setDestinatariString(String destinatariString) {
		this.destinatariString = destinatariString;
	}
	public String getAllegatiString() {
		return allegatiString;
	}
	public void setAllegatiString(String allegatiString) {
		this.allegatiString = allegatiString;
	}
	
	private Boolean cestino = Boolean.FALSE;
	private Date data_creazione = ApplicationUtil.date();
	private Date data_invio;
	
	@OneToMany(mappedBy = "messaggio")
	@OnDelete(action = OnDeleteAction.CASCADE)
	private List<Destinatario> destinatari = new ArrayList<Destinatario>();
	private Boolean eliminato = Boolean.FALSE;
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;
	
	
	@ManyToOne
	private Utente mittente;
	
	
	@Column(columnDefinition = "text")
	private String oggetto;
	private Boolean stato = Boolean.FALSE;
	@Column(columnDefinition = "text")
	private String testo;
	public int compareTo(Messaggio o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Messaggio other = (Messaggio) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	public List<AllegatoMessaggio> getAllegati() {
		return allegati;
	}
	public List<Allegato> getAllegato() {
		if(allegato==null)
			allegato = new ArrayList<Allegato>();
		return allegato;
	}
	public int getAllegatoSize() {
		return ApplicationUtil.size(allegato);
	}
	public Boolean getCestino() {
		return cestino;
	}
	public Messaggio getCopia() {
		Messaggio pr = new Messaggio();
		pr.setOggetto(this.getOggetto());
		pr.setTesto(this.getTesto());
		return pr;
	}
	public Date getData_creazione() {
		return data_creazione;
	}
	public Date getData_invio() {
		return data_invio;
	}
	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getOggetto());
	}
	public List<Destinatario> getDestinatari() {
		return new ArrayList<Destinatario>(destinatari);
	}
	
	public Boolean getEliminato() {
		return eliminato;
	}
	public Long getId() {
		return id;
	}
	public Utente getMittente() {
		return mittente;
	}
	
	
	
	public String getOggetto() {
		return oggetto;
	}
	public Boolean getStato() {
		return stato;
	}
	public String getTesto() {
		return testo;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}
	public void setAllegati(List<AllegatoMessaggio> allegati) {
		this.allegati = allegati;
	}
	
	public void setAllegato(List<Allegato> allegato) {
		this.allegato = allegato;
	}
	
	public void setCestino(Boolean cestino) {
		this.cestino = cestino;
	}
	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}
	public void setData_invio(Date data_invio) {
		this.data_invio = data_invio;
	}
	public void setDestinatari(List<Destinatario> destinatari) {
		this.destinatari = destinatari;
	}
	public void setEliminato(Boolean eliminato) {
		this.eliminato = eliminato;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setMittente(Utente mittente) {
		this.mittente = mittente;
	}

	public void setOggetto(String oggetto) {
		this.oggetto = oggetto;
	}
	
	public void setStato(Boolean stato) {
		this.stato = stato;
	}
	public void setTesto(String testo) {
		this.testo = testo;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}
	
	@Transient
	private String message;
	public void single(UploadEvent event) {
		MyFile file = ApplicationUtil.myFile(event.getUploadItem(), ConstantsGlobal.SIZE);
		if(file != null) {
			//ApplicationUtil.upload(0, file, null);
			if(allegati == null) {
				allegati = new ArrayList<AllegatoMessaggio>();
			}
			allegati.add(new AllegatoMessaggio(file.getLength(), ApplicationUtil.date(), file.getData(), file.getName()));
		} 
		message = ApplicationUtil.sizeError(event.getUploadItem(), ConstantsGlobal.SIZE);
	}
	public void clearUploadData() {
		allegati.clear();
	}
	@Transient
	private AllegatoMessaggio attached;
	
	public void rimuoviAllegato() {
		if(attached != null && allegati.contains(attached)){
			allegati.remove(attached);
		}
	}
	
	public AllegatoMessaggio getAttached() {
		return attached;
	}
	public void setAttached(AllegatoMessaggio attached) {
		this.attached = attached;
	}
	public String getMessage() {
		return message;
	}
	public void setMessage(String message) {
		this.message = message;
	}
	
}