package it.callite.model.report;

import java.io.Serializable;
import java.math.BigDecimal;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.Categoria;
import it.callite.model.Prodotto;
import it.callite.model.Qualifica;
import it.callite.utility.ApplicationUtil;


@SuppressWarnings("serial")
public class EstrattoModel implements Serializable {

	private Boolean contest = false;
	private BigDecimal gettone;
	private BigDecimal valore;

	private Long p_id;

	private Long c_id;

	private Prodotto prodotto;
	private Categoria categoria;
	private Long q_id;
	private Long d_id;
	private Qualifica qualifica;
	private Qualifica differenziale;
	private Integer stato;
	private String tipo;
	private Integer tipoUtente;
	private Long utente_id;

	public Long getC_id() {
		return c_id;
	}
	public Categoria getCategoria() {
		return categoria;
	}
	public Boolean getContest() {
		return contest;
	}
	public Long getD_id() {
		return d_id;
	}
	public String getDescrizione() {
		StringBuilder sb = new StringBuilder();
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.operatore)) {
			sb.append(ConstantsLanguage.OPERATORE.stampa());
		} 
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.supervisore)) {
			sb.append(ConstantsLanguage.SUPERVISORE.stampa());
		} 
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.coordinatore)) {
			sb.append(ConstantsLanguage.COORDINATORE.stampa());
		} 
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.agente)) {
			sb.append(ConstantsLanguage.DIRETTO_AGENTE.stampa());
		} 
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.rete)) {
			sb.append(ConstantsLanguage.RETE_AGENTE.stampa());
		} 
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.prima)) {
			sb.append(ConstantsLanguage.PRIMA_AGENTE.stampa());
		} 
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.sotto)) {
			sb.append(ConstantsLanguage.SOTTO_AGENTE.stampa());
		} 

		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.orete)) {
			sb.append(ConstantsLanguage.RETE_OAGENTE.stampa());
		} 
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.oprima)) {
			sb.append(ConstantsLanguage.PRIMA_OAGENTE.stampa());
		} 
		if(ApplicationUtil.equals(tipoUtente, ConstantsGlobal.osotto)) {
			sb.append(ConstantsLanguage.SOTTO_OAGENTE.stampa());
		} 

		if(contest) {
			sb.append(" - ");
			sb.append(ConstantsLanguage.CONTEST.stampa());
		}

		if(tipo!=null) {
			sb.append(" - ");
			sb.append(tipo);
		}

		if(qualifica!=null) {
			sb.append(" - ");
			sb.append(qualifica.getNome());
		}
		if(differenziale!=null) {
			sb.append(" - ");
			sb.append(differenziale.getNome());
		}

		sb.append(" - ");
		if(ApplicationUtil.equals(stato, ConstantsGlobal.in_pagamento)) {
			sb.append(ConstantsLanguage.STATO_IN_PAGAMENTO.stampa());
		} else {
			sb.append(ConstantsLanguage.STATO_IN_STORNO.stampa());
		}

		if(prodotto!=null) {
			sb.append(" - ");
			sb.append(prodotto.getDenominazione());
			if(!contest) {
				sb.append(" (");
				if(prodotto.getTipologia().equals(ConstantsGlobal.gpercentuale)) {
					sb.append(ConstantsLanguage.PERCENTUALE.stampa());
				} else if(prodotto.getTipologia().equals(ConstantsGlobal.gsomma)) {
					sb.append(ConstantsLanguage.SOMMA.stampa());
				} else {
					sb.append(ConstantsLanguage.CONTEGGIO.stampa());
				}
				sb.append(")");
			}
		}

		if(categoria!=null) {
			sb.append(" - ");
			sb.append(categoria.getDenominazione());
			if(!contest) {
				sb.append(" (");
				if(categoria.getTipologia().equals(ConstantsGlobal.gpercentuale)) {
					sb.append(ConstantsLanguage.PERCENTUALE.stampa());
				} else if(categoria.getTipologia().equals(ConstantsGlobal.gsomma)) {
					sb.append(ConstantsLanguage.SOMMA.stampa());
				} else {
					sb.append(ConstantsLanguage.CONTEGGIO.stampa());
				}
				sb.append(")");
			}
		}
		return sb.toString();

	}
	public Qualifica getDifferenziale() {
		return differenziale;
	}
	public BigDecimal getGettone() {
		if(gettone==null)
			return new BigDecimal(0);
		return gettone;
	}
	public Long getP_id() {
		return p_id;
	}


	public Prodotto getProdotto() {
		return prodotto;
	}
	public Long getQ_id() {
		return q_id;
	}
	public Qualifica getQualifica() {
		return qualifica;
	}
	public Integer getStato() {
		return stato;
	}
	public String getTipo() {
		return tipo;
	}

	public Integer getTipoUtente() {
		return tipoUtente;
	}
	public Long getUtente_id() {
		return utente_id;
	}
	public BigDecimal getValore() {
		if(valore==null)
			return new BigDecimal(0);
		return valore;
	}
	public void setC_id(Long c_id) {
		this.c_id = c_id;
	}
	public void setCategoria(Categoria categoria) {
		this.categoria = categoria;
	}
	public void setContest(Boolean contest) {
		this.contest = contest;
	}
	public void setD_id(Long d_id) {
		this.d_id = d_id;
	}
	public void setDifferenziale(Qualifica differenziale) {
		this.differenziale = differenziale;
	}
	public void setGettone(BigDecimal gettone) {
		this.gettone = gettone;
	}
	public void setP_id(Long p_id) {
		this.p_id = p_id;
	}
	public void setProdotto(Prodotto prodotto) {
		this.prodotto = prodotto;
	}
	public void setQ_id(Long q_id) {
		this.q_id = q_id;
	}
	public void setQualifica(Qualifica qualifica) {
		this.qualifica = qualifica;
	}
	public void setStato(Integer stato) {
		this.stato = stato;
	}
	public void setTipo(String tipo) {
		this.tipo = tipo;
	}
	public void setTipoUtente(Integer tipoUtente) {
		this.tipoUtente = tipoUtente;
	}
	public void setUtente_id(Long utente_id) {
		this.utente_id = utente_id;
	}
	public void setValore(Object valore) {
		if(valore instanceof BigDecimal)
			this.valore = (BigDecimal) valore;
		if(valore instanceof Integer)
			this.valore = new BigDecimal((Integer) valore);
		if(valore instanceof Long)
			this.valore = new BigDecimal((Long) valore);

	}


}
