package it.callite.model.sip;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import it.callite.constants.ConstantsGlobal;
import it.callite.message.Message;
import it.callite.model.Contatto;
import it.callite.model.Utente;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;






@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.ts + "cdr", indexes = {  @Index(name= "cdr", columnList = "calldate, enddate, answerdate, tipo, disposition, duration, billsec" ), @Index(name= "dati_cdr1", columnList = "calldate, tipo, disposition, duration, billsec" ), @Index(name= "dati_cdr2", columnList = "callee, caller, linkedid" ), @Index(name= "dati_cdr3", columnList = "channel, dstchannel, lastapp" ), @Index(name = "id", columnList = "userfield")  })
public class Cdr extends SelezionatoAbstract implements Serializable, Comparable<Cdr> {

	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getSrc() + " " + getDst());
	}

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	@Column(columnDefinition="datetime NOT NULL COMMENT 'data'")
	private Date calldate;
	private String clid;
	private String close = null;
	private String src;
	private String dst;
	private String dcontext;
	private String channel;
	private String dstchannel;
	private String lastapp;
	private String lastdata;
	private Integer duration = 0;
	private Integer billsec = 0;
	private String disposition;
	private int amaflags;
	private String accountcode;
	private String uniqueid;
	private String sequence;
	private String linkedid;
	private String peeraccount;
	private String userfield;
	private String descrizione;
	private Date enddate;
	private Date answerdate;

	public String getPeeraccount() {
		return peeraccount;
	}

	public void setPeeraccount(String peeraccount) {
		this.peeraccount = peeraccount;
	}

	public String getUserfield() {
		return userfield;
	}

	public void setUserfield(String userfield) {
		this.userfield = userfield;
	}

	public void setEnddate(Date enddate) {
		this.enddate = enddate;
	}

	public void setAnswerdate(Date answerdate) {
		this.answerdate = answerdate;
	}

	public String getDescrizione() {
		return descrizione;
	}

	public void setDescrizione(String descrizione) {
		this.descrizione = descrizione;
	}

	@ManyToOne
	@JoinColumn(name = "contatto")
	private Contatto contatto;

	@ManyToOne
	@JoinColumn(name = "utente")
	private Utente utente;

	public void setContatto(Contatto contatto) {
		this.contatto = contatto;
	}

	public void setUtente(Utente utente) {
		this.utente = utente;
	}

	private String callee;
	private String caller;
	private Integer tipo;

	public Integer getTipo() {
		return tipo;
	}

	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}

	public String getTipoText() {
		if(ApplicationUtil.ge(tipo, 0)) {
			return Message.addCatalogText("cdr." + tipo);
		}
		return "";
	}


	public String getCallee() {
		return ApplicationUtil.trimToEmpty(callee);
	}

	public void setCallee(String callee) {
		this.callee = ApplicationUtil.normalizeToNull(callee);
	}

	public String getCaller() {
		return ApplicationUtil.trimToEmpty(caller);
	}

	public void setCaller(String caller) {
		this.caller = ApplicationUtil.normalizeToNull(caller);
	}

	public Contatto getContatto() {
		return contatto;
	}


	public Utente getUtente() {
		return utente;
	}

	public String getSequence() {
		return sequence;
	}

	public void setSequence(String sequence) {
		this.sequence = sequence;
	}

	public String getLinkedid() {
		return linkedid;
	}

	public void setLinkedid(String linkedid) {
		this.linkedid = linkedid;
	}

	public String getAccountcode() {
		return this.accountcode;
	}

	public int getAmaflags() {
		return this.amaflags;
	}


	public Date getCalldate() {
		return this.calldate;
	}

	public String getChannel() {
		return this.channel;
	}

	public String getClid() {
		return this.clid;
	}

	public Integer getDuration() {
		if(ApplicationUtil.ltZero(duration)) {
			duration = 0;
		}
		return duration;
	}

	public void setDuration(Integer duration) {
		this.duration = duration;
	}

	public Integer getBillsec() {
		if(ApplicationUtil.ltZero(billsec)) {
			billsec = 0;
		}
		return billsec;
	}

	public void setBillsec(Integer billsec) {
		this.billsec = billsec;
	}

	public String getDcontext() {
		return this.dcontext;
	}

	public String getDisposition() {
		return this.disposition;
	}

	public String getDst() {
		return this.dst;
	}

	public String getDstchannel() {
		return this.dstchannel;
	}



	public Long getId() {
		return id;
	}

	public String getLastapp() {
		return this.lastapp;
	}

	public String getLastdata() {
		return this.lastdata;
	}

	public String getSrc() {
		return this.src;
	}

	public String getUniqueid() {
		return this.uniqueid;
	}
	public void setAccountcode(String accountcode) {
		this.accountcode = accountcode;
	}

	public void setAmaflags(int amaflags) {
		this.amaflags = amaflags;
	}



	public void setCalldate(Date calldate) {
		this.calldate = calldate;
	}

	public void setChannel(String channel) {
		this.channel = channel;
	}

	public void setClid(String clid) {
		this.clid = clid;
	}

	public void setDcontext(String dcontext) {
		this.dcontext = dcontext;
	}

	public void setDisposition(String disposition) {
		this.disposition = disposition;
	}

	public void setDst(String dst) {
		this.dst = dst;
	}

	public void setDstchannel(String dstchannel) {
		this.dstchannel = dstchannel;
	}



	public void setId(Long id) {
		this.id = id;
	}

	public void setLastapp(String lastapp) {
		this.lastapp = lastapp;
	}

	public void setLastdata(String lastdata) {
		this.lastdata = lastdata;
	}

	public void setSrc(String src) {
		this.src = src;
	}

	public void setUniqueid(String uniqueid) {
		this.uniqueid = uniqueid;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Cdr other = (Cdr) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	public int compareTo(Cdr o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}

	public String getBillsecH() {
		return ApplicationUtil.format(getBillsec());
	}
	public String getDurationH() {
		return ApplicationUtil.format(getDuration());
	}

	public String getContattoDenominazione() {
		return getContatto() != null ? getContatto().getDenominazione() : "";
	}


	public String getContattoCampagnaDenominazione() {
		return getContatto() != null ? getContatto().getCampagna_denominazione() : "";
	}

	public String getContattoListaDenominazione() {
		return getContatto() != null ? getContatto().getLista_denominazione() : "";
	}

	public String getUtenteDenominazione() {
		return getUtente() != null ? getUtente().getDenominazione() : "";
	}

	public Date getEnddate() {
		if(enddate != null) {
			return enddate;
		} else {
			int d = duration != null ? duration.intValue() : 0;
			return ApplicationUtil.posticipaSecondi(getCalldate(), d);
		}
	}

	public Date getAnswerdate() {
		if(answerdate != null) {
			return answerdate;
		} else {
			int d = duration != null ? duration.intValue() : 0;
			int r = billsec != null ? billsec.intValue() : 0;
			return r > 0 ? ApplicationUtil.posticipaSecondi(getCalldate(), d-r) : null;
		}
	}

	public String getClose() {
		return close;
	}

	public void setClose(String close) {
		this.close = close;
	}
	
	
}
