package it.callite.model.sip;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.model.Autodialer;
import it.callite.model.Risposta;
import it.callite.model.Utente;
import it.callite.model.VocalOrder;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.model.report.ColonnaModel;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.Codec;



@SuppressWarnings("serial")
@Entity
//@Audited 
@Table(name = ConstantsGlobal.ts + "sip_buddies", indexes = { @Index(name= "sip_buddies", columnList = "data_creazione, name, username" ), @Index(name= "dati_telefono", columnList = "interurbane, tipologia, attivo, max_contacts" )})
public class SipBuddies extends SelezionatoAbstract implements Serializable, Comparable<SipBuddies> {

	@Column(columnDefinition = "text")
	private String callers;
	@Column(columnDefinition = "text")
	private String musicclass;
	private String add_prefisso;
	private String antecall;
	private Integer vrx;
	private Integer vtx;	
	private Integer tastierino;
	private String tipologia;
	private String segreteria;
	private String coda;
	private String registrazione;
	private String phone;
	private String interne;
	private String interurbane;
	private String paused;
	private String reason_paused;
	private String attivo;
	private String opposizione;
	private String contact_user;
	private Date data_creazione = ApplicationUtil.date();
	private String call_group;
	private String pickup_group;
	
	private String voicemail_password;
	private String voicemail_fullname;
	
	@Column(nullable=false, unique=true)
	private String name;
	private String context;
	private String disallow;
	private String allow;
	private String username;
	private String password;
	private String mailboxes;
	private Integer max_contacts;

	private String from_domain;
	private String from_user;
	private String host;
	private Integer port;
	private String firewall;
	
	private Integer rtp1;
	private Integer rtp2;
	
	private String tz;
	
	public Integer getRtp1() {
		return rtp1;
	}
	public void setRtp1(Integer rtp1) {
		this.rtp1 = rtp1;
	}
	public Integer getRtp2() {
		return rtp2;
	}
	public void setRtp2(Integer rtp2) {
		this.rtp2 = rtp2;
	}
	@Transient
	private List<String> valore_multiplo;
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY) 
	private Long id;
	
	@OneToMany
	@JoinColumn(name = "interno_id", nullable = true, updatable=true, insertable=true)
	private List<Utente> utenti;
	
	@OneToMany
	@JoinColumn(name = "trunk_id", nullable = true, updatable=true, insertable=true)
	private List<Autodialer> autodialer = new ArrayList<Autodialer>();
	
	@OneToMany
	@JoinColumn(name = "trunk_id", nullable = true, updatable=true, insertable=true)
	private List<Risposta> risposte = new ArrayList<Risposta>();
	
	@OneToMany
	@JoinColumn(name = "trunk_id", nullable = true, updatable=true, insertable=true)
	private List<Utente> users = new ArrayList<Utente>();
	
	@OneToMany
	@JoinColumn(name = "trunk_id", nullable = true, updatable=true, insertable=true)
	private List<VocalOrder> vocals;
	
	
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = ConstantsGlobal.tp + "interno_coda", joinColumns=@JoinColumn(name="interno_id", nullable=false, updatable=true, insertable=true), inverseJoinColumns=@JoinColumn(name="coda_id", nullable=false, updatable=true, insertable=true)    )
	private List<Coda> code = new ArrayList<Coda>();
	
	public List<Coda> getCode() {
		return code;
	}
	public void setCode(List<Coda> code) {
		this.code = code;
	}
	public SipBuddies() {

	}
	public SipBuddies(Boolean tip, String codecA) {
		tipologia = ApplicationUtil.convertBS(tip);
		if(tip) {
			context = ConstantsMap.context_u;
			phone = ConstantsMap.phone1;
			port = 0;
			callers = "";
			tastierino = ConstantsMap.t2;
			opposizione = ApplicationUtil.convertBS(true);
			registrazione = ApplicationUtil.convertBS(false);
			segreteria = ApplicationUtil.convertBS(false);
			coda = ApplicationUtil.convertBS(false);
			interne = ApplicationUtil.convertBS(true);
		} else { 
			context = ConstantsMap.context_u;
			port = ConstantsGlobal.default_sip;
			callers = ConstantsGlobal.callerid;
			musicclass = "";
			vtx = ConstantsGlobal.DEFAULT_VTX;
			vrx = ConstantsGlobal.DEFAULT_VRX;
			opposizione = ApplicationUtil.convertBS(false);
			registrazione = ApplicationUtil.convertBS(false);
		}
		tz = ConstantsGlobal.TIMEZONE;
		rtp1 = 0;
		rtp2 = 0;
		interurbane = ApplicationUtil.convertBS(true);
		firewall = ApplicationUtil.convertBS(false);
		attivo = ApplicationUtil.convertBS(true);
		max_contacts = 10;
		disallow = "all";
		allow = codecA;

		
	}
	public SipBuddies(Long id, String add_prefisso, String antecall, String callers, String musicclass, String interurbane, String attivo, String name, String opposizione, String registrazione, Integer vrx, Integer vtx) {
		this.id = id;
		this.add_prefisso = add_prefisso;
		this.antecall = antecall;
		this.callers = callers;
		this.musicclass = musicclass;
		this.interurbane = interurbane;
		this.attivo = attivo;
		this.name = name;
		this.opposizione = opposizione;
		this.registrazione = registrazione;
		this.vrx = vrx;
		this.vtx = vtx;
	}
	public int compareTo(SipBuddies o) {
		if(id != null && o.id != null) {
			return id.compareTo(o.id);
		} else if(id != null && o.id == null) {
			return -1;
		} else if(id == null && o.id != null) {
			return 1;
		} else {
			return 0;
		}
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SipBuddies other = (SipBuddies) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (add_prefisso == null) {
			if (other.add_prefisso != null)
				return false;
		} else if (!add_prefisso.equals(other.add_prefisso))
			return false;
		if (allow == null) {
			if (other.allow != null)
				return false;
		} else if (!allow.equals(other.allow))
			return false;
		if (callers == null) {
			if (other.callers != null)
				return false;
		} else if (!callers.equals(other.callers))
			return false;
		if (musicclass == null) {
			if (other.musicclass != null)
				return false;
		} else if (!musicclass.equals(other.musicclass))
			return false;
		if (username == null) {
			if (other.username != null)
				return false;
		} else if (!username.equals(other.username))
			return false;
		if (from_user == null) {
			if (other.from_user != null)
				return false;
		} else if (!from_user.equals(other.from_user))
			return false;
		if (from_domain == null) {
			if (other.from_domain != null)
				return false;
		} else if (!from_domain.equals(other.from_domain))
			return false;
		if (interne == null) {
			if (other.interne != null)
				return false;
		} else if (!interne.equals(other.interne))
			return false;
		if (interurbane == null) {
			if (other.interurbane != null)
				return false;
		} else if (!interurbane.equals(other.interurbane))
			return false;
		if (attivo == null) {
			if (other.attivo != null)
				return false;
		} else if (!attivo.equals(other.attivo))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (opposizione == null) {
			if (other.opposizione != null)
				return false;
		} else if (!opposizione.equals(other.opposizione))
			return false;
		if (registrazione == null) {
			if (other.registrazione != null)
				return false;
		} else if (!registrazione.equals(other.registrazione))
			return false;
		if (password == null) {
			if (other.password != null)
				return false;
		} else if (!password.equals(other.password))
			return false;
		if (vrx == null) {
			if (other.vrx != null)
				return false;
		} else if (!vrx.equals(other.vrx))
			return false;
		if (vtx == null) {
			if (other.vtx != null)
				return false;
		} else if (!vtx.equals(other.vtx))
			return false;
		return true;
	}
	

	public String getAdd_prefisso() {
		return add_prefisso;
	}

	public String getAllow() {
		return allow;
	}

	public List<Autodialer> getAutodialer() {
		return autodialer;
	}

	public List<Risposta> getRisposte() {
		return risposte;
	}
	public void setRisposte(List<Risposta> risposte) {
		this.risposte = risposte;
	}
	public String getCallers() {
		return ApplicationUtil.trimToNull(callers);
	}


	public Boolean getCoda() {
		return ApplicationUtil.convertSB(coda);
	}

	public String getContext() {
		return context;
	}

	public Date getData_creazione() {
		return data_creazione;
	}

	public String getUsername() {
		return ApplicationUtil.trimToEmpty(username);
	}

	public String getDenominazione() {
		return ApplicationUtil.trimToEmpty(getName());
	}

	public String getDisallow() {
		return disallow;
	}

	public String getFrom_domain() {
		return ApplicationUtil.trimToEmpty(from_domain);
	}
	public String getFrom_user() {
		return ApplicationUtil.trimToEmpty(from_user);
	}

	


	public String getContact_user() {
		return ApplicationUtil.trimToEmpty(contact_user);
	}
	public void setContact_user(String contact_user) {
		this.contact_user = ApplicationUtil.trimToEmpty(contact_user);
	}
	public String getHost() {
		return ApplicationUtil.trimToEmpty(host);
	}
	public Long getId() {
		return id;
	}

	public Boolean getInterne() {
		return ApplicationUtil.convertSB(interne);
	}

	public String getInternoContext() {
		StringBuilder sb = new StringBuilder();
		sb.append(getName());
		boolean b1 = ApplicationUtil.isTrue(getInterurbane());
		boolean b2 = ApplicationUtil.isTrue(getOpposizione());
		boolean b3 = ApplicationUtil.isTrue(getInterne());
		if(b1 || b2 || b3) {
			sb.append(ConstantsGlobal.pipeSeparator);
		}
		if(b1) {
			sb.append("i");
		}
		if(b2) {
			sb.append("o");
		}
		if(b3) {
			sb.append("a");
		}
		return sb.toString();
	}

	public Boolean getInterurbane() {
		return ApplicationUtil.convertSB(interurbane);
	}
	
	public Boolean getAttivo() {
		return ApplicationUtil.convertSB(attivo);
	}

	public List<Codec> getListaCodec() {
		List<Codec> list = new ArrayList<Codec>();
		for(String valore : ConstantsMap.getValoriStatici()) {
			Codec cod = new Codec(); 
			String value = ApplicationUtil.trimToEmpty(valore);
			cod.setCodec(ApplicationUtil.upperCaseEmpty(value));
			if(getValore_multiplo().contains(value))
				cod.setAttivo(true);
			else
				cod.setAttivo(false);
			list.add(cod);
		}
		return list;
	}

	public Boolean getLoggato() {
		return Chiamate.getCodaInterni().isRegistrato(getName());
	}

	public String getMailboxes() {
		return ApplicationUtil.trimToEmpty(mailboxes);
	}

	public String getMusicclass() {
		return ApplicationUtil.trimToNull(musicclass);
	}
	
	@Transient
	private ColonnaModel model;

	public ColonnaModel getModel() {
		if(model == null) {
			model = new ColonnaModel(id);
		}
		return model;
	}
	public void setModel(ColonnaModel model) {
		this.model = model;
	}

	
	public String getName() {
		return ApplicationUtil.trimToEmpty(name);
	}

	public Boolean getOpposizione() {
		return ApplicationUtil.convertSB(opposizione);
	}

	
	public String getPhone() {
		if(phone == null) {
			phone = ConstantsMap.phone1;
		}
		return phone;
	}
	public String getPhoneS() {
		String p = getPhone();
		return p.equals(ConstantsMap.phone1) ? ConstantsMap.phone1 : ConstantsMap.phone2;
	}
	
	public Integer getPort() {
		return port;
	}
	

	public Boolean getPronto() {
		return Chiamate.getCodaInterni().isLibero(getName());
	}
	
	public Boolean getRegistrazione() {
		return ApplicationUtil.convertSB(registrazione);
	}
	
	public String getPassword() {
		return password;
	}
	public Boolean getSegreteria() {
		return ApplicationUtil.convertSB(segreteria);
	}
	
	public Boolean getSpiato() {
		return Chiamate.getCodaSpia().spiato(getName());
	}
	

	public Integer getTastierino() {
		if(tastierino == null) {
			tastierino =  ConstantsMap.t2;
		}
		return tastierino;
	}


	public Boolean getTipologia() {
		return ApplicationUtil.convertSB(tipologia);
	}
	
	public String getTrunkContext() {
		StringBuilder sb = new StringBuilder();
		sb.append(ApplicationUtil.toString(id, getName()));
		boolean b1 = ApplicationUtil.isTrue(getInterurbane());
		boolean b2 = ApplicationUtil.isTrue(getOpposizione());
		if(b1 || b2) {
			sb.append(ConstantsGlobal.pipeSeparator);
		}
		if(b1) {
			sb.append("i");
		}
		if(b2) {
			sb.append("o");
		}
		return sb.toString();
	}
	
	public List<Utente> getUsers() {
		return users;
	}
	public List<Utente> getUtenti() {
		return utenti;
	}
	public List<String> getValore_multiplo() {
		return ApplicationUtil.token(allow, ConstantsGlobal.commaSeparator);
	}
	public List<SelectItem> getValori() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		for(String valore : ConstantsMap.getValoriStatici()) {
			list.add(new SelectItem(valore, valore));
		}
		return list;
	}
	
	public List<VocalOrder> getVocals() {
		return vocals;
	}

	public Integer getVrx() {
		return vrx;
	}


	public Integer getVtx() {
		return vtx;
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((add_prefisso == null) ? 0 : add_prefisso.hashCode());
		result = prime * result + ((allow == null) ? 0 : allow.hashCode());
		result = prime * result + ((callers == null) ? 0 : callers.hashCode());
		result = prime * result + ((musicclass == null) ? 0 : musicclass.hashCode());
		result = prime * result + ((username == null) ? 0 : username.hashCode());
		result = prime * result + ((from_user == null) ? 0 : from_user.hashCode());
		result = prime * result + ((from_domain == null) ? 0 : from_domain.hashCode());
		result = prime * result + ((interne == null) ? 0 : interne.hashCode());
		result = prime * result + ((interurbane == null) ? 0 : interurbane.hashCode());
		result = prime * result + ((attivo == null) ? 0 : attivo.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((opposizione == null) ? 0 : opposizione.hashCode());
		result = prime * result + ((registrazione == null) ? 0 : registrazione.hashCode());
		result = prime * result + ((password == null) ? 0 : password.hashCode());
		result = prime * result + ((vrx == null) ? 0 : vrx.hashCode());
		result = prime * result + ((vtx == null) ? 0 : vtx.hashCode());
		return result;
	}

	public void impostaInterno() {
		this.username = getName();
		if(!ApplicationUtil.isTrue(getSegreteria())) {
			voicemail_fullname = "";
			voicemail_password = "";
		}
		
	}

	public void impostaTrunk() {
		String callers = getCallers();
		if(ApplicationUtil.isBlank(callers) || ApplicationUtil.equals(callers, ConstantsGlobal.callerid)) {
			antecall = null;
		}
	}

	public void setAdd_prefisso(String add_prefisso) {
		this.add_prefisso = ApplicationUtil.normalizeToNull(add_prefisso);
	}

	public void setAllow(String allow) {
		this.allow = allow;
	}
	public void setAutodialer(List<Autodialer> autodialer) {
		this.autodialer = autodialer;
	}
	
	public void setCallers(String callers) {
		this.callers = ApplicationUtil.aggiustaLista(false, ConstantsGlobal.colonSeparator, callers);
	}

	public Integer getMax_contacts() {
		return max_contacts;
	}
	public void setMax_contacts(Integer max_contacts) {
		this.max_contacts = max_contacts;
	}
	public void setCoda(Boolean coda) {
		this.coda = ApplicationUtil.convertBS(coda);
	}

	public void setContext(String context) {
		this.context = context;
	}

	public void setData_creazione(Date data_creazione) {
		this.data_creazione = data_creazione;
	}

	public void setUsername(String username) {
		this.username = ApplicationUtil.normalizeToEmpty(username);
	}


	public void setDisallow(String disallow) {
		this.disallow = disallow;
	}

	public void setFrom_domain(String from_domain) {
		this.from_domain = ApplicationUtil.trimToEmpty(from_domain);
	}


	public void setFrom_user(String from_user) {
		this.from_user = ApplicationUtil.normalizeToEmpty(from_user);
	}

	public void setHost(String host) {
		this.host = ApplicationUtil.normalizeToEmpty(host);
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setInterne(Boolean interne) {
		this.interne = ApplicationUtil.convertBS(interne);
	}

	public void setInterurbane(Boolean interurbane) {
		this.interurbane = ApplicationUtil.convertBS(interurbane);
	}
	
	public Boolean getFirewall() {
		return ApplicationUtil.convertSB(firewall);
	}
	public void setFirewall(Boolean firewall) {
		this.firewall = ApplicationUtil.convertBS(firewall);
	}
	public void setAttivo(Boolean attivo) {
		this.attivo = ApplicationUtil.convertBS(attivo);
	}
	
	public void setMailboxes(String mailboxes) {
		this.mailboxes = ApplicationUtil.normalizeToEmpty(mailboxes);
	}

	public void setMusicclass(String musicclass) {
		this.musicclass = ApplicationUtil.trimToNull(musicclass);
	}

	

	public void setName(String name) {
		this.name = ApplicationUtil.normalizeToEmpty(name);
	}

	public void setOpposizione(Boolean opposizione) {
		this.opposizione = ApplicationUtil.convertBS(opposizione);
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public void setPort(Integer port) {
		this.port = port;
	}

	
	public void setRegistrazione(Boolean registrazione) {
		this.registrazione = ApplicationUtil.convertBS(registrazione);
	}

	public void setPassword(String password) {
		this.password = ApplicationUtil.normalizeToEmpty(password);
	}


	public void setSegreteria(Boolean segreteria) {
		this.segreteria = ApplicationUtil.convertBS(segreteria);
	}

	public void setTastierino(Integer tastierino) {
		this.tastierino = tastierino;
	}

	public void setTipologia(Boolean tipologia) {
		this.tipologia = ApplicationUtil.convertBS(tipologia);
	}

	public void setUsers(List<Utente> users) {
		this.users = users;
	}

	public void setUtenti(List<Utente> utenti) {
		this.utenti = utenti;
	}

	public void setValore_multiplo(List<String> valoreMultiplo) {
		this.allow = ApplicationUtil.detoken(valoreMultiplo, ConstantsGlobal.commaSeparator);
	}
	
	public String getCodecs() {
		List<String> st = ApplicationUtil.token(allow, ConstantsGlobal.commaSeparator);
		Collections.sort(st);
		return ApplicationUtil.detoken(st, ConstantsGlobal.colonSeparator);
	}

	public void setVocals(List<VocalOrder> vocals) {
		this.vocals = vocals;
	}

	public void setVrx(Integer vrx) {
		this.vrx = vrx;
	}
	public void setVtx(Integer vtx) {
		this.vtx = vtx;
	}
	@Override
	public String toString() {
		return ApplicationUtil.toString(id, getDenominazione());
	}
	public String getAntecall() {
		return antecall;
	}
	public void setAntecall(String antecall) {
		this.antecall = ApplicationUtil.normalizeToNull(antecall);
	}
	public String getVoicemail_password() {
		return ApplicationUtil.trimToEmpty(voicemail_password);
	}
	public void setVoicemail_password(String voicemail_password) {
		this.voicemail_password = ApplicationUtil.normalizeToEmpty(voicemail_password);
	}
	public String getVoicemail_fullname() {
		return ApplicationUtil.trimToEmpty(voicemail_fullname);
	}
	public void setVoicemail_fullname(String voicemail_fullname) {
		this.voicemail_fullname = ApplicationUtil.normalizeToEmpty(voicemail_fullname);
	}
	
	public String getTz() {
		return tz;
	}
	public void setTz(String tz) {
		this.tz = tz;
	}
	
	public String getCall_group() {
		return ApplicationUtil.trimToEmpty(call_group);
	}
	public void setCall_group(String call_group) {
		this.call_group = ApplicationUtil.trimToEmpty(call_group);
	}
	public String getPickup_group() {
		return ApplicationUtil.trimToEmpty(pickup_group);
	}
	public void setPickup_group(String pickup_group) {
		this.pickup_group = ApplicationUtil.trimToEmpty(pickup_group);
	}
	public String getPaused() {
		return paused;
	}
	public void setPaused(String paused) {
		this.paused = paused;
	}
	public String getReason_paused() {
		return reason_paused;
	}
	public void setReason_paused(String reason_paused) {
		this.reason_paused = reason_paused;
	}
	
}
