package it.callite.security;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationFailureHandler;

import it.callite.chiamate.Chiamate;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.dao.OperazioneDao;
import it.callite.dao.UserDao;
import it.callite.message.Message;
import it.callite.model.Utente;
import it.callite.utility.ApplicationUtil;
public class AuthenticationLoginFailureHandler extends SimpleUrlAuthenticationFailureHandler {

	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private UserDao userDao;

	@Override
	public void onAuthenticationFailure(HttpServletRequest request, HttpServletResponse response, AuthenticationException exception) throws IOException, ServletException {
		setDefaultFailureUrl(ConstantsGlobal.path_web_login);
		String u = ApplicationUtil.trimToEmpty(request.getParameter("c_username"));
		Utente utente = userDao.getUserByUsername(u);
		List<String> lm = new ArrayList<String>();
		String ip = ApplicationUtil.ip(request);
		if(utente != null) {
			String motivo = ApplicationUtil.exceptionLogin(exception);
			String e = motivo != null ? ApplicationUtil.trimToEmpty(Message.addCatalogText(motivo))  : "";
			lm.add(e);
			String username = utente.getUsername();
			Chiamate.getCodaLogin().loginFailure(username, ip, motivo);
			int m1 = operazioneDao.getMassimoLogin1();
			if(m1 > 0) {
				int t1 = Chiamate.getCodaLogin().get(ip);
				lm.add(ConstantsLanguage.IP.stampa() + ": " + ip + " " + "(" + ConstantsLanguage.TENTATIVI.stampa()+ ": " + (m1 > 0 ? (t1 + "/" + m1) : (t1 + "")) +")");
				if(t1 >= m1) {
					lm.add(ConstantsLanguage.ACCOUNT_FILTRATO.stampa());
				}
			}
			operazioneDao.loginFallito(ip, username);
			int m2 = operazioneDao.getMassimoLogin2();
			if(m2 > 0) {
				int t2 = operazioneDao.contaTentativi(ip, username);
				lm.add(ConstantsLanguage.USERNAME.stampa() + ": " + username + " " + "(" + ConstantsLanguage.TENTATIVI.stampa()+ ": " + (m2 > 0 ? (t2 + "/" + m2) : (t2 + "")) +")");
				if(t2 >= m2) {
					lm.add(ConstantsLanguage.ACCOUNT_BLOCCATO.stampa() + ": " + ConstantsLanguage.ATTENDERE.stampa() + " " + ConstantsGlobal.TIME_BLOCCO + " " + ConstantsLanguage.MINUTI.stampa());
				}
			}
		} else {
			lm.add(ConstantsLanguage.USERNAME.stampa() + ": " + u + " - " + ConstantsLanguage.INESISTENTE.stampa());
		}
		ApplicationUtil.loginMessage(request, ApplicationUtil.detoken(lm, ConstantsGlobal.RETURN));
		operazioneDao.loginFailure(ApplicationUtil.detoken(lm, ConstantsGlobal.colonSeparator + " "), ip);

		super.onAuthenticationFailure(request, response, exception);
	}

}


