package it.callite.service;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Semaphore;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.dao.ComunicazioneDao;
import it.callite.dao.DownloadDao;
import it.callite.dao.ImpostazioneDao;
import it.callite.log.ErrorLog;
import it.callite.model.Email;
import it.callite.model.Rmail;
import it.callite.model.Telecom;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.EmailUtils;
import it.callite.utility.InvocaAPI;
import it.callite.utility.Label;
import it.callite.utility.MailSupport;
import it.callite.utility.thread.invio.SingoloEmailSenderThread;
import it.callite.utility.thread.invio.SingoloRmailSenderThread;
import it.callite.utility.thread.invio.SingoloTelecomSenderThread;

@SuppressWarnings("serial")
@Service("comunicazioneService")
public class ComunicazioneService implements Serializable {

	@Autowired
	private ComunicazioneDao comunicazioneDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private DownloadDao downloadDao;
	
	private Semaphore mutexE = new Semaphore(1);
	private Semaphore mutexR = new Semaphore(1);
	private Semaphore mutexT = new Semaphore(1);

	public void email(MailSupport ms, Email o) {
		if (o != null) {
			boolean s = comunicazioneDao.sleepEmailSemaforo();
			try {
				if(s) {
					mutexE.acquire();
				}
				String append = messageTemplate("email", o.getId(), o.getContatto().getId(), o.getHtml(), o.getLink(), o.getEmailTemplate().getAnalytics());
				boolean i = EmailUtils.email(ms, o, append, null);
				if(i == true) {
					o.addMotivo(ConstantsGlobal.INVIATA, null);
				} else {
					o.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
				}
			} catch (Exception e) {
				o.addMotivo(ConstantsGlobal.NON_INVIATA,e.getMessage());
				ErrorLog.log(null, getClass(), e);
			}
			comunicazioneDao.salvaEmail(o);
			ApplicationUtil.sleep(comunicazioneDao.sleepEmailTime());
			if(s) {
				mutexE.release();
			}
		}
	}

	public void rmail(MailSupport ms, Rmail o) {
		if (o != null) {
			boolean s = comunicazioneDao.sleepRmailSemaforo();
			try {
				if(s) {
					mutexR.acquire();
				}
				String append = messageTemplate("rmail", o.getId(), o.getContatto().getId(), o.getHtml(), o.getLink(), o.getRmailTemplate().getAnalytics());
				boolean i = EmailUtils.email(ms, o, append, null);
				if(i == true) {
					o.addMotivo(ConstantsGlobal.INVIATA, null);
				} else {
					o.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
				}
			} catch (Exception e) {
				o.addMotivo(ConstantsGlobal.NON_INVIATA,e.getMessage());
				ErrorLog.log(null, getClass(), e);
			}
			comunicazioneDao.salvaRmail(o);
			ApplicationUtil.sleep(comunicazioneDao.sleepRmailTime());
			if(s) {
				mutexR.release();
			}
			
		}
	}

	public void telecom(Boolean invio, Telecom o) {
		if (o != null) {
			boolean s = comunicazioneDao.sleepTelecomSemaforo();
			try {
				if(s) {
					mutexT.acquire();
				}
				List<Label> labels = o.getLabels();
				Map<String, Label> inputs = ApplicationUtil.createMap(labels);
				if(ApplicationUtil.empty(inputs)) {
					Map<String, Label> lb = ApplicationUtil.createMap(ApplicationUtil.readText(o.getInputs()));
					inputs.putAll(lb);
				} else {
					o.setInputs(ApplicationUtil.replaceInString(o.getInputs(), inputs));
				}
				o.setDriver(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getDriver(), inputs)));
				o.setCommands(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getCommands(), inputs)));
				o.setLink(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getLink(), inputs)));
				o.setBody(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getBody(), inputs)));
				o.setHeaders(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getHeaders(), inputs)));
				o.setFields(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getFields(), inputs)));
				o.setOutputs(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getOutputs(), inputs)));
				o.setParameters(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getParameters(), inputs)));
				o.setUsername(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getUsername(), inputs)));
				o.setPassword(comunicazioneDao.replace(o.getContatto(), ApplicationUtil.replaceInString(o.getPassword(), inputs)));
				String response = InvocaAPI.invoke(o.getSistema(), o.getTipo(), o.getLink(), o.getHeaders(), o.getFields(), o.getOutputs(), o.getParameters(), o.getBody(), o.getUsername(), o.getPassword(), o.getDriver(), o.getCommands());
				o.setResponse(response);
				o.addMotivo(ConstantsGlobal.INVIATA, null);
			} catch (Exception e) {
				o.addMotivo(ConstantsGlobal.NON_INVIATA,e.getMessage());
				ErrorLog.log(null, getClass(), e);
			}
			comunicazioneDao.salvaTelecom(o);
			ApplicationUtil.sleep(comunicazioneDao.sleepTelecomTime());
			if(s) {
				mutexT.release();
			}
		}
	}




	public void email(Email o) {
		new SingoloEmailSenderThread(o, this).start();
	}



	public void rmail(Rmail o) {
		new SingoloRmailSenderThread(o, this).start();
	}


	public void telecom(Telecom o) {
		new SingoloTelecomSenderThread(o, this).start();
	}

	public String messageTemplate(String tipo, Long id, Long contattoId, Boolean html, Boolean attivo, String analytics) {
		if(ApplicationUtil.isTrue(html)) {
			Map<String, Object> model = new HashMap<String, Object>();
			String url = impostazioneDao.getWebUrl();
			String ids = ApplicationUtil.base64Encoder(id.toString());
			model.put("url", url + ConstantsGlobal.path_web_message + "?type=" + tipo + "&hash=" + ids);
			model.put("monitor", url + ConstantsGlobal.path_web_message + "/" + tipo + "/" + ids + "." + ConstantsMap.png);
			model.put("analytics", analytics);
			model.put("clientId", contattoId+"."+id.toString());
			model.put("attivo", ApplicationUtil.isTrue(attivo));
			return downloadDao.mergeVelocityTemplate("email", model);
		}
		return "";
	}

}
