package it.callite.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.dao.ComunicazioneDao;
import it.callite.dao.ImpostazioneDao;
import it.callite.log.ErrorLog;
import it.callite.model.Comunicazione;
import it.callite.model.Notifica;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.EmailUtils;
import it.callite.utility.MailSupport;
import it.callite.utility.thread.email.MailSenderThread;
import it.callite.utility.thread.email.NotificaSenderThread;

@Service("smtpService")
public class SmtpService {

	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private ComunicazioneDao comunicazioneDao;


	public MailSupport getMailSupport(boolean single) {
		return impostazioneDao.getMailSupport(single);
	}

	public void invia(MailSupport ms, String sender, String to, String cc, String oggetto, String testo, List<Allegato> allegati, String template) {
		Comunicazione o = new Comunicazione(sender, to, cc, oggetto, testo, allegati, template);
		email(ms, o);
	}

	public void invia(String sender, String to, String cc, String oggetto, String testo, List<Allegato> allegati, String template) {
		Comunicazione o = new Comunicazione(sender, to, cc, oggetto, testo, allegati, template);
		new MailSenderThread(this, o).start();
	}

	public void email(MailSupport ms, Comunicazione o) {
		if (o != null) {
			try {
				boolean i = EmailUtils.email(ms, o, "", impostazioneDao.getApplicazione());
				if(i == true) {
					o.addMotivo(ConstantsGlobal.INVIATA, null);
				} else {
					o.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
				}
			} catch (Exception e) {
				o.addMotivo(ConstantsGlobal.NON_INVIATA,e.getMessage());
				ErrorLog.log(null, getClass(), e);
			}
			if(o.getId() != null || ApplicationUtil.isTrue(impostazioneDao.salvaComunicazione())) {
				comunicazioneDao.salvaComunicazione(o);
			}
		}
	}

	public void notifica(MailSupport ms, Notifica o) {
		if (o != null) {
			try {
				boolean i = EmailUtils.email(ms, o, "", impostazioneDao.getApplicazione());
				if(i == true) {
					o.addMotivo(ConstantsGlobal.INVIATA, null);
				} else {
					o.addMotivo(ConstantsGlobal.NON_INVIATA, ConstantsLanguage.ERRORE_INVIO.stampa());
				}
			} catch (Exception e) {
				o.addMotivo(ConstantsGlobal.NON_INVIATA,e.getMessage());
				ErrorLog.log(null, getClass(), e);
			}
			comunicazioneDao.salvaNotifica(o);
		}
	}

	public void email(Comunicazione o) {
		new MailSenderThread(this, o).start();
	}

	public void notifica(Notifica o) {
		new NotificaSenderThread(this, o).start();
	}

}
