package it.callite.utility;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
public class CampoSpeciale implements Serializable {

	private static final long serialVersionUID = 1L;

	private String principale;
	private String chiave;
	private String colore;
	private String stile;
	private String values;
	private String descrizione;
	private Boolean eredita;
	private String etichetta;
	private String defaults;
	private Boolean modificabile_agente;
	private Boolean modificabile_operatore;
	private Boolean obbligatorio;
	private Integer tipo;
	private Boolean visibile_agente;
	private Boolean visibile_operatore;
	public CampoSpeciale(String chiave, String principale) {
		this.chiave = chiave;
		this.principale = principale;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CampoSpeciale other = (CampoSpeciale) obj;
		if (chiave == null) {
			if (other.chiave != null)
				return false;
		} else if (!chiave.equals(other.chiave))
			return false;
		if (principale == null) {
			if (other.principale != null)
				return false;
		} else if (!principale.equals(other.principale))
			return false;
		return true;
	}
	public String getChiave() {
		return chiave;
	}
	public String getColore() {
		return colore;
	}
	public Boolean getEredita() {
		return eredita;
	}
	public String getEtichetta() {
		return etichetta;
	}
	public Boolean getModificabile_agente() {
		return modificabile_agente;
	}
	public Boolean getModificabile_operatore() {
		return modificabile_operatore;
	}
	public Boolean getObbligatorio() {
		return obbligatorio;
	}
	public Integer getTipo() {
		if (tipo == null)
			tipo = ConstantsMap.TIPO_APERTO;
		return tipo;
	}
	public Boolean getVisibile_agente() {
		return visibile_agente;
	}
	public Boolean getVisibile_operatore() {
		return visibile_operatore;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chiave == null) ? 0 : chiave.hashCode());
		result = prime * result + ((principale == null) ? 0 : principale.hashCode());
		return result;
	}
	public void setChiave(String chiave) {
		this.chiave = chiave;
	}
	public void setColore(String colore) {
		this.colore = colore;
	}
	public void setEredita(Boolean eredita) {
		this.eredita = eredita;
	}
	public void setEtichetta(String etichetta) {
		this.etichetta = etichetta;
	}
	public void setModificabile_agente(Boolean modificabile_agente) {
		this.modificabile_agente = modificabile_agente;
	}
	public void setModificabile_operatore(Boolean modificabile_operatore) {
		this.modificabile_operatore = modificabile_operatore;
	}
	public void setObbligatorio(Boolean obbligatorio) {
		this.obbligatorio = obbligatorio;
	}
	public void setTipo(Integer tipo) {
		this.tipo = tipo;
	}
	
	public String getStile() {
		return stile;
	}
	public void setStile(String stile) {
		this.stile = stile;
	}
	public void setVisibile_agente(Boolean visibile_agente) {
		this.visibile_agente = visibile_agente;
	}
	public void setVisibile_operatore(Boolean visibile_operatore) {
		this.visibile_operatore = visibile_operatore;
	}
	public String getValues() {
		return values;
	}
	public String getValuesT() {
		String f = ApplicationUtil.trimToNull(getValues()); 
		if(getData()) {
			return f != null ? f : ConstantsFormat.date.stampa();
		} 
		if(getDataora()) {
			return f != null ? f : ConstantsFormat.dateTime.stampa();
		} 
		if(getOra()) {
			return f != null ? f : ConstantsFormat.time.stampa();
		} 
		return f;
	}
	public void setValues(String values) {
		this.values = values;
	}
	public String getPrincipale() {
		return principale;
	}
	public void setPrincipale(String principale) {
		this.principale = principale;
	}
	public List<String> getValuesList() {
		return ApplicationUtil.tokenU(this.getValues(), ConstantsGlobal.colonSeparator);
	}
	public List<String> autocomplete(Object query) {
		String q = ApplicationUtil.upperCaseEmpty((String) query);
		List<String> list = new ArrayList<String>();
		if(ApplicationUtil.isNotBlank(q)) {
			List<String> valori = ApplicationUtil.tokenU(this.getValues(), ConstantsGlobal.colonSeparator);
			for(String valore : valori) {
				String v = ApplicationUtil.trimToNull(valore);
				if(v != null && ApplicationUtil.upperCaseEmpty(v).contains(q) && !list.contains(v)) {
					list.add(v);
				}
			}
		} 
		return list;
	}
	
	public Boolean getTelefonico() {
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_TELEFONICO));
	}
	public Boolean getRegulare() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_REGULARE));
	}
	public Boolean getUnivoco() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_UNIVOCO));
	}
	public Boolean getSuggerimento() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_SUGGERIMENTO));
	}
	public Boolean getCombo() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_COMBOBOX));
	}
	public Boolean getSpinner() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_SPINNER));
	}
	public Boolean getFile() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_FILE));
	}
	public Boolean getNumero() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_NUMERO));
	}
	public Boolean getIntero() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_INTERO));
	}
	public Boolean getSlider() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_SLIDER));
	}
	public Boolean getLink() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_LINK));
	}
	public Boolean getAperto() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_APERTO));
	}
	public Boolean getForm() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_FORM));
	}
	public Boolean getBinario() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_BINARIO));
	}
	public Boolean getEditor() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_EDITOR));
	}
	public Boolean getInplaceInput() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_INPLACEINPUT));
	}
	public Boolean getInplaceSelect() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_INPLACESELECT));
	}
	public Boolean getColor() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_COLOR));
	}
	public Boolean getTextarea() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_TEXTAREA));
	}
	public Boolean getBooleano() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_BOOLEANO));
	}
	public Boolean getDataora() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_DATAORA));
	}
	public Boolean getOra() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_ORA));
	}
	public Boolean getData() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_DATA));
	}
	public Boolean getOneMenu() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_ONEMENU));
	}
	public Boolean getOneRadio() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_ONERADIO));
	}
	public Boolean getManyCheckbox() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_MANYCHECKBOX));
	}
	public Boolean getPickList() { 
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_PICKLIST));
	}
	public Boolean getManyMenu() {
		return tipo!=null && (tipo.equals(ConstantsMap.TIPO_MANYMENU));
	}
	
	public void imposta() {
		if(tipo == null) {
			tipo = ConstantsMap.TIPO_APERTO;
		}
	}
	public String getDefaults() {
		return defaults;
	}
	public void setDefaults(String defaults) {
		this.defaults = defaults;
	}
	public String getDescrizione() {
		return descrizione;
	}
	public void setDescrizione(String descrizione) {
		this.descrizione = descrizione;
	}
	
}