package it.callite.utility;
import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.List;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.util.TableOrder;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

public class ChartUtils {

	public static BufferedImage createChart(String tipo, String title, String category, String value, boolean legend, boolean orientation, Integer max_width, Integer max_height, List<LabelQuery> lq) {
		if(tipo != null && ApplicationUtil.notEmpty(lq)) {
			JFreeChart chart = null;
			if(tipo.equals("line")) {
				chart = ChartFactory.createLineChart(title, category, value, createBarDataset(lq), orientation ? PlotOrientation.VERTICAL : PlotOrientation.HORIZONTAL, legend, false, false);
			}
			if(tipo.equals("bar")) {
				chart = ChartFactory.createBarChart(title, category, value, createBarDataset(lq), orientation ? PlotOrientation.VERTICAL : PlotOrientation.HORIZONTAL, legend, false, false);
			}
			if(tipo.equals("area")) {
				chart = ChartFactory.createAreaChart(title, category, value, createBarDataset(lq), orientation ? PlotOrientation.VERTICAL : PlotOrientation.HORIZONTAL, legend, false, false);
			}
			if(tipo.equals("stackedbar")) {
				chart = ChartFactory.createStackedBarChart(title, category, value, createBarDataset(lq), orientation ? PlotOrientation.VERTICAL : PlotOrientation.HORIZONTAL, legend, false, false);
			}
			if(tipo.equals("stackedarea")) {
				chart = ChartFactory.createStackedAreaChart(title, category, value, createBarDataset(lq), orientation ? PlotOrientation.VERTICAL : PlotOrientation.HORIZONTAL, legend, false, false);
			}
			if(chart != null && ApplicationUtil.gtZero(max_width) && ApplicationUtil.gtZero(max_height)) {
				chart.setBackgroundPaint(Color.white);
				if(orientation) {
					CategoryPlot catPlot = (CategoryPlot) chart.getPlot();
					CategoryAxis catAxis = catPlot.getDomainAxis();  
					catAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_90);
				}
				return chart.createBufferedImage(max_width, max_height);
			}
		}
		return null;
	}

	public static BufferedImage createPie(String title, boolean legend, boolean orientation, Integer max_width, Integer max_height, List<LabelQuery> lq) {
		if(ApplicationUtil.notEmpty(lq)) {
			JFreeChart chart = ChartFactory.createMultiplePieChart(title, createBarDataset(lq), orientation ? TableOrder.BY_ROW : TableOrder.BY_COLUMN, legend, true, true);
			if(ApplicationUtil.gtZero(max_width) && ApplicationUtil.gtZero(max_height)) {
				chart.setBackgroundPaint(Color.white);
				return chart.createBufferedImage(max_width, max_height);
			}
		}
		return null;
	}

	private static CategoryDataset createBarDataset(List<LabelQuery> lq) {
		DefaultCategoryDataset dataset = new DefaultCategoryDataset( ); 
		if(ApplicationUtil.notEmpty(lq)) {
			for(LabelQuery l : lq) {
				dataset.setValue(l.getQuantity(), l.getLabel(), l.getValue());
			}
		}
		return dataset; 
	}



}