package it.callite.utility;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.Contratto; 
import it.callite.model.Prodotto;
@SuppressWarnings("serial")
public class ContrattoProdotto implements Serializable {
	private List<DatoSondaggioContratto> datoSondaggioContratto;
	private List<Allegato> allegato;
	private Prodotto prodotto;
	private Contratto contratto;
	private Date data_stipula;
	private Date data_contabile;
	private Date data_attivazione;
	private Date data_caricamento;
	private Integer quantita;
	private BigDecimal costo_scontato;
	private String seriale;
	private String nota;
	private Long agenteId;
	private Long operatoreId;
	private Long statoId;
	public Date getData_attivazione() {
		return data_attivazione;
	}
	public void setData_attivazione(Date data_attivazione) {
		this.data_attivazione = data_attivazione;
	}
	
	public Date getData_caricamento() {
		return data_caricamento;
	}
	public void setData_caricamento(Date data_caricamento) {
		this.data_caricamento = data_caricamento;
	}
	public Contratto getContratto() {
		return contratto;
	}
	public void setContratto(Contratto contratto) {
		this.contratto = contratto;
	}
	public Long getAgenteId() {
		return agenteId;
	}
	public List<Allegato> getAllegato() {
		if(allegato==null)
			allegato = new ArrayList<Allegato>();
		return allegato;
	}
	public int getAllegatoSize() {
		return ApplicationUtil.size(allegato);
	}
	public BigDecimal getCosto_scontato() {
		if(costo_scontato == null) {
			costo_scontato = getCosto_effettivo();
		} 
		return costo_scontato;
	}
	public BigDecimal getCosto_effettivo() {
		if(prodotto !=null && prodotto.getVendita()) {
			BigDecimal cb = prodotto.getCosto_base();
			BigDecimal cf = prodotto.getCosto_fisso();
			BigDecimal ta = prodotto.getTasse();
			if(cb == null) {
				cb = new BigDecimal(0);
			}
			if(cf == null) {
				cf = new BigDecimal(0);
			}
			if(ta == null) {
				ta = new BigDecimal(0);
			}
			ta = ta.add(new BigDecimal(100)).divide(new BigDecimal(100));
			return new BigDecimal(getQuantita()).multiply(cb).add(cf).multiply(ta);
		} 
		return new BigDecimal(0);
	}
	
	public Date getData_stipula() {
		return data_stipula;
	}
	public List<DatoSondaggioContratto> getDatoSondaggioContratto() {
		if(datoSondaggioContratto==null)
			datoSondaggioContratto = new ArrayList<DatoSondaggioContratto>();
		return datoSondaggioContratto;
	}
	public Long getOperatoreId() {
		return operatoreId;
	}
	public void setOperatoreId(Long operatoreId) {
		this.operatoreId = operatoreId;
	}
	public String getNota() {
		return nota;
	}
	public Prodotto getProdotto() {
		return prodotto;
	}
	public Integer getQuantita() {
		if(ApplicationUtil.ltZero(quantita))
			return 1;
		return quantita;
	}
	public String getSeriale() {
		return seriale;
	}
	public String getTipo_seriale() {
		if(prodotto!=null && ApplicationUtil.isNotBlank(prodotto.getSeriale())) {
			return ApplicationUtil.trimToEmpty(prodotto.getSeriale());
		} else {
			return ConstantsLanguage.SERIALE.stampa();
		}
	}
	public Long getStatoId() {
		return statoId;
	}
	public void setAgenteId(Long agenteId) {
		this.agenteId = agenteId;
	}
	public void setAllegato(List<Allegato> allegato) {
		this.allegato = allegato;
	}
	public void setCosto_scontato(BigDecimal costo_scontato) {
		this.costo_scontato = costo_scontato;
	}
	public void setData_stipula(Date data_stipula) {
		this.data_stipula = data_stipula;
	}
	public Date getData_contabile() {
		return data_contabile;
	}
	public void setData_contabile(Date data_contabile) {
		this.data_contabile = data_contabile;
	}
	public void setDatoSondaggioContratto(List<DatoSondaggioContratto> datoSondaggioContratto) {
		this.datoSondaggioContratto = datoSondaggioContratto;
	}
	public void setNota(String nota) {
		this.nota = nota;
	}
	public void setProdotto(Prodotto prodotto) {
		this.prodotto = prodotto;
	}
	public void setQuantita(Integer quantita) {
		this.quantita = quantita;
	}
	public void setSeriale(String seriale) {
		this.seriale = seriale;
	}
	public void setStatoId(Long statoId) {
		this.statoId = statoId;
	}
}