package it.callite.utility;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import javax.faces.model.SelectItem;

import it.callite.constants.ConstantsFormat;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.model.CampoAggiuntivoAnagrafica;
public class DatoSondaggioAnagrafica extends DatoSondaggio implements Serializable, Comparable<DatoSondaggioAnagrafica> {

	private static final long serialVersionUID = 1L;
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((campo == null) ? 0 : campo.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DatoSondaggioAnagrafica other = (DatoSondaggioAnagrafica) obj;
		if (campo == null) {
			if (other.campo != null)
				return false;
		} else if (!campo.equals(other.campo))
			return false;
		return true;
	}
	private CampoAggiuntivoAnagrafica campo;
	private String valore;
	private Date valore_data;
	public Date getValore_time() {
		return valore_data;
	}
	public void setValore_time(Date valore_data) {
		this.valore_data = valore_data;
	}
	public String getValore_formattato() {
		String v = null;
		if(campo.getSlider() || campo.getSpinner() || campo.getIntero()) {
			v = ApplicationUtil.formatNumber(getValore());
		} else if( campo.getNumero()) {
			v = ApplicationUtil.formatNumber(ApplicationUtil.parseBigDecimal(getValore()));
		} else if(campo.getData()) {
			v = ApplicationUtil.formatDate(campo.getValoriT(), getValore_data());
		} else if(campo.getDataora()) {
			v = ApplicationUtil.formatDate(campo.getValoriT(), getValore_datatime());
		} else if(campo.getOra()) {
			v = ApplicationUtil.formatDate(ConstantsFormat.time.stampa(), getValore_time());
		} else if(campo.getFile()) {
			v = ApplicationUtil.filename(getValore());
		} else {
			v = getValore();
		}
		return ApplicationUtil.trimToEmpty(v);
	}
	public Date getValore_datatime() {
		return valore_data;
	}
	public void setValore_datatime(Date valore_data) {
		this.valore_data = valore_data;
	}
	public Date getValore_data() {
		return valore_data;
	}
	public void setValore_data(Date valore_data) {
		this.valore_data = valore_data;
	}
	private Boolean required;
	private Boolean visibile;
	private List<String> valore_multiplo;
	public DatoSondaggioAnagrafica() {
		this.valore = null;
		this.valore_data = null;
		this.valore_multiplo = new ArrayList<String>();
		this.required = Boolean.FALSE;
		this.visibile = Boolean.TRUE;
	}
	public DatoSondaggioAnagrafica(CampoAggiuntivoAnagrafica campo, Boolean obbligatoria, Boolean visibile) {
		this.campo = campo;
		this.valore = null;
		this.valore_data = null;
		this.valore_multiplo = new ArrayList<String>();
		this.required = obbligatoria;
		this.visibile = visibile;
		setChiave(campo.getChiave());
	}
	public DatoSondaggioAnagrafica(CampoAggiuntivoAnagrafica campo, String valore, Boolean obbligatoria, Boolean visibile) {
		this.campo = campo;
		this.valore = ApplicationUtil.trimToEmpty(valore);
		this.valore_data = null;
		this.valore_multiplo = new ArrayList<String>();
		this.required = obbligatoria;
		this.visibile = visibile;
		setChiave(campo.getChiave());
	}
	public DatoSondaggioAnagrafica(CampoAggiuntivoAnagrafica campo, Date valore,  Boolean obbligatoria, Boolean visibile) {
		this.campo = campo;
		this.valore = null;
		this.valore_data = valore;
		this.valore_multiplo = new ArrayList<String>();
		this.required = obbligatoria;
		this.visibile = visibile;
		setChiave(campo.getChiave());
	}
	public DatoSondaggioAnagrafica(CampoAggiuntivoAnagrafica campo, List<String> valore,  Boolean obbligatoria, Boolean visibile) {
		this.campo = campo;
		this.valore = null;
		this.valore_data = null;
		this.valore_multiplo = valore;
		this.required = obbligatoria;
		this.visibile = visibile;
		setChiave(campo.getChiave());
	}
	public String getValore() {
		return ApplicationUtil.trimToEmpty(valore);
	}
	public void setValore(String valore) {
		this.valore = ApplicationUtil.trimToEmpty(valore);
	}
	public CampoAggiuntivoAnagrafica getCampo() {
		return campo;
	}
	public void setCampo(CampoAggiuntivoAnagrafica campo) {
		this.campo = campo;
	}
	public List<String> getValore_multiplo() {
		return valore_multiplo;
	}
	public void setValore_multiplo(List<String> valoreMultiplo) {
		valore_multiplo = valoreMultiplo;
	}
	public Boolean getRequired() {
		return required;
	}
	public void setRequired(Boolean required) {
		this.required = required;
	}
	public int compareTo(DatoSondaggioAnagrafica o) {
		if(campo != null && o.campo != null) {
			return campo.compareTo(o.campo);
		} else if(campo != null && o.campo == null) {
			return -1;
		} else if(campo == null && o.campo != null) {
			return 1;
		} else {
			return 0;
		}
	}
	public List<SelectItem> getValoriCampo() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		if(campo.getBinario() || campo.getOneMenu() || campo.getInplaceSelect() ){
			if(ApplicationUtil.isFalse(getRequired())) {
				list.add(new SelectItem("", ConstantsLanguage.TRATTINI.stampa()));
			}
		}
		if(campo.getCombo() || campo.getOneMenu() || campo.getInplaceSelect() || campo.getOneRadio() || campo.getPickList() || campo.getManyCheckbox() || campo.getManyMenu()) {
			Set<String> valori = new LinkedHashSet<String>(ApplicationUtil.tokenU(campo.getValori(), ConstantsGlobal.colonSeparator));
			if(campo.getCombo() || campo.getOneMenu() || campo.getInplaceSelect() || campo.getOneRadio()) {
				valori.addAll(ApplicationUtil.tokenU(getValore(), ConstantsGlobal.colonSeparator));
			}
			if(campo.getPickList() || campo.getManyCheckbox() || campo.getManyMenu()) {
				valori.addAll(getValore_multiplo());
			}
			for(String valore : valori) {
				list.add(new SelectItem(valore, valore));
			}
		} 
		if(campo.getBinario()) {
			list.add(new SelectItem(ConstantsLanguage.SI.stampa(), ConstantsLanguage.SI.stampa()));
			list.add(new SelectItem(ConstantsLanguage.NO.stampa(), ConstantsLanguage.NO.stampa()));
		}
		return list;
	}
	
	public boolean isLinkValid() {
		if(campo.getLink()) {
			String protocol = ApplicationUtil.protocol(getValore());
			return protocol != null;
		}
		return false;
	}
	public List<String> autocompleteInputValori(Object query) {
		String q = ApplicationUtil.upperCaseEmpty((String) query);
		List<String> list = new ArrayList<String>();
		if(campo.getSuggerimento() && ApplicationUtil.isNotBlank(q)) {
			Set<String> valori = new LinkedHashSet<String>();
			valori.addAll(ApplicationUtil.tokenU(campo.getValori(), ConstantsGlobal.colonSeparator));
			valori.addAll(ApplicationUtil.tokenU(getValore(), ConstantsGlobal.colonSeparator));
			for(String valore : valori) {
				if(ApplicationUtil.upperCaseEmpty(valore).contains(q) && !list.contains(valore)) {
					list.add(valore);
				}
			}
		} 
		return list;
	}
	
	@Override
	public String toString() {
		return getValore_formattato();
	}
	public Boolean getVisibile() {
		return visibile;
	}
	public void setVisibile(Boolean visibile) {
		this.visibile = visibile;
	}
	
}