package it.callite.utility;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsMap;
import it.callite.pec.CustomSSLSocketFactory;
import it.callite.utility.thread.Posta;
import jakarta.activation.DataHandler;
import jakarta.activation.DataSource;
import jakarta.activation.FileDataSource;
import jakarta.mail.Address;
import jakarta.mail.Authenticator;
import jakarta.mail.BodyPart;
import jakarta.mail.Message;
import jakarta.mail.Message.RecipientType;
import jakarta.mail.Multipart;
import jakarta.mail.Session;
import jakarta.mail.Transport;
import jakarta.mail.internet.InternetAddress;
import jakarta.mail.internet.MimeBodyPart;
import jakarta.mail.internet.MimeMessage;
import jakarta.mail.internet.MimeMultipart;
import jakarta.mail.util.ByteArrayDataSource;
import jodd.mail.EmailAddress;
import jodd.mail.EmailAttachment;


public class EmailUtils {
	
	
	public static boolean email(MailSupport ms, Posta posta, String append, String appname) throws Exception {
		return email(ms, posta.getHtml(), posta.getSender(), posta.getTo(), posta.getCc(), posta.getSubject(), posta.getText(), posta.getAttachments(), posta.getTemplate(), posta, append, appname);
	}

	public static boolean email(MailSupport ms, Boolean html, String reply, String to, String cc, String oggetto, String testo, List<Allegato> allegati, String template, Object type, String append, String appname) throws Exception {
		boolean inviata = false;
		if(ms != null) {
			String from = ApplicationUtil.trimToNull(ms.getMail_from());
			String mittente = ApplicationUtil.trimToNull(ms.getMail_mittente());
			InternetAddress [] res = getAddressArray(reply);
			InternetAddress [] tos = getAddressArray(to);
			InternetAddress [] ccs = getAddressArray(cc);
			if(from != null && ApplicationUtil.notEmpty(tos) && ms.getSession() != null) {
				Message message = new MimeMessage(ms.getSession());
				message.setSentDate(ApplicationUtil.date());
				message.setFrom(new InternetAddress(mittente != null ? (mittente  + " " + "<" + from + ">") : from));
				message.addRecipients(Message.RecipientType.TO, tos);
				if(ApplicationUtil.notEmpty(ccs)) {
					message.addRecipients(Message.RecipientType.CC, ccs);
				}
				if(ApplicationUtil.notEmpty(res)) {
					message.setReplyTo(res);
				}
				if(appname != null) {
					StringBuilder sb =  new StringBuilder();
					String icon = ApplicationUtil.icon(ConstantsGlobal.icon);
					if(icon != null) {
						sb.append(icon + " ");
					} 
					sb.append(appname + " - " + oggetto);
					message.setSubject(sb.toString());
				} else {
					message.setSubject(oggetto);
				}
				message.setHeader("X-CRM-ID", "(" + type.getClass().getSimpleName() + ")" + " " + ApplicationUtil.trimToEmpty(type.toString()));
				message.setHeader("X-CRM-IP", "(" + ApplicationUtil.ip() + ")" + " " + ApplicationUtil.hostname());
				message.setHeader("List-Unsubscribe", "<mailto:"+from+">");
				Multipart multipart = new MimeMultipart();
				BodyPart body = new MimeBodyPart();
				if(ApplicationUtil.isTrue(html)) {
					body.setContent(testo + append, ConstantsGlobal.contentHtml); 
				} else {
					body.setText(testo + append);
				}
				multipart.addBodyPart(body);
				if (ApplicationUtil.notEmpty(allegati)) {
					for(Allegato file : new LinkedHashSet<Allegato>(allegati)) {
						String name = file.getNome();
						if(ApplicationUtil.isNotBlank(name) && (file.isFileE() || file.isPercorsoE())) {
							MimeBodyPart attachment = new MimeBodyPart();
							DataSource ds = null;
							if(file.isFileE()) {
								ds = new ByteArrayDataSource(file.getFile(), ApplicationUtil.getContentType(name));
							}
							if(file.isPercorsoE()) {
								ds = new FileDataSource(new File(file.getPercorso()));
							}
							if(ds != null) {
								attachment.setDataHandler(new DataHandler(ds));
								if(ApplicationUtil.isNotBlank(testo) && testo.contains("cid:" + name) && ApplicationUtil.asList(ConstantsMap.a_immagini).contains(ApplicationUtil.getExtensionLower(name))) {
									attachment.setHeader("Content-ID", "<" + name +">");
								}
								attachment.setFileName(name);
								multipart.addBodyPart(attachment);
							}
						}
					}
				}
				if(template != null) {
					message.setHeader("Content-Transfer-Encoding:", "7bit");
					MimeBodyPart invite = new MimeBodyPart();
					invite.setHeader("Content-Class", "urn:content-classes:calendarmessage");
					invite.setHeader("Content-ID", "calendar_message");
					invite.setHeader("Content-Disposition", "inline");
					invite.setContent(template, "text/calendar");
					multipart.addBodyPart(invite);
				}
				message.setContent(multipart);
				send(ms, message);
				inviata = true;
			}

		}
		return inviata;
	}

	public static Session getSession(Boolean mail_pec, String mail_protocol, String mail_host, Integer mail_port, String mail_user, String mail_password, String mail_from, Boolean mail_auth, String mail_encryption, Boolean mail_debug) {

		if(mail_pec) {
			Properties sysProps = System.getProperties(); 
			sysProps.setProperty("mail.smtp.ssl.enable", "true"); 
			sysProps.setProperty("mail.smtp.ssl.socketFactory.class", CustomSSLSocketFactory.class.getName()); 
			sysProps.setProperty("mail.smtp.ssl.socketFactory.fallback", "false");
		}

		Properties props = new Properties();
		props.put("mail.debug", ApplicationUtil.isTrue(mail_debug));
		props.put("mail.host", mail_host);
		if(mail_from != null) {
			props.put("mail.from", mail_from);
		}
		if(mail_protocol != null) {
			props.put("mail.transport.protocol", mail_protocol);
		}
		if(!mail_encryption.equals(ConstantsMap.NONE)) {
			props.put("mail.smtp.starttls.enable", "true");
			props.put("mail.smtp.socketFactory.port", mail_port);
			props.put("mail.smtp.ssl.trust", mail_host);
			props.put("mail.smtp.ssl.protocols", "TLSv1.2");
			if(mail_encryption.equals(ConstantsMap.SSL)) { 
				props.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
				props.put("mail.smtp.socketFactory.fallback", "false");
			}
		}
		props.put("mail.smtp.host", mail_host);
		props.put("mail.smtp.user", mail_user);
		props.put("mail.smtp.password", mail_password);
		props.put("mail.smtp.port", mail_port);
		props.put("mail.smtp.auth", ApplicationUtil.isTrue(mail_auth));

		Authenticator auth = new SMTPAuthenticator(mail_user, mail_password);
		Session session = Session.getInstance(props, auth);
		session.setDebug(Boolean.valueOf(mail_debug));
		return session;
	}


	private static void send(MailSupport mailSupport, Message message) throws Exception {
		if(message != null) {
			boolean send = false;
			if(mailSupport.isInviabile()) {
				Transport transport = mailSupport.getTransport();
				Address address [] = message.getRecipients(RecipientType.TO);
				if(transport != null && transport.isConnected() && ApplicationUtil.notEmpty(address) ) {
					send = true;
					message.saveChanges();
					transport.sendMessage(message, address);
				} 
			} 
			if(!send) {
				Transport.send(message);
			}
		}
	}

	public static byte[] message(Message ib) {
		try {
			ByteArrayOutputStream fi = new ByteArrayOutputStream();
			ib.writeTo(fi);
			fi.flush();
			fi.close();
			return fi.toByteArray();
		} catch(Exception e) {
		}
		return null;
	}


	public static InternetAddress[] getAddressArray(String valori) throws Exception {
		//InternetAddress.parse(valori);
		List<InternetAddress> emails = new ArrayList<InternetAddress>();
		if(valori != null ) {
			StringTokenizer st = new StringTokenizer(valori, ConstantsGlobal.colonSeparator);
			while(st.hasMoreTokens()) {
				String valore = ApplicationUtil.trimToNull(st.nextToken());
				if(valore != null) {
					InternetAddress ia = new InternetAddress(valore);
					if(!emails.contains(ia)) {
						emails.add(ia);
					}
				}
			}
		}
		return emails.toArray(new InternetAddress[0]);
	}

	public static String email(EmailAddress a []) {
		List<String> l = new ArrayList<String>();
		if(ApplicationUtil.notEmpty(a)) {
			for(EmailAddress em : a) {
				l.add(em.getEmail());
			}		
		}
		return ApplicationUtil.detoken(l, ";");
	}

	public static String email(EmailAddress a) {
		if(a != null) {
			return a.getEmail();
		}
		return null;
	}

	public static byte[] attachment(EmailAttachment<? extends DataSource> attachment) {
		try {
			ByteArrayOutputStream fi = new ByteArrayOutputStream();
			attachment.writeToStream(fi);
			fi.flush();
			fi.close();
			return fi.toByteArray();
		} catch(Exception e) {
		}
		return null;
	}

}