package it.callite.utility;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Iterator;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Comment;
import org.apache.poi.ss.usermodel.RichTextString;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.model.Contratto;
import it.callite.model.Estratto;
import it.callite.model.FatturaContratto;
public class ExcelFatture {
	public static void stampaExcelFatturaEstratto(Estratto estratto, String progressivoU, OutputStream result, String password) throws Exception {
		String template = estratto.getUser().getFattura().getNomeFileS();
		String ext = ApplicationUtil.getExtensionLower(template);
		InputStream stream = new ByteArrayInputStream(estratto.getUser().getFattura().getContent());
		Workbook wb = ApplicationUtil.workbook(stream, ext);
		stream.close();
		ExcelStyle s = new ExcelStyle(wb, false);
		int ns = wb.getNumberOfSheets();
		for (int i = 0; i < ns; i++) {
			Sheet sheet = wb.getSheetAt(i);
			Iterator<Row> rowIt = sheet.rowIterator();
			while (rowIt.hasNext()) {
				Row row = rowIt.next();
				Iterator<Cell> cellIt = row.cellIterator();
				while (cellIt.hasNext()) {
					Cell cell = cellIt.next();
					if(cell!= null) {
						if(cell.getCellType().equals(CellType.FORMULA)) {
							//cell.setCellType(CellType.FORMULA);
							cell.setCellFormula(cell.getCellFormula());
						}
						Comment comment =  cell.getCellComment();
						if(comment != null) {
							RichTextString rt = comment.getString();
							String commento = ApplicationUtil.trimToEmpty(rt.toString());
							cell.removeCellComment();
							if(commento.equals(ConstantsMap.fattura_importo)) {
								ApplicationUtil.formatNumber(s, cell, estratto.getTotale());
							}
							if(commento.equals(ConstantsMap.fattura_denominazione)) {
								ApplicationUtil.formatText(s, cell, estratto.getUser().getDati_fiscali());
							}
							if(commento.equals(ConstantsMap.fattura_data)) {
								ApplicationUtil.formatDate(s, cell, estratto.getData_estratto());
							}
							if(commento.equals(ConstantsMap.fattura_progressivo)) {
								ApplicationUtil.formatString(s, cell, progressivoU);
							}
							if(commento.equals(ConstantsMap.fattura_descrizione)) {
								ApplicationUtil.formatString(s, cell, ConstantsLanguage.ESTRATTO.stampa() + ": " + estratto.getId() + " - " + ConstantsLanguage.DETTAGLI.stampa() +": " +  estratto.getNdettagli());
							}
						}
					}
				}
			}
			sheet.getForceFormulaRecalculation();
		}
		ApplicationUtil.passwordXLS(wb, result, password);
	}
	public static void stampaExcelFatturaContratto(Contratto contratto, FatturaContratto fatt,  String progressivoU, OutputStream result, String password) throws Exception {
		String template = fatt.getNomeFileS();
		String ext = ApplicationUtil.getExtensionLower(template);
		InputStream stream = new ByteArrayInputStream(fatt.getContent());
		Workbook wb = ApplicationUtil.workbook(stream, ext);
		stream.close();
		ExcelStyle s = new ExcelStyle(wb, false);
		int ns = wb.getNumberOfSheets();
		for (int i = 0; i < ns; i++) {
			Sheet sheet = wb.getSheetAt(i);
			Iterator<Row> rowIt = sheet.rowIterator();
			while (rowIt.hasNext()) {
				Row row = rowIt.next();
				Iterator<Cell> cellIt = row.cellIterator();
				while (cellIt.hasNext()) {
					Cell cell = cellIt.next();
					if(cell!= null) {
						if(cell.getCellType().equals(CellType.FORMULA)) {
							//cell.setCellType(CellType.FORMULA);
							cell.setCellFormula(cell.getCellFormula());
						}
						Comment comment =  cell.getCellComment();
						if(comment != null) {
							RichTextString rt = comment.getString();
							String commento = ApplicationUtil.trimToEmpty(rt.toString());
							cell.removeCellComment();
							if(commento.equals(ConstantsMap.fattura_importo)) {
								ApplicationUtil.formatNumber(s, cell, contratto.getCosto_scontato());
							}
							if(commento.equals(ConstantsMap.fattura_denominazione)) {
								ApplicationUtil.formatText(s, cell, contratto.getContatto().getAnagrafica().getDati_fiscali());
							}
							if(commento.equals(ConstantsMap.fattura_data)) {
								ApplicationUtil.formatDate(s, cell, contratto.getData_stipula());
							}
							if(commento.equals(ConstantsMap.fattura_progressivo)) {
								ApplicationUtil.formatString(s, cell, progressivoU);
							}
							if(commento.equals(ConstantsMap.fattura_nota)) {
								ApplicationUtil.formatString(s, cell, contratto.getNota());
							}
							if(commento.equals(ConstantsMap.fattura_quantita)) {
								ApplicationUtil.formatNumber(s, cell, contratto.getQuantita());
							}
							if(commento.equals(ConstantsMap.fattura_descrizione)) {
								ApplicationUtil.formatString(s, cell, contratto.getProdotto().getDenominazione());
							}
						}
					}
				}
			}
			sheet.getForceFormulaRecalculation();
		}
		ApplicationUtil.passwordXLS(wb, result, password);
	}
}