package it.callite.utility;

import java.math.BigDecimal;
import java.net.URI;
import java.time.temporal.Temporal;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import ezvcard.Ezvcard;
import ezvcard.VCard;
import ezvcard.VCardVersion;
import ezvcard.parameter.AddressType;
import ezvcard.parameter.EmailType;
import ezvcard.parameter.TelephoneType;
import ezvcard.property.Address;
import ezvcard.property.Birthday;
import ezvcard.property.Birthplace;
import ezvcard.property.Categories;
import ezvcard.property.Email;
import ezvcard.property.FormattedName;
import ezvcard.property.Gender;
import ezvcard.property.Note;
import ezvcard.property.Organization;
import ezvcard.property.ProductId;
import ezvcard.property.StructuredName;
import ezvcard.property.Telephone;
import ezvcard.property.Url;
import ezvcard.util.GeoUri;
import it.callite.constants.ConstantsGlobal;
import it.callite.model.Agente;
import it.callite.model.Anagrafica;
import it.callite.model.Appuntamento;
import it.callite.model.Contatto;
import it.callite.model.Indirizzo;
import it.callite.model.Nota;
import it.callite.model.Utente;
import net.fortuna.ical4j.model.Calendar;
import net.fortuna.ical4j.model.Recur;
import net.fortuna.ical4j.model.component.VEvent;
import net.fortuna.ical4j.model.parameter.Cn;
import net.fortuna.ical4j.model.parameter.CuType;
import net.fortuna.ical4j.model.parameter.PartStat;
import net.fortuna.ical4j.model.parameter.Role;
import net.fortuna.ical4j.model.parameter.Rsvp;
import net.fortuna.ical4j.model.property.Attendee;
import net.fortuna.ical4j.model.property.CalScale;
import net.fortuna.ical4j.model.property.Clazz;
import net.fortuna.ical4j.model.property.Created;
import net.fortuna.ical4j.model.property.Description;
import net.fortuna.ical4j.model.property.Geo;
import net.fortuna.ical4j.model.property.LastModified;
import net.fortuna.ical4j.model.property.Location;
import net.fortuna.ical4j.model.property.Method;
import net.fortuna.ical4j.model.property.Organizer;
import net.fortuna.ical4j.model.property.Priority;
import net.fortuna.ical4j.model.property.ProdId;
import net.fortuna.ical4j.model.property.RRule;
import net.fortuna.ical4j.model.property.Sequence;
import net.fortuna.ical4j.model.property.Status;
import net.fortuna.ical4j.model.property.Summary;
import net.fortuna.ical4j.model.property.Transp;
import net.fortuna.ical4j.model.property.Uid;
import net.fortuna.ical4j.model.property.Version;
import net.fortuna.ical4j.model.property.XProperty;
import net.fortuna.ical4j.transform.recurrence.Frequency;

public class VCardUtils  {


	public static String vcf(Anagrafica anagrafica, String appname) {

		VCard vcard = new VCard(VCardVersion.V4_0);
		vcard.addExtendedProperty("CHARSET", ConstantsGlobal.ENCODING);

		StructuredName n = new StructuredName();
		n.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		n.setFamily(anagrafica.getCognome());
		n.setGiven(anagrafica.getNome());
		vcard.setStructuredName(n);

		FormattedName fn = new FormattedName(anagrafica.getDenominazione());
		fn.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		vcard.setFormattedName(fn);

		if(ApplicationUtil.isNotBlank(appname)) {
			ProductId prodid = new ProductId(appname);
			prodid.setParameter("CHARSET", ConstantsGlobal.ENCODING);
			vcard.setProductId(prodid);
		}

		Organization org = new Organization();
		org.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		org.getValues().add(anagrafica.getRagione_sociale());
		vcard.setOrganization(org);

		Indirizzo legale = anagrafica.getIndirizzo_legale();
		Address adrL = new Address();
		adrL.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		adrL.setPref(1);
		adrL.setPoBox("POSTAL");
		adrL.setCountry("");
		adrL.setStreetAddress(legale.getStrada_completo());
		adrL.setLocality(ApplicationUtil.trimToEmpty(legale.getComune()));
		adrL.setRegion(ApplicationUtil.trimToEmpty(legale.getProvincia()));
		adrL.setPostalCode(ApplicationUtil.trimToEmpty(legale.getCap()));
		adrL.getTypes().add(AddressType.HOME);
		if(legale.getLatitudine() != null && legale.getLongitudine() != null) {
			adrL.setGeo(GeoUri.parse("geo:"+legale.getLatitudine()+ConstantsGlobal.commaSeparator+legale.getLongitudine()));
		}
		vcard.addAddress(adrL);

		Indirizzo operativo = anagrafica.getIndirizzo_operativo();
		Address adrO = new Address();
		adrO.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		adrO.setPref(2);
		adrO.setCountry("");
		adrO.setStreetAddress(operativo.getStrada_completo());
		adrO.setLocality(ApplicationUtil.trimToEmpty(operativo.getComune()));
		adrO.setRegion(ApplicationUtil.trimToEmpty(operativo.getProvincia()));
		adrO.setPostalCode(ApplicationUtil.trimToEmpty(operativo.getCap()));
		adrO.getTypes().add(AddressType.WORK);
		if(operativo.getLatitudine() != null && operativo.getLongitudine() != null) {
			adrO.setGeo(GeoUri.parse("geo:"+operativo.getLatitudine()+ConstantsGlobal.commaSeparator+operativo.getLongitudine()));
		}
		vcard.addAddress(adrO);

		Telephone t1 = new Telephone(anagrafica.getTelefono_fisso());
		t1.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		t1.setPref(1); 
		t1.getTypes().add(TelephoneType.HOME);
		t1.getTypes().add(TelephoneType.VOICE);
		vcard.addTelephoneNumber(t1);

		Telephone t2 = new Telephone(anagrafica.getTelefono_mobile());
		t2.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		t2.setPref(2); 
		t2.getTypes().add(TelephoneType.CELL);
		t2.getTypes().add(TelephoneType.VOICE);
		vcard.addTelephoneNumber(t2);

		Telephone t3 = new Telephone(anagrafica.getTelefono_fax());
		t3.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		t3.setPref(3); 
		t3.getTypes().add(TelephoneType.WORK);
		t3.getTypes().add(TelephoneType.FAX);
		vcard.addTelephoneNumber(t3);

		Telephone t4 = new Telephone(anagrafica.getTelefono_altro());
		t4.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		t4.setPref(4); 
		t4.getTypes().add(TelephoneType.WORK);
		t4.getTypes().add(TelephoneType.VOICE);
		vcard.addTelephoneNumber(t4);

		Email e = new Email(anagrafica.getEmail());
		e.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		e.getTypes().add(EmailType.HOME);
		e.getTypes().add(EmailType.INTERNET);
		vcard.addEmail(e);

		Note note = new Note(ApplicationUtil.trimToEmpty(anagrafica.getNota()));
		note.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		vcard.addNote(note);

		Categories categories = new Categories();
		categories.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		categories.getValues().add(anagrafica.getSettore());
		vcard.addCategories(categories);

		Url url = new Url(anagrafica.getLink());
		url.setParameter("CHARSET", ConstantsGlobal.ENCODING);
		url.setType("pref");
		vcard.addUrl(url);

		if(anagrafica.getDate_nascita() != null) {
			Birthday birthday = new Birthday(anagrafica.getDate_nascita().toInstant());
			birthday.setParameter("CHARSET", ConstantsGlobal.ENCODING);
			vcard.setBirthday(birthday);
		}

		if(ApplicationUtil.isNotBlank(anagrafica.getLuogo_nascita())) {
			Birthplace birthplace = new Birthplace(anagrafica.getLuogo_nascita());
			birthplace.setParameter("CHARSET", ConstantsGlobal.ENCODING);
			vcard.setBirthplace(birthplace);
		}

		if(anagrafica.getMaschio() != null) {
			if(anagrafica.getMaschio()) {
				vcard.setGender(Gender.male());
			} else {
				vcard.setGender(Gender.female());
			}
		} 

		return Ezvcard.write(vcard).prodId(false).includeTrailingSemicolons(false).go();
	}


	public static String ical(List<Appuntamento> appuntamenti, List<Nota> note, List<Contatto> contatti, String appname, String link, boolean publish) {

		Calendar calendar = new Calendar();
		calendar.add(new ProdId("-//"+ ConstantsGlobal.base+"//iCal " + appname));
		calendar.add(new Version(Version.VALUE_2_0,Version.VALUE_2_0));
		calendar.add(new CalScale(CalScale.VALUE_GREGORIAN));
		calendar.add(new Method(publish ? Method.VALUE_PUBLISH : Method.VALUE_REFRESH));
		calendar.add(new XProperty("X-WR-CALNAME", appname));
		calendar.add(new XProperty("X-WR-TIMEZONE", ConstantsGlobal.UTC));
		calendar.add(new XProperty("X-PUBLISHED-TTL", "PT12H"));
		int i = 0;
		if(ApplicationUtil.notEmpty(appuntamenti)) {
			Iterator<Appuntamento> appIt = appuntamenti.iterator();
			while (appIt.hasNext()) {
				Appuntamento appuntamento = (Appuntamento) appIt.next();
				Anagrafica anagrafica = appuntamento.getContatto().getAnagrafica();
				Agente agente = appuntamento.getAgente();
				Indirizzo legale = anagrafica.getIndirizzo_legale();
				Indirizzo operativo = anagrafica.getIndirizzo_operativo();
				Long id = appuntamento.getId();
				Date data_inizio = appuntamento.getData_inizio();
				Date data_fine = appuntamento.getData_fine() != null ? appuntamento.getData_fine() : appuntamento.getData_inizio();
				Date data_lavorazione = appuntamento.getData_lavorazione() != null ? appuntamento.getData_lavorazione() : appuntamento.getData_creazione();
				Date data_creazione = appuntamento.getData_creazione();
				VEvent event = new VEvent(data_inizio.toInstant(), data_fine.toInstant(), anagrafica.getDenominazione() + " ("+appuntamento.getEsitoEtichetta()+") " + appuntamento.getAgente_denominazione());
				event.add(new Sequence(i++));
				event.add(new Uid(id.toString() + "@" + link));
				event.add(new Priority(1));
				event.add(new LastModified(data_lavorazione.toInstant()));
				event.add(new Created(data_creazione.toInstant()));
				event.add(new Transp(Transp.VALUE_OPAQUE));
				event.add(new Clazz(Clazz.VALUE_PUBLIC));
				event.add(new XProperty("X-MICROSOFT-CDO-OWNERAPPTID", id.toString()));
				event.add(new XProperty("X-MICROSOFT-CDO-BUSYSTATUS", "BUSY"));
				event.add(new XProperty("X-GOOGLE-CONFERENCE", ""));
				event.add(new Status(Status.VALUE_CONFIRMED));
				//	VAlarm alarm = new VAlarm(data_inizio.toInstant());
				//	alarm.getProperties().add(Action.DISPLAY);
				//	  event.getAlarms().add(alarm);
				Attendee attendee = new Attendee();
				if(ApplicationUtil.isNotBlank(anagrafica.getEmail())) {
					attendee.setCalAddress(URI.create("mailto:" + anagrafica.getEmail()));
				}
				attendee.add(CuType.INDIVIDUAL);
				attendee.add(Role.REQ_PARTICIPANT);
				attendee.add(PartStat.NEEDS_ACTION);
				attendee.add(Rsvp.TRUE);
				attendee.add(new Cn(anagrafica.getDenominazione()));
				event.add(attendee);
				Organizer organizer = new Organizer();
				if(agente != null && ApplicationUtil.isNotBlank(agente.getEmail())) {
					organizer.setCalAddress(URI.create("mailto:" + agente.getEmail()));
				}
				organizer.add(new Cn(appuntamento.getAgente_denominazione()));
				event.add(organizer);
				event.add(new Summary(ApplicationUtil.trimToEmpty(appuntamento.getNote())));
				event.add(new Description(ApplicationUtil.trimToEmpty(appuntamento.getNote())));
				if(ApplicationUtil.isTrue(appuntamento.getLocalizzazione())) {
					event.add(new Location(operativo.getIndirizzo_completo()));
					if(operativo.getLatitudine() != null && operativo.getLongitudine() != null) {
						event.add(new Geo(new BigDecimal(operativo.getLatitudine()), new BigDecimal(operativo.getLatitudine())));
					}
				} else {
					event.add(new Location(legale.getIndirizzo_completo()));
					if(legale.getLatitudine() != null && legale.getLongitudine() != null) {
						event.add(new Geo(new BigDecimal(legale.getLatitudine()), new BigDecimal(legale.getLatitudine())));
					}
				}
				calendar.add(event);
			}
		}

		if(ApplicationUtil.notEmpty(note)) {
			Iterator<Nota> appIt = note.iterator();
			while (appIt.hasNext()) {
				Nota nota = (Nota) appIt.next();
				Agente agente = nota.getAgente();
				Long id = nota.getId();
				Date data_inizio = nota.getData_inizio();
				Date data_fine = nota.getData_fine() != null ? nota.getData_fine() : nota.getData_inizio();
				Date data_lavorazione = nota.getData_lavorazione() != null ? nota.getData_lavorazione() : nota.getData_creazione();
				Date data_creazione = nota.getData_creazione();
				Date data_termine = nota.getData_termine();
				VEvent event = new VEvent(data_inizio.toInstant(), data_fine.toInstant(), agente.getDenominazione());
				event.add(new Sequence(i++));
				event.add(new Uid(id.toString() + "@" + link));
				event.add(new Priority(0));
				event.add(new LastModified(data_lavorazione.toInstant()));
				event.add(new Created(data_creazione.toInstant()));
				event.add(new Transp(Transp.VALUE_OPAQUE));
				event.add(new Clazz(Clazz.VALUE_PUBLIC));
				event.add(new XProperty("X-MICROSOFT-CDO-OWNERAPPTID", id.toString()));
				event.add(new XProperty("X-MICROSOFT-CDO-BUSYSTATUS", "BUSY"));
				event.add(new XProperty("X-GOOGLE-CONFERENCE", ""));
				event.add(new Status(Status.VALUE_CONFIRMED));
				//	VAlarm alarm = new VAlarm(data_inizio.toInstant());
				//	alarm.getProperties().add(Action.DISPLAY);
				//	event.getAlarms().add(alarm);
				if(ApplicationUtil.eq(nota.getRipetizioni(), 1)) {
					if(data_termine != null) {
						Recur<Temporal> recur = new Recur<Temporal>(Frequency.DAILY, data_termine.toInstant());
						RRule<Temporal> rrule = new RRule<Temporal>(recur);
						event.add(rrule);
					} else {
						Recur<Temporal> recur = new Recur<Temporal>(Frequency.DAILY, -1);
						RRule<Temporal> rrule = new RRule<Temporal>(recur);
						event.add(rrule);
					}
				}
				if(ApplicationUtil.eq(nota.getRipetizioni(), 2)) {
					if(data_termine != null) {
						Recur<Temporal> recur = new Recur<Temporal>(Frequency.WEEKLY, data_termine.toInstant());
						RRule<Temporal> rrule = new RRule<Temporal>(recur);
						event.add(rrule);
					} else {
						Recur<Temporal> recur = new Recur<Temporal>(Frequency.WEEKLY, -1);
						RRule<Temporal> rrule = new RRule<Temporal>(recur);
						event.add(rrule);
					}
				}
				if(ApplicationUtil.eq(nota.getRipetizioni(), 3)) {
					if(data_termine != null) {
						Recur<Temporal> recur = new Recur<Temporal>(Frequency.MONTHLY, data_termine.toInstant());
						RRule<Temporal> rrule = new RRule<Temporal>(recur);
						event.add(rrule);
					} else {
						Recur<Temporal> recur = new Recur<Temporal>(Frequency.MONTHLY, -1);
						RRule<Temporal> rrule = new RRule<Temporal>(recur);
						event.add(rrule);
					}
				}
				if(ApplicationUtil.eq(nota.getRipetizioni(), 4)) {
					if(data_termine != null) {
						Recur<Temporal> recur = new Recur<Temporal>(Frequency.YEARLY, data_termine.toInstant());
						RRule<Temporal> rrule = new RRule<Temporal>(recur);
						event.add(rrule);
					} else {
						Recur<Temporal> recur = new Recur<Temporal>(Frequency.YEARLY, -1);
						RRule<Temporal> rrule = new RRule<Temporal>(recur);
						event.add(rrule);
					}
				}
				Attendee attendee = new Attendee();
				if(ApplicationUtil.isNotBlank(agente.getEmail())) {
					attendee.setCalAddress(URI.create("mailto:" + agente.getEmail()));
				}
				attendee.add(CuType.INDIVIDUAL);
				attendee.add(Role.REQ_PARTICIPANT);
				attendee.add(PartStat.NEEDS_ACTION);
				attendee.add(Rsvp.TRUE);
				attendee.add(new Cn(agente.getDenominazione()));
				event.add(attendee);
				Organizer organizer = new Organizer();
				if(ApplicationUtil.isNotBlank(agente.getEmail())) {
					organizer.setCalAddress(URI.create("mailto:" + agente.getEmail()));
				}
				organizer.add(new Cn(agente.getDenominazione()));
				event.add(organizer);
				event.add(new Summary(ApplicationUtil.trimToEmpty(nota.getNote())));
				event.add(new Description(ApplicationUtil.trimToEmpty(nota.getNote())));
				calendar.add(event);
			}
		}
		
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<Contatto> appIt = contatti.iterator();
			while (appIt.hasNext()) {
				Contatto contatto = (Contatto) appIt.next();
				Anagrafica anagrafica = contatto.getAnagrafica();
				Utente utente = contatto.getProprietario();
				Long id = contatto.getId();
				Date data_inizio = contatto.getData_presentazione();
				Date data_fine = contatto.getData_presentazione();
				Date data_lavorazione = contatto.getData_lavorazione() != null ? contatto.getData_lavorazione() : contatto.getData_creazione();
				Date data_creazione = contatto.getData_creazione();
				VEvent event = new VEvent(data_inizio.toInstant(), data_fine.toInstant(), anagrafica.getDenominazione() + " ("+contatto.getEsito_generale()+")");
				event.add(new Sequence(i++));
				event.add(new Uid(ConstantsGlobal.ingressoSMB + id.toString()));
				event.add(new Priority(1));
				event.add(new LastModified(data_lavorazione.toInstant()));
				event.add(new Created(data_creazione.toInstant()));
				event.add(new Transp(Transp.VALUE_OPAQUE));
				event.add(new Clazz(Clazz.VALUE_PUBLIC));
				event.add(new XProperty("X-MICROSOFT-CDO-OWNERAPPTID", id.toString()));
				event.add(new XProperty("X-MICROSOFT-CDO-BUSYSTATUS", "BUSY"));
				event.add(new XProperty("X-GOOGLE-CONFERENCE", ""));
				event.add(new Status(Status.VALUE_CONFIRMED));
				// VAlarm alarm = new VAlarm(data_inizio.toInstant());
				// alarm.getProperties().add(Action.DISPLAY);
				// event.getAlarms().add(alarm);
				Attendee attendee = new Attendee();
				if(ApplicationUtil.isNotBlank(anagrafica.getEmail())) {
					attendee.setCalAddress(URI.create("mailto:" + anagrafica.getEmail()));
				}
				attendee.add(CuType.INDIVIDUAL);
				attendee.add(Role.REQ_PARTICIPANT);
				attendee.add(PartStat.NEEDS_ACTION);
				attendee.add(Rsvp.TRUE);
				attendee.add(new Cn(anagrafica.getDenominazione()));
				event.add(attendee);
				Organizer organizer = new Organizer();
				if(utente != null && ApplicationUtil.isNotBlank(utente.getEmail())) {
					organizer.setCalAddress(URI.create("mailto:" + utente.getEmail()));
				}
				organizer.add(new Cn(contatto.getProprietario_denominazione()));
				event.add(organizer);
				event.add(new Summary(ApplicationUtil.trimToEmpty(contatto.getNota())));
				event.add(new Description(ApplicationUtil.trimToEmpty(contatto.getNota())));
				calendar.add(event);
			}
		}
		return calendar.toString();
	}
	
	
}
