package it.callite.utility;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;

import com.google.common.io.ByteStreams;

import it.callite.constants.ConstantsLanguage;
import net.lingala.zip4j.io.outputstream.ZipOutputStream;
import net.lingala.zip4j.model.ZipParameters;
import net.lingala.zip4j.model.enums.EncryptionMethod;
public class ZipPassword {
	public static void zip(String comment, byte[]  b, OutputStream dest, String ext, String password) throws Exception {
		ZipParameters zipParam = new ZipParameters();
		zipParam.setEncryptFiles(true);
		zipParam.setEncryptionMethod(EncryptionMethod.ZIP_STANDARD);
		ZipOutputStream zos = new ZipOutputStream(dest, password.toCharArray());
		if(comment != null) {
			zos.setComment(comment);
		}
		zipParam.setFileNameInZip(ConstantsLanguage.ZIPFILE.stampa() + "." + ext);
		zos.putNextEntry(zipParam);
		zos.write(b);
		zos.closeEntry();
		zos.flush();
		zos.close();
	}

	public static boolean zip(String comment, List<Allegato> files, OutputStream dest, String password) {
		try {
			ZipParameters zipParam = new ZipParameters();
			zipParam.setEncryptFiles(true);
			zipParam.setEncryptionMethod(EncryptionMethod.ZIP_STANDARD);
			ZipOutputStream zout = new ZipOutputStream(dest, password.toCharArray());
			int i = 0;
			for(Allegato file : new LinkedHashSet<Allegato>(files)) {
				if(ApplicationUtil.isNotBlank(file.getNome()) && (file.isFileE() || file.isPercorsoE())) {
					i++;
					zipParam.setFileNameInZip(file.getNome());
					if(file.isFileE()) {
						InputStream fi = new ByteArrayInputStream(file.getFile());
						zout.putNextEntry(zipParam);
						zout.write(ByteStreams.toByteArray(fi));
						fi.close();
					}
					if(file.isPercorsoE()) {
						File f = new File(file.getPercorso());
						InputStream fi = new FileInputStream(f);
						zout.putNextEntry(zipParam);
						zout.write(ByteStreams.toByteArray(fi));
						fi.close();
					}
					zout.closeEntry();
				}
			}
			if(comment != null) {
				zout.setComment(comment);
			} else {
				zout.setComment(ConstantsLanguage.ZIPTEXT.stampa() + ": " + i);
			}
			zout.flush();
			zout.close();
		} catch(Exception e) { 
			return false;
		}
		return true;
	}

	public static void zipIt(String comment, String sourceFolder, OutputStream dest, String password) throws Exception {
		List<File> fileList = new ArrayList<File>();
		ApplicationUtil.generateFileList(fileList, new File(sourceFolder));
		ZipParameters zipParam = new ZipParameters();
		zipParam.setEncryptFiles(true);
		zipParam.setEncryptionMethod(EncryptionMethod.ZIP_STANDARD);
		ZipOutputStream zout = new ZipOutputStream(dest, password.toCharArray());
		if(comment != null) {
			zout.setComment(comment);
		}
		for (int i = 0; i < fileList.size(); i++) {
			File file = (File) fileList.get(i);
			String ff = ApplicationUtil.removeStart(file.getAbsolutePath(), sourceFolder + File.separator);
			zipParam.setFileNameInZip(ff);
			zout.putNextEntry(zipParam);
			if (file.isDirectory()) {
				zout.closeEntry();
				continue;
			}
			InputStream inputStream = new FileInputStream(file);
			zout.write(ByteStreams.toByteArray(inputStream));
			zout.closeEntry();
			inputStream.close();
		}
		zout.flush();
		zout.close();

	}
}
