package it.callite.utility.item;
import java.io.Serializable;
import java.sql.Time;
import java.util.Date;

import it.callite.constants.ConstantsGlobal;
import it.callite.model.Utente;
import it.callite.model.generic.SelezionatoAbstract;
import it.callite.utility.ApplicationUtil;
public class UtenteSI extends SelezionatoAbstract implements Serializable {

	private static final long serialVersionUID = 1L;
	
	private String cognome;
	private Boolean enabled;
	private Long id;
	private String nome;
	private String username;
	
	
	@Override
	public String toString() {
		return getDenominazione();
	}

	public UtenteSI() {
	}

	public UtenteSI(Utente utente) {
		this.id = utente.getId();
		this.username = utente.getUsername();
		this.nome = utente.getNome();
		this.cognome = utente.getCognome();
		this.enabled = utente.isEnabled();
		this.week_fine = utente.getWeek_fine();
		this.week_inizio = utente.getWeek_inizio();
		this.data_fine = utente.getData_fine();
		this.data_inizio = utente.getData_inizio();
		this.ora_fine = utente.getTime_fine();
		this.ora_inizio = utente.getTime_inizio();
		this.inverti = utente.getInverti();
	}
	
	public String getCognome() {
		return cognome;
	}
	public String getDenominazione() { 
		return ConstantsGlobal.usernameSeparator1 + ApplicationUtil.trimToEmpty(this.username) + ConstantsGlobal.usernameSeparator2 +(getStatoAttiva() ? "" : "*") + " "+  ApplicationUtil.denominazione(this.cognome, this.nome);
	}
	public Boolean getStatoAttiva() {
		return ApplicationUtil.calcolaIntervallo(getInverti(), getData_inizio(), getData_fine(), getWeek_inizio(), getWeek_fine(), getOra_inizio(), getOra_fine());
	}
	private Long week_fine;
	private Long week_inizio;
	
	public Long getWeek_fine() {
		return week_fine;
	}
	public void setWeek_fine(Long week_fine) {
		this.week_fine = week_fine;
	}
	public Long getWeek_inizio() {
		return week_inizio;
	}
	public void setWeek_inizio(Long week_inizio) {
		this.week_inizio = week_inizio;
	}
	private Date data_fine;
	private Date data_inizio;
	private Time ora_fine;
	private Time ora_inizio;
	private Boolean inverti = false;
	public Date getData_fine() {
		return data_fine;
	}
	public void setData_fine(Date data_fine) {
		this.data_fine = data_fine;
	}
	public Date getData_inizio() {
		return data_inizio;
	}
	public void setData_inizio(Date data_inizio) {
		this.data_inizio = data_inizio;
	}
	public Time getOra_fine() {
		return ora_fine;
	}
	public void setOra_fine(Time ora_fine) {
		this.ora_fine = ora_fine;
	}
	public Time getOra_inizio() {
		return ora_inizio;
	}
	public void setOra_inizio(Time ora_inizio) {
		this.ora_inizio = ora_inizio;
	}
	public void setInverti(Boolean inverti) {
		this.inverti = inverti;
	}
	public Boolean getInverti() {
		if(inverti == null) {
			inverti = false;
		}
		return inverti;
	}
	public Long getId() {
		return id;
	}
	public String getNome() {
		return nome;
	}
	public String getUsername() {
		return username;
	}
	public void setCognome(String cognome) {
		this.cognome = cognome;
	}
	public Boolean getEnabled() {
		return enabled;
	}
	public void setEnabled(Boolean enabled) {
		this.enabled = enabled;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public void setNome(String nome) {
		this.nome = nome;
	}
	public void setUsername(String username) {
		this.username = username;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((enabled == null) ? 0 : enabled.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((username == null) ? 0 : username.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		UtenteSI other = (UtenteSI) obj;
		if (enabled == null) {
			if (other.enabled != null)
				return false;
		} else if (!enabled.equals(other.enabled))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (username == null) {
			if (other.username != null)
				return false;
		} else if (!username.equals(other.username))
			return false;
		return true;
	}
	
}