package it.callite.utility.thread;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.dao.DownloadDao;
import it.callite.dao.MessaggioDao;
import it.callite.dao.UserDao;
import it.callite.model.Agente;
import it.callite.model.Appuntamento;
import it.callite.model.EsitoAppuntamento;
import it.callite.model.Utente;
import it.callite.model.messaggio.Destinatario;
import it.callite.model.messaggio.Messaggio;
import it.callite.utility.ApplicationUtil;
public class AppuntamentoMessaggioThread extends Thread {
	private Appuntamento appuntamento;
	private UserDao userDao;
	private MessaggioDao  messaggioDao;
	private DownloadDao downloadDao;
	private Utente user;
	public AppuntamentoMessaggioThread(Appuntamento appuntamento, MessaggioDao messaggioDao, DownloadDao downloadDao, UserDao userDao, Utente user) {
		this.appuntamento = appuntamento;
		this.userDao = userDao;
		this.messaggioDao = messaggioDao;
		this.downloadDao = downloadDao;
		this.user = user;
	}
	public void run() {
		Set<Destinatario> destinatari = new HashSet<Destinatario>();
		EsitoAppuntamento esito = appuntamento.getEsito();
		String emailDestinatario = ApplicationUtil.trimToEmpty(esito.getMessaggio_notifica());
		if (ApplicationUtil.isNotBlank(emailDestinatario)) {
			List<String> uns = ApplicationUtil.tokenU(emailDestinatario, ConstantsGlobal.colonSeparator);
			if(ApplicationUtil.notEmpty(uns)) {
				Iterator<String> uIt = uns.iterator();
				while (uIt.hasNext()) {
					String us = ApplicationUtil.subStringUtente((String) uIt.next());
					if(us != null) {
						Utente utente = userDao.getUserByUsername(us);
						if(utente != null && !utente.equals(user)) {
							destinatari.add(new Destinatario(utente));
						}
					}
				}
			}
		}
		if(ApplicationUtil.isTrue(esito.getMessaggio_operatore())) {
			Utente operatore = appuntamento.getOperatore();
			if(operatore != null && !operatore.equals(user)) {
				destinatari.add(new Destinatario(operatore));
			}
		}
		if(ApplicationUtil.isTrue(esito.getMessaggio_agente())) {
			Agente agente = appuntamento.getAgente();
			if(agente != null) {
				List<Utente> utenti = userDao.getUtentiByAgenteId(agente.getId());
				if(ApplicationUtil.notEmpty(utenti)) {
					Iterator<Utente> uIt = utenti.iterator();
					while (uIt.hasNext()) {
						Utente utente = uIt.next();
						if(utente != null && !utente.equals(user)) {
							destinatari.add(new Destinatario(utente));
						}
					}
				}
			}
		}
		if(ApplicationUtil.notEmpty(destinatari)) {
			Messaggio messaggio = messaggioDao.messaggioInInvio(user);
			String testo = downloadDao.notificaAppuntamentoToHTMLString(appuntamento, user);
			String oggetto = ConstantsLanguage.MODIFICA.stampa() +  " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": "+appuntamento.getId();
			messaggio.setTesto(testo);
			messaggio.setOggetto(oggetto);
			messaggio.setDestinatari(new ArrayList<Destinatario>(destinatari));
			//messaggio.setAllegati(allegati);
			messaggioDao.salvaMessaggioNuovoInvia(messaggio);
		}
	}
}