package it.callite.utility.thread;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.dao.DownloadDao;
import it.callite.dao.NotiziaDao;
import it.callite.dao.UserDao;
import it.callite.model.Agente;
import it.callite.model.Contratto;
import it.callite.model.EsitoContratto;
import it.callite.model.Notizia;
import it.callite.model.Utente;
import it.callite.utility.ApplicationUtil;
public class ContrattoNotiziaThread extends Thread {
	private Contratto contratto;
	private UserDao userDao;
	private NotiziaDao  notiziaDao;
	private DownloadDao downloadDao;
	private Utente user;
	public ContrattoNotiziaThread(Contratto contratto, NotiziaDao notiziaDao, DownloadDao downloadDao, UserDao userDao, Utente user) {
		this.contratto = contratto;
		this.userDao = userDao;
		this.notiziaDao = notiziaDao;
		this.downloadDao = downloadDao;
		this.user = user;
	}
	public void run() {
		Set<Utente> destinatari = new HashSet<Utente>();
		EsitoContratto esito = contratto.getEsito();
		String emailDestinatario = ApplicationUtil.trimToEmpty(esito.getNotizia_notifica());
		if (ApplicationUtil.isNotBlank(emailDestinatario)) {
			List<String> uns = ApplicationUtil.tokenU(emailDestinatario, ConstantsGlobal.colonSeparator);
			if(ApplicationUtil.notEmpty(uns)) {
				Iterator<String> uIt = uns.iterator();
				while (uIt.hasNext()) {
					String un = ApplicationUtil.trimToNull((String) uIt.next());
					if(un != null) {
						Utente utente = userDao.getUserByUsername(un);
						if(utente != null && !utente.equals(user)) {
							destinatari.add(utente);
						}
					}
				}
			}
		}
		if(ApplicationUtil.isTrue(esito.getNotizia_operatore())) {
			Utente operatore = contratto.getOperatore();
			if(operatore != null && !operatore.equals(user)) {
				destinatari.add(operatore);
			}
		}
		if(ApplicationUtil.isTrue(esito.getNotizia_agente())) {
			Agente agente = contratto.getAgente();
			if(agente != null) {
				List<Utente> utenti = userDao.getUtentiByAgenteId(agente.getId());
				if(ApplicationUtil.notEmpty(utenti)) {
					Iterator<Utente> uIt = utenti.iterator();
					while (uIt.hasNext()) {
						Utente utente = uIt.next();
						if(utente != null && !utente.equals(user)) {
							destinatari.add(utente);
						}
					}
				}
			}
		}
		
		if(ApplicationUtil.notEmpty(destinatari)) {
			Notizia notizia = new Notizia();
			String testo = downloadDao.notificaContrattoToHTMLString(contratto, user);
			String oggetto = ConstantsLanguage.MODIFICA.stampa() + " " +  ConstantsLanguage.CONTRATTO.stampa() +": "+contratto.getId();
			notizia.setTesto(testo);
			notizia.setOggetto(oggetto);
			notizia.setUtenti(new ArrayList<Utente>(destinatari));
			//notizia.setAllegati(allegati);
			notiziaDao.salvaNotiziaNuova(notizia);
		}
	}
}