package it.callite.utility.thread.export;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.criteria.AppuntamentoCriteria;
import it.callite.dao.AppuntamentoDao;
import it.callite.dao.DownloadDao;
import it.callite.dao.OperazioneDao;
import it.callite.log.ErrorLog;
import it.callite.model.Appuntamento;
import it.callite.model.Utente;
import it.callite.service.SmtpService;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.MailSupport;
public class EsportaAppuntamentoThread  extends EBaseThread {
	private String identificativo;
	private String tipologia;
	private AppuntamentoDao appuntamentoDao;
	private OperazioneDao operazioneDao;
	private SmtpService smtpService;
	private AppuntamentoCriteria criterio;
	private Utente utente;
	private Date inizio;
	private Integer totale;
	private Integer corrente;
	private Integer record;
	private Long tempoTotale;
	private Boolean completato;
	private Boolean scrittura;
	private Boolean interrompi;
	private Boolean excel;
	private DownloadDao downloadDao;
	public EsportaAppuntamentoThread(String tipologia, String identificativo, Boolean excel, SmtpService smtpService, AppuntamentoDao appuntamentoDao, OperazioneDao operazioneDao, DownloadDao downloadDao, AppuntamentoCriteria criterio, Utente utente, Integer record) {
		this.tipologia = tipologia;
		this.identificativo = identificativo;
		this.appuntamentoDao = appuntamentoDao;
		this.operazioneDao = operazioneDao;
		this.downloadDao = downloadDao;
		this.smtpService = smtpService;
		this.criterio = criterio;
		this.utente = utente;
		this.excel = excel;
		this.record = record;
		this.totale = 0;
		this.corrente = 0;
		this.tempoTotale = 0L;
		this.completato = false;
		this.scrittura = false;
		this.interrompi = false;
		this.inizio = ApplicationUtil.date();
	}
	public void run() {
		totale = appuntamentoDao.ricercaAppuntamentoManagerSize(criterio);
		if (ApplicationUtil.gtZero(totale)) {
			List<Appuntamento> appuntamenti = new ArrayList<Appuntamento>();
			for (int j = 0; j < getSize(); j++) {
				if(interrompi ||  corrente >= totale) { 
					break;
				}
				long time1 = System.currentTimeMillis();
				List<Appuntamento> a = appuntamentoDao.ricercaAppuntamentoManager(criterio, (j * record), record);
				if(ApplicationUtil.empty(a)) {
					interrompi = true;
				}
				appuntamenti.addAll(a);
				long time2 = System.currentTimeMillis();
				corrente = appuntamenti.size();
				tempoTotale += (time2-time1);
			}
			scrittura = true;
			try {
				String oggetto = ApplicationUtil.report(ConstantsLanguage.EXPORT.stampa() + " " + ConstantsLanguage.APPUNTAMENTI.stampa(), ConstantsGlobal.no_periodo,  inizio);
				String testo = downloadDao.preparaTemplateEmail(ConstantsLanguage.APPUNTAMENTI_IN_ALLEGATO.stampa(), inizio);
				String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
				operazioneDao.creaOperazione(ConstantsMap.LOG4,p, utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
				String file = ApplicationUtil.reportFile(inizio, ConstantsLanguage.APPUNTAMENTI.stampa()) + ".";
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				if(ApplicationUtil.isTrue(excel)) {
					downloadDao.esportaDatiExcelAppuntamento(appuntamenti, out);
				} else {
					downloadDao.esportaDatiPdfAppuntamento(appuntamenti, out);
				}
				String percorsoEmail = downloadDao.getEmailExportBackground();
				if (ApplicationUtil.isBlank(percorsoEmail)) {
					FileOutputStream outStream = new FileOutputStream(downloadDao.getPathExport() + "/" + file + (excel ? ConstantsMap.xlsx : ConstantsMap.pdf));  
					out.writeTo(outStream);  
					out.close();
					outStream.close();  
				} else {
					MailSupport ms = smtpService.getMailSupport(true);
					List<Allegato> allegati = ApplicationUtil.allegatiConvertO(out, file + (excel ? ConstantsMap.xlsx : ConstantsMap.pdf));
					smtpService.invia(ms, null, percorsoEmail, null, oggetto, testo, allegati, null);
					ms.close();
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
			if(interrompi ||  corrente >= totale) { 
				this.interrupt();
			}

		}  
		completato = true;
	}
	public synchronized int getSize(){
		if(ApplicationUtil.gtZero(totale) && ApplicationUtil.gtZero(record)) {
			return (totale/record) + ((totale % record > 0) ? 1 : 0);
		}
		else return 0;
	}
	public synchronized void setInterrompi(Boolean interrompi) {
		this.interrompi = interrompi;
	}
	public synchronized Boolean getCompletato() {
		return completato;
	}
	public synchronized Integer getCorrente() {
		if(ApplicationUtil.gtZero(totale) && ApplicationUtil.gtZero(corrente) && corrente >= totale) {
			corrente = totale;
		}
		return corrente;
	}
	public synchronized Date getInizio() {
		return inizio;
	}
	public synchronized Date getFine() {
		BigDecimal num = new BigDecimal(0);
		if(ApplicationUtil.gtZero(corrente)) {
			Double correnteT = Double.valueOf(corrente);
			Double tempoTotaleT = Double.valueOf(tempoTotale);
			num = new BigDecimal(tempoTotaleT / correnteT);
		}
		BigDecimal stak = num.multiply(new BigDecimal(getTotale() - getCorrente()));
		if(!stak.equals(new BigDecimal(0))) {
			return ApplicationUtil.addMilliseconds(ApplicationUtil.date(), stak.intValue());
		} else {
			return null;
		}
	}
	public synchronized BigDecimal getPercentuale() {
		BigDecimal bg = new BigDecimal(0);
		double percentuale = 100f;
		if(ApplicationUtil.gtZero(totale) && ApplicationUtil.gtZero(corrente)) {
			Double correnteT = Double.valueOf(corrente);
			Double totaleT = Double.valueOf(totale);
			bg = new BigDecimal(correnteT / totaleT * percentuale);
		}
		return bg.setScale(2, RoundingMode.HALF_UP);
	}
	public synchronized String getTempo() {
		Double tempoTotaleT = Double.valueOf(tempoTotale / ((double) ConstantsGlobal.millis));
		return ApplicationUtil.format(tempoTotaleT);
	}

	public synchronized String getTipo() {
		return ApplicationUtil.isTrue(excel) ? ConstantsLanguage.EXCEL.stampa() : ConstantsLanguage.PDF.stampa();
	}
	public synchronized Long getTempoTotale() {
		return tempoTotale;
	}
	public synchronized Integer getTotale() {
		return totale;
	}
	public synchronized Utente getUtente() {
		return utente;
	}

	public String getIdentificativo() {
		return identificativo;
	}
	public void setIdentificativo(String identificativo) {
		this.identificativo = identificativo;
	}
	public Boolean getScrittura() {
		return scrittura;
	}
	public void setScrittura(Boolean scrittura) {
		this.scrittura = scrittura;
	}
	public Boolean getInterrompi() {
		return interrompi;
	}
	public String getTipologia() {
		return tipologia;
	}
	public void setTipologia(String tipologia) {
		this.tipologia = tipologia;
	}
	
}