package it.callite.utility.xlsx.sst;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import it.callite.constants.ConstantsGlobal;


@SuppressWarnings("serial")
public class FileBackedList implements AutoCloseable {

	private final List<Long> pointers = new ArrayList<>();
	private final RandomAccessFile raf;
	private final FileChannel channel;
	private final Map<Integer, String> cache;

	private long filesize;

	@SuppressWarnings("rawtypes")
	public FileBackedList(File file, final int cacheSize) throws IOException {
		this.raf = new RandomAccessFile(file, "rw");
		this.channel = raf.getChannel();
		this.filesize = raf.length();
		this.cache = new LinkedHashMap<Integer, String>(cacheSize, 0.75f, true) {
			public boolean removeEldestEntry(Map.Entry eldest) {
				return size() > cacheSize;
			}
		};
	}

	public void add(String str) {
		try {
			if (str!=null && str.length()>0) {
				writeToFile(str);
			}
		} catch(IOException e) {
			throw new RuntimeException(e);
		}
	}

	public String getAt(int index) {
		if(cache.containsKey(index)) {
			return cache.get(index);
		}

		try {
			String val = readFromFile(pointers.get(index));
			cache.put(index, val);
			return val;
		} catch(IOException e) {
			throw new RuntimeException(e);
		}
	}

	private void writeToFile(String str) throws IOException {
		synchronized (channel) {
			ByteBuffer bytes = ByteBuffer.wrap(str.getBytes(ConstantsGlobal.CHARSET));
			ByteBuffer length = ByteBuffer.allocate(4).putInt(bytes.array().length);

			channel.position(filesize);
			pointers.add(channel.position());
			length.flip();
			channel.write(length);
			channel.write(bytes);

			filesize += 4 + bytes.array().length;
		}
	}

	private String readFromFile(long pointer) throws IOException {
		synchronized (channel) {
			FileChannel fc = channel.position(pointer);

			ByteBuffer buffer = ByteBuffer.wrap(new byte[4]);
			fc.read(buffer);
			buffer.flip();
			int length = buffer.getInt();

			buffer = ByteBuffer.wrap(new byte[length]);
			fc.read(buffer);
			buffer.flip();

			return new String(buffer.array(), ConstantsGlobal.CHARSET);
		}
	}

	@Override
	public void close() {
		try {
			raf.close();
		} catch(IOException e) {
			throw new RuntimeException(e);
		}
	}
}
