package it.callite.dao;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;

import javax.faces.model.SelectItem;

import org.apache.commons.net.ftp.FTPClient;
import org.hibernate.FetchMode;
import org.hibernate.Hibernate;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.type.LongType;
import org.hibernate.type.Type;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import it.callite.action.DownloadBean;
import it.callite.autodialer.controllo.CodaStatic;
import it.callite.chiamate.Chiamate;
import it.callite.column.generic.GenericColumn;
import it.callite.comparator.AgenteComparator;
import it.callite.comparator.ContattoSIComparator;
import it.callite.constants.ConstantsColumn;
import it.callite.constants.ConstantsEsito;
import it.callite.constants.ConstantsGlobal;
import it.callite.constants.ConstantsLanguage;
import it.callite.constants.ConstantsMap;
import it.callite.constants.config.DefaultsPannello;
import it.callite.criteria.CampagnaCriteria;
import it.callite.criteria.CampagnaPannelloCriteria;
import it.callite.criteria.ContattoCriteria;
import it.callite.criteria.EsitoAppuntamentoCriteria;
import it.callite.criteria.EsitoArchiviazioneCriteria;
import it.callite.criteria.EsitoContrattoCriteria;
import it.callite.criteria.FileCriteria;
import it.callite.criteria.ListaCriteria;
import it.callite.criteria.base.ExtendedCriteria;
import it.callite.dao.generic.GenericDao;
import it.callite.log.ErrorLog;
import it.callite.message.Message;
import it.callite.model.Agente;
import it.callite.model.Anagrafica;
import it.callite.model.Appuntamento;
import it.callite.model.Autodialer;
import it.callite.model.Busta;
import it.callite.model.Campagna;
import it.callite.model.CampoAggiuntivoAnagrafica;
import it.callite.model.CampoAggiuntivoContratto;
import it.callite.model.Contatto;
import it.callite.model.Contratto;
import it.callite.model.DatoAggiuntivoAnagrafica;
import it.callite.model.DatoAggiuntivoContratto;
import it.callite.model.Email;
import it.callite.model.EmailTemplate;
import it.callite.model.EsitoAppuntamento;
import it.callite.model.EsitoArchiviazione;
import it.callite.model.EsitoChiamata;
import it.callite.model.EsitoContratto;
import it.callite.model.Rmail;
import it.callite.model.RmailTemplate;
import it.callite.model.Lista;
import it.callite.model.Nota;
import it.callite.model.Prenotazione;
import it.callite.model.ProdottiInVendita;
import it.callite.model.Prodotto;
import it.callite.model.Risposta;
import it.callite.model.Schedulatore;
import it.callite.model.Team;
import it.callite.model.Telecom;
import it.callite.model.TelecomTemplate;
import it.callite.model.Template;
import it.callite.model.TemplateStore;
import it.callite.model.Utente;
import it.callite.model.Zona;
import it.callite.model.allegato.AllegatoAppuntamento;
import it.callite.model.pk.ProdottiInVenditaPK;
import it.callite.model.report.ColonnaModel;
import it.callite.model.report.ConteggioModel;
import it.callite.model.report.ListaSchedulate;
import it.callite.model.report.PannelloModel;
import it.callite.model.report.PannelloReport;
import it.callite.service.config.ConfigurazionePannelloService;
import it.callite.utility.Allegato;
import it.callite.utility.ApplicationUtil;
import it.callite.utility.DatoAggiuntivo;
import it.callite.utility.FileUpload;
import it.callite.utility.Label;
import it.callite.utility.LoggerColonna;
import it.callite.utility.LoggerImport;
import it.callite.utility.MyFile;
import it.callite.utility.Parameter;
import it.callite.utility.PulisciUtil;
import it.callite.utility.StatoIntervallo;
import it.callite.utility.UtilFTP;
import it.callite.utility.item.AgenteSI;
import it.callite.utility.item.CampagnaSI;
import it.callite.utility.item.ContattoSI;
import it.callite.utility.item.EsitoAppuntamentoSI;
import it.callite.utility.item.EsitoArchiviazioneSI;
import it.callite.utility.item.EsitoChiamataSI;
import it.callite.utility.item.EsitoContrattoSI;
import it.callite.utility.item.ListaSI;
import it.callite.utility.item.ProdottoSI;
import it.callite.utility.item.TeamSI;
import it.callite.utility.item.UtenteSI;
import it.callite.utility.item.ZonaSI;
@SuppressWarnings("serial")
@Repository("campagnaDao")
public class CampagnaDao extends GenericDao implements InitializingBean, DisposableBean{
	@Autowired
	private UserDao userDao;
	@Autowired
	private PannelloDao pannelloDao;
	@Autowired
	private ProdottoDao prodottoDao;
	@Autowired
	private ConfigurazionePannelloService configurazionePannelloService;
	@Autowired
	private OperazioneDao operazioneDao;
	@Autowired
	private AnagraficaDao anagraficaDao;
	@Autowired
	private ComunicazioneDao comunicazioneDao;
	@Autowired
	private ImpostazioneDao impostazioneDao;
	@Autowired
	private DataDao dataDao;
	@Autowired
	private DownloadDao downloadDao;
	
	
	
	public int getContattoCampagna(Long cam) {
		if(cam != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
			criteria.add(Restrictions.eq("campagna.id", cam));
			return getListSize(criteria);
		} else return 0;
	}
	public int getContattoLista(Long lst) {
		if(lst != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
			criteria.add(Restrictions.eq("lista.id", lst));
			return getListSize(criteria);
		} else return 0;
	}
	public int getAnagraficaLista(Long lst) {
		if(lst != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Anagrafica.class);
			criteria.createCriteria("lista").add(Restrictions.eq("id", lst));
			return getListSize(criteria);
		} else return 0;
	}
	
	public List<Campagna> impostaConteggioCampagna(GenericColumn c, List<Campagna> campagne) {
		if(ApplicationUtil.notEmpty(campagne))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdCampagna(campagne);
			Map<Long, ConteggioModel> lc = dataDao.getConteggio1(c.getColonna(ConstantsColumn.contatti), ll, "contatti", Campagna.class);
			Map<Long, ConteggioModel> li = dataDao.getConteggio1(c.getColonna(ConstantsColumn.prodotti),ll, "prodottiInVendita", Campagna.class);
			
			Map<Long, ConteggioModel> ls = dataDao.getConteggio1(c.getColonna(ConstantsColumn.liste),ll, "liste", Campagna.class);
			Map<Long, ConteggioModel> lz = dataDao.getConteggio1(c.getColonna(ConstantsColumn.zone),ll, "zone", Campagna.class);
			Map<Long, ConteggioModel> lt = dataDao.getConteggio1(c.getColonna(ConstantsColumn.team),ll, "team", Campagna.class);
			Map<Long, ConteggioModel> lp = dataDao.getConteggio1(c.getColonna(ConstantsColumn.partner),ll, "partner", Campagna.class);
			Map<Long, ConteggioModel> le = dataDao.getConteggio1(c.getColonna(ConstantsColumn.esiti),ll, "esiti_archiviazione", Campagna.class);
			Map<Long, ConteggioModel> re = dataDao.getConteggio1(c.getColonna(ConstantsColumn.risposte),ll, "risposte", Campagna.class);
			
			Map<Long, ConteggioModel> ce = dataDao.getConteggio1(c.getColonna(ConstantsColumn.email),ll, "email", Campagna.class);
			Map<Long, ConteggioModel> cl = dataDao.getConteggio1(c.getColonna(ConstantsColumn.rmail),ll, "rmail", Campagna.class);
			Map<Long, ConteggioModel> cm = dataDao.getConteggio1(c.getColonna(ConstantsColumn.telecom),ll, "telecom", Campagna.class);
			Iterator<Campagna> it = campagne.iterator();
			while (it.hasNext()) {
				Campagna campagna = (Campagna) it.next();
				ColonnaModel colonna = campagna.getModel();
				colonna.setNcontatti(ApplicationUtil.getInMap(lc, campagna.getId()));
				colonna.setNliste(ApplicationUtil.getInMap(ls, campagna.getId()));
				colonna.setNzone(ApplicationUtil.getInMap(lz, campagna.getId()));
				colonna.setNteam(ApplicationUtil.getInMap(lt, campagna.getId()));
				colonna.setNpartner(ApplicationUtil.getInMap(lp, campagna.getId()));
				colonna.setNprodotti(ApplicationUtil.getInMap(li, campagna.getId()));
				colonna.setNesiti(ApplicationUtil.getInMap(le, campagna.getId()));
				colonna.setNrisposte(ApplicationUtil.getInMap(re, campagna.getId()));
				
				colonna.setNemail(ApplicationUtil.getInMap(ce, campagna.getId()));
				colonna.setNrmail(ApplicationUtil.getInMap(cl, campagna.getId()));
				colonna.setNtelecom(ApplicationUtil.getInMap(cm, campagna.getId()));
				
			}
			l.log(c);
		}
		return campagne;
	}
	

	public List<Lista> impostaConteggioLista(GenericColumn c, List<Lista> liste) {
		if(ApplicationUtil.notEmpty(liste))  {
			LoggerColonna l = new LoggerColonna(impostazioneDao.getLoggerColonne());
			List<Long> ll = ApplicationUtil.prendiIdLista(liste);
			Map<Long, ConteggioModel> lc = dataDao.getConteggio2(c.getColonna(ConstantsColumn.contatti), ll, "lista", Contatto.class);
			Map<Long, ConteggioModel> ls = dataDao.getConteggio1(c.getColonna(ConstantsColumn.campagne),ll, "campagna", Lista.class);
			Map<Long, ConteggioModel> la = dataDao.getConteggio1(c.getColonna(ConstantsColumn.anagrafiche),ll, "contatti", Lista.class);
			Iterator<Lista> it = liste.iterator();
			while (it.hasNext()) {
				Lista lista = (Lista) it.next();
				ColonnaModel colonna = lista.getModel();
				colonna.setNchiamate(ApplicationUtil.getInMap(lc, lista.getId()));
				colonna.setNanagrafiche(ApplicationUtil.getInMap(la, lista.getId()));
				colonna.setNcampagne(ApplicationUtil.getInMap(ls, lista.getId()));
			}
			l.log(c);
		}
		return liste;
	}
	
	public List<Campagna> impostaConteggioCampagna(List<Campagna> campagne) {
		if(ApplicationUtil.notEmpty(campagne))  {
			List<Long> ll = ApplicationUtil.prendiIdCampagna(campagne);
			Map<Long, ConteggioModel> lc = dataDao.getConteggio1(true, ll, "contatti", Campagna.class);
			Map<Long, ConteggioModel> ls = dataDao.getConteggio1(true,ll, "liste", Campagna.class);
			Iterator<Campagna> it = campagne.iterator();
			while (it.hasNext()) {
				Campagna campagna = (Campagna) it.next();
				ColonnaModel colonna = campagna.getModel();
				colonna.setNcontatti(ApplicationUtil.getInMap(lc, campagna.getId()));
				colonna.setNliste(ApplicationUtil.getInMap(ls, campagna.getId()));
			}
		}
		return campagne;
	}
	
	
	public List<Lista> impostaConteggioLista(List<Lista> liste) {
		if(ApplicationUtil.notEmpty(liste))  {
			List<Long> ll = ApplicationUtil.prendiIdLista(liste);
			Map<Long, ConteggioModel> lc = dataDao.getConteggio2(true, ll, "lista", Contatto.class);
			Map<Long, ConteggioModel> ls = dataDao.getConteggio1(true, ll, "campagna", Lista.class);
			Map<Long, ConteggioModel> la = dataDao.getConteggio1(true, ll, "contatti", Lista.class);
			Iterator<Lista> it = liste.iterator();
			while (it.hasNext()) {
				Lista lista = (Lista) it.next();
				ColonnaModel colonna = lista.getModel();
				colonna.setNchiamate(ApplicationUtil.getInMap(lc, lista.getId()));
				colonna.setNanagrafiche(ApplicationUtil.getInMap(la, lista.getId()));
				colonna.setNcampagne(ApplicationUtil.getInMap(ls, lista.getId()));
			}
		}
		return liste;
	}
	public PulisciUtil getCampagneSI(List<ContattoSI> contatti) {
		List<CampagnaSI> cis = new ArrayList<CampagnaSI>();
		List<EsitoChiamataSI> eis = new ArrayList<EsitoChiamataSI>();
		List<EsitoArchiviazioneSI> ais = new ArrayList<EsitoArchiviazioneSI>();
		List<Long> cid = new ArrayList<Long>();
		List<Long> eid = new ArrayList<Long>();
		List<Long> aid = new ArrayList<Long>();
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<ContattoSI> objIter = contatti.iterator();
			while(objIter.hasNext()) {
				ContattoSI c = objIter.next();
				if(c.getCampagna_id() != null && !cid.contains(c.getCampagna_id())) {
					cid.add(c.getCampagna_id());
					cis.add(new CampagnaSI(c));
				}
				if(c.getEsito_chiamata_id() != null && !eid.contains(c.getEsito_chiamata_id())) {
					eid.add(c.getEsito_chiamata_id());
					eis.add(new EsitoChiamataSI(c));
				}
				if(c.getEsito_archiviazione_id() != null && !aid.contains(c.getEsito_archiviazione_id())) {
					aid.add(c.getEsito_archiviazione_id());
					ais.add(new EsitoArchiviazioneSI(c));
				}
			}
		}
		PulisciUtil pu = new PulisciUtil();
		pu.setElenco(impostazioneDao.getCampagna(cis, false, false, false, null));
		pu.setEsiti_chiamata(impostazioneDao.getEsitoChiamata(eis, false, null));
		pu.setEsiti_archiviazione(impostazioneDao.getEsitoArchiviazione(ais, false, null));
		return	pu;
	}

	@SuppressWarnings("unchecked")
	public List<Utente> getUtentiByAgenteId(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("agenti").add(Restrictions.idEq(agenteId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public void salvaImmagine(FileUpload fileUpload, Campagna campagna) {
		String file = prodottoDao.getNuovaImmagine();
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		boolean b = ApplicationUtil.salvaImmagine(fileUpload.getFiles(), path + file, ConstantsGlobal.imageW, ConstantsGlobal.imageH);
		if(b) {
			Boolean l = impostazioneDao.getContentLogo();
			campagna.setContent(ApplicationUtil.content(l, path, file));
		}
	}
	public void salvaImmagine(Campagna campagna, File logo) {
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		Boolean l = impostazioneDao.getContentLogo();
		campagna.setContent(ApplicationUtil.content(l, path, ApplicationUtil.getNameN(logo.getAbsolutePath())));
	}
	public boolean salvaImmagineInternet(String url, Campagna campagna) {
		String file = prodottoDao.getNuovaImmagine();
		String path = impostazioneDao.getPercorsoImmagini() + "/";
		boolean b = ApplicationUtil.salvaImmagine(url, path + file, ConstantsGlobal.imageW, ConstantsGlobal.imageH);
		if(b) {
			Boolean l = impostazioneDao.getContentLogo();
			campagna.setContent(ApplicationUtil.content(l, path, file));
		}
		return b;
	}
	public List<SelectItem> getTipiAutodialerSI() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(1, Message.addCatalogText("autodialer1")));
		sis.add(new SelectItem(2, Message.addCatalogText("autodialer2")));
		sis.add(new SelectItem(3, Message.addCatalogText("autodialer3")));
		sis.add(new SelectItem(4, Message.addCatalogText("autodialer4")));
		return sis;
	}

	private ExtendedCriteria trovaCriteria(CampagnaCriteria campagna) {
		if(campagna != null)  campagna = (CampagnaCriteria) campagna.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		if (ApplicationUtil.isNotBlank(campagna.getRicerca())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, campagna.getRicerca(), criteria, "nome", "descrizione");
		}
		if (ApplicationUtil.gtZero(campagna.getProdotto_id())) {
			impostazioneDao.subCriteriaCampagnaProdotto(campagna.getProdotto_id(), criteria, "id");
		}
		ApplicationUtil.statoLC(campagna.getStato_id(), campagna.getAttivo_id(), criteria);
		if (ApplicationUtil.gtZero(campagna.getTipo())) {
			criteria.add(Restrictions.eq("tipo",campagna.getTipo()));
		}
		if (ApplicationUtil.gtZero(campagna.getLista_id())) {
			criteria.createCriteria("liste").add(Restrictions.idEq(campagna.getLista_id()));
		}
		if (ApplicationUtil.gtZero(campagna.getTeam_id())) {
			criteria.createCriteria("team").add(Restrictions.idEq(campagna.getTeam_id()));
		}
		if (ApplicationUtil.gtZero(campagna.getAgente_id())) {
			criteria.createCriteria("zone").add(Restrictions.idEq(campagna.getAgente_id()));
		}
		if (ApplicationUtil.gtZero(campagna.getEsito_id())) {
			criteria.createCriteria("esiti_archiviazione").add(Restrictions.idEq(campagna.getEsito_id()));
		}
		if (ApplicationUtil.gtZero(campagna.getPartner_id())) {
			criteria.createCriteria("partner").add(Restrictions.idEq(campagna.getPartner_id()));
		}
		if (ApplicationUtil.isNotBlank(campagna.getCallers())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, campagna.getCallers(), criteria, "callers", "inbound");
		}
		if (ApplicationUtil.gtZero(campagna.getAutodialer())) {
			if (campagna.getAutodialer() == 1) {
				criteria.add(Restrictions.isNull("autodialer.id"));
			}
			if (campagna.getAutodialer() == 2) {
				criteria.add(Restrictions.isNotNull("autodialer.id"));
			} 
			if (campagna.getAutodialer() == 3) {
				criteria.createCriteria("autodialer").add(Restrictions.isNull("esitoInbound.id"));
			}
			if (campagna.getAutodialer() == 4) {
				criteria.createCriteria("autodialer").add(Restrictions.isNotNull("esitoInbound.id"));
			}
		}
		campagna.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, campagna);
	}
	public String getNomeCampagnaById(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("nome")));
		return  (String) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> trova(CampagnaCriteria campagna, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(campagna);
		criteria.getCriteria().setFetchMode("autodialer", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), campagna.getO_tipo(), campagna.getO_colonna(), new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaSize(CampagnaCriteria campagna) {
		ExtendedCriteria criteria = trovaCriteria(campagna);
		return getListSize(criteria);
	}
	public Campagna getFullCampagna(Long id) {
		if(ApplicationUtil.gtZero(id)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.add(Restrictions.idEq(id));
			criteria.setFetchMode("liste", FetchMode.JOIN);
			return (Campagna) getUniqueResult(criteria);
		} else return null;
	}
	
	@SuppressWarnings("unchecked")
	public List<Contatto> getFullContatti(Long anagraficaId, Long campagnaId, Long listaId) {
		if(ApplicationUtil.gtZero(anagraficaId) && ApplicationUtil.gtZero(campagnaId) && ApplicationUtil.gtZero(listaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class);
			criteria.add(Restrictions.eq("anagrafica.id", anagraficaId));
			criteria.add(Restrictions.eq("campagna.id", campagnaId));
			criteria.add(Restrictions.eq("lista.id", listaId));
			return getList(criteria);
		} else return null;
	}
	
	public Long getPrenotazioneByTelefono(String telefono, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prenotazione.class);
		criteria.add(Restrictions.eq("campagna.id", campagnaId));
		criteria.add(Restrictions.eq("telefono", telefono));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return (Long) getUniqueResult(criteria);
	}
	public Long getAnagraficaByTelefono(String telefono, String tipo) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Anagrafica.class);
		if(tipo != null && ConstantsMap.telefonoFMap.contains(tipo)) {
			ApplicationUtil.telefonoE(criteria, telefono, tipo);
		} else {
			ApplicationUtil.telefonoE(criteria, telefono);
		}
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return (Long) getUniqueResult(criteria);
	}
	public Campagna getFullCampagnaT(Long id) {
		if(ApplicationUtil.gtZero(id)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.add(Restrictions.idEq(id));
			criteria.setFetchMode("liste", FetchMode.JOIN);
			criteria.setFetchMode("autodialer", FetchMode.JOIN);
			return (Campagna) getUniqueResult(criteria);
		} else return null;
	}
	public Campagna getFullCampagnaA(Campagna campagna) {
		if(ApplicationUtil.gtZero(campagna.getId())) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.add(Restrictions.idEq(campagna.getId()));
			criteria.setFetchMode("liste", FetchMode.JOIN);
			criteria.setFetchMode("autodialer", FetchMode.JOIN);
			return (Campagna) getUniqueResult(criteria);
		} else return campagna;
	}
	public Lista getFullListaContatti(Long id) {
		if(ApplicationUtil.gtZero(id)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
			criteria.add(Restrictions.idEq(id));
			criteria.setFetchMode("contatti", FetchMode.JOIN);
			return (Lista) getUniqueResult(criteria);
		} else return null;
	}
	public Campagna preparaCampagna(Long id, Long utente,Long partner, Integer copia) {
		if(copia!=null) {
			if(ApplicationUtil.eq(copia, 1)) {
				return copiaCampagna(id, utente, partner, false);
			} else if(ApplicationUtil.eq(copia, 2)) {
				return copiaCampagna(id, utente, partner, true);
			} else {
				return getFullCampagnaT(id);
			}
		}
		return null;
	}
	public Campagna getFullCampagnaZone(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("zone", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteria);
	}
	public Campagna getFullCampagnaEmail(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("email", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteria);
	}
	
	public EmailTemplate getFullEmailCampagna(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return (EmailTemplate) getUniqueResult(criteria);
	}
	
	public Campagna getFullCampagnaEsito(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("esiti_archiviazione", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteria);
	}
	public Template getFullTemplate(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Template.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("template", FetchMode.JOIN);
		return (Template) getUniqueResult(criteria);
	}
	public Busta getFullBusta(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Busta.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("bustaStore", FetchMode.JOIN);
		return (Busta) getUniqueResult(criteria);
	}
	public List<SelectItem> getPartnerSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("campagne").add(Restrictions.isNotNull("id"));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, true, null);
	}
	public List<SelectItem> getCampagnePartnerSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("partner").add(Restrictions.isNotNull("id"));
		List<CampagnaSI> lis = getCampagnaSI(criteria);
		return impostazioneDao.getCampagna(lis, true, false, false, null);
	}
	public List<SelectItem> getEsitiArchiviazioneSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, true, null);
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaSI(ContattoCriteria crt) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		if(crt!= null) {
			if(ApplicationUtil.gtZero(crt.getCampagnaId())) {
				criteria.createCriteria("campagne").add(Restrictions.eq("id",crt.getCampagnaId()));
			}
			if(ApplicationUtil.isNotBlank(crt.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(crt.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaAgenteSI(Utente user, ContattoCriteria crt) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		//DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		//criteriaC.createCriteria("zone").createCriteria("agente").createCriteria("user").add(Restrictions.idEq(user.getId()));
		if(crt!= null) {
			if(ApplicationUtil.gtZero(crt.getCampagnaId())) {
				criteria.createCriteria("campagne").add(Restrictions.eq("id",crt.getCampagnaId()));
			}
			if(ApplicationUtil.isNotBlank(crt.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(crt.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaCoordinatoreSI(Utente user, ContattoCriteria crt) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		//DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		//criteriaC.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(user.getId()));
		if(crt!= null) {
			if(ApplicationUtil.gtZero(crt.getCampagnaId())) {
				criteria.createCriteria("campagne").add(Restrictions.eq("id",crt.getCampagnaId()));
			}
			if(ApplicationUtil.isNotBlank(crt.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(crt.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaPartnerSI(Utente user, ContattoCriteria crt) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		//DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		//criteriaC.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		if(crt!= null) {
			if(ApplicationUtil.gtZero(crt.getCampagnaId())) {
				criteria.createCriteria("campagne").add(Restrictions.eq("id",crt.getCampagnaId()));
			}
			if(ApplicationUtil.isNotBlank(crt.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(crt.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaSupervisoreSI(Utente user, ContattoCriteria crt) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		//DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		//criteriaC.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		if(crt!= null) {
			if(ApplicationUtil.gtZero(crt.getCampagnaId())) {
				criteria.createCriteria("campagne").add(Restrictions.eq("id",crt.getCampagnaId()));
			}
			if(ApplicationUtil.isNotBlank(crt.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(crt.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaReteSI(Utente user, ContattoCriteria crt) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		//Integer max = userDao.getMaxLivelloRete();
		//DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		//DetachedCriteria c = criteriaC.createCriteria("zone").createCriteria("agente");
		//impostazioneDao.subCriteriaAgentiSottoReteByUser(user.getId(),0,  max, c, "id");
		if(crt!= null) {
			if(ApplicationUtil.gtZero(crt.getCampagnaId())) {
				criteria.createCriteria("campagne").add(Restrictions.eq("id",crt.getCampagnaId()));
			}
			if(ApplicationUtil.isNotBlank(crt.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(crt.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaOperatoreSI(Utente utente, ContattoCriteria crt) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		//DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		//criteriaC.createCriteria("team").createCriteria("user").add(Restrictions.idEq(utente.getId()));
		if(crt!= null) {
			//if(!ApplicationUtil.isTrue(crt.getStorico())) {
			//	criteria.add(Restrictions.eq("tipo",Boolean.TRUE));
			//}
			if(ApplicationUtil.gtZero(crt.getCampagnaId())) {
				criteria.createCriteria("campagne").add(Restrictions.eq("id",crt.getCampagnaId()));
			}
			if(ApplicationUtil.isNotBlank(crt.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(crt.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}		
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaBackofficeSI(ContattoCriteria crt) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.eq("backoffice", Boolean.TRUE));
		if(crt!= null) {
			if(ApplicationUtil.gtZero(crt.getCampagnaId())) {
				criteria.createCriteria("campagne").add(Restrictions.eq("id",crt.getCampagnaId()));
			}
			if(ApplicationUtil.isNotBlank(crt.getPositivo())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getPositivo(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
				} 
			}
			if(ApplicationUtil.isNotBlank(crt.getUtile())) {
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.SI.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.TRUE));
				} 
				if(ApplicationUtil.equalsIgnoreCase(crt.getUtile(), ConstantsLanguage.NO.stampa())) {
					criteria.add(Restrictions.eq("utile", Boolean.FALSE));
				} 
			}
		}
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitiByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.createCriteria("campagne").add(Restrictions.eq("id",campagnaId));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> getEsitiIdByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.createCriteria("campagne").add(Restrictions.eq("id",campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	public EsitoArchiviazione getEsitoById(Long esitoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esitoId));
		return (EsitoArchiviazione) getUniqueResult(criteria);
	}
	public EsitoArchiviazione getFullEsitoById(Long esitoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esitoId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return (EsitoArchiviazione) getUniqueResult(criteria);
	}
	public EsitoArchiviazione salvaEsito(EsitoArchiviazione esitoArchiviazione) {
		return (EsitoArchiviazione) salvaAggiorna(esitoArchiviazione);
	}
	public ProdottiInVendita salvaProdotto(ProdottiInVendita prodottiInVendita) {
		return (ProdottiInVendita) salvaAggiorna(prodottiInVendita);
	}
	@SuppressWarnings("unchecked")
	public List<Team> getTeamByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderT(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getPartnerByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public  List<Long> getIdUtentiBySupervisore(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(userId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public  List<Long> getIdUtentiByOperatore(Long userId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("team").createCriteria("user").add(Restrictions.idEq(userId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	public DetachedCriteria trovaCampagneAttiveCriteria(boolean supervisore, Long userId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria campagnaC = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subCriteriaCampagnaUser(supervisore, userId, campagnaC, "id");
		ApplicationUtil.criteriaStato(campagnaC, true, ApplicationUtil.isTrue(ts));
		return campagnaC;
	}
	public DetachedCriteria trovaCampagneAttiveCriteria(CampagnaPannelloCriteria cmcr, boolean supervisore, Long userId) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria campagnaC = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subCriteriaCampagnaUser(supervisore, userId, campagnaC, "id");
		ApplicationUtil.criteriaStato(campagnaC, true, ApplicationUtil.isTrue(ts));
		if (ApplicationUtil.isNotBlank(cmcr.getRicerca())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, cmcr.getRicerca(), campagnaC, "nome", "descrizione");
		}
		if (ApplicationUtil.gtZero(cmcr.getAutodialer())) {
			if (cmcr.getAutodialer() == 1) {
				campagnaC.add(Restrictions.isNotNull("autodialer.id"));
			} 
			if (cmcr.getAutodialer() == 2) {
				campagnaC.add(Restrictions.isNull("autodialer.id"));
			}
		}
		return campagnaC;
	}

	public EsitoArchiviazione getFullEsitoArchiviazioneById(Long esitoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esitoId));
		criteria.setFetchMode("esito_collegato", FetchMode.JOIN);
		return (EsitoArchiviazione) getUniqueResult(criteria);
	}

	@SuppressWarnings("unchecked")
	public List<Long> prendiIdCampagnaUser(Utente user) {
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria campagnaC = DetachedCriteria.forClass(Campagna.class);
		campagnaC.createCriteria("team").createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		ApplicationUtil.criteriaStato(campagnaC, true, ApplicationUtil.isTrue(ts));
		campagnaC.setProjection(Projections.distinct(Projections.property("id")));
		return getList(campagnaC);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> trovaCampagnePannelloUtente(CampagnaPannelloCriteria cmcr, String tipo, Boolean supervisore, Boolean statisticheOperatore, Boolean statisticheSupervisore, Long userId, int start, int size) {
		DetachedCriteria criteria = trovaCampagneAttiveCriteria(cmcr, supervisore, userId);
		criteria.setFetchMode("autodialer", FetchMode.JOIN);
		ApplicationUtil.setOrdinamento(criteria, cmcr.getO_tipo(), cmcr.getO_colonna(), new Order [] { Order.desc("priorita"), Order.asc("id") } );
		List<Campagna> campagne = getList(criteria, start, size);
		if((!supervisore && ApplicationUtil.isTrue(statisticheOperatore)) || (supervisore && ApplicationUtil.isTrue(statisticheSupervisore))) {
			List<Long> utentiPubblici = new ArrayList<Long>();
			List<Long> utentiPrivati = new ArrayList<Long>();
			if(supervisore) {
				List<Long> u  = getIdUtentiBySupervisore(userId);
				utentiPubblici.addAll(u);
				utentiPrivati .addAll(u);
			} else {
				List<Long> u  = getIdUtentiByOperatore(userId);
				utentiPubblici.addAll((u));
				utentiPrivati.add(userId);
			}
			List<Long> campagneId = ApplicationUtil.prendiIdCampagna(campagne);
			List<PannelloModel> d_a = Chiamate.getCodaContatti().get(1, utentiPubblici);
			List<PannelloModel> d_b = Chiamate.getCodaAutodialer().list(1);
			List<PannelloModel> listaPubblici = getRichiamiPubbliciCampagne(tipo, campagneId);
			List<PannelloModel> listaPersonali =  getRichiamiPersonaliCampagne(tipo, campagneId, utentiPrivati);
			SortedMap<Long, ArrayList<PannelloModel>>  mappaListaPubblici = ApplicationUtil.collectionToHashPannello(listaPubblici, "getEsito_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mappaListaPersonali = ApplicationUtil.collectionToHashPannello(listaPersonali, "getEsito_id");
			List<PannelloModel> listaNuovi = ApplicationUtil.getPannello(mappaListaPubblici, 0L);
			List<PannelloModel> listaRichiami = ApplicationUtil.getPannello(mappaListaPubblici, 1L);
			List<PannelloModel> listaPrivati = ApplicationUtil.getPannello(mappaListaPersonali, 0L);
			List<PannelloModel> listaArchiviati = ApplicationUtil.getPannello(mappaListaPersonali, 1L);
			List<PannelloModel> listaPrivatiAttiva =  ApplicationUtil.getAttive(listaPrivati, d_a);
			List<PannelloModel> listaArchiviatiAttiva =  ApplicationUtil.getAttive(listaArchiviati,  d_a);
			List<PannelloModel> listaNuoviAttiva =  ApplicationUtil.getAttive(listaNuovi,  d_a);
			List<PannelloModel> listaNuoviAutoAttiva =  ApplicationUtil.getAttive(listaNuovi,  d_b);
			List<PannelloModel> listaRichiamiAttiva =  ApplicationUtil.getAttive(listaRichiami, d_a);
			List<PannelloModel> listaRichiamiAutoAttiva =  ApplicationUtil.getAttive(listaRichiami,  d_b);
			SortedMap<Long, ArrayList<PannelloModel>>  mcNuovi = ApplicationUtil.collectionToHashPannello(listaNuovi, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcRichiami = ApplicationUtil.collectionToHashPannello(listaRichiami, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcPrivati = ApplicationUtil.collectionToHashPannello(listaPrivati, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcArchiviati = ApplicationUtil.collectionToHashPannello(listaArchiviati, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcPrivatiAttiva = ApplicationUtil.collectionToHashPannello(listaPrivatiAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcArchiviatiAttiva = ApplicationUtil.collectionToHashPannello(listaArchiviatiAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcNuoviAttiva = ApplicationUtil.collectionToHashPannello(listaNuoviAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcNuoviAutoAttiva = ApplicationUtil.collectionToHashPannello(listaNuoviAutoAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcRichiamiAttiva = ApplicationUtil.collectionToHashPannello(listaRichiamiAttiva, "getCampagna_id");
			SortedMap<Long, ArrayList<PannelloModel>>  mcRichiamiAutoAttiva = ApplicationUtil.collectionToHashPannello(listaRichiamiAutoAttiva, "getCampagna_id");
			Iterator<Campagna> objIter = campagne.iterator();
			while(objIter.hasNext()) {
				Campagna campagna = objIter.next();
				PannelloReport report = new PannelloReport();
				Integer contatti_nuovi = ApplicationUtil.getPannello(mcNuovi, campagna.getId()).size();
				Integer attive_nuovi = ApplicationUtil.getPannello(mcNuoviAttiva, campagna.getId()).size();
				Integer attive_nuovi_auto = ApplicationUtil.getPannello(mcNuoviAutoAttiva, campagna.getId()).size();
				Integer contatti_richiami =  ApplicationUtil.getPannello(mcRichiami, campagna.getId()).size();
				Integer attive_richiami = ApplicationUtil.getPannello(mcRichiamiAttiva, campagna.getId()).size();
				Integer attive_richiami_auto = ApplicationUtil.getPannello(mcRichiamiAutoAttiva, campagna.getId()).size();
				report.setContatti(contatti_nuovi + contatti_richiami);
				report.setContatti_nuovi(contatti_nuovi);
				report.setContatti_richiami(contatti_richiami);
				report.setAttive(attive_nuovi + attive_richiami);
				report.setAttive_auto(attive_nuovi_auto + attive_richiami_auto);
				report.setAttive_nuovi(attive_nuovi);
				report.setAttive_nuovi_auto(attive_nuovi_auto);
				report.setAttive_richiami(attive_richiami);
				report.setAttive_richiami_auto(attive_richiami_auto);
				report.setContatti_privati( ApplicationUtil.getPannello(mcPrivati, campagna.getId()).size());
				report.setAttive_privati( ApplicationUtil.getPannello(mcPrivatiAttiva, campagna.getId()).size());
				report.setContatti_archiviati( ApplicationUtil.getPannello(mcArchiviati, campagna.getId()).size());
				report.setAttive_archiviati( ApplicationUtil.getPannello(mcArchiviatiAttiva, campagna.getId()).size());
				campagna.setReport(report);
			}
		}
		return campagne;
	}
	public PannelloReport reportisticaCampagnaSemplice(String tipo, Boolean supervisore, Long userId, Long campagnaId) {
		List<Long> utentiPubblici = new ArrayList<Long>();
		List<Long> utentiPrivati = new ArrayList<Long>();
		if(supervisore) {
			List<Long> u  = getIdUtentiBySupervisore(userId);
			utentiPubblici.addAll(u);
			utentiPrivati .addAll(u);
		} else {
			List<Long> u  = getIdUtentiByOperatore(userId);
			utentiPubblici.addAll((u));
			utentiPrivati.add(userId);
		}
		PannelloReport cmpr = new PannelloReport();
		List<PannelloModel> d_a = Chiamate.getCodaContatti().get(0, utentiPubblici);
		List<PannelloModel> d_b = Chiamate.getCodaAutodialer().list(0);
		List<PannelloModel> listaPubblici =  getRichiamiPubbliciCampagnaList(tipo, campagnaId);
		List<PannelloModel> listaPersonali =  getRichiamiPersonaliCampagnaList(tipo, utentiPrivati, campagnaId);
		SortedMap<Long, ArrayList<PannelloModel>>  mappaListaPubblici = ApplicationUtil.collectionToHashPannello(listaPubblici, "getEsito_id");
		SortedMap<Long, ArrayList<PannelloModel>>  mappaListaPersonali = ApplicationUtil.collectionToHashPannello(listaPersonali, "getEsito_id");
		List<PannelloModel> listaNuovi = ApplicationUtil.getPannello(mappaListaPubblici, 0L);
		List<PannelloModel> listaRichiami = ApplicationUtil.getPannello(mappaListaPubblici, 1L);
		List<PannelloModel> listaPrivati = ApplicationUtil.getPannello(mappaListaPersonali, 0L);
		List<PannelloModel> listaArchiviati = ApplicationUtil.getPannello(mappaListaPersonali, 1L);
		List<PannelloModel> listaPrivatiAttiva =  ApplicationUtil.getAttive(listaPrivati, d_a);
		List<PannelloModel> listaArchiviatiAttiva =  ApplicationUtil.getAttive(listaArchiviati,  d_a);
		List<PannelloModel> listaNuoviAttiva =  ApplicationUtil.getAttive(listaNuovi,  d_a);
		List<PannelloModel> listaNuoviAutoAttiva =  ApplicationUtil.getAttive(listaNuovi,  d_b);
		List<PannelloModel> listaRichiamiAttiva =  ApplicationUtil.getAttive(listaRichiami, d_a);
		List<PannelloModel> listaRichiamiAutoAttiva =  ApplicationUtil.getAttive(listaRichiami,  d_b);
		if(ApplicationUtil.gtZero(campagnaId)) {
			Integer contatti_nuovi = listaNuovi.size();
			Integer attive_nuovi = listaNuoviAttiva.size();
			Integer attive_nuovi_auto = listaNuoviAutoAttiva.size();
			Integer contatti_richiami =  listaRichiami.size();
			Integer attive_richiami = listaRichiamiAttiva.size();
			Integer attive_richiami_auto = listaRichiamiAutoAttiva.size();
			cmpr.setContatti(contatti_nuovi + contatti_richiami);
			cmpr.setAttive(attive_nuovi + attive_richiami);
			cmpr.setAttive_auto(attive_nuovi_auto + attive_richiami_auto);
			cmpr.setContatti_nuovi(contatti_nuovi);
			cmpr.setAttive_nuovi(attive_nuovi);
			cmpr.setAttive_nuovi_auto(attive_nuovi_auto);
			cmpr.setContatti_richiami(contatti_richiami);
			cmpr.setAttive_richiami(attive_richiami);
			cmpr.setAttive_richiami_auto(attive_richiami_auto);
			cmpr.setContatti_privati(listaPrivati.size());
			cmpr.setAttive_privati(listaPrivatiAttiva.size());
			cmpr.setContatti_archiviati(listaArchiviati.size());
			cmpr.setAttive_archiviati(listaArchiviatiAttiva.size());
		}
		return cmpr;
	}
	public PannelloReport reportisticaCampagnaTutte(String tipo, Boolean supervisore, Long userId) {
		List<Long> utentiPubblici = new ArrayList<Long>();
		List<Long> utentiPrivati = new ArrayList<Long>();
		if(supervisore) {
			List<Long> u  = getIdUtentiBySupervisore(userId);
			utentiPubblici.addAll(u);
			utentiPrivati .addAll(u);
		} else {
			List<Long> u  = getIdUtentiByOperatore(userId);
			utentiPubblici.addAll((u));
			utentiPrivati.add(userId);
		}
		PannelloReport cmpr = new PannelloReport();
		List<PannelloModel> d_a = Chiamate.getCodaContatti().get(0, utentiPubblici);
		List<PannelloModel> d_b = Chiamate.getCodaAutodialer().list(0);
		List<PannelloModel> listaPubblici = getRichiamiPubbliciTutti(supervisore, tipo, userId);
		List<PannelloModel> listaPersonali =  getRichiamiPersonaliTutti(tipo, utentiPrivati);
		SortedMap<Long, ArrayList<PannelloModel>>  mappaListaPubblici = ApplicationUtil.collectionToHashPannello(listaPubblici, "getEsito_id");
		SortedMap<Long, ArrayList<PannelloModel>>  mappaListaPersonali = ApplicationUtil.collectionToHashPannello(listaPersonali, "getEsito_id");
		List<PannelloModel> listaNuovi = ApplicationUtil.getPannello(mappaListaPubblici, 0L);
		List<PannelloModel> listaRichiami = ApplicationUtil.getPannello(mappaListaPubblici, 1L);
		List<PannelloModel> listaPrivati = ApplicationUtil.getPannello(mappaListaPersonali, 0L);
		List<PannelloModel> listaArchiviati = ApplicationUtil.getPannello(mappaListaPersonali, 1L);
		List<PannelloModel> listaPrivatiAttiva =  ApplicationUtil.getAttive(listaPrivati, d_a);
		List<PannelloModel> listaArchiviatiAttiva =  ApplicationUtil.getAttive(listaArchiviati,  d_a);
		List<PannelloModel> listaNuoviAttiva =  ApplicationUtil.getAttive(listaNuovi,  d_a);
		List<PannelloModel> listaNuoviAutoAttiva =  ApplicationUtil.getAttive(listaNuovi,  d_b);
		List<PannelloModel> listaRichiamiAttiva =  ApplicationUtil.getAttive(listaRichiami, d_a);
		List<PannelloModel> listaRichiamiAutoAttiva =  ApplicationUtil.getAttive(listaRichiami,  d_b);
		Integer contatti_nuovi = listaNuovi.size();
		Integer attive_nuovi = listaNuoviAttiva.size();
		Integer attive_nuovi_auto = listaNuoviAutoAttiva.size();
		Integer contatti_richiami =  listaRichiami.size();
		Integer attive_richiami = listaRichiamiAttiva.size();
		Integer attive_richiami_auto = listaRichiamiAutoAttiva.size();
		cmpr.setContatti(contatti_nuovi + contatti_richiami);
		cmpr.setAttive(attive_nuovi + attive_richiami);
		cmpr.setAttive_auto(attive_nuovi_auto + attive_richiami_auto);
		cmpr.setContatti_nuovi(contatti_nuovi);
		cmpr.setAttive_nuovi(attive_nuovi);
		cmpr.setAttive_nuovi_auto(attive_nuovi_auto);
		cmpr.setContatti_richiami(contatti_richiami);
		cmpr.setAttive_richiami(attive_richiami);
		cmpr.setAttive_richiami_auto(attive_richiami_auto);
		cmpr.setContatti_privati(listaPrivati.size());
		cmpr.setAttive_privati(listaPrivatiAttiva.size());
		cmpr.setContatti_archiviati(listaArchiviati.size());
		cmpr.setAttive_archiviati(listaArchiviatiAttiva.size());
		return cmpr;
	}
	@SuppressWarnings("unchecked")
	public List<PannelloModel>  getRichiamiPubbliciTutti(boolean supervisore, String tipo, Long userId) {
		Date ora = ApplicationUtil.date();
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		impostazioneDao.subCriteriaCampagnaUser(supervisore, userId, campagnaC, "id");
		ApplicationUtil.criteriaStato(campagnaC, true, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
		}
		criteria.add(Restrictions.isNull("proprietario.id"));
		ApplicationUtil.archiviazione(criteria, ora, false);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("cont.id").as("contatto_id")))
				.add(Projections.sqlProjection("if(esitoChiamata_id = "+ ConstantsEsito.ID_DA_CHIAMARE + ", 0, 1) as esito_id", new String [] {"esito_id"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, PannelloModel.class);
	}
	@SuppressWarnings("unchecked")
	public List<PannelloModel> getRichiamiPersonaliTutti(String tipo, List<Long> utenti) {
		Date ora = ApplicationUtil.date();
		if(ApplicationUtil.empty(utenti)) {
			return new ArrayList<PannelloModel>();
		}
		Boolean ts = impostazioneDao.getEstrazioneStato();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		DetachedCriteria campagnaC = criteria.createCriteria("campagna");
		ApplicationUtil.criteriaStato(campagnaC, false, ApplicationUtil.isTrue(ts));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		criteria.add(Restrictions.in("proprietario.id", utenti));
		criteria.add(Restrictions.le("data_presentazione", ora));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("cont.id").as("contatto_id")))
				.add(Projections.sqlProjection("if(archiviato is false, 0, 1) as esito_id", new String [] {"esito_id"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, PannelloModel.class);
	}
	@SuppressWarnings("unchecked")
	public List<PannelloModel>  getRichiamiPubbliciCampagnaList(String tipo, Long campagna) {
		Date ora = ApplicationUtil.date();
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
		}
		criteria.createCriteria("campagna", "camp").add(Restrictions.idEq(campagna));
		criteria.add(Restrictions.isNull("proprietario.id"));
		ApplicationUtil.archiviazione(criteria, ora, false);
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("cont.id").as("contatto_id")))
				.add(Projections.sqlProjection("if(esitoChiamata_id = "+ ConstantsEsito.ID_DA_CHIAMARE + ", 0, 1) as esito_id", new String [] {"esito_id"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, PannelloModel.class);
	}
	@SuppressWarnings("unchecked")
	public List<PannelloModel> getRichiamiPersonaliCampagnaList(String tipo, List<Long> utenti, Long campagna) {
		Date ora = ApplicationUtil.date();
		if(ApplicationUtil.empty(utenti)) {
			return new ArrayList<PannelloModel>();
		}
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		criteria.add(Restrictions.le("data_presentazione", ora));
		criteria.add(Restrictions.eq("campagna.id",campagna));
		criteria.add(Restrictions.in("proprietario.id", utenti));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("cont.id").as("contatto_id")))
				.add(Projections.sqlProjection("if(archiviato is false, 0, 1) as esito_id", new String [] {"esito_id"}, new Type []{LongType.INSTANCE}))
				);
		return getList(criteria, PannelloModel.class);
	}
	@SuppressWarnings("unchecked")
	public List<PannelloModel>  getRichiamiPubbliciCampagne(String tipo, List<Long> campagnaId) {
		Date ora = ApplicationUtil.date();
		if(ApplicationUtil.empty(campagnaId)) {
			return new ArrayList<PannelloModel>();
		}
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, true, ApplicationUtil.isTrue(ts));
		}
		criteria.add(Restrictions.isNull("proprietario.id"));
		ApplicationUtil.archiviazione(criteria, ora, false);
		criteria.createCriteria("campagna", "camp").add(Restrictions.in("id", campagnaId));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("cont.id").as("contatto_id")))
				.add(Projections.sqlProjection("if(esitoChiamata_id = "+ ConstantsEsito.ID_DA_CHIAMARE + ", 0, 1) as esito_id", new String [] {"esito_id"}, new Type []{LongType.INSTANCE}))
				.add(Projections.property("camp.id").as("campagna_id")));
		return getList(criteria, PannelloModel.class);
	}
	@SuppressWarnings("unchecked")
	public List<PannelloModel>  getRichiamiPersonaliCampagne(String tipo, List<Long> campagnaId, List<Long> utentiId ) {
		Date ora = ApplicationUtil.date();
		if(ApplicationUtil.empty(campagnaId) || ApplicationUtil.empty(utentiId)) {
			return new ArrayList<PannelloModel>();
		}
		DetachedCriteria criteria = DetachedCriteria.forClass(Contatto.class, "cont");
		criteria.add(Restrictions.eq("tipo", ConstantsGlobal.inviato));
		if(ApplicationUtil.isNotBlank(tipo)) {
			criteria.createCriteria("anagrafica").add(ApplicationUtil.isNotNull(tipo));
		}
		Boolean s = impostazioneDao.getEstrazioneListe();
		if(ApplicationUtil.isTrue(s)) {
			Boolean ts = impostazioneDao.getEstrazioneStato();
			DetachedCriteria listaC = criteria.createCriteria("lista");
			ApplicationUtil.criteriaStato(listaC, false, ApplicationUtil.isTrue(ts));
		}
		criteria.add(Restrictions.in("proprietario.id", utentiId));
		criteria.add(Restrictions.le("data_presentazione", ora));
		criteria.createCriteria("campagna", "camp").add(Restrictions.in("id",campagnaId));
		criteria.setProjection(Projections.projectionList()
				.add(Projections.distinct(Projections.property("cont.id").as("contatto_id")))
				.add(Projections.sqlProjection("if(archiviato is false, 0, 1) as esito_id", new String [] {"esito_id"}, new Type []{LongType.INSTANCE}))
				.add(Projections.property("camp.id").as("campagna_id")));
		return getList(criteria, PannelloModel.class);
	}
	public int trovaCampagnePannelloUtenteSize(CampagnaPannelloCriteria cmcr, boolean semaforo, Long userId) {
		DetachedCriteria criteria = trovaCampagneAttiveCriteria(cmcr, semaforo, userId);
		return getListSize(criteria);
	}
	public PulisciUtil getListeSI(List<ContattoSI> contatti) {
		List<ListaSI> lis = new ArrayList<ListaSI>();
		List<EsitoChiamataSI> eis = new ArrayList<EsitoChiamataSI>();
		List<EsitoArchiviazioneSI> ais = new ArrayList<EsitoArchiviazioneSI>();
		List<Long> lid = new ArrayList<Long>();
		List<Long> eid = new ArrayList<Long>();
		List<Long> aid = new ArrayList<Long>();
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<ContattoSI> objIter = contatti.iterator();
			while(objIter.hasNext()) {
				ContattoSI c = objIter.next();
				if(c.getLista_id() != null && !lid.contains(c.getLista_id())) {
					lid.add(c.getLista_id());
					lis.add(new ListaSI(c));
				}
				if(c.getEsito_chiamata_id() != null && !eid.contains(c.getEsito_chiamata_id())) {
					eid.add(c.getEsito_chiamata_id());
					eis.add(new EsitoChiamataSI(c));
				}
				if(c.getEsito_archiviazione_id() != null && !aid.contains(c.getEsito_archiviazione_id())) {
					aid.add(c.getEsito_archiviazione_id());
					ais.add(new EsitoArchiviazioneSI(c));
				}
			}
		}
		PulisciUtil pu = new PulisciUtil();
		pu.setElenco(impostazioneDao.getLista(lis, false, null));
		pu.setEsiti_chiamata(impostazioneDao.getEsitoChiamata(eis, false, null));
		pu.setEsiti_archiviazione(impostazioneDao.getEsitoArchiviazione(ais, false, null));
		return	pu;
	}

	public List<ContattoSI> getFiltraLista(List<ContattoSI> contatti, Long listaId, Long esitoId, Long archiviazioneId, Boolean inverti) {
		List<ContattoSI> filtrati = new ArrayList<ContattoSI>();
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<ContattoSI> itCon = contatti.iterator();
			while (itCon.hasNext()) {
				ContattoSI contatto = (ContattoSI) itCon.next();
				boolean listaOk = (ApplicationUtil.ltZero(listaId) || ApplicationUtil.eq(contatto.getLista_id(), listaId));
				boolean esitoOk = (ApplicationUtil.ltZero(esitoId) || ApplicationUtil.eq(contatto.getEsito_chiamata_id(), esitoId));
				boolean archiviazioneOK = (ApplicationUtil.ltZero(archiviazioneId) || ApplicationUtil.eq(contatto.getEsito_archiviazione_id(), archiviazioneId));
				boolean ok = listaOk && esitoOk && archiviazioneOK;
				if(ApplicationUtil.isTrue(inverti)? !ok : ok) {
					filtrati.add(contatto);
				}
			}

		}
		return filtrati;
	}
	public List<ContattoSI> getFiltraCampagna(List<ContattoSI> contatti, Long campagnaId, Long esitoId, Long archiviazioneId, Boolean inverti) {
		List<ContattoSI> filtrati = new ArrayList<ContattoSI>();
		if(ApplicationUtil.notEmpty(contatti)) {
			Iterator<ContattoSI> itCon = contatti.iterator();
			while (itCon.hasNext()) {
				ContattoSI contatto = (ContattoSI) itCon.next();
				boolean campagnaOk = (ApplicationUtil.ltZero(campagnaId) || ApplicationUtil.eq(contatto.getCampagna_id(), campagnaId));
				boolean esitoOk = (ApplicationUtil.ltZero(esitoId) || ApplicationUtil.eq(contatto.getEsito_chiamata_id(), esitoId));
				boolean archiviazioneOK = (ApplicationUtil.ltZero(archiviazioneId) || ApplicationUtil.eq(contatto.getEsito_archiviazione_id(), archiviazioneId));
				boolean ok = campagnaOk && esitoOk && archiviazioneOK;
				if(ApplicationUtil.isTrue(inverti)? !ok : ok) {
					filtrati.add(contatto);
				}
			}
		}
		return filtrati;
	}

	public void creaContattiInCampagna(Lista lista, Campagna campagna, Long  esitoChiamata) {
		Date data = ApplicationUtil.date();
		String p = ConstantsGlobal.tp;
		String orderBy = "";
		String ordine = ApplicationUtil.trimToEmpty(lista.getOrdine());
		if(ApplicationUtil.isNotBlank(ordine)) {
			if(ApplicationUtil.equals(ConstantsGlobal.randomize, ordine)) {
				orderBy = "order by RAND()";
			} else {
				String [] ords = ordine.split(ConstantsGlobal.colonSeparator);
				if(!ords[0].startsWith(ConstantsMap.indirizzo)) {
					if(ords[0].startsWith(ConstantsMap.anagrafica)) {
						String ord = ords[0].substring((ConstantsMap.anagrafica + "_").length());
						orderBy = "order by an." + ord + " " + ords[1];
					} else {
						orderBy = "order by an." + ords[0] + " " + ords[1];
					}
				} else {
					if(ords[0].startsWith(ConstantsMap.indirizzo_legale)) {
						String tipo_indirizzo = ConstantsMap.indirizzo_legale;
						String ord = ords[0].substring((tipo_indirizzo + "_").length());
						if(!ord.equals(ConstantsMap.geocodifica)) {
							orderBy = "join " + p +"indirizzo ii on an."+tipo_indirizzo+"_id = ii.id order by ii." + ord + " " + ords[1];
						} else {
							String lat = lista.getLatitudine();
							String lon = lista.getLongitudine();
							orderBy = "join " + p +"indirizzo ii on an."+tipo_indirizzo+"_id = ii.id order by " + ApplicationUtil.ordineCriteria("ii" + ".", lat, lon) + " " + ords[1];
						}
					}
					if(ords[0].startsWith(ConstantsMap.indirizzo_operativo)) {
						String tipo_indirizzo = ConstantsMap.indirizzo_operativo;
						String ord = ords[0].substring((tipo_indirizzo + "_").length());
						if(!ord.equals(ConstantsMap.geocodifica)) {
							orderBy = "join " + p +"indirizzo ii on an."+tipo_indirizzo+"_id = ii.id order by ii." + ord + " " + ords[1];
						} else {
							String lat = lista.getLatitudine();
							String lon = lista.getLongitudine();
							orderBy = "join " + p +"indirizzo ii on an."+tipo_indirizzo+"_id = ii.id order by " + ApplicationUtil.ordineCriteria("ii" + ".", lat, lon) + " " + ords[1];
						}
					}
				}
			}
		}
		String select = ConstantsGlobal.select + " distinct " + "an.id, co.lista_id, :campagna, :esitoChiamata, :data_creazione, :data_presentazione, :tipo, :archiviato from " + p +"lista_anagrafica co join " + p +"anagrafica an on co.contatti_id = an.id and co.lista_id = :lista " + orderBy;
		String hql = "insert into " + p +"contatto (anagrafica_id, lista_id, campagna_id, esitoChiamata_id, data_creazione, data_presentazione, tipo, archiviato)" + " " + select;
		execute(true, hql, new Parameter("lista", lista.getId()), new Parameter("campagna", campagna.getId()), new Parameter("esitoChiamata", esitoChiamata), new Parameter("data_creazione", data), new Parameter("data_presentazione", data), new Parameter("tipo", ConstantsGlobal.inviato), new Parameter("archiviato", false));

	}
	public void creaContattiPerListaInCampagna(List<Anagrafica> anagrafiche, Lista nlista, Campagna campagna, EsitoChiamata dachiamare) {
		if(ApplicationUtil.notEmpty(anagrafiche)) {
			Date data = ApplicationUtil.date();
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Anagrafica> objIter = anagrafiche.iterator();
			while(objIter.hasNext()) {
				Anagrafica anagrafica = objIter.next();
				if(anagrafica.getId() != null) {
					Contatto contatto = new Contatto();
					contatto.setAnagrafica(anagrafica);
					contatto.setCampagna(campagna);
					contatto.setLista(nlista);
					contatto.imposta(data, dachiamare, null);
					session.save(contatto);
					if (i++ % ConstantsGlobal.flushSize == 0) {
						session.flush();
						session.clear();
					}
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitiInlineManager(Contatto contatto, boolean inline) {
		if(contatto!= null && inline) {
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			if(!contatto.isOk()) {
				criteria.add(Restrictions.eq("solo_archiviati", Boolean.FALSE));
			}
			criteria.createCriteria("campagne").add(Restrictions.eq("id",contatto.getCampagna().getId()));
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		} else return null;
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitiInlineBackoffice(Contatto contatto, boolean inline) {
		if(contatto!= null && inline) {
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			if(!contatto.isOk()) {
				criteria.add(Restrictions.eq("solo_archiviati", Boolean.FALSE));
			}
			criteria.add(Restrictions.eq("solo_backoffice", Boolean.TRUE));
			criteria.createCriteria("campagne").add(Restrictions.eq("id",contatto.getCampagna().getId()));
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		} else return null;
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitiInlineSupervisore(Contatto contatto, boolean inline) {
		if(contatto!= null && inline) {
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			if(!contatto.isOk()) {
				criteria.add(Restrictions.eq("solo_archiviati", Boolean.FALSE));
			}
			criteria.add(Restrictions.eq("solo_supervisore", Boolean.TRUE));
			criteria.createCriteria("campagne").add(Restrictions.eq("id",contatto.getCampagna().getId()));
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		} else return null;
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitiInlineOperatore(Contatto contatto, boolean inline) {
		if(contatto!= null && inline) {
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			if(!contatto.isOk()) {
				criteria.add(Restrictions.eq("solo_archiviati", Boolean.FALSE));
			}
			criteria.add(Restrictions.eq("solo_operatore", Boolean.TRUE));
			criteria.createCriteria("campagne").add(Restrictions.eq("id",contatto.getCampagna().getId()));
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		} else return null;
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> getEsitiInlinePannello(Boolean supervisore, Contatto contatto, boolean inline) {
		if(contatto!= null && inline) {
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
			if(!contatto.isOk()) {
				criteria.add(Restrictions.eq("solo_archiviati", Boolean.FALSE));
			}
			if(ApplicationUtil.isTrue(supervisore)) {
				criteria.add(Restrictions.eq("solo_supervisore", Boolean.TRUE));
			} else {
				criteria.add(Restrictions.eq("solo_operatore", Boolean.TRUE));
			}
			criteria.createCriteria("campagne").add(Restrictions.eq("id",contatto.getCampagna().getId()));
			ApplicationUtil.orderEsito(criteria);
			return getList(criteria);
		} else return null;
	}
	public List<SelectItem> getEsitiArchiviazioneByCampagnaStoricoSI(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		if(ApplicationUtil.gtZero(campagnaId)) {
			criteria.createCriteria("campagne").add(Restrictions.eq("id",campagnaId));
		}
		List<EsitoArchiviazioneSI> lis = getEsitoArchiviazioneSI(criteria);
		return impostazioneDao.getEsitoArchiviazione(lis, false, null);
	}
	public EsitoChiamata getEsitoChiamata(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.idEq(id));
		return (EsitoChiamata) getUniqueResult(criteria);
	}
	public EsitoArchiviazione getEsitoArchiviazione(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(id));
		return (EsitoArchiviazione) getUniqueResult(criteria);
	}
	public Utente getOperatore(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(id));
		return (Utente) getUniqueResult(criteria);
	}
	public Utente getOperatore(Long id, Utente utente) {
		if(ApplicationUtil.gtZero(id)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
			criteria.add(Restrictions.idEq(id));
			Utente u = (Utente) getUniqueResult(criteria);
			if(u != null) {
				return u;
			}
		}
		return utente;
	}
	@SuppressWarnings("unchecked")
	public List<Long> getProdottiIdByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ProdottiInVendita.class);
		criteria.add(Restrictions.eq("id.campagna.id", campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id.prodotto.id")));
		List<Long> id =  getList(criteria);
		return id != null ? id : new ArrayList<Long>();
	}
	@SuppressWarnings("unchecked")
	public List<ProdottiInVendita> trovaProdottiInVendita(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(ProdottiInVendita.class);
		criteria.add(Restrictions.eq("id.campagna.id", campagnaId));
		return getList(criteria);
	}
	public void removeAssociationProdotto(Long campagnaId, Long prodottoId) {
		String hql = "delete from ProdottiInVendita where id.prodotto.id = :prodotto and id.campagna.id = :campagna";
		execute(false, hql, new Parameter("campagna", campagnaId), new Parameter("prodotto", prodottoId));
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> trovaEsito(Long camp_id) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteriaU.createCriteria("campagne").add(Restrictions.idEq(camp_id));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByEsitoId(Long esito_id) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("esiti_archiviazione").add(Restrictions.idEq(esito_id));
		ApplicationUtil.orderCL(criteriaU);
		return getList(criteriaU);
	}
	@SuppressWarnings("unchecked")
	public List<Zona> getZonaByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderZ(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> getEmailByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Agente> getTuttiAgenteByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderA(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUserSupervisoreByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("teams").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getTuttiSupervisoreAgenteByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getUserByCampagnaId(Long campagnaId){
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("team").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	public List<SelectItem> getUtentiInCampagnaSI(Long campagnaId, Utente utente){
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.createCriteria("team").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, utente);
	}
	public List<SelectItem> getUtentiInCampagnaSITeam(Campagna campagna, Utente user, Boolean supervisore) {
		if(ApplicationUtil.isTrue(supervisore)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
			DetachedCriteria critT = criteria.createCriteria("team");
			critT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
			critT.createCriteria("campagne").add(Restrictions.idEq(campagna.getId()));
			List<UtenteSI> lis = getUtenteSI(criteria);
			return impostazioneDao.getUtente(lis, false, null);
		} else {
			if(ApplicationUtil.isTrue(campagna.getRiprogramma_team())) {
				DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
				DetachedCriteria critT = criteria.createCriteria("team");
				critT.createCriteria("user").add(Restrictions.idEq(user.getId()));
				critT.createCriteria("campagne").add(Restrictions.idEq(campagna.getId()));
				List<UtenteSI> lis = getUtenteSI(criteria);
				return impostazioneDao.getUtente(lis, false, user);
			} else {
				List<UtenteSI> lis = new ArrayList<UtenteSI>();
				return impostazioneDao.getUtente(lis, false, user);
			}
		}
	}
	
	public List<ListaSI> getListeSelezionateSI(Campagna campagna, Utente user) {
		return CodaStatic.getCodaLista().getData(campagna, user);
	}
	
	public void setListeSelezionateSI(List<ListaSI> liste, Campagna campagna, Utente user) {
		CodaStatic.getCodaLista().addData(campagna, user, liste);
	}
	
	public List<ListaSI> getListeInCampagnaSI(Campagna campagna, Utente user) {
		List<ListaSI> lis = new ArrayList<ListaSI>();
		if(campagna != null && user != null) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
			DetachedCriteria campagnaC = criteria.createCriteria("campagna");
			campagnaC.add(Restrictions.idEq(campagna.getId()));
			List<Long> ids = CodaStatic.getCodaLista().getDataL(campagna, user);
			if(ApplicationUtil.notEmpty(ids)) {
				criteria.add(Restrictions.not(Restrictions.in("id",ids)));
			}
			Boolean s = impostazioneDao.getEstrazioneListe();
			if(ApplicationUtil.isTrue(s)) {
				Boolean ts = impostazioneDao.getEstrazioneStato();
				ApplicationUtil.criteriaStato(criteria, true, ApplicationUtil.isTrue(ts));
			}
			lis.addAll(getListaSI(criteria));
		}
		return lis;
	}
	public List<SelectItem> getUtentiInCampagnaSI(Long campagnaId, Utente user, Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria critT = criteria.createCriteria("team");
		critT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		critT.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, utente);
	} 
	public List<SelectItem> getUtentiInCampagnaSIPartner(Long campagnaId, Utente user, Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria critC = criteria.createCriteria("team").createCriteria("campagne");
		critC.add(Restrictions.idEq(campagnaId));
		critC.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, utente);
	} 
	@SuppressWarnings("unchecked")
	public List<Zona> trovaZoneTutte(Long agente) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Zona.class);
		criteriaC.createCriteria("agente").add(Restrictions.idEq(agente));
		criteriaC.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaC, "id");
		ApplicationUtil.orderZ(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaAgentiTutte(Long agente) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Utente.class);
		criteriaC.createCriteria("agenti").add(Restrictions.idEq(agente));
		ApplicationUtil.orderU(criteriaC);
		return getList(criteriaC);
	}
	public void removeListaFromCampagna(Campagna campagna, Lista lista) {
		if (campagna.getListe().contains(lista)) {
			campagna.getListe().remove(lista);
		}
	}
	public void removeListeFromCampagna(Campagna campagna, List<Lista> liste) {
		campagna.getListe().removeAll(liste);
	}
	public Campagna addListaToCampagna(Campagna campagna, List<Lista> listas) {
		if(ApplicationUtil.notEmpty(listas)) {
			Iterator<Lista> objIter = listas.iterator();
			while(objIter.hasNext()) {
				Lista lista = objIter.next();
				if (!campagna.getListe().contains(lista)) {
					campagna.getListe().add(lista);
				}
			}
		}
		return campagna;
	}
	public Campagna addListaToCampagna(Campagna campagna, Lista lista) {
		if (!campagna.getListe().contains(lista)) {
			campagna.getListe().add(lista);
		}
		return campagna;
	}
	public void addListaToCampagnaId(Long campagnaId, List<Lista> listas) {
		if(ApplicationUtil.gtZero(campagnaId)) {
			Campagna campagna = getFullCampagna(campagnaId);
			if (campagna != null && ApplicationUtil.notEmpty(listas)) {
				Iterator<Lista> objIter = listas.iterator();
				while(objIter.hasNext()) {
					Lista lista = objIter.next();
					if (!campagna.getListe().contains(lista)) {
						campagna.getListe().add(lista);
					}
					creaContattiInCampagna(lista, campagna, ConstantsEsito.ID_DA_CHIAMARE);
				}
				salva(campagna);
			}
		}
	}
	public Campagna addListaToCampagnaCopia(Campagna campagna, Lista listaOld) {
		if(listaOld != null) {
			Lista lista = getFullListaContatti(listaOld.getId());
			Lista copia =  lista.getCopia();
			List<Anagrafica> liste = new ArrayList<Anagrafica>();
			List<Anagrafica> cListe = lista.getContatti();
			if(ApplicationUtil.notEmpty(cListe)) {
				Iterator<Anagrafica> objIter = cListe.iterator();
				while(objIter.hasNext()) {
					Anagrafica ana = objIter.next();
					liste.add(ana);
				}
				copia.setContatti(liste);
			}
			salva(copia);
			if(ApplicationUtil.gtZero(copia.getId())) {
				copia = getLista(copia.getId());
			}
			if (!campagna.getListe().contains(copia)) {
				campagna.getListe().add(copia);
			}
			if(ApplicationUtil.gtZero(campagna.getId())) {
				aggiornaLista(campagna, copia);
				campagna = getFullCampagnaT(campagna.getId());
			}
		}
		return campagna;
	}
	public Campagna addListeToCampagnaCopia(Campagna campagna, List<Lista> listeOld) {
		if(ApplicationUtil.notEmpty(listeOld)) {
			Iterator<Lista> listeIt = listeOld.iterator();
			while (listeIt.hasNext()) {
				Lista lista = getFullListaContatti(listeIt.next().getId());
				Lista copia =  lista.getCopia();
				List<Anagrafica> liste = new ArrayList<Anagrafica>();
				List<Anagrafica> cListe = lista.getContatti();
				if(ApplicationUtil.notEmpty(cListe)) {
					Iterator<Anagrafica> objIter = cListe.iterator();
					while(objIter.hasNext()) {
						Anagrafica ana = objIter.next();
						liste.add(ana);
					}
					copia.setContatti(liste);
				}
				salva(copia);
				if(ApplicationUtil.gtZero(copia.getId())) {
					copia = getLista(copia.getId());
				}
				if (!campagna.getListe().contains(copia)) {
					campagna.getListe().add(copia);
				}
				if(ApplicationUtil.gtZero(campagna.getId())) {
					aggiornaLista(campagna, copia);
				}
			}
			if(ApplicationUtil.gtZero(campagna.getId())) {
				campagna = getFullCampagnaT(campagna.getId());
			}
		}
		return campagna;
	}
	public Campagna addListeToCampagna(Campagna campagna, List<Lista> lista) {
		campagna.getListe().addAll(lista);
		return campagna;
	}
	public void addListaToCampagnaSalva(Campagna campagna, Lista lista) {
		if (!campagna.getListe().contains(lista)) {
			campagna.getListe().add(lista);
			salva(campagna);
		}
	}
	public void removeListaFromCampagnaSalva(Campagna campagna, Lista lista) {
		if (campagna.getListe().contains(lista)) {
			campagna.getListe().remove(lista);
			salva(campagna);
		}
	}
	public Lista aggiornaListe(List<Campagna> campagne, Lista lista) {
		if(ApplicationUtil.notEmpty(campagne)) {
			Iterator<Campagna> objIter = campagne.iterator();
			while(objIter.hasNext()) {
				Campagna campagna = objIter.next();
				creaContattiInCampagna(lista, campagna, ConstantsEsito.ID_DA_CHIAMARE);
			}
		}
		return lista;
	}
	public Campagna aggiornaListe(Campagna campagna, List<Lista> listeAggiunte) {
		if(campagna != null) {
			Iterator<Lista> objIter = listeAggiunte.iterator();
			while(objIter.hasNext()) {
				Lista lista = objIter.next();
				creaContattiInCampagna(lista, campagna, ConstantsEsito.ID_DA_CHIAMARE);
			}
			salva(campagna);
		}
		return campagna;
	}
	public Campagna aggiornaLista(Campagna campagna, Lista lista) {
		if(campagna != null) {
			creaContattiInCampagna(lista, campagna, ConstantsEsito.ID_DA_CHIAMARE);
			salva(campagna);
		}
		return campagna;
	}
	public Campagna aggiornaLista(Campagna campagna, List<Lista> listas) {
		if(campagna != null && ApplicationUtil.notEmpty(listas)) {
			Iterator<Lista> objIter = listas.iterator();
			while(objIter.hasNext()) {
				Lista lista = objIter.next();
				creaContattiInCampagna(lista, campagna, ConstantsEsito.ID_DA_CHIAMARE);
			}
			salva(campagna);
		}
		return campagna;
	}
	public Campagna salva(Campagna campagna) {
		campagna = (Campagna) salvaAggiorna(campagna);
		return campagna;
	}
	public void aggiungiEsitoACampagna(EsitoArchiviazione esito, Long campagnaId) {
		Campagna campagna = getCampagna(campagnaId);
		if(esito.getCampagne() == null)
			esito.setCampagne(new ArrayList<Campagna>());
		if(!esito.getCampagne().contains(campagna)) {
			esito.getCampagne().add(campagna);
			salvaEsito(esito);
		}
	}
	public void eliminaEsitoArchiviazione(EsitoArchiviazione esito) {
		cancellaAggiorna(esito);
	}
	public Campagna getCampagna(Long id){
		return (Campagna) get(Campagna.class, id);
	}
	public Campagna getCampagnaTeam(Long id){
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Campagna.class);
		criteriaC.add(Restrictions.idEq(id));
		criteriaC.setFetchMode("team", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteriaC);
	}
	public Campagna getCampagnaPartner(Long id){
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Campagna.class);
		criteriaC.add(Restrictions.idEq(id));
		criteriaC.setFetchMode("partner", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteriaC);
	}
	public Campagna getCampagnaZona(Long id){
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Campagna.class);
		criteriaC.add(Restrictions.idEq(id));
		criteriaC.setFetchMode("zone", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteriaC);
	}
	public Campagna getCampagnaEmail(Long id){
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Campagna.class);
		criteriaC.add(Restrictions.idEq(id));
		criteriaC.setFetchMode("email", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteriaC);
	}
	public Campagna getCampagnaEsito(Long id){
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Campagna.class);
		criteriaC.add(Restrictions.idEq(id));
		criteriaC.setFetchMode("esiti_archiviazione", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteriaC);
	}
	public void salvaListe(List<Lista> ls) {
		if(ApplicationUtil.notEmpty(ls)) {
			Iterator<Lista> it = ls.iterator();
			while (it.hasNext()) {
				Lista lista = (Lista) it.next();
				salvaAggiorna(lista);
			}
		}
	}
	public Campagna salvaNuova(Campagna campagna) {
		if(campagna != null) {
			campagna = salva(campagna);
			List<ProdottiInVendita> prd = campagna.getProdottiDaAggiungere();
			if(ApplicationUtil.notEmpty(prd)) {
				for (ProdottiInVendita p : prd) {
					aggiungiCampagnaAdProdotto(campagna, p.getId().getProdotto());
				}
			}
			List<Lista> liste = campagna.getListe();
			if(ApplicationUtil.notEmpty(liste)) {
				Iterator<Lista> objIter = liste.iterator();
				while(objIter.hasNext()) {
					Lista lista = objIter.next();
					salva(lista);
					creaContattiInCampagna(lista, campagna, ConstantsEsito.ID_DA_CHIAMARE);
				}
			}
		}
		return campagna;
	}
	public Campagna salvaNuovaModifica(Campagna campagna) {
		if(campagna != null) {
			campagna = salva(campagna);
		}
		return campagna;
	}
	public EsitoArchiviazione getNuovoEsito() {
		return new EsitoArchiviazione();
	}
	public Campagna copiaCampagna(Long campagnaId) {
		Campagna campagna = getFullCampagnaT(campagnaId);
		Campagna copia =  campagna.getCopia();
		return copia;
	}
	private Campagna copiaCampagna(Long campagnaId, Long utente, Long partner, boolean listaB) {
		if(ApplicationUtil.gtZero(campagnaId)) {
			Campagna campagna = getFullCampagnaT(campagnaId);
			Campagna copia =  campagna.getCopia();
			List<Lista> liste = new ArrayList<Lista>();
			List<Lista> cListe = campagna.getListe();
			if(listaB) {
				if(ApplicationUtil.notEmpty(cListe)) {
					Iterator<Lista> objIter = cListe.iterator();
					while(objIter.hasNext()) {
						Lista lista = objIter.next();
						liste.add(lista);
					}
					copia.setListe(liste);
				}
			}
			copia.setTeam(trovaTeamPerCampagna(campagnaId, utente));
			copia.setPartner(trovaPartnerPerCampagna(campagnaId, partner));
			copia.setZone(trovaZonaPerCampagna(campagnaId));
			copia.setEmail(trovaEmailPerCampagna(campagnaId));
			copia.setRmail(trovaRmailPerCampagna(campagnaId));
			copia.setTelecom(trovaTelecomPerCampagna(campagnaId));
			copia.setRisposte(trovaRispostaPerCampagna(campagnaId));
			copia.setEsiti_archiviazione(trovaEsitoArchiviazionePerCampagna(campagnaId));
			copia.setProdottiDaAggiungere(trovaProdottiInVendita(campagnaId));
			return copia;
		} else return null;
	}
	public Lista copiaLista(Long listaId) {
		if(ApplicationUtil.gtZero(listaId)) {
			Lista lista = getFullListaContatti(listaId);
			Lista copia =  lista.getCopia();
			List<Anagrafica> liste = new ArrayList<Anagrafica>();
			List<Anagrafica> cListe = lista.getContatti();
			if(ApplicationUtil.notEmpty(cListe)) {
				Iterator<Anagrafica> objIter = cListe.iterator();
				while(objIter.hasNext()) {
					Anagrafica ana = objIter.next();
					liste.add(ana);
				}
				copia.setContatti(liste);
			}
			return copia;
		} else return null;
	}
	@SuppressWarnings("unchecked")
	public List<Team> trovaTeamPerCampagna(Long campagnaId, Long utente) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Team.class);
		criteriaC.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		if(utente != null) {
			criteriaC.createCriteria("supervisore").add(Restrictions.idEq(utente));
		}
		ApplicationUtil.orderT(criteriaC);
		return getList(criteriaC);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> trovaPartnerPerCampagna(Long campagnaId, Long utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagne");
		criteriaC.add(Restrictions.idEq(campagnaId));
		if(utente != null) {
			criteriaC.createCriteria("partner").add(Restrictions.idEq(utente));
		}
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Zona> trovaZonaPerCampagna(Long campagnaId) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Zona.class);
		criteriaC.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderZ(criteriaC);
		return getList(criteriaC);
	}
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> trovaEmailPerCampagna(Long campagnaId) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(EmailTemplate.class);
		criteriaC.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		return getList(criteriaC);
	}
	
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> trovaEsitoArchiviazionePerCampagna(Long campagnaId) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteriaC.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		ApplicationUtil.orderEsito(criteriaC);
		return getList(criteriaC);
	}
	public Campagna copiaCampagnaConListe(Long campagnaId) {
		Campagna campagna = getFullCampagnaT(campagnaId);
		Campagna copia =  campagna.getCopia();
		List<Lista> liste = new ArrayList<Lista>();
		List<Lista> cListe = campagna.getListe();
		if(ApplicationUtil.notEmpty(cListe)) {
			Iterator<Lista> objIter = cListe.iterator();
			while(objIter.hasNext()) {
				Lista lista = objIter.next();
				liste.add(lista);
			}
			copia.setListe(liste);
		}
		return copia; 
	}
	public Campagna copiaCampagnaSenzaListe(Long campagnaId) {
		Campagna campagna = getFullCampagnaT(campagnaId);
		Campagna copia =  campagna.getCopia();
		return copia; 
	}
	public EsitoArchiviazione getFullEsito(Long esitoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		criteria.add(Restrictions.idEq(esitoId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return  (EsitoArchiviazione) getUniqueResult(criteria);
	}
	public void copiaEsiti(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Long> esiti = getEsitiIdByCampagna(campagnaId_src);
		Iterator<Long> objIter = esiti.iterator();
		while(objIter.hasNext()) {
			Long agenteId = objIter.next();
			EsitoArchiviazione esito = getFullEsito(agenteId);
			if(esito.getCampagne() == null)
				esito.setCampagne(new ArrayList<Campagna>());
			if(!esito.getCampagne().contains(campagna)) {
				esito.getCampagne().add(campagna);
				salvaEsito(esito);
			}
		}
	}
	public void copiaZone(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Long> zone = getZoneByCampagna(campagnaId_src);
		Iterator<Long> objIter = zone.iterator();
		while(objIter.hasNext()) {
			Long agenteId = objIter.next();
			Zona zona = getFullZonaCampagna(agenteId);
			if(zona.getCampagne() == null)
				zona.setCampagne(new ArrayList<Campagna>());
			if(!zona.getCampagne().contains(campagna)) {
				zona.getCampagne().add(campagna);
				salvaZona(zona);
			}
		}
	}
	public void copiaEmail(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Long> emails = getEmailsByCampagna(campagnaId_src);
		Iterator<Long> objIter = emails.iterator();
		while(objIter.hasNext()) {
			Long emailId = objIter.next();
			EmailTemplate email = getFullEmailConCampagne(emailId);
			if(email.getCampagne() == null)
				email.setCampagne(new ArrayList<Campagna>());
			if(!email.getCampagne().contains(campagna)) {
				email.getCampagne().add(campagna);
				salvaEmail(email);
			}
		}
	}
	
	public void copiaTeam(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Long> teams = userDao.getTeamByCampagna(campagnaId_src);
		Iterator<Long> objIter = teams.iterator();
		while(objIter.hasNext()) {
			Long teamId = objIter.next();
			Team team = userDao.getTeamCampagne(teamId);
			if(team.getCampagne() == null)
				team.setCampagne(new ArrayList<Campagna>());
			if(!team.getCampagne().contains(campagna)) {
				team.getCampagne().add(campagna);
				userDao.salvaTeam(team);
			}
		}
	}
	public void copiaPartner(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Long> partners = userDao.getPartnerByCampagna(campagnaId_src);
		Iterator<Long> objIter = partners.iterator();
		while(objIter.hasNext()) {
			Long partnerId = objIter.next();
			Utente partner = userDao.getPartnerCampagne(partnerId);
			if(partner.getCampagne() == null)
				partner.setCampagne(new ArrayList<Campagna>());
			if(!partner.getCampagne().contains(campagna)) {
				partner.getCampagne().add(campagna);
				userDao.salvaUser(partner);
			}
		}
	}
	public Campagna getNuovaCampagna() {
		return new Campagna();
	}
	public Lista getNuovaLista() {
		return new Lista();
	}
	public void aggiungiTeamACampagna(Long campagnaId, Long teamId) {
		Campagna campagna = getCampagna(campagnaId);
		Team team = userDao.getTeamCampagne(teamId);
		if(team != null) {
			if(team.getCampagne() == null)
				team.setCampagne(new ArrayList<Campagna>());
			if(!team.getCampagne().contains(campagna)) {
				team.getCampagne().add(campagna);
				userDao.salvaTeam(team);
			}
		}
	}
	public void eliminaTeamDaCampagna(Long campagnaId, Long teamId) {
		Campagna campagna = getCampagna(campagnaId);
		Team team = userDao.getTeamCampagne(teamId);
		if(team.getCampagne() == null)
			team.setCampagne(new ArrayList<Campagna>());
		if(team.getCampagne().contains(campagna)) {
			team.getCampagne().remove(campagna);
			userDao.salvaTeam(team);
		}
	}
	public void aggiungiTeamTuttiACampagna(Long campagnaId) {
		Campagna campagna = getCampagnaTeam(campagnaId);
		List<Team> team = userDao.trovaTeam(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(team)) {
			if(campagna.getTeam() == null)
				campagna.setTeam(new ArrayList<Team>());
			campagna.getTeam().addAll(team);
			salva(campagna);
		}
	}
	public void eliminaTeamTuttiDaCampagna(Long campagnaId) {
		Campagna campagna = getCampagnaTeam(campagnaId);
		List<Team> team = getTeamByCampagnaId(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(team)) {
			if(campagna.getTeam() == null)
				campagna.setTeam(new ArrayList<Team>());
			campagna.getTeam().removeAll(team);
			salva(campagna);
		}
	}
	public void aggiungiPartnerACampagna(Long campagnaId, Long partnerId) {
		Campagna campagna = getCampagna(campagnaId);
		Utente partner = userDao.getPartnerCampagne(partnerId);
		if(partner != null) {
			if(partner.getCampagne() == null)
				partner.setCampagne(new ArrayList<Campagna>());
			if(!partner.getCampagne().contains(campagna)) {
				partner.getCampagne().add(campagna);
				userDao.salvaUser(partner);
			}
		}
	}
	public void eliminaPartnerDaCampagna(Long campagnaId, Long partnerId) {
		Campagna campagna = getCampagna(campagnaId);
		Utente partner = userDao.getPartnerCampagne(partnerId);
		if(partner.getCampagne() == null)
			partner.setCampagne(new ArrayList<Campagna>());
		if(partner.getCampagne().contains(campagna)) {
			partner.getCampagne().remove(campagna);
			userDao.salvaUser(partner);
		}
	}
	public void aggiungiPartnerTuttiACampagna(Long campagnaId) {
		Campagna campagna = getCampagnaPartner(campagnaId);
		List<Utente> partner = userDao.trovaPartner(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(partner)) {
			if(campagna.getPartner() == null)
				campagna.setPartner(new ArrayList<Utente>());
			campagna.getPartner().addAll(partner);
			salva(campagna);
		}
	}
	public void eliminaPartnerTuttiDaCampagna(Long campagnaId) {
		Campagna campagna = getCampagnaPartner(campagnaId);
		List<Utente> partner = getPartnerByCampagnaId(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(partner)) {
			if(campagna.getPartner() == null)
				campagna.setPartner(new ArrayList<Utente>());
			campagna.getPartner().removeAll(partner);
			salva(campagna);
		}
	}
	public void aggiungiZonaTuttiACampagna(Long campagnaId) {
		Campagna campagna = getCampagnaZona(campagnaId);
		List<Zona> zone = trovaZona(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(zone)) {
			if(campagna.getZone() == null)
				campagna.setZone(new ArrayList<Zona>());
			campagna.getZone().addAll(zone);
			salva(campagna);
		}
	}
	public void eliminaZonaTuttiDaCampagna(Long campagnaId) {
		Campagna campagna = getCampagnaZona(campagnaId);
		List<Zona> zone = getZonaByCampagnaId(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(zone)) {
			if(campagna.getZone() == null)
				campagna.setZone(new ArrayList<Zona>());
			campagna.getZone().removeAll(zone);
			salva(campagna);
		}
	}
	public void aggiungiEmailTuttiACampagna(Long campagnaId) {
		Campagna campagna = getCampagnaEmail(campagnaId);
		List<EmailTemplate> emails = trovaEmail(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(emails)) {
			if(campagna.getEmail() == null)
				campagna.setEmail(new ArrayList<EmailTemplate>());
			campagna.getEmail().addAll(emails);
			salva(campagna);
		}
	}
	public void eliminaEmailTuttiDaCampagna(Long campagnaId) {
		Campagna campagna = getCampagnaEmail(campagnaId);
		List<EmailTemplate> emails = getEmailByCampagnaId(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(emails)) {
			if(campagna.getEmail() == null)
				campagna.setEmail(new ArrayList<EmailTemplate>());
			campagna.getEmail().removeAll(emails);
			salva(campagna);
		}
	}
	
	public void aggiungiZonaACampagna(Long campagnaId, Long agenteId) {
		Campagna campagna = getCampagna(campagnaId);
		Zona zona = getFullZonaCampagna(agenteId);
		if(zona != null) {
			if(zona.getCampagne() == null)
				zona.setCampagne(new ArrayList<Campagna>());
			if(!zona.getCampagne().contains(campagna)) {
				zona.getCampagne().add(campagna);
				salvaZona(zona);
			}
		}
	}
	public void eliminaZonaDaCampagna(Long campagnaId, Long agenteId) {
		Campagna campagna = getCampagna(campagnaId);
		Zona zona = getFullZonaCampagna(agenteId);
		if(zona.getCampagne() == null)
			zona.setCampagne(new ArrayList<Campagna>());
		if(zona.getCampagne().contains(campagna)) {
			zona.getCampagne().remove(campagna);
			salvaZona(zona);
		}
	}
	public void aggiungiEsitoACampagna(Long campagnaId, Long esitoId) {
		Campagna campagna = getCampagna(campagnaId);
		EsitoArchiviazione esito = getFullEsito(esitoId);
		if(esito != null) {
			if(esito.getCampagne() == null)
				esito.setCampagne(new ArrayList<Campagna>());
			if(!esito.getCampagne().contains(campagna)) {
				esito.getCampagne().add(campagna);
				salvaEsito(esito);
			}
		}
	}
	public void eliminaEsitoDaCampagna(Long campagnaId, Long esitoId) {
		Campagna campagna = getCampagna(campagnaId);
		EsitoArchiviazione esito = getFullEsito(esitoId);
		if(esito.getCampagne() == null)
			esito.setCampagne(new ArrayList<Campagna>());
		if(esito.getCampagne().contains(campagna)) {
			esito.getCampagne().remove(campagna);
			salvaEsito(esito);
		}
	}
	public void aggiungiEsitoTuttiACampagna(Long campagnaId){
		Campagna campagna = getCampagnaEsito(campagnaId);
		List<EsitoArchiviazione> esiti = trovaEsito(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(esiti)) {
			if(campagna.getEsiti_archiviazione() == null)
				campagna.setEsiti_archiviazione(new ArrayList<EsitoArchiviazione>());
			campagna.getEsiti_archiviazione().addAll(esiti);
			salva(campagna);
		}
	}
	public void eliminaEsitoTuttiDaCampagna(Long campagnaId){
		Campagna campagna = getCampagnaEsito(campagnaId);
		List<EsitoArchiviazione> esiti = getEsitiByCampagna(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(esiti)) {
			if(campagna.getEsiti_archiviazione() == null)
				campagna.setEsiti_archiviazione(new ArrayList<EsitoArchiviazione>());
			campagna.getEsiti_archiviazione().removeAll(esiti);
			salva(campagna);
		}
	}
	public void unisciContatti(Integer ordine, List<ContattoSI> proposti, String proprieta, List<String> azione) {
		List<ContattoSI> duplicati = new ArrayList<ContattoSI>();
		Map<String, ContattoSI> m = new HashMap<String, ContattoSI>();
		if(ApplicationUtil.gtZero(ordine)) {
			if(ApplicationUtil.eq(ordine, 1)) {
				Collections.sort(proposti, new ContattoSIComparator(false));
			}
			if(ApplicationUtil.eq(ordine, 2)) {
				Collections.sort(proposti, new ContattoSIComparator(true));
			}
		}
		if(ApplicationUtil.notEmpty(proposti)) {
			Iterator<ContattoSI> objIter = proposti.iterator();
			while(objIter.hasNext()) {
				ContattoSI contatto = objIter.next();
				String valore = ApplicationUtil.trimToNull(ApplicationUtil.metodo(contatto, "anagrafica_" + proprieta));
				if(valore != null) {
					if(!m.keySet().contains(valore)) {
						m.put(valore, contatto);
					} else {
						duplicati.add(contatto);
						ContattoSI precedente = m.get(valore);
						if(precedente != null) {
							precedente.getDuplicati().add(contatto);
						}
					}
				}
			}
		}
		if(ApplicationUtil.notEmpty(m)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			List<CampoAggiuntivoAnagrafica> campiAgg = anagraficaDao.getAllCampiAggiuntivi();
			Iterator<String> chiaviIt = m.keySet().iterator();
			while(chiaviIt.hasNext()) {
				ContattoSI unico = m.get(chiaviIt.next());
				if(ApplicationUtil.notEmpty(unico.getDuplicati())) {
					List<Long> idA = new ArrayList<Long>();
					List<Long> idC = new ArrayList<Long>();
					Anagrafica aU = (Anagrafica) session.get(Anagrafica.class, unico.getAnagrafica_id());
					Hibernate.initialize(aU.getDettagli());
					List<DatoAggiuntivoAnagrafica> campiB = new ArrayList<DatoAggiuntivoAnagrafica>();
					Iterator<ContattoSI> cIt = unico.getDuplicati().iterator();
					while(cIt.hasNext()) {
						ContattoSI duplicato = cIt.next();
						Anagrafica aD = (Anagrafica) session.get(Anagrafica.class, duplicato.getAnagrafica_id());
						Hibernate.initialize(aD.getDettagli());
						campiB.addAll(aD.getDettagli());
						idA.add(duplicato.getAnagrafica_id());
						idC.add(duplicato.getId());
						if(ApplicationUtil.merge(unico, duplicato)) {
							ApplicationUtil.merge(aU, duplicato);
						}
					}
					anagraficaDao.salvaAnagraficaMerge(session, aU, campiAgg);
					List<DatoAggiuntivoAnagrafica> listsA = new ArrayList<DatoAggiuntivoAnagrafica>();
					if(ApplicationUtil.notEmpty(campiB)) {
						List<String> campiA = new ArrayList<String>();
						campiA.addAll(ApplicationUtil.prendiChiaveAnagrafica(aU.getDettagli()));
						Iterator<DatoAggiuntivoAnagrafica> campiIt = campiB.iterator();
						while(campiIt.hasNext()) {
							DatoAggiuntivoAnagrafica d = campiIt.next();
							if(!listsA.contains(d) && !campiA.contains(d.getCampo_aggiuntivo().getChiave())) {
								listsA.add(d);
							}
						}
					}
					if(ApplicationUtil.notEmpty(listsA)) {
						Iterator<DatoAggiuntivoAnagrafica> dIt = listsA.iterator();
						while(dIt.hasNext()) {
							DatoAggiuntivoAnagrafica dato = dIt.next();
							DatoAggiuntivoAnagrafica da = new DatoAggiuntivoAnagrafica();
							da.setCampo_aggiuntivo(dato.getCampo_aggiuntivo());
							da.setAnagrafica(aU);
							da.setValore(dato.getValore());
							da.imposta();
							session.save(da);
						}
					}
					if(ApplicationUtil.notEmpty(azione) && ApplicationUtil.notEmpty(idC)) {
						for(String a :  azione) {
							String hql = "update "+a+" set contatto.id = :new where contatto.id in (:old)";
							execute(false, session, hql, new Parameter("new", unico.getId()), new Parameter("old", idC));
						}
					}
					if (i++ % ConstantsGlobal.flushSize == 0) {
						session.flush();
						session.clear();
					}
				}
			}
			tx.commit();
			closeSession(session);
		}
		pulisciContatti(duplicati);
	}
	public void pulisciContatti(List<ContattoSI> duplicati) {
		if(ApplicationUtil.notEmpty(duplicati)) {
			List<Long> ln = ApplicationUtil.prendiIdContattoSI(duplicati);
			String id = ApplicationUtil.detoken(ln, ConstantsGlobal.commaSeparator); 
			String hql = "delete from Contatto where id in ("+id+")";
			execute(false, hql);
		}
	}
	public List<SelectItem> getValoriEredita() {
		List<SelectItem> list = new ArrayList<SelectItem>();
		String [] as = new String [] {"StoriaContatto", "Appuntamento", "Contratto", "Cdr", "AllegatoContatto", "Sondaggio", "Notifica", "Email", "Rmail", "Telecom", "AutoContatto"};
		for(String a :  as) {
			list.add(new SelectItem(a, Message.addCatalogText(ApplicationUtil.uncapitalize(a))));
		}
		return list;
	}
	public void eliminaListaDaCampagna(Lista listaSimple, Campagna campagna) {
		if(campagna != null && listaSimple != null && listaSimple.getId() != null ) {
			Lista lista = getFullLista(listaSimple.getId());
			if(lista != null) {
				String hql = "delete from Contatto where lista = :lista and campagna = :campagna";
				execute(false, hql, new Parameter("lista", lista), new Parameter("campagna", campagna));
			}
			if(lista.getCampagna() == null)
				lista.setCampagna(new ArrayList<Campagna>());
			if(lista.getCampagna().contains(campagna)) {
				lista.getCampagna().remove(campagna);
				salva(lista);
			}
		}
	}
	public void eliminaListeDaCampagna(List<Lista> listaSimpleList, Campagna campagna) {
		if(campagna != null && ApplicationUtil.notEmpty(listaSimpleList)) {
			Iterator<Lista> lsIt = listaSimpleList.iterator();
			while (lsIt.hasNext()) {
				Lista listaSimple = (Lista) lsIt.next();
				if(listaSimple != null && listaSimple.getId() != null ) {
					Lista lista = getFullLista(listaSimple.getId());
					if(lista != null) {
						String hql = "delete from Contatto where lista = :lista and campagna = :campagna";
						execute(false, hql, new Parameter("lista", lista), new Parameter("campagna", campagna));
					}
					if(lista.getCampagna() == null)
						lista.setCampagna(new ArrayList<Campagna>());
					if(lista.getCampagna().contains(campagna)) {
						lista.getCampagna().remove(campagna);
						salva(lista);
					}
				}
			}
		}
	}
	public void eliminaCampagna(Campagna campagna){
		cancellaAggiorna(campagna);
	}
	@SuppressWarnings("unchecked")
	public void eliminaCampagnaConListe(Campagna campagna){
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		criteria.createCriteria("campagna").add(Restrictions.eq("id" ,campagna.getId()));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		List<Long> lists = getList(criteria);
		eliminaInCampagnaTransaction(lists, campagna);
		cancellaAggiorna(campagna);
	}
	@SuppressWarnings("unchecked")
	public void eliminaCampagnaConAnagrafiche(Campagna campagna){
		DetachedCriteria criteria = DetachedCriteria.forClass(Anagrafica.class);
		criteria.createCriteria("lista").createCriteria("campagna").add(Restrictions.eq("id" ,campagna.getId()));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		List<Long> anags = getList(criteria);
		List<Lista> liste = getListeByCampagnaId(campagna.getId());
		eliminaInListaTransaction(anags, liste);
		eliminaCampagnaConListe(campagna);
	}
	public EsitoArchiviazione salvaEsitoArchiviazione(EsitoArchiviazione esitoArchiviazione) {
		if(ApplicationUtil.ne(esitoArchiviazione.getTipologiaS(), 3)) {
			esitoArchiviazione.setDaNonRichiamare();
		} else {
			esitoArchiviazione.setDaRichiamare();
		}
		return (EsitoArchiviazione) salvaAggiorna(esitoArchiviazione);
	}
	private ExtendedCriteria trovaCriteria(EsitoArchiviazioneCriteria ana) {
		if(ana != null)  ana = (EsitoArchiviazioneCriteria) ana.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoArchiviazione.class);
		if (ApplicationUtil.isNotBlank(ana.getOggetto())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, ana.getOggetto(), criteria, "etichetta");
		}
		if (ApplicationUtil.gtZero(ana.getPositivo())) {
			if (ApplicationUtil.eq(ana.getPositivo(), 1)) {
				criteria.add(Restrictions.eq("positivo", Boolean.TRUE));
			} else 	{
				criteria.add(Restrictions.eq("positivo", Boolean.FALSE));
			} 
		}
		if (ApplicationUtil.gtZero(ana.getUtile())) {
			if (ApplicationUtil.eq(ana.getUtile(), 1)) {
				criteria.add(Restrictions.eq("utile", Boolean.TRUE));
			} else 	{
				criteria.add(Restrictions.eq("utile", Boolean.FALSE));
			} 
		}
		ana.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, ana);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoArchiviazione> trova(EsitoArchiviazioneCriteria ana, int start, int size) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		ApplicationUtil.orderEsito(criteria.getCriteria());
		criteria.getCriteria().setFetchMode("esito_collegato", FetchMode.JOIN);
		return getList(criteria, start, size);
	}
	public int trovaSize(EsitoArchiviazioneCriteria ana) {
		ExtendedCriteria criteria = trovaCriteria(ana);
		return getListSize(criteria);
	}
	public List<SelectItem> getValori() {
		List<String> lst = new ArrayList<String>(ConstantsMap.anagraficaMap);
		List<SelectItem> sis = new ArrayList<SelectItem>();
		Iterator<String> objIter = lst.iterator();
		while(objIter.hasNext()) {
			String sty = objIter.next();
			sis.add(new SelectItem(sty, Message.addCatalogText(sty)));
		}
		return sis;
	}
	public List<Label> getValoriMap() {
		List<String> lst = new ArrayList<String>(ConstantsMap.anagraficaMap);
		List<Label> sis = new ArrayList<Label>();
		Iterator<String> objIter = lst.iterator();
		while(objIter.hasNext()) {
			String sty = objIter.next();
			Label l = new Label();
			l.setLabel(Message.addCatalogText(sty));
			l.setValue(ConstantsGlobal.pipeSeparator + sty + ConstantsGlobal.pipeSeparator);
			sis.add(l);
		}
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<SelectItem> getValoriNotInCampo(boolean anagrafica, boolean contratto) {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateStore.class);
		criteria.add(Restrictions.eq("valore", ""));
		if(anagrafica) {
			DetachedCriteria criteriaA = DetachedCriteria.forClass(CampoAggiuntivoAnagrafica.class);
			criteriaA.setProjection(Projections.distinct(Projections.property("chiave")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaA, "chiave");
		}
		if(contratto) {
			DetachedCriteria criteriaC = DetachedCriteria.forClass(CampoAggiuntivoContratto.class);
			criteriaC.setProjection(Projections.distinct(Projections.property("chiave")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaC, "chiave");
		}
		criteria.setProjection(Projections.distinct(Projections.property("chiave")));
		List<String> lst = getList(criteria);
		List<String> ids = new ArrayList<String>(); 
		if(ApplicationUtil.notEmpty(lst)) {
			lst.removeAll(ConstantsMap.totalS);
			Iterator<String> objIter = lst.iterator();
			while(objIter.hasNext()) {
				String sty = ApplicationUtil.trimToNull(objIter.next());
				if(sty != null && !ids.contains(sty) && !sty.contains(ConstantsGlobal.colonSeparator)) {
					ids.add(sty);
					sis.add(new SelectItem(sty,sty));
				}
			}
		}
		return sis;
	}
	@SuppressWarnings("unchecked")
	public void salvaCampiAggiuntivoAnagrafica(CampoAggiuntivoAnagrafica campo, List<String> lst) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateStore.class);
		criteria.add(Restrictions.in("chiave", lst));
		List<TemplateStore> tmp = getList(criteria);
		List<String> duplicati = new ArrayList<String>();
		if(ApplicationUtil.notEmpty(tmp)) {
			Iterator<TemplateStore> objIter = tmp.iterator();
			while(objIter.hasNext()) {
				TemplateStore tm = objIter.next();
				String chiave = tm.getChiave();
				if(!duplicati.contains(chiave)) {
					duplicati.add(chiave);
					CampoAggiuntivoAnagrafica c = campo.getCopia();
					c.setChiave(chiave);
					c.setEtichetta(chiave);
					c.setDescrizione(chiave);
					c.setValori(tm.getOpzioni());
					if(ApplicationUtil.ltZero(c.getTipo())) {
						c.setTipo(ConstantsMap.TIPO_APERTO);
					} else {
						c.setTipo(tm.getTipo());
					}
					salvaAggiorna(c);
					dataDao.addCampo(c);
				}
			}
		}
	}
	@SuppressWarnings("unchecked")
	public void salvaCampiAggiuntivoContratto(CampoAggiuntivoContratto campo, List<String> lst) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TemplateStore.class);
		criteria.add(Restrictions.in("chiave", lst));
		List<TemplateStore> tmp = getList(criteria);
		List<String> duplicati = new ArrayList<String>();
		if(ApplicationUtil.notEmpty(tmp)) {
			Iterator<TemplateStore> objIter = tmp.iterator();
			while(objIter.hasNext()) {
				TemplateStore tm = objIter.next();
				String chiave = tm.getChiave();
				if(!duplicati.contains(chiave)) {
					duplicati.add(chiave);
					CampoAggiuntivoContratto c = campo.getCopia();
					c.setChiave(chiave);
					c.setEtichetta(chiave);
					c.setDescrizione(chiave);
					c.setValori(tm.getOpzioni());
					if(ApplicationUtil.ltZero(c.getTipo())) {
						c.setTipo(ConstantsMap.TIPO_APERTO);
					} else {
						c.setTipo(tm.getTipo());
					}
					salvaAggiorna(c);
					dataDao.addCampo(c);
				}
			}
		}
	}
	public void scaricaAllegato(Appuntamento appuntamento, DownloadBean downloadBean) {
		List<AllegatoAppuntamento> allegati = getListaAllegati(appuntamento);
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_appuntamento);
				String file_header = ApplicationUtil.reportFile(appuntamento.getData_creazione(), ConstantsLanguage.APPUNTAMENTO.stampa(), appuntamento.getId(), appuntamento.getContatto().getAnagrafica().getDenominazione()) +"." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				List<Allegato> all = new ArrayList<Allegato>();
				Iterator<AllegatoAppuntamento> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoAppuntamento allegato = (AllegatoAppuntamento) itA.next();
					ApplicationUtil.addFile(all, allegato.getId() + "_" +allegato.getNome(), allegato.getContentFile(s), allegato.getData().getTime());
				}
				ApplicationUtil.zip(null, all, out, p);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public void scaricaAllegati(List<Appuntamento> appuntamenti, Utente utente, DownloadBean downloadBean) {
		List<AllegatoAppuntamento> allegati = getListaAllegati(appuntamenti);
		String p = impostazioneDao.getPasswordZIP();
		try {
			String pc = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
			operazioneDao.creaOperazione(ConstantsMap.LOG4,pc ,utente, ConstantsMap.OPERAZIONE_EXPORT_APPUNTAMENTO);
			String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_appuntamento);
			String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ApplicationUtil.lowerCase(ConstantsLanguage.ALLEGATI.stampa() + "_" + ConstantsLanguage.APPUNTAMENTO.stampa())) + "." + ConstantsMap.zip;
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(file_header, appuntamenti, allegati);
			List<Allegato> all = new ArrayList<Allegato>();
			if(ApplicationUtil.notEmpty(allegati)) {
				Iterator<AllegatoAppuntamento> itA = allegati.iterator();
				while (itA.hasNext()) {
					AllegatoAppuntamento allegato = (AllegatoAppuntamento) itA.next();
					ApplicationUtil.addFile(all, allegato.getAppuntamento().getId() + "_" + allegato.getId() + "_" +allegato.getNome(), allegato.getContentFile(s), allegato.getData().getTime());
				}
			}
			if (ApplicationUtil.empty(all)) {
				ApplicationUtil.fileWriter(all, ConstantsLanguage.NO_APPUNTAMENTO.stampa(), ConstantsLanguage.NO_APPUNTAMENTI.stampa());
			}
			ApplicationUtil.zip(null, all, out, p);
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void scaricaAllegato(AllegatoAppuntamento allegato, DownloadBean downloadBean) {
		try {
			String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_appuntamento);
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());
			InputStream is = new ByteArrayInputStream(allegato.getContentFile(s));
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void salvaAllegato(FileUpload fileUpload, Appuntamento appuntamento) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				String s = impostazioneDao.getPathBaseW(ConstantsGlobal.path_appuntamento);
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					AllegatoAppuntamento allegato = new AllegatoAppuntamento();
					allegato.setAppuntamento(appuntamento);
					allegato.setDimensione(ffB.length);
					allegato.setNome(ApplicationUtil.getNameN(ff.getName()));
					allegato.setContentFile(ffB, s);
					salvaAggiorna(allegato);
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoAppuntamento> getListaAllegati(Appuntamento appuntamento) {
		if(appuntamento==null)
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoAppuntamento.class);
		criteria.add(Restrictions.eq("appuntamento.id", appuntamento.getId()));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<AllegatoAppuntamento> getListaAllegati(List<Appuntamento> appuntamenti) {
		if(ApplicationUtil.empty(appuntamenti))
			return null;
		DetachedCriteria criteria = DetachedCriteria.forClass(AllegatoAppuntamento.class);
		criteria.add(Restrictions.in("appuntamento.id", ApplicationUtil.prendiIdAppuntamento(appuntamenti)));
		return getList(criteria);
	}
	public void deleteAllegato(AllegatoAppuntamento allegato)  {
		String s = impostazioneDao.getPathBaseR(ConstantsGlobal.path_appuntamento);
		allegato.delete(s);
		cancellaAggiorna(allegato);
	}
	public void deleteAllegati(Appuntamento appuntamento)  {
		List<AllegatoAppuntamento> allegati = getListaAllegati(appuntamento);
		Iterator<AllegatoAppuntamento> itAn = allegati.iterator();
		while (itAn.hasNext()) {
			AllegatoAppuntamento allegato = (AllegatoAppuntamento) itAn.next();
			deleteAllegato(allegato);
		}
	}
	public Agente salvaAgente(Agente agente) {
		return (Agente) salvaAggiorna(agente);
	}
	public Agente salva(Agente agente) {
		return salvaAgente(agente);
	}
	public Zona salvaZona(Zona zona) {
		return (Zona) salvaAggiorna(zona);
	}
	public EmailTemplate salvaEmail(EmailTemplate email) {
		return (EmailTemplate) salvaAggiorna(email);
	}
	
	public void eliminaAgente(Agente agente) {
		elimina(agente);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiByContattoId(Boolean supervisore,Utente utente, Contatto contatto) {
		Campagna cmp = contatto.getCampagna();
		Integer scelta = cmp.getModalita_scelta();
		if(scelta.equals(ConstantsMap.agenti_nessuno)) {
			return new ArrayList<Agente>();
		}
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(utente != null && !ApplicationUtil.isTrue(supervisore) && (scelta.equals(ConstantsMap.agenti_personali) || scelta.equals(ConstantsMap.agenti_rete) || scelta.equals(ConstantsMap.agenti_sottorete) || scelta.equals(ConstantsMap.agenti_primalinea))) {
			if(scelta.equals(ConstantsMap.agenti_personali)) {
				criteria.createCriteria("user").add(Restrictions.eq("id", utente.getId()));
			}
			if(scelta.equals(ConstantsMap.agenti_rete)) {
				impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 0, userDao.getMaxLivelloRete(), criteria, "id");
			}
			if(scelta.equals(ConstantsMap.agenti_sottorete)) {
				impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 1, userDao.getMaxLivelloRete(), criteria, "id");
			}
			if(scelta.equals(ConstantsMap.agenti_primalinea)) {
				impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 2, userDao.getMaxLivelloRete(), criteria, "id");
			}
		} else {
			DetachedCriteria criteriaZ = criteria.createCriteria("zone");
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(cmp.getId()));
			if(scelta.equals(ConstantsMap.agenti_autonomo)) {
				criteria.add(Restrictions.eq("autonomo", Boolean.TRUE));
			}
			if(scelta.equals(ConstantsMap.agenti_pertinenza_regione)) {
				criteriaZ.add(Restrictions.or(Restrictions.eq("regione", contatto.getAnagrafica().getIndirizzo_operativo().getRegione()), Restrictions.eq("regione", contatto.getAnagrafica().getIndirizzo_legale().getRegione())));
			}
			if(scelta.equals(ConstantsMap.agenti_pertinenza_provincia)) {
				criteriaZ.add(Restrictions.or(Restrictions.eq("provincia", contatto.getAnagrafica().getIndirizzo_operativo().getProvincia()), Restrictions.eq("provincia", contatto.getAnagrafica().getIndirizzo_legale().getProvincia())));
			}
			if(scelta.equals(ConstantsMap.agenti_pertinenza_comune)) {
				criteriaZ.add(Restrictions.or(Restrictions.eq("comune", contatto.getAnagrafica().getIndirizzo_operativo().getComune()), Restrictions.eq("comune", contatto.getAnagrafica().getIndirizzo_legale().getComune())));
			}
			if(scelta.equals(ConstantsMap.agenti_pertinenza_cap)) {
				criteriaZ.add(Restrictions.or(Restrictions.eq("cap", contatto.getAnagrafica().getIndirizzo_operativo().getCap()), Restrictions.eq("cap", contatto.getAnagrafica().getIndirizzo_legale().getCap())));
			}
		}
		ApplicationUtil.orderA(criteria);
		List<Agente> afList =  getList(criteria);
		if(ApplicationUtil.gtZero(cmp.getAgenti_scelta())) {
			if(ApplicationUtil.eq(cmp.getAgenti_scelta(), 1)) {
				Collections.sort(afList, new AgenteComparator(contatto, true));
			}
			if(ApplicationUtil.eq(cmp.getAgenti_scelta(), 2)) {
				Collections.sort(afList, new AgenteComparator(contatto, false));
			}
		}
		return afList;
	}
	public List<SelectItem> getAgentiByCampagnaId(Boolean supervisore, Utente utente, Long campagna) {
		if(ApplicationUtil.ltZero(campagna)) {
			return new ArrayList<SelectItem>();
		}
		Campagna cmp = (Campagna) get(Campagna.class , campagna);
		Integer scelta = cmp.getModalita_scelta();
		if(scelta.equals(ConstantsMap.agenti_nessuno)) {
			return new ArrayList<SelectItem>();
		}
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(utente != null && !ApplicationUtil.isTrue(supervisore) && (scelta.equals(ConstantsMap.agenti_personali) || scelta.equals(ConstantsMap.agenti_rete) || scelta.equals(ConstantsMap.agenti_sottorete) || scelta.equals(ConstantsMap.agenti_primalinea))) {
			if(scelta.equals(ConstantsMap.agenti_personali)) {
				criteria.createCriteria("user").add(Restrictions.eq("id", utente.getId()));
			}
			if(scelta.equals(ConstantsMap.agenti_rete)) {
				impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 0, userDao.getMaxLivelloRete(), criteria, "id");
			}
			if(scelta.equals(ConstantsMap.agenti_sottorete)) {
				impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 1, userDao.getMaxLivelloRete(), criteria, "id");
			}
			if(scelta.equals(ConstantsMap.agenti_primalinea)) {
				impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 2, userDao.getMaxLivelloRete(), criteria, "id");
			}
		} else {
			DetachedCriteria criteriaZ = criteria.createCriteria("zone");
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(cmp.getId()));
			if(scelta.equals(ConstantsMap.agenti_autonomo)) {
				criteria.add(Restrictions.eq("autonomo", Boolean.TRUE));
			}
		}
		ApplicationUtil.orderA(criteria);
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}
	public void elimina(Agente agente) {
		cancellaAggiorna(agente);
	}
	public Agente getFullAgenteZona(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("zone", FetchMode.JOIN);
		return  (Agente) getUniqueResult(criteria);
	}
	public Utente getFullSupervisoreZona(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("zone", FetchMode.JOIN);
		return  (Utente) getUniqueResult(criteria);
	}
	public Utente getFullUtenteAgente(Long utenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		criteria.add(Restrictions.idEq(utenteId));
		criteria.setFetchMode("agenti", FetchMode.JOIN);
		return  (Utente) getUniqueResult(criteria);
	}

	public Zona getFullZonaSupervisore(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("supervisori", FetchMode.JOIN);
		return  (Zona) getUniqueResult(criteria);
	}
	public Zona getFullZonaAgente(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("agente", FetchMode.JOIN);
		return  (Zona) getUniqueResult(criteria);
	}

	public Zona getFullZonaCampagna(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return  (Zona) getUniqueResult(criteria);
	}
	public EmailTemplate getFullEmailConCampagne(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return  (EmailTemplate) getUniqueResult(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Long> getZoneByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> getEmailsByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	

	public List<SelectItem> getZonaSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		List<ZonaSI> lis = getZonaSI(criteria);
		return impostazioneDao.getZona(lis, true, false, null);
	}
	public Agente getFullAgenteById(Long ana) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.add(Restrictions.idEq(ana));
		criteria.setFetchMode("agente_padre", FetchMode.JOIN);
		return  (Agente) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByAgenteId(Long id){
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("zone").createCriteria("agente").add(Restrictions.idEq(id));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	public List<SelectItem> getEsitiAppuntamento(Integer confermato) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		if (ApplicationUtil.gtZero(confermato)){
			if (ApplicationUtil.eq(confermato, 1)) 
				criteria.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteria.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		List<EsitoAppuntamentoSI> lis = getEsitoAppuntamentoSI(criteria);
		return impostazioneDao.getEsitoAppuntamento(lis, false, null);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoContratto> trova(EsitoContrattoCriteria esitoc,  int start, int size) {
		ExtendedCriteria criteria =  getCriteriaEsitiContratto(esitoc);
		ApplicationUtil.orderEsito(criteria.getCriteria());
		return getList(criteria, start, size);
	}
	public int trovaSize(EsitoContrattoCriteria esitoc) {
		ExtendedCriteria criteria =  getCriteriaEsitiContratto(esitoc);
		return getListSize(criteria);
	}
	public ExtendedCriteria getCriteriaEsitiContratto(EsitoContrattoCriteria esitoc) {
		if(esitoc != null)  esitoc = (EsitoContrattoCriteria) esitoc.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		if (ApplicationUtil.isNotBlank(esitoc.getEtichetta())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, esitoc.getEtichetta(), criteria, "etichetta", "descrizione");
		}
		if (ApplicationUtil.gtZero(esitoc.getConfermato())){
			if (ApplicationUtil.eq(esitoc.getConfermato(), 1)) 
				criteria.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteria.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(esitoc.getVisibile())){
			if (ApplicationUtil.eq(esitoc.getVisibile(), 1)) 
				criteria.add(Restrictions.eq("visibile", Boolean.TRUE));
			else 
				criteria.add(Restrictions.eq("visibile", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(esitoc.getUtile())){
			if (ApplicationUtil.eq(esitoc.getUtile(), 1)) 
				criteria.add(Restrictions.eq("utile", Boolean.TRUE));
			else 
				criteria.add(Restrictions.eq("utile", Boolean.FALSE));
		}
		esitoc.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, esitoc);
	}
	public void cancella(EsitoAppuntamento esito) {
		cancellaAggiorna(esito);
	}
	public void cancella(EsitoContratto esito) {
		cancellaAggiorna(esito);
	}
	public EsitoAppuntamento salvaEsito(EsitoAppuntamento esito) {
		return (EsitoAppuntamento) salvaAggiorna(esito);
	}
	public EsitoContratto salvaEsito(EsitoContratto esito) {
		return (EsitoContratto) salvaAggiorna(esito);
	}
	public void salvaPredefinito(EsitoAppuntamento esitoPredefinito) {
		List<EsitoAppuntamento> esiti = getAllEsitiAppuntamentoPredefinito();
		Iterator<EsitoAppuntamento> objIter = esiti.iterator();
		while(objIter.hasNext()) {
			EsitoAppuntamento esito = objIter.next();
			esito.setPredefinito(Boolean.FALSE);
			salvaEsito(esito);
		}
		esitoPredefinito.setPredefinito(Boolean.TRUE);
		salvaEsito(esitoPredefinito);
	}
	public void removePredefinito(EsitoAppuntamento esitoPredefinito) {
		esitoPredefinito.setPredefinito(Boolean.FALSE);
		salvaEsito(esitoPredefinito);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamento> getAllEsitiAppuntamentoPredefinito() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		criteria.add(Restrictions.eq("predefinito", Boolean.TRUE));
		return getList(criteria);
	}
	public List<SelectItem> getAgentiByCampagnaSI(Long campagnaId, Agente agente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, agente);
	}
	public List<SelectItem> getProdottiContratto(Contratto contratto) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.add(Restrictions.eq("attivo", Boolean.TRUE));
		criteria.createCriteria("prodotti_in_vendita").add(Restrictions.eq("id.campagna.id", contratto.getContatto().getCampagna().getId()));
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		List<ProdottoSI> lis = getProdottoSI(criteria);
		return impostazioneDao.getProdotto(lis, false, true, contratto.getProdotto());
	}
	public List<SelectItem> getEsitiContrattoP(Boolean supervisore) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		if(ApplicationUtil.isFalse(supervisore)) {
			criteria.add(Restrictions.eq("visibile", Boolean.TRUE));
		}
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}
	public List<SelectItem> getEsitiContratto(Boolean supervisore) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		if(ApplicationUtil.isFalse(supervisore)) {
			criteria.add(Restrictions.eq("impostabile_operatore", Boolean.TRUE));
		}
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoContratto> getEsitiContrattoModel() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		ApplicationUtil.orderEsito(criteria);
		return getList(criteria);
	}
	public List<SelectItem> getEsitiContrattoAgente() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("impostabile_agente", Boolean.TRUE));
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}
	public List<SelectItem> getEsitiContrattoOperatore() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("impostabile_operatore", Boolean.TRUE));
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}
	public List<SelectItem> getEsitiContratto() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}

	public List<SelectItem> getEsitiContrattoAgente(EsitoContratto esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("impostabile_agente", Boolean.TRUE));
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, esito);
	}
	public List<SelectItem> getEsitiContrattoOperatore(EsitoContratto esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("impostabile_operatore", Boolean.TRUE));
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, esito);
	}
	public List<SelectItem> getEsitiContratto(EsitoContratto esito) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, esito);
	}
	public List<SelectItem> getEsitiContrattoAgenteP() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("confermato", Boolean.TRUE));
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}
	public List<SelectItem> getEsitiContrattoOperatoreP() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoContratto.class);
		criteria.add(Restrictions.eq("visibile", Boolean.TRUE));
		List<EsitoContrattoSI> lis = getEsitoContrattoSI(criteria);
		return impostazioneDao.getEsitoContratto(lis, false, null);
	}
	public EsitoContratto getEsitoContratto(Long idEsito) {
		return  (EsitoContratto) get(EsitoContratto.class, idEsito);
	}
	public EsitoAppuntamento getEsitoAppuntamento(Long idEsito) {
		return  (EsitoAppuntamento) get(EsitoAppuntamento.class, idEsito);
	}
	public int trovaAppuntamentiAgenteSize(Long agenteId) {
		Date ora = ApplicationUtil.date();
		DetachedCriteria crit = DetachedCriteria.forClass(Appuntamento.class);
		crit.add(Restrictions.eq("agente.id", agenteId));
		crit.add(Restrictions.gt("data_inizio", ora));
		return getListSize(crit);
	}
	public Appuntamento getAppuntamentoById(Long appuntamentoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		criteria.add(Restrictions.idEq(appuntamentoId));
		return (Appuntamento) getUniqueResult(criteria);
	}
	public Appuntamento getFullAppuntamentoById(Long appuntamentoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Appuntamento.class);
		criteria.add(Restrictions.idEq(appuntamentoId));
		return (Appuntamento) getUniqueResult(criteria);
	}
	public EsitoAppuntamento getFullEsitoAppuntamento(Long esitoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		criteria.add(Restrictions.idEq(esitoId));
		return (EsitoAppuntamento) getUniqueResult(criteria);
	}

	public List<SelectItem> getAgentiSIPartner(Utente user, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaC = criteria.createCriteria("zone").createCriteria("campagne");
		criteriaC.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		if(ApplicationUtil.gtZero(teamId)) {
			DetachedCriteria criteriaT = criteriaC.createCriteria("team");
			criteriaT.add(Restrictions.idEq(teamId));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}
	public List<SelectItem> getAgentiSIOperatore(Utente user, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("zone").createCriteria("campagne").createCriteria("team");
		criteriaT.createCriteria("user").add(Restrictions.idEq(user.getId()));
		if(ApplicationUtil.gtZero(teamId))
			criteriaT.add(Restrictions.idEq(teamId));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}

	public List<SelectItem> getAgentiSITeam() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}
	public List<SelectItem> getAgentiSITeam(Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(teamId)) {
			criteria.createCriteria("zone").createCriteria("campagne").createCriteria("team").add(Restrictions.idEq(teamId));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}
	public List<SelectItem> getAgentiSITeam(Utente user, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("zone").createCriteria("campagne").createCriteria("team");
		criteriaT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		if(ApplicationUtil.gtZero(teamId)) {
			criteriaT.add(Restrictions.idEq(teamId));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}

	public List<SelectItem> getAgentiMultipli(Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("zone").createCriteria("campagne").createCriteria("team");
		ApplicationUtil.proprieta(false, filtro, criteriaT, "id", singolo, multiplo);
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}


	public List<SelectItem> getAgentiMultipliOperatore(Utente user, Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("zone").createCriteria("campagne").createCriteria("team");
		criteriaT.createCriteria("user").add(Restrictions.idEq(user.getId()));
		ApplicationUtil.proprieta(false, filtro, criteriaT, "id", singolo, multiplo);
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}

	public List<SelectItem> getOperatoriMultipliOperatore(Utente user, Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaT  = criteria.createCriteria("team");
		criteriaT.createCriteria("user").add(Restrictions.idEq(user.getId()));
		
		ApplicationUtil.proprieta(true, filtro, criteriaT, "id", singolo, multiplo);
		
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, true, null);
	}

	public List<SelectItem> getAgentiMultipliPartner(Utente user, Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaC = criteria.createCriteria("zone").createCriteria("campagne");
		criteriaC.createCriteria("partner").add(Restrictions.idEq(user.getId()));
		
		DetachedCriteria criteriaT = criteriaC.createCriteria("team");
		ApplicationUtil.proprieta(false, filtro, criteriaT, "id", singolo, multiplo);
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}

	public List<SelectItem> getAgentiMultipli(Utente user, Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("zone").createCriteria("campagne").createCriteria("team");
		criteriaT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		ApplicationUtil.proprieta(false, filtro, criteriaT, "id", singolo, multiplo);
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, null);
	}

	public List<SelectItem> getUtentiSITeam() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> getAgentiSIPartnerUser(Utente user, boolean trattini) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("zone").createCriteria("campagne").createCriteria("partner").add(Restrictions.idEq(user.getId()));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, trattini, false, null);
	}
	public List<SelectItem> getAgentiSITeamUser(Utente user, boolean trattini) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("zone").createCriteria("campagne").createCriteria("team");
		criteriaT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, trattini, false, null);
	}
	public List<SelectItem> getAgentiSITeamOperatore(Utente user, boolean trattini) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("zone").createCriteria("campagne").createCriteria("team");
		criteriaT.createCriteria("user").add(Restrictions.idEq(user.getId()));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, trattini, false, null);
	}
	public List<SelectItem> getAgentiSITeamSuper(Utente user, Long zona, boolean trattini) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(zona)) {
			criteria.createCriteria("zone").add(Restrictions.idEq(zona));
		} else {
			criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(user.getId()));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, trattini, false, null);
	}
	public List<SelectItem> getAgentiSIZonaTeamSuper(Utente user, Long campagna, Agente agente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		criteriaZ.createCriteria("campagne").add(Restrictions.idEq(campagna));
		criteriaZ.createCriteria("supervisori").add(Restrictions.idEq(user.getId()));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, agente);
	}
	public List<SelectItem> getOperatoriByTeamId(Utente user, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaT  = criteria.createCriteria("team");
		criteriaT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		if(ApplicationUtil.gtZero(teamId)) {
			criteriaT.add(Restrictions.idEq(teamId));
		} else { 
			criteriaT.add(Restrictions.isNotNull("id"));
		}
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> getOperatoriOperatoreByTeamId(Utente user, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaT  = criteria.createCriteria("team");
		criteriaT.createCriteria("user").add(Restrictions.idEq(user.getId()));
		if(ApplicationUtil.gtZero(teamId)) {
			criteriaT.add(Restrictions.idEq(teamId));
		} else { 
			criteriaT.add(Restrictions.isNotNull("id"));
		}
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, true, null);
	}
	public List<SelectItem> getOperatoriPartnerByTeamId(Utente user, Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaT  = criteria.createCriteria("team");
		criteriaT.createCriteria("campagne").createCriteria("partner").add(Restrictions.idEq(user.getId()));
		if(ApplicationUtil.gtZero(teamId)) {
			criteriaT.add(Restrictions.idEq(teamId));
		} else { 
			criteriaT.add(Restrictions.isNotNull("id"));
		}
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	
	public List<SelectItem> getAgentiSITeamSuper(Utente user, Long zona) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(zona)) {
			criteria.createCriteria("zone").add(Restrictions.idEq(zona));
		} else {
			criteria.createCriteria("zone").createCriteria("supervisori").add(Restrictions.idEq(user.getId()));
		}
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}

	public List<SelectItem> getAgentiMultipliCoordinatore(Utente user, Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		criteriaZ.createCriteria("supervisori").add(Restrictions.idEq(user.getId()));
		ApplicationUtil.proprieta(false, filtro, criteriaZ, "id", singolo, multiplo);
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}

	public List<SelectItem> getOperatoriMultipliPartner(Utente user, Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaT  = criteria.createCriteria("team");
		criteriaT.createCriteria("campagne").createCriteria("partner").add(Restrictions.idEq(user.getId()));
		ApplicationUtil.proprieta(true, filtro, criteriaT, "id", singolo, multiplo);
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}

	public List<SelectItem> getOperatoriByTeamId(Long teamId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(ApplicationUtil.gtZero(teamId))
			criteria.createCriteria("team").add(Restrictions.idEq(teamId));
		else 
			criteria.createCriteria("team").add(Restrictions.isNotNull("id"));
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}

	public List<SelectItem> getOperatoriMultipli(Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaT = criteria.createCriteria("team");
		ApplicationUtil.proprieta(false, filtro, criteriaT, "id", singolo, multiplo);
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}

	public List<SelectItem> getOperatoriMultipli(Utente user, Long singolo, List<String> multiplo) {
		Boolean filtro = impostazioneDao.getFiltroMultiplo();
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		DetachedCriteria criteriaT  = criteria.createCriteria("team");
		criteriaT.createCriteria("supervisore").add(Restrictions.idEq(user.getId()));
		ApplicationUtil.proprieta(true, filtro, criteriaT, "id", singolo, multiplo);
		List<UtenteSI> lis = getUtenteSI(criteria);
		return impostazioneDao.getUtente(lis, false, null);
	}
	public List<SelectItem> getAgentiALLSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, true, false, null);
	}
	public List<SelectItem> getTeamALLSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, true, false, null);
	}
	public List<SelectItem> getTeamALLSI(Utente utente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Team.class);
		criteria.createCriteria("supervisore").add(Restrictions.idEq(utente.getId()));
		List<TeamSI> lis = getTeamSI(criteria);
		return impostazioneDao.getTeam(lis, true, false, null);
	}
	public Agente getAgente(Long agenteId) {
		return (Agente) get(Agente.class, agenteId);
	}

	public List<SelectItem> getAgenteSI(List<Agente> agenti) {
		List<AgenteSI> sis = new ArrayList<AgenteSI>();
		if(ApplicationUtil.notEmpty(agenti)) {
			Iterator<Agente> objIter = agenti.iterator();
			while(objIter.hasNext()) {
				Agente c = objIter.next();
				sis.add(new AgenteSI(c));
			}
		}
		return impostazioneDao.getAgente(sis, false, false, null);
	}

	public void aggiungiAgenteAdZona(Long agenteId, Long zonaId) {
		Agente agente = getFullAgenteZona(agenteId);
		Zona zona = getZona(zonaId);
		if (agente != null && zona != null) {
			if (!agente.getZone().contains(zona)) {
				agente.getZone().add(zona);
				salvaAgente(agente);
			}
		}
	}
	public void rimuoviAgenteAdZona(Long agenteId, Long zonaId) {
		Agente agente = getFullAgenteZona(agenteId);
		Zona zona = getZona(zonaId);
		if (agente != null && zona != null) {
			if (agente.getZone().contains(zona)) {
				agente.getZone().remove(zona);
				salvaAgente(agente);
			}
		}
	}
	public void aggiungiAgenteTuttiAdZona(Long zonaId, Integer tutti) {
		Zona zona  = getFullZonaAgente(zonaId);
		List<Agente> agenti= getAgentiNotInZona(zonaId, tutti);
		if (ApplicationUtil.notEmpty(agenti) && zona != null) {
			zona.getAgente().addAll(agenti);
			salvaZona(zona);
		}
	}
	public void rimuoviAgenteTuttiAdZona(Long zonaId) {
		Zona zona  = getFullZonaAgente(zonaId);
		List<Agente> agenti= getAgentiInZona(zonaId);
		if (ApplicationUtil.notEmpty(agenti) && zona != null) {
			zona.getAgente().removeAll(agenti);
			salvaZona(zona);
		}
	}
	public void aggiungiCampagnaAdZona(Long campagnaId, Long zonaId) {
		Campagna campagna = getFullCampagnaZone(campagnaId);
		Zona zona = getZona(zonaId);
		if (campagna != null && zona != null) {
			if (!campagna.getZone().contains(zona)) {
				campagna.getZone().add(zona);
				salva(campagna);
			}
		}
	}
	public void rimuoviCampagnaAdZona(Long campagnaId, Long zonaId) {
		Campagna campagna = getFullCampagnaZone(campagnaId);
		Zona zona = getZona(zonaId);
		if (campagna != null && zona != null) {
			if (campagna.getZone().contains(zona)) {
				campagna.getZone().remove(zona);
				salva(campagna);
			}
		}
	}
	public void aggiungiCampagnaTuttiAdZona(Long zonaId) {
		Zona zona  = getFullZonaCampagna(zonaId);
		List<Campagna> campagne = getCampagneNotInZona(zonaId);
		if (ApplicationUtil.notEmpty(campagne) && zona != null) {
			zona.getCampagne().addAll(campagne);
			salvaZona(zona);
		}
	}
	public void rimuoviCampagnaTuttiAdZona(Long zonaId) {
		Zona zona  = getFullZonaCampagna(zonaId);
		List<Campagna> campagne = getCampagneInZona(zonaId);
		if (ApplicationUtil.notEmpty(campagne) && zona != null) {
			zona.getCampagne().removeAll(campagne);
			salvaZona(zona);
		}
	}
	public void aggiungiCampagnaAdEmail(Long campagnaId, Long emailId) {
		Campagna campagna = getFullCampagnaEmail(campagnaId);
		EmailTemplate email = getEmail(emailId);
		if (campagna != null && email != null) {
			if (!campagna.getEmail().contains(email)) {
				campagna.getEmail().add(email);
				salva(campagna);
			}
		}
	}
	public void rimuoviCampagnaAdEmail(Long campagnaId, Long emailId) {
		Campagna campagna = getFullCampagnaEmail(campagnaId);
		EmailTemplate email = getEmail(emailId);
		if (campagna != null && email != null) {
			if (campagna.getEmail().contains(email)) {
				campagna.getEmail().remove(email);
				salva(campagna);
			}
		}
	}
	
	public Zona getZona(Long id){
		return (Zona) get(Zona.class, id);
	}
	public EmailTemplate getEmail(Long id){
		return (EmailTemplate) get(EmailTemplate.class, id);
	}
	
	public void aggiungiZonaAdAgente(Long agenteId, Long zonaId) {
		Agente agente = getFullAgenteZona(agenteId);
		Zona zona = getZona(zonaId);
		if (agente != null && zonaId != null) {
			if (!agente.getZone().contains(zona)) {
				agente.getZone().add(zona);
				salvaAgente(agente);
			}
		}
	}
	public void eliminaZonaDaAgente(Long agenteId, Long zonaId) {
		Agente agente = getFullAgenteZona(agenteId);
		Zona zona = getZona(zonaId);
		if (agente != null)
			if (agente.getZone().contains(zona)) {
				agente.getZone().remove(zona);
				salva(agente);
			}
	}
	public void eliminaZonaTutteDaAgente(Long agenteId) {
		Agente agente = getFullAgenteZona(agenteId);
		List<Zona> zone = getZonaAgenteById(agenteId);
		if (agente != null && ApplicationUtil.notEmpty(zone)) {
			agente.getZone().removeAll(zone);
			salvaAgente(agente);
		}
	}
	public void aggiungiZonaTutteAdAgente(Long agenteId) {
		Agente agente = getFullAgenteZona(agenteId);
		List<Zona> zone = trovaZoneTutte(agenteId);
		if (agente != null && ApplicationUtil.notEmpty(zone)) {
			agente.getZone().addAll(zone);
			salvaAgente(agente);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Zona>  getZonaAgenteById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		criteria.createCriteria("agente").add(Restrictions.idEq(id));
		ApplicationUtil.orderZ(criteria);
		return  getList(criteria);
	}
	public Agente getAgenteById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.add(Restrictions.idEq(id));
		return  (Agente) getUniqueResult(criteria);
	}
	public void eliminaAppuntamenti(Utente utente, List<Appuntamento> appuntamenti) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.APPUNTAMENTO.stampa() +": " + ApplicationUtil.size(appuntamenti), appuntamenti);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_APPUNTAMENTO);
		cancellaLista(appuntamenti);
	}
	public void modificaOperatoreSelezionati( Utente utente, List<Appuntamento> appuntamenti, Long operatoreNuovoId) {
		Utente operatoreNuovo = userDao.getUser(operatoreNuovoId);
		modificaOperatore(utente, appuntamenti, operatoreNuovo);
	}
	public void modificaAgenteSelezionati( Utente utente, List<Appuntamento> appuntamenti,  Long agenteNuovoId) {
		Agente agenteNuovo = getAgente(agenteNuovoId);
		modificaAgente(utente, appuntamenti, agenteNuovo);
	}
	public void modificaStatoSelezionati( Utente utente, List<Appuntamento> appuntamenti,  Integer stato) {
		modificaStato(utente, appuntamenti, stato);
	}
	public void modificaAgente(Utente utente, List<Appuntamento> appuntamenti, Agente agenteNuovo) {
		Iterator<Appuntamento> objIter = appuntamenti.iterator();
		while(objIter.hasNext()) {
			Appuntamento appuntamento = objIter.next();
			appuntamento.setAgente(agenteNuovo);
			pannelloDao.salvaAppuntamento(appuntamento, utente, false);
		}
	}
	public void modificaStato(Utente utente, List<Appuntamento> appuntamenti, Integer stato) {
		Iterator<Appuntamento> objIter = appuntamenti.iterator();
		while(objIter.hasNext()) {
			Appuntamento appuntamento = objIter.next();
			appuntamento.setStato(stato);
			pannelloDao.salvaAppuntamento(appuntamento, utente, true);
		}
	}
	public void modificaOperatore(Utente utente, List<Appuntamento> appuntamenti, Utente operatoreNuovo) {
		Iterator<Appuntamento> objIter = appuntamenti.iterator();
		while(objIter.hasNext()) {
			Appuntamento appuntamento = objIter.next();
			appuntamento.setOperatore(operatoreNuovo);
			pannelloDao.salvaAppuntamento(appuntamento, utente, false);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Zona> trovaZona(Long campagna_id) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Zona.class);
		criteriaU.createCriteria("campagne").add(Restrictions.idEq(campagna_id));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderZ(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<EmailTemplate> trovaEmail(Long campagna_id) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(EmailTemplate.class);
		criteriaU.createCriteria("campagne").add(Restrictions.idEq(campagna_id));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(EmailTemplate.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Agente> getAllAgentiByZonaById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		criteria.createCriteria("zone").add(Restrictions.idEq(id));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Lista> getListeByCampagnaId(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		criteria.createCriteria("campagna").add(Restrictions.idEq(id));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getAllCampagneByZonaById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("zone").add(Restrictions.idEq(id));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNotInZona(Long zona) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("zone").add(Restrictions.idEq(zona));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNotInEmail(Long email) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("email").add(Restrictions.idEq(email));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNotInProdotto(Long prodottoId) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("prodottiInVendita").add(Restrictions.eq("id.prodotto.id",prodottoId));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneInProdotto(Long prodottoId) {
		if(prodottoId != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("prodottiInVendita").add(Restrictions.eq("id.prodotto.id",prodottoId));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneInZona(Long zona) {
		if(zona != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("zone").add(Restrictions.idEq(zona));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneInEmail(Long email) {
		if(email != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("email").add(Restrictions.idEq(email));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNotInEsito(Long esito) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("esiti_archiviazione").add(Restrictions.idEq(esito));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneInEsito(Long esito) {
		if(esito != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("esiti_archiviazione").add(Restrictions.idEq(esito));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiNotInZona(Long zona,  Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("zone", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Agente.class);
				criteriaU.createCriteria("zone").add(Restrictions.idEq(zona));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("zone", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Agente.class);
			criteriaU.createCriteria("zone").add(Restrictions.idEq(zona));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderA(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiInZona(Long zona) {
		if(zona != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
			criteria.createCriteria("zone").add(Restrictions.idEq(zona));
			ApplicationUtil.orderA(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Agente>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getSupervisoriNotInZona(Long zona, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("zone", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
				criteriaU.createCriteria("zone").add(Restrictions.idEq(zona));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("zone", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Utente.class);
			criteriaU.createCriteria("zone").add(Restrictions.idEq(zona));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderU(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Utente> getSupervisoriInZona(Long zona) {
		if(zona != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Utente.class);
			criteria.createCriteria("zone").add(Restrictions.idEq(zona));
			ApplicationUtil.orderU(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Utente>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Zona> getZonaInSupervisori(Long user) {
		if(user != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
			criteria.createCriteria("supervisori").add(Restrictions.idEq(user));
			ApplicationUtil.orderZ(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Zona>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Zona> getZonaNotInSupervisori(Long user) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Zona.class);
		criteriaU.createCriteria("supervisori").add(Restrictions.idEq(user));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Zona.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderZ(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiInUtente(Long user) {
		if(user != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
			criteria.createCriteria("user").add(Restrictions.idEq(user));
			ApplicationUtil.orderA(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Agente>();
		}
	}
	@SuppressWarnings("unchecked")
	public List<Agente> getAgentiNotInUtente(Long user, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("user", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Agente.class);
				criteriaU.createCriteria("user").add(Restrictions.idEq(user));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("user", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Agente.class);
			criteriaU.createCriteria("user").add(Restrictions.idEq(user));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderA(criteria);
		return getList(criteria);
	}
	public void aggiungiSupervisoreAdZona(Long superId, Long zonaId) {
		Utente utente = getFullSupervisoreZona(superId);
		Zona zona = getZona(zonaId);
		if (utente != null && zona != null) {
			if (!utente.getZone().contains(zona)) {
				utente.getZone().add(zona);
				salvaAggiorna(utente);
			}
		}
	}
	public void rimuoviSupervisoreAdZona(Long superId, Long zonaId) {
		Utente utente = getFullSupervisoreZona(superId);
		Zona zona = getZona(zonaId);
		if (utente != null && zona != null) {
			if (utente.getZone().contains(zona)) {
				utente.getZone().remove(zona);
				salvaAggiorna(utente);
			}
		}
	}
	public void aggiungiSupervisoreAdZonaTutti(Long zonaId, Integer tutti) {
		Zona zona = getFullZonaSupervisore(zonaId);
		List<Utente> utenti = getSupervisoriNotInZona(zonaId, tutti);
		if (ApplicationUtil.notEmpty(utenti) && zona != null) {
			zona.getSupervisori().addAll(utenti);
			salvaAggiorna(zona);
		}
	}
	public void rimuoviSupervisoreAdZonaTutti(Long zonaId) {
		Zona zona = getFullZonaSupervisore(zonaId);
		List<Utente> utenti = getSupervisoriInZona(zonaId);
		if (ApplicationUtil.notEmpty(utenti) && zona != null) {
			zona.getSupervisori().removeAll(utenti);
			salvaAggiorna(zona);
		}
	}
	public void aggiungiSupervisoreTuttiAdZona(Long superId) {
		Utente utente = getFullSupervisoreZona(superId);
		List<Zona> zone = getZonaNotInSupervisori(superId);
		if (utente != null && ApplicationUtil.notEmpty(zone)) {
			utente.getZone().addAll(zone);
			salvaAggiorna(utente);
		}
	}
	public void rimuoviSupervisoreTuttiAdZona(Long superId) {
		Utente utente = getFullSupervisoreZona(superId);
		List<Zona> zone = getZonaInSupervisori(superId);
		if (utente != null && ApplicationUtil.notEmpty(zone)) {
			utente.getZone().removeAll(zone);
			salvaAggiorna(utente);
		}
	}
	@SuppressWarnings("unchecked")
	public List<Lista> trova(ListaCriteria lst, int start, int size) {
		ExtendedCriteria criteria = getCriteria(lst);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), lst.getO_tipo(), lst.getO_colonna(),  new Order [] {Order.desc("id")});
		return getList(criteria, start, size); 
	}
	public int trovaSize(ListaCriteria lst) {
		ExtendedCriteria criteria = getCriteria(lst);
		return getListSize(criteria);
	}
	private ExtendedCriteria getCriteria(ListaCriteria lst) {
		if(lst != null)  lst = (ListaCriteria) lst.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		if (ApplicationUtil.isNotBlank(lst.getNome_descrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, lst.getNome_descrizione(), criteria, "nome", "descrizione");
		}
		if (ApplicationUtil.gtZero(lst.getCampagna())) {
			criteria.createCriteria("campagna").add(Restrictions.idEq(lst.getCampagna()));
		}
		if (ApplicationUtil.isNotBlank(lst.getCallers())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, lst.getCallers(), criteria, "callers");
		}
		ApplicationUtil.statoLC(lst.getStato_id(), lst.getAttivo_id(), criteria);
		lst.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, lst);
	}
	private ExtendedCriteria getCriteriaLista(Boolean s, Utente utente, ListaCriteria lst) {
		if(lst != null)  lst = (ListaCriteria) lst.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		DetachedCriteria criteriaC = criteria.createCriteria("campagna", ApplicationUtil.leftJoin());
		if (ApplicationUtil.isNotBlank(lst.getNome_descrizione())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, lst.getNome_descrizione(), criteria, "nome", "descrizione");
		}
		if (ApplicationUtil.gtZero(lst.getCampagna())) {
			criteriaC.add(Restrictions.idEq(lst.getCampagna()));
		}
		if (ApplicationUtil.isNotBlank(lst.getCallers())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, lst.getCallers(), criteria, "callers");
		}
		if (utente != null && s != null) {
			if(ApplicationUtil.isTrue(s)) {
				impostazioneDao.subCriteriaCampagnaSupervisore(utente.getId(), criteriaC, "id");
			} else {
				impostazioneDao.subCriteriaCampagnaPartner(utente.getId(), criteriaC, "id");
			}
		}
		ApplicationUtil.statoLC(lst.getStato_id(), lst.getAttivo_id(), criteria);
		lst.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, lst);
	}
	public Lista getLista(Long id) {
		return (Lista) get(Lista.class, id);
	}
	public List<SelectItem> getListeSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		List<ListaSI> lis = getListaSI(criteria);
		return impostazioneDao.getLista(lis, true, null);
	}
	public List<SelectItem> getListeSI(Long campagnaId) {
		List<ListaSI> lis = new ArrayList<ListaSI>();
		if(ApplicationUtil.gtZero(campagnaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
			criteria.createCriteria("campagna").add(Restrictions.idEq(campagnaId));
			lis.addAll(getListaSI(criteria));
		}
		return impostazioneDao.getLista(lis, true, null);
	}
	public List<SelectItem> getListeSI(Long campagnaId, StatoIntervallo stato) {
		List<ListaSI> lis = new ArrayList<ListaSI>();
		if(ApplicationUtil.gtZero(campagnaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
			DetachedCriteria campagnaC = criteria.createCriteria("campagna");
			campagnaC.add(Restrictions.idEq(campagnaId));
			ApplicationUtil.statoLC(stato, campagnaC, criteria);
			lis.addAll(getListaSI(criteria));
		}
		return impostazioneDao.getLista(lis, true, null);
	}
	
	public List<SelectItem> getListeSIRicerca(Long campagnaId, StatoIntervallo stato) {
		List<ListaSI> lis = new ArrayList<ListaSI>();
		if(ApplicationUtil.gtZero(campagnaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
			DetachedCriteria campagnaC = criteria.createCriteria("campagna");
			campagnaC.add(Restrictions.idEq(campagnaId));
			ApplicationUtil.statoLC(stato, campagnaC, criteria);
			lis.addAll(getListaSI(criteria));
		}
		return impostazioneDao.getLista(lis, false, null);
	}
		
	public List<SelectItem> getListeMasterSI(Long campagnaId) {
		if(ApplicationUtil.gtZero(campagnaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
			criteria.createCriteria("campagna").add(Restrictions.idEq(campagnaId));
			Boolean s = Boolean.valueOf(configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_INSERIMENTO_LISTE));
			if(ApplicationUtil.isTrue(s)) {
				Boolean ts = impostazioneDao.getEstrazioneStato();
				ApplicationUtil.criteriaStato(criteria, true, ApplicationUtil.isTrue(ts));
			}
			List<ListaSI> lis = getListaSI(criteria);
			return impostazioneDao.getLista(lis, false, null);
		}
		return new ArrayList<SelectItem>();
	}
	public List<SelectItem> getAgentiMasterSI(Long campagnaId) {
		if(ApplicationUtil.gtZero(campagnaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
			criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
			List<AgenteSI> lis = getAgenteSI(criteria);
			return impostazioneDao.getAgente(lis, false, false, null);
		}
		return new ArrayList<SelectItem>();
	}
	public List<SelectItem> getAgentiReteMasterSI(Utente utente, Long campagnaId) {
		Integer max = userDao.getMaxLivelloRete();
		if(ApplicationUtil.gtZero(campagnaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
			impostazioneDao.subCriteriaAgentiSottoReteByUser(utente.getId(), 0, max ,criteria, "id");
			criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
			List<AgenteSI> lis = getAgenteSI(criteria);
			return impostazioneDao.getAgente(lis, false, false, null);
		}
		return new ArrayList<SelectItem>();
	}
	public List<SelectItem> getAgentiAgenteMasterSI(Utente utente, Long campagnaId) {
		if(ApplicationUtil.gtZero(campagnaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
			criteria.createCriteria("user").add(Restrictions.idEq(utente.getId()));
			criteria.createCriteria("zone").createCriteria("campagne").add(Restrictions.idEq(campagnaId));
			List<AgenteSI> lis = getAgenteSI(criteria);
			return impostazioneDao.getAgente(lis, false, false, null);
		}
		return new ArrayList<SelectItem>();
	}
	public List<SelectItem> getAgentiMasterSI(Utente utente, Long campagnaId) {
		if(ApplicationUtil.gtZero(campagnaId)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
			DetachedCriteria criteriaZ = criteria.createCriteria("zone");
			criteriaZ.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
			criteriaZ.createCriteria("supervisori").add(Restrictions.idEq(utente.getId()));
			List<AgenteSI> lis = getAgenteSI(criteria);
			return impostazioneDao.getAgente(lis, false, false, null);
		}
		return new ArrayList<SelectItem>();
	}
	public int getCampagneByListaIdSize(Long lista_id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("liste").add(Restrictions.idEq(lista_id));
		return getListSize(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByListaId(Long idLista) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("liste").add(Restrictions.idEq(idLista));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Lista> trovaLista(Boolean s, Utente utente, ListaCriteria lst, List<Lista> liste, int start, int size) {
		ExtendedCriteria criteria = getCriteriaLista(s, utente, lst);
		List<Long> ids = ApplicationUtil.prendiIdLista(liste);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		ApplicationUtil.setOrdinamento(criteria.getCriteria(), lst.getO_tipo(), lst.getO_colonna(),  new Order [] {Order.desc("id")});
		return getList(criteria, start, size);
	}
	public int trovaListaSize(Boolean s, Utente utente, ListaCriteria lst, List<Lista> liste) {
		ExtendedCriteria criteria = getCriteriaLista(s, utente, lst);
		List<Long> ids = ApplicationUtil.prendiIdLista(liste);
		ApplicationUtil.removeCampi(criteria.getCriteria(), ids);
		return  getListSize(criteria);
	}
	public Lista getFullLista(Long listaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		criteria.add(Restrictions.idEq(listaId));
		criteria.setFetchMode("campagna", FetchMode.JOIN);
		return (Lista) getUniqueResult(criteria);
	}
	public int getContattiSize(Long listaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Anagrafica.class);
		criteria.createCriteria("lista").add(Restrictions.idEq(listaId));
		return getListSize(criteria);
	}
	public List<Anagrafica> getFullListAnagrafica(List<Anagrafica> conts) {
		List<Anagrafica> anagrafiche = new ArrayList<Anagrafica>();
		if(ApplicationUtil.notEmpty(conts)) {
			Iterator<Anagrafica> objIter = conts.iterator();
			Session session = openSession();
			while(objIter.hasNext()) {
				Anagrafica anagrafica = objIter.next();
				Anagrafica ana = (Anagrafica) session.get(Anagrafica.class, anagrafica.getId());
				Hibernate.initialize(ana.getLista());
				anagrafiche.add(ana);
			}
			closeSession(session);
		}
		return anagrafiche; 
	}

	public Anagrafica getFullListAnagrafica(Anagrafica a) {
		Anagrafica anagrafica = null;
		if(a != null) {
			Session session = openSession();
			anagrafica = (Anagrafica) session.get(Anagrafica.class, a.getId());
			Hibernate.initialize(anagrafica.getLista());
			closeSession(session);
		}
		return anagrafica;
	}
	public void rimuoviLista(List<Anagrafica> objs, Lista lista) {
		if(ApplicationUtil.notEmpty(objs)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Anagrafica> objIter = objs.iterator();
			while(objIter.hasNext()) {
				Anagrafica obj = objIter.next();
				if(obj.getLista().contains(lista))
					obj.getLista().remove(lista);
				if(ApplicationUtil.empty(obj.getLista())) {
					session.delete(obj);
				} else {
					session.update(obj);
				}
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	public void addAnagraficaToLista(Lista lista, Anagrafica anagrafica) {
		lista.getContatti().add(anagrafica);
	}
	public List<Contatto> nuovaLista() {
		return new ArrayList<Contatto>();
	}
	public void eliminaLista(Lista lista) {
		cancellaAggiorna(lista);
	}
	public List<Anagrafica> nuovaListaAnagrafiche() {
		return new ArrayList<Anagrafica>();
	}
	public void salvaLista(List<Anagrafica> objs, Lista lista , Boolean remove) {
		if(ApplicationUtil.notEmpty(objs)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Anagrafica> objIter = objs.iterator();
			while(objIter.hasNext()) {
				Anagrafica obj = objIter.next();
				if(remove) {
					if(obj.getLista().contains(lista))
						obj.getLista().remove(lista);
				} else {
					if(!obj.getLista().contains(lista))
						obj.getLista().add(lista);
				}
				session.update(obj);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	public void salvaLista(List<Anagrafica> objs, Lista lista) {
		if(ApplicationUtil.notEmpty(objs)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			int i = 0;
			Iterator<Anagrafica> objIter = objs.iterator();
			while(objIter.hasNext()) {
				Anagrafica obj = objIter.next();
				if(!obj.getLista().contains(lista))
					obj.getLista().add(lista);
				session.update(obj);
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	public Lista aggiornaLista(Lista lista, List<Anagrafica> anagraficheDaRimuovere, List<Anagrafica> anagraficheDaAggiungere, List<Anagrafica> anagraficheDaImportare) {
		Boolean geocoder = impostazioneDao.getGeocoder();
		Integer lt = impostazioneDao.getDownloadLogType();
		String operazione = ApplicationUtil.detoken(ApplicationUtil.asList(ConstantsLanguage.RIMUOVERE.stampa() + " " + anagraficheDaRimuovere.size(), ConstantsLanguage.AGGIUNGERE.stampa() + " " + anagraficheDaAggiungere.size(), ConstantsLanguage.IMPORTARE.stampa() + " " + anagraficheDaImportare.size()), ConstantsGlobal.colonSeparator);
		LoggerImport l = new LoggerImport(lt, ConstantsLanguage.LISTA.stampa() + ": " + lista.getDenominazione(), operazione);
		lista = salva(lista);
		if(ApplicationUtil.notEmpty(anagraficheDaAggiungere)) {
			List<Anagrafica> fullAgg = getFullListAnagrafica(anagraficheDaAggiungere);
			salvaLista(fullAgg, lista, false);
		}
		if(ApplicationUtil.notEmpty(anagraficheDaRimuovere)) {
			List<Anagrafica> fullRem = getFullListAnagrafica(anagraficheDaRimuovere);
			salvaLista(fullRem, lista, true);
		}
		if(ApplicationUtil.notEmpty(anagraficheDaImportare)) {
			List<Lista> lst = new ArrayList<Lista>();
			lst.add(lista);
			anagraficaDao.salvaListaConCampoAggiuntivo(anagraficheDaImportare, lst, geocoder);
		}
		l.log(null);
		return lista;
	}
	public Lista aggiornaLista(Lista lista,List<Anagrafica> anagraficheDaAggiungere, List<Anagrafica> anagraficheDaImportare) {
		Boolean geocoder = impostazioneDao.getGeocoder();
		Integer lt = impostazioneDao.getDownloadLogType();
		String operazione = ApplicationUtil.detoken(ApplicationUtil.asList(ConstantsLanguage.INSERIRE.stampa()  + " " + anagraficheDaAggiungere.size(), ConstantsLanguage.IMPORTARE.stampa() + " " + anagraficheDaImportare.size()), ConstantsGlobal.colonSeparator);
		LoggerImport l = new LoggerImport(lt, ConstantsLanguage.LISTA.stampa() + ": " + lista.getDenominazione(), operazione);
		lista = salva(lista);
		if(ApplicationUtil.notEmpty(anagraficheDaAggiungere)) {
			List<Anagrafica> fullAgg = getFullListAnagrafica(anagraficheDaAggiungere);
			salvaLista(fullAgg, lista);
		}
		if(ApplicationUtil.notEmpty(anagraficheDaImportare)) {
			List<Lista> lst = new ArrayList<Lista>();
			lst.add(lista);
			anagraficaDao.salvaListaConCampoAggiuntivo(anagraficheDaImportare, lst, geocoder);
		}
		l.log(null);
		return lista;
	}
	public void aggiornaListaECampagna(Lista lista, List<Campagna> campagne,  List<Anagrafica> aDaRimuovere, List<Anagrafica> aDaAggiungere, List<Anagrafica> aDaImportare) {
		lista = aggiornaLista(lista, aDaRimuovere, aDaAggiungere, aDaImportare);
		Lista ls = getLista(lista.getId());
		if(ApplicationUtil.notEmpty(campagne)) {
			EsitoChiamata dachiamare = ConstantsEsito.DEFAULTS.get(ConstantsEsito.ID_DA_CHIAMARE);
			Iterator<Campagna> objDIter = campagne.iterator();
			while(objDIter.hasNext()) {
				Campagna campagna = objDIter.next();
				Campagna cm = getCampagna(campagna.getId());
				creaContattiPerListaInCampagna(aDaImportare, ls, cm, dachiamare);
			}
			Iterator<Campagna> objIter = campagne.iterator();
			while(objIter.hasNext()) {
				Campagna campagna = objIter.next();
				eliminaContattoDaListaECampagna(aDaRimuovere, lista, campagna);
			}
			Iterator<Campagna> objRIter = campagne.iterator();
			while(objRIter.hasNext()) {
				Campagna campagna = objRIter.next();
				Campagna cm = getCampagna(campagna.getId());
				creaContattiPerListaInCampagna(aDaAggiungere, ls, cm, dachiamare);
			}
		}
	}
	public void eliminaContattoDaListaECampagna(List<Anagrafica> anagrafica, Lista lista, Campagna campagna){
		if(ApplicationUtil.notEmpty(anagrafica)) {
			String hql = "delete from Contatto where anagrafica in (:anagrafica) and lista = :lista and campagna = :campagna";
			execute(false, hql, new Parameter("anagrafica", anagrafica), new Parameter("lista", lista), new Parameter("campagna", campagna));
		}
	}
	public Lista salva(Lista lista) {
		lista = (Lista) salvaAggiorna(lista);
		return lista;
	}
	public Lista getByCampagnaNome(String nome, Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		criteria.createCriteria("campagna").add(Restrictions.eq("id", campagnaId));
		criteria.add(Restrictions.eq("nome", nome));
		return (Lista) getUniqueResult(criteria);
	}
	public void aggiungiAgenteAUtente(Long utenteId, Long agenteId) {
		Utente utente = getFullUtenteAgente(utenteId);
		Agente agente = getAgente(agenteId);
		if (utente != null && agente != null) {
			if (!utente.getAgenti().contains(agente)) {
				utente.getAgenti().add(agente);
				salvaAggiorna(utente);
			}
		}
	}
	public void rimuoviAgenteDaUtente(Long utenteId, Long agenteId) {
		Utente utente = getFullUtenteAgente(utenteId);
		Agente agente = getAgente(agenteId);
		if (utente != null && agente != null) {
			if (utente.getAgenti().contains(agente)) {
				utente.getAgenti().remove(agente);
				salvaAggiorna(utente);
			}
		}
	}
	public void aggiungiAgenteTuttiAUtente(Long utenteId, Integer tutti) {
		Utente utente = getFullUtenteAgente(utenteId);
		List<Agente> agenti = getAgentiNotInUtente(utenteId, tutti);
		if (utente != null && ApplicationUtil.notEmpty(agenti)) {
			utente.getAgenti().addAll(agenti);
			salvaAggiorna(utente);
		}
	}
	public void rimuoviAgenteTuttiDaUtente(Long utenteId) {
		Utente utente = getFullUtenteAgente(utenteId);
		List<Agente> agenti = getAgentiInUtente(utenteId);
		if (utente != null && ApplicationUtil.notEmpty(agenti)) {
			utente.getAgenti().removeAll(agenti);
			salvaAggiorna(utente);
		}
	}
	public void aggiungiCampagnaAdEsito(Long campagnaId, Long esitoId) {
		Campagna campagna = getFullCampagnaEsito(campagnaId);
		EsitoArchiviazione esito = getEsitoById(esitoId);
		if (campagna != null && esito != null) {
			if (!campagna.getEsiti_archiviazione().contains(esito)) {
				campagna.getEsiti_archiviazione().add(esito);
				salva(campagna);
			}
		}
	}
	public void rimuoviCampagnaAdEsito(Long campagnaId, Long esitoId) {
		Campagna campagna = getFullCampagnaEsito(campagnaId);
		EsitoArchiviazione esito = getEsitoById(esitoId);
		if (campagna != null && esito != null) {
			if (campagna.getEsiti_archiviazione().contains(esito)) {
				campagna.getEsiti_archiviazione().remove(esito);
				salva(campagna);
			}
		}
	}
	public void aggiungiCampagnaTuttiAdEsito(Long esitoId) {
		EsitoArchiviazione esito = getFullEsitoById(esitoId);
		List<Campagna> campagne = getCampagneNotInEsito(esitoId);
		if (esito != null && ApplicationUtil.notEmpty(campagne)) {
			esito.getCampagne().addAll(campagne);
			salvaEsito(esito);
		}
	}
	public void rimuoviCampagnaTuttiAdEsito(Long esitoId) {
		EsitoArchiviazione esito = getFullEsitoById(esitoId);
		List<Campagna> campagne = getCampagneInEsito(esitoId);
		if (esito != null && ApplicationUtil.notEmpty(campagne)) {
			esito.getCampagne().removeAll(campagne);
			salvaEsito(esito);
		}
	}
	public void aggiungiCampagnaAdProdotto(Campagna campagna,Prodotto  prodotto) {
		ProdottiInVenditaPK pivPK = new ProdottiInVenditaPK();
		pivPK.setCampagna(campagna);
		pivPK.setProdotto(prodotto);
		ProdottiInVendita piv = new ProdottiInVendita(pivPK);
		if (campagna != null && prodotto != null) {
			salvaProdotto(piv);
		}
	}
	public void aggiungiCampagnaAdProdotto(Long campagnaId, Long prodottoId) {
		Campagna campagna = getCampagna(campagnaId);
		Prodotto  prodotto = getProdottoById(prodottoId);
		aggiungiCampagnaAdProdotto(campagna, prodotto);
	}
	public void rimuoviCampagnaAdProdotto(Long campagnaId, Long prodottoId) {
		if (campagnaId != null && prodottoId != null) {
			removeAssociationProdotto(campagnaId, prodottoId);
		}
	}
	public void aggiungiCampagnaTuttiAdProdotto(Long prodottoId) {
		List<Campagna> campagne = getCampagneNotInProdotto(prodottoId);
		Prodotto  prodotto = getProdottoById(prodottoId);
		if(ApplicationUtil.notEmpty(campagne)) {
			Iterator<Campagna> campagnaIt = campagne.iterator();
			while (campagnaIt.hasNext()) {
				Campagna campagna = (Campagna) campagnaIt.next();
				aggiungiCampagnaAdProdotto(campagna, prodotto);
			}
		}
	}
	public void rimuoviCampagnaTuttiAdProdotto(Long prodottoId) {
		List<Campagna> campagne = getCampagneInProdotto(prodottoId);
		if(ApplicationUtil.notEmpty(campagne)) {
			Iterator<Campagna> campagnaIt = campagne.iterator();
			while (campagnaIt.hasNext()) {
				Campagna campagna = (Campagna) campagnaIt.next();
				if (campagna != null && prodottoId != null) {
					removeAssociationProdotto(campagna.getId(), prodottoId);
				}
			}
		}
	}
	public void aggiungiProdottoTuttiACampagna(Long campagnaId) {
		List<Prodotto> prodotti = trovaProdotto(campagnaId);
		Campagna  campagna = getCampagna(campagnaId);
		if(ApplicationUtil.notEmpty(prodotti)) {
			Iterator<Prodotto> pIt = prodotti.iterator();
			while (pIt.hasNext()) {
				Prodotto prodotto = (Prodotto) pIt.next();
				aggiungiCampagnaAdProdotto(campagna, prodotto);
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<Prodotto> getProdottiByCampagnaAll(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria criteriaV = criteria.createCriteria("prodotti_in_vendita");
		criteriaV.add(Restrictions.eq("id.campagna.id", campagnaId));
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.orderP(criteria);
		List<Prodotto> prodotti =  getList(criteria);
		if(prodotti==null)
			prodotti = new ArrayList<Prodotto>();
		return prodotti;
	}
	public void eliminaProdottoTuttiDaCampagna(Long campagnaId) {
		List<Prodotto> prodotti = getProdottiByCampagnaAll(campagnaId);
		if(ApplicationUtil.notEmpty(prodotti)) {
			Iterator<Prodotto> pIt = prodotti.iterator();
			while (pIt.hasNext()) {
				Prodotto prodotto = (Prodotto) pIt.next();
				if (campagnaId != null && prodotto != null) {
					removeAssociationProdotto(campagnaId, prodotto.getId());
				}
			}
		}
	}
	public void aggiungiCampagnaAdLista(Long campagnaId, Long listaId) {
		Campagna campagna = getFullCampagna(campagnaId);
		Lista lista = getLista(listaId);
		if (campagna != null && lista != null) {
			if (!campagna.getListe().contains(lista)) {
				campagna.getListe().add(lista);
				salva(campagna);
				aggiornaLista(campagna, lista);
			}
		}
	}
	public void rimuoviCampagnaAdLista(Long campagnaId, Long listaId) {
		Campagna campagna = getFullCampagna(campagnaId);
		Lista lista = getLista(listaId);
		if (campagna != null && lista != null) {
			if (campagna.getListe().contains(lista)) {
				campagna.getListe().remove(lista);
				salva(campagna);
				eliminaListaDaCampagnaContatti(lista, campagna);
			}
		}
	}
	public void aggiungiCampagnaTuttiAdLista(Long listaId) {
		Lista lista  = getFullListaCampagna(listaId);
		List<Campagna> campagne = getCampagneNotInLista(listaId);
		if (ApplicationUtil.notEmpty(campagne) && lista != null) {
			lista.getCampagna().addAll(campagne);
			salva(lista);
			aggiornaListe(campagne, lista);
		}
	}
	public void rimuoviCampagnaTuttiAdLista(Long listaId) {
		Lista lista  = getFullListaCampagna(listaId);
		List<Campagna> campagne = getCampagneInLista(listaId);
		if (ApplicationUtil.notEmpty(campagne) && lista != null) {
			lista.getCampagna().removeAll(campagne);
			salva(lista);
			eliminaListaDaCampagnaContatti(lista, campagne);
		}
	}
	public Lista getFullListaCampagna(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("campagna", FetchMode.JOIN);
		return  (Lista) getUniqueResult(criteria);
	}
	public void eliminaListaDaCampagnaContatti(Lista lista, Campagna campagna) {
		if(campagna != null) {
			if(lista != null) {
				String hql = "delete from Contatto where lista = :lista and campagna = :campagna";
				execute(false, hql, new Parameter("lista", lista), new Parameter("campagna", campagna));
			}
		}
	}
	public void eliminaListaDaCampagnaContatti(Lista lista, List<Campagna> campagne) {
		if(ApplicationUtil.notEmpty(campagne)) {
			if(lista != null) {
				if(ApplicationUtil.notEmpty(campagne)) {
					String hql = "delete from Contatto where lista = :lista and campagna in (:campagna)";
					execute(false, hql, new Parameter("lista", lista), new Parameter("campagna", campagne));
				}
			}
		}
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getAllCampagneByListaById(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("liste").add(Restrictions.idEq(id));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNotInLista(Long lista) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("liste").add(Restrictions.idEq(lista));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneInLista(Long lista) {
		if(lista != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("liste").add(Restrictions.idEq(lista));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	private Prodotto getProdottoById(Long prodottoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		criteria.add(Restrictions.idEq(prodottoId));
		return (Prodotto) getUniqueResult(criteria);
	}
	public void eliminaInListaTransaction(List<Long> anags, Lista lista) {
		if(ApplicationUtil.notEmpty(anags)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			Iterator<Long> objIter = anags.iterator();
			int i = 0;
			while(objIter.hasNext()) {
				Long id = objIter.next();
				Anagrafica obj = (Anagrafica) session.get(Anagrafica.class, id);
				Hibernate.initialize(obj.getLista());
				if(obj.getLista().contains(lista))
					obj.getLista().remove(lista);
				if(ApplicationUtil.empty(obj.getLista())) {
					session.delete(obj);
				} else {
					session.update(obj);
				}
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}

	}
	public void eliminaInListaTransaction(List<Long> anags, List<Lista> listaLst) {
		if(ApplicationUtil.notEmpty(anags)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			Iterator<Long> objIter = anags.iterator();
			int i = 0;
			while(objIter.hasNext()) {
				Long id = objIter.next();
				Anagrafica obj = (Anagrafica) session.get(Anagrafica.class, id);
				Hibernate.initialize(obj.getLista());
				obj.getLista().removeAll(listaLst);
				if(ApplicationUtil.empty(obj.getLista())) {
					session.delete(obj);
				} else {
					session.update(obj);
				}
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	public void eliminaInCampagnaTransaction(List<Long> lists, Campagna campagna) {
		if(ApplicationUtil.notEmpty(lists)) {
			Session session = openSession();
			Transaction tx = session.beginTransaction();
			Iterator<Long> objIter = lists.iterator();
			int i = 0;
			while(objIter.hasNext()) {
				Long id = objIter.next();
				Lista obj = (Lista) session.get(Lista.class, id);
				Hibernate.initialize(obj.getCampagna());
				if(obj.getCampagna().contains(campagna))
					obj.getCampagna().remove(campagna);
				if(ApplicationUtil.empty(obj.getCampagna())) {
					session.delete(obj);
				} else {
					session.update(obj);
				}
				if (i++ % ConstantsGlobal.flushSize == 0) {
					session.flush();
					session.clear();
				}
			}
			tx.commit();
			closeSession(session);
		}
	}
	public void aggiungiEmailACampagna(Long campagnaId, Long emailId) {
		Campagna campagna = getCampagna(campagnaId);
		EmailTemplate email = getFullEmailConCampagne(emailId);
		if(email != null) {
			if(email.getCampagne() == null)
				email.setCampagne(new ArrayList<Campagna>());
			if(!email.getCampagne().contains(campagna)) {
				email.getCampagne().add(campagna);
				salvaEmail(email);
			}
		}
	}
	public void eliminaEmailDaCampagna(Long campagnaId, Long emailId) {
		Campagna campagna = getCampagna(campagnaId);
		EmailTemplate email = getFullEmailConCampagne(emailId);
		if(email.getCampagne() == null)
			email.setCampagne(new ArrayList<Campagna>());
		if(email.getCampagne().contains(campagna)) {
			email.getCampagne().remove(campagna);
			salvaEmail(email);
		}
	}
	
	public void aggiungiCampagnaTuttiAdEmail(Long emailId) {
		EmailTemplate email = getFullEmailCampagna(emailId);
		List<Campagna> campagne = getCampagneNotInEmail(emailId);
		if (email != null && ApplicationUtil.notEmpty(campagne)) {
			email.getCampagne().addAll(campagne);
			salvaEmail(email);
		}
	}
	public void rimuoviCampagnaTuttiAdEmail(Long emailId) {
		EmailTemplate email = getFullEmailCampagna(emailId);
		List<Campagna> campagne = getCampagneInEmail(emailId);
		if (email != null && ApplicationUtil.notEmpty(campagne)) {
			email.getCampagne().removeAll(campagne);
			salvaEmail(email);
		}
	}
	
	
	@SuppressWarnings("unchecked")
	public List<Lista> getListaById(List<Long> liste) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Lista.class);
		criteria.add(Restrictions.in("id", liste));
		return getList(criteria);
	}
	public ListaSchedulate riepilogo_comunicazioni(Schedulatore sa)  {
		Integer codice [] = new Integer [] {ConstantsGlobal.SCHEDULATA, ConstantsGlobal.PAUSE };
		ListaSchedulate ls = new ListaSchedulate();
		if (sa != null) {
			if(sa.isTipo_email()) {
				ls.setInEmail(comunicazioneDao.getDaInviarePost(sa, Email.class,codice));
			}
			if(sa.isTipo_rmail()) {
				ls.setInRmail(comunicazioneDao.getDaInviarePost(sa, Rmail.class,codice));
			}
			if(sa.isTipo_telecom()) {
				ls.setInTelecom(comunicazioneDao.getDaInviarePost(sa, Telecom.class,codice));
			}
		}
		return ls;
	}
	public void salvaAllegatoAppuntamento(FileUpload fileUpload, Appuntamento appuntamento) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					byte [] ffB = ff.getData();
					Allegato all = new Allegato(ff.getName(), ffB);
					all.setData_modifica(ApplicationUtil.date());
					all.setDimensione(ffB.length);
					if(!appuntamento.getAllegato().contains(all)){
						appuntamento.getAllegato().add(all);
					}
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void deleteAllegatoAppuntamento(Allegato allegato, Appuntamento appuntamento) {
		appuntamento.getAllegato().remove(allegato);
	}
	public void scaricaAllegatoAppuntamento(Allegato allegato, DownloadBean downloadBean) {
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(allegato.getNome());
			InputStream is = new ByteArrayInputStream(allegato.getFile());
			ApplicationUtil.copy(is, out);
			downloadBean.close();
			is.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public void deleteAllegatoAppuntamento(Appuntamento appuntamento) {
		appuntamento.getAllegato().clear();
	}
	public void scaricaAllegatoAppuntamento(Appuntamento appuntamento, DownloadBean downloadBean) {
		List<Allegato> allegati = appuntamento.getAllegato();
		if(ApplicationUtil.notEmpty(allegati)) {
			String p = ConstantsGlobal.secretZ;
			try {
				String file_header = ApplicationUtil.reportFile(ApplicationUtil.date(), ConstantsLanguage.APPUNTAMENTO.stampa()) + "." + ConstantsMap.zip;
				OutputStream out = downloadBean.outputStream();
				downloadBean.filename(file_header, allegati);
				ApplicationUtil.zip(null,allegati, out, p);
				downloadBean.close();
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		}
	}
	public Campagna getFullCampagnaRmail(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("rmail", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteria);
	}
	
	public RmailTemplate getFullRmailCampagna(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return (RmailTemplate) getUniqueResult(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> getRmailByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setFetchMode("documenti", FetchMode.JOIN);
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	
	public Campagna getCampagnaRmail(Long id){
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Campagna.class);
		criteriaC.add(Restrictions.idEq(id));
		criteriaC.setFetchMode("rmail", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteriaC);
	}
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> trovaRmailPerCampagna(Long campagnaId) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(RmailTemplate.class);
		criteriaC.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		return getList(criteriaC);
	}
	
	public void copiaRmail(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Long> rmails = getRmailsByCampagna(campagnaId_src);
		Iterator<Long> objIter = rmails.iterator();
		while(objIter.hasNext()) {
			Long rmailId = objIter.next();
			RmailTemplate rmail = getFullRmailConCampagne(rmailId);
			if(rmail.getCampagne() == null)
				rmail.setCampagne(new ArrayList<Campagna>());
			if(!rmail.getCampagne().contains(campagna)) {
				rmail.getCampagne().add(campagna);
				salvaRmail(rmail);
			}
		}
	}
	
	
	public void copiaProdotto(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Prodotto> prodottos = getProdottiByCampagnaAll(campagnaId_src);
		Iterator<Prodotto> objIter = prodottos.iterator();
		while(objIter.hasNext()) {
			Prodotto prodotto = (Prodotto) objIter.next();
			aggiungiCampagnaAdProdotto(campagna, prodotto);
		}
	}
	public void aggiungiRmailTuttiACampagna(Long campagnaId) {
		Campagna campagna = getCampagnaRmail(campagnaId);
		List<RmailTemplate> rmails = trovaRmail(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(rmails)) {
			if(campagna.getRmail() == null)
				campagna.setRmail(new ArrayList<RmailTemplate>());
			campagna.getRmail().addAll(rmails);
			salva(campagna);
		}
	}
	public void eliminaRmailTuttiDaCampagna(Long campagnaId) {
		Campagna campagna = getCampagnaRmail(campagnaId);
		List<RmailTemplate> rmails = getRmailByCampagnaId(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(rmails)) {
			if(campagna.getRmail() == null)
				campagna.setRmail(new ArrayList<RmailTemplate>());
			campagna.getRmail().removeAll(rmails);
			salva(campagna);
		}
	}
	
	public RmailTemplate salvaRmail(RmailTemplate rmail) {
		return (RmailTemplate) salvaAggiorna(rmail);
	}
	
	public RmailTemplate getFullRmailConCampagne(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return  (RmailTemplate) getUniqueResult(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Long> getRmailsByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	
	public void aggiungiCampagnaAdRmail(Long campagnaId, Long rmailId) {
		Campagna campagna = getFullCampagnaRmail(campagnaId);
		RmailTemplate rmail = getRmail(rmailId);
		if (campagna != null && rmail != null) {
			if (!campagna.getRmail().contains(rmail)) {
				campagna.getRmail().add(rmail);
				salva(campagna);
			}
		}
	}
	public void rimuoviCampagnaAdRmail(Long campagnaId, Long rmailId) {
		Campagna campagna = getFullCampagnaRmail(campagnaId);
		RmailTemplate rmail = getRmail(rmailId);
		if (campagna != null && rmail != null) {
			if (campagna.getRmail().contains(rmail)) {
				campagna.getRmail().remove(rmail);
				salva(campagna);
			}
		}
	}

	public RmailTemplate getRmail(Long id){
		return (RmailTemplate) get(RmailTemplate.class, id);
	}
	
	@SuppressWarnings("unchecked")
	public List<RmailTemplate> trovaRmail(Long campagna_id) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(RmailTemplate.class);
		criteriaU.createCriteria("campagne").add(Restrictions.idEq(campagna_id));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(RmailTemplate.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNotInRmail(Long rmail) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("rmail").add(Restrictions.idEq(rmail));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneInRmail(Long rmail) {
		if(rmail != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("rmail").add(Restrictions.idEq(rmail));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	
	public void aggiungiRmailACampagna(Long campagnaId, Long rmailId) {
		Campagna campagna = getCampagna(campagnaId);
		RmailTemplate rmail = getFullRmailConCampagne(rmailId);
		if(rmail != null) {
			if(rmail.getCampagne() == null)
				rmail.setCampagne(new ArrayList<Campagna>());
			if(!rmail.getCampagne().contains(campagna)) {
				rmail.getCampagne().add(campagna);
				salvaRmail(rmail);
			}
		}
	}
	public void eliminaRmailDaCampagna(Long campagnaId, Long rmailId) {
		Campagna campagna = getCampagna(campagnaId);
		RmailTemplate rmail = getFullRmailConCampagne(rmailId);
		if(rmail.getCampagne() == null)
			rmail.setCampagne(new ArrayList<Campagna>());
		if(rmail.getCampagne().contains(campagna)) {
			rmail.getCampagne().remove(campagna);
			salvaRmail(rmail);
		}
	}
	
	public void aggiungiCampagnaTuttiAdRmail(Long rmailId) {
		RmailTemplate rmail = getFullRmailCampagna(rmailId);
		List<Campagna> campagne = getCampagneNotInRmail(rmailId);
		if (rmail != null && ApplicationUtil.notEmpty(campagne)) {
			rmail.getCampagne().addAll(campagne);
			salvaRmail(rmail);
		}
	}
	public void rimuoviCampagnaTuttiAdRmail(Long rmailId) {
		RmailTemplate rmail = getFullRmailCampagna(rmailId);
		List<Campagna> campagne = getCampagneInRmail(rmailId);
		if (rmail != null && ApplicationUtil.notEmpty(campagne)) {
			rmail.getCampagne().removeAll(campagne);
			salvaRmail(rmail);
		}
	}

	public 	List<MyFile> listaBackup(FileCriteria mm) {
		if(mm != null)  mm = (FileCriteria) mm.loadCriteria();
		List<MyFile> ff = new ArrayList<MyFile>();
		if(mm != null && mm.getCerca()) {
			String home = impostazioneDao.getHome();
			File perc = new File(home + ConstantsGlobal.path_backup + "/");
			if(perc.exists() && perc.isDirectory()) {
				File [] files = perc.listFiles();
				if(ApplicationUtil.notEmpty(files)) {
					for(File file : files) {
						if(mm.isRicerca(file.getName(), file.length(), file.lastModified())) {
							MyFile f = new MyFile(file);
							f.setName(file.getName());
							f.setModified(file.lastModified());
							ff.add(f);
						}
					}
				}
			}
		}
		return  ff;
	}
	public void eliminaBackupFtp(MyFile back) {
		String home = impostazioneDao.getHome();
		File ff = new File(home + ConstantsGlobal.path_backup  + "/" + back.getName());
		ApplicationUtil.deleteFile(ff);
	}
	public void eliminaBackupFtp(List<MyFile> backs) {
		if(ApplicationUtil.notEmpty(backs)) {
			Iterator<MyFile> eIt = backs.iterator();
			while (eIt.hasNext()) {
				MyFile back = (MyFile) eIt.next();
				eliminaBackupFtp(back);
			}
		}
	}
	public void eliminaBackupFtp() {
		String home = impostazioneDao.getHome();
		File perc = new File(home + ConstantsGlobal.path_backup + "/");
		if(perc.exists() && perc.isDirectory()) {
			File [] files = perc.listFiles();
			for(File file : files) {
				ApplicationUtil.deleteFile(file);
			}
		}
	}
	public void scaricaBackup(MyFile back, DownloadBean downloadBean) {
		String home = impostazioneDao.getHome();
		try {
			OutputStream out = downloadBean.outputStream();
			downloadBean.filename(back.getName());
			String nome = home + ConstantsGlobal.path_backup + "/" + back.getName();
			InputStream istream = new FileInputStream(nome);
			ApplicationUtil.copy(istream, out);
			istream.close();
			downloadBean.close();
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		} finally {
		}
	}
	@SuppressWarnings("unchecked")
	public List<SelectItem> getErroriPubblicaUnique(String ragione_pubblica) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.isNotNull("ragione_pubblica"));
		criteria.setProjection(Projections.distinct(Projections.property("ragione_pubblica")));
		ApplicationUtil.orderEsito(criteria);
		List<String> esiti = getList(criteria);
		List<String> errori =  ConstantsMap.getErrori();
		errori.removeAll(esiti);
		if(ragione_pubblica != null && !errori.contains(ragione_pubblica)) {
			errori.add(ragione_pubblica);
		}
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		for(String er : errori) {
			sis.add(new SelectItem(er, er + " - " + Message.addCatalogMessage("phone.reason." + er)));
		}
		return sis;
	}
	@SuppressWarnings("unchecked")
	public List<SelectItem> getErroriPrivataUnique(String ragione_privata) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.isNotNull("ragione_privata"));
		criteria.setProjection(Projections.distinct(Projections.property("ragione_privata")));
		ApplicationUtil.orderEsito(criteria);
		List<String> esiti = getList(criteria);
		List<String> errori =  ConstantsMap.getErrori();
		errori.removeAll(esiti);
		if(ragione_privata != null && !errori.contains(ragione_privata)) {
			errori.add(ragione_privata);
		}
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		for(String er : errori) {
			sis.add(new SelectItem(er, er + " - " + Message.addCatalogMessage("phone.reason." + er)));
		}
		return sis;
	}
	public List<SelectItem> getErrori() {
		List<SelectItem> sis = new ArrayList<SelectItem>();
		sis.add(new SelectItem(null, ConstantsLanguage.TRATTINI.stampa()));
		for(String er : ConstantsMap.getErrori()) {
			sis.add(new SelectItem(er, er + " - " + Message.addCatalogMessage("phone.reason." + er)));
		}
		return sis;
	}
	
	@SuppressWarnings("unchecked")
	public List<Prodotto> trovaProdotto(Long campagna_id) { 
		DetachedCriteria criteriaP = DetachedCriteria.forClass(Prodotto.class);
		DetachedCriteria criteriaV = criteriaP.createCriteria("prodotti_in_vendita");
		criteriaV.add(Restrictions.eq("id.campagna.id", campagna_id));
		criteriaP.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Prodotto.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaP, "id");
		criteria.createCriteria("categoria", "categoria", ApplicationUtil.leftJoin());
		ApplicationUtil.orderP(criteria);
		List<Prodotto> prodotti =  getList(criteria);
		if(prodotti==null)
			prodotti = new ArrayList<Prodotto>();
		return prodotti;
	}
	
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByAutodialerId(Long autodialerId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("autodialer").add(Restrictions.idEq(autodialerId));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagnaNotInAutodialer(Long auto, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("autodialer", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
				criteriaU.createCriteria("autodialer").add(Restrictions.idEq(auto));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("autodialer", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
			criteriaU.createCriteria("autodialer").add(Restrictions.idEq(auto));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	public void aggiungiAutodialerACampagna(Long autodialerId, Long campagnaId) {
		Autodialer autodialer = getFullCampagnaAutodialer(autodialerId);
		Campagna  campagna = getCampagna(campagnaId);
		if (campagna != null && autodialer != null) {
			if (!autodialer.getCampagne().contains(campagna)) {
				autodialer.getCampagne().add(campagna);
				salvaAggiorna(autodialer);
			}
		}
	}
	public void rimuoviAutodialerDaCampagna(Long autodialerId, Long campagnaId) {
		Autodialer autodialer = getFullCampagnaAutodialer(autodialerId);
		Campagna  campagna = getCampagna(campagnaId);
		if (campagna != null && autodialer != null) {
			if (autodialer.getCampagne().contains(campagna)) {
				autodialer.getCampagne().remove(campagna);
				salvaAggiorna(autodialer);
			}
		}
	}
	public void aggiungiAutodialerTuttiACampagna(Long autodialerId, Integer tutti) {
		Autodialer autodialer = getFullCampagnaAutodialer(autodialerId);
		List<Campagna> campagne = getCampagnaNotInAutodialer(autodialerId, tutti);
		if (autodialer != null && ApplicationUtil.notEmpty(campagne)) {
			autodialer.getCampagne().addAll(campagne);
			salvaAggiorna(autodialer);
		}
	}
	public void rimuoviAutodialerTuttiDaCampagna(Long autodialerId) {
		Autodialer autodialer = getFullCampagnaAutodialer(autodialerId);
		List<Campagna> campagne = getCampagneByAutodialerId(autodialerId);
		if (autodialer != null && ApplicationUtil.notEmpty(campagne)) {
			autodialer.getCampagne().removeAll(campagne);
			salvaAggiorna(autodialer);
		}
	}
	public Autodialer getFullCampagnaAutodialer(Long autodialerId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Autodialer.class);
		criteria.add(Restrictions.idEq(autodialerId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return  (Autodialer) getUniqueResult(criteria);
	}
	public void  eliminaEsitoAppuntamentoSelezionati(List<EsitoAppuntamento> lst) {
		if (ApplicationUtil.notEmpty(lst)) {
			Iterator<EsitoAppuntamento> itCont = lst.iterator();
			while (itCont.hasNext()) {
				EsitoAppuntamento esito = (EsitoAppuntamento) itCont.next();
				if(!esito.getPredefinito()) {
					cancellaAggiorna(esito);
				}
			}
		}
	}
	public void eliminaNota(Utente utente, Nota nota) {
		String p = ApplicationUtil.asString( ConstantsLanguage.NOTA.stampa(), nota);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_NOTA);
		cancellaAggiorna(nota);
	}
	public void eliminaNote(Utente utente, List<Nota> note) {
		String p = ApplicationUtil.asString(ConstantsLanguage.TOTALE.stampa() + " " + ConstantsLanguage.NOTA.stampa() +": " + ApplicationUtil.size(note), note);
		operazioneDao.creaOperazione(ConstantsMap.LOG3, p, utente, ConstantsMap.OPERAZIONE_ELIMINA_NOTA);
		cancellaLista(note);
	}
	public void modificaStatoCampagnaSelezionati(List<Campagna> campagnas, Integer stato) {
		Iterator<Campagna> objIter = campagnas.iterator();
		while(objIter.hasNext()) {
			Campagna campagna = objIter.next();
			campagna.setStato(stato);
			salva(campagna);
		}
	}
	public void modificaStatoListaSelezionati(List<Lista> listas, Integer stato) {
		Iterator<Lista> objIter = listas.iterator();
		while(objIter.hasNext()) {
			Lista lista = objIter.next();
			lista.setStato(stato);
			salva(lista);
		}
	}
	public ExtendedCriteria getCriteriaEsitiAppuntamento(EsitoAppuntamentoCriteria esitoc) {
		if(esitoc != null)  esitoc = (EsitoAppuntamentoCriteria) esitoc.loadCriteria();
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoAppuntamento.class);
		if (ApplicationUtil.isNotBlank(esitoc.getEtichetta())) {
			ApplicationUtil.valori(ConstantsGlobal.colonSeparator,true, esitoc.getEtichetta(), criteria, "etichetta", "descrizione");
		}
		if (ApplicationUtil.gtZero(esitoc.getConfermato())){
			if (ApplicationUtil.eq(esitoc.getConfermato(), 1)) 
				criteria.add(Restrictions.eq("confermato", Boolean.TRUE));
			else 
				criteria.add(Restrictions.eq("confermato", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(esitoc.getVisibile())){
			if (ApplicationUtil.eq(esitoc.getVisibile(), 1)) 
				criteria.add(Restrictions.eq("visibile", Boolean.TRUE));
			else 
				criteria.add(Restrictions.eq("visibile", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(esitoc.getUtile())){
			if (ApplicationUtil.eq(esitoc.getUtile(), 1)) 
				criteria.add(Restrictions.eq("utile", Boolean.TRUE));
			else 
				criteria.add(Restrictions.eq("utile", Boolean.FALSE));
		}
		if (ApplicationUtil.gtZero(esitoc.getRiprogrammabile())){
			if (ApplicationUtil.eq(esitoc.getRiprogrammabile(), 1)) 
				criteria.add(Restrictions.eq("riprogrammabile", Boolean.TRUE));
			else 
				criteria.add(Restrictions.eq("riprogrammabile", Boolean.FALSE));
		}
		esitoc.impostaRicercaDate(criteria);
		return new ExtendedCriteria(impostazioneDao.getLoggerRicerca(), criteria, esitoc);
	}
	@SuppressWarnings("unchecked")
	public List<EsitoAppuntamento> trova(EsitoAppuntamentoCriteria esitoc,  int start, int size) {
		ExtendedCriteria criteria =  getCriteriaEsitiAppuntamento(esitoc);
		ApplicationUtil.orderEsito(criteria.getCriteria());
		return getList(criteria, start, size);
	}
	public int trovaSize(EsitoAppuntamentoCriteria esitoc) {
		ExtendedCriteria criteria =  getCriteriaEsitiAppuntamento(esitoc);
		return getListSize(criteria);
	}
	public List<SelectItem> getAllEsitiSI() {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.or(Restrictions.eq("richiamare", Boolean.FALSE), Restrictions.eq("richiamare", Boolean.TRUE)));
		List<EsitoChiamataSI> lis = getEsitoChiamataSI(criteria);
		return impostazioneDao.getEsitoChiamata(lis, true, null);
	}
	public EsitoChiamata getEsito(Long id){
		return (EsitoChiamata) get(EsitoChiamata.class, id);
	}
	public List<SelectItem> getAgentiSITeamSuper(Long userId, Long campagnaId, Agente agente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		criteriaZ.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteriaZ.createCriteria("supervisori").add(Restrictions.idEq(userId));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, agente);
	}

	public List<SelectItem> getAgentiSITeamSupervisore(Long userId, Long campagnaId, Agente agente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		criteriaZ.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, agente);
	}


	public List<SelectItem> getAgentiSITeamManager(Long userId, Long campagnaId, Agente agente) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Agente.class);
		DetachedCriteria criteriaZ = criteria.createCriteria("zone");
		criteriaZ.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		List<AgenteSI> lis = getAgenteSI(criteria);
		return impostazioneDao.getAgente(lis, false, false, agente);
	}
	@SuppressWarnings("unchecked")
	public void eliminaEsitoChiamata(List<Long> ids) {
		if(ApplicationUtil.notEmpty(ids)) {
			DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
			criteria.add(Restrictions.not(Restrictions.in("id", ids)));
			criteria.add(Restrictions.isNull("immagine"));
			List<EsitoChiamata> c = getList(criteria);
			cancellaLista(c);
		}
	}
	public void inserisciEsiti(EsitoChiamata esito, Long id) {
		String p = ConstantsGlobal.tp;
		String hql = "insert into " + p +"esito_chiamata (id, etichetta, descrizione, ordine, codice, attesa, max_tentativi, conta_tentativi, data_creazione) values (:id, :etichetta, :descrizione, :ordine, :codice, :attesa, :max_tentativi, :conta_tentativi, :data_creazione)";
		execute(true, hql, new Parameter("id", id), new Parameter("etichetta", esito.getEtichetta()), new Parameter("descrizione", esito.getDescrizione()), new Parameter("ordine", 0L), new Parameter("codice", 0L), new Parameter("attesa", 0), new Parameter("max_tentativi", 0), new Parameter("conta_tentativi", false), new Parameter("data_creazione", ApplicationUtil.date()));
	}

	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneByEsitoChiamataId(Long esitoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.createCriteria("esito").add(Restrictions.idEq(esitoId));
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagnaNotInEsitoChiamata(Long auto, Integer tutti) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		if(ApplicationUtil.gtZero(tutti)) {
			if(ApplicationUtil.eq(tutti, 1)) {
				criteria.createCriteria("esito", ApplicationUtil.leftJoin()).add(Restrictions.isNull("id"));
			} else {
				DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
				criteriaU.createCriteria("esito").add(Restrictions.idEq(auto));
				criteriaU.setProjection(Projections.distinct(Projections.property("id")));
				impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
				criteria.createCriteria("esito", ApplicationUtil.leftJoin()).add(Restrictions.isNotNull("id"));
			}
		} else {
			DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
			criteriaU.createCriteria("esito").add(Restrictions.idEq(auto));
			criteriaU.setProjection(Projections.distinct(Projections.property("id")));
			impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		}
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	public void aggiungiEsitoChiamataACampagna(Long esitoId, Long campagnaId) {
		EsitoChiamata esito = getFullCampagnaEsitoChiamata(esitoId);
		Campagna  campagna = getCampagna(campagnaId);
		if (campagna != null && esito != null) {
			if (!esito.getCampagne().contains(campagna)) {
				esito.getCampagne().add(campagna);
				salvaAggiorna(esito);
			}
		}
	}
	public void rimuoviEsitoChiamataDaCampagna(Long esitoId, Long campagnaId) {
		EsitoChiamata esito = getFullCampagnaEsitoChiamata(esitoId);
		Campagna  campagna = getCampagna(campagnaId);
		if (campagna != null && esito != null) {
			if (esito.getCampagne().contains(campagna)) {
				esito.getCampagne().remove(campagna);
				salvaAggiorna(esito);
			}
		}
	}
	public void aggiungiEsitoChiamataTuttiACampagna(Long esitoId, Integer tutti) {
		EsitoChiamata esito = getFullCampagnaEsitoChiamata(esitoId);
		List<Campagna> campagne = getCampagnaNotInEsitoChiamata(esitoId, tutti);
		if (esito != null && ApplicationUtil.notEmpty(campagne)) {
			esito.getCampagne().addAll(campagne);
			salvaAggiorna(esito);
		}
	}
	public void rimuoviEsitoChiamataTuttiDaCampagna(Long esitoId) {
		EsitoChiamata esito = getFullCampagnaEsitoChiamata(esitoId);
		List<Campagna> campagne = getCampagneByEsitoChiamataId(esitoId);
		if (esito != null && ApplicationUtil.notEmpty(campagne)) {
			esito.getCampagne().removeAll(campagne);
			salvaAggiorna(esito);
		}
	}
	public EsitoChiamata getFullCampagnaEsitoChiamata(Long esitoId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(EsitoChiamata.class);
		criteria.add(Restrictions.idEq(esitoId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return  (EsitoChiamata) getUniqueResult(criteria);
	}
	public void destroy() throws Exception {
	}
	public void afterPropertiesSet() throws Exception {
		List<Long> keyList = new ArrayList<Long>(ConstantsEsito.DEFAULTS.keySet());
		if(ApplicationUtil.notEmpty(keyList)) {
			for (Long key : keyList) {
				EsitoChiamata esito =  getEsitoChiamata(key);
				if(esito == null) {
					esito = ConstantsEsito.DEFAULTS.get(key);
					inserisciEsiti(esito, key);
				} 
				ConstantsEsito.DEFAULTS.put(key, esito);
			}
			//eliminaEsitoChiamata(keyList);
		}
	}
	public Integer getLimitaCampagnaManagerS() {
		String s = configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_LIMITA_CAMPAGNA_MANAGER);
		if(s != null) {
			if(s.equals(ConstantsMap.campagna0)) return ConstantsMap.value0;
			if(s.equals(ConstantsMap.campagna1)) return ConstantsMap.value1;
			if(s.equals(ConstantsMap.campagna2)) return ConstantsMap.value2;
			if(s.equals(ConstantsMap.campagna3)) return ConstantsMap.value3;
		}
		return ConstantsMap.value0;
	}
	public Integer getLimitaCampagnaBackofficeS() {
		String s = configurazionePannelloService.getValore(DefaultsPannello.KEY_PANNELLO_LIMITA_CAMPAGNA_BACKOFFICE);
		if(s != null) {
			if(s.equals(ConstantsMap.campagna0)) return ConstantsMap.value0;
			if(s.equals(ConstantsMap.campagna1)) return ConstantsMap.value1;
			if(s.equals(ConstantsMap.campagna2)) return ConstantsMap.value2;
			if(s.equals(ConstantsMap.campagna3)) return ConstantsMap.value3;
		}
		return ConstantsMap.value0;
	}
	public void salvaDocumento(FileUpload fileUpload, Campagna campagna) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					campagna.setPdf(ff.getData());
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}
	public boolean validaCampagna(Campagna campagna){
		if(campagna != null) {
			if(campagna.getOrdineA()) {
				return campagna.getLatitudine() != null && campagna.getLongitudine() != null;
			} else {
				return true;
			}
		} else {
			return false;
		}
	}
	public boolean validaLista(Lista lista){
		if(lista != null) {
			if(lista.getOrdineA()) {
				return lista.getLatitudine() != null && lista.getLongitudine() != null;
			} else {
				return true;
			}
		} else {
			return false;
		}
	}
	public void copiaTelecom(Long campagna_dst, Long campagnaId_src) {
		Campagna campagna =  getCampagna(campagna_dst);
		List<Long> telecoms = getTelecomsByCampagna(campagnaId_src);
		Iterator<Long> objIter = telecoms.iterator();
		while(objIter.hasNext()) {
			Long telecomId = objIter.next();
			TelecomTemplate telecom = getFullTelecomConCampagne(telecomId);
			if(telecom.getCampagne() == null)
				telecom.setCampagne(new ArrayList<Campagna>());
			if(!telecom.getCampagne().contains(campagna)) {
				telecom.getCampagne().add(campagna);
				salvaTelecom(telecom);
			}
		}
	}
	public Campagna getFullCampagnaTelecom(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("telecom", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteria);
	}
	public TelecomTemplate getFullTelecomCampagna(Long id) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		criteria.add(Restrictions.idEq(id));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return (TelecomTemplate) getUniqueResult(criteria);
	}
	public Campagna getCampagnaTelecom(Long id){
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Campagna.class);
		criteriaC.add(Restrictions.idEq(id));
		criteriaC.setFetchMode("telecom", FetchMode.JOIN);
		return (Campagna) getUniqueResult(criteriaC);
	}
	public TelecomTemplate salvaTelecom(TelecomTemplate telecom) {
		return (TelecomTemplate) salvaAggiorna(telecom);
	}
	public TelecomTemplate getFullTelecomConCampagne(Long agenteId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		criteria.add(Restrictions.idEq(agenteId));
		criteria.setFetchMode("campagne", FetchMode.JOIN);
		return  (TelecomTemplate) getUniqueResult(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Long> getTelecomsByCampagna(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.setProjection(Projections.distinct(Projections.property("id")));
		return getList(criteria);
	}
	public void aggiungiCampagnaAdTelecom(Long campagnaId, Long telecomId) {
		Campagna campagna = getFullCampagnaTelecom(campagnaId);
		TelecomTemplate telecom = getTelecom(telecomId);
		if (campagna != null && telecom != null) {
			if (!campagna.getTelecom().contains(telecom)) {
				campagna.getTelecom().add(telecom);
				salva(campagna);
			}
		}
	}
	public void rimuoviCampagnaAdTelecom(Long campagnaId, Long telecomId) {
		Campagna campagna = getFullCampagnaTelecom(campagnaId);
		TelecomTemplate telecom = getTelecom(telecomId);
		if (campagna != null && telecom != null) {
			if (campagna.getTelecom().contains(telecom)) {
				campagna.getTelecom().remove(telecom);
				salva(campagna);
			}
		}
	}
	public TelecomTemplate getTelecom(Long id){
		return (TelecomTemplate) get(TelecomTemplate.class, id);
	}
	@SuppressWarnings("unchecked")
	public List<TelecomTemplate> trovaTelecom(Long campagna_id) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(TelecomTemplate.class);
		criteriaU.createCriteria("campagne").add(Restrictions.idEq(campagna_id));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneNotInTelecom(Long telecom) {
		DetachedCriteria criteriaU = DetachedCriteria.forClass(Campagna.class);
		criteriaU.createCriteria("telecom").add(Restrictions.idEq(telecom));
		criteriaU.setProjection(Projections.distinct(Projections.property("id")));
		DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
		impostazioneDao.subPropertyNotInS(criteria, criteriaU, "id");
		ApplicationUtil.orderCL(criteria);
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<Campagna> getCampagneInTelecom(Long telecom) {
		if(telecom != null ){
			DetachedCriteria criteria = DetachedCriteria.forClass(Campagna.class);
			criteria.createCriteria("telecom").add(Restrictions.idEq(telecom));
			ApplicationUtil.orderCL(criteria);
			return getList(criteria);
		}	else {
			return new ArrayList<Campagna>();
		}
	}
	public void aggiungiTelecomACampagna(Long campagnaId, Long telecomId) {
		Campagna campagna = getCampagna(campagnaId);
		TelecomTemplate telecom = getFullTelecomConCampagne(telecomId);
		if(telecom != null) {
			if(telecom.getCampagne() == null)
				telecom.setCampagne(new ArrayList<Campagna>());
			if(!telecom.getCampagne().contains(campagna)) {
				telecom.getCampagne().add(campagna);
				salvaTelecom(telecom);
			}
		}
	}
	public void eliminaTelecomDaCampagna(Long campagnaId, Long telecomId) {
		Campagna campagna = getCampagna(campagnaId);
		TelecomTemplate telecom = getFullTelecomConCampagne(telecomId);
		if(telecom.getCampagne() == null)
			telecom.setCampagne(new ArrayList<Campagna>());
		if(telecom.getCampagne().contains(campagna)) {
			telecom.getCampagne().remove(campagna);
			salvaTelecom(telecom);
		}
	}
	public void aggiungiCampagnaTuttiAdTelecom(Long telecomId) {
		TelecomTemplate telecom = getFullTelecomCampagna(telecomId);
		List<Campagna> campagne = getCampagneNotInTelecom(telecomId);
		if (telecom != null && ApplicationUtil.notEmpty(campagne)) {
			telecom.getCampagne().addAll(campagne);
			salvaTelecom(telecom);
		}
	}
	public void rimuoviCampagnaTuttiAdTelecom(Long telecomId) {
		TelecomTemplate telecom = getFullTelecomCampagna(telecomId);
		List<Campagna> campagne = getCampagneInTelecom(telecomId);
		if (telecom != null && ApplicationUtil.notEmpty(campagne)) {
			telecom.getCampagne().removeAll(campagne);
			salvaTelecom(telecom);
		}
	}
	public void aggiungiTelecomTuttiACampagna(Long campagnaId) {
		Campagna campagna = getCampagnaTelecom(campagnaId);
		List<TelecomTemplate> telecoms = trovaTelecom(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(telecoms)) {
			if(campagna.getTelecom() == null)
				campagna.setTelecom(new ArrayList<TelecomTemplate>());
			campagna.getTelecom().addAll(telecoms);
			salva(campagna);
		}
	}
	public void eliminaTelecomTuttiDaCampagna(Long campagnaId) {
		Campagna campagna = getCampagnaTelecom(campagnaId);
		List<TelecomTemplate> telecoms = getTelecomByCampagnaId(campagnaId);
		if(campagna!= null && ApplicationUtil.notEmpty(telecoms)) {
			if(campagna.getTelecom() == null)
				campagna.setTelecom(new ArrayList<TelecomTemplate>());
			campagna.getTelecom().removeAll(telecoms);
			salva(campagna);
		}
	}
	@SuppressWarnings("unchecked")
	public List<TelecomTemplate> getTelecomByCampagnaId(Long campagnaId) {
		DetachedCriteria criteria = DetachedCriteria.forClass(TelecomTemplate.class);
		criteria.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		criteria.addOrder(Order.asc("nome"));
		return getList(criteria);
	}
	@SuppressWarnings("unchecked")
	public List<TelecomTemplate> trovaTelecomPerCampagna(Long campagnaId) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(TelecomTemplate.class);
		criteriaC.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		return getList(criteriaC);
	}
	
	@SuppressWarnings("unchecked")
	public List<Risposta> trovaRispostaPerCampagna(Long campagnaId) {
		DetachedCriteria criteriaC = DetachedCriteria.forClass(Risposta.class);
		criteriaC.createCriteria("campagne").add(Restrictions.idEq(campagnaId));
		return getList(criteriaC);
	}
	public MyFile reloadFileP(Long id, String full_path) {
		File ff = new File(full_path);
		MyFile f = new MyFile(ff);
		f.setName(full_path);
		f.setModified(ff.lastModified());
		String w = downloadDao.getWatermark();
		Integer rewrite = impostazioneDao.sovrascriviPDA();
		if(ApplicationUtil.gtZero(rewrite)) {
			if(id != null) {
				f.setId(id);
				String ext = ApplicationUtil.getExtensionLower(full_path);
				if(ApplicationUtil.asList(ConstantsMap.a_pda1).contains(ApplicationUtil.lowerCaseEmpty(ext)) || ApplicationUtil.asList(ConstantsMap.a_pda2).contains(ApplicationUtil.lowerCaseEmpty(ext))) {
					Session session = openSession();
					Contratto contratto = (Contratto) session.get(Contratto.class, id);
					Hibernate.initialize(contratto.getContatto().getAnagrafica().getDettagli());
					Hibernate.initialize(contratto.getDettagli());
					closeSession(session);
					if(contratto != null) {
						List <String> valoriValidi = new ArrayList<String>(ConstantsMap.anagraficaMap);
						Map<String, DatoAggiuntivo> valori =  ApplicationUtil.convertiAnagrafica(contratto.getContatto().getAnagrafica(), valoriValidi);
						List<DatoAggiuntivoAnagrafica> dA = contratto.getContatto().getAnagrafica().getDettagli();
						if(ApplicationUtil.notEmpty(dA)) {
							valori.putAll(ApplicationUtil.convertiDatoAggiuntivoA(dA));
						}
						List<DatoAggiuntivoContratto> dC = contratto.getDettagli();
						if(ApplicationUtil.notEmpty(dC)) {
							valori.putAll(ApplicationUtil.convertiDatoAggiuntivoC(dC));
						}
						ByteArrayOutputStream out = null;
						if(ApplicationUtil.asList(ConstantsMap.a_pda1).contains(ApplicationUtil.lowerCaseEmpty(ext))) {
							out = ApplicationUtil.mergeDataIntoTemplatePDF(w, full_path, valori, false);
						} 
						if(ApplicationUtil.asList(ConstantsMap.a_pda2).contains(ApplicationUtil.lowerCaseEmpty(ext))) {
							out = ApplicationUtil.mergeDataIntoTemplateXLS(full_path, valori, false);
						} 
						if(out != null) {
							try {  
								if(ApplicationUtil.eq(rewrite, 1)) {
									f.setData(out.toByteArray());
									out.close();
								} else {
									FileOutputStream outStream = new FileOutputStream(full_path);  
									out.writeTo(outStream);  
									out.close();
									outStream.close();  
								}
							} catch (IOException e) { }  
						}
					}
				}
			}
		}
		return f;
	}

	public void salvaBackup(FileUpload fileUpload) {
		try {
			List<MyFile> fls = fileUpload.getFiles();
			if(ApplicationUtil.notEmpty(fls)) {
				String home = impostazioneDao.getHome();
				Iterator<MyFile> flsIt = fls.iterator();
				while (flsIt.hasNext()) {
					MyFile ff = (MyFile) flsIt.next();
					FileOutputStream fi = new FileOutputStream(home + ConstantsGlobal.path_backup + "/" + ApplicationUtil.getNameN(ff.getName()));
					fi.write(ff.getData());
					fi.flush();
					fi.close();
				}
			}
		} catch (Exception e) {
			ErrorLog.log(null, getClass(), e);
		}
	}

	public void salvaSuono(FileUpload fileUpload, Campagna auto) {
		if(ApplicationUtil.isNotTrue(impostazioneDao.getFTPEnabled())) {
			String home = impostazioneDao.getHome();
			try {
				List<MyFile> fls = fileUpload.getFiles();
				if(ApplicationUtil.notEmpty(fls)) {
					Iterator<MyFile> flsIt = fls.iterator();
					while (flsIt.hasNext()) {
						MyFile ff = (MyFile) flsIt.next();
						ByteArrayOutputStream bos = new ByteArrayOutputStream();
						String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
						byte [] ffB = bos.toByteArray();
						if(ffB != null  && ffB.length > 0) {
							FileOutputStream fi = new FileOutputStream(home + ConstantsGlobal.path_sounds + "/" + fileN);
							fi.write(ffB);
							fi.flush();
							fi.close();
							auto.setSuono(fileN);
						}
					}
				}
			} catch (Exception e) {
				ErrorLog.log(null, getClass(), e);
			}
		} else {
			UtilFTP ut = impostazioneDao.getUtilFTP();
			FTPClient vmftp = ApplicationUtil.createClient();
			if (ApplicationUtil.connect(vmftp, ut)) {
				try {
					List<MyFile> fls = fileUpload.getFiles();
					if(ApplicationUtil.notEmpty(fls)) {
						Iterator<MyFile> flsIt = fls.iterator();
						while (flsIt.hasNext()) {
							MyFile ff = (MyFile) flsIt.next();
							ByteArrayOutputStream bos = new ByteArrayOutputStream();
							String fileN = ApplicationUtil.salvaAudio(ff, bos, false);
							byte [] ffB = bos.toByteArray();
							if(ffB != null  && ffB.length > 0) {
								InputStream  fis = new ByteArrayInputStream(ffB);
								ApplicationUtil.storeFile(vmftp, ConstantsGlobal.path_sounds + "/" + fileN, fis);
								auto.setSuono(fileN);
								fis.close();
							}
						}
					}
				} catch (Exception e) {
					ErrorLog.log(null, getClass(), e);
				} finally {
					ApplicationUtil.disconnect(vmftp);
				}
			}	
		}
	}
}